/* Convert a DVI file to GIF. */

/* Example:

	% gcc -o dvi2gif dvi2gif.c

	% echo '$$x^2$$\end' >tmp.tex

	% tex tmp

	% dvi2gif tmp

dvi2gif is limited to the 16 basic fonts listed on p. 428 of The TeXbook.

For smaller GIFs:

        % echo '\magnification=500$$x^2$$\end' >tmp.tex

        % tex tmp

        % dvi2gif --small tmp

*/

#include <stdio.h>
#include <string.h>

/*

65,536 scaled points = 1 point

72.27 points = 1 inch

1 inch = 300 dots

M = (65,536 sp/pt) x (72.27 pt/in) / (300 dots/in) = 15,787.6224 sp/dot

*/

#define M_LARGE 15788

/*

65,536 scaled points = 1 point

72.27 points = 1 inch

1 inch = 300 dots

M = (65,536 sp/pt) x (72.27 pt/in) / (150 dots/in) = 31,575.2448 sp/dot

*/

#define M_SMALL 31575

/* 6.5 inches x 300 dots/inch = 1950 */

#define MAXX 1950

/* 4 inches x 300 dots/inch = 1200 */

#define MAXY 1200

char buf[MAXY * MAXX];

char dvi_file_name[100];
char gif_file_name[100];

int min_x, max_x, min_y, max_y;

int width, height;

char *src_buf;
int src_len;

unsigned char *dst_buf;
int dst_len;
int dst_bit;

int current_index;
int current_length;
int current_code;

int string_index[4096];
int string_length[4096];
int string_count;

main(argc, argv)
int argc;
char *argv[];
{
	int n, small;

	if (argc < 2) {
		printf("usage: dvi2gif file\n");
		exit(1);
	}

	n = 1;

	small = 0;

	if (strcmp(argv[1], "--small") == 0) {
		small = 1;
		n = 2;
		if (argc < 3) {
			printf("usage: dvi2gif --small file\n");
			exit(1);
		}
	}

	strcpy(dvi_file_name, argv[n]);
	strcpy(gif_file_name, argv[n]);

	n = strlen(argv[n]);

	strcpy(dvi_file_name + n, ".dvi");
	strcpy(gif_file_name + n, ".gif");

	min_x = MAXX;
	min_y = MAXY;

	read_dvi_file(small);

	pack();

	width = max_x - min_x + 1;
	height = max_y - min_y + 1;

	src_buf = buf;
	src_len = width * height;

	compress();

	write_gif_file();
}

pack()
{
	int x, y;
	char *p;
	p = buf;
	for (y = min_y; y <= max_y; y++)
		for (x = min_x; x <= max_x; x++)
			*p++ = buf[y * MAXX + x];
}

#define CLEAR_CODE 4
#define EOI_CODE 5

compress()
{
	/* put destination buffer at end of source buffer */

	dst_buf = src_buf + src_len + 2;
	dst_len = 0;
	dst_bit = 1;

	/* put root codes at end of source buffer */

	src_buf[src_len + 0] = 0;
	src_buf[src_len + 1] = 1;

	string_index[0] = src_len + 0;
	string_index[1] = src_len + 1;

	/* codes 2, 3, 4 and 5 have zero length so they never match anything */

	string_length[0] = 1;
	string_length[1] = 1;
	string_count = 6;

	current_index = 0;
	current_length = 0;
	current_code = -1;

	emit_code(CLEAR_CODE);

	while (current_index + current_length < src_len) {

		/* advance the endpoint of the current string */

		current_length++;

		if (check_string_table() == 0) {

			emit_code(current_code);

			if (string_count == 4096) {
				emit_code(CLEAR_CODE);
				string_count = 6;
			} else {
				string_index[string_count] = current_index;
				string_length[string_count] = current_length;
				string_count++;
			}

			/* skip chars that were sent as a code */

			current_index += current_length - 1;
			current_length = 0;
			current_code = -1;
		}
	}

	emit_code(current_code);

	emit_code(EOI_CODE);

	/* some bits in last byte? */

	if (dst_bit > 1)
		dst_len++;
}

/* see if the current string is in the string table */

check_string_table()
{
	int i, j, x;

	for (i = current_code + 1; i < string_count; i++) {

		if (current_length != string_length[i])
			continue;

		x = string_index[i];

		for (j = 0; j < current_length; j++)
			if (src_buf[x + j] != src_buf[current_index + j])
				break;

		if (j == current_length) {
			current_code = i;
			return 1;
		}
	}

	return 0;
}

/* the decoder has a string table too and it's expecting enough bits to
represent all possible codes */

emit_code(code)
int code;
{
	int mask = 1;
	while (mask < string_count) {
		emit_bit(code & mask);
		mask <<= 1;
	}
}

emit_bit(bit)
int bit;
{
	if (dst_bit == 1)
		dst_buf[dst_len] = 0;

	if (bit)
		dst_buf[dst_len] |= dst_bit;

	dst_bit <<= 1;

	if (dst_bit == 0x100) {
		dst_bit = 1;
		dst_len++;
	}
}

write_gif_file()
{
	int i, n;
	FILE *f;

	f = fopen(gif_file_name, "wb");

	if (f == NULL) {
		printf("cannot open %s\n", gif_file_name);
		exit(1);
	}

	fputs("GIF89a", f);

	fputc(width & 0xff, f);
	fputc(width >> 8, f);

	fputc(height & 0xff, f);
	fputc(height >> 8, f);

	fputc(0x80, f); /* global color map flag, 1 bit per pixel */

	fputc(0, f); /* background color */

	fputc(0, f);

	fputc(255, f); /* white */
	fputc(255, f);
	fputc(255, f);

	fputc(0, f); /* black  */
	fputc(0, f);
	fputc(0, f);



	/* graphic control extension */

	fputc(0x21, f);
	fputc(0xf9, f);
	fputc(0x04, f);
	fputc(0x01, f);
	fputc(0x00, f);
	fputc(0x00, f);
	fputc(0x00, f); /* transparent color index */
	fputc(0x00, f);


	fputc(',', f);

	fputc(0, f); /* left */
	fputc(0, f);

	fputc(0, f); /* top */
	fputc(0, f);

	fputc(width & 0xff, f);
	fputc(width >> 8, f);

	fputc(height & 0xff, f);
	fputc(height >> 8, f);

	fputc(0, f); /* use global color map, not interlaced */

	fputc(2, f); /* code size */

	for (i = 0; i < dst_len; i++) {

		if (i % 254 == 0) {
			n = dst_len - i;
			if (n > 254)
				n = 254;
			fputc(n, f); /* byte count */
		}

		fputc(dst_buf[i], f);
	}

	fputc(0, f); /* last byte count */

	fputc(';', f);

	fclose(f);
}

typedef struct {
	short int index;
	char x_offset, y_offset, ncol, nrow;
	int tfm_width;
} METRIC;

METRIC cmbx10_metric[128] = {
{4242, -2, 27, 24, 28, 453288},
{4326, -2, 27, 35, 28, 628050},
{4449, -3, 27, 30, 28, 586180},
{4554, -2, 27, 28, 28, 527926},
{4652, -2, 27, 27, 28, 502440},
{4747, -2, 27, 32, 28, 589821},
{4859, -3, 27, 27, 28, 544310},
{4954, -3, 27, 30, 28, 586180},
{5059, -3, 27, 27, 28, 544310},
{5154, -3, 27, 30, 28, 586180},
{5259, -2, 27, 29, 28, 544310},
{8214, 0, 28, 31, 29, 439635},
{8327, 0, 28, 25, 29, 418700},
{8418, 0, 28, 25, 29, 418700},
{8509, 0, 28, 38, 29, 628050},
{8647, 0, 28, 38, 29, 628050},
{6315, -1, 17, 11, 18, 209350},
{6340, 3, 17, 14, 26, 230285},
{8090, -4, 28, 9, 8, 376830},
{8099, -10, 28, 9, 8, 376830},
{8108, -6, 26, 11, 6, 376830},
{8117, -5, 28, 13, 8, 376830},
{8130, -3, 24, 17, 2, 376830},
{8135, -13, 28, 9, 7, 569796},
{8143, -5, -1, 13, 8, 334960},
{6386, -1, 28, 22, 29, 391394},
{6466, -1, 17, 31, 18, 544310},
{6536, -1, 17, 34, 18, 586180},
{6613, -1, 21, 21, 26, 376830},
{6682, -2, 27, 39, 28, 682663},
{6819, -4, 27, 43, 28, 766403},
{6970, -3, 29, 30, 32, 586180},
{8156, 0, 15, 12, 4, 209350},
{7090, -4, 28, 6, 29, 229375},
{8785, -2, 28, 17, 14, 395035},
{7134, -3, 28, 33, 37, 628050},
{6012, -3, 30, 17, 33, 376830},
{7287, -3, 30, 33, 33, 628050},
{6083, -2, 28, 32, 29, 586180},
{7424, -3, 28, 7, 14, 209350},
{7437, -4, 30, 11, 41, 293090},
{7494, -3, 30, 11, 41, 293090},
{7551, -3, 30, 17, 18, 376830},
{7590, -3, 25, 30, 31, 586180},
{7707, -3, 5, 7, 14, 209350},
{8815, -1, 10, 11, 4, 251220},
{7720, -3, 5, 6, 6, 209350},
{7725, -3, 30, 17, 41, 376830},
{5361, -2, 26, 19, 27, 376830},
{5426, -3, 26, 17, 27, 376830},
{5484, -2, 26, 19, 27, 376830},
{5549, -2, 26, 19, 27, 376830},
{5614, -1, 26, 21, 27, 376830},
{5685, -2, 26, 19, 27, 376830},
{5750, -2, 26, 19, 27, 376830},
{5815, -3, 27, 19, 28, 376830},
{5882, -2, 26, 19, 27, 376830},
{5947, -2, 26, 19, 27, 376830},
{7813, -3, 17, 6, 18, 209350},
{7827, -3, 17, 6, 26, 209350},
{7112, -4, 20, 6, 29, 229375},
{7847, -3, 16, 30, 13, 586180},
{6257, -3, 20, 16, 29, 355895},
{6199, -3, 28, 16, 29, 355895},
{7896, -3, 28, 30, 29, 586180},
{0, -2, 27, 31, 28, 569796},
{109, -2, 27, 28, 28, 536118},
{207, -3, 27, 27, 28, 544310},
{302, -2, 27, 31, 28, 577988},
{411, -2, 27, 27, 28, 495158},
{506, -2, 27, 25, 28, 474223},
{594, -3, 27, 32, 28, 592552},
{706, -2, 27, 32, 28, 589821},
{818, -1, 27, 16, 28, 285809},
{874, -1, 27, 20, 28, 389573},
{944, -2, 27, 32, 28, 590731},
{1056, -2, 27, 24, 28, 453288},
{1140, -2, 27, 40, 28, 715431},
{1280, -2, 27, 32, 28, 589821},
{1392, -3, 27, 29, 28, 566155},
{1494, -2, 27, 27, 28, 515183},
{1589, -3, 27, 29, 36, 566155},
{1720, -2, 27, 33, 28, 565245},
{1836, -3, 27, 20, 28, 418700},
{1906, -2, 27, 28, 28, 524285},
{2004, -2, 27, 32, 28, 579809},
{2116, -1, 27, 33, 28, 569796},
{2232, -1, 27, 46, 28, 779146},
{2393, -1, 27, 33, 28, 569796},
{2509, -1, 27, 33, 28, 569796},
{2625, -3, 27, 23, 28, 460570},
{8005, -4, 30, 7, 41, 209350},
{8821, -5, 28, 17, 14, 395035},
{8041, -1, 30, 7, 41, 209350},
{8162, -5, 28, 13, 7, 376830},
{8174, -3, 29, 6, 6, 209350},
{8077, -2, 28, 7, 14, 209350},
{2706, -1, 17, 21, 18, 366362},
{2754, -1, 28, 23, 29, 418700},
{2838, -2, 17, 17, 18, 334960},
{2877, -2, 28, 23, 29, 418700},
{2961, -1, 17, 19, 18, 345428},
{3004, 0, 28, 18, 29, 230285},
{3070, -1, 17, 21, 27, 376830},
{3141, -1, 28, 24, 29, 418700},
{3228, -1, 29, 11, 30, 209350},
{3270, 3, 29, 14, 38, 230285},
{3337, -1, 28, 22, 29, 397765},
{3417, -1, 28, 11, 29, 209350},
{3457, -1, 17, 37, 18, 628050},
{3541, -1, 17, 24, 18, 418700},
{3595, -1, 17, 21, 18, 376830},
{3643, -1, 17, 23, 26, 418700},
{3718, -2, 17, 22, 26, 397765},
{3790, -1, 17, 17, 18, 310385},
{3829, -2, 17, 14, 18, 297277},
{3861, -1, 25, 14, 26, 293090},
{3907, -1, 17, 24, 18, 418700},
{3961, -1, 17, 22, 18, 397765},
{4011, -1, 17, 31, 18, 544310},
{4081, -1, 17, 22, 18, 397765},
{4131, -1, 17, 22, 26, 397765},
{4203, -1, 17, 17, 18, 334960},
{8851, 0, 11, 23, 1, 376830},
{8854, 0, 11, 47, 1, 753660},
{8179, -6, 29, 13, 9, 376830},
{8194, -4, 28, 15, 4, 376830},
{8202, -4, 29, 15, 6, 376830},
};
char cmbx10_bitmap[8860] = {
0,3,128,0,0,7,0,0,0,31,0,0,0,62,0,0,
0,124,0,0,1,252,0,0,3,248,0,0,15,248,0,0,
27,240,0,0,55,224,0,0,199,224,0,1,143,192,0,6,
31,192,0,12,31,128,0,56,63,128,0,96,63,0,0,192,
126,0,3,0,126,0,7,255,252,0,31,255,252,0,48,1,
248,0,96,3,240,1,128,3,240,3,0,7,224,14,0,15,
224,24,0,15,195,255,1,255,255,254,3,255,240,255,255,248,
15,255,255,240,15,192,31,128,252,0,252,15,192,7,192,252,
0,126,15,192,7,224,252,0,126,15,192,7,224,252,0,126,
15,192,7,192,252,0,248,15,192,63,0,255,255,224,15,192,
15,128,252,0,124,15,192,7,224,252,0,62,15,192,3,240,
252,0,63,15,192,3,240,252,0,63,15,192,3,240,252,0,
126,15,192,7,224,252,1,252,255,255,255,15,255,255,192,0,
31,224,32,31,255,28,15,224,63,131,224,1,240,248,0,30,
62,0,1,207,128,0,57,240,0,3,126,0,0,111,128,0,
15,240,0,0,126,0,0,15,192,0,1,248,0,0,63,0,
0,7,224,0,0,252,0,0,31,128,0,1,240,0,1,191,
0,0,51,224,0,6,124,0,1,135,192,0,48,124,0,12,
7,192,3,0,127,1,192,3,255,240,0,15,240,0,255,255,
248,1,255,255,254,0,63,0,127,0,126,0,63,0,252,0,
31,1,248,0,63,3,240,0,62,7,224,0,126,15,192,0,
124,31,128,0,248,63,0,1,248,126,0,3,240,252,0,7,
225,248,0,15,195,240,0,31,135,224,0,63,15,192,0,126,
31,128,0,252,63,0,1,240,126,0,3,224,252,0,7,193,
248,0,31,3,240,0,62,7,224,0,248,15,192,7,224,31,
128,63,131,255,255,252,7,255,255,192,0,255,255,255,31,255,
255,224,63,0,124,7,224,3,128,252,0,48,31,128,7,3,
240,0,224,126,12,12,15,193,129,129,248,48,48,63,6,0,
7,225,192,0,255,248,0,31,255,0,3,240,224,0,126,12,
0,15,193,128,1,248,48,12,63,6,1,135,224,0,112,252,
0,12,31,128,1,131,240,0,112,126,0,14,15,192,3,193,
248,1,243,255,255,254,127,255,255,192,255,255,255,127,255,255,
131,240,7,193,248,0,224,252,0,48,126,0,28,63,0,14,
31,128,3,15,193,129,135,224,192,195,240,96,1,248,48,0,
252,56,0,127,252,0,63,254,0,31,135,0,15,193,128,7,
224,192,3,240,96,1,248,48,0,252,0,0,126,0,0,63,
0,0,31,128,0,15,192,0,7,224,0,63,255,192,31,255,
224,0,0,15,240,8,0,127,254,56,1,252,7,248,7,224,
1,248,15,128,0,120,31,0,0,120,63,0,0,56,62,0,
0,56,126,0,0,24,124,0,0,24,252,0,0,0,252,0,
0,0,252,0,0,0,252,0,0,0,252,0,0,0,252,0,
0,0,252,0,127,255,252,0,127,255,124,0,1,248,126,0,
1,248,62,0,1,248,63,0,1,248,31,0,1,248,15,128,
1,248,7,224,1,248,1,252,7,248,0,127,254,120,0,15,
248,24,255,252,63,255,255,252,63,255,15,192,3,240,15,192,
3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,192,
3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,192,
3,240,15,255,255,240,15,255,255,240,15,192,3,240,15,192,
3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,192,
3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,192,
3,240,15,192,3,240,15,192,3,240,255,252,63,255,255,252,
63,255,255,255,255,255,7,224,7,224,7,224,7,224,7,224,
7,224,7,224,7,224,7,224,7,224,7,224,7,224,7,224,
7,224,7,224,7,224,7,224,7,224,7,224,7,224,7,224,
7,224,7,224,7,224,255,255,255,255,7,255,240,127,255,0,
31,128,1,248,0,31,128,1,248,0,31,128,1,248,0,31,
128,1,248,0,31,128,1,248,0,31,128,1,248,0,31,128,
1,248,0,31,128,1,248,0,31,128,1,248,48,31,135,129,
248,252,31,143,193,248,252,31,7,131,224,56,124,0,255,0,
255,252,7,255,255,252,7,255,15,192,0,224,15,192,1,192,
15,192,3,128,15,192,6,0,15,192,12,0,15,192,56,0,
15,192,112,0,15,192,224,0,15,193,192,0,15,195,192,0,
15,199,224,0,15,207,224,0,15,251,240,0,15,243,248,0,
15,225,248,0,15,192,252,0,15,192,254,0,15,192,126,0,
15,192,63,0,15,192,31,128,15,192,31,192,15,192,15,192,
15,192,7,224,15,192,7,240,255,252,63,255,255,252,63,255,
255,255,0,255,255,0,15,192,0,15,192,0,15,192,0,15,
192,0,15,192,0,15,192,0,15,192,0,15,192,0,15,192,
0,15,192,0,15,192,0,15,192,0,15,192,0,15,192,0,
15,192,0,15,192,3,15,192,3,15,192,3,15,192,7,15,
192,7,15,192,6,15,192,14,15,192,30,15,192,126,255,255,
254,255,255,254,255,192,0,3,255,255,224,0,7,255,15,224,
0,7,240,13,240,0,13,240,13,240,0,13,240,13,240,0,
13,240,12,248,0,25,240,12,248,0,25,240,12,124,0,49,
240,12,124,0,49,240,12,62,0,97,240,12,62,0,97,240,
12,31,0,193,240,12,31,0,193,240,12,31,0,193,240,12,
15,129,129,240,12,15,129,129,240,12,7,195,1,240,12,7,
195,1,240,12,3,230,1,240,12,3,230,1,240,12,1,252,
1,240,12,1,252,1,240,12,1,252,1,240,12,0,248,1,
240,12,0,248,1,240,255,192,112,31,255,255,192,112,31,255,
255,224,3,255,255,224,3,255,15,240,0,48,15,248,0,48,
13,252,0,48,12,254,0,48,12,126,0,48,12,63,0,48,
12,31,128,48,12,31,192,48,12,15,224,48,12,7,240,48,
12,3,240,48,12,1,248,48,12,1,252,48,12,0,254,48,
12,0,127,48,12,0,63,48,12,0,31,176,12,0,31,240,
12,0,15,240,12,0,7,240,12,0,3,240,12,0,1,240,
12,0,0,240,12,0,0,240,255,192,0,112,255,192,0,48,
0,63,224,0,15,131,224,0,240,7,128,31,0,31,1,240,
0,124,15,0,1,224,248,0,15,143,192,0,126,124,0,1,
243,224,0,15,191,0,0,127,248,0,3,255,192,0,31,254,
0,0,255,240,0,7,255,128,0,63,252,0,1,255,224,0,
15,223,0,0,124,252,0,7,231,224,0,63,31,0,1,240,
252,0,31,131,224,0,248,15,128,15,128,30,0,240,0,124,
31,0,0,127,192,0,255,255,248,31,255,255,192,63,0,254,
7,224,7,192,252,0,124,31,128,15,195,240,1,248,126,0,
63,15,192,7,225,248,0,252,63,0,31,7,224,3,224,252,
0,248,31,128,126,3,255,255,0,126,0,0,15,192,0,1,
248,0,0,63,0,0,7,224,0,0,252,0,0,31,128,0,
3,240,0,0,126,0,0,15,192,0,1,248,0,3,255,240,
0,127,254,0,0,0,63,224,0,15,131,224,0,240,7,128,
31,0,31,1,240,0,124,15,128,3,224,248,0,15,143,192,
0,126,124,0,1,243,224,0,15,191,0,0,127,248,0,3,
255,192,0,31,254,0,0,255,240,0,7,255,128,0,63,252,
0,1,255,224,0,15,223,0,0,124,248,0,3,231,224,0,
63,31,0,1,240,248,28,15,131,225,24,248,15,144,111,128,
30,131,240,0,126,31,0,0,127,240,16,0,3,128,128,0,
30,12,0,0,255,224,0,3,255,0,0,31,240,0,0,127,
128,0,1,248,0,0,7,128,255,255,240,0,127,255,255,0,
3,240,15,192,1,248,1,240,0,252,0,124,0,126,0,63,
0,63,0,31,128,31,128,15,192,15,192,7,224,7,224,3,
240,3,240,1,240,1,248,1,240,0,252,3,224,0,127,255,
128,0,63,1,240,0,31,128,124,0,15,192,63,0,7,224,
15,192,3,240,7,224,1,248,3,240,0,252,1,248,0,126,
0,252,0,63,0,126,0,31,128,63,3,15,192,31,129,135,
224,7,224,255,255,1,240,223,255,128,63,192,7,248,33,255,
238,60,7,231,128,30,112,0,239,0,14,240,0,111,0,6,
248,0,15,224,0,255,224,7,255,224,63,255,3,255,248,15,
255,192,127,254,0,127,224,0,63,0,1,240,0,15,192,0,
252,0,15,192,0,238,0,14,240,1,207,192,60,239,255,8,
63,192,127,255,255,231,255,255,254,120,31,129,231,1,248,14,
96,31,128,110,1,248,7,192,31,128,60,1,248,3,192,31,
128,60,1,248,3,0,31,128,0,1,248,0,0,31,128,0,
1,248,0,0,31,128,0,1,248,0,0,31,128,0,1,248,
0,0,31,128,0,1,248,0,0,31,128,0,1,248,0,0,
31,128,0,1,248,0,0,31,128,0,1,248,0,7,255,254,
0,127,255,224,255,252,3,255,255,252,3,255,15,192,0,48,
15,192,0,48,15,192,0,48,15,192,0,48,15,192,0,48,
15,192,0,48,15,192,0,48,15,192,0,48,15,192,0,48,
15,192,0,48,15,192,0,48,15,192,0,48,15,192,0,48,
15,192,0,48,15,192,0,48,15,192,0,48,15,192,0,48,
15,192,0,48,15,192,0,48,7,192,0,48,7,192,0,96,
3,224,0,224,1,240,1,192,0,252,7,128,0,127,254,0,
0,15,248,0,255,252,1,255,255,254,0,255,195,240,0,6,
0,252,0,6,0,126,0,3,0,63,128,3,128,15,192,1,
128,7,240,1,192,1,248,0,192,0,254,0,96,0,63,0,
96,0,31,128,48,0,7,224,48,0,3,240,24,0,1,252,
28,0,0,126,12,0,0,63,142,0,0,15,198,0,0,7,
243,0,0,1,251,0,0,0,253,128,0,0,63,128,0,0,
31,192,0,0,15,224,0,0,3,224,0,0,1,240,0,0,
0,112,0,0,0,56,0,0,255,252,127,254,15,255,255,241,
255,248,63,240,252,0,126,0,12,3,240,1,252,0,48,15,
224,3,240,1,192,31,128,15,192,6,0,126,0,127,128,24,
0,252,1,254,0,192,3,240,7,248,3,0,15,224,51,240,
28,0,31,128,207,192,96,0,126,7,63,129,128,1,252,24,
126,14,0,3,240,97,248,48,0,15,195,135,240,192,0,31,
140,15,198,0,0,126,48,63,24,0,1,253,128,126,224,0,
3,246,1,251,0,0,15,248,7,252,0,0,63,192,15,240,
0,0,127,0,63,128,0,1,252,0,254,0,0,3,224,1,
240,0,0,15,128,7,192,0,0,60,0,15,0,0,0,112,
0,56,0,0,1,192,0,224,0,127,254,31,254,63,255,15,
255,1,252,0,96,0,127,0,112,0,31,192,112,0,7,224,
48,0,3,248,48,0,0,254,56,0,0,63,56,0,0,31,
216,0,0,7,248,0,0,1,252,0,0,0,254,0,0,0,
63,128,0,0,15,224,0,0,15,240,0,0,15,252,0,0,
6,127,0,0,6,31,128,0,7,15,224,0,7,3,248,0,
3,0,252,0,3,0,127,0,3,128,31,192,1,128,7,240,
1,128,3,248,63,252,15,255,255,254,7,255,240,255,252,1,
255,255,254,0,255,195,248,0,14,0,254,0,6,0,63,128,
7,0,31,192,3,0,7,240,3,0,1,252,3,128,0,254,
1,128,0,63,129,128,0,15,225,192,0,7,240,192,0,1,
252,224,0,0,127,224,0,0,63,224,0,0,15,240,0,0,
3,240,0,0,1,248,0,0,0,252,0,0,0,126,0,0,
0,63,0,0,0,31,128,0,0,15,192,0,0,7,224,0,
0,3,240,0,0,1,248,0,0,15,255,192,0,7,255,224,
0,127,255,252,255,255,249,248,7,227,192,31,199,0,63,28,
0,252,56,1,248,96,7,224,192,31,193,128,63,0,0,252,
0,3,248,0,7,224,0,31,128,0,63,0,0,252,0,3,
248,6,7,224,12,31,128,24,127,0,48,252,0,227,240,1,
199,224,7,31,128,14,127,0,60,252,1,251,255,255,247,255,
255,224,15,248,0,224,240,15,131,224,124,15,3,224,124,14,
3,224,0,31,0,15,248,7,231,192,248,62,15,1,240,248,
15,143,128,124,124,3,227,224,31,15,1,120,62,19,248,127,
15,192,255,0,1,254,0,0,124,0,0,248,0,1,240,0,
3,224,0,7,192,0,15,128,0,31,0,0,62,0,0,124,
0,0,249,252,1,254,30,3,240,14,7,192,30,15,128,30,
31,0,60,62,0,124,124,0,248,248,1,241,240,3,227,224,
7,199,192,15,143,128,30,31,0,60,62,0,240,126,1,192,
246,15,1,195,240,0,3,252,7,7,7,7,199,131,231,129,
243,192,115,224,1,240,0,248,0,124,0,62,0,31,0,7,
128,3,192,12,240,6,56,6,14,14,1,252,0,0,15,240,
0,31,224,0,7,192,0,15,128,0,31,0,0,62,0,0,
124,0,0,248,0,1,240,0,3,224,0,7,192,15,207,128,
240,127,3,128,126,15,0,124,60,0,248,120,1,241,240,3,
227,224,7,199,192,15,143,128,31,31,0,62,62,0,124,60,
0,248,120,1,240,120,3,224,112,15,192,120,127,240,63,159,
224,1,252,1,224,224,112,14,30,0,231,128,28,240,3,254,
0,127,192,15,255,255,255,0,3,224,0,124,0,7,128,0,
248,0,207,0,24,240,6,15,3,128,63,128,0,127,0,120,
224,60,124,30,31,15,135,195,224,224,248,0,62,0,15,128,
3,224,0,248,3,255,224,255,248,3,224,0,248,0,62,0,
15,128,3,224,0,248,0,62,0,15,128,3,224,0,248,0,
62,0,15,128,3,224,0,248,1,255,224,127,248,0,3,248,
240,112,121,199,131,206,56,14,99,192,120,30,3,192,240,30,
7,128,240,28,7,0,240,120,3,131,128,55,240,1,0,0,
12,0,0,96,0,3,255,224,31,255,192,127,255,7,255,252,
240,3,255,0,7,248,0,63,192,1,238,0,14,120,0,240,
240,30,1,255,192,255,0,0,255,0,0,31,0,0,31,0,
0,31,0,0,31,0,0,31,0,0,31,0,0,31,0,0,
31,0,0,31,0,0,31,15,192,31,49,224,31,64,240,31,
128,248,31,128,248,31,0,248,31,0,248,31,0,248,31,0,
248,31,0,248,31,0,248,31,0,248,31,0,248,31,0,248,
31,0,248,31,0,248,255,231,255,255,231,255,30,7,224,252,
31,131,240,60,0,0,0,0,0,0,0,0,15,241,254,7,
192,248,31,3,224,124,15,129,240,62,7,192,248,31,3,224,
124,15,143,255,255,192,0,120,3,240,15,192,63,0,252,1,
224,0,0,0,0,0,0,0,0,0,0,3,252,15,240,7,
192,31,0,124,1,240,7,192,31,0,124,1,240,7,192,31,
0,124,1,240,7,192,31,0,124,1,240,7,192,31,112,127,
225,255,135,190,60,112,224,126,0,255,0,3,252,0,1,240,
0,7,192,0,31,0,0,124,0,1,240,0,7,192,0,31,
0,0,124,0,1,240,0,7,195,254,31,15,248,124,14,1,
240,112,7,195,0,31,24,0,125,192,1,255,128,7,255,0,
31,188,0,124,248,1,241,240,7,195,192,31,15,128,124,31,
1,240,62,63,241,255,255,199,252,255,31,224,124,15,129,240,
62,7,192,248,31,3,224,124,15,129,240,62,7,192,248,31,
3,224,124,15,129,240,62,7,192,248,31,3,224,124,127,255,
254,255,15,192,126,7,249,143,12,120,7,208,60,129,224,63,
1,248,15,129,248,15,192,124,15,128,124,3,224,124,3,224,
31,3,224,31,0,248,31,0,248,7,192,248,7,192,62,7,
192,62,1,240,62,1,240,15,129,240,15,128,124,15,128,124,
3,224,124,3,224,31,3,224,31,0,248,255,231,255,63,255,
255,63,249,255,192,255,15,192,255,49,224,31,64,240,31,128,
248,31,128,248,31,0,248,31,0,248,31,0,248,31,0,248,
31,0,248,31,0,248,31,0,248,31,0,248,31,0,248,31,
0,248,31,0,248,255,231,255,255,231,255,1,252,0,120,60,
7,0,112,120,3,199,128,15,60,0,123,224,3,255,0,31,
248,0,255,192,7,254,0,63,240,1,247,128,15,60,0,120,
240,7,131,192,120,15,7,128,15,224,0,255,63,129,255,195,
192,126,3,192,248,3,193,240,7,195,224,7,135,192,15,143,
128,31,31,0,62,62,0,124,124,0,248,248,1,241,240,3,
195,224,15,135,192,30,15,192,120,31,193,224,62,126,0,124,
0,0,248,0,1,240,0,3,224,0,7,192,0,15,128,0,
255,224,1,255,192,0,3,240,96,60,51,129,224,126,15,0,
248,124,3,225,224,15,143,128,62,62,0,248,248,3,227,224,
15,143,128,62,62,0,248,120,3,225,240,15,131,192,62,7,
1,248,14,15,224,15,207,128,0,62,0,0,248,0,3,224,
0,15,128,0,62,0,0,248,0,31,252,0,127,240,254,62,
127,35,135,163,227,209,241,232,248,248,56,124,0,62,0,31,
0,15,128,7,192,3,224,1,240,0,248,0,124,0,62,0,
255,240,127,248,0,31,216,193,230,1,184,6,224,27,192,15,
248,31,248,127,240,127,224,127,192,31,192,31,0,126,1,248,
6,248,51,63,0,3,0,12,0,48,0,192,7,0,28,0,
240,3,192,63,255,255,241,240,7,192,31,0,124,1,240,7,
192,31,0,124,1,240,7,195,31,12,124,49,240,195,194,7,
152,15,192,255,7,248,255,7,248,31,0,248,31,0,248,31,
0,248,31,0,248,31,0,248,31,0,248,31,0,248,31,0,
248,31,0,248,31,0,248,31,0,248,31,0,248,31,1,248,
15,1,248,7,134,255,1,248,255,255,193,255,255,7,241,240,
6,7,224,56,15,128,192,63,3,0,124,24,1,240,96,3,
227,0,15,140,0,31,96,0,125,128,1,254,0,3,240,0,
15,192,0,30,0,0,120,0,0,192,0,255,143,248,255,255,
31,241,252,124,15,128,192,248,31,1,129,248,62,7,1,240,
126,12,3,225,188,24,3,227,120,96,7,206,248,192,15,216,
243,128,15,177,246,0,31,193,236,0,31,131,240,0,63,7,
224,0,124,7,192,0,120,15,0,0,224,14,0,1,192,28,
0,255,199,255,255,31,241,248,24,3,224,224,7,199,0,15,
184,0,31,192,0,126,0,0,248,0,1,240,0,15,224,0,
119,192,3,159,0,28,62,0,96,124,3,0,248,255,7,255,
252,31,240,255,193,255,255,7,241,240,6,7,224,56,15,128,
192,63,3,0,124,24,1,240,96,3,227,0,15,140,0,31,
112,0,125,128,0,254,0,3,240,0,15,192,0,30,0,0,
120,0,0,192,0,3,0,0,28,0,112,96,3,225,128,15,
140,0,62,96,0,115,0,0,248,0,0,63,255,158,15,140,
15,198,7,198,7,195,7,225,131,224,3,224,3,240,3,240,
1,240,97,240,49,248,24,248,28,248,12,252,14,124,15,127,
255,128,255,255,254,255,255,254,15,192,62,15,192,14,15,192,
6,15,192,7,15,192,7,15,192,3,15,192,3,15,192,3,
15,192,0,15,192,0,15,192,0,15,192,0,15,192,0,15,
192,0,15,192,0,15,192,0,15,192,0,15,192,0,15,192,
0,15,192,0,15,192,0,15,192,0,15,192,0,15,192,0,
255,255,0,255,255,0,0,0,224,0,0,0,62,0,0,0,
7,192,0,0,1,252,0,0,0,63,128,0,0,15,248,0,
0,1,191,0,0,0,103,240,0,0,28,127,0,0,3,15,
224,0,0,224,254,0,0,24,31,192,0,6,1,252,0,1,
192,31,192,0,48,3,248,0,14,0,63,128,1,128,7,240,
0,96,0,127,0,12,0,7,224,3,0,0,254,0,224,0,
15,224,24,0,1,252,7,0,0,31,192,255,255,255,248,63,
255,255,255,143,255,255,255,249,255,255,255,255,127,255,255,255,
240,0,31,224,0,3,192,240,0,60,0,240,3,224,1,240,
31,0,3,224,124,0,15,131,224,0,31,31,128,0,126,126,
0,1,249,240,0,3,239,195,3,15,255,15,252,63,252,63,
240,255,240,255,195,255,195,255,15,255,15,252,63,252,48,48,
255,240,0,3,247,192,0,15,159,0,0,62,126,0,1,248,
248,0,7,193,240,0,62,7,192,0,248,15,128,7,192,15,
0,60,0,15,3,192,0,15,252,0,0,6,0,0,0,240,
0,0,15,0,0,0,240,0,0,31,128,0,1,248,0,0,
31,128,0,3,252,0,0,63,192,0,7,254,0,0,103,224,
0,6,126,0,0,199,240,0,12,63,0,0,195,240,0,24,
63,128,1,129,248,0,56,31,192,3,0,252,0,48,15,192,
6,0,254,0,96,7,224,6,0,126,0,192,7,240,12,0,
63,1,192,3,248,255,131,255,255,248,63,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,255,255,255,128,0,0,
240,0,0,30,0,0,3,192,0,0,96,0,0,0,24,0,
48,3,255,254,0,127,255,192,15,255,248,1,255,255,0,63,
255,224,6,0,12,0,0,0,0,0,0,0,96,0,0,60,
0,0,7,128,0,0,240,0,0,31,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,240,255,255,255,255,255,
255,255,255,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,15,192,3,240,15,192,3,240,15,192,3,240,15,
192,3,240,255,252,63,255,255,252,63,255,255,255,255,223,255,
255,249,252,0,63,31,192,1,227,252,0,28,63,192,1,195,
248,0,56,63,128,3,3,248,0,96,127,128,12,7,248,0,
0,127,0,0,7,240,0,0,127,0,0,15,224,0,0,248,
0,0,14,0,0,3,128,12,0,224,1,128,48,0,48,12,
0,14,3,0,1,193,192,0,112,112,0,30,28,0,15,199,
255,255,249,255,255,255,127,255,255,224,31,128,7,224,255,0,
63,199,255,3,255,191,254,15,255,251,248,127,127,1,243,224,
48,3,207,0,0,15,252,0,0,31,224,0,0,127,128,0,
0,252,0,0,3,240,0,0,15,192,0,0,63,0,0,0,
252,0,0,3,240,0,0,15,192,0,0,63,0,0,0,252,
0,0,3,240,0,0,15,192,0,0,63,0,0,0,252,0,
0,3,240,0,0,15,192,0,0,63,0,0,63,255,240,0,
255,255,192,7,255,254,0,255,255,192,0,126,0,0,15,192,
0,1,248,0,0,63,0,0,63,248,0,62,255,224,31,31,
159,7,195,241,241,240,126,31,62,15,195,239,193,248,127,248,
63,15,255,7,225,255,224,252,63,124,31,135,207,131,240,248,
248,126,62,15,143,207,128,125,255,192,1,255,192,0,7,224,
0,0,252,0,0,31,128,0,3,240,0,31,255,248,3,255,
255,0,3,255,255,0,15,255,252,0,0,252,0,0,3,240,
0,0,15,192,0,0,63,0,15,224,252,31,207,195,240,252,
63,15,195,240,252,63,15,195,240,252,63,15,195,240,252,63,
15,195,240,252,63,15,195,240,252,63,7,195,240,248,31,143,
199,224,62,63,31,0,124,252,248,0,243,243,192,1,255,254,
0,0,255,192,0,0,252,0,0,3,240,0,0,15,192,0,
0,63,0,0,63,255,240,0,255,255,192,0,63,224,0,15,
1,224,1,224,3,192,30,0,15,1,240,0,124,31,128,3,
240,248,0,15,143,192,0,126,126,0,3,243,240,0,31,159,
128,0,252,252,0,7,227,224,0,62,31,0,1,240,252,0,
31,131,224,0,248,15,0,7,128,120,0,60,1,224,3,192,
7,0,28,12,56,0,225,224,192,6,13,130,0,32,204,24,
3,6,127,192,31,243,254,0,255,143,240,7,248,127,128,63,
192,3,248,1,227,192,112,28,30,3,195,128,56,240,7,158,
0,243,192,30,248,3,255,0,127,224,15,252,1,255,128,63,
240,7,254,0,255,192,31,248,3,255,0,127,224,15,252,1,
247,128,60,240,7,142,0,225,224,60,28,7,1,227,192,15,
224,0,0,96,0,240,3,248,31,252,15,62,0,31,0,15,
128,7,192,3,224,1,240,0,248,0,124,0,62,0,31,0,
15,128,7,192,3,224,1,240,0,248,0,124,0,62,0,31,
0,15,128,7,192,3,224,63,255,223,255,224,7,248,3,255,
192,224,254,60,7,239,192,125,248,15,255,0,255,224,31,120,
3,224,0,252,0,31,0,3,224,0,248,0,62,0,7,128,
1,192,0,112,0,28,0,6,1,129,128,48,96,6,16,1,
199,255,241,255,254,127,255,223,255,251,255,255,0,3,248,3,
255,192,240,124,30,7,199,192,124,252,15,159,1,241,192,62,
0,15,128,1,240,0,120,0,30,0,63,128,0,60,0,3,
224,0,62,0,7,192,0,252,192,31,188,3,255,192,127,248,
15,255,1,243,192,124,120,31,3,255,192,31,224,0,0,1,
128,0,28,0,1,224,0,31,0,1,248,0,31,192,1,190,
0,25,240,0,143,128,12,124,0,195,224,12,31,0,192,248,
12,7,192,192,62,12,1,240,192,15,135,255,255,255,255,254,
0,31,0,0,248,0,7,192,0,62,0,1,240,0,15,128,
15,255,192,127,254,24,1,131,255,224,127,248,15,254,1,255,
128,63,128,6,0,0,192,0,24,0,3,0,0,103,224,15,
7,1,128,248,32,15,0,1,240,0,63,0,7,224,0,253,
224,31,252,3,255,128,127,240,15,188,1,243,0,124,56,31,
3,255,192,31,192,0,0,126,0,127,224,30,6,7,129,225,
224,124,120,15,143,0,227,192,0,120,0,31,0,3,227,224,
125,135,15,160,121,248,7,63,0,247,192,31,248,3,255,0,
127,224,15,188,1,247,128,62,240,7,143,0,241,224,60,30,
15,0,255,192,15,224,0,96,0,15,255,253,255,255,191,255,
231,255,248,255,255,56,0,198,0,48,192,12,24,3,0,0,
192,0,24,0,6,0,1,192,0,56,0,15,0,1,192,0,
120,0,15,0,1,224,0,60,0,15,128,1,240,0,62,0,
7,192,0,248,0,31,0,1,192,0,3,248,1,255,192,120,
62,28,3,195,128,60,240,7,158,0,243,240,30,127,131,143,
248,224,255,248,15,254,0,255,224,15,255,7,127,225,195,254,
120,31,254,0,255,192,15,248,0,255,0,31,224,3,222,0,
115,192,28,62,7,131,255,192,15,224,0,3,248,1,255,192,
120,60,30,3,199,128,120,240,7,190,0,247,192,30,248,3,
255,0,127,224,15,252,1,247,128,126,112,15,207,2,248,112,
223,3,227,224,0,124,0,15,0,1,227,128,124,248,15,31,
1,195,192,120,56,60,3,255,0,63,128,0,0,192,0,96,
1,252,3,255,131,204,227,134,25,131,13,193,143,224,207,240,
103,252,51,255,24,239,236,3,255,1,255,224,127,248,31,254,
7,255,128,255,192,27,242,12,251,134,63,227,15,241,135,240,
195,224,97,152,49,206,24,195,141,224,255,192,31,128,1,128,
0,192,0,0,60,0,0,0,227,0,0,1,193,0,0,3,
193,128,0,3,129,128,0,7,129,128,0,7,193,128,0,7,
195,0,0,7,194,0,0,7,196,0,0,7,200,0,0,3,
240,1,255,3,224,1,255,3,224,0,48,3,240,0,96,1,
240,0,192,3,248,0,192,4,252,1,128,28,252,3,0,56,
126,3,0,120,63,6,0,120,31,140,0,248,31,216,0,248,
15,240,0,252,7,240,6,124,1,248,6,126,7,254,28,31,
254,63,248,7,240,7,240,15,240,48,60,96,30,240,31,248,
31,248,31,112,31,0,62,0,60,0,112,0,224,1,192,1,
128,1,128,3,0,3,0,3,0,3,0,3,0,0,0,0,
0,0,0,0,0,7,128,15,192,15,192,15,192,15,192,7,
128,7,128,15,192,15,192,15,192,15,192,7,128,0,0,0,
0,0,0,0,0,3,0,3,0,3,0,3,0,3,0,7,
0,6,0,6,0,14,0,28,0,60,0,120,0,248,14,248,
31,248,31,248,15,120,6,56,28,31,240,255,31,224,124,15,
129,240,62,7,192,248,31,3,224,124,15,129,240,62,7,192,
248,255,255,252,3,252,15,240,7,192,31,0,124,1,240,7,
192,31,0,124,1,240,7,192,31,0,124,1,240,7,192,31,
0,124,1,240,7,192,31,112,127,225,255,135,190,60,112,224,
126,0,0,254,0,14,14,0,248,62,3,192,120,31,1,240,
124,7,193,240,31,7,192,124,31,1,224,124,15,1,240,120,
63,199,128,255,3,128,124,7,1,240,14,7,192,60,31,0,
248,124,1,225,240,7,199,192,31,31,0,124,124,1,241,240,
7,199,192,31,31,24,120,124,241,225,243,207,63,198,120,255,
15,128,15,248,31,192,56,62,240,224,248,63,128,225,240,63,
1,227,224,124,1,195,128,248,3,192,1,240,7,128,3,224,
15,0,255,255,254,31,143,128,0,248,31,0,3,224,62,0,
15,128,60,0,31,0,124,0,254,0,248,1,252,3,248,6,
126,30,120,56,63,224,31,192,1,252,3,248,3,193,227,195,
129,192,29,192,112,240,7,240,30,120,0,248,3,158,0,62,
0,255,128,15,128,63,224,3,224,15,248,0,255,255,254,0,
62,0,15,128,15,128,3,224,3,224,0,120,0,248,0,31,
0,127,0,51,192,31,192,12,120,15,120,6,15,7,143,7,
0,127,0,127,0,0,0,32,0,2,0,0,48,0,1,0,
31,208,7,131,192,120,15,7,128,188,120,4,243,192,71,190,
6,63,240,33,255,130,15,252,16,127,225,3,255,24,31,120,
128,243,200,7,143,64,120,60,3,128,240,120,2,254,0,32,
0,3,0,0,16,0,1,0,0,0,0,31,255,255,240,0,
63,255,255,224,0,12,252,7,192,0,49,248,3,128,0,99,
240,3,0,1,199,224,7,0,3,15,192,14,0,6,31,128,
12,0,24,63,0,24,0,48,126,12,48,0,192,252,24,0,
1,129,248,48,0,7,3,240,96,0,12,7,225,192,0,31,
255,255,128,0,127,255,255,0,0,192,63,14,0,3,128,126,
12,12,6,0,252,24,24,12,1,248,48,48,48,3,240,96,
192,96,7,224,1,129,128,15,192,7,3,0,31,128,14,14,
0,63,0,60,24,0,126,1,243,255,7,255,255,231,254,15,
255,255,192,0,31,255,255,255,0,30,7,255,255,224,15,0,
63,0,124,3,192,7,224,3,128,240,0,252,0,48,62,0,
31,128,7,15,128,3,240,0,227,240,0,126,1,140,126,0,
15,192,49,143,128,0,248,6,51,240,0,31,1,192,126,0,
3,224,120,15,192,0,127,255,1,248,0,15,255,224,63,0,
1,240,60,7,224,0,62,3,128,252,0,7,192,48,31,128,
0,248,6,13,248,0,31,0,193,191,0,7,224,0,119,224,
0,252,0,12,124,0,31,128,1,135,192,3,240,0,112,252,
0,126,0,14,15,128,15,192,3,192,124,1,248,1,240,3,
192,255,255,254,0,15,255,255,255,192,0,0,0,64,0,0,
3,0,1,254,8,0,60,15,64,3,192,15,0,62,0,31,
1,240,0,126,7,128,2,120,62,0,9,241,248,0,71,231,
192,2,15,159,0,24,62,252,0,64,255,240,2,3,255,192,
24,15,255,0,64,63,252,2,0,255,240,24,3,255,192,64,
15,255,2,0,63,124,24,0,249,240,64,3,231,226,0,31,
143,144,0,124,30,64,1,224,126,0,15,128,248,0,124,0,
240,3,192,2,240,60,0,16,255,192,0,192,0,0,2,0,
0,0,123,255,255,255,255,222,121,231,158,121,231,140,48,195,
0,0,0,30,255,255,255,120,123,255,255,253,224,0,0,3,
12,48,199,158,121,231,158,121,239,255,255,255,255,120,0,1,
192,112,0,0,224,56,0,0,112,28,0,0,112,28,0,0,
56,14,0,0,28,7,0,0,14,3,128,0,14,3,128,0,
7,1,192,0,3,128,224,0,1,192,112,0,1,192,112,15,
255,255,255,255,255,255,255,253,255,255,255,254,0,56,14,0,
0,56,14,0,0,28,7,0,0,14,3,128,0,7,1,192,
0,3,128,224,0,3,128,224,1,255,255,255,255,255,255,255,
255,255,255,255,255,128,112,28,0,0,112,28,0,0,56,14,
0,0,28,7,0,0,14,3,128,0,14,3,128,0,7,1,
192,0,3,128,224,0,1,192,112,0,1,192,112,0,0,224,
56,0,0,112,28,0,0,7,128,0,24,12,48,0,28,14,
12,0,30,14,7,128,62,7,1,63,238,7,128,192,14,3,
192,96,15,1,224,48,15,0,240,24,7,0,120,12,7,0,
60,6,7,128,30,3,7,128,7,1,3,128,3,129,131,128,
0,224,131,192,0,48,131,192,0,7,129,192,120,0,1,192,
194,0,1,224,224,128,0,224,224,96,0,224,112,16,0,240,
120,12,0,240,60,6,0,112,30,3,0,112,15,1,128,120,
7,128,192,120,3,192,96,56,1,224,48,56,0,112,16,60,
0,56,24,28,0,14,8,28,0,3,8,6,0,0,120,0,
121,251,247,255,239,64,129,4,8,32,67,8,0,0,32,8,
6,1,192,48,12,3,128,224,28,7,128,224,28,7,128,240,
30,7,192,240,30,3,192,120,15,1,224,60,7,128,240,31,
1,224,60,7,128,112,14,1,224,28,3,128,56,3,0,48,
7,0,96,2,0,32,128,8,0,192,28,1,128,24,3,128,
56,7,0,240,14,1,192,60,7,128,240,31,1,224,60,7,
128,240,30,3,192,120,15,1,224,124,15,1,224,60,7,0,
224,60,7,0,224,56,6,1,128,112,12,2,0,128,0,1,
192,1,240,0,112,8,56,47,28,127,206,127,247,126,125,124,
7,240,3,248,15,175,159,187,255,156,255,142,61,7,4,3,
128,3,224,0,224,0,0,3,128,0,0,14,0,0,0,56,
0,0,0,224,0,0,3,128,0,0,14,0,0,0,56,0,
0,0,224,0,0,3,128,0,0,14,0,0,0,56,0,0,
0,224,0,0,3,128,0,0,14,0,15,255,255,255,255,255,
255,255,255,255,255,252,0,14,0,0,0,56,0,0,0,224,
0,0,3,128,0,0,14,0,0,0,56,0,0,0,224,0,
0,3,128,0,0,14,0,0,0,56,0,0,0,224,0,0,
3,128,0,0,14,0,0,0,56,0,0,121,251,247,255,239,
64,129,4,8,32,67,8,0,123,255,255,253,224,0,1,128,
1,192,0,224,0,224,0,112,0,56,0,56,0,28,0,28,
0,14,0,7,0,7,0,3,128,1,192,1,192,0,224,0,
224,0,112,0,56,0,56,0,28,0,14,0,14,0,7,0,
3,128,3,128,1,192,1,192,0,224,0,112,0,112,0,56,
0,28,0,28,0,14,0,14,0,7,0,3,128,3,128,1,
192,0,192,0,0,123,255,255,253,224,0,0,0,0,123,255,
255,253,224,123,255,255,253,224,0,0,0,0,115,239,255,253,
240,65,8,32,132,33,0,255,255,255,255,255,255,255,247,255,
255,255,192,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,7,255,255,255,
255,255,255,255,255,255,255,252,0,31,224,0,3,128,112,0,
48,0,48,1,128,0,96,12,0,0,192,96,0,1,131,0,
252,3,8,14,24,4,96,240,16,25,131,192,124,108,30,1,
240,240,120,7,195,195,224,31,15,15,128,124,60,62,1,240,
240,248,7,195,195,224,31,15,7,128,124,60,30,1,240,216,
60,7,194,96,240,31,24,128,225,188,67,0,252,126,6,0,
0,0,12,0,0,0,24,0,0,0,48,0,3,192,56,1,
252,0,31,252,0,255,255,255,14,28,56,112,225,195,135,14,
28,56,112,225,195,135,14,28,56,112,225,195,135,14,28,56,
112,225,195,135,14,28,63,255,254,255,255,248,112,225,195,135,
14,28,56,112,225,195,135,14,28,56,112,225,195,135,14,28,
56,112,225,195,135,14,28,56,112,225,255,255,254,4,48,129,
4,8,32,64,189,255,251,247,231,128,120,62,63,143,195,240,
124,30,3,15,15,143,231,231,231,195,193,128,128,60,31,239,
159,192,224,8,0,128,12,0,96,2,128,38,3,24,48,127,
1,240,255,255,255,255,192,62,49,176,120,60,27,24,248,15,
128,126,0,124,1,240,15,128,249,255,144,0,0,240,255,255,
143,0,2,0,56,3,224,59,135,143,112,29,0,64,123,255,
255,253,224,112,119,199,254,63,241,255,143,248,123,131,152,24,
64,64,30,2,127,249,255,228,7,128,56,56,248,251,241,255,
227,247,199,199,7,0,0,63,227,248,3,224,126,56,15,0,
248,248,62,3,225,240,248,7,195,225,240,15,131,131,224,31,
0,7,192,62,0,15,128,124,0,31,0,248,0,62,1,240,
7,255,255,254,15,255,255,252,1,240,15,128,3,224,31,0,
7,192,62,0,15,128,124,0,31,0,248,0,62,1,240,0,
124,3,224,0,248,7,192,1,240,15,128,3,224,31,0,7,
192,62,0,15,128,124,0,31,0,248,0,62,1,240,3,255,
15,254,7,254,31,252,0,0,63,192,0,248,24,0,240,14,
0,248,15,128,248,7,192,124,3,224,62,0,224,31,0,0,
15,128,0,7,192,0,3,224,0,31,255,255,143,255,255,192,
124,3,224,62,1,240,31,0,248,15,128,124,7,192,62,3,
224,31,1,240,15,128,248,7,192,124,3,224,62,1,240,31,
0,248,15,128,124,7,192,62,3,224,31,15,252,63,247,254,
31,248,0,63,252,0,248,62,0,240,31,0,248,15,128,248,
7,192,124,3,224,62,1,240,31,0,248,15,128,124,7,192,
62,3,224,31,31,255,255,143,255,255,192,124,3,224,62,1,
240,31,0,248,15,128,124,7,192,62,3,224,31,1,240,15,
128,248,7,192,124,3,224,62,1,240,31,0,248,15,128,124,
7,192,62,3,224,31,15,254,127,247,255,63,248,0,63,193,
254,0,7,192,254,6,0,60,3,224,28,1,240,31,128,248,
15,128,124,3,224,62,1,240,15,128,248,7,192,28,3,224,
31,0,0,15,128,124,0,0,62,1,240,0,0,248,7,192,
0,63,255,255,255,248,255,255,255,255,224,62,1,240,15,128,
248,7,192,62,3,224,31,0,248,15,128,124,3,224,62,1,
240,15,128,248,7,192,62,3,224,31,0,248,15,128,124,3,
224,62,1,240,15,128,248,7,192,62,3,224,31,0,248,15,
128,124,3,224,62,1,240,15,128,248,7,192,62,31,248,127,
195,255,127,225,255,15,252,0,63,225,255,224,7,192,254,15,
128,60,3,224,62,1,240,31,128,248,15,128,124,3,224,62,
1,240,15,128,248,7,192,62,3,224,31,0,248,15,128,124,
3,224,62,1,240,15,128,248,7,192,62,63,255,255,255,248,
255,255,255,255,224,62,1,240,15,128,248,7,192,62,3,224,
31,0,248,15,128,124,3,224,62,1,240,15,128,248,7,192,
62,3,224,31,0,248,15,128,124,3,224,62,1,240,15,128,
248,7,192,62,3,224,31,0,248,15,128,124,3,224,62,1,
240,15,128,248,7,192,62,31,252,255,231,255,127,243,255,159,
252,120,30,126,31,191,15,223,199,255,227,251,208,244,8,2,
4,1,4,1,2,0,130,0,129,0,67,0,194,0,128,255,
255,255,255,255,240,4,1,12,3,8,2,4,1,4,1,2,
0,130,0,129,0,64,188,47,127,31,255,143,239,195,247,225,
249,224,120,255,255,254,255,255,255,255,255,254,
};
METRIC cmbx5_metric[128] = {
{1217, -2, 13, 14, 14, 293996},
{1242, -2, 13, 20, 14, 406862},
{1277, -2, 13, 18, 14, 380466},
{1309, -1, 13, 17, 14, 338596},
{1339, -1, 13, 17, 14, 327674},
{1369, -2, 13, 18, 14, 377735},
{1401, -2, 13, 16, 14, 354070},
{1429, -2, 13, 18, 14, 380466},
{1461, -2, 13, 16, 14, 354070},
{1489, -2, 13, 18, 14, 380466},
{1521, -1, 13, 18, 14, 354070},
{2431, 0, 13, 19, 14, 299002},
{2465, 0, 13, 14, 14, 285804},
{2490, 0, 13, 14, 14, 285804},
{2515, 0, 13, 23, 14, 428706},
{2556, 0, 13, 23, 14, 428706},
{1832, -1, 8, 5, 9, 142902},
{1838, 1, 8, 8, 13, 156100},
{2391, -3, 13, 5, 4, 248486},
{2394, -6, 13, 5, 4, 248486},
{2397, -4, 13, 6, 3, 248486},
{2400, -3, 13, 8, 4, 248486},
{2404, -2, 12, 10, 1, 248486},
{2406, -7, 13, 7, 3, 364992},
{2409, -3, -1, 8, 4, 222090},
{1851, -2, 13, 11, 14, 249854},
{1871, -1, 8, 18, 9, 354070},
{1892, -1, 8, 20, 9, 380466},
{1915, -1, 10, 12, 13, 248486},
{1935, -1, 13, 24, 14, 438719},
{1977, -2, 13, 26, 14, 491511},
{2023, -2, 14, 18, 16, 380466},
{2413, 0, 7, 7, 2, 142902},
{2059, -2, 13, 4, 14, 155645},
{2597, -1, 13, 11, 8, 252128},
{2073, -2, 13, 20, 18, 406862},
{1744, -2, 15, 10, 17, 248486},
{2118, -2, 15, 20, 17, 406862},
{1766, -1, 13, 19, 14, 380466},
{2161, -2, 13, 4, 8, 142902},
{2165, -1, 15, 7, 22, 195694},
{2185, -2, 15, 7, 22, 195694},
{2205, -1, 15, 12, 9, 248486},
{2219, -2, 14, 18, 20, 380466},
{2264, -2, 3, 4, 8, 142902},
{2608, 0, 5, 7, 3, 169298},
{2268, -2, 3, 4, 4, 142902},
{2270, -2, 15, 10, 21, 248486},
{1553, -1, 12, 12, 13, 248486},
{1573, -2, 12, 10, 13, 248486},
{1590, -2, 12, 10, 13, 248486},
{1607, -1, 12, 12, 13, 248486},
{1627, -1, 12, 12, 13, 248486},
{1647, -2, 12, 10, 13, 248486},
{1664, -1, 12, 12, 13, 248486},
{1684, -2, 12, 12, 13, 248486},
{1704, -1, 12, 12, 13, 248486},
{1724, -1, 12, 12, 13, 248486},
{2297, -2, 8, 4, 9, 142902},
{2302, -2, 8, 4, 13, 142902},
{2066, -2, 9, 4, 14, 155645},
{2309, -2, 8, 18, 8, 380466},
{1816, -2, 9, 9, 14, 235288},
{1800, -2, 13, 9, 14, 235288},
{2327, -2, 13, 18, 14, 380466},
{0, -1, 13, 19, 14, 364992},
{34, -2, 13, 16, 14, 346333},
{62, -2, 13, 16, 14, 354070},
{90, -2, 13, 18, 14, 372729},
{122, -2, 13, 15, 14, 320392},
{149, -2, 13, 14, 14, 307194},
{174, -2, 13, 19, 14, 383197},
{208, -2, 13, 18, 14, 377735},
{240, -1, 13, 7, 14, 180220},
{253, -2, 13, 11, 14, 253947},
{273, -2, 13, 18, 14, 378190},
{305, -2, 13, 14, 14, 293996},
{330, -2, 13, 23, 14, 456923},
{371, -2, 13, 18, 14, 377735},
{403, -2, 13, 17, 14, 367723},
{433, -2, 13, 15, 14, 333135},
{460, -2, 13, 17, 18, 367723},
{499, -2, 13, 19, 14, 361352},
{533, -2, 13, 11, 14, 274882},
{553, -1, 13, 18, 14, 341327},
{585, -2, 13, 18, 14, 371364},
{617, -1, 13, 19, 14, 364992},
{651, -1, 13, 27, 14, 496972},
{699, -1, 13, 19, 14, 364992},
{733, -1, 13, 19, 14, 364992},
{767, -2, 13, 13, 14, 301278},
{2359, -2, 15, 5, 21, 143794},
{2611, -2, 13, 11, 8, 252128},
{2373, 0, 15, 5, 21, 143794},
{2415, -4, 13, 6, 4, 248486},
{2418, -2, 14, 3, 4, 142902},
{2387, -2, 13, 4, 8, 142902},
{790, -1, 8, 12, 9, 241887},
{804, -2, 13, 12, 14, 274882},
{825, -1, 8, 10, 9, 222090},
{837, -1, 13, 12, 14, 274882},
{858, -1, 8, 10, 9, 226869},
{870, 0, 13, 10, 14, 156100},
{888, -1, 8, 13, 14, 248486},
{911, -2, 13, 11, 14, 274882},
{931, -1, 14, 5, 15, 142902},
{941, 1, 14, 8, 19, 156100},
{960, -2, 13, 13, 14, 261684},
{983, -1, 13, 5, 14, 142902},
{992, -2, 8, 21, 9, 406862},
{1016, -2, 8, 11, 9, 274882},
{1029, -1, 8, 12, 9, 248486},
{1043, -2, 8, 12, 13, 274882},
{1063, -1, 8, 13, 13, 261684},
{1085, -2, 8, 9, 9, 203887},
{1096, -1, 8, 9, 9, 198334},
{1107, -1, 12, 7, 13, 195694},
{1119, -2, 8, 11, 9, 274882},
{1132, -1, 8, 13, 9, 261684},
{1147, -1, 8, 18, 9, 354070},
{1168, -1, 8, 13, 9, 261684},
{1183, -1, 8, 13, 13, 261684},
{1205, -1, 8, 10, 9, 222090},
{2622, 0, 5, 14, 1, 248486},
{2624, 0, 5, 29, 1, 496972},
{2420, -3, 14, 8, 4, 248486},
{2424, -3, 13, 8, 3, 248486},
{2427, -3, 14, 8, 4, 248486},
};
char cmbx5_bitmap[2628] = {
0,224,0,62,0,7,192,1,220,0,51,128,14,120,1,135,
0,48,224,12,14,1,255,192,96,28,12,3,131,0,57,248,
31,192,255,248,56,28,56,14,56,14,56,14,56,28,63,248,
56,30,56,7,56,7,56,7,56,7,56,14,255,248,7,241,
28,15,48,7,112,3,96,3,224,0,224,0,224,0,224,0,
96,3,112,3,48,6,28,12,7,240,255,248,14,3,131,128,
48,224,14,56,1,206,0,115,128,28,224,7,56,1,206,0,
115,128,56,224,12,56,14,63,254,0,255,248,112,112,224,113,
192,99,140,7,24,15,240,28,96,56,192,112,0,224,25,192,
115,129,223,255,128,255,248,224,227,129,206,3,57,128,230,3,
248,14,96,57,128,224,3,128,14,0,56,3,252,0,3,248,
131,129,240,224,14,56,0,198,0,25,192,0,56,0,7,1,
255,224,3,140,0,113,192,14,28,1,193,192,56,7,251,0,
254,31,206,1,195,128,112,224,28,56,7,14,1,195,255,240,
224,28,56,7,14,1,195,128,112,224,28,56,7,63,135,240,
254,112,225,195,135,14,28,56,112,225,195,159,192,31,224,112,
14,1,192,56,7,0,224,28,3,128,113,14,113,206,48,252,
0,254,15,206,1,195,129,192,224,224,56,112,14,112,3,190,
0,255,128,60,112,14,14,3,129,192,224,56,56,7,63,131,
240,255,0,224,3,128,14,0,56,0,224,3,128,14,0,56,
0,224,3,128,206,7,56,59,255,224,248,0,62,120,0,240,
240,1,225,176,6,195,96,13,134,96,51,12,192,102,24,193,
140,48,198,24,97,140,48,193,176,97,131,96,195,3,129,159,
135,15,192,248,15,207,0,195,224,48,220,12,51,131,12,112,
195,14,48,195,140,48,115,12,14,195,1,240,192,60,48,7,
63,0,192,7,240,14,14,12,1,142,0,238,0,63,0,31,
128,15,192,7,224,3,240,1,220,1,199,1,193,193,192,63,
128,255,240,112,56,224,57,192,115,128,231,1,206,7,31,248,
56,0,112,0,224,1,192,3,128,31,192,0,7,240,14,14,
12,1,142,0,238,0,63,0,31,128,15,192,7,224,3,240,
1,220,1,198,56,193,227,192,63,132,0,98,0,63,0,15,
0,7,0,255,240,7,3,128,224,56,28,7,3,128,224,112,
56,15,252,1,193,192,56,28,7,3,128,224,112,28,14,51,
129,199,252,15,128,63,76,59,3,96,108,1,248,31,224,254,
0,224,15,1,224,62,13,127,0,127,255,152,56,110,14,15,
3,131,0,224,0,56,0,14,0,3,128,0,224,0,56,0,
14,0,3,128,0,224,1,255,0,254,15,206,0,195,128,48,
224,12,56,3,14,0,195,128,48,224,12,56,3,14,0,195,
128,48,96,24,14,12,0,254,0,254,7,231,0,48,224,14,
14,1,129,192,48,28,12,3,129,128,56,96,7,12,0,115,
0,14,96,0,248,0,31,0,1,192,0,252,31,135,231,1,
224,48,224,60,6,28,7,128,193,193,184,48,56,55,6,7,
142,225,192,113,142,48,14,49,198,0,236,29,128,29,131,176,
3,240,126,0,60,7,128,7,128,240,0,255,31,195,129,192,
56,48,3,140,0,59,0,3,224,0,120,0,15,128,3,248,
0,231,128,56,120,14,7,129,128,121,252,31,192,254,7,231,
128,112,120,28,7,3,0,112,192,15,48,0,254,0,15,128,
0,224,0,28,0,3,128,0,112,0,14,0,7,240,0,255,
255,3,176,61,129,192,28,1,192,30,1,224,14,0,224,14,
6,240,55,3,255,252,62,7,48,115,128,120,127,142,56,227,
142,56,60,240,240,7,0,112,7,0,112,7,248,112,231,6,
112,119,7,112,119,6,112,198,248,31,156,118,31,128,224,56,
6,1,195,31,128,1,224,14,0,224,14,0,225,254,48,230,
14,224,238,14,224,230,14,113,225,255,31,28,102,15,131,255,
248,6,1,195,31,128,7,131,113,220,112,28,63,129,192,112,
28,7,1,192,112,28,15,192,30,241,9,216,100,195,2,16,
31,1,128,15,248,63,227,255,176,13,128,102,6,31,224,240,
14,1,192,56,7,0,239,30,115,142,113,206,57,199,56,231,
29,243,192,103,188,192,3,206,115,156,231,59,224,6,15,15,
6,0,0,31,7,7,7,7,7,7,7,7,7,231,230,124,
240,3,128,28,0,224,7,0,57,249,199,14,224,126,3,248,
28,224,227,135,6,124,124,243,156,231,57,206,115,156,231,124,
243,227,227,227,227,156,28,28,224,224,231,7,7,56,56,57,
193,193,206,14,14,248,248,248,247,143,57,199,56,231,28,227,
156,115,142,249,224,31,135,14,96,110,7,224,126,7,224,119,
14,31,128,255,135,14,112,103,7,112,119,7,112,103,12,127,
135,0,112,7,0,248,0,31,177,135,152,29,192,238,7,112,
57,129,198,30,31,240,3,128,28,0,224,15,128,239,53,220,
238,7,3,129,192,224,252,0,63,97,176,223,199,240,63,7,
194,254,0,24,48,225,207,231,14,28,56,112,249,241,192,243,
206,57,199,56,231,28,227,156,115,158,63,224,248,123,129,142,
24,112,193,204,14,96,62,1,240,7,0,249,243,220,48,103,
14,56,231,140,57,243,7,205,129,243,96,124,248,14,28,0,
248,249,199,7,112,31,0,112,7,192,119,7,28,248,248,252,
123,193,142,24,56,193,236,7,96,62,0,240,3,2,24,57,
129,156,7,128,0,255,248,236,112,60,30,15,3,141,199,255,
192,255,248,224,227,129,206,3,56,0,224,3,128,14,0,56,
0,224,3,128,14,0,56,3,252,0,0,240,0,31,128,1,
184,0,51,192,7,30,0,96,224,12,15,1,192,120,24,3,
131,128,60,48,1,199,255,254,255,255,255,255,255,7,248,7,
3,131,128,113,192,14,99,49,184,252,126,63,31,143,199,227,
49,248,0,119,0,56,224,28,28,14,1,254,0,1,192,1,
240,0,248,0,124,0,103,0,51,128,49,224,24,112,12,56,
12,14,6,7,7,3,195,0,231,225,252,255,255,255,255,240,
0,96,0,0,0,0,192,96,127,240,63,248,24,12,0,0,
0,0,24,0,63,255,255,255,252,255,255,206,1,195,128,112,
224,28,56,7,14,1,195,128,112,224,28,56,7,14,1,195,
128,112,224,28,56,7,63,135,240,255,254,120,14,60,7,30,
3,14,0,7,0,3,128,1,128,3,0,14,3,28,7,56,
14,127,254,255,254,60,15,63,199,252,59,140,6,192,1,224,
0,56,0,14,0,3,128,0,224,0,56,0,14,0,3,128,
0,224,1,255,0,15,248,1,192,1,192,15,240,121,222,225,
199,225,199,225,199,225,199,121,222,15,240,1,192,1,192,15,
248,7,252,0,56,0,14,3,195,143,56,231,14,57,195,142,
112,227,156,28,238,3,187,0,63,128,3,128,0,224,1,255,
0,7,248,7,3,131,128,113,192,14,112,3,156,0,231,0,
56,224,28,24,6,3,3,12,64,143,248,127,126,31,159,135,
224,31,131,12,96,110,7,224,126,7,224,126,7,224,126,7,
96,103,14,31,128,6,63,128,224,56,14,3,128,224,56,14,
3,128,224,56,127,192,62,24,238,31,135,1,192,96,56,24,
12,6,51,253,254,255,128,31,131,12,112,231,14,1,192,248,
0,224,7,0,126,7,224,118,14,63,128,3,128,120,15,129,
184,19,131,56,99,140,56,255,240,56,3,128,56,31,240,65,
159,199,225,192,96,31,196,24,7,65,248,126,29,142,62,0,
15,193,134,48,230,14,224,14,248,240,238,7,224,126,7,96,
115,14,31,128,127,247,254,255,204,24,3,0,96,14,0,192,
28,1,192,28,1,192,8,0,31,131,12,112,231,14,124,195,
248,63,199,62,224,254,7,224,119,14,63,128,31,135,12,224,
110,7,224,126,7,112,241,247,0,119,6,112,226,24,31,0,
4,1,3,241,150,196,241,126,95,240,127,15,224,127,147,228,
241,54,88,248,4,0,7,128,1,140,0,113,128,14,48,1,
204,0,59,15,199,128,96,112,24,63,6,12,241,131,143,96,
112,248,7,15,134,127,127,128,127,113,248,232,224,192,192,96,
48,0,0,6,7,131,192,192,24,30,15,3,0,0,0,96,
48,24,24,24,92,126,59,248,243,156,231,57,206,248,31,7,
7,7,7,7,7,7,7,7,231,230,124,31,6,49,199,56,
231,25,230,28,115,134,112,238,29,195,185,247,61,227,0,63,
159,28,60,103,14,12,3,131,31,255,220,56,14,14,3,131,
195,127,159,128,31,143,195,13,134,96,112,62,7,3,224,127,
254,7,0,96,112,3,13,131,31,143,224,0,32,4,31,135,
30,97,110,39,230,126,71,232,119,142,31,130,0,64,0,3,
255,252,1,156,28,1,156,14,3,28,6,3,28,0,6,28,
48,6,28,48,15,255,240,12,28,48,12,28,48,24,28,3,
24,28,7,48,28,14,252,127,254,7,255,255,7,3,193,195,
128,240,57,192,60,6,96,15,12,56,1,195,14,0,127,195,
128,28,48,224,7,12,60,3,192,7,0,240,12,224,60,7,
28,15,3,129,255,255,224,0,3,1,254,129,192,224,192,44,
112,19,152,12,110,2,31,129,7,224,129,248,64,126,48,29,
200,14,52,3,7,3,129,127,128,192,0,111,255,246,102,0,
111,246,111,246,0,102,111,255,246,0,97,128,6,24,0,97,
128,12,48,0,195,15,255,255,255,255,240,48,192,3,12,0,
48,192,3,12,15,255,255,255,255,240,195,0,12,48,1,134,
0,24,96,1,134,0,56,0,198,224,28,231,207,142,99,48,
230,6,14,96,192,230,24,6,195,0,56,96,0,12,28,1,
131,32,48,115,6,7,48,192,115,24,7,51,0,50,16,1,
192,111,247,18,36,2,24,96,195,14,24,48,225,195,135,14,
28,24,48,112,96,96,192,192,64,128,192,193,129,131,131,6,
14,28,56,112,225,195,6,28,48,193,134,16,0,6,0,96,
230,119,254,15,7,254,230,112,96,6,0,0,192,0,48,0,
12,0,3,0,0,192,0,48,0,12,0,3,0,0,192,63,
255,255,255,252,3,0,0,192,0,48,0,12,0,3,0,0,
192,0,48,0,12,0,3,0,111,247,18,36,111,246,0,64,
48,24,6,1,128,192,48,24,6,3,0,192,48,24,6,3,
0,192,96,24,6,3,0,192,0,111,246,6,255,96,111,246,
6,255,113,34,64,255,255,255,255,240,0,0,0,0,0,0,
0,0,15,255,255,255,255,7,248,7,3,131,0,49,143,134,
71,28,179,135,60,225,207,56,115,206,28,209,199,102,62,240,
192,0,28,3,193,255,128,255,241,140,99,24,198,49,140,99,
24,198,49,255,128,255,198,49,140,99,24,198,49,140,99,24,
199,255,128,36,72,239,246,231,190,48,59,255,128,135,247,128,
129,195,126,60,255,192,125,141,240,62,15,7,254,15,224,49,
236,225,255,240,66,231,231,231,49,126,140,231,255,255,231,3,
255,193,199,156,112,227,142,28,1,195,129,255,254,7,14,0,
225,192,28,56,3,135,0,112,224,14,28,1,195,128,124,252,
0,3,240,56,97,195,135,4,28,3,255,225,195,135,14,28,
56,112,225,195,135,14,28,56,249,240,3,248,56,225,195,135,
14,28,59,255,225,195,135,14,28,56,112,225,195,135,14,28,
56,249,240,3,243,240,28,60,24,112,112,112,224,224,65,193,
192,31,255,255,135,7,7,14,14,14,28,28,28,56,56,56,
112,112,112,224,224,225,193,193,199,199,199,192,3,251,252,28,
60,56,112,112,112,224,224,225,193,193,223,255,255,135,7,7,
14,14,14,28,28,28,56,56,56,112,112,112,224,224,225,193,
193,199,199,199,192,96,222,63,199,184,113,2,64,136,18,4,
255,255,248,32,72,17,2,64,142,29,227,252,123,6,255,252,
255,255,255,248,
};
METRIC cmbx7_metric[128] = {
{2327, -2, 19, 17, 20, 354804},
{2370, -2, 19, 26, 20, 492430},
{2435, -2, 19, 24, 20, 459844},
{2495, -1, 19, 23, 20, 411420},
{2553, -2, 19, 20, 20, 394672},
{2603, -2, 19, 24, 20, 459662},
{2663, -2, 19, 22, 20, 427258},
{2718, -2, 19, 24, 20, 459844},
{2778, -2, 19, 22, 20, 427258},
{2833, -2, 19, 24, 20, 459844},
{2893, -2, 19, 22, 20, 427258},
{4565, 0, 19, 24, 20, 349433},
{4625, 0, 19, 19, 20, 333140},
{4673, 0, 19, 19, 20, 333140},
{4721, 0, 19, 30, 20, 499710},
{4796, 0, 19, 30, 20, 499710},
{3465, -1, 12, 8, 13, 166570},
{3478, 2, 12, 11, 19, 182863},
{4492, -4, 19, 6, 5, 296914},
{4496, -8, 19, 6, 5, 296914},
{4500, -5, 18, 8, 4, 296914},
{4504, -4, 19, 10, 6, 296914},
{4512, -3, 17, 12, 2, 296914},
{4515, -9, 19, 9, 5, 444006},
{4521, -4, -1, 10, 6, 264328},
{3505, -1, 19, 16, 20, 306836},
{3545, -1, 12, 24, 13, 427258},
{3584, -1, 12, 26, 13, 459844},
{3627, -1, 15, 16, 19, 296914},
{3665, -1, 19, 31, 20, 533390},
{3743, -3, 19, 33, 20, 598562},
{3826, -2, 20, 24, 22, 459844},
{4529, 0, 11, 10, 4, 166570},
{3892, -3, 19, 5, 20, 182226},
{4871, -1, 19, 14, 11, 306380},
{3919, -2, 19, 26, 26, 492430},
{3295, -2, 21, 14, 24, 296914},
{4004, -2, 21, 26, 24, 492430},
{3337, -2, 19, 24, 20, 459844},
{4082, -3, 19, 6, 11, 166570},
{4091, -3, 21, 9, 29, 231742},
{4124, -2, 21, 9, 29, 231742},
{4157, -2, 21, 14, 13, 296914},
{4180, -2, 19, 24, 25, 459844},
{4255, -3, 4, 5, 11, 166570},
{4891, 0, 7, 10, 3, 199156},
{4262, -3, 4, 5, 5, 166570},
{4266, -2, 21, 14, 29, 296914},
{2948, -1, 18, 16, 19, 296914},
{2986, -3, 18, 12, 19, 296914},
{3015, -2, 18, 14, 19, 296914},
{3049, -2, 18, 14, 19, 296914},
{3083, -1, 18, 16, 19, 296914},
{3121, -2, 18, 14, 19, 296914},
{3155, -2, 18, 14, 19, 296914},
{3189, -2, 19, 15, 20, 296914},
{3227, -2, 18, 14, 19, 296914},
{3261, -2, 18, 14, 19, 296914},
{4317, -3, 12, 5, 13, 166570},
{4326, -3, 12, 5, 19, 166570},
{3905, -3, 14, 5, 21, 182226},
{4338, -2, 12, 24, 11, 459844},
{3430, -2, 14, 13, 21, 280621},
{3397, -2, 19, 13, 20, 280621},
{4371, -2, 19, 24, 20, 459844},
{0, -1, 19, 25, 20, 444006},
{63, -2, 19, 22, 20, 419339},
{118, -2, 19, 22, 20, 427258},
{173, -2, 19, 24, 20, 451925},
{233, -2, 19, 21, 20, 387390},
{286, -2, 19, 19, 20, 371097},
{334, -2, 19, 25, 20, 464031},
{397, -2, 19, 24, 20, 459662},
{457, -2, 19, 10, 20, 219909},
{482, -2, 19, 13, 20, 305288},
{515, -2, 19, 25, 20, 460299},
{578, -2, 19, 17, 20, 354804},
{621, -3, 19, 28, 20, 557420},
{691, -2, 19, 24, 20, 459662},
{751, -2, 19, 23, 20, 444188},
{809, -2, 19, 21, 20, 403046},
{862, -2, 19, 23, 26, 444188},
{937, -2, 19, 25, 20, 440365},
{1000, -2, 19, 16, 20, 329500},
{1040, -1, 19, 23, 20, 411602},
{1098, -2, 19, 24, 20, 451834},
{1158, -1, 19, 25, 20, 444006},
{1221, -1, 19, 35, 20, 606936},
{1309, -1, 19, 25, 20, 444006},
{1372, -1, 19, 25, 20, 444006},
{1435, -2, 19, 18, 20, 362086},
{4431, -3, 21, 7, 29, 166989},
{4895, -3, 19, 14, 11, 306380},
{4457, 0, 21, 7, 29, 166989},
{4534, -5, 19, 8, 5, 296914},
{4539, -2, 20, 6, 5, 166570},
{4483, -2, 19, 6, 11, 166570},
{1480, -1, 12, 16, 13, 288767},
{1506, -1, 19, 18, 20, 329500},
{1551, -1, 12, 14, 13, 264328},
{1574, -1, 19, 18, 20, 329500},
{1619, -1, 12, 14, 13, 271747},
{1642, 0, 19, 14, 20, 182863},
{1677, -1, 12, 16, 20, 296914},
{1717, -1, 19, 18, 20, 329500},
{1762, -1, 20, 8, 21, 166570},
{1783, 2, 20, 11, 27, 182863},
{1821, -1, 19, 17, 20, 313207},
{1864, -1, 19, 8, 20, 166570},
{1884, -1, 12, 28, 13, 492430},
{1930, -1, 12, 18, 13, 329500},
{1960, -1, 12, 16, 13, 296914},
{1986, -1, 12, 18, 19, 329500},
{2029, -1, 12, 18, 19, 313207},
{2072, -1, 12, 13, 13, 245214},
{2094, -1, 12, 12, 13, 235001},
{2114, -1, 17, 11, 18, 231742},
{2139, -1, 12, 18, 13, 329500},
{2169, -1, 12, 17, 13, 313207},
{2197, -1, 12, 24, 13, 427258},
{2236, -1, 12, 17, 13, 313207},
{2264, -1, 12, 17, 19, 313207},
{2305, -1, 12, 13, 13, 264328},
{4915, 0, 8, 18, 1, 296914},
{4918, 0, 8, 37, 1, 593828},
{4543, -5, 20, 11, 6, 296914},
{4552, -3, 19, 12, 3, 296914},
{4557, -3, 20, 12, 5, 296914},
};
char cmbx7_bitmap[4923] = {
0,28,0,0,14,0,0,15,128,0,7,192,0,7,240,0,
3,120,0,3,190,0,1,143,0,0,199,128,0,193,224,0,
96,240,0,96,60,0,48,30,0,63,255,128,31,255,192,28,
1,240,12,0,120,6,0,60,63,192,127,255,224,63,240,255,
255,131,255,255,129,224,31,7,128,62,30,0,248,120,3,225,
224,15,135,128,60,30,1,224,127,255,1,224,31,7,128,62,
30,0,124,120,1,241,224,7,199,128,31,30,0,248,120,7,
239,255,255,63,255,240,0,255,12,15,255,112,252,7,199,192,
15,60,0,29,240,0,55,128,0,254,0,0,248,0,3,224,
0,15,128,0,62,0,0,248,0,1,224,0,55,192,0,207,
0,7,31,0,24,63,1,192,63,254,0,63,192,255,255,128,
255,255,224,30,1,248,30,0,124,30,0,60,30,0,30,30,
0,30,30,0,31,30,0,31,30,0,31,30,0,31,30,0,
31,30,0,31,30,0,30,30,0,30,30,0,62,30,0,124,
30,1,248,255,255,224,255,255,128,255,255,231,255,255,7,128,
120,60,0,193,224,7,15,0,24,120,48,195,193,128,30,28,
0,255,224,7,255,0,60,56,1,224,193,143,6,12,120,0,
99,192,7,30,0,112,240,7,191,255,253,255,255,224,255,255,
223,255,248,120,15,15,0,97,224,14,60,0,199,131,24,240,
96,30,28,3,255,128,127,240,15,14,1,224,192,60,24,7,
128,0,240,0,30,0,3,192,3,255,128,127,240,0,0,255,
12,1,255,238,3,240,31,3,224,7,131,192,1,195,224,0,
97,224,0,49,240,0,0,248,0,0,124,0,0,62,0,0,
31,0,255,255,128,127,251,192,1,225,240,0,240,120,0,120,
31,0,60,7,224,30,0,255,255,0,31,249,128,255,195,255,
255,195,255,30,0,120,30,0,120,30,0,120,30,0,120,30,
0,120,30,0,120,30,0,120,31,255,248,31,255,248,30,0,
120,30,0,120,30,0,120,30,0,120,30,0,120,30,0,120,
30,0,120,255,195,255,255,195,255,255,255,241,224,120,30,7,
129,224,120,30,7,129,224,120,30,7,129,224,120,30,7,143,
255,255,31,248,255,192,120,3,192,30,0,240,7,128,60,1,
224,15,0,120,3,192,30,0,240,7,140,60,241,231,142,24,
240,126,0,255,192,255,127,224,127,135,128,28,3,192,24,1,
224,56,0,240,56,0,120,56,0,60,56,0,30,60,0,15,
127,0,7,247,128,3,241,224,1,224,120,0,240,30,0,120,
7,128,60,3,224,30,0,240,15,0,60,63,240,63,255,248,
31,240,255,224,127,240,7,128,3,192,1,224,0,240,0,120,
0,60,0,30,0,15,0,7,128,3,192,1,224,24,240,12,
120,14,60,7,30,7,15,7,191,255,223,255,224,254,0,7,
255,240,0,255,31,0,15,129,240,0,248,27,128,27,129,184,
1,184,25,192,51,129,156,3,56,24,224,99,129,142,6,56,
24,224,99,129,135,12,56,24,112,195,129,131,152,56,24,57,
131,129,129,240,56,24,31,3,129,129,240,56,255,14,31,255,
240,225,255,254,0,255,255,0,255,31,128,24,27,192,24,27,
224,24,25,224,24,24,240,24,24,120,24,24,60,24,24,30,
24,24,31,24,24,15,24,24,7,152,24,3,216,24,1,248,
24,0,248,24,0,248,24,0,120,255,0,56,255,0,24,1,
255,0,15,7,128,120,3,193,224,3,195,192,7,143,0,7,
158,0,15,124,0,31,248,0,63,240,0,127,224,0,255,192,
1,255,128,3,239,0,7,159,0,31,30,0,60,62,0,248,
60,1,224,30,15,0,15,248,0,255,255,7,255,254,7,128,
124,60,1,225,224,15,143,0,124,120,3,227,192,31,30,0,
240,240,15,7,255,240,60,0,1,224,0,15,0,0,120,0,
3,192,0,30,0,0,240,0,63,240,1,255,128,0,1,255,
0,15,7,128,120,3,193,240,7,195,192,7,143,128,15,158,
0,15,124,0,31,248,0,63,240,0,127,224,0,255,192,1,
255,128,3,239,0,7,159,0,31,30,0,60,60,120,120,61,
9,224,30,15,0,15,248,16,0,56,32,0,63,192,0,127,
128,0,254,0,0,252,0,0,224,255,254,0,127,255,224,7,
128,248,3,192,60,1,224,31,0,240,15,128,120,7,192,60,
3,192,30,3,192,15,255,128,7,129,192,3,192,112,1,224,
60,0,240,30,0,120,15,0,60,7,192,30,3,225,143,1,
248,255,240,124,95,248,15,192,15,226,63,254,112,30,224,14,
224,6,224,6,240,0,254,0,127,240,63,252,15,254,0,254,
0,15,0,7,192,7,192,7,224,6,240,14,255,252,143,240,
127,255,252,255,255,249,192,240,115,1,224,110,3,192,248,7,
128,240,15,1,128,30,0,0,60,0,0,120,0,0,240,0,
1,224,0,3,192,0,7,128,0,15,0,0,30,0,0,60,
0,0,120,0,15,255,0,31,254,0,255,192,255,255,192,255,
30,0,24,30,0,24,30,0,24,30,0,24,30,0,24,30,
0,24,30,0,24,30,0,24,30,0,24,30,0,24,30,0,
24,30,0,24,30,0,24,30,0,24,15,0,48,7,128,224,
3,255,192,0,127,0,255,128,63,255,192,31,199,128,3,1,
224,3,0,240,1,128,124,1,192,30,0,192,15,128,224,3,
192,96,1,240,112,0,120,48,0,60,24,0,15,24,0,7,
140,0,1,236,0,0,246,0,0,62,0,0,31,0,0,7,
0,0,3,128,0,255,135,254,15,255,240,255,193,252,120,3,
192,12,15,0,124,1,129,240,15,128,112,30,1,240,12,3,
192,111,1,128,60,13,224,96,7,131,30,12,0,248,99,195,
128,15,12,120,96,1,227,7,140,0,30,96,243,0,3,220,
31,96,0,127,1,252,0,7,224,63,0,0,248,3,224,0,
15,0,120,0,1,224,15,0,0,56,0,224,0,127,193,254,
63,224,255,3,224,24,0,248,24,0,62,28,0,15,28,0,
3,220,0,1,252,0,0,124,0,0,31,0,0,15,128,0,
15,224,0,14,248,0,6,62,0,6,15,128,7,3,224,7,
0,240,7,0,124,63,224,127,255,240,63,240,255,192,63,255,
224,31,199,192,7,1,240,7,0,124,3,0,31,3,128,7,
131,128,3,225,128,0,249,128,0,63,192,0,15,192,0,3,
192,0,1,224,0,0,240,0,0,120,0,0,60,0,0,30,
0,0,15,0,0,63,240,0,31,248,0,127,255,223,255,247,
192,121,192,60,112,31,24,15,134,3,192,1,224,0,240,0,
124,0,62,0,15,0,7,128,195,192,49,240,28,248,7,60,
3,158,1,239,255,251,255,254,31,224,120,112,120,56,120,60,
48,60,1,252,31,60,120,60,240,60,240,60,240,60,120,95,
63,143,252,0,63,0,3,192,0,240,0,60,0,15,0,3,
192,0,247,224,63,14,15,1,195,192,56,240,15,60,3,207,
0,243,192,60,240,15,60,3,143,1,195,224,224,199,224,7,
240,112,243,131,220,15,240,27,192,15,0,60,0,240,1,192,
3,128,199,6,7,240,0,63,0,15,192,0,240,0,60,0,
15,0,3,192,0,240,31,188,28,31,14,3,199,0,243,192,
60,240,15,60,3,207,0,243,192,60,112,15,14,3,193,195,
252,31,191,15,192,112,195,131,156,6,240,31,192,127,255,252,
0,240,1,192,3,128,199,6,7,240,3,240,28,112,227,199,
143,30,24,120,1,224,63,240,255,192,120,1,224,7,128,30,
0,120,1,224,7,128,30,0,120,7,248,31,224,15,206,56,
119,112,63,112,56,112,56,112,56,56,112,47,192,32,0,96,
0,63,240,63,252,63,254,112,15,224,7,224,7,224,7,112,
14,56,28,15,240,252,0,63,0,3,192,0,240,0,60,0,
15,0,3,192,0,241,240,61,142,15,131,195,192,240,240,60,
60,15,15,3,195,192,240,240,60,60,15,15,3,207,243,255,
252,255,56,124,124,124,56,0,0,0,252,252,60,60,60,60,
60,60,60,60,60,255,255,1,192,124,15,129,240,28,0,0,
0,0,7,224,252,7,128,240,30,3,192,120,15,1,224,60,
7,128,240,30,3,216,127,143,241,220,112,252,0,252,0,126,
0,15,0,7,128,3,192,1,224,0,240,0,120,126,60,63,
30,28,15,24,7,184,3,252,1,255,0,243,192,120,240,60,
56,30,14,63,207,255,231,240,252,252,60,60,60,60,60,60,
60,60,60,60,60,60,60,60,60,60,255,255,252,124,31,15,
216,230,56,62,15,131,195,192,240,60,60,15,3,195,192,240,
60,60,15,3,195,192,240,60,60,15,3,195,192,240,60,60,
15,3,207,243,252,255,255,63,207,240,252,124,63,99,131,224,
240,240,60,60,15,15,3,195,192,240,240,60,60,15,15,3,
195,192,243,252,255,255,63,192,7,224,28,56,120,30,112,14,
240,15,240,15,240,15,240,15,240,15,112,14,120,30,60,60,
7,224,253,248,63,195,131,192,112,240,14,60,3,207,0,243,
192,60,240,15,60,3,207,1,227,192,112,248,56,61,248,15,
0,3,192,0,240,0,60,0,63,192,15,240,0,7,227,7,
13,195,129,241,224,60,240,15,60,3,207,0,243,192,60,240,
15,28,3,195,128,240,112,252,7,239,0,3,192,0,240,0,
60,0,15,0,15,240,3,252,248,231,203,206,158,116,243,195,
30,0,240,7,128,60,1,224,15,1,254,15,240,0,31,230,
14,192,108,6,254,7,252,63,225,255,0,124,3,192,63,6,
223,128,12,1,128,48,14,1,192,127,191,241,224,60,7,128,
240,30,3,192,120,207,25,227,28,65,240,252,63,63,15,195,
192,240,240,60,60,15,15,3,195,192,240,240,60,60,15,15,
3,195,193,240,112,255,15,207,192,254,15,255,7,207,1,131,
193,129,224,192,120,192,60,96,15,96,7,176,1,240,0,248,
0,56,0,28,0,254,127,31,254,127,31,60,28,12,60,62,
28,30,62,24,30,63,24,15,103,48,15,103,176,15,231,240,
7,195,224,7,195,224,3,129,192,3,129,192,254,63,255,31,
199,134,1,230,0,126,0,30,0,15,128,7,224,7,248,7,
60,7,15,31,135,255,195,248,254,15,255,7,207,129,131,193,
129,240,192,120,192,62,96,15,96,7,240,1,240,0,248,0,
60,0,28,0,14,1,134,1,231,0,243,0,115,0,31,0,
0,127,251,135,152,124,195,198,60,3,192,62,1,224,30,25,
224,223,12,240,239,255,0,255,255,127,255,135,129,195,192,97,
224,56,240,12,120,6,60,0,30,0,15,0,7,128,3,192,
1,224,0,240,0,120,0,60,0,30,0,15,0,63,248,31,
252,0,0,30,0,0,7,128,0,3,240,0,0,252,0,0,
111,128,0,57,240,0,12,60,0,6,15,128,1,129,224,0,
192,124,0,112,15,128,24,1,224,12,0,124,3,0,15,1,
128,3,224,224,0,124,48,0,15,31,255,255,239,255,255,255,
255,255,255,0,255,0,7,129,224,15,0,240,30,0,120,60,
0,60,124,0,62,120,0,30,248,195,31,248,255,31,248,255,
31,248,255,31,248,195,31,248,0,31,120,0,30,124,0,62,
60,0,60,62,0,124,15,0,240,7,129,224,0,255,0,0,
56,0,0,112,0,1,240,0,3,224,0,15,224,0,27,192,
0,55,128,0,207,128,1,143,0,7,31,0,12,30,0,24,
60,0,96,124,0,192,120,3,128,248,6,0,240,12,1,224,
48,3,227,252,31,255,248,63,240,255,255,255,255,255,255,255,
252,0,3,192,0,48,0,0,24,1,129,128,24,31,255,129,
255,248,31,255,129,128,24,24,1,128,0,0,192,0,60,0,
3,192,0,63,255,255,255,255,255,255,255,255,255,255,255,255,
255,30,0,120,30,0,120,30,0,120,30,0,120,30,0,120,
30,0,120,30,0,120,30,0,120,30,0,120,30,0,120,30,
0,120,30,0,120,30,0,120,30,0,120,30,0,120,30,0,
120,255,195,255,255,195,255,255,255,251,255,255,231,192,7,143,
128,14,31,0,28,62,0,48,124,0,192,248,0,1,240,0,
7,224,0,15,128,0,24,0,0,192,0,6,0,48,48,0,
195,128,7,28,0,56,224,1,231,255,255,191,255,254,30,0,
120,127,129,254,255,195,255,255,231,255,1,231,128,0,126,0,
0,126,0,0,60,0,0,60,0,0,60,0,0,60,0,0,
60,0,0,60,0,0,60,0,0,60,0,0,60,0,0,60,
0,0,60,0,3,255,192,3,255,192,7,255,128,31,254,0,
7,128,0,30,0,0,120,0,15,252,1,231,158,31,30,62,
248,120,127,225,225,255,135,135,254,30,31,124,120,248,121,231,
128,63,240,0,30,0,0,120,0,1,224,0,127,248,1,255,
224,3,255,192,3,255,192,0,60,0,0,60,0,0,60,0,
248,60,31,60,60,60,60,60,60,60,60,60,60,60,60,60,
60,60,30,60,120,14,60,112,7,189,224,1,255,128,0,60,
0,0,60,0,0,60,0,3,255,192,3,255,192,3,255,0,
60,15,3,192,15,31,0,62,120,0,123,224,1,255,128,7,
254,0,31,248,0,125,224,1,231,192,15,143,0,60,28,0,
224,56,7,12,96,24,240,128,67,227,3,29,252,15,231,240,
63,159,192,254,7,224,28,56,56,28,112,14,112,14,240,15,
240,15,240,15,240,15,240,15,240,15,240,15,240,15,240,15,
112,14,112,14,56,28,28,56,7,224,1,128,248,255,143,120,
7,128,120,7,128,120,7,128,120,7,128,120,7,128,120,7,
128,120,7,143,255,255,240,31,192,255,198,15,188,31,240,61,
128,240,7,192,30,0,120,3,192,28,0,224,6,12,48,49,
129,207,254,127,251,255,239,255,128,15,192,255,135,15,30,30,
120,120,195,224,15,0,120,15,192,3,192,7,128,31,0,125,
129,255,7,252,30,224,249,255,193,252,0,0,56,0,120,0,
248,1,248,3,120,6,120,12,120,24,120,48,120,96,120,192,
120,255,255,255,255,0,120,0,120,0,120,0,120,7,255,7,
255,96,25,255,199,254,31,240,127,1,128,6,0,27,240,112,
113,128,224,3,128,15,96,63,192,255,3,248,14,112,112,255,
129,248,0,3,240,63,225,225,206,15,120,61,192,111,0,60,
240,252,115,224,239,3,252,15,240,63,192,247,3,220,14,56,
120,127,192,252,0,96,0,255,253,255,251,255,231,255,156,6,
48,24,96,96,1,192,3,0,14,0,24,0,112,0,224,3,
192,7,128,15,0,30,0,60,0,48,0,15,192,127,195,135,
28,14,112,57,224,231,227,15,248,31,224,127,195,63,156,63,
224,63,128,126,1,248,7,120,56,255,192,252,0,15,192,255,
135,135,28,14,240,59,192,255,3,252,15,240,61,193,243,143,
195,207,0,61,128,239,7,188,28,224,225,255,1,240,0,3,
0,12,0,252,15,252,115,57,140,126,51,248,207,243,27,252,
7,252,15,252,31,248,15,230,51,252,199,243,31,140,126,49,
156,222,63,240,63,0,48,0,192,1,224,0,7,48,0,14,
24,0,30,24,0,30,24,0,30,16,0,30,32,0,30,64,
127,31,128,127,15,0,24,15,128,48,15,128,96,55,192,192,
115,224,192,241,241,128,240,251,0,240,126,0,120,63,3,127,
255,254,31,193,252,63,195,7,188,31,224,246,7,0,112,14,
0,96,6,0,48,1,128,12,0,0,0,0,0,1,192,31,
0,248,7,192,28,0,14,0,248,7,192,62,0,224,0,0,
0,0,0,6,0,48,3,0,24,0,192,14,0,224,14,0,
240,127,131,252,30,224,227,252,0,252,252,60,60,60,60,60,
60,60,60,60,255,255,7,224,252,7,128,240,30,3,192,120,
15,1,224,60,7,128,240,30,3,216,127,143,241,220,112,252,
0,7,240,30,60,60,30,60,30,60,30,60,28,60,56,252,
240,252,56,60,28,60,30,60,14,60,15,60,15,60,15,60,
15,60,239,60,238,252,252,252,120,31,225,248,120,119,12,120,
62,14,120,60,7,48,60,7,0,60,7,7,255,255,62,60,
0,120,60,0,240,28,0,240,62,3,120,119,6,63,193,252,
7,224,126,7,14,112,195,129,248,57,192,60,7,240,15,1,
252,3,192,127,0,255,255,192,60,0,240,15,0,28,3,192,
3,129,248,12,112,231,6,7,224,127,0,0,4,0,12,0,
8,7,240,28,56,120,62,112,78,240,79,240,143,241,143,241,
15,242,15,114,14,124,30,60,60,15,224,16,0,48,0,32,
0,0,127,255,248,0,255,255,240,0,207,1,224,1,158,0,
192,6,60,1,192,12,120,1,128,56,240,99,0,97,224,192,
1,195,195,128,3,255,255,0,7,255,254,0,24,30,28,0,
48,60,24,96,192,120,48,193,128,240,1,135,1,224,7,12,
3,192,28,24,7,128,123,252,63,255,247,248,127,255,224,0,
255,255,254,3,192,255,255,3,192,60,7,135,192,30,0,195,
192,15,0,115,224,7,128,25,224,3,192,205,240,1,224,96,
248,0,240,112,124,0,127,248,62,0,63,252,31,0,30,14,
15,128,15,3,27,192,7,129,141,240,3,192,6,120,1,224,
7,62,0,240,7,15,128,120,7,129,224,127,255,192,31,255,
255,224,0,0,8,0,255,16,7,129,240,14,0,112,28,0,
120,56,0,156,120,1,30,112,3,14,240,6,15,240,4,15,
240,8,15,240,16,15,240,32,15,240,96,15,112,192,14,120,
128,30,57,0,28,62,0,60,14,0,112,15,129,224,8,255,
0,16,0,0,119,255,255,189,206,115,156,96,0,14,255,254,
224,119,255,247,0,0,51,156,231,57,223,255,255,247,0,0,
28,28,0,7,7,0,1,193,192,0,224,224,0,56,56,0,
14,14,0,3,131,131,255,255,255,255,255,255,223,255,255,240,
14,14,0,3,131,128,1,193,192,0,112,112,0,28,28,1,
255,255,255,255,255,255,255,255,255,240,56,56,0,28,28,0,
7,7,0,1,193,192,0,112,112,0,56,56,0,14,14,0,
3,131,128,0,30,0,6,14,96,3,135,30,3,227,195,127,
240,240,192,120,60,48,60,15,12,14,3,195,7,0,240,195,
192,28,97,224,3,144,240,0,120,56,0,0,28,30,0,15,
14,64,7,135,24,3,195,195,0,224,240,192,112,60,48,60,
15,12,30,3,195,15,0,240,195,128,28,97,192,3,144,48,
0,120,115,239,255,116,16,66,16,132,0,0,129,129,129,129,
192,192,224,112,112,56,60,30,15,7,131,193,224,240,120,60,
14,7,1,192,224,48,28,6,1,128,96,8,128,48,12,3,
1,192,96,56,28,7,3,129,224,240,120,60,30,15,7,131,
193,224,224,112,112,56,24,28,12,12,12,8,0,3,0,14,
4,48,188,207,251,124,255,192,120,15,252,251,127,204,244,48,
128,224,3,0,0,28,0,0,28,0,0,28,0,0,28,0,
0,28,0,0,28,0,0,28,0,0,28,0,0,28,0,0,
28,0,0,28,0,255,255,255,255,255,255,255,255,255,0,28,
0,0,28,0,0,28,0,0,28,0,0,28,0,0,28,0,
0,28,0,0,28,0,0,28,0,0,28,0,0,28,0,119,
255,247,132,34,17,16,119,255,247,0,0,12,0,112,1,192,
14,0,56,1,192,7,0,56,0,224,3,128,28,0,112,3,
128,14,0,56,1,192,7,0,56,0,224,7,0,28,0,112,
3,128,14,0,112,1,192,14,0,56,0,192,0,119,255,247,
0,0,119,255,247,0,119,255,247,0,0,119,255,247,132,34,
17,16,255,255,255,255,255,255,127,255,255,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,127,255,255,255,255,255,
255,255,255,0,255,0,7,0,224,12,0,48,16,0,8,48,
126,12,97,195,6,99,193,230,195,129,227,199,129,227,199,129,
227,199,129,227,199,129,227,195,129,227,99,193,226,97,195,230,
48,126,248,16,0,0,12,0,0,7,0,31,0,255,240,255,
255,255,14,28,56,112,225,195,135,14,28,56,112,225,195,135,
14,28,56,112,225,195,255,255,224,255,255,248,112,225,195,135,
14,28,56,112,225,195,135,14,28,56,112,225,195,135,14,31,
255,255,224,8,66,16,130,11,191,253,243,128,227,207,159,28,
28,247,254,224,129,255,126,24,128,96,28,13,134,63,7,128,
255,255,255,62,96,240,120,51,224,31,1,240,60,15,255,160,
0,0,199,255,251,128,24,60,126,231,129,49,239,222,48,97,
158,123,239,249,238,57,134,0,62,23,254,135,192,112,239,159,
249,255,159,112,224,1,254,252,7,131,199,14,7,143,30,7,
143,30,7,134,30,7,128,30,7,128,255,255,240,255,255,240,
30,7,128,30,7,128,30,7,128,30,7,128,30,7,128,30,
7,128,30,7,128,30,7,128,30,7,128,127,15,224,127,15,
224,1,254,0,240,96,56,30,15,3,193,224,120,60,0,7,
128,7,255,252,255,255,131,192,240,120,30,15,3,193,224,120,
60,15,7,129,224,240,60,30,7,131,192,241,252,63,191,135,
240,1,255,128,240,240,56,30,15,3,193,224,120,60,15,7,
129,231,255,252,255,255,131,192,240,120,30,15,3,193,224,120,
60,15,7,129,224,240,60,30,7,131,192,241,254,127,191,207,
240,1,254,63,192,30,7,193,128,224,60,15,7,128,240,60,
30,3,192,240,120,15,0,1,224,60,0,63,255,255,252,255,
255,255,240,120,15,3,193,224,60,15,7,128,240,60,30,3,
192,240,120,15,3,193,224,60,15,7,128,240,60,30,3,192,
240,120,15,3,199,248,255,63,223,227,252,255,1,255,63,240,
30,7,195,192,224,60,15,7,128,240,60,30,3,192,240,120,
15,3,193,224,60,15,63,255,255,252,255,255,255,240,120,15,
3,193,224,60,15,7,128,240,60,30,3,192,240,120,15,3,
193,224,60,15,7,128,240,60,30,3,192,240,120,15,3,199,
248,255,63,223,227,252,255,112,115,227,239,207,255,63,116,116,
16,16,64,66,2,16,16,128,132,4,0,255,255,255,252,8,
8,64,66,2,16,16,128,130,2,11,139,191,63,252,253,241,
243,131,128,255,255,192,255,255,255,255,248,
};
METRIC cmex10_metric[128] = {
{0, -6, 1, 11, 49, 300375},
{1212, -1, 1, 11, 49, 300375},
{2424, -8, 1, 7, 49, 273068},
{3135, -1, 1, 7, 49, 273068},
{3846, -8, 1, 10, 49, 309477},
{4395, -1, 1, 10, 49, 309477},
{4944, -8, 1, 10, 49, 309477},
{5493, -1, 1, 10, 49, 309477},
{6042, -5, 1, 13, 49, 382295},
{7153, -5, 1, 13, 49, 382295},
{8402, -4, 1, 12, 50, 309477},
{9255, -3, 1, 12, 50, 309477},
{13310, -6, 0, 2, 27, 218454},
{13317, -6, 0, 10, 27, 364090},
{10108, -2, 1, 19, 49, 378653},
{11709, -2, 1, 19, 49, 378653},
{68, -8, 1, 15, 74, 391397},
{1280, -1, 1, 15, 74, 391397},
{207, -9, 1, 20, 99, 482420},
{1419, -1, 1, 20, 99, 482420},
{2541, -10, 1, 11, 99, 345886},
{3252, 0, 1, 11, 99, 345886},
{4001, -10, 1, 13, 99, 382295},
{4550, 0, 1, 13, 99, 382295},
{5099, -10, 1, 13, 99, 382295},
{5648, 0, 1, 13, 99, 382295},
{6298, -5, 1, 20, 99, 491521},
{7409, -5, 1, 20, 99, 491521},
{8635, -5, 1, 21, 99, 491521},
{9488, -4, 1, 21, 99, 491521},
{10494, -2, 1, 38, 99, 684489},
{12095, -2, 1, 38, 99, 684489},
{455, -10, 1, 21, 124, 518829},
{1667, -1, 1, 21, 124, 518829},
{2678, -11, 1, 12, 124, 382295},
{3389, 0, 1, 12, 124, 382295},
{4162, -11, 1, 15, 124, 418704},
{4711, 0, 1, 15, 124, 418704},
{5260, -11, 1, 15, 124, 418704},
{5809, 0, 1, 15, 124, 418704},
{6546, -5, 1, 22, 124, 527932},
{7657, -5, 1, 22, 124, 527932},
{8895, -5, 1, 23, 125, 527932},
{9748, -4, 1, 23, 125, 527932},
{10965, -2, 1, 48, 124, 837407},
{12566, -2, 1, 48, 124, 837407},
{10225, -2, 1, 29, 74, 531572},
{11826, -2, 1, 29, 74, 531572},
{781, -12, 0, 22, 75, 573442},
{1993, -1, 0, 22, 75, 573442},
{2864, -13, 0, 14, 74, 436908},
{3575, 0, 0, 14, 74, 436908},
{2994, -13, 1, 14, 74, 436908},
{3705, 0, 1, 14, 74, 436908},
{3124, -13, 0, 3, 27, 436908},
{3835, -11, 0, 3, 27, 436908},
{6887, -16, -2, 13, 37, 582544},
{7998, -7, -2, 14, 37, 582544},
{6948, -16, 0, 13, 37, 582544},
{8063, -7, 0, 14, 37, 582544},
{7009, -7, 0, 14, 77, 582544},
{8128, -16, 0, 13, 77, 582544},
{7144, -16, 0, 5, 14, 582544},
{13459, -13, 0, 2, 27, 436908},
{988, -12, 2, 22, 75, 573442},
{2200, -1, 2, 22, 75, 573442},
{1195, -12, 0, 5, 27, 573442},
{2407, -18, 0, 5, 27, 573442},
{8477, -4, 1, 17, 74, 400499},
{9330, -3, 1, 17, 74, 400499},
{15665, -2, -1, 30, 42, 546135},
{15823, -2, -1, 41, 58, 728180},
{20237, -2, -1, 23, 46, 309476},
{20370, -2, -1, 36, 92, 364090},
{16121, -2, -1, 41, 42, 728180},
{16337, -2, -1, 58, 58, 990325},
{16758, -2, -1, 41, 42, 728180},
{16974, -2, -1, 58, 58, 990325},
{17395, -2, -1, 41, 42, 728180},
{17611, -2, -1, 58, 58, 990325},
{18032, -2, -1, 39, 42, 691771},
{18636, -2, -1, 34, 42, 618953},
{19690, -2, -1, 23, 46, 309476},
{20784, -2, -1, 30, 42, 546135},
{21240, -2, -1, 30, 42, 546135},
{21696, -2, -1, 30, 42, 546135},
{22152, -2, -1, 30, 42, 546135},
{22608, -2, -1, 30, 42, 546135},
{18237, -2, -1, 55, 58, 946634},
{18815, -2, -1, 48, 58, 837407},
{19823, -2, -1, 36, 92, 364090},
{20942, -2, -1, 41, 58, 728180},
{21398, -2, -1, 41, 58, 728180},
{21854, -2, -1, 41, 58, 728180},
{22310, -2, -1, 41, 58, 728180},
{22766, -2, -1, 41, 58, 728180},
{19163, -2, -1, 34, 42, 618953},
{19342, -2, -1, 48, 58, 837407},
{23064, 0, 30, 22, 7, 364090},
{23084, 0, 31, 41, 8, 655362},
{23125, 0, 31, 59, 8, 946634},
{23184, 0, 29, 22, 5, 364090},
{23198, 0, 30, 41, 5, 655362},
{23224, 0, 30, 59, 5, 946634},
{2467, -10, 1, 8, 74, 309477},
{3178, -1, 1, 8, 74, 309477},
{3908, -10, 1, 10, 74, 345886},
{4457, -1, 1, 10, 74, 345886},
{5006, -10, 1, 10, 74, 345886},
{5555, -1, 1, 10, 74, 345886},
{6122, -4, 1, 19, 74, 436908},
{7233, -4, 1, 19, 74, 436908},
{13656, -4, 1, 39, 50, 655362},
{13900, -4, 1, 39, 75, 655362},
{14266, -4, 1, 39, 100, 655362},
{14754, -4, 1, 39, 125, 655362},
{15364, -5, 0, 25, 76, 691771},
{15602, -28, 0, 2, 27, 691771},
{15609, -28, 1, 17, 26, 691771},
{13622, -11, 0, 10, 27, 509726},
{13351, -5, -2, 18, 24, 436908},
{13405, -5, -1, 18, 24, 436908},
{8254, 1, 4, 21, 14, 294915},
{8291, 1, 4, 21, 14, 294915},
{8328, 1, 13, 21, 14, 294915},
{8365, 1, 13, 21, 14, 294915},
{13466, -3, -2, 26, 24, 509726},
{13544, -3, -1, 26, 24, 509726},
};
char cmex10_bitmap[23261] = {
0,32,8,2,0,128,48,12,1,128,96,12,3,0,96,28,
3,0,96,28,3,128,112,12,3,128,112,14,1,192,56,7,
0,224,28,3,128,112,14,1,192,56,3,0,112,14,1,192,
24,3,0,112,6,0,192,12,1,128,24,3,0,48,2,0,
32,2,0,32,0,6,0,24,0,96,1,128,7,0,12,0,
48,0,224,1,128,7,0,12,0,56,0,96,1,192,3,0,
14,0,28,0,48,0,224,1,192,3,128,6,0,28,0,56,
0,112,0,224,1,192,3,128,14,0,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,28,0,56,0,112,0,
224,1,192,3,128,3,0,7,0,14,0,28,0,24,0,56,
0,112,0,96,0,224,0,192,1,192,1,128,3,128,3,0,
7,0,6,0,6,0,14,0,12,0,12,0,12,0,12,0,
0,48,0,6,0,0,192,0,24,0,3,0,0,112,0,14,
0,0,192,0,24,0,3,128,0,48,0,7,0,0,224,0,
12,0,1,192,0,24,0,3,128,0,56,0,3,0,0,112,
0,6,0,0,224,0,14,0,0,192,0,28,0,1,192,0,
28,0,1,128,0,56,0,3,128,0,56,0,3,128,0,112,
0,7,0,0,112,0,7,0,0,112,0,7,0,0,112,0,
14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,
0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,
224,0,14,0,0,112,0,7,0,0,112,0,7,0,0,112,
0,7,0,0,112,0,3,128,0,56,0,3,128,0,56,0,
1,128,0,28,0,1,192,0,28,0,0,192,0,14,0,0,
224,0,6,0,0,112,0,3,0,0,56,0,3,128,0,24,
0,1,192,0,12,0,0,224,0,7,0,0,48,0,3,128,
0,24,0,0,192,0,14,0,0,112,0,3,0,0,24,0,
0,192,0,6,0,0,48,0,0,24,0,1,128,0,24,0,
1,192,0,12,0,0,192,0,14,0,0,224,0,6,0,0,
112,0,3,0,0,56,0,3,128,0,28,0,1,192,0,14,
0,0,224,0,7,0,0,120,0,3,128,0,28,0,1,192,
0,14,0,0,112,0,7,128,0,56,0,1,192,0,30,0,
0,224,0,7,0,0,120,0,3,192,0,30,0,0,224,0,
7,0,0,120,0,3,192,0,30,0,0,240,0,7,0,0,
120,0,3,192,0,30,0,0,240,0,7,128,0,60,0,1,
224,0,15,0,0,112,0,7,128,0,60,0,1,224,0,15,
0,0,120,0,3,192,0,30,0,0,240,0,7,128,0,60,
0,1,224,0,15,0,0,120,0,3,192,0,30,0,0,240,
0,7,128,0,60,0,1,224,0,15,0,0,120,0,3,192,
0,30,0,0,240,0,7,128,0,60,0,0,224,0,7,128,
0,60,0,1,224,0,15,0,0,120,0,3,192,0,30,0,
0,240,0,3,128,0,30,0,0,240,0,7,128,0,60,0,
0,224,0,7,0,0,60,0,1,224,0,15,0,0,56,0,
1,192,0,15,0,0,56,0,1,192,0,15,0,0,56,0,
1,192,0,14,0,0,56,0,1,192,0,15,0,0,56,0,
1,192,0,7,0,0,56,0,0,224,0,7,0,0,28,0,
0,96,0,3,128,0,12,0,0,112,0,1,192,0,6,0,
0,24,0,0,224,0,3,0,0,12,0,0,48,0,0,12,
0,0,96,0,3,128,0,12,0,0,96,0,3,128,0,12,
0,0,112,0,3,128,0,14,0,0,112,0,1,192,0,14,
0,0,56,0,1,192,0,7,0,0,56,0,0,224,0,7,
128,0,28,0,0,240,0,3,192,0,14,0,0,120,0,1,
224,0,15,0,0,60,0,0,240,0,3,192,0,30,0,0,
120,0,1,224,0,15,128,0,60,0,0,240,0,3,192,0,
31,0,0,124,0,1,224,0,7,128,0,30,0,0,248,0,
3,224,0,15,128,0,62,0,0,240,0,3,192,0,15,0,
0,124,0,1,240,0,7,192,0,31,0,0,124,0,1,240,
0,7,192,0,31,0,0,120,0,1,224,0,15,128,0,62,
0,0,248,0,3,224,0,15,128,0,62,0,0,248,0,3,
224,0,15,128,0,62,0,0,248,0,3,224,0,15,128,0,
62,0,0,248,0,3,224,0,15,128,0,0,248,0,3,224,
0,15,128,0,62,0,0,248,0,3,224,0,15,128,0,62,
0,0,248,0,3,224,0,15,128,0,62,0,0,248,0,3,
224,0,15,128,0,62,0,0,248,0,1,224,0,7,128,0,
31,0,0,124,0,1,240,0,7,192,0,31,0,0,124,0,
1,240,0,7,192,0,15,0,0,60,0,0,240,0,3,224,
0,15,128,0,62,0,0,248,0,1,224,0,7,128,0,30,
0,0,124,0,1,240,0,3,192,0,15,0,0,60,0,0,
248,0,1,224,0,7,128,0,30,0,0,60,0,0,240,0,
3,192,0,15,0,0,30,0,0,120,0,0,224,0,3,192,
0,15,0,0,28,0,0,120,0,0,224,0,3,128,0,7,
0,0,28,0,0,56,0,0,224,0,1,192,0,7,0,0,
14,0,0,56,0,0,112,0,0,192,0,3,128,0,6,0,
0,12,0,0,56,0,0,96,0,0,192,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,255,254,128,8,0,128,
8,1,128,24,3,0,48,6,0,96,12,1,192,24,3,0,
112,14,1,192,24,3,128,112,14,1,192,56,7,0,224,28,
3,128,112,14,1,192,56,6,1,192,56,7,0,192,24,7,
0,192,24,6,0,192,48,6,1,128,32,8,2,0,128,0,
192,0,192,0,192,0,192,1,192,1,128,1,128,3,128,3,
0,7,0,6,0,14,0,12,0,28,0,24,0,56,0,112,
0,96,0,224,1,192,3,128,3,0,7,0,14,0,28,0,
56,0,112,0,224,0,224,1,192,3,128,7,0,14,0,28,
0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,
56,0,112,0,224,1,192,7,0,14,0,28,0,56,0,112,
0,224,1,128,7,0,14,0,28,0,48,0,224,1,192,3,
0,14,0,24,0,112,0,192,3,128,6,0,28,0,48,0,
192,3,128,6,0,24,0,96,1,128,0,192,0,6,0,0,
48,0,1,128,0,12,0,0,224,0,7,0,0,48,0,1,
128,0,28,0,0,192,0,14,0,0,112,0,3,0,0,56,
0,1,128,0,28,0,1,192,0,12,0,0,224,0,6,0,
0,112,0,7,0,0,48,0,3,128,0,56,0,3,128,0,
24,0,1,192,0,28,0,1,192,0,28,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,0,224,0,7,0,
0,112,0,7,0,0,112,0,7,0,0,112,0,7,0,0,
112,0,7,0,0,112,0,7,0,0,112,0,7,0,0,112,
0,7,0,0,112,0,7,0,0,112,0,7,0,0,112,0,
7,0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,28,0,1,192,0,28,0,1,192,0,24,0,
3,128,0,56,0,3,128,0,48,0,7,0,0,112,0,6,
0,0,224,0,12,0,1,192,0,28,0,1,128,0,56,0,
3,0,0,112,0,14,0,0,192,0,28,0,1,128,0,48,
0,7,0,0,224,0,12,0,1,128,0,48,0,6,0,0,
192,0,0,192,0,3,0,0,12,0,0,112,0,1,128,0,
6,0,0,56,0,0,224,0,3,0,0,28,0,0,96,0,
3,128,0,14,0,0,112,0,1,192,0,14,0,0,56,0,
1,192,0,15,0,0,56,0,1,192,0,7,0,0,56,0,
1,192,0,15,0,0,56,0,1,192,0,15,0,0,56,0,
1,192,0,15,0,0,120,0,3,192,0,14,0,0,112,0,
3,192,0,30,0,0,240,0,7,128,0,28,0,0,240,0,
7,128,0,60,0,1,224,0,15,0,0,120,0,3,192,0,
30,0,0,112,0,3,192,0,30,0,0,240,0,7,128,0,
60,0,1,224,0,15,0,0,120,0,3,192,0,30,0,0,
240,0,7,128,0,60,0,1,224,0,15,0,0,120,0,3,
192,0,30,0,0,240,0,7,128,0,60,0,1,224,0,15,
0,0,120,0,3,192,0,30,0,0,224,0,15,0,0,120,
0,3,192,0,30,0,0,240,0,7,128,0,60,0,1,224,
0,14,0,0,240,0,7,128,0,60,0,1,224,0,14,0,
0,112,0,7,128,0,60,0,1,224,0,14,0,0,112,0,
7,128,0,56,0,1,192,0,30,0,0,224,0,7,0,0,
56,0,3,128,0,28,0,1,224,0,14,0,0,112,0,7,
0,0,56,0,3,128,0,28,0,1,192,0,12,0,0,224,
0,6,0,0,112,0,7,0,0,48,0,3,0,0,56,0,
1,128,0,24,0,1,128,0,0,192,0,1,128,0,7,0,
0,12,0,0,24,0,0,112,0,0,192,0,3,128,0,7,
0,0,28,0,0,56,0,0,224,0,1,192,0,7,0,0,
14,0,0,56,0,0,112,0,1,192,0,7,128,0,14,0,
0,60,0,0,240,0,1,192,0,7,128,0,30,0,0,60,
0,0,240,0,3,192,0,15,0,0,30,0,0,120,0,1,
224,0,7,192,0,15,0,0,60,0,0,240,0,3,224,0,
15,128,0,30,0,0,120,0,1,224,0,7,192,0,31,0,
0,124,0,1,240,0,3,192,0,15,0,0,60,0,0,248,
0,3,224,0,15,128,0,62,0,0,248,0,3,224,0,15,
128,0,62,0,0,120,0,1,224,0,7,192,0,31,0,0,
124,0,1,240,0,7,192,0,31,0,0,124,0,1,240,0,
7,192,0,31,0,0,124,0,1,240,0,7,192,0,31,0,
0,124,0,1,240,0,7,192,0,0,124,0,1,240,0,7,
192,0,31,0,0,124,0,1,240,0,7,192,0,31,0,0,
124,0,1,240,0,7,192,0,31,0,0,124,0,1,240,0,
7,192,0,31,0,0,124,0,1,224,0,7,128,0,62,0,
0,248,0,3,224,0,15,128,0,62,0,0,248,0,3,224,
0,15,128,0,60,0,0,240,0,3,192,0,31,0,0,124,
0,1,240,0,7,192,0,30,0,0,120,0,1,224,0,15,
128,0,62,0,0,240,0,3,192,0,15,0,0,124,0,1,
224,0,7,128,0,30,0,0,240,0,3,192,0,15,0,0,
60,0,1,224,0,7,128,0,28,0,0,240,0,3,192,0,
14,0,0,120,0,1,192,0,7,0,0,56,0,0,224,0,
7,0,0,28,0,0,224,0,3,128,0,28,0,0,112,0,
3,128,0,12,0,0,112,0,1,128,0,12,0,0,112,0,
1,128,0,12,0,0,0,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,254,255,255,6,12,24,48,96,193,
131,6,12,24,48,96,193,131,6,12,24,48,96,193,131,6,
12,24,48,96,193,131,6,12,24,48,96,193,131,6,12,24,
48,127,254,255,255,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,192,255,255,255,255,255,
255,240,14,1,192,56,7,0,224,28,3,128,112,14,1,192,
56,7,0,224,28,3,128,112,14,1,192,56,7,0,224,28,
3,128,112,14,1,192,56,7,0,224,28,3,128,112,14,1,
192,56,7,0,224,28,3,128,112,14,1,192,56,7,0,224,
28,3,128,112,14,1,192,56,7,0,224,28,3,128,112,14,
1,192,56,7,0,224,28,3,128,112,14,1,192,56,7,0,
224,28,3,128,112,14,1,192,56,7,0,224,28,3,128,112,
14,1,192,56,7,0,224,28,3,128,112,14,1,192,56,7,
0,255,255,255,255,128,255,255,255,255,254,0,224,14,0,224,
14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,
0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,
224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,
14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,
0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,
224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,
14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,
0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,
224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,
14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,
0,224,14,0,224,14,0,224,14,0,224,15,255,255,255,255,
255,255,255,255,255,248,0,224,3,128,14,0,56,0,224,3,
128,14,0,56,0,224,3,128,14,0,56,0,224,3,128,14,
0,56,0,224,3,128,14,0,56,0,224,3,128,14,0,56,
0,224,3,128,14,0,56,0,224,3,128,14,0,56,0,224,
3,128,14,0,56,0,224,3,128,14,0,56,0,224,3,128,
14,0,56,0,224,3,128,14,0,56,0,224,3,128,14,0,
56,0,224,3,128,14,0,56,0,224,3,128,14,0,56,0,
224,3,128,14,0,56,0,224,3,128,14,0,56,0,224,3,
128,0,224,3,128,14,0,56,0,224,3,128,14,0,56,0,
224,3,128,14,0,56,0,224,3,128,14,0,56,0,224,3,
128,14,0,56,0,224,3,128,14,0,56,0,224,3,128,14,
0,56,0,224,3,128,14,0,56,0,224,3,128,14,0,56,
0,224,3,128,14,0,56,0,224,3,128,14,0,56,0,224,
3,128,14,0,56,0,224,3,128,14,0,56,0,224,3,128,
14,0,56,0,224,3,128,14,0,56,0,224,3,128,14,0,
56,0,224,3,128,14,0,56,0,224,3,128,14,0,63,255,
255,255,255,240,255,255,255,255,255,255,255,255,255,255,128,255,
252,24,48,96,193,131,6,12,24,48,96,193,131,6,12,24,
48,96,193,131,6,12,24,48,96,193,131,6,12,24,48,96,
193,131,6,12,24,48,96,193,255,254,255,255,3,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,255,255,255,255,255,255,128,112,14,1,192,56,7,0,
224,28,3,128,112,14,1,192,56,7,0,224,28,3,128,112,
14,1,192,56,7,0,224,28,3,128,112,14,1,192,56,7,
0,224,28,3,128,112,14,1,192,56,7,0,224,28,3,128,
112,14,1,192,56,7,0,224,28,3,128,112,14,1,192,56,
7,0,224,28,3,128,112,14,1,192,56,7,0,224,28,3,
128,112,14,1,192,56,7,0,224,28,3,128,112,14,1,192,
56,7,0,224,28,3,128,112,14,1,192,56,7,0,224,28,
3,128,112,14,1,192,56,7,255,255,255,255,128,255,255,255,
255,240,7,0,112,7,0,112,7,0,112,7,0,112,7,0,
112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,
7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,
0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,
112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,
7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,
0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,
112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,
7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,
0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,
112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,
7,0,127,255,255,255,255,255,255,255,255,255,192,7,0,28,
0,112,1,192,7,0,28,0,112,1,192,7,0,28,0,112,
1,192,7,0,28,0,112,1,192,7,0,28,0,112,1,192,
7,0,28,0,112,1,192,7,0,28,0,112,1,192,7,0,
28,0,112,1,192,7,0,28,0,112,1,192,7,0,28,0,
112,1,192,7,0,28,0,112,1,192,7,0,28,0,112,1,
192,7,0,28,0,112,1,192,7,0,28,0,112,1,192,7,
0,28,0,112,1,192,7,0,28,0,112,1,192,7,0,28,
0,112,1,192,7,0,28,0,112,0,28,0,112,1,192,7,
0,28,0,112,1,192,7,0,28,0,112,1,192,7,0,28,
0,112,1,192,7,0,28,0,112,1,192,7,0,28,0,112,
1,192,7,0,28,0,112,1,192,7,0,28,0,112,1,192,
7,0,28,0,112,1,192,7,0,28,0,112,1,192,7,0,
28,0,112,1,192,7,0,28,0,112,1,192,7,0,28,0,
112,1,192,7,0,28,0,112,1,192,7,0,28,0,112,1,
192,7,0,28,0,112,1,192,7,0,28,0,112,1,192,7,
0,28,0,112,1,255,255,255,255,255,240,255,255,255,255,255,
255,255,255,255,255,128,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,255,255,192,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,255,255,
240,224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,
0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,
192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,
112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,128,
28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,224,
7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,
1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,
0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,
128,28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,
224,7,0,56,1,192,14,0,112,3,128,28,0,255,255,255,
255,254,224,1,192,3,128,7,0,14,0,28,0,56,0,112,
0,224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,
192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,
3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,3,
128,7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,
7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,
0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,
14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,
0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,
28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,
0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,
56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,
0,112,0,224,1,255,255,255,255,255,240,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,15,255,255,192,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,255,255,240,0,56,1,192,14,0,112,3,128,28,
0,224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,
0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,
192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,
112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,128,
28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,224,
7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,
1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,
0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,
128,28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,
224,7,255,255,255,255,254,0,14,0,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,
192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,
3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,3,
128,7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,
7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,
0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,
14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,
0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,
28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,
0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,
56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,
0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,0,
112,0,224,1,192,3,128,7,0,15,255,255,255,255,255,240,
255,255,252,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,0,255,255,
252,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,0,255,255,255,255,255,
192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,
112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,128,
28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,224,
7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,
1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,
0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,
128,28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,
224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,
56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,3,128,28,0,224,7,0,56,0,255,255,255,255,
255,255,0,14,0,28,0,56,0,112,0,224,1,192,3,128,
7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,
0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,
14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,
0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,
28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,
0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,
56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,
0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,0,
112,0,224,1,192,3,128,7,0,14,0,28,0,56,0,112,
0,224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,
192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,
3,128,7,0,0,255,255,240,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,255,255,240,12,3,0,192,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,
255,255,255,255,254,0,112,3,128,28,0,224,7,0,56,1,
192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,
112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,128,
28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,224,
7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,
1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,14,
0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,
128,28,0,224,7,0,56,1,192,14,0,112,3,128,28,0,
224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,
56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,
14,255,255,255,255,255,248,0,112,0,224,1,192,3,128,7,
0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,
14,0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,
0,28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,
28,0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,
0,56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,
56,0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,
0,112,0,224,1,192,3,128,7,0,14,0,28,0,56,0,
112,0,224,1,192,3,128,7,0,14,0,28,0,56,0,112,
0,224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,
192,3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,
3,128,7,0,14,0,28,0,56,0,112,0,224,1,192,3,
128,7,0,14,0,28,0,56,0,112,0,24,3,0,112,7,
0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,3,
128,28,0,224,7,0,56,1,192,14,0,112,7,0,112,6,
0,192,1,128,7,0,28,0,112,3,128,28,0,224,7,0,
56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,1,192,7,0,12,0,24,0,0,224,0,124,0,
62,0,15,0,3,192,0,224,0,28,0,7,0,0,224,0,
28,0,3,128,0,112,0,14,0,1,192,0,56,0,7,0,
0,224,0,28,0,3,128,0,112,0,14,0,1,192,0,56,
0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,1,
192,0,112,0,14,0,3,128,0,224,0,56,0,30,0,15,
0,1,224,0,15,0,0,112,0,7,0,0,112,0,7,0,
0,224,0,14,0,1,192,0,56,0,7,0,0,224,0,28,
0,3,128,0,112,0,14,0,1,192,0,56,0,7,0,0,
224,0,28,0,3,128,0,112,0,14,0,1,192,0,56,0,
7,0,0,224,0,28,0,3,128,0,56,0,7,0,0,120,
0,7,128,0,124,0,3,224,0,28,0,0,112,0,31,0,
3,192,0,120,0,14,0,1,192,0,56,0,7,0,0,112,
0,15,0,0,224,0,14,0,0,224,0,14,0,0,224,0,
14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,
0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,
224,0,14,0,0,224,0,14,0,0,224,0,14,0,1,192,
0,28,0,1,192,0,56,0,7,0,0,96,0,14,0,3,
128,0,112,0,12,0,0,112,0,3,128,0,14,0,0,96,
0,7,0,0,56,0,1,192,0,28,0,1,192,0,14,0,
0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,
224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,
0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,
14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,15,0,0,112,0,7,0,
0,56,0,1,192,0,14,0,0,120,0,3,192,0,31,0,
0,112,0,0,28,0,0,240,0,15,128,0,120,0,3,192,
0,30,0,0,240,0,3,128,0,30,0,0,240,0,3,192,
0,15,0,0,120,0,1,224,0,7,128,0,30,0,0,120,
0,1,224,0,7,128,0,30,0,0,120,0,1,224,0,7,
128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,0,
120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,0,
7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,
0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,
0,7,128,0,30,0,0,120,0,1,224,0,15,0,0,60,
0,0,240,0,7,128,0,30,0,0,240,0,3,128,0,28,
0,0,224,0,7,0,0,120,0,3,128,0,14,0,0,30,
0,0,28,0,0,56,0,0,112,0,0,224,0,3,192,0,
7,128,0,30,0,0,60,0,0,240,0,3,192,0,7,128,
0,30,0,0,120,0,1,224,0,7,128,0,30,0,0,120,
0,1,224,0,7,128,0,30,0,0,120,0,1,224,0,7,
128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,0,
120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,0,
7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,
0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,
0,7,128,0,30,0,0,60,0,0,240,0,3,192,0,7,
128,0,14,0,0,60,0,0,120,0,0,240,0,1,224,0,
3,224,0,3,192,0,7,0,24,3,192,60,3,192,60,3,
192,60,3,224,30,1,240,15,0,248,7,192,60,3,224,31,
0,248,7,192,62,1,240,15,128,124,3,224,31,0,248,7,
192,62,1,240,15,128,124,3,224,31,0,248,7,192,62,1,
240,15,128,0,248,7,192,62,1,240,15,128,124,3,224,31,
0,248,7,192,62,1,240,15,128,124,3,224,31,0,248,7,
192,62,1,240,15,128,124,3,224,15,0,124,3,224,15,0,
124,1,224,15,128,60,0,240,3,192,15,0,60,0,240,1,
128,0,124,1,240,7,192,31,0,124,1,240,7,192,31,0,
124,1,240,7,192,31,0,124,1,240,7,192,31,0,124,1,
240,7,192,31,0,124,1,240,7,192,31,0,248,3,224,15,
128,60,1,240,7,128,62,0,240,7,128,28,0,224,7,0,
56,3,192,12,0,60,0,56,0,112,0,224,1,192,7,128,
15,0,62,0,120,1,240,3,192,15,128,62,0,248,1,240,
7,192,31,0,124,1,240,7,192,31,0,124,1,240,7,192,
31,0,124,1,240,7,192,31,0,124,1,240,7,192,31,0,
124,1,240,7,192,31,0,124,255,255,255,255,255,255,255,255,
252,192,1,128,7,0,28,0,112,3,128,28,0,224,7,0,
56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,1,192,7,0,12,0,24,3,0,112,7,0,112,
3,128,28,0,224,7,0,56,1,192,14,0,112,3,128,28,
0,224,7,0,56,1,192,14,0,112,7,0,112,6,0,192,
0,224,0,31,0,0,248,0,7,128,0,120,0,3,128,0,
112,0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,
1,192,0,56,0,7,0,0,224,0,28,0,3,128,0,112,
0,14,0,1,192,0,56,0,7,0,0,224,0,28,0,3,
128,0,112,0,14,0,1,192,0,28,0,3,128,0,56,0,
3,128,0,56,0,3,192,0,30,0,3,192,1,224,0,112,
0,28,0,7,0,1,192,0,56,0,14,0,1,192,0,56,
0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,1,
192,0,56,0,7,0,0,224,0,28,0,3,128,0,112,0,
14,0,1,192,0,56,0,7,0,0,224,0,28,0,3,128,
0,224,0,28,0,15,0,3,192,1,240,0,248,0,28,0,
0,224,0,15,0,0,124,0,1,224,0,15,0,0,112,0,
3,128,0,28,0,1,192,0,14,0,0,224,0,14,0,0,
224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,
0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,
14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,
0,224,0,14,0,0,112,0,7,0,0,56,0,3,128,0,
28,0,0,224,0,7,0,0,60,0,0,240,0,7,0,0,
224,0,60,0,7,0,0,224,0,28,0,3,128,0,56,0,
7,0,0,112,0,14,0,0,224,0,14,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,
0,224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,
224,0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,
0,14,0,0,224,0,14,0,0,224,0,14,0,0,224,0,
14,0,1,192,0,28,0,3,128,0,112,0,15,0,1,224,
0,124,0,15,0,0,224,0,0,224,0,3,192,0,7,192,
0,7,128,0,15,0,0,30,0,0,60,0,0,112,0,1,
224,0,3,192,0,15,0,0,60,0,0,120,0,1,224,0,
7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,
0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,
0,7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,
0,0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,
224,0,7,128,0,30,0,0,120,0,1,224,0,7,128,0,
30,0,0,120,0,1,224,0,7,128,0,30,0,0,120,0,
1,224,0,3,192,0,15,0,0,60,0,0,120,0,1,224,
0,3,192,0,7,0,0,14,0,0,28,0,0,56,0,0,
120,0,0,112,0,1,192,0,30,0,0,224,0,7,0,0,
56,0,1,192,0,15,0,0,120,0,1,224,0,15,0,0,
60,0,0,240,0,7,128,0,30,0,0,120,0,1,224,0,
7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,0,
0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,224,
0,7,128,0,30,0,0,120,0,1,224,0,7,128,0,30,
0,0,120,0,1,224,0,7,128,0,30,0,0,120,0,1,
224,0,7,128,0,30,0,0,120,0,1,224,0,7,128,0,
30,0,0,120,0,1,224,0,7,128,0,30,0,0,240,0,
3,192,0,15,0,0,120,0,1,192,0,15,0,0,120,0,
3,192,0,30,0,1,240,0,15,0,0,56,0,0,192,3,
192,7,192,7,128,15,0,30,0,124,0,240,1,224,7,192,
15,0,62,0,248,3,224,7,192,31,0,124,1,240,7,192,
31,0,124,1,240,7,192,31,0,124,1,240,7,192,31,0,
124,1,240,7,192,31,0,124,1,240,7,192,31,0,124,0,
124,1,240,7,192,31,0,124,1,240,7,192,31,0,124,1,
240,7,192,31,0,124,1,240,7,192,31,0,124,1,240,7,
192,31,0,124,1,240,7,192,62,0,248,3,224,15,0,124,
1,224,15,0,124,1,224,15,0,120,7,192,60,0,192,0,
248,7,192,62,1,240,15,128,124,3,224,31,0,248,7,192,
62,1,240,15,128,124,3,224,31,0,248,7,192,62,1,240,
15,128,124,3,224,31,0,120,3,224,31,0,120,3,192,31,
0,120,1,224,15,0,60,0,240,3,128,14,0,60,0,96,
15,0,224,14,0,240,15,0,240,7,128,120,7,192,60,1,
224,31,0,248,7,128,124,3,224,31,0,248,7,192,62,1,
240,15,128,124,3,224,31,0,248,7,192,62,1,240,15,128,
124,3,224,31,0,248,7,192,62,1,240,15,128,0,0,1,
248,0,127,192,15,254,1,255,240,31,255,129,255,0,31,128,
1,240,0,30,0,1,192,0,28,0,0,192,0,14,0,0,
96,0,0,252,0,7,252,0,63,248,1,255,240,15,255,192,
1,255,0,0,252,0,1,240,0,3,192,0,7,0,0,28,
0,0,96,0,3,128,0,12,192,0,7,0,0,24,0,0,
224,0,3,128,0,15,0,0,62,0,0,252,0,3,254,0,
15,255,192,63,254,0,127,240,0,255,128,0,252,0,0,24,
0,1,192,0,12,0,0,224,0,14,0,1,224,0,62,0,
7,224,3,254,7,255,224,63,254,1,255,192,15,248,0,126,
0,0,0,16,3,0,96,6,0,96,12,0,192,24,1,128,
48,3,0,48,6,0,96,12,0,192,12,1,128,24,3,0,
48,6,0,96,6,0,192,12,0,96,6,0,96,3,0,48,
1,128,24,0,192,12,0,192,6,0,96,3,0,48,3,0,
24,1,128,12,0,192,6,0,96,6,0,48,1,0,1,128,
0,192,0,224,0,96,0,112,0,48,0,24,0,28,0,12,
0,14,0,6,0,7,0,3,0,1,128,1,192,0,192,0,
224,0,96,0,112,0,48,0,24,0,28,0,12,0,14,0,
6,0,3,0,3,128,1,128,1,192,0,192,0,224,0,96,
0,48,0,56,0,24,0,28,0,12,0,6,0,3,128,0,
192,0,112,0,24,0,12,0,7,0,1,128,0,224,0,48,
0,28,0,6,0,3,0,1,192,0,96,0,56,0,12,0,
6,0,3,128,0,192,0,112,0,24,0,14,0,3,0,1,
128,0,224,0,48,0,28,0,6,0,3,128,0,192,0,96,
0,56,0,12,0,7,0,1,128,0,192,0,0,16,0,1,
192,0,14,0,0,224,0,7,0,0,56,0,3,128,0,28,
0,1,192,0,14,0,0,112,0,7,0,0,56,0,1,192,
0,28,0,0,224,0,14,0,0,112,0,3,128,0,56,0,
1,192,0,28,0,0,224,0,7,0,0,112,0,3,128,0,
28,0,1,192,0,14,0,0,224,0,7,0,0,56,0,3,
128,0,28,0,0,224,0,14,0,0,112,0,7,0,0,56,
0,1,192,0,28,0,0,224,0,7,0,0,112,0,3,128,
0,56,0,1,192,0,14,0,0,224,0,7,0,0,56,0,
0,224,0,7,0,0,56,0,0,224,0,7,0,0,28,0,
0,224,0,7,0,0,28,0,0,224,0,7,0,0,28,0,
0,224,0,3,128,0,28,0,0,224,0,3,128,0,28,0,
0,224,0,3,128,0,28,0,0,112,0,3,128,0,28,0,
0,112,0,3,128,0,28,0,0,112,0,3,128,0,14,0,
0,112,0,3,128,0,14,0,0,112,0,1,192,0,14,0,
0,112,0,1,192,0,14,0,0,112,0,1,192,0,14,0,
0,56,0,1,192,0,14,0,0,56,0,1,192,0,4,0,
0,4,0,0,28,0,0,56,0,0,224,0,1,192,0,3,
128,0,14,0,0,28,0,0,56,0,0,224,0,1,192,0,
3,128,0,14,0,0,28,0,0,56,0,0,224,0,1,192,
0,3,128,0,14,0,0,28,0,0,56,0,0,224,0,1,
192,0,3,128,0,14,0,0,28,0,0,56,0,0,224,0,
1,192,0,3,128,0,14,0,0,28,0,0,56,0,0,112,
0,1,192,0,3,128,0,7,0,0,28,0,0,56,0,0,
112,0,1,192,0,3,128,0,7,0,0,28,0,0,56,0,
0,112,0,1,192,0,3,128,0,7,0,0,28,0,0,56,
0,0,112,0,1,192,0,3,128,0,7,0,0,28,0,0,
56,0,0,112,0,1,192,0,3,128,0,7,0,0,28,0,
0,56,0,0,112,0,0,112,0,0,224,0,1,192,0,1,
192,0,3,128,0,7,0,0,7,0,0,14,0,0,28,0,
0,28,0,0,56,0,0,112,0,0,112,0,0,224,0,1,
192,0,1,192,0,3,128,0,7,0,0,7,0,0,14,0,
0,28,0,0,28,0,0,56,0,0,112,0,0,112,0,0,
224,0,1,192,0,1,192,0,3,128,0,7,0,0,14,0,
0,14,0,0,28,0,0,56,0,0,56,0,0,112,0,0,
224,0,0,224,0,1,192,0,3,128,0,3,128,0,7,0,
0,14,0,0,14,0,0,28,0,0,56,0,0,56,0,0,
112,0,0,224,0,0,224,0,1,192,0,3,128,0,3,128,
0,7,0,0,14,0,0,14,0,0,28,0,0,56,0,0,
56,0,0,112,0,0,64,192,12,0,96,6,0,96,3,0,
48,1,128,24,0,192,12,0,192,6,0,96,3,0,48,3,
0,24,1,128,12,0,192,6,0,96,6,0,48,3,0,96,
6,0,96,12,0,192,24,1,128,48,3,0,48,6,0,96,
12,0,192,12,1,128,24,3,0,48,6,0,96,6,0,192,
12,0,192,0,96,0,56,0,12,0,7,0,1,128,0,192,
0,112,0,24,0,14,0,3,0,1,192,0,96,0,48,0,
28,0,6,0,3,128,0,192,0,112,0,24,0,12,0,7,
0,1,128,0,224,0,48,0,24,0,14,0,3,0,1,192,
0,96,0,56,0,12,0,6,0,3,128,0,192,0,112,0,
24,0,12,0,14,0,6,0,7,0,3,0,1,128,1,192,
0,192,0,224,0,96,0,112,0,48,0,24,0,28,0,12,
0,14,0,6,0,3,0,3,128,1,128,1,192,0,192,0,
224,0,96,0,48,0,56,0,24,0,28,0,12,0,14,0,
6,0,3,0,3,128,1,128,1,192,0,192,0,96,0,0,
64,0,7,0,0,56,0,0,224,0,7,0,0,56,0,0,
224,0,7,0,0,28,0,0,224,0,7,0,0,28,0,0,
224,0,7,0,0,28,0,0,224,0,3,128,0,28,0,0,
224,0,3,128,0,28,0,0,112,0,3,128,0,28,0,0,
112,0,3,128,0,28,0,0,112,0,3,128,0,14,0,0,
112,0,3,128,0,14,0,0,112,0,3,128,0,14,0,0,
112,0,1,192,0,14,0,0,112,0,1,192,0,14,0,0,
112,0,1,192,0,14,0,0,56,0,1,192,0,14,0,0,
56,0,1,192,0,14,0,0,224,0,7,0,0,56,0,3,
128,0,28,0,1,192,0,14,0,0,112,0,7,0,0,56,
0,1,192,0,28,0,0,224,0,14,0,0,112,0,3,128,
0,56,0,1,192,0,14,0,0,224,0,7,0,0,112,0,
3,128,0,28,0,1,192,0,14,0,0,112,0,7,0,0,
56,0,3,128,0,28,0,0,224,0,14,0,0,112,0,7,
0,0,56,0,1,192,0,28,0,0,224,0,7,0,0,112,
0,3,128,0,56,0,1,192,0,14,0,0,224,0,7,0,
0,16,0,0,64,0,1,192,0,3,128,0,3,128,0,7,
0,0,14,0,0,14,0,0,28,0,0,56,0,0,56,0,
0,112,0,0,224,0,0,224,0,1,192,0,3,128,0,3,
128,0,7,0,0,14,0,0,14,0,0,28,0,0,56,0,
0,56,0,0,112,0,0,224,0,0,224,0,1,192,0,3,
128,0,3,128,0,7,0,0,14,0,0,14,0,0,28,0,
0,56,0,0,112,0,0,112,0,0,224,0,1,192,0,1,
192,0,3,128,0,7,0,0,7,0,0,14,0,0,28,0,
0,28,0,0,56,0,0,112,0,0,112,0,0,224,0,1,
192,0,1,192,0,3,128,0,7,0,0,7,0,0,14,0,
0,28,0,0,28,0,0,56,0,0,112,0,0,112,0,0,
224,0,1,192,0,1,192,0,3,128,0,7,0,0,28,0,
0,56,0,0,112,0,1,192,0,3,128,0,7,0,0,28,
0,0,56,0,0,112,0,1,192,0,3,128,0,7,0,0,
28,0,0,56,0,0,112,0,1,192,0,3,128,0,7,0,
0,28,0,0,56,0,0,112,0,1,192,0,3,128,0,7,
0,0,28,0,0,56,0,0,112,0,1,192,0,3,128,0,
7,0,0,14,0,0,56,0,0,112,0,0,224,0,3,128,
0,7,0,0,14,0,0,56,0,0,112,0,0,224,0,3,
128,0,7,0,0,14,0,0,56,0,0,112,0,0,224,0,
3,128,0,7,0,0,14,0,0,56,0,0,112,0,0,224,
0,3,128,0,7,0,0,14,0,0,56,0,0,112,0,0,
224,0,3,128,0,7,0,0,4,0,0,0,0,0,96,0,
12,0,3,128,0,96,0,12,0,3,128,0,96,0,28,0,
3,0,0,96,0,28,0,3,0,0,96,0,28,0,3,0,
0,96,0,28,0,3,0,0,224,0,24,0,3,0,0,224,
0,24,0,3,0,0,224,0,24,0,3,0,0,224,0,24,
0,3,0,0,224,0,24,0,7,0,0,192,0,24,0,7,
0,0,192,0,24,0,7,0,0,192,0,24,0,7,0,0,
192,0,56,0,6,0,0,192,0,56,0,6,0,0,192,0,
0,0,0,0,24,0,0,0,192,0,0,14,0,0,0,96,
0,0,7,0,0,0,48,0,0,1,128,0,0,28,0,0,
0,192,0,0,6,0,0,0,112,0,0,3,0,0,0,56,
0,0,1,128,0,0,12,0,0,0,224,0,0,6,0,0,
0,48,0,0,3,128,0,0,24,0,0,1,192,0,0,12,
0,0,0,96,0,0,7,0,0,0,48,0,0,1,128,0,
0,28,0,0,0,192,0,0,14,0,0,0,96,0,0,3,
0,0,0,56,0,0,1,128,0,0,12,0,0,0,224,0,
0,6,0,0,0,112,0,0,3,0,0,0,24,0,0,1,
192,0,0,12,0,0,0,96,0,0,7,0,0,0,48,0,
0,1,128,0,0,28,0,0,0,192,0,0,14,0,0,0,
96,0,0,3,0,0,0,56,0,0,1,128,0,0,12,0,
0,0,224,0,0,6,0,0,0,112,0,0,3,0,0,0,
24,0,0,1,192,0,0,12,0,0,0,96,0,0,7,0,
0,0,48,0,0,3,128,0,0,24,0,0,0,192,0,0,
14,0,0,0,96,0,0,3,0,0,0,56,0,0,1,128,
0,0,28,0,0,0,192,0,0,6,0,0,0,0,0,0,
0,0,8,0,0,0,0,112,0,0,0,1,192,0,0,0,
14,0,0,0,0,56,0,0,0,0,224,0,0,0,7,0,
0,0,0,28,0,0,0,0,224,0,0,0,3,128,0,0,
0,14,0,0,0,0,112,0,0,0,1,192,0,0,0,7,
0,0,0,0,56,0,0,0,0,224,0,0,0,3,128,0,
0,0,28,0,0,0,0,112,0,0,0,3,128,0,0,0,
14,0,0,0,0,56,0,0,0,1,192,0,0,0,7,0,
0,0,0,28,0,0,0,0,224,0,0,0,3,128,0,0,
0,14,0,0,0,0,112,0,0,0,1,192,0,0,0,14,
0,0,0,0,56,0,0,0,0,224,0,0,0,7,0,0,
0,0,28,0,0,0,0,112,0,0,0,3,128,0,0,0,
14,0,0,0,0,56,0,0,0,1,192,0,0,0,7,0,
0,0,0,56,0,0,0,0,224,0,0,0,3,128,0,0,
0,28,0,0,0,0,112,0,0,0,1,192,0,0,0,14,
0,0,0,0,56,0,0,0,1,192,0,0,0,7,0,0,
0,0,28,0,0,0,0,224,0,0,0,3,128,0,0,0,
14,0,0,0,0,112,0,0,0,1,192,0,0,0,7,0,
0,0,0,56,0,0,0,0,224,0,0,0,7,0,0,0,
0,28,0,0,0,0,112,0,0,0,3,128,0,0,0,14,
0,0,0,0,56,0,0,0,1,192,0,0,0,7,0,0,
0,0,28,0,0,0,0,224,0,0,0,3,128,0,0,0,
28,0,0,0,0,112,0,0,0,1,192,0,0,0,14,0,
0,0,0,56,0,0,0,0,224,0,0,0,7,0,0,0,
0,28,0,0,0,0,112,0,0,0,3,128,0,0,0,14,
0,0,0,0,112,0,0,0,1,192,0,0,0,7,0,0,
0,0,56,0,0,0,0,224,0,0,0,3,128,0,0,0,
28,0,0,0,0,112,0,0,0,1,192,0,0,0,14,0,
0,0,0,56,0,0,0,1,192,0,0,0,7,0,0,0,
0,28,0,0,0,0,224,0,0,0,3,128,0,0,0,4,
0,0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,
7,0,0,0,0,0,7,0,0,0,0,0,14,0,0,0,
0,0,14,0,0,0,0,0,14,0,0,0,0,0,28,0,
0,0,0,0,28,0,0,0,0,0,56,0,0,0,0,0,
56,0,0,0,0,0,56,0,0,0,0,0,112,0,0,0,
0,0,112,0,0,0,0,0,112,0,0,0,0,0,224,0,
0,0,0,0,224,0,0,0,0,1,192,0,0,0,0,1,
192,0,0,0,0,1,192,0,0,0,0,3,128,0,0,0,
0,3,128,0,0,0,0,3,128,0,0,0,0,7,0,0,
0,0,0,7,0,0,0,0,0,14,0,0,0,0,0,14,
0,0,0,0,0,14,0,0,0,0,0,28,0,0,0,0,
0,28,0,0,0,0,0,28,0,0,0,0,0,56,0,0,
0,0,0,56,0,0,0,0,0,112,0,0,0,0,0,112,
0,0,0,0,0,112,0,0,0,0,0,224,0,0,0,0,
0,224,0,0,0,0,0,224,0,0,0,0,1,192,0,0,
0,0,1,192,0,0,0,0,3,128,0,0,0,0,3,128,
0,0,0,0,3,128,0,0,0,0,7,0,0,0,0,0,
7,0,0,0,0,0,7,0,0,0,0,0,14,0,0,0,
0,0,14,0,0,0,0,0,14,0,0,0,0,0,28,0,
0,0,0,0,28,0,0,0,0,0,56,0,0,0,0,0,
56,0,0,0,0,0,56,0,0,0,0,0,112,0,0,0,
0,0,112,0,0,0,0,0,112,0,0,0,0,0,224,0,
0,0,0,0,224,0,0,0,0,1,192,0,0,0,0,1,
192,0,0,0,0,1,192,0,0,0,0,3,128,0,0,0,
0,3,128,0,0,0,0,3,128,0,0,0,0,7,0,0,
0,0,0,7,0,0,0,0,0,14,0,0,0,0,0,14,
0,0,0,0,0,14,0,0,0,0,0,28,0,0,0,0,
0,28,0,0,0,0,0,28,0,0,0,0,0,56,0,0,
0,0,0,56,0,0,0,0,0,112,0,0,0,0,0,112,
0,0,0,0,0,112,0,0,0,0,0,224,0,0,0,0,
0,224,0,0,0,0,0,224,0,0,0,0,1,192,0,0,
0,0,1,192,0,0,0,0,1,192,0,0,0,0,3,128,
0,0,0,0,3,128,0,0,0,0,7,0,0,0,0,0,
7,0,0,0,0,0,7,0,0,0,0,0,14,0,0,0,
0,0,14,0,0,0,0,0,14,0,0,0,0,0,28,0,
0,0,0,0,28,0,0,0,0,0,56,0,0,0,0,0,
56,0,0,0,0,0,56,0,0,0,0,0,112,0,0,0,
0,0,112,0,0,0,0,0,112,0,0,0,0,0,224,0,
0,0,0,0,224,0,0,0,0,1,192,0,0,0,0,1,
192,0,0,0,0,1,192,0,0,0,0,3,128,0,0,0,
0,3,128,0,0,0,0,3,128,0,0,0,0,7,0,0,
0,0,0,7,0,0,0,0,0,14,0,0,0,0,0,14,
0,0,0,0,0,14,0,0,0,0,0,28,0,0,0,0,
0,28,0,0,0,0,0,28,0,0,0,0,0,56,0,0,
0,0,0,56,0,0,0,0,0,112,0,0,0,0,0,112,
0,0,0,0,0,112,0,0,0,0,0,224,0,0,0,0,
0,224,0,0,0,0,0,224,0,0,0,0,0,192,0,24,
0,3,128,0,48,0,6,0,0,224,0,12,0,1,192,0,
24,0,3,0,0,112,0,6,0,0,192,0,28,0,1,128,
0,48,0,7,0,0,96,0,14,0,0,192,0,24,0,3,
128,0,48,0,6,0,0,224,0,12,0,1,128,0,56,0,
3,0,0,96,0,14,0,0,192,0,28,0,1,128,0,48,
0,7,0,0,96,0,12,0,1,192,0,24,0,3,0,0,
112,0,6,0,0,224,0,12,0,1,128,0,56,0,3,0,
0,96,192,0,0,6,0,0,0,56,0,0,0,192,0,0,
7,0,0,0,24,0,0,0,192,0,0,7,0,0,0,24,
0,0,0,192,0,0,7,0,0,0,24,0,0,0,224,0,
0,3,0,0,0,24,0,0,0,224,0,0,3,0,0,0,
24,0,0,0,224,0,0,3,0,0,0,28,0,0,0,96,
0,0,3,0,0,0,28,0,0,0,96,0,0,3,0,0,
0,28,0,0,0,96,0,0,3,128,0,0,12,0,0,0,
96,0,0,3,128,0,0,12,0,0,0,96,0,0,3,128,
0,0,12,0,0,0,96,0,0,3,128,0,0,12,0,0,
0,112,0,0,1,128,0,0,12,0,0,0,112,0,0,1,
128,0,0,12,0,0,0,112,0,0,1,128,0,0,14,0,
0,0,48,0,0,1,128,0,0,14,0,0,0,48,0,0,
1,128,0,0,14,0,0,0,48,0,0,1,192,0,0,6,
0,0,0,48,0,0,1,192,0,0,6,0,0,0,48,0,
0,1,192,0,0,6,0,0,0,56,0,0,0,192,0,0,
6,0,0,0,56,0,0,0,192,0,0,6,0,0,0,56,
0,0,0,192,0,0,7,0,0,0,24,0,0,0,192,64,
0,0,0,3,128,0,0,0,14,0,0,0,0,28,0,0,
0,0,112,0,0,0,1,192,0,0,0,3,128,0,0,0,
14,0,0,0,0,28,0,0,0,0,112,0,0,0,1,192,
0,0,0,3,128,0,0,0,14,0,0,0,0,56,0,0,
0,0,112,0,0,0,1,192,0,0,0,7,0,0,0,0,
14,0,0,0,0,56,0,0,0,0,112,0,0,0,1,192,
0,0,0,7,0,0,0,0,14,0,0,0,0,56,0,0,
0,0,224,0,0,0,1,192,0,0,0,7,0,0,0,0,
28,0,0,0,0,56,0,0,0,0,224,0,0,0,1,192,
0,0,0,7,0,0,0,0,28,0,0,0,0,56,0,0,
0,0,224,0,0,0,3,128,0,0,0,7,0,0,0,0,
28,0,0,0,0,112,0,0,0,0,224,0,0,0,3,128,
0,0,0,7,0,0,0,0,28,0,0,0,0,112,0,0,
0,0,224,0,0,0,3,128,0,0,0,14,0,0,0,0,
28,0,0,0,0,112,0,0,0,1,192,0,0,0,3,128,
0,0,0,14,0,0,0,0,28,0,0,0,0,112,0,0,
0,1,192,0,0,0,3,128,0,0,0,14,0,0,0,0,
56,0,0,0,0,112,0,0,0,1,192,0,0,0,3,128,
0,0,0,14,0,0,0,0,56,0,0,0,0,112,0,0,
0,1,192,0,0,0,7,0,0,0,0,14,0,0,0,0,
56,0,0,0,0,224,0,0,0,1,192,0,0,0,7,0,
0,0,0,14,0,0,0,0,56,0,0,0,0,224,0,0,
0,1,192,0,0,0,7,0,0,0,0,28,0,0,0,0,
56,0,0,0,0,224,0,0,0,3,128,0,0,0,7,0,
0,0,0,28,0,0,0,0,56,0,0,0,0,224,0,0,
0,3,128,0,0,0,7,0,0,0,0,28,0,0,0,0,
112,0,0,0,0,224,0,0,0,3,128,0,0,0,14,0,
0,0,0,28,0,0,0,0,112,0,0,0,0,224,0,0,
0,3,128,0,0,0,14,0,0,0,0,28,0,0,0,0,
112,0,0,0,0,128,224,0,0,0,0,0,224,0,0,0,
0,0,224,0,0,0,0,0,112,0,0,0,0,0,112,0,
0,0,0,0,112,0,0,0,0,0,56,0,0,0,0,0,
56,0,0,0,0,0,28,0,0,0,0,0,28,0,0,0,
0,0,28,0,0,0,0,0,14,0,0,0,0,0,14,0,
0,0,0,0,14,0,0,0,0,0,7,0,0,0,0,0,
7,0,0,0,0,0,3,128,0,0,0,0,3,128,0,0,
0,0,3,128,0,0,0,0,1,192,0,0,0,0,1,192,
0,0,0,0,1,192,0,0,0,0,0,224,0,0,0,0,
0,224,0,0,0,0,0,112,0,0,0,0,0,112,0,0,
0,0,0,112,0,0,0,0,0,56,0,0,0,0,0,56,
0,0,0,0,0,56,0,0,0,0,0,28,0,0,0,0,
0,28,0,0,0,0,0,14,0,0,0,0,0,14,0,0,
0,0,0,14,0,0,0,0,0,7,0,0,0,0,0,7,
0,0,0,0,0,7,0,0,0,0,0,3,128,0,0,0,
0,3,128,0,0,0,0,1,192,0,0,0,0,1,192,0,
0,0,0,1,192,0,0,0,0,0,224,0,0,0,0,0,
224,0,0,0,0,0,224,0,0,0,0,0,112,0,0,0,
0,0,112,0,0,0,0,0,112,0,0,0,0,0,56,0,
0,0,0,0,56,0,0,0,0,0,28,0,0,0,0,0,
28,0,0,0,0,0,28,0,0,0,0,0,14,0,0,0,
0,0,14,0,0,0,0,0,14,0,0,0,0,0,7,0,
0,0,0,0,7,0,0,0,0,0,3,128,0,0,0,0,
3,128,0,0,0,0,3,128,0,0,0,0,1,192,0,0,
0,0,1,192,0,0,0,0,1,192,0,0,0,0,0,224,
0,0,0,0,0,224,0,0,0,0,0,112,0,0,0,0,
0,112,0,0,0,0,0,112,0,0,0,0,0,56,0,0,
0,0,0,56,0,0,0,0,0,56,0,0,0,0,0,28,
0,0,0,0,0,28,0,0,0,0,0,14,0,0,0,0,
0,14,0,0,0,0,0,14,0,0,0,0,0,7,0,0,
0,0,0,7,0,0,0,0,0,7,0,0,0,0,0,3,
128,0,0,0,0,3,128,0,0,0,0,3,128,0,0,0,
0,1,192,0,0,0,0,1,192,0,0,0,0,0,224,0,
0,0,0,0,224,0,0,0,0,0,224,0,0,0,0,0,
112,0,0,0,0,0,112,0,0,0,0,0,112,0,0,0,
0,0,56,0,0,0,0,0,56,0,0,0,0,0,28,0,
0,0,0,0,28,0,0,0,0,0,28,0,0,0,0,0,
14,0,0,0,0,0,14,0,0,0,0,0,14,0,0,0,
0,0,7,0,0,0,0,0,7,0,0,0,0,0,3,128,
0,0,0,0,3,128,0,0,0,0,3,128,0,0,0,0,
1,192,0,0,0,0,1,192,0,0,0,0,1,192,0,0,
0,0,0,224,0,0,0,0,0,224,0,0,0,0,0,112,
0,0,0,0,0,112,0,0,0,0,0,112,0,0,0,0,
0,56,0,0,0,0,0,56,0,0,0,0,0,56,0,0,
0,0,0,28,0,0,0,0,0,28,0,0,0,0,0,14,
0,0,0,0,0,14,0,0,0,0,0,14,0,0,0,0,
0,7,0,0,0,0,0,7,0,0,0,0,0,7,255,255,
255,255,255,255,252,192,240,60,15,3,192,240,60,15,3,192,
240,60,15,3,192,240,60,15,3,192,240,60,15,3,192,240,
60,15,3,192,240,60,12,0,192,0,120,0,63,0,31,224,
14,220,14,49,207,12,63,3,3,0,192,0,48,0,12,0,
3,0,0,192,0,48,0,12,0,3,0,0,192,0,48,0,
12,0,3,0,0,192,0,48,0,12,0,3,0,0,192,0,
48,0,12,0,3,0,0,192,0,48,0,12,0,3,0,0,
192,0,48,0,12,0,3,0,0,192,0,48,0,12,0,3,
0,192,192,252,48,243,140,112,59,112,7,248,0,252,0,30,
0,3,0,255,255,255,255,255,255,252,0,12,0,0,3,0,
0,1,224,0,0,252,0,0,51,0,0,24,96,0,12,12,
0,7,3,128,3,192,240,3,176,55,1,204,12,225,195,3,
14,224,192,193,224,48,48,16,12,12,0,3,3,0,0,192,
192,0,48,48,0,12,12,0,3,3,0,0,192,192,0,48,
48,0,12,12,0,3,3,0,0,192,192,0,48,48,0,12,
12,0,3,3,0,0,192,192,0,48,48,0,12,12,0,3,
3,0,0,192,192,0,48,48,8,12,12,7,131,3,7,112,
192,195,135,48,51,128,236,13,192,15,3,192,1,192,224,0,
48,48,0,6,24,0,0,204,0,0,63,0,0,7,128,0,
0,192,0,0,48,0,192,240,60,15,3,192,240,60,15,3,
192,240,60,15,3,192,240,60,15,3,192,240,60,15,3,192,
240,60,15,3,192,240,60,12,0,0,0,0,2,0,0,0,
0,12,0,0,0,0,48,0,0,0,0,96,0,0,0,1,
128,0,0,0,3,0,0,0,0,12,0,0,0,0,24,0,
0,0,0,96,0,0,0,0,192,0,0,0,3,0,0,0,
0,6,0,0,0,0,24,0,0,0,0,48,0,0,0,0,
192,0,0,0,1,128,0,0,0,6,0,0,0,0,12,0,
0,0,0,48,0,0,0,0,96,0,0,0,1,128,0,0,
0,3,0,0,0,0,12,0,0,0,0,24,0,0,0,0,
96,0,16,0,0,192,0,112,0,3,0,1,224,0,6,0,
12,224,0,24,0,1,192,0,48,0,3,128,0,192,0,3,
128,1,128,0,7,0,6,0,0,7,0,12,0,0,14,0,
48,0,0,14,0,96,0,0,28,1,128,0,0,60,3,0,
0,0,56,12,0,0,0,112,24,0,0,0,112,96,0,0,
0,224,192,0,0,0,227,0,0,0,1,198,0,0,0,1,
216,0,0,0,3,176,0,0,0,7,192,0,0,0,7,128,
0,0,0,14,0,0,0,0,12,0,0,0,0,0,0,0,
2,0,0,0,0,12,0,0,0,0,24,0,0,0,0,96,
0,0,0,0,192,0,0,0,1,128,0,0,0,6,0,0,
0,0,12,0,0,0,0,24,0,0,0,0,96,0,0,0,
0,192,0,0,0,1,128,0,0,0,6,0,0,0,0,12,
0,0,0,0,24,0,0,0,0,96,0,0,0,0,192,0,
0,0,1,128,0,0,0,6,0,0,0,0,12,0,0,0,
0,24,0,0,0,0,96,0,0,0,0,192,0,0,0,1,
128,0,0,0,6,0,0,0,0,12,0,0,0,0,24,0,
0,0,0,96,0,0,0,0,192,0,0,0,1,128,0,0,
0,6,0,0,0,0,12,0,0,0,0,24,0,0,0,0,
96,0,0,0,0,192,0,0,0,1,128,0,0,0,6,0,
0,0,0,12,0,2,0,0,24,0,14,0,0,96,0,60,
0,0,192,0,248,0,1,128,3,112,0,6,0,4,112,0,
12,0,0,224,0,24,0,1,192,0,96,0,1,192,0,192,
0,3,128,1,128,0,7,0,6,0,0,7,0,12,0,0,
14,0,24,0,0,28,0,96,0,0,56,0,192,0,0,56,
1,128,0,0,112,6,0,0,0,224,12,0,0,0,224,24,
0,0,1,192,96,0,0,3,128,192,0,0,7,1,128,0,
0,7,6,0,0,0,14,12,0,0,0,28,24,0,0,0,
28,96,0,0,0,56,192,0,0,0,113,128,0,0,0,118,
0,0,0,0,236,0,0,0,1,216,0,0,0,3,224,0,
0,0,3,192,0,0,0,7,128,0,0,0,14,0,0,0,
0,12,0,0,0,0,24,0,0,0,0,0,0,0,2,0,
0,0,0,12,0,0,0,0,24,0,0,0,0,96,0,0,
0,0,192,0,0,0,1,128,0,0,0,3,0,0,0,0,
12,0,0,0,0,24,0,0,0,0,48,0,0,0,0,96,
0,0,0,1,128,0,0,0,3,0,0,0,0,6,0,0,
0,0,12,0,0,0,0,48,0,0,0,0,96,0,0,0,
0,192,0,0,0,1,128,0,0,0,6,0,0,0,0,12,
0,0,0,0,24,0,0,0,0,48,0,0,0,0,192,0,
0,0,1,128,0,0,0,3,0,0,0,0,6,0,0,0,
0,12,0,0,0,0,48,0,0,0,0,96,0,0,0,0,
192,0,0,0,1,128,0,0,0,6,0,0,0,0,12,0,
0,0,0,24,0,0,0,0,48,0,0,0,0,192,0,0,
0,1,128,0,0,0,3,0,0,0,0,6,0,0,0,0,
24,0,0,0,0,48,0,0,0,0,96,0,0,0,0,192,
0,0,0,3,0,0,0,0,6,0,0,0,0,12,0,0,
0,0,24,0,0,0,0,96,0,0,0,0,192,0,0,0,
1,128,0,64,0,3,0,1,192,0,12,0,3,128,0,24,
0,15,0,0,48,0,46,0,0,96,0,92,0,1,128,1,
28,0,3,0,0,56,0,6,0,0,112,0,12,0,0,224,
0,48,0,0,224,0,96,0,1,192,0,192,0,3,128,1,
128,0,7,0,6,0,0,14,0,12,0,0,14,0,24,0,
0,28,0,48,0,0,56,0,192,0,0,112,1,128,0,0,
112,3,0,0,0,224,6,0,0,1,192,24,0,0,3,128,
48,0,0,7,0,96,0,0,7,0,192,0,0,14,1,128,
0,0,28,6,0,0,0,56,12,0,0,0,112,24,0,0,
0,112,48,0,0,0,224,192,0,0,1,193,128,0,0,3,
131,0,0,0,3,134,0,0,0,7,24,0,0,0,14,48,
0,0,0,28,96,0,0,0,56,192,0,0,0,59,0,0,
0,0,118,0,0,0,0,236,0,0,0,1,216,0,0,0,
1,224,0,0,0,3,192,0,0,0,7,128,0,0,0,15,
0,0,0,0,28,0,0,0,0,24,0,0,0,0,48,0,
0,0,0,0,0,0,2,0,0,0,0,12,0,0,0,0,
24,0,0,0,0,48,0,0,0,0,192,0,0,0,1,128,
0,0,0,3,0,0,0,0,6,0,0,0,0,12,0,0,
0,0,48,0,0,0,0,96,0,0,0,0,192,0,0,0,
1,128,0,0,0,3,0,0,0,0,12,0,0,0,0,24,
0,0,0,0,48,0,0,0,0,96,0,0,0,0,192,0,
0,0,3,0,0,0,0,6,0,0,0,0,12,0,0,0,
0,24,0,0,0,0,48,0,0,0,0,192,0,0,0,1,
128,0,0,0,3,0,0,0,0,6,0,0,0,0,12,0,
0,0,0,48,0,0,0,0,96,0,0,0,0,192,0,0,
0,1,128,0,0,0,3,0,0,0,0,12,0,0,0,0,
24,0,0,0,0,48,0,0,0,0,96,0,0,0,0,192,
0,0,0,3,0,0,0,0,6,0,0,0,0,12,0,0,
0,0,24,0,0,0,0,48,0,0,0,0,96,0,0,0,
1,128,0,0,0,3,0,0,0,0,6,0,0,0,0,12,
0,0,0,0,24,0,0,0,0,96,0,0,0,0,192,0,
0,0,1,128,0,0,0,3,0,0,0,0,6,0,0,0,
0,24,0,0,0,0,48,0,0,0,0,96,0,0,0,0,
192,0,0,0,1,128,0,0,0,6,0,0,0,0,12,0,
0,0,0,24,0,4,0,0,48,0,8,0,0,96,0,56,
0,1,128,0,112,0,3,0,1,224,0,6,0,3,192,0,
12,0,11,128,0,24,0,23,0,0,96,0,71,0,0,192,
0,14,0,1,128,0,28,0,3,0,0,56,0,6,0,0,
112,0,24,0,0,224,0,48,0,0,224,0,96,0,1,192,
0,192,0,3,128,1,128,0,7,0,6,0,0,14,0,12,
0,0,14,0,24,0,0,28,0,48,0,0,56,0,96,0,
0,112,0,192,0,0,224,3,0,0,1,192,6,0,0,1,
192,12,0,0,3,128,24,0,0,7,0,48,0,0,14,0,
192,0,0,28,1,128,0,0,56,3,0,0,0,56,6,0,
0,0,112,12,0,0,0,224,48,0,0,1,192,96,0,0,
3,128,192,0,0,3,129,128,0,0,7,3,0,0,0,14,
12,0,0,0,28,24,0,0,0,56,48,0,0,0,112,96,
0,0,0,112,192,0,0,0,227,0,0,0,1,198,0,0,
0,3,140,0,0,0,7,24,0,0,0,14,48,0,0,0,
14,192,0,0,0,29,128,0,0,0,59,0,0,0,0,118,
0,0,0,0,236,0,0,0,0,240,0,0,0,1,224,0,
0,0,3,192,0,0,0,7,128,0,0,0,15,0,0,0,
0,28,0,0,0,0,24,0,0,0,0,48,0,0,0,0,
96,0,0,0,0,0,1,128,0,0,192,0,0,96,0,0,
48,0,0,24,0,0,12,0,0,6,0,0,3,0,0,1,
132,0,0,195,0,0,99,128,0,49,192,0,25,112,0,12,
184,0,6,156,0,3,71,0,1,195,128,0,225,192,0,96,
224,0,48,56,0,24,28,0,12,14,0,6,3,128,3,1,
192,1,128,224,0,192,56,0,96,28,0,48,14,0,24,3,
128,12,1,192,6,0,224,3,0,56,1,128,28,0,192,14,
0,96,7,0,48,1,192,24,0,224,12,0,112,6,0,28,
3,0,14,1,128,7,0,192,1,192,96,0,224,48,0,112,
24,0,28,12,0,14,6,0,7,3,0,3,129,128,0,224,
192,0,112,96,0,56,48,0,14,24,0,7,12,0,3,134,
0,0,227,0,0,113,128,0,56,192,0,14,96,0,7,48,
0,3,152,0,0,236,0,0,118,0,0,59,0,0,29,128,
0,7,192,0,3,224,0,1,240,0,0,120,0,0,60,0,
0,30,0,0,7,0,0,3,128,0,1,192,0,0,96,0,
0,32,255,255,255,255,255,255,252,127,255,255,255,240,0,24,
0,12,0,6,0,3,0,1,128,0,192,0,96,0,48,0,
24,0,12,0,6,0,3,0,1,128,0,192,0,96,0,48,
0,24,0,12,0,6,0,3,0,1,128,0,192,0,96,0,
0,224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,
224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,
0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,
0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,0,
31,128,0,0,126,0,0,1,248,0,0,7,224,0,0,31,
128,0,0,126,0,0,1,248,0,0,7,224,0,0,31,128,
0,0,126,0,0,1,248,0,0,7,224,0,0,31,128,0,
0,126,0,0,1,248,0,0,7,224,0,0,31,128,0,0,
126,0,0,1,248,0,0,7,224,0,0,31,128,0,0,126,
0,0,1,255,255,255,255,255,255,255,253,255,255,255,240,224,
0,0,0,3,240,0,0,0,1,248,0,0,0,0,252,0,
0,0,0,126,0,0,0,0,63,0,0,0,0,31,128,0,
0,0,15,192,0,0,0,7,224,0,0,0,3,240,0,0,
0,1,248,0,0,0,0,252,0,0,0,0,126,0,0,0,
0,63,0,0,0,0,31,128,0,0,0,15,192,0,0,0,
7,224,0,0,0,3,240,0,0,0,1,248,0,0,0,0,
252,0,0,0,0,126,0,0,0,0,63,0,0,0,0,31,
128,0,0,0,15,192,0,0,0,7,224,0,0,0,3,240,
0,0,0,1,248,0,0,0,0,252,0,0,0,0,126,0,
0,0,0,63,0,0,0,0,31,128,0,0,0,15,192,0,
0,0,7,224,0,0,0,3,240,0,0,0,1,248,0,0,
0,0,252,0,0,0,0,126,0,0,0,0,63,0,0,0,
0,31,128,0,0,0,15,192,0,0,0,7,224,0,0,0,
3,240,0,0,0,1,248,0,0,0,0,252,0,0,0,0,
126,0,0,0,0,63,0,0,0,0,31,128,0,0,0,15,
192,0,0,0,7,224,0,0,0,3,240,0,0,0,1,248,
0,0,0,0,252,0,0,0,0,126,0,0,0,0,63,0,
0,0,0,31,128,0,0,0,15,255,255,255,255,255,255,255,
255,255,255,191,255,255,255,255,192,0,0,255,128,0,0,3,
255,248,0,0,7,255,255,0,0,15,224,15,224,0,15,128,
0,248,0,15,0,0,30,0,15,0,0,7,128,15,0,0,
1,224,15,0,0,0,120,15,0,0,0,30,7,0,0,0,
7,7,0,0,0,1,195,128,0,0,0,227,128,0,0,0,
57,192,0,0,0,28,224,0,0,0,14,112,0,0,0,7,
112,0,0,0,1,248,0,7,0,0,252,0,7,192,0,126,
0,3,224,0,63,0,1,240,0,31,128,0,248,0,15,192,
0,56,0,7,224,0,0,0,3,184,0,0,0,3,156,0,
0,0,1,206,0,0,0,0,231,0,0,0,0,113,192,0,
0,0,112,224,0,0,0,56,56,0,0,0,56,30,0,0,
0,60,7,128,0,0,60,1,224,0,0,60,0,120,0,0,
60,0,30,0,0,60,0,7,192,0,124,0,1,252,1,252,
0,0,63,255,248,0,0,7,255,240,0,0,0,127,192,0,
0,0,0,0,255,192,0,0,0,0,3,255,255,0,0,0,
0,3,255,255,240,0,0,0,3,252,0,255,0,0,0,3,
240,0,3,240,0,0,3,240,0,0,63,0,0,1,240,0,
0,3,224,0,0,240,0,0,0,60,0,0,120,0,0,0,
7,128,0,60,0,0,0,0,240,0,30,0,0,0,0,30,
0,15,0,0,0,0,3,192,7,128,0,0,0,0,120,1,
192,0,0,0,0,14,0,224,0,0,0,0,1,192,56,0,
0,0,0,0,112,28,0,0,0,0,0,14,7,0,0,0,
0,0,3,131,128,0,0,0,0,0,112,224,0,0,0,0,
0,28,112,0,0,0,0,0,3,156,0,0,0,0,0,0,
231,0,0,0,0,0,0,57,192,0,0,0,0,0,14,224,
0,0,0,0,0,1,248,0,0,0,0,0,0,126,0,0,
1,224,0,0,31,128,0,0,252,0,0,7,224,0,0,63,
0,0,1,248,0,0,15,192,0,0,126,0,0,3,240,0,
0,31,128,0,0,120,0,0,7,224,0,0,0,0,0,1,
248,0,0,0,0,0,0,119,0,0,0,0,0,0,57,192,
0,0,0,0,0,14,112,0,0,0,0,0,3,156,0,0,
0,0,0,0,227,128,0,0,0,0,0,112,224,0,0,0,
0,0,28,28,0,0,0,0,0,14,7,0,0,0,0,0,
3,128,224,0,0,0,0,1,192,56,0,0,0,0,0,112,
7,0,0,0,0,0,56,1,224,0,0,0,0,30,0,60,
0,0,0,0,15,0,7,128,0,0,0,7,128,0,240,0,
0,0,3,192,0,30,0,0,0,1,224,0,3,192,0,0,
0,240,0,0,124,0,0,0,248,0,0,15,192,0,0,252,
0,0,0,252,0,0,252,0,0,0,15,240,3,252,0,0,
0,0,255,255,252,0,0,0,0,15,255,252,0,0,0,0,
0,63,240,0,0,0,0,0,255,128,0,0,3,255,248,0,
0,7,255,255,0,0,15,227,143,224,0,15,129,192,248,0,
15,0,224,30,0,15,0,112,7,128,15,0,56,1,224,15,
0,28,0,120,15,0,14,0,30,7,0,7,0,7,7,0,
3,128,1,195,128,1,192,0,227,128,0,224,0,57,192,0,
112,0,28,224,0,56,0,14,112,0,28,0,7,112,0,14,
0,1,248,0,7,0,0,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,128,0,112,0,15,192,0,56,0,
7,224,0,28,0,3,184,0,14,0,3,156,0,7,0,1,
206,0,3,128,0,231,0,1,192,0,113,192,0,224,0,112,
224,0,112,0,56,56,0,56,0,56,30,0,28,0,60,7,
128,14,0,60,1,224,7,0,60,0,120,3,128,60,0,30,
1,192,60,0,7,192,224,124,0,1,252,113,252,0,0,63,
255,248,0,0,7,255,240,0,0,0,127,192,0,0,0,0,
0,255,192,0,0,0,0,3,255,255,0,0,0,0,3,255,
255,240,0,0,0,3,252,56,255,0,0,0,3,240,14,3,
240,0,0,3,240,3,128,63,0,0,1,240,0,224,3,224,
0,0,240,0,56,0,60,0,0,120,0,14,0,7,128,0,
60,0,3,128,0,240,0,30,0,0,224,0,30,0,15,0,
0,56,0,3,192,7,128,0,14,0,0,120,1,192,0,3,
128,0,14,0,224,0,0,224,0,1,192,56,0,0,56,0,
0,112,28,0,0,14,0,0,14,7,0,0,3,128,0,3,
131,128,0,0,224,0,0,112,224,0,0,56,0,0,28,112,
0,0,14,0,0,3,156,0,0,3,128,0,0,231,0,0,
0,224,0,0,57,192,0,0,56,0,0,14,224,0,0,14,
0,0,1,248,0,0,3,128,0,0,126,0,0,0,224,0,
0,31,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,254,0,0,0,224,0,0,31,128,
0,0,56,0,0,7,224,0,0,14,0,0,1,248,0,0,
3,128,0,0,119,0,0,0,224,0,0,57,192,0,0,56,
0,0,14,112,0,0,14,0,0,3,156,0,0,3,128,0,
0,227,128,0,0,224,0,0,112,224,0,0,56,0,0,28,
28,0,0,14,0,0,14,7,0,0,3,128,0,3,128,224,
0,0,224,0,1,192,56,0,0,56,0,0,112,7,0,0,
14,0,0,56,1,224,0,3,128,0,30,0,60,0,0,224,
0,15,0,7,128,0,56,0,7,128,0,240,0,14,0,3,
192,0,30,0,3,128,1,224,0,3,192,0,224,0,240,0,
0,124,0,56,0,248,0,0,15,192,14,0,252,0,0,0,
252,3,128,252,0,0,0,15,240,227,252,0,0,0,0,255,
255,252,0,0,0,0,15,255,252,0,0,0,0,0,63,240,
0,0,0,0,0,255,128,0,0,3,255,248,0,0,7,255,
255,0,0,15,224,15,224,0,15,128,0,248,0,15,0,0,
30,0,15,0,0,7,128,15,0,0,1,224,15,192,0,1,
248,15,240,0,1,254,7,60,0,1,231,7,15,0,1,225,
195,131,192,1,224,227,128,240,1,224,57,192,60,1,224,28,
224,15,1,224,14,112,3,193,224,7,112,0,241,224,1,248,
0,61,224,0,252,0,15,224,0,126,0,3,224,0,63,0,
1,240,0,31,128,1,252,0,15,192,1,239,0,7,224,1,
227,192,3,184,1,224,240,3,156,1,224,60,1,206,1,224,
15,0,231,1,224,3,192,113,193,224,0,240,112,225,224,0,
60,56,57,224,0,15,56,31,224,0,3,252,7,224,0,0,
252,1,224,0,0,60,0,120,0,0,60,0,30,0,0,60,
0,7,192,0,124,0,1,252,1,252,0,0,63,255,248,0,
0,7,255,240,0,0,0,127,192,0,0,0,0,0,255,192,
0,0,0,0,3,255,255,0,0,0,0,3,255,255,240,0,
0,0,3,252,0,255,0,0,0,3,240,0,3,240,0,0,
3,240,0,0,63,0,0,1,240,0,0,3,224,0,0,240,
0,0,0,60,0,0,120,0,0,0,7,128,0,60,0,0,
0,0,240,0,31,128,0,0,0,62,0,15,240,0,0,0,
31,192,7,158,0,0,0,15,120,1,195,192,0,0,7,142,
0,224,120,0,0,3,193,192,56,15,0,0,1,224,112,28,
1,224,0,0,240,14,7,0,60,0,0,120,3,131,128,7,
128,0,60,0,112,224,0,240,0,30,0,28,112,0,30,0,
15,0,3,156,0,3,192,7,128,0,231,0,0,120,3,192,
0,57,192,0,15,1,224,0,14,224,0,1,224,240,0,1,
248,0,0,60,120,0,0,126,0,0,7,188,0,0,31,128,
0,0,254,0,0,7,224,0,0,31,0,0,1,248,0,0,
7,192,0,0,126,0,0,3,248,0,0,31,128,0,1,239,
0,0,7,224,0,0,241,224,0,1,248,0,0,120,60,0,
0,119,0,0,60,7,128,0,57,192,0,30,0,240,0,14,
112,0,15,0,30,0,3,156,0,7,128,3,192,0,227,128,
3,192,0,120,0,112,224,1,224,0,15,0,28,28,0,240,
0,1,224,14,7,0,120,0,0,60,3,128,224,60,0,0,
7,129,192,56,30,0,0,0,240,112,7,15,0,0,0,30,
56,1,231,128,0,0,3,222,0,63,192,0,0,0,127,0,
7,224,0,0,0,15,128,0,240,0,0,0,3,192,0,30,
0,0,0,1,224,0,3,192,0,0,0,240,0,0,124,0,
0,0,248,0,0,15,192,0,0,252,0,0,0,252,0,0,
252,0,0,0,15,240,3,252,0,0,0,0,255,255,252,0,
0,0,0,15,255,252,0,0,0,0,0,63,240,0,0,0,
255,255,255,255,225,255,255,255,255,225,192,0,0,127,195,192,
0,0,15,131,192,0,0,7,131,128,0,0,3,3,128,0,
0,3,7,128,0,0,2,7,0,0,0,4,7,0,0,0,
4,15,0,0,0,0,15,0,0,0,0,14,0,0,0,0,
14,0,0,0,0,30,0,0,0,0,28,0,0,0,0,28,
0,0,0,0,60,0,0,0,0,60,0,0,0,0,56,0,
0,0,0,56,0,0,0,0,112,0,0,0,0,192,0,0,
0,3,128,0,0,0,6,0,0,0,0,24,0,0,0,0,
96,0,0,0,1,192,0,0,0,3,0,0,0,0,12,0,
0,0,0,48,0,0,0,0,224,0,0,0,1,128,0,0,
2,6,0,0,0,8,24,0,0,0,16,112,0,0,0,96,
192,0,0,1,131,0,0,0,15,12,0,0,0,124,56,0,
0,15,248,127,255,255,255,241,255,255,255,255,192,255,255,255,
255,255,255,193,255,255,255,255,255,255,193,252,0,0,0,7,
255,131,248,0,0,0,0,255,3,248,0,0,0,0,63,3,
248,0,0,0,0,30,3,240,0,0,0,0,14,7,240,0,
0,0,0,12,7,240,0,0,0,0,12,7,240,0,0,0,
0,8,15,224,0,0,0,0,16,15,224,0,0,0,0,16,
15,224,0,0,0,0,0,15,224,0,0,0,0,0,31,192,
0,0,0,0,0,31,192,0,0,0,0,0,31,192,0,0,
0,0,0,31,128,0,0,0,0,0,63,128,0,0,0,0,
0,63,128,0,0,0,0,0,63,128,0,0,0,0,0,127,
0,0,0,0,0,0,127,0,0,0,0,0,0,127,0,0,
0,0,0,0,126,0,0,0,0,0,0,254,0,0,0,0,
0,0,254,0,0,0,0,0,0,254,0,0,0,0,0,1,
252,0,0,0,0,0,1,248,0,0,0,0,0,1,224,0,
0,0,0,0,1,192,0,0,0,0,0,7,0,0,0,0,
0,0,28,0,0,0,0,0,0,112,0,0,0,0,0,1,
192,0,0,0,0,0,7,0,0,0,0,0,0,14,0,0,
0,0,0,0,56,0,0,0,0,0,0,224,0,0,0,0,
0,3,128,0,0,0,0,0,14,0,0,0,0,0,0,56,
0,0,0,0,0,0,112,0,0,0,0,0,1,192,0,0,
0,0,32,7,0,0,0,0,0,128,28,0,0,0,0,1,
0,112,0,0,0,0,6,1,224,0,0,0,0,24,3,128,
0,0,0,0,112,14,0,0,0,0,3,224,56,0,0,0,
0,15,128,224,0,0,0,0,255,3,128,0,0,0,63,254,
15,255,255,255,255,255,248,31,255,255,255,255,255,240,127,255,
255,255,255,255,225,255,255,255,255,255,255,128,255,255,255,255,
255,255,255,255,241,240,0,3,224,60,0,0,240,15,0,0,
60,3,192,0,15,0,240,0,3,192,60,0,0,240,15,0,
0,60,3,192,0,15,0,240,0,3,192,60,0,0,240,15,
0,0,60,3,192,0,15,0,240,0,3,192,60,0,0,240,
15,0,0,60,3,192,0,15,0,240,0,3,192,60,0,0,
240,15,0,0,60,3,192,0,15,0,240,0,3,192,60,0,
0,240,15,0,0,60,3,192,0,15,0,240,0,3,192,60,
0,0,240,15,0,0,60,3,192,0,15,0,240,0,3,192,
60,0,0,240,15,0,0,60,3,192,0,15,0,240,0,3,
192,60,0,0,240,15,0,0,60,3,192,0,15,0,240,0,
3,192,126,0,1,248,255,240,3,255,255,252,0,255,240,255,
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
255,15,248,0,0,31,240,3,248,0,0,31,192,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,
0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,
248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,
128,1,248,0,0,31,128,1,248,0,0,31,128,3,252,0,
0,63,192,15,255,0,0,255,240,255,255,240,15,255,255,255,
255,240,15,255,255,255,255,240,15,255,255,255,240,3,255,255,
252,0,255,241,248,0,7,224,60,0,0,240,15,0,0,60,
3,192,0,15,0,240,0,3,192,60,0,0,240,15,0,0,
60,3,192,0,15,0,240,0,3,192,60,0,0,240,15,0,
0,60,3,192,0,15,0,240,0,3,192,60,0,0,240,15,
0,0,60,3,192,0,15,0,240,0,3,192,60,0,0,240,
15,0,0,60,3,192,0,15,0,240,0,3,192,60,0,0,
240,15,0,0,60,3,192,0,15,0,240,0,3,192,60,0,
0,240,15,0,0,60,3,192,0,15,0,240,0,3,192,60,
0,0,240,15,0,0,60,3,192,0,15,0,240,0,3,192,
60,0,0,240,15,0,0,60,3,192,0,15,0,240,0,3,
192,124,0,0,248,255,255,255,255,255,255,255,255,240,255,255,
240,15,255,255,255,255,240,15,255,255,255,255,240,15,255,255,
15,255,0,0,255,240,3,252,0,0,63,192,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,1,248,0,0,
31,128,1,248,0,0,31,128,1,248,0,0,31,128,1,248,
0,0,31,128,1,248,0,0,31,128,1,248,0,0,31,128,
1,248,0,0,31,128,1,248,0,0,31,128,3,248,0,0,
31,192,15,248,0,0,31,240,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,255,255,255,0,0,248,0,3,8,
0,12,24,0,48,112,0,96,64,1,192,0,3,128,0,7,
0,0,12,0,0,56,0,0,112,0,0,224,0,1,192,0,
3,128,0,7,0,0,14,0,0,28,0,0,56,0,0,224,
0,1,192,0,3,128,0,7,0,0,14,0,0,28,0,0,
56,0,0,112,0,0,224,0,1,192,0,7,0,0,14,0,
0,28,0,0,56,0,0,112,0,0,224,0,1,192,0,3,
128,0,7,0,0,12,0,0,56,0,0,112,0,0,224,0,
129,128,3,131,0,6,12,0,4,48,0,7,192,0,0,0,
0,0,3,128,0,0,0,102,0,0,0,12,112,0,0,0,
207,0,0,0,12,240,0,0,1,198,0,0,0,24,0,0,
0,3,128,0,0,0,56,0,0,0,3,128,0,0,0,112,
0,0,0,7,0,0,0,0,112,0,0,0,7,0,0,0,
0,240,0,0,0,14,0,0,0,0,224,0,0,0,14,0,
0,0,1,224,0,0,0,30,0,0,0,1,192,0,0,0,
28,0,0,0,3,192,0,0,0,60,0,0,0,3,192,0,
0,0,60,0,0,0,7,128,0,0,0,120,0,0,0,7,
128,0,0,0,120,0,0,0,15,128,0,0,0,248,0,0,
0,15,0,0,0,0,240,0,0,0,31,0,0,0,1,240,
0,0,0,31,0,0,0,1,240,0,0,0,30,0,0,0,
3,224,0,0,0,62,0,0,0,3,224,0,0,0,62,0,
0,0,3,192,0,0,0,124,0,0,0,7,192,0,0,0,
124,0,0,0,7,192,0,0,0,120,0,0,0,15,128,0,
0,0,248,0,0,0,15,128,0,0,0,248,0,0,0,15,
0,0,0,1,240,0,0,0,31,0,0,0,1,240,0,0,
0,31,0,0,0,1,224,0,0,0,30,0,0,0,3,224,
0,0,0,62,0,0,0,3,192,0,0,0,60,0,0,0,
3,192,0,0,0,60,0,0,0,7,128,0,0,0,120,0,
0,0,7,128,0,0,0,120,0,0,0,7,0,0,0,0,
112,0,0,0,15,0,0,0,0,240,0,0,0,14,0,0,
0,0,224,0,0,0,14,0,0,0,1,224,0,0,0,28,
0,0,0,1,192,0,0,0,28,0,0,0,1,128,0,0,
0,56,0,0,0,3,128,0,0,0,48,0,0,0,7,0,
0,0,96,96,0,0,15,6,0,0,0,240,192,0,0,14,
24,0,0,0,99,0,0,0,1,224,0,0,0,0,0,248,
0,3,8,0,12,24,0,48,112,0,96,64,1,192,0,3,
128,0,7,0,0,12,0,0,56,0,0,112,0,0,224,0,
1,192,0,3,128,0,7,0,0,62,0,1,255,0,7,63,
0,24,227,0,49,198,0,195,134,1,135,12,3,14,24,6,
28,48,12,56,96,24,112,192,24,227,0,49,198,0,63,56,
0,63,224,0,31,0,0,56,0,0,112,0,0,224,0,1,
192,0,3,128,0,7,0,0,12,0,0,56,0,0,112,0,
0,224,0,129,128,3,131,0,6,12,0,4,48,0,7,192,
0,0,0,0,0,3,128,0,0,0,102,0,0,0,12,112,
0,0,0,207,0,0,0,12,240,0,0,1,198,0,0,0,
24,0,0,0,3,128,0,0,0,56,0,0,0,3,128,0,
0,0,112,0,0,0,7,0,0,0,0,112,0,0,0,7,
0,0,0,0,240,0,0,0,14,0,0,0,0,224,0,0,
0,14,0,0,0,1,224,0,0,0,30,0,0,0,1,192,
0,0,0,28,0,0,0,3,192,0,0,0,60,0,0,0,
3,192,0,0,0,60,0,0,0,7,128,0,0,0,120,0,
0,0,7,128,0,0,0,120,0,0,0,15,128,0,0,0,
248,0,0,0,15,0,0,0,0,240,0,0,0,31,0,0,
0,7,240,0,0,3,255,192,0,0,121,254,0,0,12,30,
48,0,1,131,225,128,0,48,62,12,0,3,3,224,192,0,
48,62,12,0,6,3,192,96,0,96,124,6,0,6,7,192,
96,0,96,124,6,0,6,7,192,96,0,96,120,6,0,3,
15,128,192,0,48,248,12,0,3,15,128,192,0,24,248,24,
0,0,207,3,0,0,7,241,224,0,0,63,252,0,0,1,
254,0,0,0,31,0,0,0,1,224,0,0,0,30,0,0,
0,3,224,0,0,0,62,0,0,0,3,192,0,0,0,60,
0,0,0,3,192,0,0,0,60,0,0,0,7,128,0,0,
0,120,0,0,0,7,128,0,0,0,120,0,0,0,7,0,
0,0,0,112,0,0,0,15,0,0,0,0,240,0,0,0,
14,0,0,0,0,224,0,0,0,14,0,0,0,1,224,0,
0,0,28,0,0,0,1,192,0,0,0,28,0,0,0,1,
128,0,0,0,56,0,0,0,3,128,0,0,0,48,0,0,
0,7,0,0,0,96,96,0,0,15,6,0,0,0,240,192,
0,0,14,24,0,0,0,99,0,0,0,1,224,0,0,0,
224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,
0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,
0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,0,
31,128,0,0,126,0,0,1,248,0,0,7,224,0,0,31,
128,0,0,126,0,0,1,248,0,0,7,224,0,0,31,128,
0,0,126,0,0,1,248,0,0,7,224,0,0,31,128,0,
0,126,0,0,1,248,0,0,7,224,0,0,31,128,0,0,
126,0,0,1,220,0,0,14,112,0,0,56,224,0,1,195,
128,0,7,7,0,0,56,30,0,1,224,62,0,31,0,126,
1,248,0,127,255,128,0,127,248,0,0,127,128,0,224,0,
0,0,3,240,0,0,0,1,248,0,0,0,0,252,0,0,
0,0,126,0,0,0,0,63,0,0,0,0,31,128,0,0,
0,15,192,0,0,0,7,224,0,0,0,3,240,0,0,0,
1,248,0,0,0,0,252,0,0,0,0,126,0,0,0,0,
63,0,0,0,0,31,128,0,0,0,15,192,0,0,0,7,
224,0,0,0,3,240,0,0,0,1,248,0,0,0,0,252,
0,0,0,0,126,0,0,0,0,63,0,0,0,0,31,128,
0,0,0,15,192,0,0,0,7,224,0,0,0,3,240,0,
0,0,1,248,0,0,0,0,252,0,0,0,0,126,0,0,
0,0,63,0,0,0,0,31,128,0,0,0,15,192,0,0,
0,7,224,0,0,0,3,240,0,0,0,1,248,0,0,0,
0,252,0,0,0,0,126,0,0,0,0,63,0,0,0,0,
31,128,0,0,0,15,192,0,0,0,7,224,0,0,0,3,
240,0,0,0,1,220,0,0,0,1,206,0,0,0,0,231,
0,0,0,0,113,192,0,0,0,112,224,0,0,0,56,56,
0,0,0,56,30,0,0,0,60,7,0,0,0,28,1,192,
0,0,28,0,248,0,0,62,0,62,0,0,62,0,7,192,
0,124,0,1,252,1,252,0,0,63,255,248,0,0,7,255,
240,0,0,0,127,192,0,0,0,31,224,0,1,255,224,0,
31,255,224,1,248,7,224,15,128,7,192,120,0,7,129,192,
0,14,14,0,0,28,56,0,0,113,192,0,0,231,0,0,
3,184,0,0,7,224,0,0,31,128,0,0,126,0,0,1,
248,0,0,7,224,0,0,31,128,0,0,126,0,0,1,248,
0,0,7,224,0,0,31,128,0,0,126,0,0,1,248,0,
0,7,224,0,0,31,128,0,0,126,0,0,1,248,0,0,
7,224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,
224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,
0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,
0,31,128,0,0,112,0,0,255,128,0,0,3,255,248,0,
0,7,255,255,0,0,15,224,15,224,0,15,128,0,248,0,
31,0,0,31,0,31,0,0,7,192,14,0,0,0,224,14,
0,0,0,56,15,0,0,0,30,7,0,0,0,7,7,0,
0,0,1,195,128,0,0,0,227,128,0,0,0,57,192,0,
0,0,28,224,0,0,0,14,224,0,0,0,3,240,0,0,
0,1,248,0,0,0,0,252,0,0,0,0,126,0,0,0,
0,63,0,0,0,0,31,128,0,0,0,15,192,0,0,0,
7,224,0,0,0,3,240,0,0,0,1,248,0,0,0,0,
252,0,0,0,0,126,0,0,0,0,63,0,0,0,0,31,
128,0,0,0,15,192,0,0,0,7,224,0,0,0,3,240,
0,0,0,1,248,0,0,0,0,252,0,0,0,0,126,0,
0,0,0,63,0,0,0,0,31,128,0,0,0,15,192,0,
0,0,7,224,0,0,0,3,240,0,0,0,1,248,0,0,
0,0,252,0,0,0,0,126,0,0,0,0,63,0,0,0,
0,31,128,0,0,0,15,192,0,0,0,7,224,0,0,0,
3,240,0,0,0,1,248,0,0,0,0,252,0,0,0,0,
126,0,0,0,0,63,0,0,0,0,31,128,0,0,0,15,
192,0,0,0,7,224,0,0,0,3,240,0,0,0,1,192,
224,0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,
0,0,31,128,0,0,126,0,0,1,248,0,0,7,224,0,
0,31,128,0,0,126,0,0,1,248,0,192,7,224,3,128,
31,128,14,0,126,0,56,1,248,0,224,7,224,3,128,31,
128,14,0,126,31,255,225,248,255,255,199,227,255,255,31,128,
14,0,126,0,56,1,248,0,224,7,224,3,128,31,128,14,
0,126,0,56,1,248,0,192,7,224,3,0,31,128,0,0,
126,0,0,1,220,0,0,14,112,0,0,56,224,0,1,195,
128,0,7,7,0,0,56,30,0,1,224,62,0,31,0,126,
1,248,0,127,255,128,0,127,248,0,0,127,128,0,224,0,
0,0,3,240,0,0,0,1,248,0,0,0,0,252,0,0,
0,0,126,0,0,0,0,63,0,0,0,0,31,128,0,0,
0,15,192,0,0,0,7,224,0,0,0,3,240,0,0,0,
1,248,0,0,0,0,252,0,0,0,0,126,0,0,0,0,
63,0,0,224,0,31,128,0,112,0,15,192,0,56,0,7,
224,0,28,0,3,240,0,14,0,1,248,0,7,0,0,252,
0,3,128,0,126,0,1,192,0,63,0,0,224,0,31,128,
0,112,0,15,192,0,56,0,7,224,0,28,0,3,240,0,
14,0,1,248,255,255,255,248,252,127,255,255,252,126,31,255,
255,252,63,0,0,224,0,31,128,0,112,0,15,192,0,56,
0,7,224,0,28,0,3,240,0,14,0,1,248,0,7,0,
0,252,0,3,128,0,126,0,1,192,0,63,0,0,224,0,
31,128,0,112,0,15,192,0,56,0,7,224,0,28,0,3,
240,0,4,0,1,220,0,0,0,1,206,0,0,0,0,231,
0,0,0,0,113,192,0,0,0,112,224,0,0,0,56,56,
0,0,0,56,30,0,0,0,60,7,0,0,0,28,1,192,
0,0,28,0,248,0,0,62,0,62,0,0,62,0,7,192,
0,124,0,1,252,1,252,0,0,63,255,248,0,0,7,255,
240,0,0,0,127,192,0,0,0,3,0,0,0,14,0,0,
0,120,0,0,1,224,0,0,15,192,0,0,63,0,0,0,
252,0,0,7,56,0,0,28,224,0,0,115,128,0,3,135,
0,0,14,28,0,0,56,112,0,1,192,224,0,7,3,128,
0,28,14,0,0,224,28,0,3,128,112,0,14,1,192,0,
112,3,128,1,192,14,0,7,0,56,0,56,0,112,0,224,
1,192,3,128,7,0,28,0,14,0,112,0,56,3,128,0,
112,14,0,1,192,56,0,7,1,192,0,14,7,0,0,56,
28,0,0,224,224,0,1,195,128,0,7,14,0,0,28,112,
0,0,57,192,0,0,231,0,0,3,184,0,0,7,224,0,
0,31,0,0,0,48,0,0,12,0,0,0,0,14,0,0,
0,0,7,0,0,0,0,7,192,0,0,0,3,224,0,0,
0,1,240,0,0,0,1,220,0,0,0,0,238,0,0,0,
0,119,0,0,0,0,113,192,0,0,0,56,224,0,0,0,
28,112,0,0,0,28,28,0,0,0,14,14,0,0,0,7,
7,0,0,0,7,1,192,0,0,3,128,224,0,0,3,128,
56,0,0,1,192,28,0,0,0,224,14,0,0,0,224,3,
128,0,0,112,1,192,0,0,56,0,224,0,0,56,0,56,
0,0,28,0,28,0,0,14,0,14,0,0,14,0,3,128,
0,7,0,1,192,0,3,128,0,224,0,3,128,0,56,0,
1,192,0,28,0,0,224,0,14,0,0,224,0,3,128,0,
112,0,1,192,0,56,0,0,224,0,56,0,0,56,0,28,
0,0,28,0,14,0,0,14,0,14,0,0,3,128,7,0,
0,1,192,3,128,0,0,224,3,128,0,0,56,1,192,0,
0,28,1,192,0,0,7,0,224,0,0,3,128,112,0,0,
1,192,112,0,0,0,112,56,0,0,0,56,28,0,0,0,
28,28,0,0,0,7,14,0,0,0,3,135,0,0,0,1,
199,0,0,0,0,115,128,0,0,0,57,192,0,0,0,29,
192,0,0,0,7,224,0,0,0,3,224,0,0,0,0,192,
192,0,0,15,128,0,0,126,0,0,1,220,0,0,14,112,
0,0,57,192,0,0,227,128,0,7,14,0,0,28,56,0,
0,112,112,0,3,129,192,0,14,7,0,0,56,14,0,1,
192,56,0,7,0,224,0,28,1,192,0,224,7,0,3,128,
14,0,28,0,56,0,112,0,224,1,192,1,192,14,0,7,
0,56,0,28,0,224,0,56,7,0,0,224,28,0,3,128,
112,0,7,3,128,0,28,14,0,0,112,56,0,0,225,192,
0,3,135,0,0,14,28,0,0,28,224,0,0,115,128,0,
1,206,0,0,3,240,0,0,15,192,0,0,63,0,0,0,
120,0,0,1,224,0,0,3,128,0,0,12,0,0,192,0,
0,0,1,240,0,0,0,1,248,0,0,0,0,238,0,0,
0,0,231,0,0,0,0,115,128,0,0,0,56,224,0,0,
0,56,112,0,0,0,28,56,0,0,0,14,14,0,0,0,
14,7,0,0,0,7,3,128,0,0,3,128,224,0,0,3,
128,112,0,0,1,192,56,0,0,0,224,14,0,0,0,224,
7,0,0,0,112,1,192,0,0,112,0,224,0,0,56,0,
112,0,0,28,0,28,0,0,28,0,14,0,0,14,0,7,
0,0,7,0,1,192,0,7,0,0,224,0,3,128,0,112,
0,1,192,0,28,0,1,192,0,14,0,0,224,0,7,0,
0,112,0,1,192,0,112,0,0,224,0,56,0,0,112,0,
28,0,0,28,0,28,0,0,14,0,14,0,0,7,0,7,
0,0,1,192,7,0,0,0,224,3,128,0,0,112,1,192,
0,0,28,1,192,0,0,14,0,224,0,0,7,0,112,0,
0,1,192,112,0,0,0,224,56,0,0,0,56,56,0,0,
0,28,28,0,0,0,14,14,0,0,0,3,142,0,0,0,
1,199,0,0,0,0,227,128,0,0,0,59,128,0,0,0,
29,192,0,0,0,14,224,0,0,0,3,224,0,0,0,1,
240,0,0,0,0,248,0,0,0,0,56,0,0,0,0,28,
0,0,0,0,6,0,0,0,0,48,0,3,240,0,28,224,
1,192,224,28,0,224,192,0,204,0,0,192,0,0,28,0,
0,0,0,127,192,0,0,1,248,252,0,0,7,224,15,192,
0,31,0,0,124,0,124,0,0,7,192,224,0,0,0,57,
128,0,0,0,3,0,0,0,31,0,0,0,0,0,0,63,
254,0,0,0,0,0,127,199,252,0,0,0,0,255,0,7,
248,0,0,1,252,0,0,7,240,0,7,240,0,0,0,7,
240,15,192,0,0,0,0,7,231,0,0,0,0,0,0,7,
7,192,4,127,128,99,7,131,24,7,248,128,15,128,0,127,
128,0,0,129,255,252,0,3,135,128,31,192,15,14,0,1,
255,252,8,0,0,15,240,0,0,7,255,128,0,0,0,96,
31,255,255,0,0,0,112,30,0,0,254,0,0,240,28,0,
0,1,255,255,240,12,0,0,0,3,255,192,0,
};
METRIC cmmi10_metric[128] = {
{4034, -2, 27, 27, 28, 403229},
{4129, -2, 28, 30, 29, 546135},
{4238, -2, 28, 28, 30, 499893},
{4343, -1, 28, 26, 29, 455112},
{4438, -2, 27, 30, 28, 486515},
{4543, -2, 27, 33, 28, 544769},
{4659, -2, 27, 31, 28, 511091},
{4768, -1, 28, 27, 29, 382293},
{4866, 0, 27, 26, 28, 436907},
{4957, -1, 27, 27, 28, 401226},
{5052, -3, 28, 29, 29, 506199},
{5158, -2, 17, 22, 18, 419235},
{5208, -1, 28, 23, 37, 370689},
{5315, -1, 17, 20, 27, 339300},
{5383, -2, 28, 15, 29, 291271},
{5438, -2, 17, 13, 18, 266013},
{5468, -1, 28, 17, 37, 286721},
{5547, -2, 17, 18, 27, 325407},
{5608, -2, 28, 16, 29, 307655},
{5666, -2, 17, 10, 18, 231956},
{5689, -2, 17, 19, 18, 377592},
{5732, -2, 28, 19, 29, 382295},
{5801, -1, 17, 21, 27, 394887},
{5872, -2, 17, 19, 18, 323737},
{5915, 0, 28, 17, 37, 286721},
{5994, -2, 17, 21, 18, 373573},
{6042, -1, 17, 19, 27, 338831},
{6107, -2, 17, 21, 18, 374482},
{6155, -2, 17, 19, 18, 286494},
{6198, -2, 17, 19, 18, 354078},
{6241, -2, 28, 20, 37, 390486},
{6334, -1, 17, 22, 26, 410054},
{6406, -2, 28, 23, 37, 426896},
{6513, 0, 17, 23, 18, 407932},
{6565, -1, 18, 16, 20, 305606},
{6605, -2, 28, 20, 29, 387606},
{6678, -2, 17, 31, 18, 542723},
{6748, -3, 17, 17, 26, 338831},
{6804, -1, 17, 15, 22, 237796},
{6846, -2, 17, 23, 27, 428715},
{7742, -2, 16, 37, 8, 655362},
{7779, -2, 10, 37, 8, 655362},
{7816, -2, 16, 37, 8, 655362},
{7853, -2, 10, 37, 8, 655362},
{7890, -2, 19, 7, 11, 182045},
{7900, -2, 19, 7, 11, 182045},
{7910, -1, 19, 18, 20, 327681},
{7955, -2, 19, 17, 20, 327681},
{7228, -2, 18, 16, 20, 327681},
{7268, -4, 18, 13, 19, 327681},
{7299, -2, 18, 16, 19, 327681},
{7337, -2, 18, 16, 28, 327681},
{7393, -1, 18, 18, 27, 327681},
{7454, -2, 18, 16, 28, 327681},
{7510, -2, 27, 16, 29, 327681},
{7568, -2, 18, 17, 28, 327681},
{7628, -2, 27, 16, 29, 327681},
{7686, -2, 18, 16, 28, 327681},
{7998, -4, 3, 4, 4, 182045},
{8000, -4, 3, 4, 12, 182045},
{8006, -3, 21, 25, 24, 509726},
{8081, -2, 30, 16, 41, 327681},
{8163, -3, 21, 25, 24, 509726},
{8238, 0, 19, 21, 20, 327681},
{6924, -2, 29, 20, 31, 347933},
{0, -1, 28, 28, 29, 491521},
{102, -2, 27, 28, 28, 497097},
{200, -2, 28, 29, 30, 468400},
{309, -2, 27, 31, 28, 542584},
{418, -2, 27, 29, 28, 483784},
{520, -2, 27, 28, 28, 421434},
{618, -2, 28, 29, 30, 515276},
{727, -2, 27, 33, 28, 544769},
{843, -2, 27, 18, 28, 288086},
{906, -3, 27, 21, 29, 363406},
{983, -2, 27, 33, 28, 556602},
{1099, -2, 27, 23, 28, 446010},
{1180, -2, 27, 40, 28, 635791},
{1320, -2, 27, 33, 28, 526564},
{1436, -2, 28, 28, 30, 499893},
{1541, -2, 27, 28, 28, 420750},
{1639, -2, 28, 28, 37, 518098},
{1769, -2, 27, 29, 29, 497608},
{1875, -2, 28, 24, 30, 401864},
{1965, -1, 27, 28, 28, 382977},
{2063, -3, 27, 27, 29, 447465},
{2161, -3, 27, 28, 29, 382294},
{2263, -3, 27, 40, 29, 618952},
{2408, -1, 27, 33, 28, 542949},
{2524, -1, 27, 29, 28, 380474},
{2626, -2, 27, 27, 28, 447375},
{8291, -2, 30, 11, 32, 254863},
{8335, -3, 30, 9, 40, 254863},
{8380, -2, 29, 11, 39, 254863},
{8434, -2, 15, 37, 10, 655362},
{8481, -2, 16, 37, 11, 655362},
{7002, 0, 28, 16, 29, 273069},
{2721, -2, 17, 17, 18, 346417},
{2760, -2, 28, 14, 29, 281259},
{2811, -2, 17, 15, 18, 283611},
{2845, -2, 28, 19, 29, 341107},
{2914, -2, 17, 14, 18, 305153},
{2946, -2, 28, 21, 37, 320855},
{3044, -1, 17, 17, 26, 312586},
{3100, -2, 28, 19, 29, 377592},
{3169, -2, 27, 10, 28, 225780},
{3204, 0, 27, 16, 36, 269882},
{3276, -2, 28, 18, 29, 341183},
{3342, -2, 28, 9, 29, 195546},
{3375, -2, 17, 32, 18, 575415},
{3447, -2, 17, 20, 18, 393370},
{3492, -2, 17, 17, 18, 317668},
{3531, 1, 17, 21, 26, 329729},
{3600, -2, 17, 15, 26, 292562},
{3649, -2, 17, 16, 18, 295672},
{3685, -2, 17, 15, 18, 307200},
{3719, 0, 25, 12, 26, 236659},
{3758, -2, 17, 19, 18, 375166},
{3801, -2, 17, 16, 18, 317669},
{3837, -2, 17, 26, 18, 469184},
{3896, -2, 17, 19, 18, 374557},
{3939, -2, 17, 18, 26, 321311},
{3998, -2, 17, 16, 18, 304775},
{7060, -2, 17, 9, 18, 211325},
{7081, 0, 17, 14, 26, 251678},
{7127, -3, 18, 21, 28, 417110},
{7201, -8, 29, 17, 8, 327681},
{7218, -11, 27, 16, 5, 182045},
};
char cmmi10_bitmap[8532] = {
0,0,12,0,0,0,192,0,0,28,0,0,1,192,0,0,
60,0,0,7,192,0,0,92,0,0,9,192,0,0,142,0,
0,16,224,0,1,14,0,0,32,224,0,4,14,0,0,64,
224,0,8,14,0,0,128,224,0,16,14,0,2,0,224,0,
63,254,0,4,0,112,0,64,7,0,8,0,112,1,0,7,
0,16,0,112,2,0,7,0,32,0,112,6,0,7,1,224,
0,112,255,128,127,240,1,255,255,0,3,192,28,0,56,0,
224,3,128,15,0,56,0,112,3,128,7,0,112,0,112,7,
0,15,0,112,0,240,7,0,30,0,224,3,192,14,0,120,
0,224,31,0,15,255,192,1,192,15,0,28,0,120,1,192,
3,192,28,0,60,3,128,3,192,56,0,60,3,128,3,192,
56,0,60,7,0,7,128,112,0,112,7,0,14,0,112,1,
192,14,0,120,15,255,252,0,0,1,248,8,0,112,48,192,
14,0,78,0,192,1,224,28,0,7,1,192,0,56,28,0,
0,193,224,0,4,14,0,0,32,224,0,1,15,0,0,8,
120,0,0,7,128,0,0,60,0,0,1,224,0,0,30,0,
0,0,240,0,0,7,128,0,0,60,0,0,1,224,0,2,
15,0,0,16,120,0,0,129,192,0,8,14,0,0,64,48,
0,4,1,192,0,64,7,0,4,0,28,0,192,0,112,24,
0,0,127,0,0,1,255,255,128,0,120,3,192,0,224,1,
192,1,192,1,192,3,128,1,192,7,0,3,128,28,0,7,
0,56,0,15,0,112,0,30,0,224,0,60,3,128,0,120,
7,0,0,240,14,0,1,224,28,0,3,192,112,0,15,0,
224,0,30,1,192,0,60,3,128,0,112,14,0,1,224,28,
0,3,128,56,0,15,0,112,0,28,1,192,0,112,3,128,
1,192,7,0,7,0,14,0,56,0,56,1,224,7,255,254,
0,0,1,255,255,248,1,224,3,192,14,0,6,0,112,0,
32,3,128,1,0,28,0,8,1,192,0,64,14,0,2,0,
112,16,16,3,128,128,0,56,8,0,1,192,64,0,14,6,
0,0,127,240,0,7,1,0,0,56,8,0,1,192,64,0,
14,2,0,0,224,32,16,7,0,0,128,56,0,8,1,192,
0,64,28,0,4,0,224,0,32,7,0,3,0,56,0,48,
3,128,15,129,255,255,248,0,1,255,255,240,3,192,15,0,
56,0,48,3,128,2,0,56,0,32,3,128,2,0,112,0,
32,7,0,2,0,112,16,32,7,1,0,0,224,32,0,14,
2,0,0,224,96,0,15,254,0,1,192,64,0,28,4,0,
1,192,64,0,28,4,0,3,128,128,0,56,0,0,3,128,
0,0,56,0,0,7,0,0,0,112,0,0,7,0,0,0,
112,0,0,15,0,0,15,255,0,0,0,1,248,8,0,112,
48,192,14,0,78,0,192,1,224,28,0,7,1,192,0,56,
28,0,0,193,224,0,4,14,0,0,32,224,0,1,15,0,
0,8,120,0,0,7,128,0,0,60,0,0,1,224,0,0,
30,0,0,0,240,0,0,7,128,3,255,188,0,1,225,224,
0,14,15,0,0,112,120,0,3,129,192,0,28,14,0,1,
192,48,0,14,1,192,0,112,7,0,7,128,28,0,200,0,
112,24,64,0,127,0,0,1,255,195,255,128,30,0,60,0,
14,0,28,0,7,0,14,0,3,128,7,0,1,192,3,128,
1,192,3,128,0,224,1,192,0,112,0,224,0,56,0,112,
0,56,0,112,0,28,0,56,0,14,0,28,0,7,255,254,
0,7,0,14,0,3,128,7,0,1,192,3,128,0,224,1,
192,0,224,1,192,0,112,0,224,0,56,0,112,0,28,0,
56,0,28,0,56,0,14,0,28,0,7,0,14,0,3,128,
7,0,3,192,7,128,31,252,63,248,0,3,255,192,15,0,
3,128,0,224,0,56,0,14,0,7,0,1,192,0,112,0,
28,0,14,0,3,128,0,224,0,56,0,28,0,7,0,1,
192,0,112,0,56,0,14,0,3,128,0,224,0,112,0,28,
0,7,0,1,192,0,240,3,255,192,0,127,248,0,30,0,
0,224,0,7,0,0,56,0,1,192,0,28,0,0,224,0,
7,0,0,56,0,3,128,0,28,0,0,224,0,7,0,0,
112,0,3,128,0,28,0,0,224,0,14,0,0,112,0,3,
128,16,28,1,193,192,30,14,0,224,96,4,7,0,32,112,
0,135,0,3,224,0,0,1,255,192,127,128,30,0,15,0,
14,0,6,0,7,0,4,0,3,128,4,0,1,192,4,0,
1,192,8,0,0,224,8,0,0,112,8,0,0,56,8,0,
0,56,16,0,0,28,24,0,0,14,30,0,0,7,23,0,
0,7,17,192,0,3,160,224,0,1,224,56,0,0,224,28,
0,0,224,14,0,0,112,3,128,0,56,1,192,0,28,0,
112,0,28,0,56,0,14,0,28,0,7,0,7,0,3,128,
3,128,3,192,1,224,31,252,7,254,0,1,255,224,0,120,
0,0,224,0,1,192,0,3,128,0,7,0,0,28,0,0,
56,0,0,112,0,0,224,0,3,128,0,7,0,0,14,0,
0,28,0,0,112,0,0,224,0,1,192,0,3,128,0,14,
0,8,28,0,16,56,0,32,112,0,129,192,1,3,128,6,
7,0,24,14,0,112,56,3,199,255,255,128,1,254,0,0,
255,0,62,0,0,240,0,46,0,1,224,0,46,0,2,224,
0,46,0,2,224,0,46,0,4,224,0,78,0,9,192,0,
78,0,9,192,0,71,0,17,192,0,71,0,17,192,0,135,
0,35,128,0,135,0,67,128,0,135,0,67,128,0,135,0,
131,128,1,7,1,7,0,1,7,1,7,0,1,3,130,7,
0,1,3,130,7,0,2,3,132,14,0,2,3,136,14,0,
2,3,136,14,0,2,3,144,14,0,4,3,160,28,0,4,
3,160,28,0,4,1,192,28,0,12,1,192,28,0,28,1,
128,60,0,255,129,3,255,128,1,252,0,255,128,14,0,14,
0,11,128,6,0,5,192,2,0,2,224,1,0,1,56,0,
128,1,28,0,128,0,134,0,64,0,67,128,32,0,33,192,
16,0,32,112,16,0,16,56,8,0,8,28,4,0,4,7,
2,0,4,3,130,0,2,0,225,0,1,0,112,128,0,128,
56,64,0,128,14,64,0,64,7,32,0,32,3,144,0,16,
0,232,0,16,0,120,0,8,0,28,0,4,0,14,0,6,
0,7,0,7,0,1,0,31,240,0,128,0,0,3,248,0,
0,224,224,0,56,3,128,14,0,28,1,192,1,192,56,0,
14,7,0,0,224,240,0,15,30,0,0,241,192,0,15,60,
0,0,243,192,0,15,120,0,0,247,128,0,15,120,0,0,
255,0,0,30,240,0,1,239,0,0,30,240,0,3,207,0,
0,60,240,0,7,143,0,0,120,240,0,15,7,0,0,224,
112,0,28,3,128,3,128,60,0,112,1,192,28,0,7,7,
128,0,31,192,0,1,255,255,0,3,192,60,0,56,0,224,
3,128,15,0,56,0,112,3,128,7,0,112,0,240,7,0,
15,0,112,0,240,7,0,14,0,224,1,224,14,0,60,0,
224,7,128,14,1,224,1,255,240,0,28,0,0,1,192,0,
0,28,0,0,3,128,0,0,56,0,0,3,128,0,0,56,
0,0,7,0,0,0,112,0,0,7,0,0,0,112,0,0,
15,0,0,15,254,0,0,0,3,248,0,0,224,224,0,56,
3,128,14,0,28,1,192,1,192,56,0,14,7,0,0,224,
240,0,15,30,0,0,241,192,0,15,60,0,0,243,192,0,
15,120,0,0,247,128,0,15,120,0,0,255,0,0,30,240,
0,1,239,0,0,30,240,0,3,207,0,0,60,240,0,3,
143,0,0,120,240,0,15,7,3,192,224,112,66,28,3,136,
35,128,56,131,112,1,200,60,0,7,135,129,0,31,248,16,
0,3,131,0,0,56,32,0,3,142,0,0,63,192,0,3,
248,0,0,31,128,0,1,224,0,1,255,254,0,1,224,30,
0,14,0,56,0,112,1,224,3,128,7,0,28,0,56,1,
192,3,192,14,0,30,0,112,0,240,3,128,15,0,56,0,
112,1,192,15,0,14,1,224,0,127,248,0,7,0,192,0,
56,3,0,1,192,28,0,14,0,224,0,224,7,0,7,0,
56,0,56,1,192,1,192,14,0,28,0,240,0,224,7,129,
7,0,60,8,56,1,224,131,192,7,133,255,192,28,64,0,
0,124,0,0,15,193,0,48,51,0,64,15,0,128,6,1,
128,6,3,0,6,3,0,6,6,0,4,6,0,4,7,0,
0,7,0,0,7,128,0,3,240,0,1,255,0,0,255,192,
0,63,224,0,3,224,0,0,240,0,0,112,0,0,48,0,
0,48,32,0,48,32,0,48,96,0,96,96,0,96,96,0,
192,96,0,128,240,3,0,204,6,0,131,248,0,31,255,255,
241,192,56,7,48,7,0,50,0,112,2,96,7,0,36,0,
112,2,64,14,0,40,0,224,2,128,14,0,32,0,224,0,
0,28,0,0,1,192,0,0,28,0,0,1,192,0,0,56,
0,0,3,128,0,0,56,0,0,3,128,0,0,112,0,0,
7,0,0,0,112,0,0,7,0,0,0,224,0,0,14,0,
0,0,224,0,0,14,0,0,1,224,0,7,255,240,0,127,
240,63,225,224,0,224,56,0,24,7,0,2,0,224,0,64,
28,0,8,7,0,2,0,224,0,64,28,0,8,3,128,1,
0,224,0,64,28,0,8,3,128,1,0,112,0,32,28,0,
8,3,128,1,0,112,0,32,14,0,4,3,128,1,0,112,
0,32,14,0,4,1,192,1,0,56,0,32,7,0,8,0,
96,2,0,12,0,128,0,192,32,0,12,24,0,0,124,0,
0,255,192,15,241,224,0,56,28,0,3,1,192,0,32,28,
0,4,1,224,0,64,14,0,8,0,224,0,128,14,0,16,
0,224,2,0,14,0,32,0,224,4,0,14,0,128,0,224,
8,0,15,1,0,0,112,16,0,7,2,0,0,112,64,0,
7,4,0,0,112,128,0,7,24,0,0,113,0,0,7,160,
0,0,58,0,0,3,192,0,0,56,0,0,3,128,0,0,
48,0,0,3,0,0,0,255,193,255,193,255,28,0,60,0,
60,28,0,28,0,16,28,0,28,0,32,28,0,60,0,32,
28,0,124,0,64,28,0,92,0,64,28,0,156,0,128,28,
0,156,0,128,28,1,28,1,0,28,1,28,1,0,28,2,
28,2,0,28,2,28,4,0,28,4,28,4,0,28,8,28,
8,0,30,8,28,8,0,14,16,28,16,0,14,16,28,16,
0,14,32,14,32,0,14,32,14,96,0,14,64,14,64,0,
14,64,14,128,0,14,128,14,128,0,15,0,15,0,0,15,
0,15,0,0,14,0,14,0,0,14,0,14,0,0,12,0,
12,0,0,12,0,8,0,0,1,255,192,255,128,15,0,30,
0,7,128,12,0,1,192,4,0,0,240,4,0,0,56,4,
0,0,28,4,0,0,15,4,0,0,3,132,0,0,1,228,
0,0,0,116,0,0,0,62,0,0,0,14,0,0,0,7,
0,0,0,3,192,0,0,2,224,0,0,2,112,0,0,2,
28,0,0,2,14,0,0,2,7,128,0,2,1,192,0,2,
0,240,0,2,0,56,0,3,0,28,0,3,0,15,0,3,
0,3,128,7,128,3,224,31,240,7,254,0,255,224,7,248,
120,0,15,3,192,0,96,14,0,2,0,112,0,32,3,192,
2,0,14,0,32,0,120,2,0,1,192,48,0,14,1,0,
0,120,16,0,1,193,0,0,15,16,0,0,57,0,0,1,
216,0,0,15,128,0,0,56,0,0,1,192,0,0,28,0,
0,0,224,0,0,7,0,0,0,56,0,0,3,128,0,0,
28,0,0,0,224,0,0,7,0,0,0,112,0,0,63,248,
0,0,0,255,255,224,30,0,56,7,0,14,0,192,3,128,
16,0,224,2,0,60,0,128,7,0,16,1,192,2,0,112,
0,0,28,0,0,7,0,0,1,192,0,0,112,0,0,30,
0,0,7,128,0,0,224,0,0,56,0,0,14,0,64,3,
128,16,0,224,2,0,56,0,64,14,0,16,3,192,2,0,
112,0,192,28,0,48,7,0,14,1,192,7,128,127,255,240,
0,1,227,3,139,131,3,195,1,227,128,225,128,113,192,56,
224,28,224,28,112,14,56,7,28,3,142,3,143,1,199,128,
226,192,241,49,153,15,7,0,63,0,28,0,112,3,128,14,
0,56,0,224,7,0,28,0,112,1,192,14,120,58,48,240,
99,129,156,6,112,29,192,119,1,248,14,224,59,128,238,3,
56,28,224,99,131,6,28,8,192,30,0,1,240,6,24,56,
48,224,243,131,198,3,28,0,56,0,224,1,192,3,128,7,
0,14,0,28,0,152,2,48,8,48,96,63,0,0,7,224,
0,28,0,3,128,0,224,0,28,0,3,128,0,112,0,28,
0,3,128,0,112,0,14,0,243,128,113,112,24,30,6,3,
193,192,112,48,14,14,1,193,192,56,112,14,14,1,193,192,
56,56,7,7,1,196,224,56,156,7,17,129,226,24,204,129,
224,224,1,240,28,32,192,134,1,56,8,192,39,3,31,240,
224,3,128,14,0,56,0,224,3,128,22,0,152,4,48,96,
62,0,0,1,224,0,49,128,3,158,0,25,224,1,198,0,
14,0,0,112,0,3,128,0,60,0,1,192,0,14,0,7,
255,128,3,128,0,28,0,1,192,0,14,0,0,112,0,3,
128,0,28,0,1,192,0,14,0,0,112,0,3,128,0,28,
0,1,192,0,14,0,0,112,0,3,128,0,28,0,0,192,
0,14,0,0,112,0,99,0,7,152,0,61,128,1,200,0,
7,128,0,0,0,113,128,197,192,193,224,192,240,224,112,96,
56,112,28,56,14,56,14,28,7,14,3,135,1,195,129,193,
192,224,224,112,48,120,8,248,3,156,0,14,0,7,0,7,
3,3,131,193,129,225,192,225,128,63,0,0,15,192,0,56,
0,7,0,1,192,0,56,0,7,0,0,224,0,56,0,7,
0,0,224,0,28,0,7,31,0,236,48,30,7,3,128,224,
240,28,28,3,131,128,112,112,14,28,3,131,128,112,112,14,
14,3,131,128,113,112,14,46,3,133,192,113,112,6,70,0,
112,1,192,240,60,6,0,0,0,0,0,0,0,1,192,156,
71,17,200,114,28,14,3,129,192,112,28,14,3,136,226,112,
156,67,32,112,0,7,0,15,0,15,0,6,0,0,0,0,
0,0,0,0,0,0,0,0,0,112,0,156,1,12,2,12,
2,28,4,28,0,28,0,28,0,56,0,56,0,56,0,56,
0,112,0,112,0,112,0,112,0,224,0,224,0,224,0,224,
1,192,97,192,241,128,243,0,230,0,124,0,15,192,0,112,
0,28,0,14,0,3,128,0,224,0,56,0,28,0,7,0,
1,192,0,112,0,56,28,14,24,131,136,112,226,60,113,30,
28,131,7,64,1,224,0,254,0,57,192,14,56,3,142,1,
195,132,112,225,28,56,71,14,35,129,136,96,60,0,31,129,
192,224,224,112,56,28,28,14,7,3,131,129,192,224,112,112,
56,28,14,14,7,3,129,193,200,228,114,57,12,131,128,56,
31,129,240,78,32,198,24,70,64,232,28,70,128,240,28,143,
0,240,28,142,0,224,28,14,0,224,28,14,0,224,28,28,
1,192,56,28,1,192,56,28,1,192,56,28,1,192,112,56,
3,128,113,56,3,128,113,56,3,128,225,56,3,128,226,112,
7,0,100,48,3,0,56,56,31,4,230,24,70,129,196,112,
28,143,1,200,224,28,14,1,192,224,28,28,3,129,192,56,
28,3,129,192,112,56,7,19,128,113,56,14,19,128,226,112,
6,67,0,56,0,248,1,134,3,129,131,128,97,128,49,128,
25,192,14,224,7,224,7,112,3,184,1,220,1,206,0,231,
0,225,128,96,192,96,48,224,7,192,0,7,7,128,78,67,
2,52,12,17,192,97,28,3,8,224,28,7,0,224,56,7,
3,128,112,28,3,128,224,28,7,0,192,112,14,3,128,96,
28,6,0,240,96,14,134,0,115,192,3,128,0,28,0,1,
192,0,14,0,0,112,0,3,128,0,60,0,7,252,0,0,
1,194,12,76,48,120,192,243,129,198,3,156,7,56,14,224,
57,192,115,128,231,1,206,7,28,14,56,28,48,120,35,224,
57,192,3,128,7,0,28,0,56,0,112,0,224,3,192,63,
240,56,60,78,66,70,135,71,15,142,30,142,12,14,0,14,
0,28,0,28,0,28,0,28,0,56,0,56,0,56,0,56,
0,112,0,48,0,1,240,12,24,16,16,96,113,128,195,128,
7,192,7,240,15,240,7,240,0,225,0,231,1,158,3,60,
4,64,24,96,96,63,0,0,192,28,1,192,28,3,128,56,
3,128,56,255,240,112,7,0,112,14,0,224,14,0,224,28,
1,192,28,1,192,56,35,130,56,67,132,24,128,240,28,0,
196,224,57,28,7,35,128,232,112,57,14,7,3,128,224,112,
28,28,7,3,128,224,112,28,14,3,129,128,226,48,28,70,
3,136,224,241,12,38,64,248,112,28,2,39,7,71,7,71,
3,135,1,135,1,14,1,14,1,28,2,28,2,28,2,28,
4,24,4,24,8,24,8,28,16,12,32,7,192,28,0,192,
137,192,112,116,112,28,29,28,7,3,135,3,128,97,192,224,
16,224,56,4,56,14,1,28,7,0,135,1,192,33,192,112,
8,112,28,4,24,6,1,6,1,128,129,192,224,32,112,60,
16,14,19,8,0,248,124,0,7,135,129,25,8,67,195,16,
120,242,14,60,129,195,0,56,0,7,0,1,192,0,56,0,
7,0,0,224,6,56,9,231,1,60,224,71,44,8,132,198,
15,15,0,28,0,201,192,116,112,29,28,7,135,3,161,192,
224,224,56,56,14,28,7,7,1,193,192,112,112,28,24,14,
6,3,129,128,224,112,120,12,60,1,247,0,1,192,0,96,
96,56,60,12,15,6,3,131,0,65,128,15,128,0,3,129,
7,193,15,230,8,28,16,8,0,16,0,32,0,64,0,128,
1,0,2,0,4,0,8,4,16,4,44,24,119,248,67,240,
129,192,1,255,255,224,7,128,28,0,224,1,128,28,0,32,
3,128,4,0,112,0,128,28,0,16,3,128,2,0,112,0,
64,14,0,0,3,128,0,0,112,0,0,14,0,0,1,192,
0,0,112,0,0,14,0,0,1,192,0,0,56,0,0,14,
0,0,1,192,0,0,56,0,0,7,0,0,1,192,0,0,
56,0,0,7,0,0,0,224,0,0,60,0,0,127,248,0,
0,0,0,6,0,0,0,24,0,0,0,224,0,0,7,192,
0,0,39,0,0,0,156,0,0,4,112,0,0,33,224,0,
1,3,128,0,12,14,0,0,32,60,0,1,0,240,0,8,
1,192,0,96,7,0,1,0,30,0,8,0,120,0,64,0,
224,3,0,3,128,8,0,15,0,64,0,60,2,0,0,112,
24,0,1,192,64,0,7,130,0,0,30,16,0,0,56,192,
0,0,227,255,255,255,223,255,255,255,255,255,255,252,0,3,
248,0,0,224,224,0,56,3,128,14,0,28,1,192,1,192,
56,0,14,7,0,0,224,240,0,15,30,0,0,241,192,0,
15,60,0,0,243,192,0,15,120,128,8,247,143,255,143,120,
255,248,255,31,255,30,241,255,241,239,16,1,30,240,0,3,
207,0,0,60,240,0,3,143,0,0,120,240,0,15,7,0,
0,224,112,0,28,3,128,3,128,56,0,112,1,192,28,0,
7,7,128,0,31,192,0,0,0,24,0,0,6,0,0,3,
128,0,0,224,0,0,120,0,0,30,0,0,11,128,0,4,
224,0,1,56,0,0,142,0,0,35,192,0,16,240,0,4,
28,0,2,7,0,1,1,192,0,64,112,0,32,28,0,8,
7,0,4,1,192,1,0,112,0,128,28,0,64,7,0,16,
1,192,8,0,112,2,0,30,1,0,7,128,192,0,224,112,
0,120,255,129,255,192,1,255,255,252,7,255,255,240,48,0,
0,128,128,0,2,2,0,0,8,8,0,0,32,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,8,0,32,0,
32,0,128,0,255,254,0,3,255,248,0,16,0,64,0,64,
1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,4,0,0,16,16,0,0,64,
64,0,1,1,0,0,12,15,255,255,224,63,255,255,128,1,
255,255,255,128,30,0,28,0,14,0,28,0,7,0,14,0,
3,128,7,0,1,192,3,128,1,192,3,128,0,224,1,192,
0,112,0,224,0,56,0,112,0,56,0,112,0,28,0,56,
0,14,0,28,0,7,0,14,0,7,0,14,0,3,128,7,
0,1,192,3,128,0,224,1,192,0,224,1,192,0,112,0,
224,0,56,0,112,0,28,0,56,0,28,0,56,0,14,0,
28,0,7,0,14,0,3,128,7,0,3,192,7,128,31,252,
63,248,0,1,255,255,254,3,192,0,124,3,128,0,56,7,
0,0,32,7,0,0,64,14,0,0,128,30,0,1,0,28,
0,2,0,60,0,4,0,56,0,0,0,112,0,0,0,240,
0,0,0,224,0,0,1,224,0,0,1,128,0,0,2,0,
0,0,8,0,0,0,32,0,0,0,128,0,128,2,0,1,
0,8,0,2,0,32,0,8,0,128,0,48,6,0,0,96,
16,0,1,128,64,0,31,1,255,255,252,7,255,255,248,0,
15,128,15,131,248,7,248,255,129,255,176,120,120,124,7,28,
7,0,99,0,96,12,192,8,1,184,0,0,54,0,0,6,
128,0,0,240,0,0,30,0,0,3,128,0,0,112,0,0,
14,0,0,3,128,0,0,112,0,0,14,0,0,1,192,0,
0,112,0,0,14,0,0,1,192,0,0,56,0,0,14,0,
0,1,192,0,0,56,0,0,7,0,0,1,224,0,7,255,
192,0,0,63,254,0,0,240,0,0,56,0,0,14,0,0,
3,128,0,7,248,0,14,115,192,14,28,56,15,7,7,7,
129,192,227,192,224,56,224,56,15,120,14,3,222,3,128,255,
1,192,123,192,112,30,240,28,7,28,7,3,199,3,129,224,
224,224,240,28,56,112,3,206,112,0,31,224,0,1,192,0,
0,112,0,0,28,0,0,15,0,0,127,252,0,0,255,248,
0,1,224,0,0,56,0,0,7,0,0,0,224,1,192,28,
3,220,7,1,225,192,224,56,56,28,14,7,3,129,193,192,
224,112,56,28,14,7,3,129,192,224,112,48,28,28,14,3,
131,129,192,112,112,112,14,14,12,1,195,131,128,24,112,224,
1,142,48,0,25,220,0,0,252,0,0,14,0,0,1,192,
0,0,56,0,0,15,0,0,63,254,0,0,0,1,252,0,
0,120,60,0,14,0,112,0,224,1,192,30,0,7,1,224,
0,56,14,0,1,224,240,0,15,15,0,0,120,120,0,3,
195,192,0,30,60,0,1,225,224,0,15,15,0,0,240,56,
0,7,129,192,0,120,14,0,3,128,56,0,56,1,192,1,
128,14,0,24,0,48,1,128,1,128,8,1,4,0,193,8,
32,12,16,129,128,64,132,12,4,8,63,192,63,193,254,3,
254,15,240,31,224,0,0,240,0,12,48,0,224,96,71,1,
193,56,7,8,192,28,39,0,121,28,1,228,224,3,163,128,
14,142,0,60,56,0,224,224,3,131,128,14,6,0,120,24,
6,226,48,97,144,62,3,128,0,1,240,0,12,48,0,32,
48,0,128,96,2,0,224,8,1,192,32,3,128,64,14,1,
0,28,2,0,48,4,0,224,16,127,128,33,12,0,65,252,
0,128,12,2,0,24,4,0,48,8,0,112,16,0,224,64,
3,128,128,7,1,0,14,2,0,56,12,0,112,24,0,192,
40,3,0,80,12,1,24,112,2,15,128,4,0,0,8,0,
0,32,0,0,64,0,0,128,0,1,0,0,4,0,0,8,
0,0,0,7,128,17,252,1,63,224,39,255,2,96,48,76,
1,8,128,16,128,0,144,0,9,0,0,160,0,10,0,0,
160,0,12,0,0,192,0,8,0,0,128,0,8,0,1,128,
0,24,0,1,128,0,48,0,3,0,0,48,0,3,0,0,
96,0,6,0,0,64,0,0,124,1,132,4,8,24,0,96,
0,192,1,128,3,128,7,128,7,128,7,128,15,128,55,129,
199,7,7,28,14,48,12,224,25,192,55,0,78,0,156,3,
56,4,112,8,224,48,192,65,129,1,132,1,240,0,1,248,
56,3,0,48,3,128,24,1,192,15,248,224,7,0,56,1,
192,14,0,48,1,128,6,0,24,96,124,0,0,16,0,8,
0,4,0,3,240,3,8,2,120,2,0,2,0,2,0,3,
0,3,0,1,0,1,128,1,128,0,192,0,64,0,96,0,
48,0,24,0,28,0,12,0,6,0,3,128,1,192,0,224,
0,56,0,30,0,7,224,1,254,0,63,128,7,224,0,120,
0,28,0,14,0,6,0,98,0,14,0,56,31,19,152,100,
104,29,28,7,143,1,227,128,112,224,28,56,7,28,3,135,
0,225,192,56,112,14,56,7,14,1,195,128,112,224,28,112,
14,12,3,128,0,224,0,56,0,28,0,7,0,1,192,0,
112,0,56,0,14,0,3,0,0,120,0,204,1,134,3,6,
6,6,14,6,12,6,28,7,24,7,56,6,56,6,48,14,
112,14,112,14,127,254,224,28,224,28,224,28,224,24,224,56,
192,56,192,112,192,96,192,96,192,192,193,128,97,128,98,0,
60,0,6,3,128,224,56,28,7,1,192,112,56,14,3,129,
192,112,92,30,11,130,99,15,0,6,1,193,192,252,56,39,
135,8,97,194,0,56,128,7,96,0,240,0,63,240,7,7,
0,224,112,28,14,7,1,194,224,56,92,7,11,128,226,224,
12,76,0,240,7,128,0,56,0,3,128,0,112,0,15,0,
0,224,0,28,0,3,128,0,56,0,7,0,0,224,0,30,
0,1,192,0,56,0,7,128,0,112,0,30,0,7,224,1,
156,0,99,128,24,120,6,7,1,192,224,112,28,28,3,199,
0,57,192,7,112,0,246,0,14,1,128,48,28,3,128,224,
28,7,0,224,112,14,3,128,112,28,3,128,224,28,14,1,
192,112,14,3,128,112,28,3,129,192,56,142,1,196,112,14,
35,192,241,62,25,145,223,7,14,0,0,112,0,7,0,0,
56,0,1,192,0,14,0,0,224,0,7,0,0,48,0,0,
126,0,97,192,28,56,3,135,0,113,192,28,56,3,135,0,
224,224,24,56,7,7,1,192,224,48,28,12,7,3,0,224,
192,28,112,3,152,0,252,0,30,0,0,0,16,0,8,0,
4,0,3,240,15,8,14,120,14,0,14,0,14,0,7,0,
3,128,1,192,0,224,0,112,0,27,240,7,8,6,252,4,
0,6,0,6,0,2,0,3,0,1,128,1,192,0,224,0,
112,0,28,0,15,128,3,248,0,255,0,31,224,1,248,0,
62,0,6,0,3,0,33,0,15,0,15,255,248,255,255,207,
255,252,193,8,4,8,64,64,70,0,4,48,0,33,128,3,
12,0,24,96,0,131,0,12,56,0,97,192,7,14,0,48,
112,3,129,192,28,14,0,192,96,0,0,30,0,14,48,3,
3,0,192,96,56,12,6,1,193,192,56,56,7,14,1,193,
192,56,56,7,7,0,193,192,56,56,6,7,1,128,240,96,
58,24,7,60,0,224,0,28,0,7,0,0,224,0,28,0,
3,128,0,224,0,28,0,3,0,0,0,1,255,248,31,255,
195,255,252,60,60,1,128,224,28,3,1,192,24,14,0,192,
224,14,7,0,112,56,3,129,192,24,14,1,192,112,12,3,
128,192,12,12,0,48,192,0,248,0,0,15,255,227,255,252,
255,255,48,96,4,12,1,1,0,0,32,0,12,0,1,128,
0,48,0,12,0,1,128,0,48,0,14,0,1,192,0,56,
0,14,0,0,192,0,28,0,68,224,29,28,3,163,128,56,
112,3,14,0,67,128,8,112,1,28,0,67,128,8,112,1,
14,0,65,192,8,48,2,7,0,128,224,32,14,8,0,126,
0,0,1,0,0,32,0,2,0,0,32,0,2,0,0,64,
0,4,0,0,64,0,4,0,0,128,0,8,0,7,224,1,
201,128,113,14,12,16,97,129,7,56,16,55,2,3,112,32,
62,2,7,224,32,126,4,7,224,64,238,4,14,96,65,198,
8,56,48,134,1,201,192,7,224,0,16,0,1,0,0,16,
0,1,0,0,32,0,2,0,0,32,0,2,0,0,30,0,
4,142,0,34,60,0,128,112,4,1,192,32,7,129,0,14,
8,0,56,64,0,241,0,1,200,0,7,64,0,30,0,0,
56,0,0,224,0,7,128,0,31,0,0,156,0,4,112,0,
33,192,1,7,128,8,14,0,32,56,1,0,240,8,1,194,
64,3,146,0,7,128,0,0,64,0,0,128,0,2,0,0,
4,0,0,8,0,0,16,0,0,64,0,0,128,0,1,0,
0,2,0,0,8,0,224,16,34,112,32,232,224,65,209,193,
1,195,130,1,135,4,2,28,8,4,56,32,8,224,64,33,
192,128,67,129,0,135,4,2,14,8,8,28,16,32,56,32,
128,48,130,0,57,24,0,31,192,0,4,0,0,16,0,0,
32,0,0,64,0,0,128,0,2,0,0,4,0,0,8,0,
0,8,0,4,32,0,28,64,0,57,0,0,50,0,0,104,
0,0,80,2,0,160,12,1,128,24,5,0,48,10,0,192,
52,1,128,76,3,1,152,30,6,63,255,252,63,223,240,127,
31,192,124,30,0,0,252,3,254,14,7,24,0,16,0,32,
0,32,0,16,128,23,64,31,192,32,0,64,0,64,0,128,
0,128,0,128,4,64,8,112,48,63,224,15,128,0,7,128,
0,140,0,16,96,1,6,0,32,96,2,6,0,32,96,2,
7,0,32,112,2,6,0,16,99,128,142,78,4,228,96,62,
70,0,248,224,28,142,1,192,224,28,14,1,129,192,56,28,
3,129,192,112,28,6,3,128,96,56,12,1,129,128,24,24,
0,194,0,7,192,0,15,255,255,254,63,255,255,252,255,255,
255,243,8,0,0,68,16,0,0,144,64,0,1,1,0,32,
2,2,0,64,4,8,1,0,16,16,2,0,32,32,12,0,
192,64,24,1,0,128,112,6,1,1,176,24,3,14,96,224,
7,248,255,128,15,224,254,0,7,128,240,0,0,120,0,227,
0,192,192,192,96,224,48,96,28,112,14,56,7,56,7,28,
3,142,1,199,0,199,128,227,192,97,224,96,176,96,76,96,
67,192,32,0,16,0,8,0,6,0,3,255,128,255,224,63,
240,0,8,0,0,254,6,12,48,0,192,3,0,4,0,24,
0,48,0,224,1,192,3,128,3,128,7,192,7,224,7,240,
3,240,1,224,0,224,1,192,3,0,4,0,112,0,8,0,
240,32,7,240,64,31,241,0,112,114,0,192,104,2,0,80,
4,0,160,16,1,128,32,5,0,128,10,1,0,54,2,0,
68,8,3,14,16,12,15,32,240,15,255,192,15,255,0,3,
240,0,12,0,0,24,0,0,48,0,0,224,0,1,192,0,
3,0,0,14,0,0,28,0,0,48,0,0,0,62,0,12,
48,1,0,128,32,12,3,0,96,120,6,7,128,112,48,3,
0,0,48,0,3,0,0,48,15,199,3,130,112,96,23,12,
1,113,128,15,56,0,227,0,14,112,0,231,0,28,224,1,
206,0,28,224,1,142,0,56,224,3,14,0,96,224,6,6,
0,192,48,24,1,134,0,15,128,0,0,14,0,17,0,49,
0,97,0,97,0,193,0,193,1,194,1,130,3,132,3,132,
7,8,7,8,7,16,7,32,14,64,14,64,14,128,15,0,
14,0,12,0,12,0,28,0,44,0,76,0,140,2,6,12,
2,16,1,224,28,19,145,200,232,116,56,56,28,28,14,7,
7,3,137,197,194,226,50,14,0,0,112,2,112,16,192,131,
2,28,16,112,1,192,7,0,56,0,224,3,128,14,0,112,
1,192,7,0,28,0,224,3,128,14,0,56,1,193,135,15,
24,60,192,230,1,240,0,3,0,0,96,0,2,1,248,32,
48,99,2,3,144,32,13,130,0,108,32,3,98,0,31,32,
1,249,0,15,208,0,127,0,3,56,0,57,225,129,135,136,
28,60,65,194,241,24,19,199,128,142,0,4,112,0,67,0,
2,24,0,16,192,0,132,0,4,96,0,34,0,0,224,0,
0,0,2,0,1,128,0,223,255,255,255,248,0,56,0,48,
0,16,7,240,24,12,32,2,64,1,128,1,7,224,28,56,
48,12,112,14,96,6,96,6,224,7,224,7,224,7,224,7,
224,7,224,7,224,7,224,7,96,6,96,6,112,14,48,12,
28,56,7,224,1,0,120,61,192,14,0,112,3,128,28,0,
224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,
56,63,254,15,224,48,56,64,28,64,14,192,15,224,7,64,
7,0,15,0,14,0,28,0,56,0,112,0,192,1,1,6,
1,8,3,31,254,127,254,255,254,7,224,24,56,32,28,32,
28,120,30,120,14,120,30,48,30,0,28,0,28,0,56,0,
96,7,224,0,48,0,28,0,28,0,14,0,15,0,15,96,
15,240,15,240,15,224,15,128,14,64,28,32,28,24,48,7,
224,0,12,0,3,0,1,192,0,240,0,60,0,23,0,9,
192,6,112,1,28,0,135,0,97,192,16,112,8,28,4,7,
3,1,192,128,112,64,28,48,7,15,255,252,0,112,0,28,
0,7,0,1,192,0,112,0,28,0,7,0,31,252,48,12,
63,248,63,240,63,192,32,0,32,0,32,0,32,0,32,0,
32,0,35,224,44,56,48,12,32,12,0,14,0,6,0,7,
0,7,96,7,240,7,240,7,224,7,128,6,64,14,64,12,
32,24,24,48,7,192,0,240,3,12,6,4,12,14,24,30,
48,30,48,12,112,0,112,0,96,0,227,224,228,48,232,24,
240,12,240,14,224,6,224,7,224,7,224,7,224,7,224,7,
96,7,96,7,112,6,48,14,48,12,24,24,12,48,3,224,
64,0,63,255,223,255,207,255,228,0,36,0,34,0,17,0,
16,0,16,0,8,0,8,0,8,0,4,0,6,0,3,0,
3,0,1,128,1,192,0,224,0,112,0,56,0,60,0,30,
0,15,0,7,128,3,192,1,224,0,96,0,7,224,12,48,
16,8,48,12,96,6,96,6,96,6,96,6,112,6,56,12,
60,8,31,48,15,224,3,224,7,240,24,120,48,60,96,14,
96,7,192,7,192,3,192,3,192,3,192,3,96,2,96,6,
48,12,28,24,7,224,7,192,12,48,24,24,48,12,112,12,
96,14,224,6,224,6,224,7,224,7,224,7,224,7,96,7,
96,15,48,15,24,23,12,39,7,199,0,6,0,6,0,14,
48,12,120,12,120,24,112,24,32,48,48,192,15,128,1,0,
0,0,0,16,0,0,0,1,0,0,0,0,8,0,0,0,
0,128,0,0,0,24,0,0,0,1,255,255,255,255,255,255,
255,255,255,255,255,255,255,251,255,255,255,255,204,0,0,0,
0,16,0,0,0,0,64,0,0,0,2,0,0,0,0,8,
0,0,0,0,32,0,0,0,0,0,0,4,0,0,0,0,
16,0,0,0,0,64,0,0,0,2,0,0,0,0,8,0,
0,0,0,51,255,255,255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,192,0,0,0,24,0,0,0,1,0,
0,0,0,16,0,0,0,0,128,0,0,0,8,0,0,0,
0,128,30,125,134,12,24,48,96,96,124,120,241,240,48,48,
96,193,131,13,243,192,64,0,60,0,15,128,3,56,0,199,
128,48,120,12,7,131,0,112,192,7,48,0,252,0,63,0,
28,192,28,48,30,12,30,3,30,0,206,0,62,0,15,0,
1,0,0,0,1,128,3,192,3,224,7,48,15,24,30,12,
60,6,56,3,112,1,240,0,248,0,110,0,49,192,24,120,
12,15,6,1,227,0,57,128,7,192,1,224,0,48,111,246,
111,247,17,17,34,72,0,0,3,128,0,7,128,0,15,0,
0,30,0,0,60,0,0,120,0,0,240,0,1,224,0,3,
192,0,7,128,0,15,0,0,30,0,0,15,0,0,1,224,
0,0,60,0,0,7,128,0,0,240,0,0,30,0,0,3,
192,0,0,120,0,0,15,0,0,1,224,0,0,60,0,0,
7,0,1,0,3,0,6,0,6,0,6,0,12,0,12,0,
12,0,24,0,24,0,24,0,48,0,48,0,48,0,96,0,
96,0,192,0,192,0,192,1,128,1,128,1,128,3,0,3,
0,3,0,6,0,6,0,12,0,12,0,12,0,24,0,24,
0,24,0,48,0,48,0,48,0,96,0,96,0,96,0,192,
0,192,0,224,0,0,60,0,0,7,128,0,0,240,0,0,
30,0,0,3,192,0,0,120,0,0,15,0,0,1,224,0,
0,60,0,0,7,128,0,0,240,0,0,120,0,0,240,0,
1,224,0,3,192,0,7,128,0,15,0,0,30,0,0,60,
0,0,120,0,0,240,0,1,224,0,1,192,0,0,0,32,
0,1,0,0,8,0,0,64,0,2,0,0,16,0,1,192,
24,14,3,124,113,240,127,252,0,255,128,1,240,0,15,128,
0,238,0,6,48,0,96,192,2,2,0,48,24,3,0,96,
16,1,0,128,16,2,0,64,8,1,0,32,4,0,128,16,
2,0,64,9,241,195,48,116,7,128,240,30,3,192,120,15,
3,160,116,12,131,144,226,24,70,9,129,96,56,4,0,128,
64,32,16,8,4,2,1,0,128,64,32,113,251,255,247,227,
129,128,192,96,48,24,12,6,3,1,128,192,96,48,24,28,
126,255,253,248,224,64,32,16,8,4,2,1,0,128,16,130,
16,66,8,65,8,33,4,32,132,28,159,159,255,249,249,56,
33,4,32,132,16,130,16,66,8,65,8,33,4,32,132,16,
130,16,114,126,127,255,231,228,224,132,16,130,16,66,8,64,
8,0,128,0,0,0,10,0,0,0,0,136,0,0,0,8,
32,0,0,0,128,128,0,0,8,3,0,0,1,128,14,0,
0,56,0,31,0,31,0,0,63,255,224,0,0,31,240,0,
0,0,0,32,0,0,0,127,252,0,0,31,255,252,0,3,
192,0,120,0,48,0,0,96,2,0,0,0,128,96,0,0,
3,6,0,0,0,12,32,0,0,0,34,0,0,0,0,160,
0,0,0,2,
};
METRIC cmmi5_metric[128] = {
{1226, -2, 13, 15, 14, 272842},
{1253, -3, 13, 18, 14, 377748},
{1285, -2, 13, 17, 14, 343249},
{1315, -2, 13, 15, 14, 316762},
{1342, -2, 13, 18, 14, 333487},
{1374, -2, 13, 21, 14, 363866},
{1411, -2, 13, 18, 14, 349075},
{1443, -1, 13, 17, 14, 275346},
{1473, -2, 13, 16, 14, 309480},
{1501, -1, 13, 17, 14, 292777},
{1531, -3, 13, 17, 14, 347539},
{1561, -2, 8, 14, 9, 299277},
{1577, -2, 13, 13, 18, 259075},
{1607, -2, 8, 13, 13, 244746},
{1629, -2, 13, 10, 14, 214359},
{1647, -2, 8, 8, 9, 201047},
{1656, -2, 13, 9, 18, 218456},
{1677, -3, 8, 11, 13, 243716},
{1695, -1, 13, 10, 14, 221868},
{1713, -2, 8, 7, 9, 180150},
{1721, -2, 8, 12, 9, 271174},
{1735, -3, 13, 11, 14, 275346},
{1755, -1, 8, 13, 13, 278456},
{1777, -2, 8, 12, 9, 233323},
{1791, -2, 13, 8, 18, 218456},
{1809, -3, 8, 13, 9, 272350},
{1824, -1, 8, 11, 13, 237912},
{1842, -2, 8, 13, 9, 267115},
{1857, -3, 8, 11, 9, 223349},
{1870, -3, 8, 11, 9, 259417},
{1883, -2, 13, 13, 18, 275800},
{1913, -2, 8, 13, 13, 283310},
{1935, -3, 13, 14, 18, 304929},
{1967, -2, 8, 14, 9, 290441},
{1983, -2, 8, 10, 9, 221072},
{1995, -3, 13, 11, 14, 280277},
{2015, -3, 8, 19, 9, 375131},
{2037, -2, 8, 10, 13, 237912},
{2054, -1, 8, 10, 11, 189442},
{2068, -2, 8, 13, 13, 300832},
{2334, -2, 8, 23, 5, 446016},
{2349, -2, 5, 23, 5, 446016},
{2364, -2, 8, 23, 5, 446016},
{2379, -2, 5, 23, 5, 446016},
{2394, -2, 10, 5, 7, 150188},
{2399, -2, 10, 5, 7, 150188},
{2404, -1, 10, 12, 12, 241212},
{2422, -2, 10, 11, 12, 241212},
{2179, -2, 8, 10, 9, 241212},
{2191, -3, 8, 8, 9, 241212},
{2200, -2, 8, 10, 9, 241212},
{2212, -2, 8, 10, 13, 241212},
{2229, -2, 8, 10, 13, 241212},
{2246, -2, 8, 10, 13, 241212},
{2263, -2, 12, 10, 13, 241212},
{2280, -2, 9, 11, 14, 241212},
{2300, -2, 12, 10, 13, 241212},
{2317, -2, 8, 10, 13, 241212},
{2439, -3, 2, 3, 3, 150188},
{2441, -3, 2, 3, 7, 150188},
{2444, -3, 12, 15, 16, 354992},
{2474, -2, 15, 10, 21, 241212},
{2501, -3, 12, 15, 16, 354992},
{2531, -1, 10, 13, 11, 241212},
{2090, -2, 13, 13, 14, 245420},
{0, -2, 13, 17, 14, 339518},
{30, -2, 13, 18, 14, 339802},
{62, -2, 13, 17, 14, 326365},
{92, -2, 13, 19, 14, 363684},
{126, -2, 13, 17, 14, 329960},
{156, -2, 13, 16, 14, 284220},
{184, -2, 13, 17, 14, 348665},
{214, -2, 13, 21, 14, 363866},
{251, -2, 13, 10, 14, 205371},
{269, -4, 13, 12, 14, 253272},
{290, -2, 13, 20, 14, 373197},
{325, -2, 13, 14, 14, 313121},
{350, -2, 13, 25, 14, 420756},
{394, -2, 13, 21, 14, 352488},
{431, -2, 13, 17, 14, 343249},
{461, -2, 13, 18, 14, 286837},
{493, -2, 13, 17, 18, 354627},
{532, -2, 13, 17, 14, 337498},
{562, -2, 13, 14, 14, 280580},
{587, -1, 13, 17, 14, 272729},
{617, -2, 13, 18, 14, 309297},
{649, -3, 13, 18, 14, 271250},
{681, -3, 13, 25, 14, 419164},
{725, -2, 13, 19, 14, 364663},
{759, -2, 13, 18, 14, 270112},
{791, -2, 13, 16, 14, 309025},
{2549, -2, 15, 7, 16, 195700},
{2563, -3, 14, 5, 19, 195700},
{2575, -2, 13, 7, 18, 195700},
{2591, -2, 7, 23, 5, 446016},
{2606, -2, 8, 23, 6, 446016},
{2113, -1, 13, 9, 14, 195699},
{819, -2, 8, 11, 9, 253765},
{832, -2, 13, 9, 14, 207532},
{848, -2, 8, 10, 9, 212804},
{860, -2, 13, 11, 14, 242387},
{880, -2, 8, 10, 9, 221845},
{892, -1, 13, 12, 18, 224144},
{919, -3, 8, 10, 13, 229568},
{936, -2, 13, 12, 14, 271174},
{957, -3, 14, 6, 15, 174841},
{969, -1, 14, 10, 19, 192741},
{993, -2, 13, 11, 14, 248418},
{1013, -2, 13, 4, 14, 157394},
{1020, -3, 8, 20, 9, 402401},
{1043, -3, 8, 13, 9, 288621},
{1058, -2, 8, 10, 9, 230288},
{1070, -2, 8, 11, 13, 242388},
{1088, -2, 8, 10, 13, 215914},
{1105, -3, 8, 10, 9, 219973},
{1117, -3, 8, 8, 9, 219593},
{1126, -2, 12, 7, 13, 184702},
{1138, -3, 8, 12, 9, 277243},
{1152, -3, 8, 10, 9, 236661},
{1164, -3, 8, 16, 9, 330624},
{1182, -1, 8, 13, 9, 259416},
{1197, -3, 8, 11, 13, 239392},
{1215, -2, 8, 9, 9, 225662},
{2129, -3, 8, 6, 9, 174841},
{2136, -1, 8, 8, 13, 181363},
{2149, -3, 8, 13, 13, 291389},
{2171, -6, 13, 9, 4, 241212},
{2176, -7, 12, 12, 2, 150188},
};
char cmmi5_bitmap[2624] = {
0,24,0,28,0,14,0,11,0,9,128,4,96,4,48,4,
24,6,12,3,255,2,1,130,0,195,0,103,192,252,31,254,
1,128,96,96,12,24,3,12,1,131,0,192,255,224,48,28,
24,1,134,0,97,128,24,96,12,48,14,63,254,0,1,248,
131,3,198,0,102,0,54,0,22,0,3,0,1,128,0,192,
0,96,1,48,0,140,0,131,1,128,127,0,31,254,0,192,
48,24,3,3,0,32,192,6,24,0,195,0,24,96,3,24,
0,195,0,24,96,6,12,1,131,0,193,255,224,0,31,255,
131,0,65,128,32,192,16,193,8,96,128,63,192,24,96,24,
33,12,0,134,0,131,0,67,0,231,255,224,31,255,6,1,
6,1,6,1,12,17,12,16,15,240,12,48,24,32,24,0,
24,0,24,0,48,0,254,0,1,248,131,3,198,0,102,0,
54,0,22,0,3,0,1,128,127,192,6,96,3,48,1,140,
0,195,0,192,127,160,31,195,248,48,6,1,128,48,12,1,
128,192,24,6,0,192,63,254,1,128,48,24,3,0,192,24,
6,0,192,48,6,3,0,96,126,15,192,31,193,128,96,24,
12,3,0,192,48,24,6,1,128,96,48,63,128,7,240,12,
0,192,12,1,128,24,1,128,24,3,0,48,3,12,48,134,
7,128,31,193,240,96,24,6,3,0,96,64,12,16,0,198,
0,12,224,0,243,0,28,24,1,129,128,24,12,1,128,96,
48,6,15,192,252,31,192,24,0,96,1,128,12,0,48,0,
192,3,0,24,0,96,17,128,198,2,48,59,255,192,31,0,
15,131,128,14,1,96,11,0,176,9,128,152,5,128,76,4,
192,35,4,96,17,132,48,16,194,48,8,98,24,4,26,12,
2,14,6,3,7,6,7,195,15,192,30,1,248,56,2,1,
96,16,11,0,128,140,8,4,48,64,33,130,1,6,16,16,
25,0,128,200,4,3,64,32,14,3,0,96,124,1,0,1,
248,7,7,6,0,198,0,38,0,30,0,15,0,7,128,3,
192,3,96,1,176,1,140,1,131,3,0,254,0,31,254,1,
128,224,96,24,24,3,12,1,131,0,96,192,96,63,224,24,
0,6,0,1,128,0,96,0,48,0,63,0,0,1,248,7,
7,6,0,198,0,38,0,30,0,15,0,7,128,3,192,3,
96,1,176,1,140,225,131,139,0,254,16,3,8,1,136,0,
124,0,24,0,31,252,3,1,129,128,96,192,48,192,48,96,
48,63,224,24,24,24,12,12,6,6,3,3,1,131,0,207,
224,56,3,244,48,49,0,68,1,16,0,112,0,252,0,60,
0,24,0,36,0,144,4,192,34,254,0,127,255,176,96,80,
48,40,24,24,24,16,12,0,6,0,3,0,3,0,1,128,
0,192,0,96,0,96,1,254,0,254,15,204,0,131,0,32,
192,8,96,4,24,1,6,0,65,128,16,192,8,48,2,12,
0,129,0,64,96,96,7,224,0,252,7,204,0,131,0,96,
224,16,24,8,6,4,1,131,0,112,128,12,64,3,32,0,
200,0,52,0,6,0,1,0,0,252,63,15,152,6,3,12,
7,1,6,5,129,3,2,192,129,130,96,128,194,48,128,49,
12,64,25,6,64,13,3,64,6,129,160,3,128,224,1,192,
96,0,192,48,0,15,195,224,96,48,14,8,0,226,0,12,
128,1,224,0,24,0,7,128,1,184,0,99,0,16,112,4,
6,1,0,97,248,63,0,252,7,206,0,129,128,64,112,32,
14,16,1,136,0,116,0,14,0,3,0,0,192,0,48,0,
12,0,6,0,7,224,0,15,255,14,6,8,12,8,24,16,
48,0,96,0,192,1,128,3,4,14,4,24,4,48,12,96,
24,255,248,29,134,113,134,97,140,49,134,48,202,58,59,128,
240,24,12,6,6,3,113,198,195,193,224,240,120,100,97,224,
15,140,54,9,0,192,48,12,5,134,62,0,3,192,24,3,
0,96,24,59,12,227,12,195,24,99,12,97,148,116,119,0,
15,12,38,11,252,192,48,12,5,134,62,0,0,224,27,1,
176,24,1,128,254,3,0,48,3,0,48,6,0,96,6,0,
96,6,4,64,204,7,128,14,204,118,13,6,193,176,108,25,
28,59,0,192,50,24,252,0,120,1,128,24,1,128,48,3,
112,56,131,12,97,134,24,97,134,49,195,44,28,8,32,0,
0,7,38,176,195,24,101,167,0,0,192,32,0,0,0,0,
0,240,70,17,128,192,48,12,3,1,128,96,24,6,35,15,
0,120,3,0,96,12,3,0,97,140,217,162,120,15,193,140,
49,156,53,131,0,243,51,102,102,204,205,222,115,199,9,198,
136,152,48,195,134,24,48,97,131,6,24,48,99,22,12,50,
96,193,192,115,132,226,38,24,97,131,12,24,96,198,44,50,
96,224,15,12,38,13,3,192,240,60,25,12,60,0,57,201,
205,49,140,49,134,48,198,49,196,55,6,0,192,48,15,128,
14,76,118,13,6,193,176,108,25,28,59,0,192,48,24,31,
128,119,166,57,136,192,48,12,3,1,128,96,0,62,67,66,
112,60,6,194,132,120,12,24,48,207,227,6,24,48,96,201,
163,128,112,201,140,152,193,152,49,131,24,49,147,58,29,192,
112,166,57,140,98,48,140,35,16,204,28,0,112,98,152,99,
152,99,24,194,48,194,48,194,48,196,49,196,30,120,28,241,
24,200,196,12,0,96,3,1,24,89,196,115,192,112,115,14,
97,140,99,12,97,140,49,140,31,128,49,12,99,7,192,28,
159,136,128,128,128,132,134,254,142,0,31,254,12,4,24,8,
48,16,192,33,128,3,0,6,0,24,0,48,0,96,0,192,
3,0,31,192,0,0,24,0,14,0,5,192,3,48,1,140,
0,67,128,32,96,16,24,8,7,4,0,194,0,57,0,6,
255,255,191,255,240,1,248,7,7,6,0,198,0,38,0,30,
64,79,63,231,159,243,208,19,96,1,176,1,140,1,131,3,
0,254,0,0,48,0,224,1,192,5,128,11,0,38,0,142,
1,12,4,24,16,48,32,96,128,195,1,159,15,192,31,255,
199,255,241,0,4,64,1,0,0,1,1,0,127,192,31,240,
8,8,0,0,4,0,17,0,4,255,254,63,255,128,31,255,
248,48,6,1,128,48,12,1,128,192,24,6,0,192,48,6,
1,128,48,24,3,0,192,24,6,0,192,48,6,3,0,96,
126,15,192,31,255,195,0,48,96,4,24,1,3,0,64,224,
0,24,0,2,0,3,0,1,0,32,128,8,64,4,96,7,
63,255,128,60,15,63,29,224,200,32,40,0,28,0,12,0,
6,0,3,0,3,0,1,128,0,192,0,96,0,96,1,254,
0,7,248,0,192,0,192,7,248,57,142,97,131,193,131,193,
131,195,6,99,24,31,224,3,0,6,0,63,192,7,248,0,
96,0,48,24,24,118,24,99,12,97,134,48,195,48,99,24,
25,184,3,240,0,96,0,96,1,254,0,1,252,7,3,134,
0,102,0,54,0,27,0,13,128,12,96,6,48,6,8,4,
34,4,81,4,79,135,231,195,224,15,0,194,22,12,80,58,
192,115,1,204,6,16,120,62,56,1,224,16,129,6,16,49,
3,11,240,95,130,6,32,49,1,136,12,64,197,12,39,129,
0,8,0,128,4,0,0,60,11,240,96,196,2,64,10,0,
96,3,0,24,0,128,4,0,32,2,0,16,0,7,130,17,
128,96,30,3,195,57,130,64,176,44,11,4,98,15,0,15,
48,96,254,192,192,64,97,30,2,1,0,225,177,129,129,128,
192,192,96,48,24,6,3,240,124,6,1,7,0,115,147,138,
97,152,99,12,97,140,51,12,97,128,48,6,1,128,48,7,
2,33,140,195,48,216,55,253,255,193,176,108,17,12,102,14,
0,48,96,195,6,12,48,226,120,48,227,22,54,7,128,127,
6,24,97,156,26,192,192,120,3,128,48,6,0,96,12,0,
192,24,15,131,48,198,48,108,13,0,192,24,96,195,6,24,
97,131,12,24,96,195,46,58,126,227,0,24,1,128,12,0,
0,240,51,3,48,102,6,96,198,24,103,13,192,224,0,4,
4,15,27,48,96,96,62,62,64,192,192,192,248,127,15,1,
14,63,251,255,162,64,18,1,144,8,192,198,6,48,32,128,
3,129,140,97,136,51,6,96,204,51,140,111,12,1,128,96,
12,0,15,249,255,152,96,195,12,24,96,195,12,8,192,60,
0,63,239,250,32,4,1,128,48,6,1,128,48,0,112,83,
14,97,140,35,4,96,140,33,136,30,0,0,128,4,0,32,
1,0,16,3,224,100,204,35,98,30,16,240,135,132,102,78,
15,192,16,0,128,8,0,64,0,112,13,192,134,8,24,128,
200,3,128,24,0,224,11,0,156,8,96,131,168,7,0,0,
64,1,0,4,0,32,0,129,194,41,136,230,67,25,8,196,
35,16,140,132,26,96,62,0,32,1,0,4,0,16,0,32,
9,0,52,0,224,66,131,10,12,108,115,31,248,57,192,31,
143,244,9,240,124,32,8,3,252,62,0,1,192,200,17,130,
48,70,228,230,124,195,48,198,24,194,24,195,48,60,0,63,
255,239,255,250,64,2,8,32,66,4,8,65,131,8,120,193,
251,240,28,60,0,7,6,51,12,131,96,216,54,26,140,158,
32,8,3,252,127,0,15,204,6,3,0,192,28,7,192,124,
7,0,192,224,32,114,7,208,67,4,24,32,194,13,144,199,
252,31,128,96,3,0,48,1,128,0,7,128,67,6,12,32,
48,1,135,204,225,108,7,64,54,1,176,9,128,134,24,31,
128,3,3,65,33,144,144,200,104,56,56,24,12,26,1,8,
120,114,107,12,49,134,90,112,14,19,19,6,6,6,6,12,
12,12,12,152,240,16,1,15,25,133,144,61,1,240,15,136,
206,76,187,196,192,38,1,96,6,0,0,6,127,255,224,192,
63,204,3,63,24,108,15,3,192,240,60,13,134,63,0,24,
248,24,24,24,24,24,24,255,127,48,124,12,3,1,129,193,
133,254,255,128,62,16,102,24,6,3,7,128,24,3,0,240,
60,13,6,62,0,3,1,192,112,44,19,8,198,51,12,255,
192,192,48,12,31,192,32,143,195,192,128,32,15,194,24,3,
0,240,56,13,6,62,0,15,4,98,25,128,192,55,142,27,
3,192,240,54,12,134,30,0,64,15,253,255,64,72,16,4,
0,128,32,4,1,128,48,6,0,192,24,0,30,8,198,25,
134,113,15,131,241,142,192,240,60,13,134,63,0,30,24,76,
11,3,192,240,54,28,123,0,192,38,25,140,60,0,8,0,
0,16,0,0,192,0,3,255,255,255,255,255,224,255,255,254,
255,255,252,192,0,0,64,0,0,128,0,0,0,0,32,0,
0,64,0,0,103,255,255,255,255,255,224,255,255,255,255,255,
252,0,0,96,0,1,0,0,2,0,59,241,140,60,224,231,
134,49,251,128,64,15,0,252,12,240,195,204,15,192,252,60,
207,15,192,240,4,0,0,96,60,31,143,55,135,192,248,27,
195,30,96,252,7,128,48,93,0,93,146,160,0,6,0,60,
1,224,15,0,56,1,192,15,0,120,0,240,0,120,0,56,
0,28,0,30,0,15,0,7,128,3,0,64,48,24,6,1,
128,192,48,24,6,3,0,192,48,24,6,3,0,192,96,24,
6,3,0,192,0,192,1,224,0,240,0,120,0,56,0,28,
0,30,0,15,0,30,0,240,3,128,28,0,240,7,128,60,
0,96,0,2,0,16,0,128,4,12,33,159,240,62,0,224,
13,128,68,4,16,129,2,4,8,16,32,126,135,14,28,104,
211,56,64,132,33,9,255,249,140,99,24,207,255,200,66,4,
137,18,55,255,177,34,68,137,18,36,111,255,98,68,128,128,
0,2,192,0,24,224,0,224,127,255,0,31,240,0,0,16,
0,7,255,0,63,255,129,128,0,196,0,0,80,0,0,64,

};
METRIC cmmi7_metric[128] = {
{2155, -1, 19, 21, 20, 318261},
{2208, -2, 19, 23, 20, 437819},
{2266, -2, 19, 21, 20, 398623},
{2319, -1, 19, 19, 20, 365912},
{2367, -1, 19, 22, 20, 387415},
{2422, -1, 19, 26, 20, 429218},
{2487, -2, 19, 22, 20, 406598},
{2542, -1, 19, 20, 20, 310841},
{2592, -1, 19, 19, 20, 353167},
{2640, -1, 19, 20, 20, 328873},
{2690, -2, 19, 21, 20, 403720},
{2743, -2, 12, 17, 13, 340706},
{2771, -1, 19, 16, 26, 297167},
{2823, 0, 12, 16, 19, 275286},
{2861, -2, 19, 11, 20, 238206},
{2889, -3, 12, 9, 13, 218432},
{2904, -1, 19, 12, 26, 238479},
{2943, -1, 12, 13, 19, 270020},
{2974, -2, 19, 11, 20, 249628},
{3002, -2, 12, 7, 13, 193955},
{3014, -2, 12, 14, 13, 306823},
{3037, -2, 19, 15, 20, 310843},
{3075, -2, 12, 15, 19, 318657},
{3111, -2, 12, 13, 13, 262753},
{3133, -1, 19, 12, 26, 238479},
{3172, -2, 12, 15, 13, 306467},
{3197, -2, 12, 13, 19, 271908},
{3228, -2, 12, 15, 13, 303690},
{3253, -2, 12, 13, 13, 241688},
{3275, -1, 12, 14, 13, 290363},
{3298, -2, 19, 16, 26, 315120},
{3350, -1, 12, 17, 19, 327454},
{3391, -1, 19, 18, 26, 346797},
{3450, 0, 12, 18, 13, 330155},
{3480, -1, 12, 11, 13, 247604},
{3498, -1, 19, 15, 20, 316472},
{3536, -2, 12, 23, 13, 435795},
{3574, -3, 12, 12, 19, 271908},
{3603, -2, 12, 10, 16, 201501},
{3623, -2, 12, 17, 19, 344703},
{4083, -2, 11, 27, 6, 522470},
{4104, -2, 7, 27, 6, 522470},
{4125, -2, 11, 27, 6, 522470},
{4146, -2, 7, 27, 6, 522470},
{4167, -2, 14, 4, 9, 155649},
{4172, -2, 14, 4, 9, 155649},
{4177, -1, 13, 13, 14, 268517},
{4200, -2, 13, 12, 14, 268517},
{3823, -2, 12, 11, 13, 268517},
{3841, -3, 12, 9, 13, 268517},
{3856, -1, 12, 12, 13, 268517},
{3876, -1, 12, 13, 19, 268517},
{3907, -1, 12, 13, 19, 268517},
{3938, -2, 12, 11, 19, 268517},
{3965, -2, 18, 11, 19, 268517},
{3992, -2, 13, 13, 20, 268517},
{4025, -1, 18, 13, 19, 268517},
{4056, -2, 12, 11, 19, 268517},
{4221, -3, 2, 3, 3, 155649},
{4223, -3, 2, 3, 9, 155649},
{4227, -3, 16, 18, 20, 409602},
{4272, -2, 21, 11, 29, 268517},
{4312, -3, 16, 18, 20, 409602},
{4357, 1, 14, 17, 16, 268517},
{3664, -2, 19, 15, 20, 279871},
{0, -1, 19, 21, 20, 394129},
{53, -1, 19, 22, 20, 395984},
{108, -2, 19, 21, 20, 375896},
{161, -1, 19, 23, 20, 428507},
{219, -1, 19, 22, 20, 384753},
{274, -1, 19, 22, 20, 332370},
{329, -2, 19, 21, 20, 408026},
{382, -1, 19, 26, 20, 429218},
{447, -1, 19, 14, 20, 232268},
{482, -2, 19, 16, 20, 289952},
{522, -1, 19, 26, 20, 440369},
{587, -1, 19, 19, 20, 359540},
{635, -1, 19, 31, 20, 499760},
{713, -1, 19, 26, 20, 415109},
{778, -2, 19, 21, 20, 398623},
{831, -1, 19, 22, 20, 333666},
{886, -2, 19, 21, 26, 412732},
{955, -1, 19, 23, 20, 394806},
{1013, -2, 19, 17, 20, 321811},
{1056, 0, 19, 21, 20, 309545},
{1109, -2, 19, 21, 20, 357009},
{1162, -2, 19, 21, 20, 309478},
{1215, -2, 19, 30, 20, 492888},
{1290, -1, 19, 25, 20, 429787},
{1353, -1, 19, 22, 20, 308068},
{1408, -2, 19, 20, 20, 357082},
{4391, -2, 21, 7, 22, 212083},
{4411, -2, 20, 7, 27, 212083},
{4435, -2, 19, 7, 26, 212083},
{4458, -2, 10, 27, 7, 522470},
{4482, -2, 11, 27, 8, 522470},
{3702, 0, 19, 12, 20, 219366},
{1458, -2, 12, 13, 13, 284272},
{1480, -2, 19, 10, 20, 230469},
{1505, -2, 12, 11, 13, 234209},
{1523, -2, 19, 14, 20, 272818},
{1558, -2, 12, 11, 13, 248652},
{1576, -1, 19, 14, 26, 255548},
{1622, -1, 12, 13, 19, 255676},
{1653, -2, 19, 14, 20, 306823},
{1688, -1, 19, 8, 20, 185421},
{1708, 1, 19, 12, 26, 216863},
{1747, -2, 19, 13, 20, 278606},
{1780, -2, 19, 6, 20, 165737},
{1795, -1, 12, 25, 13, 465051},
{1836, -1, 12, 16, 13, 323966},
{1862, -2, 12, 12, 13, 258686},
{1882, 1, 12, 15, 19, 270163},
{1918, -2, 12, 11, 19, 240202},
{1945, -1, 12, 12, 13, 243335},
{1965, -2, 12, 10, 13, 247354},
{1982, -1, 17, 9, 18, 197975},
{2003, -1, 12, 15, 13, 309858},
{2028, -1, 12, 12, 13, 262146},
{2048, -1, 12, 19, 13, 379129},
{2079, -1, 12, 15, 13, 297189},
{2104, -1, 12, 13, 19, 265788},
{2135, -1, 12, 12, 13, 250390},
{3732, -1, 12, 7, 13, 182881},
{3744, 1, 12, 11, 19, 202755},
{3771, -3, 12, 16, 19, 334941},
{3809, -6, 20, 12, 6, 268517},
{3818, -8, 18, 12, 3, 155649},
};
char cmmi7_bitmap[4509] = {
0,1,0,0,24,0,1,192,0,15,0,0,184,0,13,192,
0,78,0,4,112,0,35,128,2,28,0,32,224,1,7,128,
16,28,1,255,224,8,7,0,128,56,4,1,192,64,14,6,
0,121,252,15,240,7,255,224,3,128,224,28,1,128,112,7,
1,192,28,7,0,112,56,3,128,224,28,3,128,224,15,255,
0,112,14,1,192,28,7,0,48,28,0,224,224,7,3,128,
28,14,0,224,56,3,129,192,56,63,255,128,0,63,8,6,
4,192,192,28,12,0,96,192,3,12,0,24,224,0,142,0,
0,112,0,3,128,0,56,0,1,192,0,14,0,0,112,0,
33,128,2,12,0,16,48,1,1,128,16,3,7,0,15,192,
0,7,255,224,1,192,224,7,0,96,14,0,96,28,0,192,
56,1,192,224,3,129,192,7,3,128,14,7,0,28,28,0,
112,56,0,224,112,1,192,224,7,3,128,12,7,0,56,14,
0,224,28,3,128,112,28,7,255,224,0,7,255,252,3,128,
112,28,0,192,112,3,1,192,12,7,0,32,56,16,128,224,
64,3,131,0,15,252,0,112,32,1,192,128,7,2,8,28,
0,64,224,1,3,128,4,14,0,32,56,1,129,192,30,63,
255,240,7,255,252,3,128,112,28,0,192,112,3,1,192,12,
7,0,32,56,32,128,224,128,3,130,0,15,248,0,112,64,
1,193,0,7,4,0,28,0,0,224,0,3,128,0,14,0,
0,56,0,1,192,0,63,240,0,0,63,8,6,4,192,192,
28,12,0,96,192,3,12,0,24,224,0,142,0,0,112,0,
3,128,0,56,0,1,192,63,238,0,28,112,0,225,128,14,
12,0,112,48,3,129,128,60,3,2,192,15,226,0,7,252,
127,192,56,3,128,28,1,192,7,0,112,1,192,28,0,112,
7,0,56,3,128,14,0,224,3,128,56,0,255,254,0,112,
7,0,28,1,192,7,0,112,1,192,28,0,224,14,0,56,
3,128,14,0,224,3,128,56,1,192,28,3,254,63,224,7,
252,3,128,28,0,112,1,192,7,0,56,0,224,3,128,14,
0,112,1,192,7,0,28,0,224,3,128,14,0,56,1,192,
63,224,1,255,0,28,0,56,0,56,0,56,0,56,0,112,
0,112,0,112,0,112,0,224,0,224,0,224,0,224,1,192,
113,192,241,192,227,128,199,0,124,0,7,252,31,192,56,1,
128,28,0,128,7,0,64,1,192,96,0,112,32,0,56,16,
0,14,24,0,3,142,0,0,231,128,0,118,112,0,30,28,
0,7,3,128,1,192,224,0,224,28,0,56,7,0,14,0,
224,3,128,56,1,192,15,3,254,15,240,7,254,0,28,0,
7,0,0,224,0,28,0,3,128,0,224,0,28,0,3,128,
0,112,0,28,0,3,128,0,112,0,14,0,67,128,16,112,
2,14,0,129,192,48,112,30,127,255,128,7,240,0,254,1,
224,1,224,5,192,7,128,11,128,23,0,23,0,46,0,46,
0,156,0,156,2,112,1,28,4,224,2,56,17,192,4,112,
67,128,16,224,142,0,33,194,28,0,67,136,56,0,131,144,
112,2,7,65,192,4,15,3,128,8,30,7,0,16,56,14,
0,96,96,56,7,240,195,254,0,7,224,31,192,56,1,128,
23,0,64,5,192,16,1,56,4,0,78,1,0,33,192,128,
8,112,32,2,14,8,0,131,130,0,64,113,0,16,28,64,
4,3,144,1,0,228,0,128,30,0,32,7,128,8,0,224,
2,0,56,1,128,4,3,248,1,0,0,63,0,14,14,0,
192,24,12,0,224,192,3,12,0,28,224,0,238,0,7,112,
0,59,128,1,248,0,29,192,0,238,0,7,112,0,115,128,
3,12,0,56,112,3,129,192,48,7,7,0,15,192,0,7,
255,224,3,129,192,28,1,128,112,7,1,192,28,7,0,112,
56,3,128,224,14,3,128,112,14,7,0,127,240,1,192,0,
7,0,0,28,0,0,224,0,3,128,0,14,0,0,56,0,
1,192,0,63,224,0,0,63,0,14,14,0,192,56,12,0,
224,192,3,12,0,28,224,0,238,0,7,112,0,59,128,1,
248,0,29,192,0,238,0,7,112,0,115,128,3,12,0,56,
113,195,129,209,48,7,15,0,15,224,128,3,4,0,28,96,
0,230,0,7,240,0,31,0,0,112,0,7,255,192,1,192,
224,7,0,96,14,0,224,28,1,192,56,3,128,224,14,1,
192,56,3,128,192,7,254,0,28,15,0,56,14,0,112,14,
0,224,28,3,128,112,7,0,224,14,1,192,28,3,132,112,
7,23,252,3,192,0,248,129,130,193,128,192,128,32,192,16,
96,8,48,0,28,0,15,224,3,252,0,127,0,3,128,0,
224,0,113,0,48,128,24,64,8,48,12,52,8,17,248,0,
31,255,249,192,224,200,14,6,64,112,20,3,128,160,28,10,
1,192,64,14,0,0,112,0,3,128,0,56,0,1,192,0,
14,0,0,112,0,7,0,0,56,0,1,192,0,14,0,0,
224,0,127,240,0,255,131,248,224,6,14,0,32,112,1,3,
128,8,28,0,65,192,4,14,0,32,112,1,3,128,8,56,
0,129,192,4,14,0,32,112,1,3,0,16,24,0,128,192,
8,3,0,128,12,24,0,63,0,0,255,1,249,192,3,14,
0,16,112,1,3,192,16,14,0,128,112,8,3,128,128,28,
4,0,224,64,7,132,0,28,32,0,226,0,7,32,0,57,
0,1,208,0,15,0,0,56,0,1,128,0,8,0,0,255,
31,224,252,224,14,0,195,128,56,2,14,1,224,16,56,7,
128,64,224,46,2,3,128,184,24,14,4,224,64,56,51,130,
0,240,142,8,1,196,60,64,7,16,115,0,28,129,200,0,
118,7,64,1,208,29,0,7,128,120,0,30,1,224,0,112,
7,0,1,192,24,0,6,0,96,0,7,252,127,128,112,14,
0,60,4,0,14,4,0,7,132,0,1,196,0,0,228,0,
0,124,0,0,28,0,0,15,0,0,7,128,0,7,224,0,
6,112,0,6,56,0,6,14,0,6,7,0,6,3,192,2,
0,224,7,0,120,31,192,255,0,255,128,252,112,0,193,224,
4,3,128,48,15,1,128,60,4,0,112,32,1,225,0,3,
136,0,14,64,0,62,0,0,112,0,1,192,0,7,0,0,
56,0,0,224,0,3,128,0,14,0,0,112,0,15,248,0,
7,255,240,112,14,12,1,192,192,56,8,7,0,128,240,16,
14,0,1,192,0,56,0,7,0,0,224,0,28,0,3,128,
128,112,16,15,1,0,224,16,28,2,3,128,96,112,30,15,
255,192,7,176,99,132,28,96,198,6,48,49,129,152,24,192,
206,6,80,114,197,163,199,0,124,3,1,128,96,24,6,3,
0,220,56,140,54,13,131,96,216,60,27,6,193,16,198,96,
224,7,129,136,67,152,230,0,192,24,6,0,192,8,5,1,
16,193,224,0,124,0,48,1,128,6,0,24,0,96,3,1,
236,12,112,65,195,6,24,24,96,97,129,140,12,48,50,192,
201,7,38,45,15,28,7,129,136,64,152,38,4,255,24,2,
0,192,8,5,129,16,193,224,0,56,1,176,7,192,19,0,
192,3,0,12,1,254,0,192,6,0,24,0,96,1,128,6,
0,48,0,192,3,0,12,0,48,0,192,6,0,24,6,96,
57,0,200,1,192,0,1,216,17,193,14,24,97,131,12,24,
96,198,12,48,97,131,4,56,51,128,236,0,96,3,12,48,
225,134,24,31,0,62,0,24,0,192,3,0,12,0,48,1,
128,6,120,30,48,96,195,131,12,12,48,48,192,198,6,24,
25,96,197,131,28,12,176,28,6,7,6,0,0,0,0,56,
76,76,140,152,24,24,48,50,98,98,100,56,0,96,7,0,
96,0,0,0,0,0,0,56,4,192,70,8,192,140,0,192,
12,1,128,24,1,128,24,3,0,48,3,0,48,102,14,96,
204,7,128,62,0,48,3,0,24,0,192,6,0,96,3,14,
24,184,201,204,140,104,3,128,31,1,140,12,98,99,19,24,
176,201,131,128,124,49,134,24,99,12,48,198,24,97,140,52,
211,77,24,48,248,124,44,134,67,19,131,65,147,129,192,201,
128,192,96,192,96,48,96,48,24,96,48,24,48,24,12,152,
12,12,76,6,6,44,6,3,38,3,0,224,48,248,89,12,
78,12,156,12,152,12,24,12,24,12,48,24,48,25,48,49,
48,49,96,50,96,28,3,128,198,24,35,3,96,54,3,96,
60,6,192,100,12,96,130,48,30,0,12,120,45,24,76,17,
48,50,96,96,192,193,129,134,6,12,12,24,16,48,64,209,
1,156,3,0,6,0,24,0,48,0,96,3,240,0,7,33,
28,67,152,102,12,193,152,54,12,193,152,49,14,51,131,176,
6,0,192,48,6,0,192,254,0,49,229,167,76,121,198,152,
1,128,24,3,0,48,3,0,48,6,0,96,0,7,6,33,
156,198,56,7,128,240,14,65,184,108,18,8,124,0,2,3,
1,128,192,199,252,48,24,24,12,6,3,3,1,136,196,100,
50,14,0,56,12,152,25,48,52,96,201,129,131,3,6,6,
24,24,48,50,96,100,193,200,197,160,241,192,56,20,195,76,
24,193,152,17,129,24,19,2,48,35,4,48,65,136,15,0,
56,24,41,131,13,48,96,198,24,25,131,2,48,96,70,12,
9,131,2,48,96,70,12,16,193,130,12,88,128,241,224,14,
60,39,156,142,58,24,100,48,0,96,0,192,3,0,6,9,
140,19,152,70,81,7,28,0,56,26,96,211,7,24,105,131,
12,24,96,198,12,48,97,131,12,56,35,128,236,0,96,2,
28,48,227,2,48,15,0,7,16,242,16,224,4,0,128,16,
2,0,192,16,34,2,56,68,120,131,0,7,255,248,7,1,
192,112,6,3,128,48,28,1,128,224,8,14,0,64,112,0,
3,128,0,28,0,1,192,0,14,0,0,112,0,3,128,0,
56,0,1,192,0,14,0,0,112,0,7,0,1,255,128,0,
0,1,128,0,7,0,0,30,0,0,62,0,0,156,0,2,
56,0,8,120,0,48,240,0,64,224,1,1,192,4,3,192,
16,3,128,96,7,0,128,15,2,0,14,8,0,28,32,0,
60,255,255,249,255,255,247,255,255,240,0,63,0,14,14,0,
192,56,12,0,224,192,3,12,0,28,224,0,238,32,71,113,
254,59,143,241,248,255,29,199,248,238,32,71,112,0,115,128,
3,12,0,56,112,3,129,192,48,7,7,0,15,192,0,0,
2,0,0,192,0,56,0,7,0,1,224,0,62,0,11,192,
3,56,0,71,0,16,224,2,28,0,131,128,48,112,4,14,
1,1,192,32,60,8,7,131,0,112,224,14,126,15,240,3,
255,252,15,255,240,64,0,129,0,2,4,0,8,0,0,0,
0,0,0,0,0,2,0,64,15,255,0,127,248,1,0,32,
0,0,0,0,0,0,0,0,16,0,8,64,0,33,0,0,
143,255,252,63,255,240,7,255,255,192,56,3,128,28,1,192,
7,0,112,1,192,28,0,112,7,0,56,3,128,14,0,224,
3,128,56,0,224,14,0,112,7,0,28,1,192,7,0,112,
1,192,28,0,224,14,0,56,3,128,14,0,224,3,128,56,
1,192,28,3,254,63,224,7,255,252,30,0,112,56,0,192,
240,3,1,192,12,7,128,32,14,0,128,60,0,0,112,0,
1,192,0,7,0,0,8,0,0,64,16,6,0,128,32,2,
1,0,24,8,0,64,192,7,7,255,252,63,255,224,30,1,
227,240,127,99,142,56,25,193,0,216,0,15,0,0,240,0,
14,0,0,224,0,14,0,1,192,0,28,0,1,192,0,28,
0,3,128,0,56,0,3,128,0,56,0,7,0,7,255,0,
1,255,192,3,128,0,224,0,28,0,31,224,12,119,7,28,
113,195,134,112,112,238,14,31,131,135,112,112,230,14,56,225,
206,14,115,0,127,128,1,192,0,56,0,14,0,31,252,0,
3,255,128,3,128,0,112,0,7,0,224,112,247,7,28,112,
225,199,14,56,112,227,135,14,56,113,199,7,28,112,113,206,
3,29,192,27,184,0,254,0,3,128,0,56,0,7,0,7,
255,0,0,63,128,7,7,0,224,12,14,0,112,224,1,135,
0,12,112,0,227,128,7,28,0,56,224,3,135,0,24,24,
1,192,192,12,6,0,194,16,12,80,128,66,132,6,39,224,
63,63,3,249,248,31,128,3,192,6,16,70,12,38,3,38,
1,147,0,209,128,105,128,56,192,24,32,12,24,14,4,25,
161,240,96,0,60,0,194,1,3,2,3,4,3,4,2,8,
6,8,252,8,252,8,4,16,6,16,6,16,6,16,6,32,
12,32,12,48,24,48,16,72,96,71,192,64,0,64,0,128,
0,128,0,128,0,128,0,15,1,63,129,121,194,64,66,128,
100,0,36,0,40,0,40,0,40,0,48,0,48,0,48,0,
32,0,32,0,32,0,64,0,64,0,64,0,64,3,192,196,
48,6,0,192,24,1,192,60,13,194,24,195,48,38,4,193,
176,38,4,65,136,32,136,14,0,15,140,8,12,12,7,243,
1,0,128,96,48,12,35,224,0,128,8,0,240,23,2,0,
64,8,1,128,48,3,0,32,6,0,96,6,0,96,14,0,
96,6,0,112,3,224,15,128,28,0,192,12,8,128,112,48,
242,216,211,7,56,57,129,140,12,96,102,6,48,49,129,140,
12,192,198,6,0,48,1,128,24,0,192,6,0,32,3,128,
136,49,140,49,6,96,204,27,3,96,111,255,255,96,108,13,
131,48,102,8,195,24,193,16,28,0,24,48,97,131,6,24,
48,99,135,22,103,0,24,56,99,225,147,12,128,52,0,224,
3,248,24,112,96,197,131,22,12,112,50,192,112,28,0,12,
0,12,0,24,0,56,0,48,0,96,0,224,0,192,1,128,
3,128,15,0,55,0,198,3,12,12,12,48,24,192,51,0,
54,0,112,12,12,24,24,48,48,192,193,129,131,3,6,6,
24,24,48,50,96,100,193,203,197,166,241,204,0,24,0,96,
0,192,1,128,2,0,0,120,24,192,198,6,96,99,3,24,
48,193,12,24,97,131,24,25,129,176,14,0,0,1,0,16,
1,240,111,12,1,128,48,3,0,48,1,0,31,193,252,48,
6,0,96,12,0,192,12,0,224,7,128,62,0,248,1,192,
12,8,128,112,63,254,255,255,17,4,34,0,68,1,152,2,
48,4,96,24,192,49,128,195,1,135,2,4,0,1,224,25,
129,4,24,49,129,140,12,96,102,6,48,49,129,12,16,209,
6,112,48,1,128,24,0,192,6,0,32,0,3,254,31,252,
97,129,129,134,3,12,6,24,12,96,48,192,97,129,129,2,
1,24,1,224,0,63,251,255,240,129,4,0,96,3,0,16,
0,128,12,0,96,3,0,48,1,128,0,56,5,48,52,192,
99,1,152,4,96,17,128,76,2,48,8,192,67,2,6,16,
7,128,0,16,0,16,0,32,0,32,0,32,0,32,0,64,
1,240,6,76,24,70,48,130,32,131,96,131,96,131,193,6,
65,6,97,12,97,24,50,112,15,192,2,0,2,0,4,0,
4,0,4,0,4,0,56,0,166,0,131,128,128,192,128,96,
128,24,64,12,64,3,64,1,192,0,192,0,112,0,88,0,
78,0,67,0,65,128,64,96,32,49,32,12,160,3,128,0,
4,0,1,0,0,128,0,32,0,8,0,2,0,1,0,224,
65,76,16,211,4,24,194,6,96,129,24,32,70,8,19,4,
8,193,4,48,65,6,16,128,200,192,31,192,0,128,0,32,
0,16,0,4,0,1,0,0,64,0,16,0,68,0,50,0,
4,128,1,64,96,80,24,20,6,7,3,2,192,192,144,48,
70,62,97,249,248,60,56,0,7,131,252,193,32,4,0,124,
15,130,0,128,16,3,2,63,195,224,0,112,1,16,2,48,
8,96,16,192,33,128,67,24,70,88,108,152,62,96,100,192,
193,129,131,6,12,12,24,16,48,96,97,128,66,0,120,0,
63,255,254,255,255,255,16,0,36,64,0,64,129,0,130,2,
1,4,12,2,16,24,8,32,112,16,64,160,64,198,99,0,
248,252,0,224,240,0,3,192,102,8,33,131,48,51,3,48,
54,6,96,102,4,96,139,16,158,8,0,128,8,0,255,7,
248,0,128,7,198,19,1,128,192,48,12,3,0,192,28,3,
192,56,3,0,192,32,48,32,14,16,31,144,24,104,8,24,
8,12,4,6,4,3,2,2,129,3,97,3,28,135,7,254,
0,252,0,48,0,16,0,8,0,12,0,6,0,2,0,0,
3,224,8,32,32,32,224,97,192,64,0,192,1,129,227,12,
54,48,44,192,51,0,102,0,204,3,48,6,96,8,64,48,
192,192,195,0,248,0,0,96,26,1,32,50,2,32,98,4,
64,196,12,128,208,25,1,160,28,1,128,24,3,128,88,9,
129,8,96,120,56,153,52,105,131,6,24,50,197,139,35,128,
3,128,152,17,132,96,140,1,128,48,12,1,128,48,6,1,
128,48,6,0,195,48,230,25,129,224,0,24,0,48,124,97,
134,66,2,196,3,200,3,208,3,224,6,224,6,193,4,97,
12,177,24,184,224,152,0,140,0,152,0,152,0,144,0,96,
0,0,192,12,255,255,255,0,192,24,31,198,2,128,16,31,
4,17,1,96,60,7,128,240,30,3,192,120,13,131,48,97,
240,8,124,6,3,1,128,192,96,48,24,12,6,3,15,248,
31,130,14,112,119,3,0,48,6,0,224,24,3,16,193,31,
231,254,255,224,15,129,135,28,28,224,231,7,0,56,1,128,
24,15,128,7,0,28,0,112,3,176,31,192,254,7,224,115,
7,7,224,0,192,6,0,112,5,128,108,2,96,35,3,24,
16,193,6,16,49,129,143,255,128,96,3,0,24,0,192,6,
1,254,96,207,241,252,36,4,0,128,16,2,248,97,136,24,
3,0,112,15,193,248,63,6,65,204,112,120,0,7,129,12,
67,144,118,0,128,48,6,120,209,156,27,1,224,60,7,128,
208,27,2,32,198,48,60,0,64,3,255,223,252,128,72,2,
64,32,2,0,32,1,0,24,0,128,12,0,96,3,0,56,
1,192,14,0,112,3,128,8,0,15,128,131,8,8,192,102,
3,56,25,241,135,216,31,128,255,12,124,193,236,3,224,15,
0,120,3,96,49,131,3,224,30,6,49,131,32,108,5,128,
240,30,3,192,108,28,197,143,48,6,0,128,55,4,225,136,
96,240,0,4,0,0,1,0,0,0,32,0,0,24,0,0,
7,255,255,255,255,255,255,192,255,255,255,239,255,255,252,192,
0,0,4,0,0,0,128,0,0,8,0,0,0,0,0,4,
0,0,0,64,0,0,8,0,0,0,207,255,255,255,255,255,
255,192,255,255,255,255,255,255,252,0,0,6,0,0,1,0,
0,0,32,0,0,8,0,55,204,204,199,48,206,51,51,62,
192,64,7,128,63,1,156,12,56,96,243,1,248,15,193,230,
28,51,129,248,15,0,32,0,0,48,15,3,240,115,28,55,
131,224,62,3,120,49,195,7,48,63,0,240,3,93,0,93,
146,82,128,0,0,192,0,240,0,112,0,112,0,120,0,120,
0,120,0,56,0,56,0,60,0,15,0,0,224,0,14,0,
1,224,0,30,0,1,224,0,28,0,1,192,0,60,0,3,
0,32,12,1,128,96,12,1,128,96,12,1,128,96,12,1,
128,96,12,3,0,96,12,3,0,96,12,3,0,96,12,3,
0,96,12,3,0,96,12,0,192,0,60,0,3,128,0,56,
0,7,128,0,120,0,7,128,0,112,0,7,0,0,240,0,
60,0,28,0,28,0,30,0,30,0,30,0,14,0,14,0,
15,0,3,0,0,0,128,0,64,0,32,0,16,0,8,0,
4,3,194,30,63,248,7,240,0,224,0,216,0,68,0,99,
0,96,192,32,32,32,8,129,2,4,8,16,32,64,129,115,
52,56,112,225,198,141,50,69,12,16,0,129,2,4,8,16,
33,223,251,134,12,24,48,96,193,131,6,12,59,255,112,129,
2,4,8,4,137,18,36,72,223,254,196,137,18,36,72,145,
34,68,137,27,255,216,145,34,68,128,128,0,0,40,0,0,
8,128,0,2,12,0,1,128,240,1,224,7,255,240,0,31,
240,0,0,4,0,0,31,252,0,15,255,224,7,0,7,1,
128,0,48,64,0,1,16,0,0,20,0,0,1,
};
METRIC cmr10_metric[128] = {
{3696, -2, 27, 21, 28, 409601},
{3770, -2, 28, 30, 29, 546135},
{3879, -2, 28, 27, 30, 509726},
{3981, -1, 28, 26, 29, 455112},
{4076, -2, 27, 23, 28, 436908},
{4157, -2, 27, 26, 28, 491521},
{4248, -2, 27, 25, 28, 473317},
{4336, -2, 28, 27, 29, 509726},
{4434, -2, 27, 25, 28, 473317},
{4522, -2, 27, 27, 28, 509726},
{4617, -2, 28, 25, 29, 473317},
{7190, 0, 28, 26, 29, 382295},
{7285, 0, 28, 21, 29, 364090},
{7362, 0, 28, 21, 29, 364090},
{7439, 0, 28, 33, 29, 546135},
{7559, 0, 28, 33, 29, 546135},
{5564, -1, 17, 9, 18, 182045},
{5585, 3, 17, 11, 26, 200250},
{7106, -5, 28, 6, 7, 327681},
{7112, -9, 28, 6, 7, 327681},
{7118, -5, 25, 10, 4, 327681},
{7123, -4, 28, 12, 7, 327681},
{7134, -3, 24, 14, 1, 327681},
{7136, -11, 29, 8, 7, 491521},
{7143, -5, -2, 10, 7, 291272},
{5621, -1, 28, 18, 29, 327682},
{5687, -2, 17, 26, 18, 473317},
{5746, -1, 17, 29, 18, 509726},
{5812, -1, 21, 18, 26, 327681},
{5871, -1, 27, 34, 28, 591646},
{5990, -3, 28, 37, 30, 664464},
{6129, -2, 29, 27, 32, 509726},
{7152, -1, 15, 9, 4, 182045},
{6237, -4, 29, 4, 30, 182045},
{7679, -1, 28, 13, 12, 327681},
{6267, -2, 28, 30, 37, 546135},
{5283, -2, 30, 16, 33, 327681},
{6406, -2, 30, 30, 33, 546135},
{5349, -2, 29, 27, 31, 509726},
{6530, -4, 28, 5, 12, 182045},
{6538, -3, 30, 10, 42, 254863},
{6591, -2, 30, 10, 42, 254863},
{6644, -2, 30, 16, 18, 327681},
{6680, -2, 23, 27, 28, 509726},
{6775, -4, 3, 4, 12, 182045},
{7699, 0, 9, 11, 2, 218454},
{6781, -4, 3, 4, 4, 182045},
{6783, -2, 30, 16, 41, 327681},
{4708, -2, 27, 16, 29, 327681},
{4766, -4, 27, 13, 28, 327681},
{4812, -2, 27, 16, 28, 327681},
{4868, -2, 27, 16, 29, 327681},
{4926, -1, 27, 18, 28, 327681},
{4989, -2, 27, 16, 29, 327681},
{5047, -2, 27, 16, 29, 327681},
{5105, -2, 27, 17, 29, 327681},
{5167, -2, 27, 16, 29, 327681},
{5225, -2, 27, 16, 29, 327681},
{6865, -4, 17, 4, 18, 182045},
{6874, -4, 17, 4, 26, 182045},
{6252, -4, 20, 4, 30, 182045},
{6887, -2, 15, 27, 12, 509726},
{5509, -2, 20, 15, 29, 309477},
{5454, -2, 28, 15, 29, 309477},
{6928, -2, 28, 27, 29, 509726},
{0, -1, 28, 28, 29, 491521},
{102, -2, 27, 24, 28, 464215},
{186, -2, 28, 25, 30, 473317},
{280, -2, 27, 27, 28, 500624},
{375, -2, 27, 24, 28, 446010},
{459, -2, 27, 22, 28, 427806},
{536, -2, 28, 28, 30, 514277},
{641, -2, 27, 26, 28, 491521},
{732, -1, 27, 12, 28, 236658},
{774, -1, 27, 16, 29, 336783},
{832, -2, 27, 28, 28, 509726},
{930, -2, 27, 21, 28, 409601},
{1004, -2, 27, 33, 28, 600748},
{1120, -2, 27, 26, 28, 491521},
{1211, -2, 28, 27, 30, 509726},
{1313, -2, 27, 23, 28, 446010},
{1394, -2, 28, 27, 37, 509726},
{1519, -2, 27, 28, 29, 482419},
{1621, -2, 28, 18, 30, 364090},
{1689, -1, 27, 27, 28, 473317},
{1784, -2, 27, 26, 29, 491521},
{1879, -1, 27, 28, 29, 491521},
{1981, -1, 27, 40, 29, 673566},
{2126, -1, 27, 28, 28, 491521},
{2224, 0, 27, 30, 28, 491521},
{2329, -2, 27, 20, 28, 400499},
{7026, -4, 30, 7, 41, 182045},
{7702, -6, 28, 13, 12, 327681},
{7062, 0, 30, 7, 41, 182045},
{7157, -5, 27, 10, 5, 327681},
{7164, -4, 28, 3, 4, 182045},
{7098, -3, 28, 5, 12, 182045},
{2399, -2, 17, 18, 18, 327681},
{2440, -1, 28, 19, 29, 364090},
{2509, -2, 17, 14, 18, 291272},
{2541, -2, 28, 19, 29, 364090},
{2610, -1, 17, 15, 18, 291272},
{2644, 0, 28, 15, 29, 200250},
{2699, -1, 18, 18, 28, 327681},
{2762, -1, 28, 20, 29, 364090},
{2835, -1, 28, 9, 29, 182045},
{2868, 3, 28, 11, 37, 200250},
{2919, -1, 28, 19, 29, 345886},
{2988, -1, 28, 9, 29, 182045},
{3021, -1, 17, 33, 18, 546135},
{3096, -1, 17, 20, 18, 364090},
{3141, -1, 17, 18, 18, 327681},
{3182, -1, 17, 19, 26, 364090},
{3244, -2, 17, 19, 26, 345885},
{3306, -1, 17, 13, 18, 256684},
{3336, -1, 17, 13, 18, 258504},
{3366, -1, 25, 12, 26, 254863},
{3405, -1, 17, 20, 18, 364090},
{3450, -1, 17, 19, 18, 345886},
{3493, -1, 17, 27, 18, 473317},
{3554, 0, 17, 21, 18, 345886},
{3602, -1, 17, 19, 26, 345886},
{3664, -1, 17, 14, 18, 291272},
{7722, 0, 11, 20, 1, 327681},
{7725, 0, 11, 41, 1, 655362},
{7166, -5, 28, 12, 7, 327681},
{7177, -3, 27, 14, 4, 327681},
{7184, -4, 28, 12, 4, 327681},
};
char cmr10_bitmap[7731] = {
0,6,0,0,0,96,0,0,6,0,0,0,240,0,0,15,
0,0,0,240,0,0,23,128,0,1,120,0,0,23,128,0,
2,60,0,0,35,192,0,2,60,0,0,65,224,0,4,30,
0,0,65,224,0,8,15,0,0,128,240,0,24,15,128,1,
0,120,0,31,255,128,3,0,124,0,32,3,192,2,0,60,
0,96,3,224,4,0,30,0,64,1,224,12,0,31,1,224,
1,240,255,128,255,240,255,255,192,15,0,240,15,0,56,15,
0,60,15,0,28,15,0,30,15,0,30,15,0,30,15,0,
30,15,0,28,15,0,60,15,0,120,15,1,240,15,255,224,
15,0,120,15,0,60,15,0,30,15,0,14,15,0,15,15,
0,15,15,0,15,15,0,15,15,0,15,15,0,30,15,0,
30,15,0,60,15,0,120,255,255,224,0,31,128,128,112,48,
192,96,6,96,224,0,240,224,0,56,224,0,28,112,0,6,
112,0,3,120,0,0,188,0,0,92,0,0,62,0,0,15,
0,0,7,128,0,3,192,0,1,224,0,0,240,0,0,120,
0,0,60,0,0,14,0,0,23,128,0,11,192,0,4,224,
0,2,56,0,2,28,0,1,7,0,1,1,192,1,0,48,
1,0,14,3,0,0,254,0,255,255,192,1,224,30,0,60,
0,240,7,128,7,0,240,0,112,30,0,14,3,192,0,224,
120,0,30,15,0,1,193,224,0,56,60,0,7,135,128,0,
240,240,0,30,30,0,3,195,192,0,120,120,0,15,15,0,
1,225,224,0,60,60,0,7,7,128,0,224,240,0,60,30,
0,7,3,192,1,224,120,0,56,15,0,14,1,224,3,128,
60,1,192,127,255,224,0,255,255,252,15,0,60,15,0,12,
15,0,4,15,0,4,15,0,6,15,0,2,15,0,2,15,
2,2,15,2,0,15,2,0,15,2,0,15,6,0,15,254,
0,15,6,0,15,2,0,15,2,0,15,2,0,15,2,1,
15,0,1,15,0,2,15,0,2,15,0,2,15,0,6,15,
0,6,15,0,12,15,0,60,255,255,252,255,255,248,60,1,
224,240,1,131,192,2,15,0,8,60,0,48,240,0,67,192,
1,15,2,4,60,8,0,240,32,3,192,128,15,6,0,63,
248,0,240,96,3,192,128,15,2,0,60,8,0,240,32,3,
192,0,15,0,0,60,0,0,240,0,3,192,0,15,0,0,
60,0,0,248,0,63,254,0,0,31,128,128,14,6,24,1,
128,25,128,112,0,120,14,0,3,129,192,0,56,28,0,1,
131,128,0,24,120,0,0,135,128,0,8,112,0,0,143,0,
0,0,240,0,0,15,0,0,0,240,0,0,15,0,0,0,
240,0,0,15,0,15,255,240,0,15,135,0,0,120,120,0,
7,135,128,0,120,56,0,7,129,192,0,120,28,0,7,128,
224,0,120,7,0,11,128,24,1,24,0,224,96,128,1,248,
0,255,243,255,195,192,15,0,240,3,192,60,0,240,15,0,
60,3,192,15,0,240,3,192,60,0,240,15,0,60,3,192,
15,0,240,3,192,60,0,240,15,0,60,3,255,255,0,240,
3,192,60,0,240,15,0,60,3,192,15,0,240,3,192,60,
0,240,15,0,60,3,192,15,0,240,3,192,60,0,240,15,
0,60,3,192,15,0,240,3,195,255,207,255,255,240,240,15,
0,240,15,0,240,15,0,240,15,0,240,15,0,240,15,0,
240,15,0,240,15,0,240,15,0,240,15,0,240,15,0,240,
15,0,240,15,15,255,31,255,0,248,0,120,0,120,0,120,
0,120,0,120,0,120,0,120,0,120,0,120,0,120,0,120,
0,120,0,120,0,120,0,120,0,120,0,120,0,120,0,120,
112,120,248,120,248,120,248,120,240,240,64,224,33,192,31,0,
255,240,63,224,240,0,240,15,0,12,0,240,0,128,15,0,
16,0,240,2,0,15,0,64,0,240,8,0,15,1,0,0,
240,32,0,15,4,0,0,240,224,0,15,31,0,0,242,240,
0,15,39,128,0,244,120,0,15,131,192,0,240,30,0,15,
1,224,0,240,15,0,15,0,248,0,240,7,128,15,0,60,
0,240,3,192,15,0,30,0,240,1,240,15,0,31,143,255,
7,255,255,248,0,124,0,3,192,0,30,0,0,240,0,7,
128,0,60,0,1,224,0,15,0,0,120,0,3,192,0,30,
0,0,240,0,7,128,0,60,0,1,224,0,15,0,0,120,
0,3,192,2,30,0,16,240,0,135,128,12,60,0,97,224,
2,15,0,48,120,3,131,192,125,255,255,224,255,128,0,255,
135,192,0,124,3,224,0,62,1,120,0,47,0,188,0,23,
128,94,0,11,192,39,128,9,224,19,192,4,240,8,240,4,
120,4,120,2,60,2,60,1,30,1,15,1,15,0,135,128,
135,128,67,192,67,192,32,240,65,224,16,120,32,240,8,60,
16,120,4,15,16,60,2,7,136,30,1,3,196,15,0,128,
244,7,128,64,122,3,192,32,30,1,224,16,15,0,240,8,
7,128,120,4,1,128,60,7,0,192,30,31,240,96,255,240,
255,0,127,195,224,3,128,248,0,64,47,0,16,9,224,4,
2,120,1,0,143,0,64,35,224,16,8,120,4,2,15,1,
0,131,192,64,32,120,16,8,15,4,2,3,193,0,128,120,
64,32,31,16,8,3,196,2,0,121,0,128,30,64,32,3,
208,8,0,124,2,0,31,0,128,3,192,32,0,240,8,0,
28,2,0,3,1,192,0,195,254,0,16,0,63,128,0,28,
28,0,14,0,224,3,128,14,0,224,0,224,56,0,14,15,
0,1,225,192,0,28,120,0,3,207,0,0,121,192,0,7,
120,0,0,255,0,0,31,224,0,3,252,0,0,127,128,0,
15,240,0,1,254,0,0,63,192,0,7,184,0,0,231,128,
0,60,240,0,7,142,0,0,225,224,0,60,28,0,7,1,
192,1,192,28,0,112,1,192,28,0,14,14,0,0,127,0,
0,255,255,128,30,1,192,60,1,224,120,1,224,240,1,193,
224,3,195,192,7,135,128,15,15,0,30,30,0,60,60,0,
112,120,1,224,240,7,129,224,28,3,255,224,7,128,0,15,
0,0,30,0,0,60,0,0,120,0,0,240,0,1,224,0,
3,192,0,7,128,0,15,0,0,30,0,0,60,0,7,255,
128,0,0,63,128,0,28,28,0,14,0,224,3,128,14,0,
224,0,224,56,0,14,15,0,1,225,192,0,28,120,0,3,
207,0,0,121,192,0,7,120,0,0,255,0,0,31,224,0,
3,252,0,0,127,128,0,15,240,0,1,254,0,0,63,192,
0,7,184,0,0,231,128,0,60,240,0,7,142,0,0,225,
224,112,60,28,17,7,1,196,17,192,28,131,112,1,208,124,
0,15,14,2,0,127,192,64,0,24,8,0,3,131,0,0,
120,224,0,7,248,0,0,255,0,0,15,192,0,0,240,255,
255,0,0,240,30,0,15,0,120,0,240,3,192,15,0,28,
0,240,1,224,15,0,30,0,240,1,224,15,0,30,0,240,
1,192,15,0,60,0,240,7,128,15,1,224,0,255,240,0,
15,3,192,0,240,14,0,15,0,240,0,240,7,128,15,0,
120,0,240,7,128,15,0,120,0,240,7,128,15,0,120,0,
240,7,129,15,0,120,16,240,3,129,15,0,60,47,255,1,
194,0,0,7,192,7,224,135,6,99,0,89,192,14,96,1,
184,0,110,0,11,128,2,224,0,188,0,15,128,1,240,0,
127,192,15,254,1,255,224,31,252,0,255,128,3,224,0,124,
0,15,0,1,224,0,120,0,30,0,7,192,1,176,0,110,
0,51,64,8,204,12,32,254,0,127,255,255,206,1,224,57,
128,60,3,32,7,128,36,0,240,5,128,30,0,96,3,192,
12,0,120,1,128,15,0,32,1,224,0,0,60,0,0,7,
128,0,0,240,0,0,30,0,0,3,192,0,0,120,0,0,
15,0,0,1,224,0,0,60,0,0,7,128,0,0,240,0,
0,30,0,0,3,192,0,0,120,0,0,15,0,0,1,224,
0,0,126,0,1,255,254,0,255,240,127,195,192,3,128,240,
0,64,60,0,16,15,0,4,3,192,1,0,240,0,64,60,
0,16,15,0,4,3,192,1,0,240,0,64,60,0,16,15,
0,4,3,192,1,0,240,0,64,60,0,16,15,0,4,3,
192,1,0,240,0,64,60,0,16,15,0,4,3,192,1,0,
112,0,128,30,0,32,3,128,16,0,96,4,0,12,2,0,
1,195,0,0,31,0,0,255,224,15,241,240,0,60,15,0,
1,128,240,0,16,15,128,3,0,120,0,32,7,128,2,0,
60,0,64,3,192,4,0,60,0,64,1,224,8,0,30,0,
128,1,240,8,0,15,1,0,0,240,16,0,15,131,0,0,
120,32,0,7,130,0,0,60,64,0,3,196,0,0,60,64,
0,1,232,0,0,30,128,0,1,248,0,0,15,0,0,0,
240,0,0,6,0,0,0,96,0,0,6,0,0,255,224,255,
224,255,31,0,31,0,60,30,0,30,0,24,15,0,31,0,
16,15,0,31,0,16,15,0,31,0,16,7,128,31,0,32,
7,128,39,128,32,7,128,39,128,32,3,192,39,128,64,3,
192,67,192,64,3,192,67,192,64,3,224,67,192,64,1,224,
129,224,128,1,224,129,224,128,1,224,129,224,128,0,241,0,
241,0,0,241,0,241,0,0,241,0,241,0,0,121,0,250,
0,0,122,0,122,0,0,122,0,122,0,0,62,0,124,0,
0,60,0,60,0,0,60,0,60,0,0,60,0,60,0,0,
24,0,24,0,0,24,0,24,0,0,24,0,24,0,127,240,
255,192,252,3,224,7,128,24,0,60,1,128,3,224,16,0,
30,2,0,1,240,96,0,15,4,0,0,120,128,0,7,216,
0,0,61,0,0,1,224,0,0,31,0,0,0,240,0,0,
15,128,0,0,248,0,0,19,192,0,2,62,0,0,33,224,
0,4,31,0,0,192,248,0,8,7,128,1,0,124,0,48,
3,192,2,0,30,0,96,1,240,31,0,63,143,252,15,255,
255,240,7,252,62,0,7,128,120,0,8,1,240,0,96,3,
192,1,0,15,128,8,0,31,0,32,0,60,1,0,0,248,
4,0,1,224,32,0,7,193,128,0,15,4,0,0,30,32,
0,0,124,128,0,0,244,0,0,3,240,0,0,7,128,0,
0,30,0,0,0,120,0,0,1,224,0,0,7,128,0,0,
30,0,0,0,120,0,0,1,224,0,0,7,128,0,0,30,
0,0,0,120,0,0,31,254,0,127,255,247,192,31,112,1,
230,0,60,96,3,196,0,120,64,15,132,0,240,64,30,0,
1,224,0,60,0,7,192,0,120,0,15,0,0,240,0,30,
0,3,224,0,60,1,7,128,16,120,1,15,0,17,240,3,
30,0,35,192,2,60,0,103,128,14,248,3,239,255,254,31,
192,12,28,7,131,129,224,112,48,28,0,7,0,1,192,7,
240,15,28,14,7,7,1,193,128,112,224,28,120,7,30,1,
197,128,241,48,78,135,225,192,252,0,3,128,0,112,0,14,
0,1,192,0,56,0,7,0,0,224,0,28,0,3,128,0,
112,0,14,62,1,216,96,60,6,7,0,96,224,14,28,0,
195,128,28,112,3,142,0,113,192,14,56,1,199,0,56,224,
6,28,1,195,128,48,120,12,12,131,1,15,128,7,224,48,
193,135,140,30,112,49,128,14,0,56,0,224,3,128,14,0,
56,0,96,1,192,19,0,70,2,12,48,31,0,0,63,0,
0,224,0,28,0,3,128,0,112,0,14,0,1,192,0,56,
0,7,0,0,224,0,28,1,243,128,193,112,48,30,12,1,
195,128,56,96,7,28,0,227,128,28,112,3,142,0,113,192,
14,56,1,195,0,56,112,7,6,0,224,96,60,6,27,128,
124,126,3,224,24,96,96,97,128,103,0,236,0,248,1,255,
255,224,1,192,3,128,7,0,6,0,14,0,76,0,140,2,
12,24,7,192,0,248,3,24,28,120,48,240,224,193,192,3,
128,7,0,14,0,28,0,56,7,255,0,224,1,192,3,128,
7,0,14,0,28,0,56,0,112,0,224,1,192,3,128,7,
0,14,0,28,0,56,0,112,7,254,0,0,3,128,241,48,
195,140,112,226,24,24,14,7,3,129,192,224,112,56,28,6,
6,1,195,128,48,192,19,192,4,0,3,0,0,96,0,31,
248,7,255,193,255,248,192,14,96,1,240,0,60,0,15,0,
3,96,1,140,0,193,192,224,31,224,252,0,1,192,0,28,
0,1,192,0,28,0,1,192,0,28,0,1,192,0,28,0,
1,192,0,28,0,1,199,192,28,135,1,208,48,30,3,129,
192,56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,
56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,56,
255,159,240,24,30,15,3,0,0,0,0,0,0,0,0,31,
129,192,224,112,56,28,14,7,3,129,192,224,112,56,28,14,
7,3,143,248,0,192,60,7,128,96,0,0,0,0,0,0,
0,0,0,7,240,14,1,192,56,7,0,224,28,3,128,112,
14,1,192,56,7,0,224,28,3,128,112,14,1,192,56,7,
96,254,27,199,48,195,224,252,0,3,128,0,112,0,14,0,
1,192,0,56,0,7,0,0,224,0,28,0,3,128,0,112,
0,14,31,225,192,240,56,24,7,2,0,224,128,28,32,3,
136,0,115,128,14,240,1,231,0,56,240,7,14,0,225,224,
28,28,3,129,192,112,60,14,7,207,249,254,252,14,7,3,
129,192,224,112,56,28,14,7,3,129,192,224,112,56,28,14,
7,3,129,192,224,112,56,28,14,7,3,143,248,252,126,7,
224,14,65,196,28,7,64,100,6,3,192,60,3,129,192,28,
1,192,224,14,0,224,112,7,0,112,56,3,128,56,28,1,
192,28,14,0,224,14,7,0,112,7,3,128,56,3,129,192,
28,1,192,224,14,0,224,112,7,0,112,56,3,128,56,28,
1,192,28,127,199,252,127,192,252,124,1,200,112,29,3,1,
224,56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,
56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,56,
28,3,143,249,255,3,240,3,135,1,128,96,192,12,112,3,
152,0,110,0,31,128,7,224,1,248,0,126,0,31,128,7,
96,1,156,0,227,0,48,96,24,14,28,0,252,0,252,124,
3,176,192,120,12,14,0,193,192,28,56,1,135,0,56,224,
7,28,0,227,128,28,112,3,142,0,113,192,28,56,3,135,
0,96,240,24,29,6,3,159,0,112,0,14,0,1,192,0,
56,0,7,0,0,224,0,28,0,31,240,0,3,193,1,134,
96,96,44,24,7,135,0,112,224,14,56,1,199,0,56,224,
7,28,0,227,128,28,112,3,134,0,112,224,14,12,3,192,
192,120,12,55,0,248,224,0,28,0,3,128,0,112,0,14,
0,1,192,0,56,0,7,0,7,252,252,224,233,135,158,60,
241,195,14,0,112,3,128,28,0,224,7,0,56,1,192,14,
0,112,3,128,28,7,254,0,31,145,131,144,13,128,44,1,
112,11,224,15,240,63,224,127,128,63,0,120,1,224,15,0,
124,2,208,100,126,0,4,0,64,4,0,64,12,0,192,28,
3,192,255,225,192,28,1,192,28,1,192,28,1,192,28,1,
192,28,17,193,28,17,193,28,16,193,14,32,60,252,31,129,
192,56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,
56,28,3,129,192,56,28,3,129,192,56,28,3,129,192,56,
28,7,128,192,120,14,27,128,62,63,255,7,231,128,112,112,
4,14,0,128,224,32,28,4,1,193,0,56,32,7,4,0,
113,0,14,32,1,236,0,29,0,3,160,0,56,0,7,0,
0,224,0,8,0,255,63,207,231,129,224,112,112,28,6,14,
3,128,129,192,176,16,28,23,4,3,130,224,128,112,140,16,
7,17,196,0,226,56,128,28,131,16,1,208,116,0,58,14,
128,7,128,208,0,112,28,0,14,3,128,1,128,48,0,16,
4,0,127,143,240,120,28,3,192,192,14,4,0,56,64,0,
228,0,7,96,0,30,0,0,112,0,3,192,0,62,0,1,
56,0,16,224,1,7,0,24,28,0,128,112,30,7,199,248,
127,192,255,7,231,128,112,112,4,14,0,128,224,32,28,4,
1,193,0,56,32,7,4,0,113,0,14,32,1,236,0,29,
0,3,160,0,56,0,7,0,0,224,0,8,0,1,0,0,
64,0,8,1,225,0,60,64,7,152,0,102,0,7,128,0,
127,253,192,230,3,144,28,64,241,3,132,28,0,240,3,128,
28,0,240,67,129,28,4,240,51,128,156,2,112,59,255,224,
255,255,240,120,7,131,192,12,30,0,32,240,1,7,128,12,
60,0,33,224,1,15,0,8,120,0,3,192,0,30,0,0,
240,0,7,128,0,60,0,1,224,0,15,0,0,120,0,3,
192,0,30,0,0,240,0,7,128,0,60,0,1,224,0,15,
0,0,120,0,3,224,1,255,240,0,0,3,0,0,0,12,
0,0,0,120,0,0,1,224,0,0,15,192,0,0,47,0,
0,1,62,0,0,4,120,0,0,33,240,0,0,131,192,0,
4,15,128,0,16,30,0,0,128,124,0,2,0,240,0,16,
3,224,0,64,7,128,2,0,31,0,8,0,60,0,64,0,
248,1,0,1,224,8,0,7,192,32,0,15,1,0,0,62,
4,0,0,120,32,0,0,240,128,0,3,199,255,255,255,191,
255,255,255,255,255,255,252,0,63,128,0,28,28,0,14,0,
224,3,128,14,0,224,0,224,56,0,14,15,0,1,225,192,
0,28,120,0,3,207,0,0,121,192,0,7,120,0,0,255,
16,1,31,227,255,227,252,127,252,127,143,255,143,241,0,17,
254,0,0,63,192,0,7,184,0,0,231,128,0,60,240,0,
7,142,0,0,225,224,0,60,28,0,7,1,192,1,192,28,
0,112,1,192,28,0,14,14,0,0,127,0,0,0,12,0,
0,3,0,0,0,192,0,0,120,0,0,30,0,0,7,128,
0,3,240,0,0,188,0,0,47,0,0,19,224,0,4,120,
0,1,30,0,0,199,192,0,32,240,0,8,60,0,6,15,
128,1,1,224,0,64,120,0,16,30,0,8,3,192,2,0,
240,0,128,60,0,64,7,128,16,1,224,4,0,120,2,0,
15,0,192,3,192,120,1,248,255,131,255,192,255,255,255,255,
255,255,255,255,252,0,0,24,0,0,48,0,0,96,0,0,
128,0,0,0,0,0,0,0,0,32,0,128,64,1,0,255,
254,1,255,252,3,255,248,4,0,16,8,0,32,0,0,0,
0,0,0,0,0,0,0,0,16,0,0,96,0,0,192,0,
1,128,0,3,255,255,255,255,255,255,255,255,240,255,255,255,
195,192,15,0,240,3,192,60,0,240,15,0,60,3,192,15,
0,240,3,192,60,0,240,15,0,60,3,192,15,0,240,3,
192,60,0,240,15,0,60,3,192,15,0,240,3,192,60,0,
240,15,0,60,3,192,15,0,240,3,192,60,0,240,15,0,
60,3,192,15,0,240,3,192,60,0,240,15,0,60,3,192,
15,0,240,3,195,255,207,255,255,255,255,60,0,15,159,0,
1,199,192,0,97,224,0,16,248,0,12,62,0,2,15,0,
1,7,192,0,129,240,0,0,120,0,0,62,0,0,15,0,
0,3,192,0,1,224,0,0,96,0,0,32,0,0,32,0,
0,32,0,32,32,0,16,32,0,8,16,0,12,16,0,6,
16,0,6,16,0,7,16,0,15,159,255,255,223,255,255,224,
31,0,31,7,248,7,241,255,129,255,48,112,120,108,7,28,
7,0,115,128,96,14,96,8,0,220,0,0,31,128,0,3,
224,0,0,60,0,0,7,128,0,0,240,0,0,30,0,0,
3,192,0,0,120,0,0,15,0,0,1,224,0,0,60,0,
0,7,128,0,0,240,0,0,30,0,0,3,192,0,0,120,
0,0,15,0,0,1,224,0,0,60,0,0,15,192,0,31,
255,128,3,255,240,0,31,128,0,7,128,0,3,192,0,1,
224,0,7,252,0,14,123,128,28,60,112,28,30,28,28,15,
7,30,7,131,222,3,192,255,1,224,127,128,240,63,192,120,
31,224,60,15,240,30,7,188,15,7,142,7,131,131,131,195,
128,225,227,128,28,247,0,3,254,0,0,60,0,0,30,0,
0,15,0,0,15,192,0,127,254,0,1,255,248,0,3,240,
0,0,60,0,0,7,128,0,0,240,1,240,30,7,207,3,
193,225,224,120,60,30,15,15,3,193,225,224,120,60,60,15,
7,135,129,224,240,240,60,30,30,7,131,195,192,112,120,120,
15,15,30,1,225,227,192,28,60,112,1,199,158,0,28,243,
128,1,223,192,0,15,224,0,0,120,0,0,15,0,0,1,
224,0,0,126,0,0,255,252,0,0,127,0,1,224,240,1,
192,28,3,128,3,131,192,1,225,192,0,113,224,0,60,224,
0,14,240,0,7,248,0,3,252,0,1,254,0,0,255,0,
0,123,128,0,57,224,0,60,240,0,30,56,0,14,14,0,
14,7,0,7,1,192,7,0,96,3,0,48,1,130,12,1,
131,2,0,129,129,0,64,160,192,96,159,224,63,207,240,31,
231,248,15,240,3,192,12,48,24,24,48,12,48,12,112,14,
96,6,96,6,224,7,224,7,224,7,224,7,224,7,224,7,
224,7,224,7,224,7,224,7,224,7,224,7,224,7,96,6,
96,6,112,14,48,12,48,12,24,24,12,48,7,224,3,0,
56,15,193,142,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,
3,128,28,0,224,7,0,56,3,225,255,240,7,192,24,48,
32,28,64,12,64,14,240,15,248,15,248,7,248,7,112,7,
0,15,0,14,0,14,0,28,0,28,0,56,0,112,0,96,
0,192,1,128,3,0,6,1,12,1,24,1,16,2,63,254,
127,254,255,254,7,224,24,48,32,28,32,28,120,30,120,14,
120,30,56,30,0,28,0,28,0,24,0,48,0,96,7,224,
0,48,0,28,0,28,0,14,0,15,0,15,112,15,248,15,
248,15,248,15,240,14,64,28,32,28,24,48,7,224,0,12,
0,3,0,1,192,0,240,0,60,0,23,0,13,192,2,112,
1,28,0,199,0,33,192,16,112,12,28,2,7,1,1,192,
192,112,32,28,16,7,12,1,195,255,255,0,28,0,7,0,
1,192,0,112,0,28,0,7,0,1,192,7,255,48,12,63,
248,63,240,63,192,32,0,32,0,32,0,32,0,32,0,32,
0,35,224,36,48,40,24,48,28,32,14,0,14,0,15,0,
15,0,15,96,15,240,15,240,15,240,15,128,14,64,30,64,
28,32,56,24,112,7,192,0,240,3,12,6,4,12,14,24,
30,48,30,48,12,112,0,112,0,96,0,227,224,228,48,232,
24,240,12,240,14,224,6,224,7,224,7,224,7,224,7,224,
7,96,7,96,7,112,6,48,14,48,12,24,24,12,48,3,
224,64,0,63,255,223,255,207,255,228,0,36,0,34,0,17,
0,16,0,16,0,8,0,8,0,12,0,4,0,6,0,3,
0,3,128,1,128,0,192,0,224,0,112,0,56,0,28,0,
30,0,15,0,7,128,3,192,1,224,0,240,0,48,0,3,
224,12,48,16,8,32,12,32,6,96,6,96,6,96,6,112,
6,120,12,62,8,63,176,31,224,7,240,7,248,24,252,48,
126,96,30,96,15,192,7,192,3,192,3,192,3,192,3,96,
2,96,4,48,12,28,16,7,224,3,192,12,48,24,24,48,
12,112,12,96,14,224,6,224,6,224,7,224,7,224,7,224,
7,224,7,96,7,112,15,48,15,24,23,12,39,7,199,0,
6,0,6,0,14,48,12,120,12,120,24,112,16,32,48,48,
192,15,128,0,128,0,128,7,224,24,152,48,132,96,130,64,
129,192,135,192,143,192,143,192,134,224,128,240,128,126,128,63,
192,31,240,15,248,3,252,0,190,0,142,0,135,96,135,240,
131,240,131,224,131,128,130,64,130,64,132,32,136,24,144,7,
224,0,128,0,128,0,224,0,0,50,0,0,12,32,0,1,
132,0,0,112,128,0,14,16,0,1,194,0,0,56,64,0,
7,16,0,0,226,0,0,28,128,0,3,160,0,0,60,3,
254,7,0,30,0,224,1,128,28,0,32,5,192,4,1,56,
1,0,67,128,64,24,112,8,6,7,2,0,192,224,64,56,
14,16,7,1,226,0,224,28,128,28,1,224,7,128,28,0,
184,3,192,35,0,156,4,48,97,195,1,240,15,128,15,224,
96,113,0,119,0,127,0,254,1,252,3,128,6,0,28,0,
96,1,128,6,0,8,0,48,0,64,0,128,1,0,2,0,
4,0,8,0,0,0,0,0,0,0,0,0,0,6,0,30,
0,60,0,48,0,3,0,15,0,30,0,24,0,0,0,0,
0,0,0,0,0,0,2,0,4,0,16,0,32,0,64,0,
128,3,0,4,0,24,0,96,1,192,7,0,28,0,56,7,
240,15,224,31,192,29,128,17,128,192,254,0,252,14,7,3,
129,192,224,112,56,28,14,7,3,129,192,224,112,56,28,127,
192,15,224,28,3,128,112,14,1,192,56,7,0,224,28,3,
128,112,14,1,192,56,7,0,224,28,3,128,112,14,193,252,
55,142,97,135,192,1,248,1,195,128,224,112,48,12,28,3,
135,0,225,192,56,112,14,28,7,7,1,129,193,195,243,192,
28,24,7,3,1,192,96,112,12,28,3,135,0,225,192,28,
112,7,28,1,199,0,113,192,28,112,7,28,1,135,56,225,
206,48,115,152,252,124,0,31,192,124,28,28,96,199,131,176,
25,224,120,6,48,30,0,192,7,0,48,1,192,12,7,255,
255,15,28,0,14,7,0,7,1,192,1,128,112,0,224,28,
0,56,7,128,30,1,224,5,192,204,2,56,97,195,7,224,
31,0,3,224,15,128,96,193,131,6,3,24,12,96,13,128,
103,0,124,1,176,1,192,15,128,14,0,124,0,127,255,224,
3,128,7,0,28,0,56,0,224,1,192,7,0,6,0,56,
0,56,3,224,4,192,27,0,35,1,140,2,12,24,56,96,
31,0,124,0,0,0,128,0,64,0,48,0,8,3,244,3,
135,1,128,224,192,44,112,19,152,12,110,2,31,129,7,224,
65,248,32,126,8,31,132,7,99,1,156,128,227,64,48,112,
24,14,28,2,252,1,0,0,192,0,32,0,16,0,0,0,
127,255,255,0,1,124,3,192,0,79,0,48,0,35,192,4,
0,8,240,1,0,2,60,0,96,1,15,0,8,0,67,192,
2,0,16,240,0,128,8,60,8,0,2,15,2,0,0,131,
192,128,0,64,240,32,0,16,60,24,0,15,255,254,0,2,
3,193,128,0,128,240,32,0,64,60,8,0,16,15,2,4,
4,3,192,129,2,0,240,0,128,128,60,0,32,32,15,0,
8,16,3,192,2,4,0,240,1,131,0,60,0,193,224,15,
0,243,254,31,255,252,0,63,0,0,0,7,7,255,255,0,
224,30,0,120,14,0,240,0,192,224,3,128,2,14,0,28,
0,16,240,0,224,0,199,0,7,0,2,120,0,56,0,19,
192,1,192,16,156,0,14,0,129,224,0,112,4,15,0,3,
128,96,120,0,28,7,3,192,0,255,248,30,0,7,1,192,
240,0,56,6,7,128,1,192,16,60,0,14,0,128,240,0,
112,4,23,128,3,128,0,188,0,28,0,8,240,0,224,0,
71,128,7,0,2,30,0,56,0,48,112,1,192,1,129,192,
30,0,24,7,0,240,3,192,14,15,255,254,0,31,128,0,
0,0,0,1,0,7,240,96,3,131,136,1,192,30,0,112,
1,192,28,0,28,7,0,5,193,192,1,156,56,0,35,143,
0,8,121,192,1,7,56,0,64,239,0,16,31,224,2,3,
252,0,128,127,128,32,15,240,4,1,254,1,0,63,192,64,
7,248,8,0,247,2,0,28,224,128,3,158,16,0,241,196,
0,28,57,128,3,131,160,0,224,56,0,56,3,128,14,0,
120,3,128,17,193,192,6,15,224,0,128,0,0,111,255,255,
255,102,102,102,102,102,102,96,0,0,111,246,111,246,0,0,
6,102,102,102,102,102,102,255,255,255,246,0,3,1,128,0,
12,6,0,0,48,24,0,1,128,192,0,6,3,0,0,24,
12,0,0,96,48,0,1,128,192,0,12,6,0,0,48,24,
0,0,192,96,0,3,1,128,0,24,12,1,255,255,255,239,
255,255,255,192,12,6,0,0,48,24,0,0,192,96,0,3,
1,128,0,24,12,0,0,96,48,0,1,128,192,0,6,3,
0,63,255,255,255,127,255,255,248,3,1,128,0,24,12,0,
0,96,48,0,1,128,192,0,6,3,0,0,48,24,0,0,
192,96,0,3,1,128,0,12,6,0,0,96,48,0,1,128,
192,0,6,3,0,0,15,0,0,192,98,0,3,3,6,0,
56,28,14,1,192,96,39,251,3,128,64,24,14,1,0,192,
56,4,3,0,224,16,24,3,128,64,96,14,1,3,0,56,
4,24,0,96,32,96,1,192,131,0,3,4,24,0,6,32,
96,0,15,3,3,192,0,24,24,128,0,96,193,0,3,7,
2,0,24,24,8,0,96,224,16,3,3,128,64,24,14,1,
0,96,56,4,3,0,224,16,12,3,128,64,96,14,1,3,
0,24,8,12,0,112,32,96,0,193,3,0,1,136,4,0,
3,192,103,190,208,132,34,17,17,0,0,64,32,16,8,6,
3,0,192,96,24,12,3,1,192,96,24,6,3,128,224,56,
14,3,128,224,56,14,3,128,224,56,14,1,128,96,24,7,
0,192,48,6,1,128,48,12,1,128,32,4,0,128,16,128,
16,2,0,64,24,3,0,192,24,6,0,192,48,14,1,128,
96,24,7,1,192,112,28,7,1,192,112,28,7,1,192,112,
28,6,1,128,96,56,12,3,1,128,96,48,12,6,1,0,
128,64,32,0,1,128,1,128,1,128,1,128,65,130,241,143,
57,156,15,240,3,192,3,192,15,240,57,156,241,143,65,130,
1,128,1,128,1,128,1,128,0,6,0,0,0,192,0,0,
24,0,0,3,0,0,0,96,0,0,12,0,0,1,128,0,
0,48,0,0,6,0,0,0,192,0,0,24,0,0,3,0,
0,0,96,1,255,255,255,255,255,255,248,0,48,0,0,6,
0,0,0,192,0,0,24,0,0,3,0,0,0,96,0,0,
12,0,0,1,128,0,0,48,0,0,6,0,0,0,192,0,
0,24,0,0,3,0,0,111,247,17,17,34,72,111,246,0,
1,0,3,0,6,0,6,0,6,0,12,0,12,0,12,0,
24,0,24,0,24,0,48,0,48,0,48,0,96,0,96,0,
192,0,192,0,192,1,128,1,128,1,128,3,0,3,0,3,
0,6,0,6,0,12,0,12,0,12,0,24,0,24,0,24,
0,48,0,48,0,48,0,96,0,96,0,96,0,192,0,192,
0,111,246,0,0,0,0,0,111,246,111,246,0,0,0,0,
0,111,247,17,17,34,72,127,255,255,223,255,255,252,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,3,255,255,255,191,255,255,224,
0,63,128,0,24,12,0,12,0,96,2,0,2,0,128,0,
32,32,0,2,8,7,192,33,3,132,4,64,224,64,72,24,
7,9,7,0,225,64,192,28,24,56,3,131,7,0,112,96,
224,14,12,28,1,193,131,128,56,48,48,7,5,7,0,224,
160,96,28,36,14,7,132,64,225,49,8,7,195,192,128,0,
0,8,0,0,0,128,0,0,12,0,7,128,96,15,192,3,
255,0,255,255,6,12,24,48,96,193,131,6,12,24,48,96,
193,131,6,12,24,48,96,193,131,6,12,24,48,96,193,131,
6,12,24,48,127,254,255,252,24,48,96,193,131,6,12,24,
48,96,193,131,6,12,24,48,96,193,131,6,12,24,48,96,
193,131,6,12,24,48,96,193,255,254,8,136,132,66,16,183,
222,96,227,143,28,56,48,64,28,115,206,115,8,0,192,216,
99,240,48,128,24,1,192,52,2,112,227,252,15,0,255,252,
60,66,129,129,129,66,60,28,7,128,120,15,1,192,239,240,
1,135,14,24,0,12,4,130,17,2,128,64,255,240,56,115,
135,56,119,14,96,198,12,193,128,28,4,252,36,63,32,56,
96,111,15,240,246,6,0,126,31,0,112,108,96,48,62,60,
28,15,15,14,3,193,131,128,112,0,224,28,0,56,7,0,
14,1,192,3,128,112,0,224,28,3,255,255,240,14,1,192,
3,128,112,0,224,28,0,56,7,0,14,1,192,3,128,112,
0,224,28,0,56,7,0,14,1,192,3,128,112,0,224,28,
0,56,7,0,14,1,192,3,128,112,0,224,28,0,56,7,
0,127,135,252,0,0,126,0,14,12,0,192,96,14,7,128,
224,60,7,0,192,56,0,1,192,0,14,0,0,112,0,3,
128,1,255,255,128,224,28,7,0,224,56,7,1,192,56,14,
1,192,112,14,3,128,112,28,3,128,224,28,7,0,224,56,
7,1,192,56,14,1,192,112,14,3,128,112,28,3,135,248,
127,128,0,127,192,14,14,0,192,240,14,7,128,224,28,7,
0,224,56,7,1,192,56,14,1,192,112,14,3,128,113,255,
255,128,224,28,7,0,224,56,7,1,192,56,14,1,192,112,
14,3,128,112,28,3,128,224,28,7,0,224,56,7,1,192,
56,14,1,192,112,14,3,128,112,28,3,135,252,255,128,0,
63,7,224,0,224,78,12,0,224,60,6,0,224,62,7,128,
224,30,3,192,112,7,0,192,56,3,128,0,28,1,192,0,
14,0,224,0,7,0,112,0,3,128,56,0,31,255,255,255,
128,224,14,1,192,112,7,0,224,56,3,128,112,28,1,192,
56,14,0,224,28,7,0,112,14,3,128,56,7,1,192,28,
3,128,224,14,1,192,112,7,0,224,56,3,128,112,28,1,
192,56,14,0,224,28,7,0,112,14,3,128,56,7,1,192,
28,3,135,252,127,207,248,0,63,7,252,0,224,110,14,0,
224,124,15,0,224,62,7,128,224,30,1,192,112,7,0,224,
56,3,128,112,28,1,192,56,14,0,224,28,7,0,112,14,
3,128,56,7,31,255,255,255,128,224,14,1,192,112,7,0,
224,56,3,128,112,28,1,192,56,14,0,224,28,7,0,112,
14,3,128,56,7,1,192,28,3,128,224,14,1,192,112,7,
0,224,56,3,128,112,28,1,192,56,14,0,224,28,7,0,
112,14,3,128,56,7,1,192,28,3,135,252,127,207,248,96,
103,135,190,62,208,208,128,132,4,32,34,2,16,17,1,16,
17,1,0,255,255,252,8,8,128,136,8,128,132,4,64,66,
2,16,16,176,183,199,222,30,96,96,255,255,240,255,255,255,
255,255,128,
};
METRIC cmr5_metric[128] = {
{1117, -2, 13, 13, 14, 272161},
{1140, -2, 13, 19, 14, 359544},
{1174, -2, 13, 17, 14, 336788},
{1204, -2, 13, 15, 14, 298558},
{1231, -2, 13, 14, 14, 291276},
{1256, -2, 13, 16, 14, 321314},
{1284, -2, 13, 16, 14, 314032},
{1312, -2, 13, 17, 14, 336788},
{1342, -2, 13, 16, 14, 314032},
{1370, -2, 13, 17, 14, 336788},
{1400, -2, 13, 16, 14, 314032},
{2250, -3, 12, 8, 17, 223008},
{2233, -3, 13, 8, 17, 223008},
{2327, -3, 13, 2, 6, 131984},
{1887, -3, 9, 3, 14, 131984},
{1665, -2, 9, 9, 14, 211630},
{1681, -1, 8, 6, 9, 131984},
{1688, 0, 8, 6, 13, 143362},
{2178, -4, 13, 4, 4, 223008},
{2180, -6, 13, 4, 4, 223008},
{2182, -4, 12, 6, 2, 223008},
{2184, -3, 13, 8, 4, 223008},
{2188, -3, 11, 8, 1, 223008},
{2189, -7, 13, 6, 3, 321314},
{2192, -4, -1, 6, 5, 200252},
{1698, -2, 13, 10, 14, 223008},
{1716, -2, 8, 16, 9, 314032},
{1734, -2, 8, 17, 9, 336788},
{1754, -2, 10, 10, 13, 223008},
{1771, -2, 13, 20, 14, 385941},
{1806, -2, 13, 23, 14, 431453},
{1847, -2, 14, 17, 16, 336788},
{2196, -1, 7, 6, 2, 131984},
{1881, -3, 13, 3, 14, 131984},
{2212, -2, 13, 8, 7, 223008},
{1893, -2, 13, 19, 18, 359544},
{1597, -2, 15, 10, 17, 223008},
{1936, -2, 15, 19, 17, 359544},
{1619, -2, 13, 17, 14, 336788},
{1977, -3, 13, 3, 7, 131984},
{1980, -3, 15, 6, 22, 177496},
{1997, -2, 15, 6, 22, 177496},
{2014, -2, 15, 10, 9, 223008},
{2026, -2, 13, 17, 18, 336788},
{2065, -3, 2, 3, 7, 131984},
{2219, -1, 4, 7, 2, 154740},
{2068, -3, 2, 3, 3, 131984},
{2070, -2, 15, 10, 21, 223008},
{1428, -2, 12, 10, 13, 223008},
{1445, -3, 12, 8, 13, 223008},
{1458, -2, 12, 10, 13, 223008},
{1475, -2, 12, 10, 13, 223008},
{1492, -2, 12, 10, 13, 223008},
{1509, -2, 12, 10, 13, 223008},
{1526, -2, 12, 10, 13, 223008},
{1543, -2, 13, 11, 14, 223008},
{1563, -2, 12, 10, 13, 223008},
{1580, -2, 12, 10, 13, 223008},
{2097, -3, 8, 3, 9, 131984},
{2101, -3, 8, 3, 13, 131984},
{2267, -3, 12, 15, 16, 336788},
{2106, -2, 8, 17, 8, 336788},
{2297, -3, 12, 15, 16, 336788},
{1649, -2, 13, 9, 14, 211630},
{2123, -2, 13, 17, 14, 336788},
{0, -2, 13, 16, 14, 321314},
{28, -2, 13, 15, 14, 306295},
{55, -2, 13, 16, 14, 314032},
{83, -2, 13, 17, 14, 329051},
{113, -2, 13, 15, 14, 294917},
{140, -2, 13, 14, 14, 283539},
{165, -2, 13, 17, 14, 338609},
{195, -2, 13, 16, 14, 321314},
{223, -2, 13, 6, 14, 162022},
{234, -3, 13, 9, 14, 226649},
{250, -2, 13, 17, 14, 332692},
{280, -2, 13, 13, 14, 272161},
{303, -2, 13, 21, 14, 389582},
{340, -2, 13, 16, 14, 321314},
{368, -2, 13, 17, 14, 336788},
{398, -2, 13, 15, 14, 294917},
{425, -2, 13, 17, 18, 336788},
{464, -2, 13, 17, 14, 317673},
{494, -2, 13, 12, 14, 245764},
{515, -2, 13, 16, 14, 314032},
{543, -2, 13, 16, 14, 321314},
{571, -2, 13, 16, 14, 321314},
{599, -2, 13, 24, 14, 435094},
{641, -2, 13, 16, 14, 321314},
{669, -2, 13, 16, 14, 321314},
{697, -2, 13, 13, 14, 268520},
{2153, -3, 15, 4, 21, 131984},
{2221, -4, 13, 8, 7, 223008},
{2164, -1, 15, 4, 21, 131984},
{2198, -4, 13, 6, 3, 223008},
{2201, -2, 14, 4, 3, 131984},
{2175, -3, 13, 3, 7, 131984},
{720, -2, 8, 11, 9, 223008},
{733, -2, 13, 12, 14, 245764},
{754, -2, 8, 9, 9, 200252},
{765, -2, 13, 12, 14, 245764},
{786, -2, 8, 9, 9, 200252},
{797, -1, 13, 8, 14, 143362},
{811, -2, 8, 11, 14, 223008},
{831, -2, 13, 12, 14, 245764},
{852, -1, 14, 6, 15, 131984},
{864, 0, 14, 6, 19, 143362},
{879, -2, 13, 12, 14, 234386},
{900, -1, 13, 6, 14, 131984},
{911, -2, 8, 20, 9, 359544},
{934, -2, 8, 12, 9, 245764},
{948, -2, 8, 10, 9, 223008},
{960, -2, 8, 12, 13, 245764},
{980, -2, 8, 12, 13, 234386},
{1000, -2, 8, 8, 9, 177496},
{1009, -2, 8, 7, 9, 179772},
{1017, -1, 12, 8, 13, 177496},
{1030, -2, 8, 12, 9, 245764},
{1044, -2, 8, 11, 9, 234386},
{1057, -2, 8, 16, 9, 314032},
{1075, -2, 8, 11, 9, 234386},
{1088, -2, 8, 11, 13, 234386},
{1106, -2, 8, 9, 9, 200252},
{2228, -1, 5, 12, 1, 223008},
{2230, -2, 5, 24, 1, 446016},
{2203, -4, 13, 7, 3, 223008},
{2206, -3, 13, 8, 3, 223008},
{2209, -3, 14, 8, 3, 223008},
};
char cmr5_bitmap[2329] = {
1,128,1,128,2,192,2,192,2,192,4,96,4,96,8,48,
8,48,31,248,16,24,16,24,48,12,248,63,255,224,96,48,
192,49,128,99,0,198,3,15,252,24,14,48,14,96,12,192,
25,128,115,1,223,254,0,3,241,12,15,48,3,96,1,96,
1,192,0,192,0,192,0,192,0,96,1,96,1,48,2,12,
12,3,240,255,240,24,14,12,1,134,0,99,0,17,128,12,
192,6,96,3,48,1,152,0,140,0,198,0,195,1,199,255,
128,255,252,96,24,192,17,128,19,4,38,8,15,240,24,32,
48,66,96,4,192,25,128,35,0,223,255,128,255,248,192,99,
0,140,1,48,68,193,3,252,12,16,48,64,192,3,0,12,
0,48,3,248,0,7,226,14,15,12,1,140,0,68,0,38,
0,3,0,1,128,127,192,6,32,3,24,1,134,0,193,192,
96,63,208,252,63,48,12,48,12,48,12,48,12,48,12,63,
252,48,12,48,12,48,12,48,12,48,12,48,12,252,63,252,
195,12,48,195,12,48,195,12,51,240,63,131,1,128,192,96,
48,24,12,6,3,1,152,200,67,192,252,31,24,6,12,6,
6,4,3,4,1,140,0,203,0,105,128,56,96,24,24,12,
6,6,1,131,0,103,224,124,254,1,128,12,0,96,3,0,
24,0,192,6,0,48,9,128,76,6,96,35,3,127,248,248,
0,249,192,7,11,0,88,88,2,194,96,38,19,1,48,140,
17,132,96,140,33,136,97,12,67,8,52,24,65,160,194,6,
6,124,48,252,240,31,56,4,44,4,38,4,38,4,35,4,
33,132,32,196,32,100,32,52,32,52,32,28,32,12,248,4,
7,240,14,14,12,1,140,0,100,0,22,0,15,0,7,128,
3,192,1,176,1,152,0,198,0,193,193,192,63,128,255,240,
96,56,192,25,128,51,0,102,0,204,7,31,248,48,0,96,
0,192,1,128,3,0,31,128,0,7,240,14,14,12,1,140,
0,102,0,54,0,15,0,7,128,3,192,1,160,0,152,0,
198,56,193,227,192,63,132,0,66,0,51,0,15,0,7,0,
255,224,24,12,12,3,6,1,131,0,193,128,192,255,128,96,
96,48,24,24,12,12,6,6,3,19,1,143,224,120,31,166,
6,128,40,2,128,7,0,63,128,60,0,32,1,128,24,1,
192,107,248,255,255,193,131,129,129,129,129,129,129,1,128,1,
128,1,128,1,128,1,128,1,128,1,128,1,128,15,240,252,
31,48,4,48,4,48,4,48,4,48,4,48,4,48,4,48,
4,48,4,48,4,24,8,12,16,3,224,252,31,48,4,48,
4,24,8,24,8,12,16,12,16,14,48,6,32,7,96,3,
64,3,64,1,128,1,128,252,62,31,48,28,6,48,28,4,
48,28,4,24,38,8,24,38,8,12,67,16,12,67,16,12,
67,16,6,129,160,6,129,160,7,129,224,3,0,192,3,0,
192,252,62,24,24,28,48,14,32,6,64,3,128,1,128,3,
192,2,224,4,96,8,48,24,24,48,28,248,63,252,31,48,
4,24,8,28,24,14,16,6,32,3,64,3,192,1,128,1,
128,1,128,1,128,1,128,7,224,127,251,129,152,24,129,196,
12,0,192,12,0,192,12,8,192,78,2,96,54,3,255,252,
60,12,64,12,15,134,49,134,48,206,25,63,192,240,3,0,
48,3,0,48,3,120,56,67,2,48,51,3,48,51,2,56,
66,120,31,16,208,24,12,6,1,0,65,31,0,3,192,12,
0,192,12,0,193,236,33,196,12,192,204,12,192,196,12,33,
193,239,30,48,144,127,252,6,1,0,65,31,0,6,11,24,
24,24,254,24,24,24,24,24,24,24,126,29,228,69,140,49,
130,32,120,24,1,248,63,136,27,3,96,102,24,126,0,240,
3,0,48,3,0,48,3,120,56,195,12,48,195,12,48,195,
12,48,207,223,33,194,0,0,15,12,48,195,12,48,207,192,
8,112,128,0,3,195,12,48,195,12,48,195,15,39,0,240,
3,0,48,3,0,48,3,30,49,131,48,54,3,240,51,3,
24,48,207,159,240,195,12,48,195,12,48,195,12,51,240,247,
207,131,135,12,48,96,195,6,12,48,96,195,6,12,48,96,
195,6,12,253,251,240,247,131,140,48,195,12,48,195,12,48,
195,12,253,240,30,24,100,11,3,192,240,52,9,134,30,0,
247,131,132,48,99,3,48,51,3,48,99,140,55,131,0,48,
3,0,252,0,30,67,28,96,204,12,192,204,12,96,194,28,
30,192,12,0,192,12,3,240,254,51,48,48,48,48,48,48,
252,125,10,23,131,192,224,225,188,8,8,24,56,254,24,24,
24,24,25,25,25,14,243,195,12,48,195,12,48,195,12,48,
195,28,30,240,249,236,24,194,24,65,144,50,3,128,112,4,
0,251,231,97,198,49,196,50,100,50,100,30,104,28,56,28,
56,8,16,249,230,48,108,7,0,224,30,4,193,12,243,224,
249,236,24,194,24,65,144,50,3,128,112,4,0,128,32,108,
7,0,255,225,161,145,129,192,196,194,195,255,128,255,241,129,
140,4,96,19,0,152,0,192,6,0,48,1,128,12,0,96,
3,0,127,0,0,64,0,28,0,7,192,0,152,0,33,128,
12,56,1,3,0,64,112,8,6,2,0,96,128,14,16,0,
199,255,253,255,255,192,7,240,14,14,12,1,140,0,102,0,
54,64,79,63,231,159,243,200,9,160,0,152,0,198,0,193,
193,192,63,128,1,0,7,0,14,0,28,0,76,0,152,2,
56,4,48,8,96,32,96,64,193,0,194,1,159,15,192,255,
255,255,248,0,96,1,0,0,64,129,254,7,248,16,32,0,
8,0,96,1,255,255,255,240,255,255,48,12,48,12,48,12,
48,12,48,12,48,12,48,12,48,12,48,12,48,12,48,12,
48,12,252,63,255,254,96,6,48,2,24,1,28,1,14,0,
6,0,2,0,4,0,8,1,16,1,32,2,64,6,255,254,
60,30,127,31,224,200,32,44,0,12,0,6,0,3,0,1,
128,0,192,0,96,0,48,0,24,0,12,0,63,192,15,240,
1,128,1,128,15,240,49,140,97,134,193,131,193,131,97,134,
49,140,15,240,1,128,1,128,15,240,7,248,0,96,0,48,
28,24,115,12,97,134,48,195,24,97,140,24,204,6,108,0,
252,0,24,0,12,0,63,192,15,240,56,28,96,6,192,3,
192,3,192,3,192,3,96,6,48,12,16,8,136,17,140,49,
124,62,124,62,30,24,100,11,3,192,240,60,15,3,192,240,
52,9,134,30,0,24,248,24,24,24,24,24,24,24,24,24,
24,255,62,16,104,15,3,0,192,48,24,12,4,2,19,5,
254,255,128,62,16,102,24,6,3,7,128,24,3,0,240,60,
13,6,62,0,3,1,192,112,44,19,8,198,51,12,255,192,
192,48,12,31,192,32,143,195,192,128,32,15,194,24,3,0,
240,56,13,6,62,0,15,4,98,25,128,192,55,142,27,3,
192,240,54,12,134,30,0,64,15,253,255,64,72,16,4,0,
128,32,4,1,128,48,6,0,192,24,0,30,8,198,25,134,
113,15,131,241,142,192,240,60,13,134,63,0,30,24,76,11,
3,192,240,54,28,123,0,192,38,25,140,60,0,4,1,1,
241,18,132,97,56,65,208,63,3,224,79,17,196,97,20,88,
248,4,0,7,0,4,64,6,32,3,16,1,144,0,208,124,
48,24,56,8,46,8,35,8,48,200,24,56,22,30,25,241,
248,126,64,240,96,96,96,96,32,16,0,0,0,1,1,192,
64,8,14,2,0,0,0,0,32,16,24,24,24,24,60,9,
248,240,195,12,48,195,12,252,60,48,195,12,48,195,12,48,
242,112,15,4,35,12,195,49,189,195,16,198,48,204,51,12,
203,51,188,192,62,60,97,198,1,131,31,255,113,128,193,128,
193,128,225,65,62,62,30,30,48,241,144,48,120,31,252,12,
6,6,1,3,0,195,65,30,31,0,0,128,97,241,142,66,
177,60,207,35,80,156,99,225,128,64,0,7,255,224,22,6,
3,96,32,38,1,2,96,16,70,16,4,97,0,255,240,8,
97,17,134,17,16,96,49,6,2,48,96,111,159,254,7,255,
252,56,56,24,192,112,19,0,224,22,1,193,56,1,130,48,
3,252,96,6,8,192,12,18,192,56,5,128,112,25,128,224,
33,193,192,192,255,255,128,0,2,3,250,7,7,6,1,198,
1,50,1,11,1,7,129,131,193,129,224,128,208,128,76,128,
99,128,96,224,224,95,192,64,0,95,254,146,0,46,128,93,
0,18,95,254,128,0,195,0,24,96,3,12,0,97,128,24,
96,255,255,191,255,248,24,96,3,12,0,195,0,24,96,127,
255,247,255,252,24,96,6,24,0,195,0,24,96,3,12,0,
56,3,12,128,227,15,248,97,6,12,33,129,132,48,48,140,
3,35,0,56,192,0,24,112,6,25,1,134,16,48,194,12,
24,67,3,8,192,50,8,3,128,93,146,160,4,33,12,33,
132,48,195,12,48,195,12,16,96,131,4,8,16,129,2,12,
16,96,131,12,48,195,12,48,194,24,67,8,66,0,12,3,
14,221,254,30,31,238,220,48,12,0,0,192,0,96,0,48,
0,24,0,12,0,6,0,3,0,1,128,255,255,255,255,192,
48,0,24,0,12,0,6,0,3,0,1,128,0,192,0,96,
0,93,146,160,93,0,0,64,48,24,6,1,128,192,48,24,
6,3,0,192,48,24,6,3,0,192,96,24,6,3,0,192,
0,93,0,23,64,93,0,23,100,168,127,255,127,255,192,0,
0,0,0,0,0,0,3,255,254,255,254,7,240,12,6,8,
0,136,120,36,99,148,96,198,48,99,24,49,140,24,163,28,
144,243,132,0,1,128,56,63,224,255,204,204,204,204,204,204,
204,204,207,240,255,51,51,51,51,51,51,51,51,63,240,42,
77,208,206,113,55,232,204,192,129,129,66,60,255,122,23,128,
56,48,255,128,31,128,49,232,64,111,96,102,207,48,49,126,
140,102,231,102,66,231,99,33,33,66,132,255,252,33,66,132,
132,198,231,66,255,240,255,255,255,24,24,24,24,24,24,24,
24,24,24,24,24,24,24,153,126,60,60,126,153,24,24,24,
24,24,24,24,24,24,24,24,24,24,24,0,6,0,60,1,
224,15,0,56,1,192,15,0,120,0,240,0,120,0,56,0,
28,0,30,0,15,0,7,128,3,192,1,224,0,240,0,120,
0,56,0,28,0,30,0,15,0,30,0,240,3,128,28,0,
240,7,128,60,0,96,0,255,240,
};
METRIC cmr7_metric[128] = {
{2029, -1, 19, 18, 20, 324041},
{2074, -2, 19, 22, 20, 430537},
{2129, -2, 19, 20, 20, 402320},
{2179, -1, 19, 20, 20, 358630},
{2229, -1, 19, 19, 20, 345886},
{2277, -1, 19, 22, 20, 386847},
{2332, -2, 19, 19, 20, 374103},
{2380, -2, 19, 20, 20, 402320},
{2430, -2, 19, 19, 20, 374103},
{2478, -2, 19, 20, 20, 402320},
{2528, -1, 19, 21, 20, 374103},
{3923, 0, 19, 21, 20, 310843},
{3976, 0, 19, 17, 20, 296734},
{4019, 0, 19, 17, 20, 296734},
{4062, 0, 19, 26, 20, 445101},
{4127, 0, 19, 26, 20, 445101},
{3033, -1, 12, 6, 13, 148367},
{3043, 2, 12, 8, 19, 162476},
{3870, -4, 19, 5, 5, 261235},
{3874, -7, 19, 5, 5, 261235},
{3878, -4, 17, 8, 3, 261235},
{3881, -3, 19, 10, 5, 261235},
{3888, -2, 16, 12, 1, 261235},
{3890, -9, 19, 6, 5, 386847},
{3894, -4, -1, 8, 6, 233018},
{3062, -1, 19, 14, 20, 261236},
{3097, -2, 12, 20, 13, 374103},
{3130, -1, 12, 22, 13, 402320},
{3166, -1, 14, 14, 18, 261235},
{3198, -1, 19, 26, 20, 465126},
{3263, -2, 19, 29, 20, 521560},
{3336, -2, 20, 20, 22, 402320},
{3900, -1, 11, 7, 3, 148367},
{3391, -3, 19, 3, 20, 148367},
{4192, -1, 19, 9, 9, 261235},
{3407, -2, 19, 22, 26, 430537},
{2887, -2, 21, 12, 24, 261235},
{3479, -2, 21, 22, 24, 430537},
{2923, -1, 19, 21, 20, 402320},
{3545, -3, 19, 3, 9, 148367},
{3549, -2, 21, 8, 30, 204801},
{3579, -2, 21, 8, 30, 204801},
{3609, -2, 21, 12, 13, 261235},
{3629, -2, 17, 20, 22, 402320},
{3684, -3, 2, 3, 9, 148367},
{4203, 0, 6, 8, 2, 176584},
{3688, -3, 2, 3, 3, 148367},
{3690, -2, 21, 12, 29, 261235},
{2581, -2, 18, 12, 19, 261235},
{2610, -3, 18, 10, 19, 261235},
{2634, -2, 18, 12, 19, 261235},
{2663, -1, 18, 14, 19, 261235},
{2697, -1, 18, 14, 19, 261235},
{2731, -2, 18, 12, 19, 261235},
{2760, -2, 18, 12, 19, 261235},
{2789, -2, 19, 14, 20, 261235},
{2824, -1, 18, 14, 19, 261235},
{2858, -2, 18, 12, 19, 261235},
{3734, -3, 12, 3, 13, 148367},
{3739, -3, 12, 3, 19, 148367},
{3399, -3, 14, 3, 21, 148367},
{3747, -2, 11, 20, 10, 402320},
{3004, -2, 14, 11, 21, 247127},
{2976, -2, 19, 11, 20, 247127},
{3772, -2, 19, 20, 20, 402320},
{0, -1, 19, 22, 20, 386847},
{55, -1, 19, 19, 20, 366367},
{103, -2, 19, 19, 20, 374103},
{151, -1, 19, 21, 20, 394584},
{204, -1, 19, 19, 20, 352258},
{252, -1, 19, 18, 20, 338150},
{297, -2, 19, 21, 20, 405506},
{350, -1, 19, 22, 20, 386847},
{405, -1, 19, 9, 20, 189328},
{428, -1, 19, 13, 20, 267607},
{461, -1, 19, 22, 20, 400956},
{516, -1, 19, 18, 20, 324041},
{561, -1, 19, 27, 20, 471498},
{629, -1, 19, 22, 20, 386847},
{684, -2, 19, 20, 20, 402320},
{734, -1, 19, 18, 20, 352258},
{779, -2, 19, 20, 26, 402320},
{844, -1, 19, 22, 20, 380475},
{899, -2, 19, 13, 20, 289452},
{932, -1, 19, 21, 20, 374103},
{985, -1, 19, 22, 20, 386847},
{1040, -1, 19, 22, 20, 386847},
{1095, -1, 19, 30, 20, 527932},
{1170, -1, 19, 22, 20, 386847},
{1225, -1, 19, 22, 20, 386847},
{1280, -2, 19, 15, 20, 317669},
{3822, -2, 21, 6, 29, 148367},
{4205, -6, 19, 9, 9, 261235},
{3844, 0, 21, 6, 29, 148367},
{3903, -4, 19, 8, 4, 261235},
{3907, -2, 20, 4, 3, 148367},
{3866, -3, 19, 3, 9, 148367},
{1318, -2, 12, 14, 13, 261235},
{1341, -1, 19, 15, 20, 289452},
{1379, -1, 12, 12, 13, 233018},
{1399, -1, 19, 15, 20, 289452},
{1437, -1, 12, 12, 13, 233018},
{1457, 0, 19, 11, 20, 162476},
{1485, -1, 12, 15, 20, 261235},
{1523, -1, 19, 15, 20, 289452},
{1561, -1, 20, 6, 21, 148367},
{1577, 2, 20, 8, 27, 162476},
{1604, -1, 19, 14, 20, 275344},
{1639, -1, 19, 6, 20, 148367},
{1654, -1, 12, 24, 13, 430537},
{1693, -1, 12, 15, 13, 289452},
{1718, -1, 12, 14, 13, 261235},
{1741, -1, 12, 15, 19, 289452},
{1777, -1, 12, 15, 19, 275343},
{1813, -1, 12, 11, 13, 204801},
{1831, -1, 12, 10, 13, 207623},
{1848, -1, 17, 9, 18, 204801},
{1869, -1, 12, 15, 13, 289452},
{1894, -1, 12, 14, 13, 275344},
{1917, -1, 12, 21, 13, 374103},
{1952, -1, 12, 14, 13, 275344},
{1975, -1, 12, 14, 19, 275344},
{2009, -1, 12, 12, 13, 233018},
{4216, 0, 8, 16, 1, 261235},
{4218, 0, 8, 32, 1, 522470},
{3909, -4, 19, 9, 5, 261235},
{3915, -3, 19, 10, 3, 261235},
{3919, -3, 20, 10, 3, 261235},
};
char cmr7_bitmap[4222] = {
0,48,0,0,192,0,7,128,0,30,0,0,120,0,2,112,
0,9,192,0,71,128,1,14,0,4,56,0,32,112,0,129,
192,4,3,128,31,254,0,64,56,2,0,112,8,1,192,64,
3,131,128,14,63,129,255,255,252,3,128,224,112,14,14,0,
225,192,28,56,3,135,0,112,224,28,28,7,3,255,192,112,
14,14,0,225,192,14,56,1,199,0,56,224,7,28,0,227,
128,56,112,14,127,255,0,0,252,32,224,108,48,3,140,0,
51,0,6,224,0,88,0,15,0,0,224,0,28,0,3,128,
0,112,0,14,0,0,192,0,92,0,9,128,1,24,0,65,
128,16,28,12,0,126,0,255,252,0,224,56,7,0,112,56,
1,193,192,6,14,0,56,112,0,195,128,7,28,0,56,224,
1,199,0,14,56,0,113,192,3,142,0,24,112,1,195,128,
12,28,0,224,224,14,7,0,225,255,248,0,255,255,131,128,
112,112,2,14,0,65,192,4,56,0,135,2,16,224,64,28,
24,3,255,0,112,96,14,4,1,192,130,56,0,71,0,8,
224,3,28,0,67,128,24,112,7,127,255,224,255,255,135,0,
225,192,8,112,2,28,0,71,0,17,193,4,112,64,28,48,
7,252,1,195,0,112,64,28,16,7,0,1,192,0,112,0,
28,0,7,0,1,192,3,255,0,0,252,32,56,27,3,0,
56,48,0,195,0,6,56,0,17,128,0,156,0,0,224,0,
7,0,0,56,0,1,192,31,254,0,14,48,0,113,192,3,
134,0,28,24,0,224,96,7,1,192,216,1,248,64,255,135,
252,112,3,129,192,14,7,0,56,28,0,224,112,3,129,192,
14,7,0,56,28,0,224,127,255,129,192,14,7,0,56,28,
0,224,112,3,129,192,14,7,0,56,28,0,224,112,3,129,
192,14,63,225,255,255,142,7,3,129,192,224,112,56,28,14,
7,3,129,192,224,112,56,28,14,7,31,240,15,248,7,0,
56,1,192,14,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,3,140,28,240,231,134,24,96,126,0,255,131,248,
112,3,129,192,8,7,0,64,28,2,0,112,48,1,193,128,
7,8,0,28,112,0,114,224,1,211,128,7,135,0,28,14,
0,112,60,1,192,112,7,0,224,28,1,192,112,7,129,192,
14,63,224,255,255,192,7,0,1,192,0,112,0,28,0,7,
0,1,192,0,112,0,28,0,7,0,1,192,0,112,0,28,
0,71,0,17,192,4,112,3,28,0,135,0,97,192,59,255,
254,254,0,15,227,192,1,224,92,0,92,11,128,11,129,112,
1,112,39,0,78,4,224,9,192,142,2,56,17,192,71,2,
28,16,224,67,130,28,8,112,67,129,7,16,112,32,226,14,
4,14,129,192,129,208,56,16,28,7,2,3,128,224,224,112,
28,127,4,31,240,252,1,252,120,1,193,112,2,5,224,8,
19,128,32,71,0,129,14,2,4,28,8,16,112,32,64,224,
129,1,194,4,3,136,16,15,32,64,28,129,0,58,4,0,
120,16,0,224,64,3,131,128,6,63,128,8,1,248,0,112,
224,28,3,131,128,28,48,0,199,0,14,96,0,110,0,7,
224,0,126,0,7,224,0,126,0,7,224,0,119,0,14,112,
0,227,0,12,56,1,193,192,56,7,14,0,31,128,255,252,
7,1,193,192,56,112,7,28,1,199,0,113,192,28,112,7,
28,3,135,1,193,255,192,112,0,28,0,7,0,1,192,0,
112,0,28,0,7,0,1,192,3,254,0,1,248,0,112,224,
28,3,131,128,28,48,0,199,0,14,96,0,110,0,7,224,
0,126,0,7,224,0,126,0,7,224,0,118,0,6,112,0,
227,0,12,56,241,193,209,184,7,14,0,31,193,0,12,16,
0,193,0,6,48,0,126,0,3,224,0,28,255,248,0,112,
28,1,192,56,7,0,112,28,1,192,112,7,1,192,28,7,
0,224,28,7,0,127,224,1,192,224,7,1,192,28,3,128,
112,14,1,192,56,7,0,224,28,3,132,112,14,17,192,28,
255,224,62,31,17,135,152,13,128,108,1,96,11,128,14,0,
127,1,254,3,252,1,224,3,128,14,0,112,3,128,30,1,
188,25,31,128,127,255,243,3,129,144,28,4,128,224,40,7,
0,192,56,6,1,192,32,14,0,0,112,0,3,128,0,28,
0,0,224,0,7,0,0,56,0,1,192,0,14,0,0,112,
0,3,128,0,28,0,15,254,0,255,129,252,112,1,193,192,
2,7,0,8,28,0,32,112,0,129,192,2,7,0,8,28,
0,32,112,0,129,192,2,7,0,8,28,0,32,112,0,129,
192,2,3,0,8,14,0,64,24,2,0,56,48,0,63,0,
255,0,252,240,0,193,192,2,7,0,8,14,0,64,56,1,
0,112,8,1,192,32,7,129,128,14,4,0,56,16,0,112,
128,1,194,0,3,144,0,14,64,0,63,0,0,120,0,1,
224,0,3,0,0,12,0,255,31,240,252,224,14,0,195,128,
56,3,7,0,240,8,28,3,192,32,120,15,1,128,224,78,
4,3,129,56,16,7,4,224,128,28,33,194,0,112,135,8,
0,226,28,64,3,144,57,0,14,64,228,0,31,3,224,0,
120,7,128,1,224,30,0,3,0,48,0,12,0,192,0,48,
3,0,255,131,248,120,7,0,224,24,1,192,64,7,130,0,
15,16,0,28,192,0,58,0,0,240,0,1,224,0,7,128,
0,23,0,0,158,0,4,60,0,48,112,0,128,224,4,3,
192,32,7,1,192,30,63,128,255,255,0,252,240,0,193,224,
2,3,128,16,7,0,64,30,2,0,60,16,0,112,64,1,
226,0,3,216,0,7,64,0,14,0,0,56,0,0,224,0,
3,128,0,14,0,0,56,0,0,224,0,3,128,0,127,192,
255,255,224,59,0,118,1,200,7,144,14,32,56,0,112,1,
192,7,0,14,0,56,0,224,33,192,71,0,158,3,56,4,
224,25,192,119,255,224,127,3,135,14,4,16,24,0,96,31,
131,134,24,24,192,99,1,156,6,88,121,62,56,240,0,96,
0,192,1,128,3,0,6,0,12,0,25,240,52,16,112,16,
192,49,128,51,0,102,0,204,1,152,3,48,12,112,16,208,
193,31,0,15,225,135,32,118,2,192,12,0,192,12,0,192,
6,0,32,17,130,15,192,0,120,0,48,0,96,0,192,1,
128,3,0,6,7,204,24,120,64,113,128,102,0,204,1,152,
3,48,6,96,12,96,24,64,112,65,96,124,240,15,129,4,
32,38,3,192,31,255,192,12,0,192,6,0,32,17,130,15,
192,3,193,156,99,140,33,128,48,6,7,248,24,3,0,96,
12,1,128,48,6,0,192,24,3,0,96,63,128,15,60,97,
204,129,3,3,6,6,12,12,8,16,24,96,47,0,192,1,
128,1,255,3,255,12,3,48,3,96,6,192,12,192,48,192,
192,126,0,240,0,96,0,192,1,128,3,0,6,0,12,0,
25,240,52,48,112,48,192,97,128,195,1,134,3,12,6,24,
12,48,24,96,48,192,103,227,240,33,194,0,0,0,0,240,
195,12,48,195,12,48,195,12,252,2,7,2,0,0,0,0,
0,15,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
67,226,230,124,240,0,192,3,0,12,0,48,0,192,3,0,
12,62,48,96,193,3,8,12,192,55,0,238,3,24,12,48,
48,96,193,195,3,63,31,240,195,12,48,195,12,48,195,12,
48,195,12,48,195,63,243,225,240,52,50,24,56,28,12,48,
24,12,48,24,12,48,24,12,48,24,12,48,24,12,48,24,
12,48,24,12,48,24,12,48,24,12,252,126,63,243,224,104,
96,224,97,128,195,1,134,3,12,6,24,12,48,24,96,48,
192,97,128,207,199,224,15,192,97,130,1,24,6,192,15,0,
60,0,240,3,192,13,128,102,1,142,28,15,192,243,224,104,
96,224,33,128,99,0,102,0,204,1,152,3,48,6,96,24,
224,33,161,131,62,6,0,12,0,24,0,48,0,96,3,240,
0,15,136,48,144,128,227,0,204,1,152,3,48,6,96,12,
192,24,192,48,128,224,194,192,249,128,3,0,6,0,12,0,
24,0,48,1,248,243,198,156,227,152,35,0,96,12,1,128,
48,6,0,192,24,15,224,62,152,108,11,2,224,31,131,248,
15,128,224,60,15,134,159,0,16,8,4,6,7,7,252,192,
96,48,24,12,6,3,1,132,194,97,17,7,0,240,120,96,
48,192,97,128,195,1,134,3,12,6,24,12,48,24,96,48,
192,224,194,192,249,224,248,124,192,99,1,6,8,24,32,96,
128,196,3,16,6,128,26,0,120,0,192,3,0,248,124,251,
131,129,140,22,8,96,176,65,136,132,12,70,32,98,49,1,
160,144,13,6,128,104,52,1,128,192,12,6,0,96,48,0,
248,124,192,193,134,3,16,12,128,28,0,48,1,224,12,192,
35,129,6,12,12,248,124,248,124,192,99,1,6,8,24,32,
96,128,196,3,16,6,128,26,0,120,0,192,3,0,8,0,
32,57,128,228,3,160,7,0,0,255,252,6,128,200,28,129,
128,48,6,0,193,24,19,131,48,38,6,255,224,255,255,135,
0,225,192,8,112,2,28,0,71,0,17,192,4,112,0,28,
0,7,0,1,192,0,112,0,28,0,7,0,1,192,0,112,
0,28,0,7,0,1,192,3,255,0,0,48,0,0,192,0,
7,128,0,63,0,0,156,0,6,120,0,16,224,0,131,192,
2,7,0,16,30,0,64,56,2,0,112,24,1,224,64,3,
131,0,15,8,0,28,64,0,121,255,255,239,255,255,255,255,
255,1,248,0,112,224,28,3,131,128,28,48,0,199,0,14,
96,0,110,32,71,227,252,126,63,199,227,252,126,32,71,224,
0,118,0,6,112,0,227,0,12,56,1,193,192,56,7,14,
0,31,128,0,96,0,6,0,0,96,0,15,0,0,240,0,
23,128,1,56,0,19,128,2,28,0,33,192,2,28,0,64,
224,4,14,0,128,240,8,7,0,128,112,16,7,129,0,56,
56,3,207,192,255,127,255,207,255,249,0,1,32,0,36,0,
4,0,0,0,0,0,0,0,16,1,3,255,224,127,252,8,
0,128,0,0,0,0,0,0,4,0,1,128,0,48,0,7,
255,255,255,255,240,255,255,252,112,3,129,192,14,7,0,56,
28,0,224,112,3,129,192,14,7,0,56,28,0,224,112,3,
129,192,14,7,0,56,28,0,224,112,3,129,192,14,7,0,
56,28,0,224,112,3,129,192,14,63,225,255,255,255,206,0,
57,224,3,30,0,33,192,6,28,0,67,192,8,60,0,3,
128,0,56,0,7,0,0,64,0,16,2,4,0,65,0,24,
64,3,16,0,196,0,57,255,255,127,255,224,60,3,199,240,
126,199,142,56,25,129,0,216,0,13,128,0,112,0,7,0,
0,112,0,7,0,0,112,0,7,0,0,112,0,7,0,0,
112,0,7,0,0,112,0,7,0,0,112,0,127,240,15,254,
0,28,0,3,128,0,112,0,63,128,57,206,12,56,99,135,
14,224,224,252,28,31,131,131,240,112,119,14,28,97,195,7,
57,192,31,192,0,224,0,28,0,3,128,7,255,0,7,255,
0,7,0,0,112,0,7,0,240,112,243,135,28,56,113,195,
135,28,56,113,195,135,28,56,113,193,135,24,28,115,128,231,
48,7,118,0,31,128,0,112,0,7,0,0,112,0,127,240,
1,252,0,56,56,7,0,112,112,1,195,0,6,56,0,57,
192,1,206,0,14,112,0,113,128,3,14,0,56,48,1,129,
128,12,6,0,194,48,6,48,128,33,198,3,27,240,31,159,
128,252,252,7,224,15,3,12,96,102,6,64,44,3,192,60,
3,192,60,3,192,60,3,192,60,3,64,38,6,96,99,12,
15,0,12,7,14,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,192,48,12,3,15,252,31,6,12,96,111,7,
240,54,3,0,112,7,0,96,12,1,192,24,2,0,64,8,
17,1,32,39,254,255,224,15,192,193,199,3,156,14,112,56,
0,224,3,0,56,15,192,1,192,3,128,6,0,29,128,127,
1,252,6,224,57,129,193,252,0,0,96,3,128,14,0,88,
2,96,25,128,198,2,24,16,96,129,134,6,48,24,255,252,
1,128,6,0,24,0,96,1,128,63,192,96,103,252,127,132,
64,64,4,0,64,4,240,112,196,14,0,96,7,0,126,7,
224,126,6,64,226,28,31,0,7,192,194,16,114,7,96,6,
0,192,12,240,208,206,6,192,44,3,192,60,3,64,54,2,
32,97,12,15,0,64,1,255,247,255,144,4,128,18,0,128,
4,0,32,1,0,4,0,48,0,128,6,0,24,0,224,3,
128,14,0,56,0,224,1,0,15,192,65,194,1,152,6,96,
25,192,99,195,15,216,15,128,63,131,31,24,30,192,31,0,
60,0,240,3,96,24,192,192,252,0,15,3,8,96,68,6,
192,44,3,192,60,3,64,54,7,48,176,243,0,48,2,0,
110,4,224,132,16,62,0,2,0,32,15,131,36,98,44,33,
194,124,39,226,47,32,127,3,252,15,224,46,2,116,35,226,
60,35,130,36,38,34,193,240,2,0,32,3,128,0,50,0,
3,8,0,24,64,0,194,0,6,32,0,49,0,1,144,63,
7,0,224,48,2,3,128,32,38,2,2,56,16,32,193,3,
3,8,24,12,128,192,56,11,0,192,92,27,132,63,7,192,
31,140,27,1,240,62,6,1,128,96,24,2,0,128,16,2,
0,64,0,0,0,0,0,0,128,56,2,0,4,1,192,16,
0,0,0,0,0,0,32,4,0,128,16,4,0,128,32,24,
6,0,192,248,31,1,176,97,240,240,195,12,48,195,12,48,
195,12,252,15,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,67,226,230,124,7,192,96,195,1,140,6,48,24,192,
195,6,60,240,48,96,192,195,1,140,6,48,12,192,51,0,
204,3,48,12,204,99,49,60,120,127,7,142,29,134,224,112,
36,7,1,0,96,16,127,255,56,96,6,6,0,192,96,12,
7,0,192,80,22,24,194,63,7,192,15,129,224,65,24,98,
2,192,152,15,1,192,24,7,0,127,252,1,128,48,6,0,
192,24,1,128,240,2,2,64,68,16,194,15,129,240,0,8,
0,96,253,6,24,32,113,129,108,8,240,99,193,15,8,60,
96,217,6,104,24,225,193,252,8,0,96,1,0,0,1,255,
255,0,11,128,192,4,224,16,1,56,4,0,142,0,128,35,
128,32,8,224,8,4,56,32,1,14,8,0,195,134,0,63,
255,128,8,56,96,4,14,8,65,3,130,16,192,224,4,32,
56,3,8,14,0,132,3,128,35,128,224,59,248,255,254,1,
255,255,224,56,60,7,3,0,224,8,48,7,0,67,0,56,
1,56,1,192,9,128,14,4,92,0,112,32,224,3,131,7,
0,31,248,56,0,224,193,192,7,2,14,0,56,16,184,1,
192,5,192,14,0,39,0,112,3,56,3,128,16,224,28,1,
129,193,224,28,3,255,255,224,0,0,64,31,136,6,7,1,
192,56,56,3,195,0,76,112,12,230,0,134,224,16,126,3,
7,224,32,126,4,7,224,192,126,8,7,97,0,103,48,14,
50,0,195,192,28,28,3,128,224,96,17,248,2,0,0,95,
255,250,73,36,0,11,160,93,0,2,73,37,255,255,244,0,
48,96,0,193,128,3,6,0,24,48,0,96,192,1,131,0,
6,12,0,48,96,127,255,251,255,255,240,24,48,0,96,192,
1,131,0,12,24,0,48,96,0,193,128,255,255,253,255,255,
224,96,192,3,6,0,12,24,0,48,96,0,193,128,6,12,
0,24,48,0,96,192,0,28,0,48,136,1,198,48,14,48,
127,240,193,1,131,4,6,12,16,48,48,65,128,193,6,1,
136,48,2,33,128,7,12,0,0,48,112,1,130,32,12,24,
128,96,193,1,131,4,12,12,16,96,48,65,128,193,12,3,
4,96,6,35,0,8,132,0,28,93,146,82,128,1,2,4,
8,16,48,32,96,96,64,192,192,192,192,192,192,192,192,192,
192,64,96,96,32,48,16,8,4,2,1,128,64,32,16,8,
12,4,6,6,2,3,3,3,3,3,3,3,3,3,3,2,
6,6,4,12,8,16,32,64,128,6,0,96,6,12,99,246,
241,248,6,1,248,246,252,99,6,0,96,6,0,0,96,0,
6,0,0,96,0,6,0,0,96,0,6,0,0,96,0,6,
0,0,96,0,6,0,255,255,255,255,255,0,96,0,6,0,
0,96,0,6,0,0,96,0,6,0,0,96,0,6,0,0,
96,0,6,0,93,146,82,128,93,0,0,16,3,0,96,6,
0,96,12,0,192,12,1,128,24,3,0,48,3,0,96,6,
0,96,12,0,192,12,1,128,24,3,0,48,3,0,96,6,
0,96,12,0,192,0,93,0,0,1,116,93,0,0,1,118,
73,74,0,127,255,239,255,255,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,255,255,247,255,254,1,248,0,96,
96,8,1,1,0,8,32,240,68,48,130,70,7,40,64,49,
140,3,24,192,49,140,3,24,192,49,132,3,20,96,113,67,
11,34,15,28,16,0,0,128,0,6,0,240,31,248,255,252,
48,195,12,48,195,12,48,195,12,48,195,12,48,195,12,48,
195,12,63,252,255,240,195,12,48,195,12,48,195,12,48,195,
12,48,195,12,48,195,12,48,255,252,41,73,55,64,199,28,
48,128,25,221,136,0,195,60,24,128,96,20,9,134,30,0,
255,240,122,24,97,120,28,6,3,6,126,128,2,59,128,24,
60,102,129,111,96,49,152,216,200,72,64,56,95,232,112,97,
184,118,24,1,248,240,48,57,195,3,142,48,12,33,128,96,
12,3,0,96,24,31,255,248,24,6,0,192,48,6,1,128,
48,12,1,128,96,12,3,0,96,24,3,0,192,24,6,0,
192,48,6,1,128,252,63,128,1,248,3,2,3,3,131,1,
193,128,0,192,0,96,1,255,252,24,6,12,3,6,1,131,
0,193,128,96,192,48,96,24,48,12,24,6,12,3,6,1,
143,195,240,1,254,3,7,3,3,131,0,193,128,96,192,48,
96,25,255,252,24,6,12,3,6,1,131,0,193,128,96,192,
48,96,24,48,12,24,6,12,3,6,1,143,195,240,1,248,
252,1,129,96,128,192,240,112,96,56,28,24,6,0,6,1,
128,1,128,96,3,255,255,252,24,6,3,6,1,128,193,128,
96,48,96,24,12,24,6,3,6,1,128,193,128,96,48,96,
24,12,24,6,3,6,1,128,193,128,96,49,248,126,63,1,
248,255,1,129,225,192,192,224,112,96,56,12,24,6,3,6,
1,128,193,128,96,51,255,255,252,24,6,3,6,1,128,193,
128,96,48,96,24,12,24,6,3,6,1,128,193,128,96,48,
96,24,12,24,6,3,6,1,128,193,128,96,49,248,126,63,
65,113,216,100,18,9,5,4,130,130,0,255,255,32,160,144,
80,72,36,19,13,199,65,0,255,255,255,255,255,255,
};
METRIC cmsl10_metric[128] = {
{4034, -2, 27, 25, 28, 409601},
{4122, -2, 28, 30, 29, 546135},
{4231, -4, 28, 28, 30, 509726},
{4336, -1, 28, 26, 29, 455112},
{4431, -2, 27, 28, 28, 436908},
{4529, -2, 27, 31, 28, 491521},
{4638, -2, 27, 29, 28, 473317},
{4740, -6, 28, 27, 29, 509726},
{4838, -4, 27, 25, 28, 473317},
{4926, -6, 27, 27, 28, 509726},
{5021, -3, 28, 27, 29, 473317},
{7931, -1, 28, 29, 29, 382295},
{8037, -1, 28, 21, 29, 364090},
{8114, -1, 28, 22, 29, 364090},
{8194, -1, 28, 33, 29, 546135},
{8314, -1, 28, 34, 29, 546135},
{6050, -1, 17, 9, 18, 182045},
{6071, 4, 17, 15, 26, 200250},
{7842, -9, 28, 6, 8, 327681},
{7848, -13, 28, 7, 7, 327681},
{7855, -9, 25, 10, 4, 327681},
{7860, -9, 28, 12, 7, 327681},
{7871, -7, 24, 14, 1, 327681},
{7873, -17, 29, 7, 7, 529954},
{7880, -3, -2, 11, 8, 291272},
{6120, -1, 28, 21, 29, 327682},
{6197, -3, 17, 27, 18, 473317},
{6258, -2, 17, 30, 18, 509726},
{6326, -2, 21, 19, 26, 327681},
{6388, -1, 27, 37, 28, 591646},
{6518, -5, 28, 38, 30, 664464},
{6661, -4, 29, 28, 32, 509726},
{7891, -3, 15, 10, 4, 182045},
{6773, -4, 29, 9, 30, 182045},
{8438, -4, 28, 14, 12, 327681},
{6837, -3, 28, 31, 37, 546135},
{5745, -3, 30, 19, 33, 327681},
{6981, -6, 30, 27, 33, 546135},
{5824, -3, 29, 29, 31, 509726},
{7093, -7, 28, 6, 12, 182045},
{7102, -4, 30, 14, 42, 254863},
{7176, 0, 30, 14, 42, 254863},
{7250, -5, 30, 17, 18, 327681},
{7289, -4, 23, 26, 28, 509726},
{7380, -3, 3, 5, 12, 182045},
{8459, -1, 9, 12, 2, 218454},
{7388, -4, 3, 4, 4, 182045},
{7390, -1, 30, 22, 41, 327681},
{5119, -4, 27, 17, 29, 327681},
{5181, -4, 27, 13, 28, 327681},
{5227, -2, 27, 19, 28, 327681},
{5294, -3, 27, 18, 29, 327681},
{5360, -2, 27, 18, 28, 327681},
{5423, -3, 27, 18, 29, 327681},
{5489, -4, 27, 17, 29, 327681},
{5551, -5, 27, 18, 29, 327681},
{5617, -3, 27, 18, 29, 327681},
{5683, -4, 27, 17, 29, 327681},
{7503, -4, 17, 7, 18, 182045},
{7519, -3, 17, 8, 26, 182045},
{6807, -3, 20, 8, 30, 182045},
{7545, -3, 15, 28, 12, 509726},
{5992, -1, 20, 16, 29, 309477},
{5937, -6, 28, 15, 29, 309477},
{7587, -4, 28, 28, 29, 509726},
{0, -1, 28, 28, 29, 491521},
{102, -2, 27, 27, 28, 464215},
{197, -4, 28, 28, 30, 473317},
{302, -2, 27, 30, 28, 500624},
{407, -2, 27, 27, 28, 446010},
{502, -2, 27, 26, 28, 427806},
{593, -4, 28, 28, 30, 514277},
{698, -2, 27, 31, 28, 491521},
{807, -1, 27, 17, 28, 236658},
{867, -2, 27, 20, 29, 336783},
{940, -2, 27, 31, 28, 509726},
{1049, -2, 27, 22, 28, 409601},
{1126, -2, 27, 38, 28, 600748},
{1259, -2, 27, 31, 28, 491521},
{1368, -4, 28, 28, 30, 509726},
{1473, -2, 27, 26, 28, 446010},
{1564, -4, 28, 28, 37, 509726},
{1694, -2, 27, 29, 29, 482419},
{1800, -2, 28, 22, 30, 364090},
{1883, -4, 27, 28, 28, 473317},
{1981, -6, 27, 27, 29, 491521},
{2079, -5, 27, 29, 29, 491521},
{2185, -5, 27, 41, 29, 673566},
{2334, -1, 27, 31, 28, 491521},
{2443, -5, 27, 29, 28, 491521},
{2545, -2, 27, 25, 28, 400499},
{7689, -2, 30, 14, 41, 182045},
{8462, -9, 28, 14, 12, 327681},
{7761, 2, 30, 14, 41, 182045},
{7896, -9, 28, 10, 6, 327681},
{7904, -8, 28, 4, 4, 182045},
{7833, -6, 28, 6, 12, 182045},
{2633, -3, 17, 18, 18, 327681},
{2674, -4, 28, 18, 29, 364090},
{2740, -3, 17, 15, 18, 291272},
{2774, -3, 28, 20, 29, 364090},
{2847, -2, 17, 16, 18, 291272},
{2883, -1, 28, 18, 29, 200250},
{2949, 0, 18, 22, 28, 327681},
{3026, -1, 28, 20, 29, 364090},
{3099, -1, 28, 10, 29, 182045},
{3136, 4, 28, 16, 37, 200250},
{3210, -1, 28, 21, 29, 345886},
{3287, -1, 28, 11, 29, 182045},
{3327, -1, 17, 33, 18, 546135},
{3402, -1, 17, 20, 18, 364090},
{3447, -2, 17, 19, 18, 327681},
{3490, 0, 17, 22, 26, 364090},
{3562, -3, 17, 18, 26, 345885},
{3621, -1, 17, 16, 18, 256684},
{3657, -1, 17, 15, 18, 258504},
{3691, -4, 25, 11, 26, 254863},
{3727, -4, 17, 17, 18, 364090},
{3766, -4, 17, 19, 18, 345886},
{3809, -4, 17, 27, 18, 473317},
{3870, -1, 17, 22, 18, 345886},
{3920, 0, 17, 23, 26, 345886},
{3995, -1, 17, 17, 18, 291272},
{8483, -2, 11, 20, 1, 327681},
{8486, -2, 11, 41, 1, 655362},
{7906, -9, 28, 13, 7, 327681},
{7918, -7, 27, 14, 4, 327681},
{7925, -9, 28, 11, 4, 327681},
};
char cmsl10_bitmap[8492] = {
0,0,32,0,0,3,0,0,0,112,0,0,7,0,0,0,
240,0,0,15,0,0,1,240,0,0,55,128,0,2,120,0,
0,103,128,0,4,120,0,0,199,128,0,8,124,0,1,131,
192,0,16,60,0,2,3,192,0,32,60,0,4,3,192,0,
64,62,0,15,255,224,0,128,30,0,16,1,224,1,0,30,
0,32,1,224,2,0,15,0,64,0,240,4,0,15,1,224,
1,240,255,128,255,240,15,255,252,0,31,0,224,3,192,14,
0,120,1,224,15,0,28,1,224,3,192,60,0,120,15,0,
14,1,224,3,192,60,0,120,7,128,30,0,240,7,128,30,
3,192,7,255,240,0,240,7,128,30,0,120,3,192,7,128,
120,0,240,15,0,30,3,192,3,192,120,0,120,15,0,15,
1,224,1,224,60,0,120,7,128,30,1,224,7,128,60,1,
224,127,255,240,0,0,7,240,16,1,192,195,0,112,2,96,
12,0,30,3,128,0,224,112,0,14,14,0,0,96,224,0,
6,28,0,0,67,192,0,4,56,0,0,71,128,0,0,120,
0,0,7,128,0,0,240,0,0,15,0,0,0,240,0,0,
15,0,0,0,240,0,0,15,0,0,8,240,0,1,7,0,
0,16,112,0,1,3,128,0,32,56,0,4,1,192,0,64,
12,0,24,0,96,2,0,3,128,192,0,7,240,0,15,255,
252,0,3,224,30,0,15,0,28,0,60,0,56,0,240,0,
112,3,192,1,192,15,0,7,128,120,0,14,1,224,0,56,
7,128,0,224,30,0,3,128,120,0,15,1,224,0,56,15,
0,0,224,60,0,7,128,240,0,30,3,192,0,120,15,0,
1,192,60,0,15,1,224,0,56,7,128,0,224,30,0,7,
0,120,0,56,1,224,1,224,7,128,14,0,60,0,112,0,
240,7,0,63,255,240,0,15,255,255,224,31,0,60,3,192,
3,128,120,0,32,15,0,4,1,224,0,128,60,0,16,15,
0,2,1,224,64,64,60,8,0,7,129,0,0,240,96,0,
30,28,0,7,255,0,0,240,96,0,30,4,0,3,192,128,
0,120,16,0,15,2,2,3,192,0,64,120,0,8,15,0,
2,1,224,0,64,60,0,24,7,128,6,1,224,1,192,60,
0,248,127,255,254,0,15,255,255,192,62,0,240,15,0,28,
3,192,2,0,240,0,128,60,0,32,15,0,8,7,128,2,
1,224,64,128,120,16,0,30,4,0,7,131,0,1,225,192,
0,255,224,0,60,24,0,15,2,0,3,192,128,0,240,32,
0,60,8,0,30,0,0,7,128,0,1,224,0,0,120,0,
0,30,0,0,7,128,0,3,192,0,0,248,0,3,255,224,
0,0,7,240,16,1,192,195,0,112,2,96,12,0,30,3,
128,0,224,112,0,14,14,0,0,96,224,0,6,28,0,0,
67,192,0,4,56,0,0,71,128,0,0,120,0,0,7,128,
0,0,240,0,0,15,0,0,0,240,0,0,15,0,15,255,
240,0,15,143,0,0,120,240,0,15,7,0,0,240,112,0,
15,3,128,0,240,56,0,15,1,192,1,240,12,0,30,0,
96,6,96,3,129,130,0,7,224,0,15,255,159,254,1,240,
7,192,3,192,15,0,7,128,30,0,15,0,60,0,30,0,
120,0,60,0,240,0,240,3,192,1,224,7,128,3,192,15,
0,7,128,30,0,15,0,60,0,30,0,120,0,127,255,224,
0,240,3,192,1,224,7,128,3,192,15,0,7,128,30,0,
15,0,60,0,60,0,240,0,120,1,224,0,240,3,192,1,
224,7,128,3,192,15,0,7,128,30,0,30,0,120,0,62,
0,248,7,255,159,254,0,15,255,128,124,0,60,0,30,0,
15,0,7,128,3,192,3,192,1,224,0,240,0,120,0,60,
0,30,0,30,0,15,0,7,128,3,192,1,224,0,240,0,
240,0,120,0,60,0,30,0,15,0,7,128,7,128,3,224,
31,255,0,1,255,240,0,248,0,15,0,0,240,0,15,0,
0,240,0,15,0,1,224,0,30,0,1,224,0,30,0,1,
224,0,30,0,3,192,0,60,0,3,192,0,60,0,3,192,
0,60,0,7,128,0,120,7,7,128,248,120,15,135,128,240,
240,14,14,0,129,192,4,56,0,62,0,0,15,255,131,254,
1,240,1,240,3,192,3,128,7,128,4,0,15,0,16,0,
30,0,64,0,60,1,0,0,240,4,0,1,224,16,0,3,
192,128,0,7,130,0,0,15,14,0,0,30,60,0,0,120,
188,0,0,242,120,0,1,232,248,0,3,224,240,0,7,129,
224,0,15,1,224,0,60,3,192,0,120,3,192,0,240,7,
128,1,224,7,128,3,192,15,0,7,128,15,0,30,0,30,
0,62,0,126,7,255,131,255,128,15,255,192,3,224,0,15,
0,0,60,0,0,240,0,3,192,0,15,0,0,120,0,1,
224,0,7,128,0,30,0,0,120,0,1,224,0,15,0,0,
60,0,0,240,0,3,192,0,15,0,0,60,0,65,224,1,
7,128,4,30,0,16,120,0,129,224,2,7,128,24,60,0,
224,240,15,63,255,252,15,248,0,7,252,3,224,0,63,0,
11,192,0,248,0,47,0,5,224,0,188,0,23,128,2,240,
0,158,0,11,192,2,120,0,71,128,19,192,1,30,0,79,
0,4,120,2,60,0,17,224,8,240,0,71,128,67,192,1,
15,1,15,0,8,60,8,120,0,32,240,33,224,0,131,193,
7,128,2,15,4,30,0,8,30,32,120,0,32,121,1,224,
1,1,228,15,0,4,7,160,60,0,16,30,128,240,0,64,
124,3,192,1,0,240,15,0,4,3,128,60,0,48,14,1,
224,1,224,48,7,192,63,224,193,255,224,15,248,7,254,1,
240,1,224,3,240,1,128,5,224,2,0,11,224,4,0,19,
192,8,0,39,128,16,0,135,128,64,1,15,0,128,2,15,
1,0,4,30,2,0,8,30,4,0,16,60,8,0,64,60,
32,0,128,120,64,1,0,120,128,2,0,241,0,4,1,242,
0,8,1,228,0,32,3,240,0,64,3,224,0,128,7,192,
1,0,7,128,2,0,15,0,4,0,14,0,24,0,24,0,
120,0,16,7,252,0,32,0,0,7,240,0,3,129,192,0,
224,14,0,28,0,48,3,128,3,128,112,0,28,14,0,1,
193,224,0,30,28,0,0,227,192,0,14,56,0,0,231,128,
0,15,120,0,0,247,128,0,15,240,0,1,239,0,0,30,
240,0,1,239,0,0,30,240,0,3,207,0,0,60,240,0,
3,199,0,0,120,112,0,7,7,128,0,240,56,0,30,3,
192,1,192,28,0,56,0,224,14,0,3,131,128,0,15,224,
0,15,255,252,0,62,3,192,15,0,56,3,192,15,0,240,
1,192,60,0,112,15,0,28,7,128,15,1,224,3,192,120,
0,240,30,0,56,7,128,28,1,224,14,0,240,15,0,63,
254,0,15,0,0,3,192,0,0,240,0,0,60,0,0,30,
0,0,7,128,0,1,224,0,0,120,0,0,30,0,0,7,
128,0,3,192,0,0,248,0,3,255,192,0,0,7,240,0,
3,129,192,0,224,14,0,28,0,112,3,128,3,128,112,0,
28,14,0,1,193,224,0,30,28,0,0,227,192,0,14,56,
0,0,231,128,0,15,120,0,0,247,128,0,15,240,0,1,
239,0,0,30,240,0,1,239,0,0,30,240,0,1,207,0,
0,60,240,0,3,199,0,0,120,112,0,7,7,129,192,240,
56,34,14,3,132,17,192,28,129,56,0,232,30,0,3,195,
128,128,15,248,8,0,1,192,128,0,28,16,0,1,195,0,
0,31,240,0,0,254,0,0,15,192,0,0,120,0,15,255,
248,0,7,192,112,0,60,1,224,1,224,7,128,15,0,28,
0,120,0,240,3,192,7,128,60,0,120,1,224,3,192,15,
0,28,0,120,1,192,3,192,28,0,30,3,192,1,255,240,
0,15,1,192,0,120,7,0,3,192,60,0,30,0,224,0,
240,7,128,15,0,120,0,120,3,192,3,192,30,0,30,0,
240,0,240,7,128,7,128,60,8,120,1,224,131,224,7,5,
255,224,56,64,0,0,124,0,0,63,4,3,3,48,24,3,
128,192,14,6,0,24,56,0,96,224,1,131,128,6,30,0,
16,120,0,1,240,0,3,224,0,15,248,0,31,252,0,63,
248,0,127,240,0,63,224,0,15,128,0,30,0,0,56,0,
0,225,0,3,132,0,14,16,0,56,64,0,193,128,7,14,
0,24,60,0,192,204,14,2,15,224,0,31,255,255,243,192,
124,15,48,7,128,50,0,120,2,96,7,128,36,0,120,2,
64,7,128,36,0,240,2,128,15,0,32,0,240,0,0,15,
0,0,0,240,0,0,15,0,0,1,224,0,0,30,0,0,
1,224,0,0,30,0,0,1,224,0,0,30,0,0,3,192,
0,0,60,0,0,3,192,0,0,60,0,0,3,192,0,0,
60,0,0,7,128,0,0,124,0,1,255,254,0,255,248,127,
225,240,1,224,60,0,24,7,128,2,0,240,0,64,30,0,
8,3,192,1,0,240,0,64,30,0,8,3,192,1,0,120,
0,32,15,0,4,1,224,0,128,120,0,32,15,0,4,1,
224,0,128,60,0,16,7,128,2,0,240,0,64,60,0,16,
7,128,2,0,240,0,64,14,0,16,1,192,2,0,56,0,
128,3,128,32,0,48,8,0,3,134,0,0,31,0,0,255,
240,15,248,252,0,30,3,192,0,96,30,0,2,0,240,0,
32,3,192,1,0,30,0,16,0,240,0,128,7,128,8,0,
62,0,64,0,240,4,0,7,128,32,0,60,2,0,1,224,
16,0,15,129,0,0,60,8,0,1,224,128,0,15,12,0,
0,120,64,0,3,228,0,0,15,32,0,0,122,0,0,3,
208,0,0,31,0,0,0,248,0,0,3,128,0,0,28,0,
0,0,192,0,0,6,0,0,0,255,240,255,224,255,143,128,
15,128,30,3,192,7,192,6,1,224,1,224,2,0,240,1,
240,1,0,120,0,248,1,0,60,0,188,0,128,30,0,94,
0,192,15,128,79,0,64,3,192,39,192,32,1,224,33,224,
32,0,240,16,240,16,0,120,16,120,16,0,60,8,60,8,
0,30,8,30,8,0,15,132,15,4,0,3,196,7,196,0,
1,226,1,226,0,0,242,0,242,0,0,121,0,121,0,0,
61,128,61,0,0,30,128,30,128,0,15,192,15,128,0,3,
192,7,192,0,1,224,1,192,0,0,224,0,224,0,0,112,
0,96,0,0,48,0,48,0,0,24,0,24,0,0,7,255,
135,254,1,252,3,224,1,240,7,0,1,224,12,0,3,224,
16,0,3,192,64,0,7,129,0,0,15,132,0,0,15,24,
0,0,31,32,0,0,30,128,0,0,62,0,0,0,60,0,
0,0,124,0,0,0,248,0,0,1,248,0,0,4,240,0,
0,17,224,0,0,67,224,0,1,131,192,0,2,7,192,0,
8,7,128,0,32,15,128,0,128,15,0,3,0,31,0,12,
0,30,0,124,0,126,7,254,7,255,128,255,240,7,248,252,
0,30,3,192,0,96,31,0,2,0,120,0,32,3,224,2,
0,15,0,32,0,124,1,0,3,224,16,0,15,1,0,0,
124,16,0,1,224,128,0,15,136,0,0,60,128,0,1,232,
0,0,15,192,0,0,60,0,0,1,224,0,0,14,0,0,
0,240,0,0,7,128,0,0,60,0,0,1,224,0,0,15,
0,0,0,112,0,0,7,128,0,0,62,0,0,31,254,0,
0,3,255,255,131,224,7,129,192,7,128,192,7,128,96,7,
192,32,3,192,16,3,192,16,3,192,8,3,224,0,1,224,
0,1,224,0,1,224,0,1,240,0,0,240,0,0,240,0,
0,240,0,0,248,0,0,120,2,0,120,2,0,120,1,0,
120,0,128,60,0,192,60,0,64,60,0,96,60,0,48,30,
0,120,30,0,248,31,255,252,0,7,240,7,6,1,224,192,
112,56,24,14,0,3,128,0,224,7,248,15,14,7,7,3,
1,193,192,112,224,28,120,7,30,3,199,129,114,112,157,143,
195,128,63,0,3,192,0,224,0,56,0,14,0,3,128,0,
224,0,56,0,28,0,7,0,1,192,0,113,224,29,134,7,
128,195,192,56,224,6,56,1,142,0,115,128,28,224,7,112,
3,156,0,231,0,57,192,12,112,7,28,3,143,0,195,48,
192,131,192,0,1,248,14,56,48,120,192,227,129,134,0,28,
0,56,0,112,1,192,3,128,7,0,6,0,12,0,156,2,
24,8,24,32,31,128,0,3,240,0,15,0,0,224,0,14,
0,0,224,0,14,0,0,224,0,14,0,1,192,0,28,0,
1,192,15,28,3,13,192,192,60,28,3,131,128,56,48,3,
135,0,56,112,3,135,0,56,224,7,14,0,112,224,7,6,
0,112,96,7,7,0,240,48,30,1,134,240,7,143,192,1,
248,7,12,12,6,28,7,56,3,48,3,112,3,127,255,112,
0,224,0,224,0,224,0,96,0,96,2,48,4,48,8,28,
48,7,192,0,15,128,14,112,6,28,3,143,1,193,128,112,
0,28,0,7,0,3,128,0,224,0,56,0,255,224,3,128,
0,224,0,112,0,28,0,7,0,1,192,0,112,0,28,0,
14,0,3,128,0,224,0,56,0,14,0,3,128,1,192,0,
120,0,255,192,0,0,0,56,0,243,48,12,105,192,96,194,
3,131,128,28,14,0,112,56,1,192,224,7,3,128,28,28,
0,48,96,0,227,0,4,240,0,16,0,0,192,0,3,0,
0,15,254,0,31,254,0,255,252,6,0,112,32,0,193,128,
3,140,0,12,48,0,48,96,1,128,128,12,1,128,224,1,
252,0,7,224,0,30,0,1,192,0,28,0,1,192,0,28,
0,1,192,0,28,0,3,128,0,56,0,3,128,0,56,248,
3,176,192,60,14,7,128,224,120,14,7,0,224,112,14,7,
0,224,112,14,14,1,192,224,28,14,1,192,224,28,14,1,
192,224,28,28,3,129,224,60,255,159,240,0,192,112,28,6,
0,0,0,0,0,0,0,0,0,126,7,128,224,112,28,7,
1,192,112,28,14,3,128,224,56,14,3,129,192,120,255,128,
0,3,0,7,0,7,0,6,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,254,0,30,0,14,0,28,0,28,
0,28,0,28,0,28,0,28,0,56,0,56,0,56,0,56,
0,56,0,56,0,112,0,112,0,112,0,112,0,112,0,112,
96,224,240,224,241,192,99,128,62,0,7,224,0,15,0,0,
112,0,3,128,0,28,0,0,224,0,7,0,0,56,0,3,
128,0,28,0,0,224,0,7,15,240,56,30,1,192,192,28,
8,0,224,128,7,8,0,56,128,1,220,0,15,240,0,243,
128,7,30,0,56,112,1,195,192,14,14,0,112,120,7,1,
192,56,31,15,249,254,0,7,224,60,7,0,224,28,3,128,
112,14,3,128,112,14,1,192,56,7,1,192,56,7,0,224,
28,3,128,224,28,3,128,112,14,1,192,112,15,15,248,31,
143,192,252,3,200,48,131,0,248,29,129,192,240,15,0,224,
120,7,128,112,56,3,128,56,28,1,192,28,14,0,224,14,
7,0,112,7,7,0,112,7,3,128,56,3,129,192,28,1,
192,224,14,0,224,112,7,0,112,56,3,128,56,56,3,128,
56,30,1,224,30,127,199,252,127,192,31,143,128,123,12,3,
192,224,120,14,7,128,224,112,14,7,0,224,112,14,7,0,
224,224,28,14,1,192,224,28,14,1,192,224,28,14,1,193,
192,56,30,3,207,249,255,0,252,0,96,224,56,6,14,0,
227,128,12,96,1,156,0,59,128,7,112,0,252,0,59,128,
7,112,0,230,0,24,192,7,28,1,193,192,112,28,24,0,
252,0,15,199,128,15,97,128,30,3,0,240,14,3,128,56,
14,0,96,56,1,192,224,7,3,128,28,28,0,224,112,3,
129,192,14,7,0,112,28,1,192,112,14,3,192,112,14,195,
0,56,240,0,224,0,3,128,0,14,0,0,112,0,1,192,
0,7,0,0,28,0,3,254,0,0,0,240,64,194,48,224,
92,112,14,56,3,142,0,231,0,57,192,14,112,3,184,1,
206,0,115,128,28,96,7,28,1,199,0,240,192,120,24,110,
1,227,128,0,224,0,56,0,14,0,7,0,1,192,0,112,
0,28,0,63,224,31,156,7,238,3,207,7,142,7,140,7,
0,7,0,7,0,7,0,14,0,14,0,14,0,14,0,14,
0,14,0,28,0,30,0,255,192,3,242,24,28,96,24,128,
35,0,71,0,143,128,15,240,15,240,15,240,0,242,0,228,
0,200,1,152,6,112,8,208,97,31,128,2,0,64,8,3,
0,96,24,7,1,224,255,227,128,112,28,3,128,112,14,1,
192,56,14,1,193,56,39,4,224,156,35,132,49,3,192,252,
31,158,3,199,0,231,0,227,128,113,192,56,224,28,112,14,
56,7,56,7,28,3,142,1,199,0,227,128,241,192,120,224,
120,48,94,15,207,192,255,15,231,128,112,112,12,14,1,1,
192,96,56,8,7,130,0,112,64,14,16,1,194,0,56,128,
3,144,0,116,0,15,128,1,224,0,56,0,3,0,0,64,
0,255,63,207,231,129,224,112,224,24,12,28,7,1,1,192,
240,32,56,46,8,7,5,193,0,225,56,64,28,35,8,3,
136,98,0,57,14,64,7,65,208,0,232,58,0,30,7,128,
3,192,112,0,112,12,0,6,1,128,0,128,32,0,31,227,
252,31,7,128,56,24,0,112,64,1,194,0,3,144,0,14,
192,0,30,0,0,112,0,0,224,0,7,128,0,39,0,1,
30,0,8,56,0,64,112,3,1,192,60,7,131,248,127,192,
15,240,254,7,128,112,7,0,192,14,1,0,28,6,0,56,
8,0,120,32,0,112,64,0,225,0,1,194,0,3,136,0,
3,144,0,7,64,0,15,128,0,30,0,0,56,0,0,48,
0,0,64,0,0,128,0,2,0,0,4,0,14,16,0,60,
64,0,120,128,0,230,0,0,240,0,0,15,255,135,3,130,
3,131,1,193,1,192,129,192,65,192,1,192,1,192,0,224,
0,224,128,224,64,224,32,224,32,224,16,112,24,112,56,127,
252,0,15,255,255,128,124,3,192,60,0,224,30,0,32,15,
0,16,7,128,8,3,192,4,3,192,2,1,224,1,0,240,
0,0,120,0,0,60,0,0,30,0,0,30,0,0,15,0,
0,7,128,0,3,192,0,1,224,0,0,240,0,0,240,0,
0,120,0,0,60,0,0,30,0,0,15,0,0,7,128,0,
7,128,0,3,224,0,31,255,0,0,0,0,24,0,0,0,
224,0,0,3,128,0,0,31,0,0,0,252,0,0,2,240,
0,0,19,224,0,0,143,128,0,2,30,0,0,16,124,0,
0,129,240,0,2,3,192,0,16,15,128,0,128,30,0,2,
0,120,0,16,1,240,0,128,3,192,6,0,15,0,16,0,
62,0,128,0,120,6,0,1,224,16,0,7,192,128,0,15,
6,0,0,60,16,0,0,248,128,0,1,231,255,255,255,159,
255,255,255,255,255,255,252,0,7,240,0,3,129,192,0,224,
14,0,28,0,112,3,128,3,128,112,0,28,14,0,1,193,
224,0,30,28,0,0,227,192,0,14,56,0,0,231,128,0,
15,120,128,8,247,143,255,143,241,255,241,239,31,255,30,241,
0,17,239,0,0,30,240,0,1,207,0,0,60,240,0,3,
199,0,0,120,112,0,7,7,128,0,240,56,0,14,3,128,
1,192,28,0,56,0,224,14,0,3,131,128,0,15,224,0,
0,0,64,0,0,24,0,0,14,0,0,3,128,0,1,224,
0,0,120,0,0,62,0,0,15,192,0,5,240,0,1,60,
0,0,143,0,0,35,192,0,16,240,0,4,60,0,2,15,
128,0,129,224,0,64,120,0,16,30,0,8,7,128,2,1,
224,1,0,124,0,64,31,0,32,3,192,8,0,240,4,0,
60,1,0,15,0,192,3,192,120,0,248,255,131,255,192,7,
255,255,240,255,255,254,15,255,255,224,128,0,2,8,0,0,
32,128,0,2,8,0,0,32,0,0,0,0,0,0,0,0,
0,0,1,0,4,0,16,0,64,1,255,252,0,63,255,128,
3,255,248,0,32,0,128,2,0,8,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,4,0,0,16,64,0,1,4,
0,0,16,64,0,3,15,255,255,224,255,255,254,15,255,255,
224,15,255,255,254,1,224,7,192,3,192,15,0,7,128,30,
0,15,0,60,0,30,0,120,0,60,0,240,0,240,3,192,
1,224,7,128,3,192,15,0,7,128,30,0,15,0,60,0,
30,0,120,0,120,1,224,0,240,3,192,1,224,7,128,3,
192,15,0,7,128,30,0,15,0,60,0,60,0,240,0,120,
1,224,0,240,3,192,1,224,7,128,3,192,15,0,7,128,
30,0,30,0,120,0,62,0,248,7,255,159,254,0,7,255,
255,248,62,0,7,192,240,0,14,7,192,0,96,30,0,1,
0,248,0,8,3,192,0,64,31,0,2,0,120,0,16,3,
224,0,0,31,0,0,0,124,0,0,3,224,0,0,15,128,
0,0,120,0,0,1,128,0,0,8,0,0,0,128,0,0,
8,0,8,0,128,0,128,8,0,4,0,128,0,96,24,0,
2,1,0,0,48,16,0,3,129,0,0,124,31,255,255,193,
255,255,254,0,15,128,15,135,248,7,248,255,129,255,48,120,
120,124,7,12,7,0,115,128,96,14,224,8,1,216,0,0,
27,0,0,3,224,0,0,120,0,0,15,0,0,1,224,0,
0,60,0,0,15,0,0,1,224,0,0,60,0,0,7,128,
0,0,240,0,0,30,0,0,7,128,0,0,240,0,0,30,
0,0,3,192,0,0,120,0,0,15,0,0,3,192,0,0,
124,0,1,255,248,0,0,255,254,0,3,224,0,1,224,0,
0,240,0,0,120,0,1,254,0,7,158,224,14,30,60,14,
15,7,14,7,131,142,3,193,239,1,224,247,128,240,127,128,
240,63,192,120,31,224,60,15,240,30,7,56,15,7,158,7,
131,135,7,131,129,195,199,0,57,239,0,7,252,0,0,120,
0,0,60,0,0,60,0,0,31,0,1,255,248,0,1,255,
252,0,1,240,0,0,60,0,0,7,128,0,0,240,1,240,
30,7,207,3,193,225,224,240,120,60,30,15,7,131,195,192,
240,120,120,30,15,15,3,193,225,224,120,120,120,14,15,15,
1,193,225,224,60,60,120,7,135,143,0,112,243,192,14,60,
112,0,231,156,0,14,255,0,0,127,0,0,3,192,0,0,
120,0,0,30,0,0,3,224,0,15,255,192,0,0,15,240,
0,7,3,128,3,128,28,1,224,1,192,120,0,60,30,0,
3,131,128,0,120,240,0,15,30,0,1,231,128,0,60,240,
0,7,158,0,0,243,192,0,28,120,0,7,143,0,0,224,
224,0,60,28,0,7,1,192,1,192,56,0,48,3,0,14,
0,96,3,128,6,0,96,32,192,24,36,8,2,4,129,0,
193,16,32,16,35,252,3,252,127,128,255,15,240,31,224,0,
248,1,199,1,129,129,128,96,192,48,192,28,224,14,112,7,
48,3,184,1,220,0,238,0,119,0,63,0,59,128,29,192,
14,224,7,112,3,184,1,156,1,204,0,230,0,115,0,49,
128,56,192,24,48,24,24,24,6,24,1,240,0,0,24,3,
128,252,24,224,7,0,56,1,192,28,0,224,7,0,56,1,
192,14,0,224,7,0,56,1,192,14,0,112,7,0,56,1,
192,14,0,112,3,128,56,1,225,255,240,0,126,0,48,112,
8,7,2,0,112,64,14,30,1,195,192,56,248,7,31,0,
225,192,28,0,7,128,0,224,0,56,0,7,0,1,192,0,
112,0,28,0,7,0,1,128,0,96,0,24,0,6,1,1,
128,32,64,8,16,1,7,255,225,255,248,127,255,0,0,124,
0,97,192,32,56,16,14,15,3,195,192,240,240,60,56,14,
0,3,128,1,192,0,96,0,48,0,56,0,252,0,1,192,
0,56,0,14,0,3,192,0,240,0,60,112,15,62,3,207,
128,243,192,56,224,30,32,7,4,3,128,193,192,15,192,0,
0,0,192,0,96,0,56,0,14,0,7,128,2,224,1,56,
0,156,0,103,0,49,192,8,112,4,28,2,7,1,3,128,
128,224,96,56,16,14,8,3,132,0,227,255,255,0,28,0,
7,0,1,192,0,112,0,28,0,14,0,3,192,15,254,3,
1,192,255,224,63,224,31,240,4,0,1,0,0,64,0,16,
0,4,0,2,0,0,143,128,44,48,12,14,2,1,192,128,
112,0,28,0,7,128,1,224,0,121,192,60,240,15,60,3,
207,0,226,0,120,128,28,16,14,4,3,0,195,128,15,128,
0,0,62,0,112,128,96,32,96,112,96,120,96,60,112,28,
48,0,56,0,24,0,28,124,14,195,7,128,199,64,115,192,
25,224,12,224,7,112,3,184,1,220,1,204,0,230,0,115,
0,57,128,56,224,24,48,28,24,12,6,24,0,248,0,16,
0,15,255,243,255,252,255,254,96,1,16,0,132,0,66,0,
32,0,8,0,4,0,2,0,1,0,0,192,0,32,0,24,
0,6,0,3,0,1,192,0,112,0,60,0,14,0,3,128,
1,224,0,120,0,30,0,15,0,3,192,0,240,0,24,0,
0,0,124,0,96,192,32,24,24,6,12,0,195,0,48,192,
12,112,6,28,1,135,128,192,248,96,63,48,7,240,0,254,
0,63,192,49,248,24,63,12,3,198,0,113,128,12,192,3,
48,0,204,0,51,0,8,192,6,24,1,6,1,128,224,192,
15,192,0,0,248,1,130,1,129,129,128,97,192,49,192,24,
224,14,96,7,112,3,184,1,220,0,238,0,119,0,123,128,
57,192,60,96,30,24,23,4,19,129,243,128,1,192,0,192,
0,225,128,97,224,112,240,48,112,48,32,48,8,112,3,224,
0,0,8,0,1,0,1,248,0,196,192,33,4,8,32,130,
4,8,192,135,24,17,227,2,60,224,135,28,16,1,194,0,
62,64,7,252,0,127,224,7,254,0,63,224,1,124,0,35,
128,4,48,64,135,28,32,199,132,24,224,131,16,16,66,2,
24,32,66,4,16,128,98,96,3,240,0,8,0,1,0,0,
0,14,0,0,0,200,0,0,12,64,0,0,194,0,0,6,
16,0,0,112,128,0,3,132,0,0,28,64,0,0,194,0,
0,6,32,0,0,114,0,0,3,144,0,0,29,0,255,128,
112,1,224,3,128,6,0,60,0,96,2,224,2,0,39,0,
32,2,60,2,0,48,224,16,3,7,1,0,56,28,16,1,
128,225,0,28,3,136,0,224,28,128,7,0,120,1,56,3,
128,17,192,14,0,135,1,184,8,24,48,225,128,62,1,240,
0,7,240,48,48,128,51,0,127,0,254,1,248,3,128,14,
0,56,0,224,3,0,12,0,16,0,64,1,128,2,0,4,
0,16,0,32,0,64,0,0,0,0,0,0,0,0,0,0,
112,0,224,3,192,3,0,0,0,48,0,120,0,120,0,48,
0,0,0,0,0,0,0,0,0,0,0,64,0,64,0,64,
0,64,0,192,0,128,1,128,1,0,3,0,6,0,28,0,
56,0,112,0,112,14,112,15,224,14,96,6,112,4,48,56,
31,192,31,131,192,224,224,112,56,28,14,7,7,3,129,192,
224,112,56,56,30,127,192,0,254,0,60,0,56,0,224,1,
192,3,128,7,0,14,0,28,0,112,0,224,1,192,3,128,
7,0,14,0,56,0,112,0,224,1,192,3,128,7,12,28,
60,56,120,224,99,128,124,0,0,15,128,1,195,0,24,12,
1,192,112,28,3,128,224,28,7,0,224,56,14,3,128,96,
28,6,0,224,224,63,60,0,56,48,1,192,192,28,7,0,
224,24,7,0,224,56,7,1,192,56,14,1,192,224,14,7,
0,112,56,3,129,192,28,14,1,192,115,14,7,60,224,56,
206,15,195,192,0,7,240,31,3,131,12,48,120,54,3,14,
7,192,49,128,240,6,0,28,0,192,3,128,24,15,255,255,
15,14,0,3,131,128,0,192,112,0,56,14,0,14,1,192,
1,192,60,0,184,7,128,39,1,184,8,112,227,130,3,240,
31,128,0,248,3,224,12,24,48,192,192,51,1,135,0,252,
3,56,1,224,12,192,7,0,55,0,28,0,220,0,127,255,
112,1,192,3,128,14,0,14,0,56,0,56,0,224,0,96,
7,128,1,192,31,0,39,0,236,1,12,7,56,8,28,48,
112,64,31,0,126,0,0,0,32,0,12,0,1,0,0,64,
15,208,6,14,3,128,224,224,46,56,8,198,1,25,192,67,
184,16,119,4,15,192,131,184,32,119,8,14,98,1,140,64,
113,208,28,28,7,1,193,128,47,192,8,0,2,0,0,192,
0,16,0,0,0,3,255,255,248,0,1,124,3,192,0,19,
192,14,0,1,158,0,32,0,8,240,1,0,0,135,128,8,
0,4,60,0,64,0,67,192,2,0,2,30,0,16,0,32,
240,32,0,1,7,129,0,0,16,60,8,0,0,129,224,192,
0,8,30,12,0,0,127,255,224,0,4,7,131,0,0,32,
60,8,0,2,1,224,64,0,16,15,2,8,1,0,240,32,
64,8,7,128,2,0,128,60,0,32,12,1,224,1,0,64,
15,0,24,6,0,120,0,128,32,7,128,12,7,128,60,1,
225,255,15,255,254,0,0,7,224,0,0,0,224,127,255,240,
15,1,240,3,192,112,3,128,7,3,128,14,0,8,28,0,
56,0,32,240,0,224,0,135,128,3,128,2,28,0,28,0,
8,240,0,112,4,35,128,1,192,16,30,0,7,0,64,120,
0,28,3,1,224,0,112,28,15,0,3,255,224,60,0,14,
1,128,240,0,56,2,3,192,0,224,8,15,0,3,128,32,
60,0,14,0,132,240,0,112,0,17,192,1,192,0,71,128,
7,0,2,30,0,28,0,8,56,0,112,0,96,240,3,192,
1,1,224,14,0,28,3,128,120,1,240,3,131,255,255,128,
3,240,0,0,0,0,0,0,32,0,127,4,0,56,28,192,
14,0,104,1,192,3,0,56,0,56,7,0,5,192,224,0,
156,30,0,24,225,192,1,14,60,0,32,227,128,4,14,120,
0,128,247,128,16,15,120,3,0,255,0,32,30,240,4,1,
239,0,128,30,240,16,1,239,3,0,28,240,32,3,207,4,
0,60,112,128,3,135,16,0,112,114,0,15,3,224,0,224,
60,0,28,1,192,3,128,30,0,96,2,56,56,0,64,254,
0,12,0,0,0,3,3,193,224,224,240,120,60,28,14,7,
3,129,128,192,96,48,48,24,12,6,2,1,0,0,0,0,
0,0,28,14,15,3,0,6,15,15,6,0,0,0,0,0,
8,8,8,8,24,24,24,24,56,56,56,48,112,112,112,112,
240,240,240,240,224,0,0,32,16,0,0,192,96,0,1,128,
192,0,3,1,128,0,12,6,0,0,24,12,0,0,48,24,
0,0,192,96,0,1,128,192,0,6,3,0,0,12,6,0,
0,24,12,0,0,96,48,7,255,255,255,159,255,255,255,128,
12,6,0,0,24,12,0,0,48,24,0,0,192,96,0,1,
128,192,0,3,1,128,0,12,6,0,0,24,12,0,127,255,
255,254,127,255,255,248,3,1,128,0,6,3,0,0,24,12,
0,0,48,24,0,0,192,96,0,1,128,192,0,3,1,128,
0,12,6,0,0,24,12,0,0,96,48,0,0,192,96,0,
1,0,128,0,0,7,128,0,97,136,0,28,96,128,7,24,
28,3,199,1,127,176,192,32,12,56,4,1,135,0,128,96,
224,16,24,28,2,6,3,0,129,128,96,16,96,12,4,12,
1,192,131,0,24,32,192,1,8,48,0,30,12,7,128,1,
131,8,0,96,193,0,24,56,16,6,6,2,1,129,192,64,
48,48,8,12,14,1,3,1,192,32,192,56,4,48,7,0,
140,0,224,33,128,28,4,96,1,129,24,0,48,70,0,3,
16,128,0,60,0,24,247,205,4,16,130,16,70,32,0,4,
0,32,3,0,24,0,192,2,0,24,0,192,6,0,24,0,
192,3,0,24,0,96,3,128,12,0,48,1,192,7,0,24,
0,96,1,128,14,0,56,0,224,3,128,14,0,56,0,224,
3,128,6,0,24,0,96,1,128,2,0,12,0,48,0,96,
0,128,3,0,4,0,8,0,1,0,2,0,4,0,24,0,
32,0,192,1,0,6,0,24,0,96,1,192,3,0,12,0,
48,0,192,3,0,12,0,48,0,192,7,0,28,0,112,1,
128,6,0,56,0,224,3,0,12,0,112,1,128,14,0,48,
1,192,6,0,48,0,192,6,0,48,1,128,12,0,96,2,
0,0,0,96,0,48,0,24,0,8,3,4,25,194,60,115,
56,15,240,1,224,0,240,3,172,7,147,135,24,246,12,16,
6,0,2,0,3,0,1,128,0,0,1,0,0,0,96,0,
0,24,0,0,12,0,0,3,0,0,0,192,0,0,48,0,
0,12,0,0,3,0,0,1,128,0,0,96,0,0,24,0,
0,6,0,63,255,255,255,255,255,252,0,48,0,0,12,0,
0,3,0,0,0,192,0,0,48,0,0,12,0,0,6,0,
0,1,128,0,0,96,0,0,24,0,0,6,0,0,1,128,
0,0,64,0,59,222,112,132,66,33,17,0,119,246,0,0,
4,0,0,48,0,1,128,0,6,0,0,48,0,0,192,0,
6,0,0,24,0,0,192,0,3,0,0,24,0,0,192,0,
3,0,0,24,0,0,96,0,3,0,0,12,0,0,96,0,
1,128,0,12,0,0,48,0,1,128,0,6,0,0,48,0,
0,192,0,6,0,0,24,0,0,192,0,3,0,0,24,0,
0,192,0,3,0,0,24,0,0,96,0,3,0,0,12,0,
0,96,0,1,128,0,12,0,0,48,0,0,128,0,0,12,
60,112,224,0,0,0,0,0,0,0,0,28,56,240,192,6,
15,14,14,0,0,0,0,0,0,0,0,0,0,56,120,120,
56,8,8,16,16,32,32,64,128,63,255,255,231,255,255,255,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,231,
255,255,252,0,7,240,0,3,128,192,0,192,2,0,16,0,
16,2,0,0,128,64,0,4,8,7,128,65,1,134,2,16,
48,32,34,6,1,194,32,224,28,20,28,1,193,65,192,28,
20,28,1,193,131,128,56,40,56,3,130,131,128,56,40,56,
3,130,129,128,56,72,28,7,132,64,192,176,132,6,19,152,
64,62,30,2,0,0,0,16,0,0,0,128,0,0,4,0,
3,128,48,7,224,0,255,192,0,1,252,7,240,24,0,96,
3,0,12,0,48,0,192,3,0,12,0,96,1,128,6,0,
24,0,96,1,128,12,0,48,0,192,3,0,12,0,48,1,
128,6,0,24,0,96,1,128,6,0,48,0,192,3,0,12,
0,48,0,192,6,0,24,0,96,1,128,6,0,31,192,254,
0,1,252,7,240,0,192,3,0,24,0,96,1,128,6,0,
24,0,96,3,0,12,0,48,0,192,3,0,12,0,96,1,
128,6,0,24,0,96,1,128,12,0,48,0,192,3,0,12,
0,48,1,128,6,0,24,0,96,1,128,6,0,48,0,192,
3,0,12,0,48,31,192,254,0,4,33,8,33,4,16,187,
231,140,99,199,14,24,112,193,14,60,113,199,24,32,0,192,
216,99,240,112,128,24,2,128,44,4,225,199,248,62,0,255,
252,28,133,12,24,40,142,0,7,0,240,7,0,112,14,3,
159,228,0,1,193,131,131,0,4,3,129,176,198,64,160,16,
111,246,28,56,225,206,28,96,198,12,48,99,6,0,30,4,
254,54,63,160,56,96,222,63,199,176,96,0,7,225,240,0,
224,185,192,28,15,14,1,192,248,240,28,7,131,0,224,28,
0,7,0,224,0,56,7,0,3,128,112,0,28,3,128,0,
224,28,0,127,255,254,0,56,7,0,1,192,56,0,28,3,
128,0,224,28,0,7,0,224,0,56,7,0,1,192,56,0,
14,1,192,0,224,28,0,7,0,224,0,56,7,0,1,192,
56,0,14,1,192,0,112,14,0,7,0,224,0,56,7,128,
15,248,255,128,0,0,7,224,0,224,128,28,14,1,192,240,
28,7,128,224,24,7,0,0,56,0,3,128,0,28,0,0,
224,0,127,255,224,56,15,1,192,56,28,3,128,224,28,7,
0,224,56,7,1,192,56,14,1,192,224,28,7,0,224,56,
7,1,192,56,14,1,192,112,14,7,0,224,56,7,143,248,
255,0,0,7,252,0,112,240,7,7,128,56,30,1,192,56,
7,0,224,28,3,128,112,14,3,128,112,14,1,192,56,7,
15,255,252,3,128,112,14,1,192,112,14,1,192,56,7,0,
224,28,3,128,112,14,1,192,56,14,1,192,56,7,0,224,
28,3,128,112,14,1,192,56,7,1,192,56,7,128,240,255,
159,240,0,7,240,126,0,14,4,224,128,28,7,192,224,28,
7,192,240,28,3,192,120,14,1,224,24,7,0,112,0,3,
128,56,0,3,128,56,0,1,192,28,0,0,224,14,0,7,
255,255,255,224,56,3,128,240,28,1,192,56,28,1,192,56,
14,0,224,28,7,0,112,14,3,128,56,7,1,192,28,3,
128,224,14,1,192,224,14,1,192,112,7,0,224,56,3,128,
112,28,1,192,56,14,0,224,28,7,0,112,14,7,0,112,
14,3,192,60,7,143,248,255,159,240,0,7,240,127,192,7,
2,112,240,7,1,240,120,3,128,248,30,1,192,60,3,128,
112,7,0,224,28,1,192,56,7,0,112,14,3,128,56,7,
0,224,14,1,192,56,3,128,112,255,255,255,252,3,128,56,
7,0,224,14,1,192,112,7,0,224,28,1,192,56,7,0,
112,14,1,192,28,3,128,112,7,0,224,28,1,192,56,14,
0,224,28,3,128,56,7,0,224,14,1,192,56,3,128,112,
14,0,224,28,3,128,56,7,1,192,28,3,128,120,7,128,
240,255,143,249,255,0,24,24,240,247,199,205,13,4,4,16,
16,128,130,2,16,16,64,70,6,32,32,127,255,254,4,4,
32,33,1,8,8,32,33,1,4,4,16,16,184,187,227,231,
135,140,12,255,255,240,255,255,255,255,255,128,
};
METRIC cmsy10_metric[128] = {
{2892, -3, 10, 25, 2, 509726},
{4590, -4, 11, 4, 4, 182045},
{4951, -6, 19, 19, 20, 509726},
{4999, -2, 18, 16, 18, 327681},
{5035, -2, 20, 28, 22, 509726},
{4592, -1, 19, 19, 20, 327681},
{2899, -2, 26, 27, 28, 509726},
{5112, -2, 20, 27, 28, 509726},
{2994, -2, 23, 27, 28, 509726},
{5207, -2, 23, 27, 28, 509726},
{3089, -2, 23, 27, 28, 509726},
{5302, -2, 23, 27, 28, 509726},
{5397, -2, 23, 27, 28, 509726},
{5492, -2, 29, 37, 39, 655362},
{5673, -2, 17, 16, 16, 327681},
{5705, -2, 17, 16, 16, 327681},
{5737, -2, 19, 27, 20, 509726},
{4429, -2, 19, 27, 20, 509726},
{5805, -3, 25, 25, 34, 509726},
{5912, -3, 25, 25, 34, 509726},
{4215, -3, 25, 25, 34, 509726},
{4322, -3, 25, 25, 34, 509726},
{6019, -3, 25, 25, 34, 509726},
{6126, -3, 25, 25, 34, 509726},
{6233, -2, 14, 27, 9, 509726},
{6264, -2, 19, 27, 19, 509726},
{3805, -3, 22, 25, 26, 509726},
{3887, -4, 22, 24, 26, 509726},
{6329, -2, 23, 37, 28, 655362},
{6459, -2, 23, 37, 28, 655362},
{6589, -3, 21, 25, 25, 509726},
{6668, -3, 21, 25, 25, 509726},
{3614, -3, 16, 36, 14, 655362},
{3677, -2, 16, 37, 14, 655362},
{3549, -3, 28, 14, 37, 327681},
{3484, -3, 28, 14, 37, 327681},
{3742, -3, 16, 36, 14, 655362},
{6747, -2, 28, 37, 37, 655362},
{6919, -2, 28, 37, 37, 655362},
{7091, -2, 18, 27, 19, 509726},
{7156, -2, 21, 37, 24, 655362},
{7267, -2, 21, 37, 24, 655362},
{7378, -1, 28, 23, 37, 400499},
{7485, -1, 28, 23, 37, 400499},
{7592, -1, 21, 39, 24, 655362},
{7709, -2, 28, 37, 37, 655362},
{7881, -2, 28, 37, 37, 655362},
{8053, -2, 17, 27, 18, 509726},
{8114, -1, 22, 8, 21, 180224},
{4784, -2, 17, 37, 18, 655362},
{3965, -3, 22, 21, 26, 436908},
{8135, -3, 22, 21, 26, 436908},
{8204, -2, 29, 32, 30, 582544},
{8324, -2, 20, 32, 30, 582544},
{8444, -6, 29, 19, 40, 0},
{8539, -2, 16, 3, 14, 0},
{4034, 0, 28, 22, 30, 364090},
{4117, -2, 28, 18, 29, 364090},
{4183, -2, 14, 23, 11, 436908},
{8545, -2, 31, 16, 35, 327681},
{8615, -2, 29, 27, 31, 473317},
{8720, -2, 28, 26, 29, 473317},
{8815, -3, 27, 27, 28, 509726},
{8910, -3, 27, 27, 28, 509726},
{9005, -2, 28, 20, 29, 400499},
{0, 0, 29, 31, 32, 523286},
{124, -1, 28, 26, 30, 430447},
{222, -1, 28, 21, 30, 345066},
{301, 0, 27, 30, 28, 505540},
{406, -1, 28, 22, 30, 345887},
{489, -1, 27, 32, 29, 471039},
{605, -1, 28, 23, 34, 389851},
{703, 0, 27, 32, 30, 553463},
{823, 1, 27, 25, 28, 356854},
{911, -2, 27, 31, 33, 444190},
{1039, -1, 28, 28, 30, 499352},
{1144, -1, 28, 25, 30, 452018},
{1238, -1, 28, 44, 31, 787022},
{1409, 1, 31, 40, 34, 537717},
{1579, -2, 28, 29, 30, 521741},
{1688, 0, 27, 29, 30, 455842},
{1797, -4, 28, 27, 34, 535213},
{1912, 0, 27, 33, 29, 555420},
{2032, -1, 28, 25, 30, 396859},
{2126, -1, 28, 30, 29, 356938},
{2235, 1, 27, 28, 29, 410145},
{2337, -2, 27, 24, 30, 401593},
{2427, -2, 27, 40, 30, 647354},
{2577, -2, 27, 30, 28, 467466},
{2682, -1, 27, 27, 33, 438001},
{2794, -2, 27, 28, 28, 474956},
{3184, -2, 24, 23, 26, 436908},
{3259, -2, 24, 23, 26, 436908},
{9078, -2, 24, 23, 26, 436908},
{3334, -2, 24, 23, 26, 436908},
{3409, -2, 24, 23, 26, 436908},
{9153, -2, 28, 20, 29, 400499},
{9226, -2, 28, 20, 29, 400499},
{9299, -7, 30, 10, 42, 291272},
{9352, 0, 30, 10, 42, 291272},
{9405, -7, 30, 10, 42, 291272},
{9458, 0, 30, 10, 42, 291272},
{4640, -3, 30, 14, 41, 327681},
{4712, -3, 30, 14, 41, 327681},
{9511, -4, 30, 9, 42, 254863},
{9559, -2, 30, 9, 42, 254863},
{4579, -5, 30, 2, 41, 182045},
{9607, -4, 30, 12, 42, 327681},
{9670, -3, 31, 14, 43, 327681},
{9746, -1, 31, 23, 43, 400499},
{4497, -2, 30, 16, 41, 327681},
{9870, -2, 23, 7, 28, 182045},
{9895, -2, 1, 34, 42, 546135},
{10074, -2, 27, 26, 28, 491521},
{10165, -2, 27, 30, 29, 546135},
{4868, -2, 29, 17, 39, 273068},
{10274, -3, 24, 21, 25, 436908},
{10340, -3, 24, 21, 25, 436908},
{10406, -3, 25, 26, 34, 509726},
{10517, -2, 25, 26, 34, 509726},
{10628, -3, 28, 11, 37, 291273},
{10679, -2, 28, 13, 38, 291272},
{10741, -2, 28, 13, 37, 291272},
{10802, -2, 28, 21, 37, 400499},
{10900, -1, 29, 30, 35, 509726},
{11032, -3, 29, 27, 36, 509726},
{11154, -2, 29, 28, 31, 509726},
{11263, -2, 29, 28, 35, 509726},
};
char cmsy10_bitmap[11386] = {
0,0,0,32,0,0,1,192,0,0,7,128,0,0,15,0,
0,0,62,0,0,0,124,0,0,1,120,0,0,2,240,0,
0,9,224,0,0,19,192,0,0,71,128,0,1,143,0,0,
2,30,0,0,12,60,0,0,16,120,0,0,64,240,0,1,
129,224,0,2,3,192,0,12,7,128,0,48,15,0,0,64,
30,0,1,255,254,0,7,255,252,0,24,0,120,0,48,0,
240,0,192,1,225,131,0,3,195,140,0,7,199,248,0,15,
159,224,0,15,223,128,0,31,30,0,0,0,0,96,63,0,
120,127,224,126,48,252,7,176,31,1,216,3,192,126,0,240,
63,0,56,15,128,14,3,224,7,0,248,1,0,60,1,128,
31,1,128,7,131,192,1,227,254,0,121,63,192,28,1,248,
15,0,62,3,192,7,192,240,1,240,56,0,60,14,0,15,
7,128,3,193,192,0,224,112,0,56,60,0,28,14,96,6,
3,188,2,1,223,131,0,99,255,0,48,127,0,0,0,3,
240,0,127,192,12,62,0,128,240,24,7,129,128,56,28,1,
193,192,28,14,0,224,224,12,7,0,0,112,0,3,128,0,
60,0,1,224,0,14,0,0,112,0,7,128,0,60,0,1,
224,0,15,0,0,120,0,3,192,0,159,0,12,248,1,195,
224,12,31,128,192,126,8,1,255,128,7,240,0,0,63,254,
0,7,255,255,0,113,225,254,2,7,0,252,56,28,1,249,
192,240,3,230,3,192,7,240,15,0,31,0,60,0,60,0,
224,0,240,3,128,3,192,30,0,15,0,120,0,60,1,224,
0,224,7,0,3,128,28,0,30,0,240,0,112,3,128,1,
128,14,0,14,0,120,0,112,1,192,1,128,7,0,8,0,
60,0,64,0,224,6,0,3,128,96,0,28,14,0,0,255,
224,0,7,252,0,0,0,3,248,0,63,240,3,7,192,24,
15,0,192,60,6,0,192,56,2,1,224,0,7,128,0,30,
0,0,124,0,1,248,0,3,248,0,7,248,0,7,192,0,
48,0,3,0,0,24,0,0,192,0,6,0,0,56,0,1,
192,0,7,0,2,60,0,24,240,1,195,224,6,15,192,32,
31,131,0,63,240,0,127,0,0,0,1,255,255,0,15,255,
255,0,56,224,14,0,65,224,12,1,193,224,0,3,129,192,
0,3,1,192,0,6,3,192,0,0,3,192,0,0,3,128,
0,0,3,128,0,0,7,128,0,0,7,0,0,0,7,0,
0,0,15,255,192,0,15,255,0,0,14,0,0,0,28,0,
0,0,28,0,0,0,60,0,0,0,56,0,0,0,56,0,
0,0,112,0,0,0,112,0,0,48,224,0,0,112,192,0,
0,253,128,0,0,255,0,0,0,60,0,0,0,0,3,252,
0,63,252,1,192,248,4,0,240,16,1,192,224,3,131,128,
14,6,0,24,28,0,96,112,0,0,224,0,3,192,0,7,
0,0,14,0,0,60,0,2,120,0,28,240,0,121,224,0,
243,192,1,199,128,7,143,128,31,31,0,60,31,0,184,63,
134,240,63,241,192,31,131,128,0,14,0,0,28,0,0,112,
0,0,224,6,3,128,63,4,0,127,240,0,63,128,0,0,
252,0,3,3,254,0,7,12,62,0,14,24,30,0,14,56,
30,0,28,112,30,0,28,96,28,0,56,192,28,0,56,0,
60,0,120,0,60,0,112,0,60,0,112,0,56,0,224,0,
63,255,224,0,127,255,224,0,112,1,224,0,112,1,192,0,
240,1,192,0,240,3,192,0,224,3,192,0,224,3,128,1,
192,3,128,1,192,7,128,3,192,7,128,3,128,7,128,3,
128,7,128,7,128,7,130,7,0,7,134,6,0,7,220,12,
0,7,240,0,0,3,192,0,63,255,128,255,255,129,192,48,
1,128,56,1,192,56,1,128,28,0,128,28,0,0,14,0,
0,15,0,0,7,128,0,3,128,0,1,192,0,1,224,0,
0,240,0,0,112,0,0,56,0,0,60,0,0,28,0,0,
14,0,0,15,0,0,7,0,0,3,128,0,3,128,0,1,
192,48,1,192,112,0,192,48,31,255,224,31,255,224,0,0,
0,31,254,0,0,255,248,0,6,1,128,0,48,6,0,0,
192,28,0,3,0,112,0,14,0,224,0,56,3,128,0,96,
7,0,1,128,28,0,0,0,56,0,0,0,240,0,0,1,
192,0,0,3,128,0,0,15,0,0,0,30,0,0,0,56,
0,0,0,112,0,0,1,224,0,0,3,128,0,0,7,0,
0,0,30,0,0,0,60,0,24,0,112,0,112,0,224,1,
224,3,128,3,192,7,0,7,128,28,0,15,128,48,0,31,
128,192,0,31,130,0,0,31,248,0,0,31,192,0,0,0,
96,0,224,30,0,63,7,224,12,240,30,3,6,1,224,64,
64,28,24,0,1,194,0,0,60,64,0,3,200,0,0,61,
0,0,3,240,0,0,63,0,0,3,240,0,0,127,0,0,
7,120,0,0,119,128,0,15,56,0,0,243,128,0,14,60,
0,0,227,192,0,30,30,0,1,193,224,0,28,15,0,3,
192,240,1,56,7,128,99,128,120,4,120,3,192,135,0,62,
16,96,1,254,12,0,15,128,0,0,126,0,0,255,128,0,
199,192,0,193,224,0,192,240,0,192,120,0,224,48,0,240,
16,0,112,0,0,56,0,0,56,0,0,28,0,0,30,0,
0,15,0,0,7,0,0,3,128,0,3,192,0,1,192,0,
0,224,0,0,240,0,0,120,0,0,56,0,0,28,0,0,
28,0,1,14,0,3,143,240,3,135,255,1,134,63,241,134,
3,255,134,0,63,0,0,1,128,0,0,16,0,56,0,0,
3,0,3,128,0,0,48,0,56,0,0,6,0,7,192,0,
0,224,0,124,0,0,30,0,5,192,0,3,224,0,92,0,
0,126,0,5,192,0,7,224,0,94,0,0,220,0,9,224,
0,25,192,0,142,0,3,60,0,8,224,0,115,192,1,15,
0,14,60,0,16,240,1,195,192,1,7,0,56,60,0,32,
112,3,3,192,2,7,128,96,56,0,32,120,12,3,128,4,
3,129,128,120,0,64,60,56,7,128,4,3,199,0,120,0,
128,28,224,7,128,8,1,252,0,120,1,0,31,128,7,128,
16,0,240,0,120,98,0,14,0,7,143,224,0,64,0,124,
252,0,0,0,7,255,192,0,0,0,60,120,0,0,0,0,
0,0,0,0,0,15,0,0,0,0,63,0,0,0,0,127,
0,1,0,0,126,0,3,128,0,224,0,7,128,0,128,0,
7,128,1,0,0,7,128,1,0,0,7,192,1,0,0,7,
192,2,0,0,5,192,2,0,0,13,192,2,0,0,9,224,
2,0,0,9,224,4,0,0,8,224,4,0,0,8,240,4,
0,0,16,240,8,0,0,16,112,8,0,0,16,120,8,0,
0,16,120,8,0,0,32,56,16,0,0,32,60,16,0,0,
32,60,16,0,0,64,28,16,0,0,64,30,32,0,0,64,
30,32,0,0,128,15,32,0,0,128,15,32,0,1,0,7,
224,0,51,0,7,192,0,126,0,3,192,0,126,0,3,192,
0,252,0,1,128,0,56,0,0,0,0,0,12,31,128,0,
131,255,0,8,33,252,1,135,3,224,24,112,15,1,131,0,
124,24,56,3,225,195,128,15,28,28,0,120,225,192,3,206,
8,0,30,112,0,0,247,128,0,7,184,0,0,57,192,0,
1,222,0,0,30,240,0,0,231,128,0,7,60,0,0,121,
224,0,3,143,0,0,24,124,0,1,195,224,0,28,15,0,
0,192,124,0,12,3,240,0,128,15,192,8,0,63,131,128,
0,255,240,0,1,252,0,0,0,63,255,0,15,255,254,1,
199,131,248,16,56,7,227,129,192,31,184,30,0,125,128,240,
1,248,7,128,15,0,60,0,120,1,192,3,128,14,0,28,
0,240,1,192,7,128,14,0,60,0,224,1,192,6,0,14,
0,64,0,240,12,0,7,1,128,0,57,240,0,3,222,0,
0,30,0,0,0,224,0,0,7,0,0,0,112,0,0,3,
128,0,0,60,0,0,1,192,0,0,14,0,0,0,224,0,
0,6,0,0,0,0,3,252,0,1,255,224,0,192,254,0,
32,7,192,24,0,124,6,0,15,129,128,0,248,112,0,31,
28,0,1,227,128,0,60,224,0,7,156,0,0,247,0,0,
30,224,0,3,252,0,0,119,128,0,14,240,0,3,222,0,
0,115,192,0,14,124,0,3,143,128,0,96,248,6,28,31,
195,7,1,255,192,192,15,192,32,0,0,8,0,0,6,0,
0,7,0,1,255,128,2,127,224,1,193,255,128,112,7,254,
12,0,31,254,0,0,127,0,0,63,255,0,0,255,255,240,
1,199,131,252,1,3,128,63,3,129,192,15,131,129,224,3,
193,128,240,1,225,128,120,0,240,0,60,0,112,0,28,0,
56,0,14,0,56,0,15,0,24,0,7,128,24,0,3,192,
16,0,1,192,48,0,0,227,224,0,0,243,240,0,0,112,
248,0,0,56,62,0,0,60,31,0,0,28,7,192,0,14,
3,224,0,15,0,240,0,7,0,124,3,3,128,62,7,3,
192,15,131,1,192,7,226,0,192,1,254,0,192,0,124,0,
0,1,252,0,3,255,128,2,15,224,6,1,240,7,0,120,
7,0,60,3,128,24,3,192,8,1,224,0,0,248,0,0,
126,0,0,31,128,0,7,224,0,1,252,0,0,127,128,0,
15,224,0,1,248,0,0,126,2,0,31,7,0,7,135,0,
3,195,128,1,227,192,0,225,224,0,112,248,0,112,124,0,
48,31,0,32,15,224,96,3,255,192,0,127,128,0,0,0,
0,4,3,255,255,224,63,255,254,1,0,56,0,28,0,224,
0,224,7,0,3,0,28,0,28,0,240,0,224,3,192,3,
0,14,0,0,0,56,0,0,1,224,0,0,7,128,0,0,
28,0,0,0,112,0,0,3,192,0,0,14,0,0,0,56,
0,0,1,224,0,0,7,128,0,0,28,0,0,0,112,0,
0,3,128,0,0,14,0,0,0,112,0,0,1,128,0,0,
12,0,0,1,255,192,0,15,252,0,0,3,224,0,48,254,
0,14,49,224,0,224,30,0,30,1,192,1,192,28,0,28,
3,192,3,192,56,0,56,3,128,7,128,112,0,120,7,0,
15,0,240,0,240,14,0,31,0,224,1,224,28,0,62,1,
192,3,224,56,0,126,3,128,15,192,120,0,220,7,0,27,
192,112,3,188,15,0,115,128,240,6,56,15,0,199,128,240,
24,120,15,130,7,128,248,192,120,7,240,7,224,124,0,120,
0,120,0,12,252,0,30,62,0,31,30,0,31,15,0,15,
15,0,7,7,0,3,7,0,2,7,128,2,7,128,2,3,
128,4,3,128,4,3,128,8,3,128,24,3,128,48,3,128,
48,3,128,96,3,128,192,3,129,128,3,131,128,3,135,0,
3,142,0,3,156,0,3,184,0,3,240,0,3,224,0,7,
192,0,7,128,0,6,0,0,4,0,0,120,0,3,0,6,
252,0,7,0,15,62,0,7,0,15,31,0,7,128,15,15,
0,15,128,7,7,0,27,128,3,7,128,19,128,3,7,128,
51,192,2,3,128,35,192,2,3,128,65,192,2,3,128,193,
192,4,3,128,129,224,4,3,129,129,224,8,3,129,0,224,
8,3,130,0,224,16,3,134,0,224,16,3,132,0,224,32,
3,140,0,240,96,3,152,0,240,64,3,48,0,112,192,3,
48,0,113,128,7,96,0,115,0,7,192,0,115,0,7,128,
0,118,0,7,128,0,124,0,7,0,0,120,0,6,0,0,
120,0,12,0,0,112,0,8,0,0,96,0,8,0,0,64,
0,0,31,0,60,0,254,1,240,14,248,3,192,97,224,15,
0,7,192,56,0,31,0,192,0,60,6,0,0,240,32,0,
3,193,0,0,15,24,0,0,60,128,0,0,252,0,0,3,
192,0,0,15,0,0,0,124,0,0,2,240,0,0,51,192,
0,1,15,0,0,24,60,0,0,128,240,0,4,3,192,0,
96,15,0,3,0,62,0,28,0,248,0,240,1,225,131,192,
7,220,15,128,31,192,60,0,62,0,7,128,1,131,252,0,
56,143,192,7,128,120,0,48,7,128,2,0,240,0,64,15,
0,8,1,224,1,0,28,0,64,3,128,8,0,112,1,0,
15,0,64,1,224,24,0,28,3,0,3,128,192,0,112,16,
0,14,6,0,1,193,128,0,56,112,0,7,12,0,0,227,
0,0,28,192,0,3,184,0,0,126,0,0,15,128,0,1,
224,2,0,56,0,64,30,0,12,7,128,1,195,224,0,31,
240,0,1,252,0,0,30,0,0,0,0,7,248,16,1,255,
255,0,96,127,224,12,0,12,1,192,1,128,56,0,48,2,
0,6,0,0,0,192,0,0,24,0,0,3,128,0,0,112,
0,0,6,0,0,31,252,0,3,255,192,0,3,24,0,0,
96,0,0,12,0,0,1,128,0,0,48,0,0,4,0,0,
0,128,0,192,48,0,28,6,0,3,128,192,0,48,24,0,
7,3,255,192,192,127,255,248,8,3,254,0,255,255,255,255,
255,255,192,0,4,0,0,0,192,0,0,24,0,0,3,0,
0,0,96,0,0,12,0,0,1,128,0,0,48,0,0,6,
0,0,0,192,0,0,24,0,0,3,0,0,0,96,1,255,
255,255,255,255,255,248,0,48,0,0,6,0,0,0,192,0,
0,24,0,0,3,0,0,0,96,0,0,12,0,0,1,128,
0,0,48,0,0,6,0,0,0,192,3,255,255,255,255,255,
255,240,0,63,128,0,24,140,0,12,16,96,2,2,2,0,
128,64,32,32,8,2,8,1,0,33,0,32,4,64,4,0,
72,0,128,10,0,16,0,192,2,0,24,0,64,3,255,255,
255,224,1,0,12,0,32,1,128,4,0,48,0,128,5,0,
16,1,32,2,0,34,0,64,8,64,8,1,4,1,0,64,
64,32,16,4,4,4,0,96,131,0,3,17,128,0,31,192,
0,0,63,128,0,24,12,0,12,0,96,2,0,2,0,128,
0,32,40,0,10,8,128,2,33,8,0,132,64,128,32,72,
8,8,10,0,130,0,192,8,128,24,0,160,3,0,8,0,
96,1,0,12,0,80,1,128,17,0,48,4,16,5,1,1,
1,32,64,16,34,16,1,8,68,0,17,5,0,1,64,64,
0,16,4,0,4,0,96,3,0,3,1,128,0,31,192,0,
64,0,3,128,0,15,0,0,30,0,0,60,0,0,120,0,
0,240,0,1,224,0,3,192,0,7,128,0,15,0,0,30,
0,0,60,0,0,120,0,0,240,0,1,224,0,3,192,0,
7,128,0,15,0,0,27,0,0,102,0,0,198,0,3,7,
0,28,7,128,240,3,255,128,1,252,0,0,254,0,7,255,
0,60,7,128,224,3,131,0,1,140,0,1,152,0,3,96,
0,3,192,0,7,128,0,15,0,0,30,0,0,60,0,0,
120,0,0,240,0,1,224,0,3,192,0,7,128,0,15,0,
0,30,0,0,60,0,0,120,0,0,240,0,1,224,0,3,
192,0,6,128,0,4,0,16,0,0,112,0,0,224,0,3,
96,0,6,192,0,13,128,0,49,128,0,99,0,1,131,0,
3,6,0,12,6,0,24,12,0,96,12,0,192,24,1,128,
48,6,0,48,12,0,96,48,0,96,96,0,193,128,0,195,
0,1,134,0,3,24,0,3,48,0,6,192,0,7,128,0,
4,192,0,3,128,0,13,128,0,51,0,0,99,0,1,134,
0,3,12,0,6,12,0,24,24,0,48,24,0,192,48,1,
128,48,6,0,96,12,0,192,24,0,192,96,1,128,192,1,
131,0,3,6,0,3,24,0,6,48,0,6,192,0,13,128,
0,27,0,0,28,0,0,56,0,0,32,0,3,0,12,0,
48,0,192,3,0,12,0,48,0,192,3,0,12,0,48,0,
192,3,0,12,0,48,0,192,3,0,12,0,48,0,192,3,
0,12,0,48,0,192,3,0,12,0,48,0,192,3,2,12,
22,49,132,200,11,64,30,0,48,0,192,1,0,1,0,12,
0,48,1,224,11,64,76,134,49,160,193,3,0,12,0,48,
0,192,3,0,12,0,48,0,192,3,0,12,0,48,0,192,
3,0,12,0,48,0,192,3,0,12,0,48,0,192,3,0,
12,0,48,0,192,3,0,12,0,48,0,192,3,0,2,0,
0,0,0,64,0,0,0,4,0,0,0,0,128,0,0,0,
16,0,0,0,2,0,0,0,0,255,255,255,255,255,255,255,
255,255,32,0,0,0,1,0,0,0,0,8,0,0,0,0,
64,0,0,0,4,0,0,0,0,32,0,0,0,0,0,0,
4,0,0,0,0,16,0,0,0,0,128,0,0,0,2,0,
0,0,0,8,0,0,0,0,35,255,255,255,255,255,255,255,
255,255,0,0,0,0,64,0,0,0,4,0,0,0,0,64,
0,0,0,4,0,0,0,0,32,0,0,0,2,0,2,0,
0,8,0,64,0,0,64,4,0,0,4,0,128,0,0,32,
16,0,0,1,2,0,0,0,8,255,255,255,255,255,255,255,
255,255,32,0,0,0,129,0,0,0,16,8,0,0,2,0,
64,0,0,64,4,0,0,4,0,32,0,0,128,0,127,255,
129,255,255,193,224,0,1,128,0,1,128,0,1,128,0,0,
192,0,0,192,0,0,96,0,0,96,0,0,48,0,0,24,
0,0,12,0,0,6,0,0,3,0,0,1,128,0,0,192,
0,0,48,0,0,24,0,0,6,0,0,3,0,0,0,192,
0,0,56,0,0,15,0,0,1,255,255,128,63,255,192,255,
254,0,255,255,128,0,1,224,0,0,112,0,0,24,0,0,
12,0,0,12,0,0,6,0,0,6,0,0,3,0,0,3,
0,0,3,0,0,3,0,0,3,0,0,3,0,0,3,0,
0,3,0,0,6,0,0,6,0,0,12,0,0,12,0,0,
24,0,0,48,0,1,224,255,255,192,255,254,0,0,127,248,
15,255,193,224,0,28,0,1,128,0,24,0,0,192,0,12,
0,0,96,0,6,0,0,48,0,1,128,0,15,255,255,255,
255,255,0,0,24,0,0,192,0,3,0,0,24,0,0,96,
0,3,0,0,12,0,0,56,0,0,240,0,1,255,248,3,
255,192,64,0,7,0,0,54,0,1,152,0,6,96,0,24,
192,0,195,0,3,12,0,12,24,0,96,96,1,129,255,254,
3,255,240,12,0,192,48,3,0,96,24,1,128,96,3,3,
0,12,12,0,48,48,0,97,128,1,134,0,6,24,0,12,
192,0,51,0,0,204,0,1,224,0,7,128,0,30,0,0,
48,0,0,192,0,255,255,255,255,240,0,12,0,3,0,0,
192,0,48,0,12,0,3,0,0,192,0,48,0,12,0,3,
0,0,207,255,247,255,252,0,3,0,0,192,0,48,0,12,
0,3,0,0,192,0,48,0,12,0,3,0,0,192,0,48,
0,15,255,255,255,255,192,255,255,255,255,255,252,0,0,24,
0,0,48,0,0,96,0,0,192,0,1,128,0,3,0,0,
6,0,0,12,0,0,24,0,0,1,128,0,3,192,0,7,
128,0,15,0,0,30,0,0,60,0,0,120,0,0,240,0,
1,224,0,3,192,0,7,128,0,15,0,0,14,0,0,3,
192,0,0,120,0,0,15,0,0,1,224,0,0,60,0,0,
7,128,0,0,240,0,0,30,0,0,3,192,0,0,120,0,
0,15,0,0,1,128,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,127,255,255,127,255,
255,192,192,0,0,120,0,0,15,0,0,1,224,0,0,60,
0,0,7,128,0,0,240,0,0,30,0,0,3,192,0,0,
120,0,0,15,0,0,1,224,0,0,56,0,0,120,0,0,
240,0,1,224,0,3,192,0,7,128,0,15,0,0,30,0,
0,60,0,0,120,0,0,240,0,0,224,0,0,192,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,127,255,255,127,255,255,192,255,255,255,
239,255,255,252,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,31,255,255,255,255,
255,255,128,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,1,255,255,255,255,255,255,
240,192,0,192,0,96,0,96,0,96,0,48,0,48,0,48,
0,24,0,24,0,24,0,12,0,12,0,12,0,6,0,6,
0,3,0,3,0,3,0,1,128,1,128,1,128,0,192,0,
192,0,192,0,96,0,96,0,48,0,48,0,48,0,24,0,
24,0,24,0,12,0,12,0,12,0,6,0,6,0,6,0,
3,0,1,255,255,255,255,255,255,255,255,255,255,192,111,246,
0,64,0,28,0,6,192,1,140,0,96,192,24,12,6,0,
193,128,12,96,0,216,0,15,0,1,176,0,99,0,24,48,
6,3,1,128,48,96,3,24,0,54,0,3,128,0,32,0,
0,60,3,128,28,0,96,3,128,14,0,56,0,224,3,128,
14,0,56,0,224,3,128,14,0,56,0,224,3,128,12,0,
112,7,0,240,0,112,0,112,0,192,3,128,14,0,56,0,
224,3,128,14,0,56,0,224,3,128,14,0,56,0,224,3,
128,6,0,28,0,56,0,60,240,0,112,0,112,0,192,3,
128,14,0,56,0,224,3,128,14,0,56,0,224,3,128,14,
0,56,0,224,3,128,6,0,28,0,56,0,60,3,128,28,
0,96,3,128,14,0,56,0,224,3,128,14,0,56,0,224,
3,128,14,0,56,0,224,3,128,12,0,112,7,0,240,0,
7,224,0,63,0,127,192,7,140,6,15,128,96,16,64,30,
6,0,68,0,120,64,1,32,1,230,0,10,0,7,96,0,
48,0,30,0,1,128,0,240,0,12,0,3,192,0,96,0,
30,0,3,0,1,184,0,20,0,25,224,1,32,0,135,128,
8,128,24,30,0,130,1,128,124,24,12,120,0,255,128,63,
0,1,248,0,0,14,0,12,128,12,96,6,112,3,16,3,
0,1,128,0,192,0,96,0,48,0,56,0,28,0,14,0,
7,0,3,128,1,128,1,192,0,224,0,112,0,56,0,28,
0,14,0,7,0,3,0,3,128,1,192,0,224,0,112,0,
56,0,24,0,12,0,6,0,3,0,1,128,17,128,28,192,
12,96,2,96,0,224,0,64,0,56,0,13,128,3,24,0,
193,128,48,24,12,1,131,0,24,192,1,176,0,28,0,3,
128,0,216,0,49,128,12,24,3,1,128,192,24,48,1,140,
0,27,0,1,160,0,16,1,128,1,128,1,128,1,128,65,
130,241,143,57,156,15,240,3,192,3,192,15,240,57,156,241,
143,65,130,1,128,1,128,1,128,1,128,0,15,0,0,0,
240,0,0,15,0,0,0,240,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
255,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,15,0,0,0,240,
0,0,15,0,0,0,240,0,255,255,255,255,255,255,252,0,
24,0,0,3,0,0,0,96,0,0,12,0,0,1,128,0,
0,48,0,0,6,0,0,0,192,0,0,24,0,0,3,0,
0,0,96,1,255,255,255,255,255,255,248,0,48,0,0,6,
0,0,0,192,0,0,24,0,0,3,0,0,0,96,0,0,
12,0,0,1,128,0,0,48,0,0,6,0,0,0,192,0,
0,24,0,0,2,0,0,0,63,128,0,24,12,0,12,0,
96,2,0,2,0,128,0,32,32,0,2,8,0,0,33,0,
0,4,64,0,0,72,0,0,10,0,0,0,192,0,0,24,
0,0,3,255,255,255,224,0,0,12,0,0,1,128,0,0,
48,0,0,5,0,0,1,32,0,0,34,0,0,8,64,0,
1,4,0,0,64,64,0,16,4,0,4,0,96,3,0,3,
1,128,0,31,192,0,0,63,128,0,24,12,0,12,0,96,
2,0,2,0,128,0,32,32,0,10,8,0,2,33,0,0,
132,64,0,32,72,0,8,10,0,2,0,192,0,128,24,0,
32,3,0,8,0,96,1,0,12,0,64,1,128,16,0,48,
4,0,5,1,0,1,32,64,0,34,16,0,8,68,0,1,
5,0,0,64,64,0,16,4,0,4,0,96,3,0,3,1,
128,0,31,192,0,0,63,128,0,24,12,0,12,0,96,2,
0,2,0,128,0,32,32,0,2,8,0,0,33,0,0,4,
64,0,0,72,0,0,10,0,0,0,192,7,0,24,1,240,
3,0,62,0,96,7,192,12,0,248,1,128,14,0,48,0,
0,5,0,0,1,32,0,0,34,0,0,8,64,0,1,4,
0,0,64,64,0,16,4,0,4,0,96,3,0,3,1,128,
0,31,192,0,0,3,254,0,0,0,127,252,0,0,15,0,
120,0,1,224,0,240,0,28,0,1,192,1,128,0,3,0,
24,0,0,12,1,128,0,0,48,24,0,0,0,192,192,0,
0,6,12,0,0,0,24,96,0,0,0,198,0,0,0,3,
48,0,0,0,25,128,0,0,0,216,0,0,0,3,192,0,
0,0,30,0,0,0,0,240,0,0,0,7,128,0,0,0,
60,0,0,0,1,224,0,0,0,15,0,0,0,0,120,0,
0,0,3,96,0,0,0,51,0,0,0,1,152,0,0,0,
12,96,0,0,0,195,0,0,0,6,12,0,0,0,96,96,
0,0,3,1,128,0,0,48,6,0,0,3,0,24,0,0,
48,0,112,0,7,0,1,224,0,240,0,3,192,30,0,0,
7,255,192,0,0,15,248,0,0,3,192,15,240,28,56,48,
12,96,6,96,6,192,3,192,3,192,3,192,3,96,6,96,
6,48,12,28,56,15,240,3,192,3,192,15,240,31,248,63,
252,127,254,127,254,255,255,255,255,255,255,255,255,127,254,127,
254,63,252,31,248,15,240,3,192,192,0,0,124,0,0,28,
224,0,14,15,0,7,128,124,7,192,3,255,224,0,15,224,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,127,0,0,127,252,0,62,3,224,30,
0,15,7,0,0,115,128,0,3,224,0,0,48,0,127,255,
128,255,255,193,224,0,1,192,0,1,128,0,1,128,0,0,
192,0,0,192,0,0,96,0,0,96,0,0,48,0,0,24,
0,0,12,0,0,6,0,0,3,0,0,1,128,0,0,96,
0,0,48,0,0,12,0,0,6,0,0,1,128,0,0,112,
0,0,30,0,0,3,255,255,0,127,255,128,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,127,255,255,191,255,255,192,127,255,0,63,255,224,0,0,
60,0,0,7,0,0,0,192,0,0,48,0,0,24,0,0,
6,0,0,3,0,0,0,192,0,0,96,0,0,48,0,0,
24,0,0,12,0,0,6,0,0,3,0,0,3,0,0,1,
128,0,1,128,0,0,192,0,0,192,0,1,192,0,3,192,
255,255,128,127,255,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,255,255,255,127,
255,255,128,0,0,0,128,0,0,192,0,0,96,0,0,96,
0,0,48,0,0,24,0,0,24,0,0,24,0,0,56,0,
0,120,0,0,240,0,15,224,15,255,192,7,255,224,0,1,
252,0,0,7,128,0,0,240,0,0,28,0,0,3,0,0,
0,192,0,0,48,0,0,24,0,0,6,0,0,3,0,0,
0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,255,255,255,255,255,255,192,128,0,
0,96,0,0,48,0,0,12,0,0,6,0,0,3,0,0,
0,192,0,0,48,0,0,14,0,0,3,192,0,0,120,0,
0,15,224,0,1,255,248,0,255,252,1,252,0,3,192,0,
7,128,0,7,0,0,6,0,0,6,0,0,6,0,0,3,
0,0,3,0,0,1,128,0,0,192,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,255,255,255,255,255,255,192,7,192,0,35,252,0,4,
255,224,0,188,30,0,62,1,240,15,128,15,7,160,0,255,
228,0,7,248,128,0,124,0,7,192,0,35,252,0,4,255,
224,0,188,30,0,62,1,240,15,128,15,7,160,0,255,228,
0,7,248,128,0,124,0,0,0,0,31,0,0,143,240,0,
19,255,128,2,240,120,0,248,7,192,62,0,60,30,128,3,
255,144,0,31,226,0,1,240,0,0,0,1,128,24,0,0,
60,3,192,0,3,128,56,0,0,112,7,0,0,15,0,240,
0,1,224,30,0,0,28,1,192,0,3,128,56,0,0,120,
7,128,0,15,0,240,0,1,224,30,0,0,28,1,192,0,
3,128,56,0,0,120,7,128,0,3,192,60,0,0,7,0,
112,0,0,14,0,224,0,0,60,3,192,0,0,120,7,128,
0,0,224,14,0,0,1,192,28,0,0,7,128,120,0,0,
15,0,240,0,0,30,1,224,0,0,56,3,128,0,0,112,
7,0,0,1,224,30,0,0,3,0,48,192,12,0,0,7,
128,120,0,0,14,0,224,0,0,28,1,192,0,0,120,7,
128,0,0,240,15,0,0,1,192,28,0,0,3,128,56,0,
0,15,0,240,0,0,30,1,224,0,0,60,3,192,0,0,
112,7,0,0,0,224,14,0,0,3,192,60,0,0,30,1,
224,0,1,192,28,0,0,56,3,128,0,7,128,120,0,0,
240,15,0,0,14,0,224,0,1,192,28,0,0,60,3,192,
0,7,128,120,0,0,240,15,0,0,14,0,224,0,1,192,
28,0,0,60,3,192,0,1,128,24,0,0,0,0,0,0,
128,0,0,192,0,0,96,0,0,96,0,0,48,0,0,24,
0,0,24,0,0,24,0,0,56,0,0,120,0,0,240,0,
15,224,15,255,192,7,255,224,0,1,252,0,0,7,128,0,
0,240,0,0,28,0,0,3,0,0,0,192,0,0,48,0,
0,24,0,0,6,0,0,3,0,0,1,128,128,0,0,96,
0,0,48,0,0,12,0,0,6,0,0,3,0,0,0,192,
0,0,48,0,0,14,0,0,3,192,0,0,120,0,0,15,
224,0,1,255,248,0,255,252,1,252,0,3,192,0,7,128,
0,7,0,0,6,0,0,6,0,0,6,0,0,3,0,0,
3,0,0,1,128,0,0,192,0,0,0,0,0,0,48,24,
0,0,0,127,192,0,0,0,28,0,0,0,1,224,0,0,
0,29,0,0,0,1,200,0,0,0,28,64,0,0,1,194,
0,0,0,28,16,0,0,1,192,64,0,0,28,2,0,0,
1,192,0,0,0,28,0,0,0,1,192,0,0,0,28,0,
0,0,1,192,0,0,0,28,0,0,0,1,192,0,0,0,
28,0,0,0,1,192,0,0,0,28,0,0,0,1,192,0,
0,0,28,0,0,0,1,192,0,0,0,28,0,0,0,1,
192,0,0,0,28,0,0,0,1,192,0,0,0,28,0,0,
0,1,192,0,0,0,28,0,0,0,1,192,0,0,0,28,
0,0,0,1,192,0,0,0,28,0,0,0,1,192,0,0,
0,12,0,0,0,0,0,192,0,0,0,7,0,0,0,0,
28,0,0,0,0,112,0,0,0,1,192,0,0,0,7,0,
0,0,0,28,0,0,0,0,112,0,0,0,1,192,0,0,
0,7,0,0,0,0,28,0,0,0,0,112,0,0,0,1,
192,0,0,0,7,0,0,0,0,28,0,0,0,0,112,0,
0,0,1,192,0,0,0,7,0,0,0,0,28,0,0,0,
0,112,0,0,0,1,192,0,0,0,7,0,0,0,0,28,
0,0,0,0,112,0,0,0,1,192,0,0,0,7,0,0,
0,0,28,2,0,0,0,112,16,0,0,1,193,0,0,0,
7,8,0,0,0,28,64,0,0,0,114,0,0,0,1,208,
0,0,0,7,128,0,0,0,28,0,0,0,31,240,0,0,
3,1,128,7,192,0,35,252,0,4,255,224,0,188,30,0,
62,1,240,15,128,15,7,160,0,255,228,0,7,248,128,0,
124,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,15,255,255,255,
255,255,255,128,0,6,0,0,0,0,48,0,0,0,3,0,
0,0,0,24,0,0,0,1,128,0,0,0,24,0,0,0,
1,255,255,255,192,31,255,255,255,3,128,0,0,0,56,0,
0,0,15,0,0,0,1,224,0,0,0,3,192,0,0,0,
7,0,0,0,0,12,0,0,0,0,56,0,0,0,0,255,
255,255,248,3,255,255,255,128,12,0,0,0,0,48,0,0,
0,0,192,0,0,0,6,0,0,0,0,24,0,0,0,0,
192,0,0,0,0,3,0,0,0,0,24,0,0,0,0,96,
0,0,0,3,0,0,0,0,12,0,0,0,0,48,1,255,
255,255,192,31,255,255,255,0,0,0,0,14,0,0,0,0,
56,0,0,0,0,120,0,0,0,0,240,0,0,0,30,0,
0,0,1,192,0,0,0,24,0,0,0,3,128,255,255,255,
248,3,255,255,255,128,0,0,0,24,0,0,0,1,128,0,
0,0,24,0,0,0,0,192,0,0,0,12,0,0,0,0,
96,0,0,16,0,0,32,0,0,224,0,1,192,0,6,192,
0,13,128,0,49,128,0,99,0,1,131,0,6,3,0,28,
7,0,248,15,131,176,27,156,96,49,240,192,97,129,128,192,
3,1,128,6,3,0,12,6,0,24,12,0,48,24,0,96,
48,0,192,96,1,128,192,3,1,128,6,3,0,12,6,0,
24,12,0,48,24,0,96,48,0,192,96,1,128,192,3,1,
128,6,3,0,12,6,0,24,12,0,48,24,0,3,1,128,
6,3,0,12,6,0,24,12,0,48,24,0,96,48,0,192,
96,1,128,192,3,1,128,6,3,0,12,6,0,24,12,0,
48,24,0,96,48,0,192,96,1,128,192,3,1,128,6,3,
0,12,6,0,24,12,0,48,24,0,96,48,48,192,97,241,
128,199,59,1,184,62,3,224,28,7,0,24,12,0,24,48,
0,24,192,0,49,128,0,54,0,0,108,0,0,112,0,0,
224,0,0,128,0,1,0,0,0,24,0,48,0,0,48,0,
96,0,0,192,0,96,0,1,128,0,192,0,6,0,0,192,
0,24,0,0,192,0,127,255,255,192,1,255,255,255,192,7,
0,0,1,192,24,0,0,0,192,224,0,0,0,231,128,0,
0,0,247,128,0,0,3,195,128,0,0,14,1,128,0,0,
48,1,128,0,0,192,1,255,255,255,0,1,255,255,252,0,
3,0,0,24,0,3,0,0,96,0,3,0,1,128,0,6,
0,3,0,0,6,0,12,0,0,12,0,24,0,192,96,0,
0,7,252,0,0,0,28,0,0,0,0,240,0,0,0,5,
192,0,0,0,39,0,0,0,1,28,0,0,0,8,112,0,
0,0,65,192,0,0,4,7,0,0,0,32,28,0,0,0,
0,112,0,0,0,1,192,0,0,0,7,0,0,0,0,28,
0,0,0,0,112,0,0,0,1,192,0,0,0,7,0,0,
0,0,28,0,0,0,0,112,0,0,0,1,192,0,0,0,
7,0,0,0,0,28,0,0,0,0,112,0,0,0,1,192,
0,0,0,7,0,0,0,0,28,0,0,0,0,112,0,0,
0,1,192,0,0,0,7,0,0,0,0,28,0,0,0,0,
112,0,0,0,1,192,0,0,0,7,0,0,0,0,28,0,
0,0,0,112,0,0,0,1,128,0,0,0,0,24,0,0,
0,1,192,0,0,0,28,0,0,0,1,192,0,0,0,28,
0,0,0,1,192,0,0,0,28,0,0,0,1,192,0,0,
0,28,0,0,0,1,192,0,0,0,28,0,0,0,1,192,
0,0,0,28,0,0,0,1,192,0,0,0,28,0,0,0,
1,192,0,0,0,28,0,0,0,1,192,0,0,0,28,0,
0,0,1,192,0,0,0,28,0,0,0,1,192,0,0,0,
28,0,0,0,1,192,0,0,0,28,0,0,0,1,192,0,
0,32,28,0,0,1,1,192,0,0,4,28,0,0,0,33,
192,0,0,1,28,0,0,0,9,192,0,0,0,92,0,0,
0,3,192,0,0,0,28,0,0,0,1,255,0,0,0,12,
6,0,0,0,0,7,192,1,227,254,0,240,193,224,48,16,
14,12,4,0,227,0,128,14,192,32,1,248,4,0,30,0,
128,3,192,16,0,60,2,0,7,128,64,1,248,4,0,55,
0,128,12,112,24,3,7,1,0,192,120,24,240,7,224,248,
0,60,6,15,15,15,30,30,30,28,28,60,56,56,56,48,
112,112,96,96,224,192,192,127,240,3,255,224,0,3,192,0,
7,0,0,12,0,0,48,0,1,128,0,6,0,0,48,0,
0,192,0,6,0,0,63,255,255,255,255,252,0,0,96,0,
3,0,0,24,0,1,128,0,12,0,0,192,0,6,0,0,
96,0,14,0,1,224,127,252,3,255,128,0,0,1,128,0,
0,1,128,0,0,3,192,0,0,3,192,0,0,6,96,0,
0,6,96,0,0,12,48,0,0,12,48,0,0,24,24,0,
0,48,12,0,0,48,12,0,0,96,6,0,0,96,6,0,
0,192,3,0,0,192,3,0,1,128,1,128,1,128,1,128,
3,0,0,192,3,0,0,192,6,0,0,96,6,0,0,96,
12,0,0,48,24,0,0,24,24,0,0,24,48,0,0,12,
48,0,0,12,96,0,0,6,96,0,0,6,255,255,255,255,
127,255,255,255,127,255,255,255,255,255,255,255,96,0,0,6,
96,0,0,6,48,0,0,12,48,0,0,12,24,0,0,24,
24,0,0,24,12,0,0,48,6,0,0,96,6,0,0,96,
3,0,0,192,3,0,0,192,1,128,1,128,1,128,1,128,
0,192,3,0,0,192,3,0,0,96,6,0,0,96,6,0,
0,48,12,0,0,48,12,0,0,24,24,0,0,12,48,0,
0,12,48,0,0,6,96,0,0,6,96,0,0,3,192,0,
0,3,192,0,0,1,128,0,0,1,128,0,0,0,96,0,
12,0,3,0,0,96,0,24,0,3,0,0,96,0,24,0,
3,0,0,192,0,24,0,6,0,0,192,0,48,0,6,0,
0,192,0,48,0,6,0,1,128,0,48,0,12,0,1,128,
0,96,0,12,0,3,0,0,96,0,12,0,3,0,0,96,
0,24,0,3,0,0,192,0,24,0,6,0,0,192,0,24,
0,6,0,0,192,0,48,0,2,0,0,219,109,191,219,109,
128,0,8,0,24,3,216,12,48,24,56,48,60,48,60,112,
110,96,102,96,102,96,102,224,199,224,199,224,199,224,199,225,
135,225,135,225,135,225,135,225,135,227,7,227,7,227,7,227,
7,230,7,102,6,102,6,118,14,60,12,60,12,28,24,12,
48,27,192,24,0,24,0,0,0,240,3,192,111,0,132,16,
224,32,196,14,8,13,1,225,1,224,28,32,24,3,198,3,
0,56,96,112,3,134,14,0,120,97,192,7,4,56,0,112,
71,0,12,8,224,6,1,31,255,0,35,135,0,140,112,224,
17,14,28,2,33,195,128,56,56,56,0,7,7,0,0,224,
224,0,28,28,0,3,131,128,0,96,112,12,12,14,3,3,
129,192,16,96,56,34,12,7,24,35,0,116,3,192,7,0,
3,248,0,3,3,128,3,0,120,0,128,15,0,64,1,224,
32,0,60,8,0,7,6,0,0,226,128,0,15,32,0,0,
4,0,0,0,128,0,60,16,0,24,131,0,12,16,60,7,
4,0,1,193,0,0,112,0,0,28,0,0,3,128,0,0,
224,0,0,28,62,0,7,1,224,0,224,28,0,56,3,128,
14,0,112,7,0,15,3,128,0,225,192,0,31,224,0,255,
255,255,255,255,255,252,0,48,0,0,6,0,0,0,192,0,
0,24,0,0,3,0,0,0,96,0,0,12,0,0,1,128,
0,0,48,0,0,6,0,0,0,192,0,0,24,0,0,3,
0,0,0,96,0,0,12,0,0,1,128,0,0,48,0,0,
6,0,0,0,192,0,0,24,0,0,3,0,0,0,96,0,
0,12,0,0,1,128,0,0,48,0,0,2,0,0,0,4,
0,0,1,128,0,0,48,0,0,6,0,0,0,192,0,0,
24,0,0,3,0,0,0,96,0,0,12,0,0,1,128,0,
0,48,0,0,6,0,0,0,192,0,0,24,0,0,3,0,
0,0,96,0,0,12,0,0,1,128,0,0,48,0,0,6,
0,0,0,192,0,0,24,0,0,3,0,0,0,96,0,0,
12,0,0,1,128,3,255,255,255,255,255,255,240,64,2,12,
0,96,192,7,14,0,112,96,3,131,0,24,56,0,193,192,
14,14,0,240,96,11,11,0,177,184,18,25,193,3,14,16,
48,97,3,7,16,112,57,7,1,224,112,14,7,0,96,112,
7,3,128,56,60,1,193,192,12,30,0,97,224,7,30,0,
49,224,3,252,0,32,64,0,3,128,0,15,0,0,30,0,
0,60,0,0,120,3,0,240,6,1,224,12,3,192,24,7,
128,48,15,0,96,30,63,254,60,127,252,120,3,0,240,6,
1,224,12,3,192,24,7,128,48,15,0,96,27,0,0,102,
0,0,198,0,3,7,0,28,7,128,240,3,255,128,1,252,
0,64,0,12,0,0,192,0,12,0,0,192,0,12,0,0,
192,0,12,0,0,192,0,12,0,0,192,0,12,0,0,192,
0,15,255,254,255,255,252,0,0,192,0,12,0,0,192,0,
12,0,0,192,0,12,0,0,192,0,12,0,0,192,0,12,
0,0,192,0,12,0,0,64,0,0,0,0,16,0,3,0,
0,48,0,3,0,0,48,0,3,0,0,48,0,3,0,0,
48,0,3,0,0,48,0,3,0,0,55,255,255,255,255,240,
0,3,0,0,48,0,3,0,0,48,0,3,0,0,48,0,
3,0,0,48,0,3,0,0,48,0,3,0,0,48,0,3,
0,0,16,64,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,255,223,240,0,64,48,12,3,0,192,48,
12,3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,255,255,240,127,255,252,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
16,0,255,255,240,12,3,0,192,48,12,3,0,192,48,12,
3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,
0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,
192,48,12,3,0,192,16,0,128,192,192,96,48,48,24,12,
12,6,3,3,1,128,192,192,96,48,48,24,12,12,6,1,
128,192,96,24,12,6,1,128,192,96,24,12,6,1,128,192,
96,24,12,6,1,128,64,192,96,24,12,6,1,128,192,96,
24,12,6,1,128,192,96,24,12,6,1,128,192,96,24,12,
12,6,3,3,1,128,192,192,96,48,48,24,12,12,6,3,
3,1,128,192,192,96,0,64,44,3,192,60,3,192,60,3,
192,60,3,192,60,3,192,60,3,192,60,3,192,60,3,192,
60,3,192,60,3,192,60,3,192,60,3,192,60,3,192,60,
3,192,60,3,192,60,3,192,60,3,192,60,3,192,60,3,
192,60,3,192,52,2,1,0,12,0,48,1,224,11,64,76,
134,49,160,193,3,0,12,0,48,0,192,3,0,12,0,48,
0,192,3,0,12,0,48,0,192,3,0,12,0,48,0,192,
3,0,12,0,48,0,192,3,0,12,0,48,0,192,3,0,
12,0,48,32,193,99,24,76,128,180,1,224,3,0,12,0,
16,0,0,16,0,0,32,0,0,224,0,3,96,0,6,192,
0,24,192,0,96,192,1,128,192,7,1,192,62,3,225,236,
6,247,24,12,120,48,24,32,96,48,0,192,96,1,128,192,
3,1,128,6,3,0,12,6,0,24,12,0,48,24,0,96,
48,0,192,96,1,128,192,3,1,128,6,3,0,12,6,0,
24,12,0,48,24,0,96,48,32,192,96,241,128,199,123,1,
188,62,3,224,28,7,0,24,12,0,24,48,0,24,192,0,
27,0,0,54,0,0,56,0,0,32,0,0,64,0,240,112,
112,224,225,195,135,14,28,112,225,199,14,56,112,227,135,14,
28,56,112,112,224,224,240,0,0,0,0,64,0,0,0,48,
0,0,0,24,0,0,0,6,0,0,0,3,0,0,0,0,
192,0,0,0,96,0,0,0,24,0,0,0,12,0,0,0,
3,0,0,0,1,128,0,0,0,96,0,0,0,48,0,0,
0,12,0,0,0,6,0,0,0,1,128,0,0,0,192,0,
0,0,48,0,0,0,24,0,0,0,6,0,0,0,3,0,
3,0,0,192,3,192,0,96,1,56,0,24,0,142,0,12,
0,1,192,3,0,0,112,1,128,0,14,0,96,0,3,128,
48,0,0,224,12,0,0,28,6,0,0,7,1,128,0,0,
224,192,0,0,56,48,0,0,7,24,0,0,1,198,0,0,
0,59,0,0,0,14,192,0,0,1,224,0,0,0,120,0,
0,0,12,0,0,0,3,0,0,0,255,225,255,195,128,7,
0,224,1,192,56,0,112,14,0,28,3,128,7,0,224,1,
192,56,0,112,14,0,28,3,128,7,0,224,1,192,56,0,
112,14,0,28,3,128,7,0,224,1,192,56,0,112,14,0,
28,3,128,7,0,224,1,192,56,0,112,14,0,28,3,128,
7,0,224,1,192,56,0,112,14,0,28,3,128,7,0,224,
1,195,255,255,255,255,255,255,255,255,255,255,247,255,255,255,
142,0,0,4,56,0,0,16,112,0,0,129,192,0,2,3,
128,0,16,14,0,0,64,28,0,2,0,112,0,8,0,224,
0,64,3,128,1,0,7,0,8,0,28,0,32,0,56,1,
0,0,224,4,0,1,192,32,0,7,0,128,0,14,4,0,
0,56,16,0,0,112,128,0,1,226,0,0,3,144,0,0,
15,64,0,0,30,0,0,0,120,0,0,0,192,0,0,3,
0,0,192,0,30,0,0,240,0,7,128,0,60,0,1,224,
0,15,0,0,120,0,3,192,0,30,0,0,240,0,7,128,
0,60,0,1,224,0,15,0,0,120,0,3,192,0,30,0,
0,240,0,7,128,0,60,0,1,224,0,15,0,0,127,255,
255,127,255,248,255,255,255,255,255,240,0,7,128,0,60,0,
1,224,0,15,0,0,120,0,3,192,0,30,0,0,240,0,
7,128,0,60,0,1,224,0,15,0,0,120,0,3,192,0,
30,0,0,240,0,7,128,0,60,0,1,224,0,15,0,0,
120,0,3,64,0,8,127,255,255,255,255,255,252,0,0,3,
0,0,0,192,0,0,48,0,0,12,0,0,3,0,0,0,
192,0,0,48,0,0,12,0,0,3,0,0,0,192,0,0,
48,0,0,12,0,0,3,0,0,0,192,0,0,48,0,0,
12,0,0,3,0,0,0,192,0,0,48,0,0,12,0,0,
3,255,255,255,255,255,255,192,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,127,255,
255,191,255,255,240,255,255,255,255,255,255,240,0,0,12,0,
0,3,0,0,0,192,0,0,48,0,0,12,0,0,3,0,
0,0,192,0,0,48,0,0,12,0,0,3,0,0,0,192,
0,0,48,0,0,12,0,0,3,0,0,0,192,0,0,48,
0,0,12,0,0,3,0,0,0,192,0,0,48,0,0,13,
255,255,255,255,255,255,192,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,127,255,255,
255,255,255,240,31,6,17,193,48,62,15,193,248,23,0,96,
14,0,192,12,1,192,102,28,99,14,224,220,31,131,240,118,
14,225,140,112,204,7,0,96,6,0,224,12,1,208,63,7,
224,248,25,7,16,193,240,2,0,56,1,192,14,0,112,3,
128,28,0,64,2,0,16,28,157,255,247,39,1,0,8,0,
224,7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,
56,1,192,14,0,112,3,128,28,0,224,2,0,16,0,128,
4,0,32,1,0,2,0,56,1,192,14,0,112,1,0,8,
8,66,255,255,255,192,128,4,0,32,3,128,28,0,224,7,
0,16,0,0,4,0,112,3,128,28,0,224,2,0,16,0,
129,255,255,255,161,8,8,0,64,7,0,56,1,192,14,0,
32,0,1,255,248,63,255,195,254,48,63,241,131,255,140,63,
252,97,255,227,31,255,24,255,248,199,255,198,63,254,49,255,
241,143,255,140,63,252,97,255,227,7,255,24,31,248,192,127,
198,0,254,48,1,241,128,1,140,0,12,96,0,99,0,3,
24,0,24,192,0,198,0,6,48,0,49,128,1,140,0,12,
96,0,99,0,3,24,0,24,192,0,198,0,6,48,0,49,
128,1,140,0,0,15,192,0,0,127,128,0,3,255,0,0,
31,254,0,0,127,248,0,3,255,240,0,15,255,192,0,63,
255,0,0,255,252,0,3,255,240,0,15,255,192,0,31,254,
0,0,127,248,0,0,255,192,0,1,254,0,3,207,252,240,
31,255,255,224,255,255,255,199,255,255,255,159,255,255,254,255,
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,247,255,19,255,159,252,207,254,63,243,63,
240,127,140,127,128,252,48,252,0,0,192,0,0,7,128,0,
0,30,0,0,0,120,0,0,0,12,0,0,1,128,0,0,
120,0,0,15,0,0,3,48,0,0,195,0,0,24,96,0,
6,6,0,1,128,96,0,48,12,0,12,0,192,3,0,12,
0,192,0,192,24,0,24,6,0,1,129,128,0,24,96,0,
1,152,0,0,31,0,0,3,176,0,0,195,0,0,48,48,
0,12,3,0,3,0,96,0,96,6,0,24,0,96,6,0,
6,1,128,0,192,48,0,12,12,0,0,195,0,0,24,96,
0,1,152,0,0,30,0,0,3,192,0,0,48,0,0,6,
0,0,15,128,31,1,254,7,248,48,112,224,198,1,152,6,
96,15,0,108,0,240,3,192,6,0,60,0,96,3,192,2,
0,60,0,0,3,192,0,0,60,0,0,3,192,0,0,54,
0,0,6,96,0,0,99,0,0,12,48,0,0,193,128,0,
24,12,0,3,0,96,0,96,3,0,12,0,48,0,192,1,
192,56,0,14,7,0,0,48,192,0,3,12,0,0,25,128,
0,0,240,0,0,15,0,0,0,96,0,0,6,0,0,0,
6,0,0,0,96,0,0,15,0,0,0,240,0,0,31,128,
0,1,248,0,0,63,192,0,7,254,0,0,127,224,0,15,
255,0,3,255,252,0,127,255,224,15,255,255,0,255,255,240,
31,255,255,131,255,255,252,127,255,255,231,255,255,254,255,255,
255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,247,255,255,254,127,226,127,231,254,103,
254,63,198,63,193,248,97,248,15,6,15,0,0,96,0,0,
15,0,0,0,240,0,0,15,0,0,
};
METRIC cmsy5_metric[128] = {
{855, -3, 5, 15, 2, 354992},
{1460, -3, 6, 3, 3, 150188},
{1572, -5, 11, 11, 14, 354992},
{1592, -2, 9, 10, 9, 241212},
{1604, -2, 12, 17, 16, 354992},
{1462, -2, 10, 11, 12, 241212},
{859, -2, 15, 17, 18, 354992},
{1638, -2, 11, 17, 18, 354992},
{898, -2, 13, 17, 18, 354992},
{1677, -2, 13, 17, 18, 354992},
{937, -2, 13, 17, 18, 354992},
{1716, -2, 13, 17, 18, 354992},
{1755, -2, 13, 17, 18, 354992},
{1794, -2, 13, 23, 18, 446016},
{1846, -2, 9, 10, 10, 241212},
{1859, -2, 9, 10, 10, 241212},
{1872, -2, 11, 17, 14, 354992},
{1401, -2, 10, 17, 12, 354992},
{1902, -3, 14, 15, 21, 354992},
{1942, -3, 14, 15, 21, 354992},
{1321, -3, 14, 15, 21, 354992},
{1361, -3, 14, 15, 21, 354992},
{1982, -3, 14, 15, 21, 354992},
{2022, -3, 14, 15, 21, 354992},
{2062, -2, 7, 17, 6, 354992},
{2075, -2, 10, 17, 11, 354992},
{1179, -3, 12, 15, 16, 354992},
{1209, -4, 12, 14, 16, 354992},
{2099, -2, 13, 23, 18, 446016},
{2151, -2, 13, 23, 18, 446016},
{2203, -3, 11, 15, 16, 354992},
{2233, -3, 11, 15, 16, 354992},
{1112, -3, 8, 22, 8, 446016},
{1134, -2, 8, 23, 8, 446016},
{1094, -3, 13, 8, 18, 241212},
{1076, -3, 13, 8, 18, 241212},
{1157, -3, 8, 22, 8, 446016},
{2263, -2, 13, 23, 18, 446016},
{2315, -2, 13, 23, 18, 446016},
{2367, -2, 10, 17, 12, 354992},
{2393, -2, 10, 23, 12, 446016},
{2428, -2, 10, 23, 12, 446016},
{2463, -1, 13, 15, 18, 286724},
{2497, -1, 13, 15, 18, 286724},
{2531, -2, 10, 23, 12, 446016},
{2566, -2, 13, 23, 18, 446016},
{2618, -2, 13, 23, 18, 446016},
{2670, -2, 8, 17, 9, 354992},
{2690, -2, 11, 5, 11, 144499},
{1521, -2, 8, 23, 9, 446016},
{1237, -3, 12, 13, 16, 309480},
{2697, -3, 12, 13, 16, 309480},
{2723, -2, 13, 20, 14, 400504},
{2758, -2, 9, 20, 14, 400504},
{2793, -5, 13, 11, 18, 0},
{2818, -2, 8, 2, 8, 0},
{1263, -1, 13, 14, 14, 263968},
{1288, -2, 13, 12, 14, 263968},
{1309, -2, 7, 15, 6, 309480},
{2820, -2, 15, 10, 17, 241212},
{2842, -2, 13, 17, 14, 332236},
{2872, -2, 13, 16, 14, 332236},
{2900, -3, 13, 17, 14, 354992},
{2930, -3, 13, 17, 14, 354992},
{2960, -2, 13, 13, 14, 286724},
{0, -1, 14, 19, 16, 364867},
{38, -2, 13, 15, 14, 301242},
{65, -2, 13, 12, 14, 256276},
{86, -2, 13, 18, 14, 355175},
{118, -2, 13, 13, 14, 252590},
{141, -2, 13, 18, 14, 323815},
{173, -2, 13, 13, 16, 278669},
{199, -2, 13, 20, 15, 391425},
{237, -2, 13, 15, 14, 268543},
{264, -2, 13, 19, 16, 314031},
{302, -2, 13, 17, 14, 345709},
{332, -3, 13, 14, 14, 316125},
{357, -2, 13, 27, 15, 513757},
{408, -1, 15, 24, 17, 368987},
{459, -3, 13, 17, 14, 356903},
{489, -2, 13, 17, 15, 326912},
{521, -4, 13, 17, 16, 370921},
{555, -2, 13, 20, 14, 391949},
{590, -2, 13, 15, 14, 284448},
{617, -2, 14, 18, 15, 269495},
{651, -1, 13, 16, 14, 286313},
{679, -2, 13, 16, 15, 294553},
{709, -2, 13, 25, 15, 448156},
{756, -2, 13, 19, 14, 326579},
{790, -1, 13, 17, 17, 304565},
{827, -3, 13, 16, 14, 324863},
{976, -2, 12, 15, 13, 309480},
{1001, -2, 12, 15, 13, 309480},
{2983, -2, 12, 15, 13, 309480},
{1026, -2, 12, 15, 13, 309480},
{1051, -2, 12, 15, 13, 309480},
{3008, -2, 13, 13, 14, 286724},
{3031, -3, 13, 12, 14, 286724},
{3052, -5, 15, 7, 22, 218456},
{3072, -1, 15, 7, 22, 218456},
{3092, -5, 15, 7, 22, 218456},
{3112, -1, 15, 7, 22, 218456},
{1479, -3, 15, 8, 21, 241212},
{1500, -3, 15, 8, 21, 241212},
{3132, -3, 15, 6, 22, 195700},
{3149, -2, 15, 6, 22, 195700},
{1454, -4, 15, 2, 21, 150188},
{3166, -3, 15, 8, 22, 241212},
{3188, -3, 15, 8, 21, 241212},
{3209, -1, 15, 15, 21, 286724},
{1427, -2, 15, 10, 21, 241212},
{3249, -2, 11, 5, 14, 150188},
{3258, -2, 1, 22, 22, 359544},
{3319, -2, 13, 17, 14, 339518},
{3349, -2, 13, 19, 14, 377748},
{1547, -2, 13, 11, 18, 207078},
{3383, -3, 12, 13, 13, 309480},
{3405, -3, 12, 13, 13, 309480},
{3427, -3, 14, 16, 21, 354992},
{3469, -2, 14, 16, 21, 354992},
{3511, -3, 13, 7, 18, 218456},
{3527, -2, 13, 10, 18, 218456},
{3550, -2, 13, 10, 18, 218456},
{3573, -2, 13, 13, 18, 286724},
{3603, -2, 13, 18, 17, 354992},
{3642, -2, 13, 18, 17, 354992},
{3681, -2, 13, 18, 14, 354992},
{3713, -2, 13, 18, 17, 354992},
};
char cmsy5_bitmap[3752] = {
0,2,0,0,192,0,24,0,7,128,1,176,0,38,0,8,
192,3,24,0,67,0,16,96,7,254,1,128,192,32,25,152,
3,31,0,121,128,0,24,124,113,28,108,24,240,33,193,131,
28,6,127,24,7,48,6,96,12,128,27,0,37,129,155,252,
0,3,240,195,16,50,2,96,68,0,192,12,0,192,12,0,
224,6,2,112,195,240,31,252,8,195,196,48,58,12,7,3,
0,192,192,48,32,12,24,2,6,1,129,128,64,64,32,48,
48,8,112,7,224,0,3,240,97,196,4,96,3,0,30,0,
62,3,0,32,2,0,48,1,128,78,12,63,128,7,255,194,
48,49,12,0,131,0,0,128,0,96,0,24,0,7,240,3,
0,0,128,0,96,0,16,0,200,0,60,0,0,3,248,96,
196,6,64,38,2,32,3,0,56,2,192,54,1,184,24,225,
195,244,0,97,6,31,192,31,0,54,48,2,67,0,104,48,
4,3,0,192,32,8,7,255,128,96,24,6,1,0,64,48,
12,3,0,192,48,8,3,17,128,51,0,3,224,31,254,192,
131,2,0,12,0,16,0,96,0,192,1,128,2,0,12,0,
24,0,32,128,131,31,252,0,0,63,224,24,32,4,12,1,
1,0,96,96,0,8,0,3,0,0,96,0,8,0,3,0,
0,96,16,8,4,3,1,192,64,28,48,1,248,0,24,15,
156,24,134,48,3,32,1,160,0,176,0,216,0,110,0,51,
128,16,192,24,112,12,28,52,7,22,1,240,0,240,4,224,
33,129,0,12,0,32,1,128,6,0,16,0,192,2,0,24,
1,127,10,15,192,0,192,0,32,24,0,8,7,0,3,0,
240,0,224,22,0,60,2,192,13,128,92,3,32,17,129,204,
2,56,113,128,131,24,48,16,118,6,2,7,128,200,128,224,
25,240,0,3,220,0,0,0,0,0,15,0,0,31,1,128,
16,1,128,32,1,192,32,1,192,32,2,224,64,2,96,64,
2,112,64,2,48,128,4,56,128,4,24,128,4,28,128,8,
13,0,136,15,0,240,7,0,112,0,0,6,62,4,99,132,
64,228,64,54,96,26,0,15,0,7,128,2,192,3,96,1,
56,1,14,1,3,131,0,254,0,31,254,17,131,144,192,240,
96,48,48,24,24,8,8,12,12,12,6,24,3,112,1,0,
1,128,0,128,0,192,0,64,0,1,248,3,15,6,1,132,
0,230,0,50,0,27,0,13,128,4,192,6,112,2,28,34,
7,230,0,12,3,248,4,63,4,1,252,31,255,130,48,28,
67,0,200,48,12,3,0,128,48,16,2,6,0,103,128,6,
56,0,97,128,4,28,0,192,225,8,7,33,128,60,1,252,
4,28,16,24,96,0,192,1,224,0,252,0,28,0,28,192,
27,0,54,0,79,3,7,248,0,0,0,71,255,226,6,1,
1,128,192,64,0,48,0,12,0,2,0,1,128,0,96,0,
16,0,12,0,2,0,1,0,1,252,0,120,1,152,3,24,
2,16,6,48,6,32,12,96,28,96,28,64,44,192,72,192,
152,193,24,230,24,120,28,224,14,112,15,56,1,24,1,24,
1,28,2,12,6,12,12,12,24,12,48,12,96,13,192,15,
128,14,0,12,0,224,6,3,56,7,1,204,3,192,39,2,
96,17,129,48,8,193,28,8,97,134,4,49,131,4,24,129,
134,12,128,230,6,192,54,3,192,30,1,192,14,0,192,6,
0,64,2,0,3,192,224,152,12,3,129,0,48,64,6,48,
0,216,0,28,0,15,128,6,48,3,6,0,128,192,32,24,
12,3,145,192,60,0,60,7,39,3,193,192,32,112,16,24,
8,12,8,6,4,3,132,0,198,0,102,0,54,0,30,0,
14,0,14,3,14,0,252,0,60,0,0,7,241,8,62,16,
4,0,8,0,16,0,32,7,248,1,16,2,0,4,0,24,
2,32,6,127,12,131,240,255,255,255,252,0,128,0,96,0,
48,0,24,0,12,0,6,0,3,0,1,128,255,255,255,255,
192,48,0,24,0,12,0,6,0,3,0,1,128,255,255,255,
255,192,3,224,6,76,4,33,4,16,68,8,18,4,10,2,
3,1,1,255,255,192,64,96,32,48,16,20,8,18,4,8,
130,8,33,8,12,152,1,240,0,3,224,6,12,4,1,6,
0,196,128,146,32,138,8,131,2,129,128,128,192,64,96,80,
48,68,20,65,18,64,72,192,24,32,8,12,24,1,240,0,
64,3,128,15,0,30,0,60,0,120,0,240,1,224,3,192,
7,128,13,192,113,255,192,254,0,15,224,127,241,192,118,0,
60,0,120,0,240,1,224,3,192,7,128,15,0,30,0,52,
0,32,1,0,7,0,14,0,54,0,198,1,140,6,12,12,
24,48,24,192,25,128,54,0,60,0,32,192,3,128,13,128,
51,0,99,1,131,6,6,12,6,48,12,96,13,128,14,0,
28,0,16,0,24,24,24,24,24,24,24,24,24,24,24,24,
24,24,219,60,24,8,8,24,60,219,24,24,24,24,24,24,
24,24,24,24,24,24,24,24,16,0,0,64,0,2,0,0,
63,255,255,255,255,252,128,0,1,0,0,4,0,0,0,0,
32,0,0,64,0,0,71,255,255,255,255,255,224,0,2,0,
0,8,0,0,16,16,0,64,64,1,2,0,2,63,255,255,
255,255,252,128,0,129,0,4,4,0,16,7,254,63,252,224,
3,0,6,0,24,0,48,0,96,0,192,1,128,3,0,3,
0,6,0,7,0,7,255,131,255,255,131,255,128,7,0,6,
0,24,0,48,0,192,3,0,12,0,48,0,192,6,0,24,
1,207,254,63,224,3,248,127,199,0,96,6,0,48,3,0,
31,255,255,254,0,24,0,192,3,0,14,0,63,224,127,64,
7,0,54,1,152,6,63,240,255,193,134,6,24,12,192,51,
0,120,1,224,3,0,12,0,255,255,255,0,48,3,0,48,
3,127,247,255,0,48,3,0,48,3,255,255,255,255,255,255,
252,0,24,0,48,0,96,0,192,0,6,0,60,1,224,7,
0,56,1,224,15,0,120,0,240,0,120,0,56,0,28,0,
30,0,15,0,7,128,3,0,0,0,0,0,3,255,239,255,
224,192,1,224,0,240,0,112,0,56,0,60,0,30,0,15,
0,30,0,240,3,128,28,0,240,7,128,28,0,96,0,0,
0,0,0,0,3,255,239,255,224,255,255,191,255,192,0,0,
0,0,0,7,255,255,255,254,0,0,0,0,0,0,31,255,
255,255,240,192,48,6,1,128,96,12,3,0,96,24,3,0,
192,48,6,1,128,48,12,1,128,96,24,3,0,64,255,255,
255,255,255,192,93,0,4,1,192,108,24,198,13,128,240,27,
6,49,131,96,56,2,0,7,12,24,24,24,24,24,24,24,
48,224,48,24,24,24,24,24,24,24,12,7,224,48,24,24,
24,24,24,24,24,12,7,12,24,24,24,24,24,24,24,48,
224,31,1,240,239,134,25,3,176,20,3,192,24,3,128,48,
7,128,80,27,129,48,195,238,31,1,240,0,192,44,13,129,
128,48,6,0,128,48,6,0,192,24,2,0,192,24,3,6,
64,200,14,0,64,56,13,131,24,193,176,54,3,128,112,27,
3,96,198,48,108,6,128,64,12,3,14,221,254,30,31,238,
220,48,12,0,0,128,0,224,0,248,0,56,0,8,0,0,
0,0,1,255,255,255,255,128,0,0,0,0,16,0,28,0,
31,0,7,0,1,0,255,255,255,255,192,48,0,24,0,12,
0,6,0,3,0,1,128,255,255,255,255,192,48,0,24,0,
12,0,6,0,3,0,1,128,0,192,0,64,0,3,224,6,
12,4,1,4,0,68,0,18,0,10,0,3,0,1,255,255,
192,0,96,0,48,0,20,0,18,0,8,128,8,32,8,12,
24,1,240,0,3,224,6,12,4,1,4,0,196,0,146,0,
138,0,131,0,129,128,128,192,64,96,64,48,64,20,64,18,
64,8,192,8,32,8,12,24,1,240,0,3,224,6,12,4,
1,4,0,68,0,18,0,10,0,3,3,129,129,192,192,224,
96,112,48,0,20,0,18,0,8,128,8,32,8,12,24,1,
240,0,0,254,0,7,255,0,60,7,128,224,3,131,0,1,
140,0,1,152,0,3,96,0,3,192,0,7,128,0,15,0,
0,27,0,0,102,0,0,198,0,3,7,0,28,7,128,240,
3,255,128,1,252,0,30,15,198,27,3,192,240,60,13,134,
63,7,128,30,15,199,251,255,255,255,255,253,254,63,7,128,
128,0,240,1,220,1,195,131,128,255,128,31,0,0,0,0,
0,3,224,7,252,7,7,14,0,238,0,60,0,4,7,254,
63,252,224,3,0,6,0,24,0,48,0,96,0,192,1,128,
3,0,3,0,6,0,7,0,7,255,131,255,0,0,0,0,
0,3,255,247,255,224,127,192,255,224,0,224,0,96,0,192,
0,192,1,128,3,0,6,0,12,0,24,0,96,0,192,7,
31,252,63,224,0,0,0,0,0,7,255,239,255,192,0,2,
0,12,0,24,0,96,1,128,14,0,248,127,192,255,128,7,
192,1,192,0,192,0,192,0,192,1,128,1,0,0,0,0,
0,7,255,255,255,224,128,1,128,3,0,3,0,3,0,3,
128,3,224,1,255,3,254,31,0,112,1,128,6,0,24,0,
48,0,96,0,0,0,0,0,0,7,255,255,255,224,62,0,
191,192,127,240,120,63,248,7,244,1,240,62,0,255,224,255,
255,248,63,248,3,224,0,0,248,3,255,131,255,255,224,255,
224,15,128,0,3,6,0,30,60,0,112,224,3,135,0,30,
60,0,241,224,3,135,0,28,56,0,241,224,1,227,192,0,
225,192,0,112,224,0,120,240,0,60,120,0,28,56,0,14,
28,0,15,30,0,6,12,193,128,1,227,192,0,225,192,0,
112,224,0,120,240,0,60,120,0,28,56,0,14,28,0,15,
30,0,30,60,0,112,224,3,135,0,30,60,0,241,224,3,
135,0,28,56,0,241,224,1,131,0,0,0,2,0,12,0,
24,0,96,1,128,14,0,248,127,192,255,128,7,192,1,192,
0,192,0,192,0,192,1,128,3,128,1,128,3,0,3,0,
3,0,3,128,3,224,1,255,3,254,31,0,112,1,128,6,
0,24,0,48,0,96,0,0,0,102,0,0,60,0,0,240,
0,3,160,0,14,32,0,48,64,1,192,0,7,0,0,28,
0,0,224,0,3,128,0,14,0,0,112,0,1,192,0,7,
0,0,56,0,0,224,0,1,128,0,0,192,0,1,192,0,
1,192,0,0,224,0,0,224,0,0,224,0,0,112,0,0,
112,0,0,112,0,0,56,0,0,56,0,0,56,0,0,24,
32,0,28,64,0,29,0,0,30,0,0,30,0,0,204,62,
0,191,192,127,240,120,63,248,7,244,1,240,0,0,0,0,
0,0,0,0,31,255,255,255,240,0,192,0,3,0,0,15,
255,240,63,255,241,192,0,30,0,0,14,0,0,7,0,0,
7,255,254,7,255,248,6,0,0,6,0,0,0,6,0,0,
6,1,255,254,7,255,254,0,0,7,0,0,3,192,0,14,
0,0,112,255,255,192,255,255,0,0,12,0,0,48,0,1,
0,7,0,14,0,54,0,198,7,143,63,31,230,51,12,96,
24,192,49,128,99,0,198,1,140,3,24,6,48,12,96,24,
192,12,96,24,192,49,128,99,0,198,1,140,3,24,6,48,
12,96,24,195,49,159,227,243,199,129,140,1,176,1,192,3,
128,2,0,1,131,0,6,3,0,31,255,0,127,255,3,128,
3,158,0,3,206,0,14,14,0,56,15,255,224,15,255,128,
12,6,0,12,24,0,204,0,1,224,0,1,224,0,2,224,
0,8,224,0,16,96,0,0,112,0,0,112,0,0,112,0,
0,56,0,0,56,0,0,56,0,0,28,0,0,28,0,0,
28,0,0,14,0,0,14,0,0,12,0,0,6,0,0,28,
0,0,112,0,3,128,0,14,0,0,56,0,1,192,0,7,
0,0,28,0,0,224,0,3,128,0,14,0,8,48,0,17,
192,0,23,0,0,60,0,0,240,0,1,152,0,0,30,3,
187,199,16,54,16,14,8,7,4,3,129,3,96,199,30,30,
3,128,24,198,99,25,140,102,48,126,3,252,0,112,0,192,
3,0,24,0,127,255,255,248,0,192,12,0,96,6,0,225,
254,15,192,0,96,0,6,0,0,240,0,25,128,3,12,0,
48,192,6,6,0,192,48,24,1,129,128,24,48,0,198,0,
6,255,255,247,255,255,127,255,255,255,255,96,0,99,0,12,
24,1,129,128,24,12,3,0,96,96,3,12,0,48,192,1,
152,0,15,0,0,96,0,6,0,0,96,12,3,0,96,24,
6,0,192,48,6,1,128,48,12,1,128,96,24,3,0,192,
8,0,255,255,1,0,193,241,158,70,177,188,111,51,204,243,
61,143,99,216,214,39,24,248,48,0,120,56,67,102,32,195,
136,96,194,48,48,156,12,79,253,39,96,99,176,1,152,0,
198,8,99,12,97,153,224,112,31,128,96,224,128,49,128,14,
128,0,128,14,96,25,30,49,0,24,0,12,240,12,28,12,
7,12,1,248,255,255,255,255,192,96,0,48,0,24,0,12,
0,6,0,3,0,1,128,0,192,0,96,0,48,0,24,0,
4,0,0,128,0,192,0,96,0,48,0,24,0,12,0,6,
0,3,0,1,128,0,192,0,96,0,48,15,255,255,255,252,
128,134,7,24,28,96,113,130,142,16,153,12,104,97,195,6,
12,24,96,97,129,248,4,64,3,128,15,0,30,12,60,24,
120,248,241,241,224,195,193,135,128,13,192,113,255,192,254,0,
64,6,0,48,1,128,12,0,96,3,255,255,255,192,6,0,
48,1,128,12,0,32,0,0,16,3,0,48,3,0,48,3,
255,255,255,0,48,3,0,48,3,0,48,1,65,131,6,12,
24,48,96,193,131,6,12,24,48,96,193,131,6,15,239,192,
2,12,24,48,96,193,131,6,12,24,48,96,193,131,6,12,
24,63,255,192,127,255,6,12,24,48,96,193,131,6,12,24,
48,96,193,131,6,12,8,0,255,252,24,48,96,193,131,6,
12,24,48,96,193,131,6,12,24,48,96,64,4,49,134,24,
195,24,97,140,48,97,134,12,48,97,134,12,16,195,6,24,
96,195,6,24,96,195,24,97,140,49,134,24,195,0,66,195,
195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,195,
195,195,195,66,24,24,60,219,24,24,24,24,24,24,24,24,
24,24,24,24,24,219,60,24,24,1,0,7,0,31,0,119,
3,199,159,143,243,25,134,48,12,96,24,192,49,128,99,0,
198,1,140,51,25,254,63,60,120,29,192,31,0,28,0,16,
0,241,206,115,157,206,231,57,206,60,0,0,4,0,0,48,
0,1,128,0,12,0,0,48,0,1,128,0,12,0,0,48,
0,1,128,0,12,0,0,48,24,1,128,176,6,0,192,48,
1,129,128,3,6,0,12,48,0,25,128,0,102,0,0,240,
0,3,128,0,6,0,0,252,31,152,3,12,1,134,0,195,
0,97,128,48,192,24,96,12,48,6,24,3,12,1,134,0,
195,0,103,255,252,255,255,239,255,248,192,2,28,0,65,128,
16,24,4,3,128,128,48,32,7,12,0,97,0,6,64,0,
248,0,14,0,0,128,0,192,30,0,240,7,128,60,1,224,
15,0,120,3,192,30,0,240,7,255,247,255,128,255,255,255,
240,7,128,60,1,224,15,0,120,3,192,30,0,240,7,128,
52,0,128,127,255,255,255,192,0,192,0,192,0,192,0,192,
0,192,0,192,0,192,0,192,0,192,0,192,0,192,0,255,
255,255,255,0,0,0,0,0,0,127,254,255,255,255,255,255,
255,0,3,0,3,0,3,0,3,0,3,0,3,0,3,0,
3,0,3,0,3,0,3,0,3,127,255,255,255,0,0,0,
0,0,0,127,255,255,255,56,143,30,12,12,30,98,199,141,
25,224,192,193,227,196,112,12,3,0,192,48,255,255,240,192,
48,12,3,0,192,48,12,3,0,192,48,12,3,0,12,3,
0,192,48,255,195,0,192,48,12,0,0,192,48,12,63,255,
252,48,12,3,0,31,249,255,223,153,252,207,230,127,51,249,
143,204,62,96,243,1,152,12,192,102,3,48,25,128,204,6,
96,51,0,3,240,1,254,0,255,192,63,240,15,252,1,254,
0,63,0,223,236,127,255,191,255,255,255,255,255,255,124,79,
142,49,192,12,0,7,128,1,224,0,0,192,0,48,0,30,
0,12,192,6,24,3,3,3,128,113,192,14,192,0,220,0,
227,128,112,48,48,6,24,0,204,0,30,0,3,0,0,192,
0,63,63,31,255,236,12,15,1,3,192,0,240,0,54,0,
24,192,12,24,6,3,135,0,115,128,7,128,0,192,0,48,
0,0,192,0,48,0,30,0,15,192,15,252,7,255,131,255,
241,255,254,255,255,255,255,255,255,255,255,255,124,79,142,49,
192,12,0,7,128,1,224,0,
};
METRIC cmsy7_metric[128] = {
{1522, -3, 7, 18, 2, 409602},
{2435, -3, 8, 3, 3, 155649},
{2630, -5, 14, 15, 16, 409602},
{2660, -2, 13, 11, 13, 268517},
{2678, -2, 16, 21, 20, 409602},
{2437, -1, 14, 15, 16, 268517},
{1527, -2, 20, 20, 22, 409602},
{2731, -2, 14, 20, 22, 409602},
{1582, -2, 17, 20, 22, 409602},
{2786, -2, 17, 20, 22, 409602},
{1637, -2, 17, 20, 22, 409602},
{2841, -2, 17, 20, 22, 409602},
{2896, -2, 17, 20, 22, 409602},
{2951, -2, 19, 27, 26, 522470},
{3039, -2, 13, 11, 14, 268517},
{3059, -2, 13, 11, 14, 268517},
{3079, -2, 14, 20, 16, 409602},
{2347, -2, 14, 20, 16, 409602},
{3119, -3, 19, 18, 27, 409602},
{3180, -3, 19, 18, 27, 409602},
{2225, -3, 19, 18, 27, 409602},
{2286, -3, 19, 18, 27, 409602},
{3241, -3, 19, 18, 27, 409602},
{3302, -3, 19, 18, 27, 409602},
{3363, -2, 10, 20, 8, 409602},
{3383, -2, 14, 20, 15, 409602},
{2006, -3, 16, 18, 20, 409602},
{2051, -4, 16, 17, 20, 409602},
{3421, -2, 17, 27, 22, 522470},
{3496, -2, 17, 27, 22, 522470},
{3571, -3, 16, 18, 20, 409602},
{3616, -3, 16, 18, 20, 409602},
{1906, -3, 11, 26, 10, 522470},
{1939, -2, 11, 27, 10, 522470},
{1867, -2, 19, 12, 26, 268517},
{1828, -2, 19, 12, 26, 268517},
{1973, -3, 11, 26, 10, 522470},
{3661, -2, 19, 27, 27, 522470},
{3753, -2, 20, 27, 27, 522470},
{3845, -2, 13, 20, 15, 409602},
{3883, -2, 15, 27, 18, 522470},
{3944, -2, 15, 27, 18, 522470},
{4005, 0, 19, 19, 26, 324951},
{4067, 0, 19, 19, 26, 324951},
{4129, -2, 15, 27, 18, 522470},
{4190, -2, 19, 27, 27, 522470},
{4282, -2, 20, 27, 27, 522470},
{4374, -2, 12, 20, 13, 409602},
{4407, -1, 15, 7, 15, 151097},
{2547, -2, 12, 27, 13, 522470},
{2094, -3, 16, 14, 20, 353168},
{4421, -3, 16, 14, 20, 353168},
{4456, -2, 19, 24, 20, 466036},
{4516, -2, 13, 24, 20, 466036},
{4576, -4, 19, 16, 26, 0},
{4628, -2, 11, 2, 10, 0},
{2129, 0, 19, 18, 20, 296734},
{2174, -2, 19, 13, 20, 296734},
{2207, -2, 10, 16, 9, 353168},
{4631, -2, 21, 11, 24, 268517},
{4664, -1, 19, 21, 20, 381385},
{4717, -2, 19, 19, 20, 381385},
{4765, -3, 19, 19, 20, 409602},
{4813, -3, 19, 19, 20, 409602},
{4861, -2, 19, 15, 20, 324951},
{0, -1, 20, 23, 22, 420952},
{64, -2, 19, 18, 20, 345641},
{109, 0, 19, 16, 20, 284511},
{149, -1, 19, 23, 20, 408037},
{207, -1, 19, 16, 20, 282627},
{247, -1, 19, 24, 20, 375422},
{307, -1, 19, 17, 23, 315859},
{356, -1, 19, 24, 21, 448955},
{419, 0, 19, 19, 20, 296584},
{467, -2, 19, 24, 23, 358812},
{536, -1, 19, 22, 20, 399884},
{591, -2, 19, 18, 20, 363200},
{636, -1, 19, 34, 21, 615961},
{726, 0, 22, 31, 24, 429195},
{819, -1, 19, 22, 20, 415555},
{874, -1, 19, 22, 21, 371200},
{932, -3, 19, 20, 24, 429355},
{992, -1, 19, 25, 20, 450052},
{1055, 0, 19, 19, 20, 322130},
{1103, -1, 20, 23, 21, 297189},
{1164, 1, 19, 22, 20, 329292},
{1219, -1, 19, 19, 21, 330268},
{1269, -1, 19, 31, 21, 520733},
{1351, -2, 19, 22, 20, 375652},
{1406, -1, 19, 21, 24, 350657},
{1469, -2, 19, 21, 20, 377618},
{1692, -2, 16, 16, 17, 353168},
{1726, -2, 16, 16, 17, 353168},
{4899, -2, 16, 16, 17, 353168},
{1760, -2, 16, 16, 17, 353168},
{1794, -2, 16, 16, 17, 353168},
{4933, -2, 19, 15, 20, 324951},
{4971, -3, 19, 14, 20, 324951},
{5006, -5, 21, 8, 30, 240300},
{5036, 0, 21, 8, 30, 240300},
{5066, -5, 21, 8, 30, 240300},
{5096, 0, 21, 8, 30, 240300},
{2467, -2, 21, 11, 29, 268517},
{2507, -2, 21, 11, 29, 268517},
{5126, -3, 21, 6, 30, 212083},
{5149, -2, 21, 6, 30, 212083},
{2427, -3, 21, 2, 29, 155649},
{5172, -3, 21, 9, 30, 268517},
{5206, -2, 21, 12, 29, 268517},
{5250, 0, 21, 19, 29, 324951},
{2387, -2, 21, 11, 29, 268517},
{5319, -2, 16, 4, 20, 155649},
{5329, -1, 1, 26, 30, 430537},
{5427, -1, 19, 21, 20, 394129},
{5480, -2, 19, 22, 20, 437819},
{2591, -2, 19, 12, 26, 226420},
{5535, -2, 16, 16, 17, 353168},
{5569, -2, 16, 16, 17, 353168},
{5603, -3, 19, 19, 27, 409602},
{5668, -2, 19, 19, 27, 409602},
{5733, -2, 19, 9, 26, 240301},
{5763, -2, 19, 10, 26, 240300},
{5796, -2, 19, 10, 26, 240300},
{5829, -2, 19, 16, 26, 324951},
{5881, -1, 19, 22, 24, 409602},
{5947, -3, 19, 19, 24, 409602},
{6004, -2, 19, 20, 20, 409602},
{6054, -2, 19, 20, 24, 409602},
};
char cmsy7_bitmap[6114] = {
0,0,96,0,1,192,0,3,128,0,15,0,0,62,0,0,
124,0,1,184,0,2,112,0,12,224,0,49,192,0,99,128,
1,135,0,6,14,0,12,30,0,63,252,0,255,248,3,128,
113,6,0,227,56,1,231,224,3,255,192,3,134,0,0,0,
6,15,135,143,242,196,60,50,7,29,1,135,128,65,224,32,
112,112,28,126,6,63,227,128,252,224,15,56,1,204,0,115,
0,25,192,6,96,3,27,129,141,255,130,63,128,0,63,0,
255,3,7,4,6,8,6,24,12,48,8,48,0,96,0,96,
0,224,0,224,0,224,0,224,0,240,0,240,2,120,12,124,
16,63,224,31,128,3,255,192,31,255,224,195,135,243,6,3,
236,12,1,240,56,3,192,112,3,128,224,7,1,192,14,3,
0,24,6,0,48,28,0,224,48,1,128,96,2,1,192,8,
3,0,32,6,1,128,24,28,0,63,224,0,254,0,0,0,
126,1,255,2,15,4,6,12,4,28,0,30,0,31,0,15,
224,7,192,12,0,16,0,48,0,96,0,96,0,224,4,240,
24,248,32,127,192,63,0,0,63,255,0,255,255,3,12,6,
6,28,0,12,28,0,8,24,0,0,24,0,0,56,0,0,
48,0,0,63,240,0,127,224,0,96,0,0,96,0,0,192,
0,0,192,0,1,128,0,1,128,0,115,0,0,254,0,0,
120,0,0,0,63,0,255,192,193,224,128,96,128,48,128,48,
192,16,192,0,96,0,112,0,184,0,220,0,238,0,119,128,
115,192,56,248,124,127,204,15,134,0,6,0,2,1,2,1,
254,1,252,0,3,224,1,31,224,3,48,224,6,96,224,6,
192,224,12,128,192,12,0,192,24,1,192,24,1,255,240,1,
255,240,1,128,112,3,128,112,3,0,96,3,0,96,7,0,
224,6,0,224,6,0,224,12,0,225,12,0,242,24,0,252,
0,0,120,3,255,225,255,248,64,96,16,12,4,3,0,0,
96,0,28,0,3,0,0,96,0,28,0,3,128,0,96,0,
12,0,3,128,0,96,0,12,0,3,3,0,64,193,255,240,
127,248,0,0,3,255,0,31,254,0,48,48,0,64,96,0,
128,64,1,128,192,3,1,128,2,1,128,0,3,128,0,3,
0,0,3,0,0,7,0,0,7,0,0,6,0,0,6,0,
0,14,0,96,12,0,224,12,0,224,24,0,240,16,0,248,
32,0,127,192,0,63,0,0,3,0,120,60,6,113,112,96,
129,198,0,7,32,0,25,0,0,104,0,3,224,0,15,128,
0,55,0,0,204,0,7,48,0,24,224,0,97,128,3,135,
0,12,12,0,48,56,9,128,112,102,0,227,48,1,240,0,
15,0,15,192,4,120,2,28,1,2,0,192,0,112,0,24,
0,6,0,3,128,0,192,0,48,0,28,0,6,0,1,128,
0,192,1,63,0,159,248,76,127,226,3,240,0,24,0,0,
128,14,0,0,96,3,128,0,56,0,240,0,30,0,124,0,
7,128,23,0,3,224,5,192,1,184,1,120,0,236,0,94,
0,115,0,35,128,57,192,8,240,28,112,2,60,14,28,1,
7,15,7,0,65,231,129,192,48,123,192,112,8,15,192,28,
6,3,224,7,35,0,112,1,207,192,8,0,127,224,0,0,
30,112,0,0,0,0,0,0,0,30,0,0,0,252,0,0,
3,240,1,128,7,0,7,0,8,0,15,0,32,0,30,0,
64,0,60,0,128,0,124,2,0,1,120,4,0,2,112,8,
0,4,240,16,0,9,224,64,0,17,224,128,0,67,193,0,
0,131,130,0,1,7,136,0,4,15,16,0,8,15,32,0,
48,30,64,8,64,31,0,31,128,62,0,62,0,60,0,60,
0,48,0,0,67,224,6,63,224,33,15,129,8,15,8,96,
60,67,0,115,8,1,220,64,7,96,0,29,128,0,110,0,
1,184,0,14,224,0,51,128,0,143,0,4,62,0,32,124,
1,0,248,24,1,255,128,3,248,0,3,255,192,63,255,195,
14,31,152,48,31,192,192,62,7,0,112,28,1,128,112,6,
1,192,48,6,0,128,24,4,0,224,224,3,62,0,13,192,
0,112,0,1,128,0,6,0,0,56,0,0,192,0,3,0,
0,24,0,0,0,63,0,15,252,3,7,224,64,30,8,0,
241,0,15,48,0,118,0,7,96,0,126,0,7,224,0,110,
0,6,240,0,207,0,8,124,49,135,254,48,31,6,0,0,
128,0,112,3,252,1,127,192,48,255,134,1,255,192,3,240,
3,255,240,7,255,252,12,56,31,12,24,7,140,12,1,196,
14,0,192,7,0,96,3,128,96,1,192,32,0,192,96,0,
99,192,0,115,192,0,48,240,0,24,124,0,28,30,0,12,
7,128,6,3,224,134,0,248,131,0,63,131,0,15,0,0,
31,192,15,252,2,7,128,128,112,48,12,14,0,1,224,0,
62,0,3,248,0,63,192,0,252,0,7,193,128,120,96,7,
24,0,231,0,24,240,2,31,129,129,255,192,15,224,0,0,
0,6,15,255,248,63,255,193,128,96,6,1,128,12,3,0,
48,14,0,0,28,0,0,48,0,0,96,0,1,192,0,3,
0,0,6,0,0,28,0,0,48,0,0,96,0,1,128,0,
2,0,0,8,0,0,127,128,1,254,0,0,15,0,12,252,
0,100,112,1,129,128,12,6,0,48,56,1,192,192,6,3,
0,56,24,0,224,96,7,131,0,60,12,0,176,96,5,193,
128,38,14,1,24,56,8,224,224,67,131,198,14,15,224,62,
30,0,224,112,1,159,0,57,240,7,159,0,49,224,2,28,
0,67,192,24,120,2,7,0,192,224,48,28,14,3,131,128,
112,96,14,56,1,206,0,59,192,7,224,0,248,0,30,0,
3,128,0,96,0,112,0,96,13,240,1,192,29,240,7,128,
57,224,15,128,17,192,63,0,35,192,79,0,71,129,158,1,
135,6,28,2,14,12,56,12,28,48,120,16,56,192,240,96,
113,128,225,128,230,1,195,1,216,3,140,3,240,7,176,7,
192,15,224,15,0,15,128,28,0,30,0,56,0,56,0,96,
0,96,0,128,0,128,0,0,240,28,15,192,240,103,129,192,
30,6,0,56,16,0,225,128,3,136,0,14,192,0,60,0,
0,224,0,7,128,0,46,0,3,60,0,16,240,1,129,192,
8,7,0,96,28,3,128,121,15,1,248,56,3,192,15,0,
97,252,3,145,240,6,7,128,16,30,0,128,240,4,3,128,
64,28,2,0,240,16,7,129,0,28,24,0,224,128,7,8,
0,56,192,1,204,0,14,192,0,124,0,3,192,0,28,1,
0,192,8,12,0,97,192,1,252,0,7,128,0,0,127,8,
15,255,128,129,248,12,0,192,192,12,0,0,64,0,4,0,
0,64,0,12,0,15,252,0,8,0,0,128,0,8,0,0,
128,0,8,0,193,128,12,16,0,97,254,6,31,255,225,3,
252,0,255,255,255,255,240,0,32,0,6,0,0,96,0,6,
0,0,96,0,6,0,0,96,0,6,0,0,96,0,6,0,
255,255,255,255,255,0,96,0,6,0,0,96,0,6,0,0,
96,0,6,0,0,96,0,6,0,255,255,255,255,255,1,248,
0,98,96,8,33,1,2,8,32,32,66,2,4,64,32,36,
2,2,128,32,24,2,1,255,255,248,2,1,128,32,24,2,
1,64,32,36,2,2,32,32,66,2,4,16,32,128,130,16,
6,38,0,31,128,1,248,0,96,96,8,1,1,0,8,40,
1,66,64,36,68,2,36,32,66,129,8,24,9,1,128,96,
24,6,1,128,144,24,16,129,66,4,36,64,34,36,2,66,
128,20,16,0,128,128,16,6,6,0,31,128,64,1,192,3,
192,3,192,3,192,3,192,3,192,3,192,3,192,3,192,3,
192,3,192,3,192,3,96,6,56,28,31,248,7,224,7,224,
31,248,56,28,96,6,192,3,192,3,192,3,192,3,192,3,
192,3,192,3,192,3,192,3,192,3,192,3,192,3,64,1,
1,128,1,128,3,192,3,192,6,96,6,96,12,48,12,48,
24,24,24,24,24,24,48,12,48,12,96,6,96,6,192,3,
192,1,192,1,192,3,96,6,96,6,48,12,48,12,24,24,
24,24,24,24,12,48,12,48,6,96,6,96,3,192,3,192,
1,128,1,128,6,0,96,6,0,96,6,0,96,6,0,96,
6,0,96,6,0,96,6,0,96,6,0,96,6,0,96,6,
0,96,134,22,102,22,128,240,6,0,32,2,0,96,15,1,
104,102,104,97,6,0,96,6,0,96,6,0,96,6,0,96,
6,0,96,6,0,96,6,0,96,6,0,96,6,0,96,6,
0,96,8,0,0,2,0,0,1,0,0,0,128,0,0,255,
255,255,255,255,255,242,0,0,0,64,0,0,8,0,0,2,
0,0,0,0,0,4,0,0,0,128,0,0,8,0,0,0,
143,255,255,255,255,255,255,192,0,0,64,0,0,16,0,0,
4,0,0,0,128,8,0,8,2,0,2,1,0,0,64,128,
0,8,255,255,255,255,255,255,242,0,0,32,64,0,16,8,
0,8,2,0,2,0,3,255,195,255,241,192,0,192,0,96,
0,24,0,12,0,3,0,0,192,0,48,0,12,0,3,0,
0,192,0,48,0,6,0,1,128,0,48,0,7,0,0,255,
252,15,255,255,224,127,252,0,7,0,0,192,0,48,0,24,
0,6,0,3,0,1,128,0,192,0,96,0,48,0,24,0,
12,0,12,0,6,0,6,0,14,63,254,31,252,0,3,252,
63,241,192,12,0,96,1,128,6,0,48,0,192,3,255,255,
255,240,0,192,1,128,6,0,24,0,48,0,112,0,255,192,
255,64,0,112,0,54,0,25,128,6,48,3,12,0,195,255,
240,127,248,24,6,3,3,0,192,192,24,96,6,24,0,204,
0,51,0,12,192,1,224,0,120,0,12,0,3,0,255,255,
255,192,6,0,48,1,128,12,0,96,3,0,27,255,223,254,
0,48,1,128,12,0,96,3,0,24,0,255,255,255,240,255,
255,255,255,0,3,0,3,0,3,0,3,0,3,0,3,0,
3,0,0,192,0,240,0,112,0,112,0,120,0,120,0,120,
0,56,0,56,0,60,0,15,0,0,224,0,14,0,1,224,
0,30,0,1,224,0,28,0,1,192,0,60,0,3,0,0,
0,0,0,0,0,0,0,0,0,31,255,239,255,252,192,0,
60,0,3,128,0,56,0,7,128,0,120,0,7,128,0,112,
0,7,0,0,240,0,60,0,28,0,28,0,30,0,30,0,
30,0,14,0,14,0,7,0,3,0,0,0,0,0,0,0,
0,0,0,0,0,0,31,255,239,255,252,255,255,247,255,255,
0,0,0,0,0,0,0,0,0,0,0,0,15,255,255,255,
255,240,0,0,0,0,0,0,0,0,0,0,0,0,127,255,
255,255,255,192,24,3,0,48,6,0,192,12,1,128,48,3,
0,96,12,0,192,24,3,0,48,6,0,96,12,1,128,24,
3,0,96,6,0,192,24,1,128,48,2,255,255,255,255,255,
255,255,192,93,0,1,0,7,0,27,0,99,1,131,6,3,
24,3,96,3,192,6,192,24,192,96,193,128,198,0,216,0,
224,0,128,0,224,96,24,3,0,96,12,1,128,48,6,0,
192,24,3,0,96,56,60,0,224,6,0,192,24,3,0,96,
12,1,128,48,6,0,192,24,1,128,14,240,3,128,24,3,
0,96,12,1,128,48,6,0,192,24,3,0,96,6,0,56,
24,6,0,192,24,3,0,96,12,1,128,48,6,0,192,24,
14,15,0,15,0,30,7,120,12,49,129,130,1,32,24,128,
40,1,160,3,0,28,0,96,3,128,12,0,112,1,128,11,
0,40,2,48,9,0,131,3,24,96,61,192,240,1,224,0,
224,9,1,176,19,1,0,48,3,0,48,3,0,48,6,0,
96,6,0,96,6,0,96,12,0,192,12,0,192,12,0,128,
200,13,128,144,7,0,128,3,128,13,128,49,128,193,131,1,
140,1,176,1,192,3,128,13,128,49,128,193,131,1,140,1,
176,1,192,1,4,0,128,16,98,62,78,127,3,129,252,228,
248,140,16,2,0,64,0,32,0,3,128,0,62,0,0,224,
0,2,0,0,0,0,0,0,0,0,0,0,0,7,255,255,
255,255,254,0,0,0,0,0,0,0,0,0,0,0,4,0,
0,112,0,7,192,0,28,0,0,64,0,255,255,255,255,255,
0,96,0,6,0,0,96,0,6,0,0,96,0,6,0,0,
96,0,6,0,255,255,255,255,255,0,96,0,6,0,0,96,
0,6,0,0,96,0,6,0,0,96,0,6,0,0,96,0,
2,0,1,248,0,96,96,8,1,1,0,8,32,0,66,0,
4,64,0,36,0,2,128,0,24,0,1,255,255,248,0,1,
128,0,24,0,1,64,0,36,0,2,32,0,66,0,4,16,
0,128,128,16,6,6,0,31,128,1,248,0,96,96,8,1,
1,0,8,32,1,66,0,36,64,2,36,0,66,128,8,24,
1,1,128,32,24,4,1,128,128,24,16,1,66,0,36,64,
2,36,0,66,128,4,16,0,128,128,16,6,6,0,31,128,
1,248,0,96,96,8,1,1,0,8,32,0,66,0,4,64,
0,36,0,2,128,0,24,6,1,128,240,24,15,1,128,96,
24,0,1,64,0,36,0,2,32,0,66,0,4,16,0,128,
128,16,6,6,0,31,128,0,63,128,0,31,252,0,15,1,
224,3,128,14,0,192,0,96,48,0,6,12,0,0,97,128,
0,12,96,0,0,204,0,0,27,0,0,1,224,0,0,60,
0,0,7,128,0,0,240,0,0,30,0,0,3,96,0,0,
204,0,0,24,192,0,6,24,0,0,193,128,0,48,24,0,
12,1,192,7,0,30,3,192,0,255,224,0,7,240,0,14,
3,224,198,48,102,13,128,240,30,3,192,108,25,131,24,193,
240,28,0,14,3,224,254,63,231,253,255,255,255,255,255,239,
249,255,31,193,240,28,0,192,0,62,0,7,48,0,193,224,
120,15,255,0,31,128,0,0,0,0,0,0,0,0,0,0,
1,248,0,255,240,30,7,131,0,12,224,0,124,0,3,3,
255,195,255,241,192,0,192,0,96,0,24,0,12,0,3,0,
0,192,0,48,0,12,0,3,0,0,192,0,48,0,6,0,
1,128,0,48,0,7,0,0,255,252,15,255,0,0,0,0,
0,0,0,0,0,0,0,31,255,247,255,252,127,240,31,255,
0,0,224,0,12,0,1,128,0,96,0,12,0,3,0,0,
192,0,48,0,12,0,3,0,0,192,0,48,0,24,0,6,
0,3,0,3,135,255,193,255,192,0,0,0,0,0,0,0,
0,0,0,0,63,255,239,255,248,0,0,64,0,48,0,12,
0,6,0,1,128,0,192,0,224,0,240,1,240,63,240,15,
252,0,7,192,0,60,0,3,128,0,48,0,6,0,1,128,
0,48,0,12,0,1,0,0,0,0,0,0,0,0,0,0,
0,63,255,255,255,252,128,0,48,0,12,0,1,128,0,96,
0,12,0,1,192,0,60,0,3,224,0,63,240,15,252,15,
128,15,0,7,0,3,0,1,128,0,96,0,48,0,12,0,
3,0,0,0,0,0,0,0,0,0,0,0,0,0,63,255,
255,255,252,30,0,19,248,1,127,192,30,30,3,192,120,120,
3,254,128,31,200,0,120,30,0,23,248,1,127,224,62,31,
135,192,127,232,1,254,128,7,128,0,0,30,0,19,248,1,
127,224,62,31,135,192,127,232,1,254,128,7,128,0,0,192,
96,0,56,28,0,28,14,0,15,7,128,3,129,192,1,192,
224,0,112,56,0,56,28,0,30,15,0,7,3,128,3,129,
192,0,112,56,0,3,129,192,0,60,30,0,1,192,224,0,
14,7,0,0,224,112,0,7,3,128,0,120,60,0,3,129,
192,0,28,14,0,1,128,192,192,96,0,28,14,0,0,224,
112,0,15,7,128,0,112,56,0,3,129,192,0,56,28,0,
1,192,224,0,30,15,0,0,224,112,0,7,3,128,0,224,
112,0,112,56,0,60,30,0,14,7,0,7,3,128,1,192,
224,0,224,112,0,120,60,0,28,14,0,14,7,0,1,128,
192,0,0,0,0,64,0,48,0,12,0,6,0,1,128,0,
192,0,224,0,240,1,240,63,240,15,252,0,7,192,0,60,
0,3,128,0,48,0,6,0,1,128,0,48,0,12,0,3,
128,0,48,0,12,0,1,128,0,96,0,12,0,1,192,0,
60,0,3,224,0,63,240,15,252,15,128,15,0,7,0,3,
0,1,128,0,96,0,48,0,12,0,3,0,0,0,0,24,
96,0,0,252,0,0,7,0,0,1,224,0,0,116,0,0,
28,128,0,7,8,0,1,193,0,0,112,0,0,28,0,0,
7,0,0,1,192,0,0,112,0,0,28,0,0,7,0,0,
1,192,0,0,112,0,0,28,0,0,7,0,0,1,192,0,
0,112,0,0,28,0,0,7,0,0,1,192,0,0,112,0,
0,28,0,0,3,0,0,0,0,192,0,0,28,0,0,1,
192,0,0,28,0,0,1,192,0,0,28,0,0,1,192,0,
0,28,0,0,1,192,0,0,28,0,0,1,192,0,0,28,
0,0,1,192,0,0,28,0,0,1,192,0,0,28,0,0,
1,192,0,0,28,0,0,1,192,0,0,28,16,0,1,194,
0,0,28,128,0,1,208,0,0,30,0,0,1,192,0,0,
252,0,0,97,128,30,0,23,248,1,127,224,62,31,135,192,
127,232,1,254,128,7,128,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,7,255,255,255,255,240,0,48,0,0,6,
0,0,1,128,0,0,96,0,0,31,255,252,7,255,255,193,
128,0,1,224,0,0,240,0,0,7,0,0,0,56,0,0,
3,128,0,0,31,255,254,1,255,255,128,24,0,0,3,0,
0,0,48,0,0,6,0,0,0,1,128,0,0,48,0,0,
3,0,0,0,48,7,255,255,1,255,255,240,0,0,3,0,
0,0,60,0,0,1,224,0,0,112,0,0,56,0,0,14,
15,255,255,0,255,255,192,0,0,48,0,0,6,0,0,1,
128,0,0,48,0,0,64,0,8,0,3,128,0,216,0,27,
0,6,48,1,131,0,112,112,62,15,158,193,191,24,49,131,
6,0,96,192,12,24,1,131,0,48,96,6,12,0,193,128,
24,48,3,6,0,96,192,12,24,1,131,0,48,96,6,12,
0,193,128,6,12,0,193,128,24,48,3,6,0,96,192,12,
24,1,131,0,48,96,6,12,0,193,128,24,48,3,6,0,
96,192,12,24,1,131,6,48,99,246,13,231,193,240,56,56,
3,6,0,49,128,3,96,0,108,0,7,0,0,64,0,8,
0,0,64,64,0,24,12,0,3,1,128,0,192,24,0,63,
255,128,15,255,248,3,0,1,129,192,0,28,240,0,1,231,
0,0,112,112,0,28,7,0,7,0,127,255,192,7,255,240,
0,96,12,0,6,3,0,0,192,96,0,8,8,0,195,0,
0,31,128,0,1,192,0,0,60,0,0,5,192,0,0,156,
0,0,33,192,0,4,28,0,0,1,192,0,0,28,0,0,
1,192,0,0,28,0,0,1,192,0,0,28,0,0,1,192,
0,0,28,0,0,1,192,0,0,28,0,0,1,192,0,0,
28,0,0,1,192,0,0,28,0,0,1,192,0,0,28,0,
0,1,192,0,0,28,0,0,1,128,0,0,0,96,0,0,
28,0,0,7,0,0,1,192,0,0,112,0,0,28,0,0,
7,0,0,1,192,0,0,112,0,0,28,0,0,7,0,0,
1,192,0,0,112,0,0,28,0,0,7,0,0,1,192,0,
0,112,0,0,28,0,0,7,0,0,65,192,0,8,112,0,
0,156,0,0,23,0,0,3,192,0,0,112,0,0,31,128,
0,3,12,0,0,0,30,0,115,56,24,64,194,4,6,64,
128,60,8,3,128,128,24,8,1,192,128,60,4,2,96,64,
67,2,24,28,30,0,112,4,28,56,225,195,142,28,48,225,
195,6,24,48,0,127,1,255,0,14,0,12,0,24,0,96,
1,128,3,0,15,255,255,255,192,3,0,12,0,96,1,128,
6,0,48,3,135,252,31,192,0,24,0,0,24,0,0,60,
0,0,102,0,0,102,0,0,195,0,0,195,0,1,129,128,
3,0,192,3,0,192,6,0,96,6,0,96,12,0,48,24,
0,24,24,0,24,48,0,12,48,0,12,96,0,6,255,255,
255,127,255,255,127,255,255,255,255,255,96,0,6,48,0,12,
48,0,12,24,0,24,24,0,24,12,0,48,6,0,96,6,
0,96,3,0,192,3,0,192,1,129,128,0,195,0,0,195,
0,0,102,0,0,102,0,0,60,0,0,24,0,0,24,0,
0,3,0,3,0,6,0,6,0,12,0,24,0,24,0,48,
0,48,0,96,0,192,0,192,1,128,1,128,3,0,3,0,
6,0,12,0,12,0,24,0,24,0,48,0,96,0,96,0,
192,0,64,0,255,255,240,0,128,48,126,25,198,60,134,144,
214,27,198,120,207,25,227,60,199,152,243,30,99,216,107,9,
227,60,99,152,126,12,1,128,28,15,1,25,188,16,240,224,
131,3,132,28,30,16,224,120,71,1,193,56,7,9,192,48,
79,254,18,115,128,147,156,3,28,112,0,227,128,7,28,0,
56,224,97,135,6,12,56,8,192,230,60,3,192,15,128,6,
30,1,1,224,64,14,24,0,253,0,0,32,0,4,0,60,
64,12,70,3,132,56,112,128,14,0,0,224,0,14,62,0,
224,224,28,14,3,128,224,112,15,60,0,126,0,255,255,255,
255,252,3,0,0,96,0,12,0,1,128,0,48,0,6,0,
0,192,0,24,0,3,0,0,96,0,12,0,1,128,0,48,
0,6,0,0,192,0,24,0,3,0,0,32,0,0,64,0,
24,0,3,0,0,96,0,12,0,1,128,0,48,0,6,0,
0,192,0,24,0,3,0,0,96,0,12,0,1,128,0,48,
0,6,0,0,192,0,24,3,255,255,255,255,240,64,17,128,
99,0,231,1,231,0,231,1,199,3,15,4,55,16,70,32,
134,67,14,134,15,12,14,24,14,24,14,56,14,56,12,112,
31,192,32,64,1,192,3,192,3,192,3,192,3,193,131,193,
131,199,227,199,227,193,131,193,131,193,131,192,3,96,6,56,
28,31,248,7,224,64,1,128,3,0,6,0,12,0,24,0,
48,0,96,0,192,1,255,255,255,254,0,12,0,24,0,48,
0,96,0,192,1,128,3,0,2,0,0,0,4,0,48,0,
192,3,0,12,0,48,0,192,3,0,15,255,255,255,192,3,
0,12,0,48,0,192,3,0,12,0,48,0,192,1,64,192,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,255,127,1,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,3,3,3,3,3,3,255,255,127,255,192,192,192,192,
192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,192,
192,192,192,192,192,192,192,64,255,255,3,3,3,3,3,3,
3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
3,3,3,3,3,1,4,48,198,24,99,12,48,198,24,99,
12,48,193,134,24,48,195,12,24,97,131,12,16,195,12,24,
97,131,12,48,193,134,24,48,195,12,97,134,48,195,12,97,
134,48,195,0,128,224,240,120,60,30,15,7,131,193,224,240,
120,60,30,15,7,131,193,224,240,120,60,30,15,7,131,193,
224,240,120,60,28,4,2,0,96,15,1,104,102,104,97,6,
0,96,6,0,96,6,0,96,6,0,96,6,0,96,6,0,
96,6,0,96,6,0,96,6,8,97,102,97,104,15,0,96,
6,0,0,64,0,28,0,3,128,0,216,0,49,128,12,24,
7,131,195,240,126,230,12,224,193,128,24,48,3,6,0,96,
192,12,24,1,131,0,48,96,6,12,0,193,128,24,48,3,
6,14,96,206,252,31,135,131,192,48,96,3,24,0,54,0,
3,128,0,112,0,4,0,231,119,119,119,119,238,238,238,238,
231,0,0,0,64,0,0,48,0,0,24,0,0,6,0,0,
3,0,0,0,192,0,0,96,0,0,24,0,0,12,0,0,
6,0,0,1,128,0,0,192,0,0,48,0,0,24,0,0,
6,0,192,3,0,216,0,192,6,0,96,0,192,24,0,48,
12,0,6,3,0,1,129,128,0,48,192,0,12,48,0,1,
152,0,0,102,0,0,15,0,0,3,192,0,0,96,0,0,
24,0,0,255,143,248,224,14,7,0,112,56,3,129,192,28,
14,0,224,112,7,3,128,56,28,1,192,224,14,7,0,112,
56,3,129,192,28,14,0,224,112,7,3,128,56,28,1,192,
224,14,7,0,113,255,255,240,255,255,255,255,255,247,255,255,
158,0,2,56,0,16,240,0,193,192,2,7,128,24,14,0,
64,28,2,0,120,8,0,224,64,3,193,0,7,8,0,30,
96,0,57,0,0,252,0,1,224,0,3,0,0,12,0,192,
3,192,3,192,3,192,3,192,3,192,3,192,3,192,3,192,
3,192,3,192,3,192,3,192,3,192,3,192,3,255,255,127,
255,255,255,255,255,192,3,192,3,192,3,192,3,192,3,192,
3,192,3,192,3,192,3,192,3,192,3,192,3,192,3,192,
3,64,1,127,255,255,255,255,0,0,96,0,12,0,1,128,
0,48,0,6,0,0,192,0,24,0,3,0,0,96,0,12,
0,1,128,0,48,0,6,0,0,192,0,24,0,3,255,255,
255,255,240,0,0,0,0,0,0,0,0,0,0,0,15,255,
251,255,255,128,255,255,255,255,252,0,1,128,0,48,0,6,
0,0,192,0,24,0,3,0,0,96,0,12,0,1,128,0,
48,0,6,0,0,192,0,24,0,3,0,0,96,0,13,255,
255,255,255,240,0,0,0,0,0,0,0,0,0,0,0,15,
255,255,255,255,128,28,49,16,88,108,6,1,0,64,24,27,
24,216,60,30,15,6,198,54,6,0,128,32,24,13,134,130,
35,14,0,12,3,0,192,48,12,3,15,255,255,12,3,0,
192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,
48,12,3,0,12,3,0,192,48,12,3,15,252,48,12,3,
0,192,48,12,0,0,192,48,12,3,0,195,255,255,195,0,
192,48,12,3,0,7,255,31,255,63,24,127,24,127,24,255,
24,255,24,255,24,255,24,127,24,127,24,63,24,31,24,7,
24,3,24,3,24,3,24,3,24,3,24,3,24,3,24,3,
24,3,24,3,24,3,24,3,24,0,252,0,7,248,0,63,
240,1,255,224,7,255,128,31,254,0,127,248,1,255,224,3,
255,0,247,251,199,255,255,191,255,255,255,255,255,255,255,255,
255,255,255,255,255,255,255,253,252,79,227,243,63,7,140,120,
0,48,0,0,192,0,7,128,0,30,0,0,192,0,24,0,
7,128,0,240,0,51,0,12,48,3,3,0,96,96,24,6,
6,0,97,128,6,96,0,124,0,14,192,3,12,0,192,192,
48,12,12,1,129,128,24,96,1,152,0,30,0,3,192,0,
48,0,6,0,31,15,131,249,252,96,240,108,6,3,192,96,
60,2,3,192,0,60,0,3,192,0,54,0,6,96,0,99,
0,12,24,1,128,192,48,6,6,0,48,192,1,152,0,15,
0,0,96,0,6,0,0,96,0,6,0,0,240,0,15,0,
1,248,0,63,192,7,254,0,255,240,31,255,131,255,252,127,
255,231,255,254,255,255,255,255,255,255,255,255,255,255,255,255,
255,226,127,126,103,227,198,60,0,96,0,6,0,0,240,0,
15,0,
};
METRIC cmti10_metric[128] = {
{4824, -3, 27, 25, 28, 411054},
{4912, -3, 28, 27, 29, 535936},
{5010, -6, 28, 26, 30, 502440},
{5108, -2, 28, 24, 29, 453652},
{5195, -3, 27, 28, 28, 435448},
{5293, -3, 27, 31, 28, 487148},
{5402, -3, 27, 29, 28, 468944},
{5504, -9, 28, 25, 29, 502440},
{5595, -6, 27, 24, 28, 468944},
{5679, -9, 27, 25, 28, 502440},
{5767, -4, 28, 27, 29, 468944},
{1555, 1, 28, 32, 37, 401952},
{1703, 1, 28, 24, 37, 368456},
{1814, 1, 28, 26, 37, 385204},
{1935, 1, 28, 38, 37, 577806},
{2111, 1, 28, 39, 37, 586180},
{1213, -4, 17, 8, 18, 200976},
{1231, 1, 17, 13, 26, 217724},
{8335, -12, 28, 5, 8, 334960},
{8340, -14, 28, 8, 8, 334960},
{8348, -11, 25, 10, 5, 334960},
{8355, -12, 28, 10, 7, 334960},
{8364, -10, 24, 12, 1, 334960},
{8366, -20, 29, 8, 7, 544795},
{8373, -4, -1, 10, 8, 301464},
{1274, 1, 28, 24, 37, 351708},
{1385, -4, 17, 25, 18, 468944},
{1442, -5, 17, 24, 18, 468944},
{1496, -3, 21, 18, 26, 334960},
{6824, -2, 27, 37, 28, 578534},
{6954, -7, 28, 36, 30, 645526},
{7089, -5, 29, 28, 32, 502440},
{8383, -3, 15, 11, 4, 167480},
{7201, -4, 29, 11, 30, 200976},
{8422, -7, 28, 13, 12, 337145},
{7285, -4, 28, 30, 37, 535936},
{6518, -4, 28, 25, 29, 504033},
{7424, -6, 30, 28, 33, 535936},
{6609, -5, 29, 28, 31, 502440},
{7540, -9, 28, 6, 12, 200976},
{7549, -5, 30, 16, 42, 267968},
{7633, 0, 30, 16, 42, 267968},
{7717, -7, 30, 17, 18, 334960},
{7756, -6, 22, 24, 26, 502440},
{7834, -3, 3, 5, 12, 200976},
{8442, -3, 9, 10, 3, 234472},
{7842, -4, 3, 5, 4, 200976},
{7845, -1, 30, 23, 41, 334960},
{5865, -5, 27, 17, 29, 334960},
{5927, -4, 27, 15, 28, 334960},
{5980, -4, 27, 18, 29, 334960},
{6046, -4, 27, 18, 29, 334960},
{6112, -2, 27, 18, 36, 334960},
{6193, -4, 27, 19, 29, 334960},
{6262, -5, 27, 17, 29, 334960},
{6324, -7, 27, 18, 29, 334960},
{6390, -4, 27, 18, 29, 334960},
{6456, -5, 27, 17, 29, 334960},
{7963, -4, 17, 8, 18, 200976},
{7981, -3, 17, 9, 26, 200976},
{7243, -2, 20, 11, 30, 200976},
{8011, -4, 15, 28, 12, 502440},
{6771, -3, 20, 14, 30, 334960},
{6718, -8, 29, 14, 30, 334960},
{8053, -6, 28, 26, 29, 502440},
{2292, -2, 28, 26, 29, 487148},
{2387, -3, 27, 26, 28, 461298},
{2478, -6, 28, 27, 30, 468944},
{2580, -3, 27, 28, 28, 494794},
{2678, -3, 27, 27, 28, 444550},
{2773, -3, 27, 26, 28, 427802},
{2864, -6, 28, 27, 30, 506991},
{2966, -3, 27, 31, 28, 487148},
{3075, -2, 27, 18, 28, 252676},
{3138, -5, 27, 20, 29, 344062},
{3211, -3, 27, 32, 28, 503896},
{3323, -3, 27, 21, 28, 411054},
{3397, -3, 27, 37, 28, 587636},
{3527, -3, 27, 31, 28, 487148},
{3636, -6, 28, 26, 30, 502440},
{3734, -3, 27, 26, 28, 444550},
{3825, -6, 28, 26, 37, 502440},
{3946, -3, 27, 26, 29, 478046},
{4041, -3, 28, 22, 30, 368456},
{4124, -7, 27, 26, 28, 468944},
{4215, -9, 27, 25, 29, 487148},
{4306, -9, 27, 26, 29, 487148},
{4401, -9, 27, 36, 29, 654628},
{4532, -2, 27, 31, 28, 487148},
{4641, -8, 27, 27, 28, 487148},
{4736, -3, 27, 25, 28, 401952},
{8148, -2, 30, 17, 41, 200976},
{8446, -11, 28, 14, 12, 337145},
{8236, 1, 30, 17, 41, 200976},
{8389, -11, 28, 9, 6, 334960},
{8396, -11, 28, 4, 4, 200976},
{8324, -8, 28, 7, 12, 200976},
{0, -5, 17, 15, 18, 334960},
{34, -5, 28, 13, 29, 301464},
{82, -5, 17, 14, 18, 301464},
{114, -5, 28, 17, 29, 334960},
{176, -5, 17, 13, 18, 301464},
{206, 1, 28, 20, 37, 200976},
{299, -3, 17, 16, 26, 301464},
{351, -3, 28, 17, 29, 334960},
{413, -4, 27, 9, 28, 200976},
{445, 1, 27, 15, 36, 200976},
{513, -3, 28, 17, 29, 301464},
{575, -4, 28, 9, 29, 167480},
{608, -4, 17, 29, 18, 535936},
{674, -4, 17, 18, 18, 368456},
{715, -5, 17, 15, 18, 334960},
{749, -1, 17, 19, 26, 334960},
{811, -5, 17, 14, 26, 301464},
{857, -4, 17, 16, 18, 276342},
{893, -3, 17, 14, 18, 267968},
{925, -3, 25, 11, 26, 217724},
{961, -4, 17, 17, 18, 351708},
{1000, -4, 17, 15, 18, 301464},
{1034, -4, 17, 24, 18, 435448},
{1088, -3, 17, 17, 18, 304013},
{1127, -4, 17, 16, 26, 318212},
{1179, -3, 17, 15, 18, 267968},
{8467, -4, 11, 18, 1, 334960},
{8470, -5, 11, 37, 1, 669920},
{8398, -11, 28, 12, 8, 334960},
{8410, -10, 27, 13, 4, 334960},
{8417, -12, 28, 10, 4, 334960},
};
char cmti10_bitmap[8475] = {
3,204,12,120,48,240,192,227,131,134,7,28,14,56,28,224,
113,192,227,129,199,3,142,14,56,28,112,56,176,242,98,100,
120,112,63,0,56,1,192,28,0,224,7,0,56,3,128,28,
0,224,7,0,115,131,230,28,24,224,206,7,112,59,129,220,
15,192,238,7,112,59,129,156,28,192,198,14,24,96,198,3,
192,0,1,240,28,32,192,134,7,56,56,192,7,0,28,0,
224,3,128,14,0,56,0,224,3,128,46,1,24,8,48,192,
124,0,0,31,128,1,192,0,224,0,224,0,112,0,56,0,
28,0,28,0,14,0,7,0,3,128,123,128,99,192,97,224,
96,112,112,112,48,56,56,28,28,14,28,14,14,7,7,3,
131,129,193,193,196,192,226,96,113,24,121,12,76,131,195,128,
1,224,56,131,4,48,19,129,56,9,193,143,240,224,7,0,
56,1,192,14,0,112,5,128,76,4,48,192,248,0,0,3,
192,0,103,0,12,112,1,198,0,28,0,1,192,0,56,0,
3,128,0,56,0,3,128,0,56,0,63,248,0,112,0,7,
0,0,112,0,7,0,0,112,0,14,0,0,224,0,14,0,
0,224,0,14,0,1,192,0,28,0,1,192,0,28,0,1,
192,0,56,0,3,128,0,56,0,3,0,0,48,0,7,0,
12,96,0,230,0,12,192,0,120,0,0,0,243,1,143,3,
15,6,7,14,14,12,14,28,14,28,14,56,28,56,28,56,
28,56,28,56,56,48,56,48,56,24,120,24,240,15,112,0,
112,0,112,0,224,0,224,192,192,225,192,195,0,126,0,15,
192,0,224,0,112,0,112,0,56,0,28,0,14,0,14,0,
7,0,3,128,1,192,1,207,0,232,192,120,112,56,56,56,
28,28,14,14,7,7,3,135,3,131,129,193,192,224,224,224,
224,113,112,56,184,56,92,28,92,6,70,1,192,1,129,192,
64,0,0,0,0,0,0,0,7,4,196,114,58,57,28,14,
14,7,3,131,129,193,196,226,113,57,28,135,128,0,6,0,
28,0,24,0,0,0,0,0,0,0,0,0,0,0,0,0,
3,192,8,192,33,128,67,1,14,2,28,0,56,0,112,1,
192,3,128,7,0,14,0,56,0,112,0,224,1,192,7,0,
14,0,28,0,56,0,224,1,192,195,1,206,3,56,3,192,
0,15,192,0,224,0,112,0,112,0,56,0,28,0,14,0,
14,0,7,0,3,128,1,192,1,193,224,225,16,113,28,57,
14,57,6,28,128,14,128,7,128,7,240,3,156,1,199,0,
227,128,225,196,112,226,56,113,28,57,28,12,134,3,128,31,
129,192,224,224,112,56,28,28,14,7,3,131,129,192,224,112,
112,56,28,14,14,7,3,129,193,200,228,114,57,13,3,128,
60,30,7,129,51,24,198,17,160,232,56,142,7,129,200,224,
56,14,71,1,192,112,56,14,3,129,192,112,28,28,7,1,
192,224,56,14,7,1,192,112,56,14,7,3,128,224,56,156,
7,1,196,224,56,28,39,1,192,226,112,28,3,33,128,96,
14,0,60,60,9,145,132,104,113,28,28,142,7,35,129,192,
224,112,56,28,28,14,7,3,129,192,224,112,112,56,28,78,
7,19,131,132,224,226,112,25,12,3,128,1,224,14,48,48,
48,192,99,128,198,1,220,3,184,7,224,29,192,59,128,119,
0,206,3,156,6,56,24,48,96,49,128,60,0,7,135,0,
155,48,35,131,4,112,97,28,14,35,129,192,112,56,14,7,
3,129,192,112,56,14,7,1,192,192,112,56,14,6,1,193,
192,56,48,14,140,1,206,0,56,0,7,0,1,192,0,56,
0,7,0,0,224,0,60,0,31,240,0,3,196,24,176,195,
198,7,56,56,192,231,3,156,14,224,115,129,206,7,56,28,
224,227,3,140,14,24,120,99,192,247,0,28,0,112,3,128,
14,0,56,0,224,7,128,255,192,60,60,38,194,70,135,71,
7,142,6,142,0,14,0,14,0,28,0,28,0,28,0,28,
0,56,0,56,0,56,0,56,0,112,0,48,0,1,240,24,
32,192,131,7,24,24,112,1,240,7,240,15,240,31,192,7,
128,14,96,51,128,204,3,32,24,96,192,124,0,0,192,56,
7,0,224,56,7,0,224,28,255,224,224,28,3,128,224,28,
3,128,112,28,3,128,112,14,3,132,112,142,17,196,25,1,
192,30,3,19,131,145,193,200,224,232,112,228,56,112,56,56,
28,28,28,28,14,14,7,7,3,131,131,131,137,193,196,96,
226,48,114,28,89,3,199,0,30,6,78,29,28,58,56,56,
112,48,224,67,128,135,1,28,4,56,8,112,16,224,65,128,
135,1,6,4,12,16,12,64,15,0,30,1,131,39,3,135,
71,3,135,71,3,131,135,7,1,135,7,1,14,7,1,14,
7,1,28,14,2,28,14,2,28,14,2,28,14,4,24,12,
4,24,28,4,24,28,8,28,28,16,12,38,48,7,195,192,
7,14,12,200,132,56,228,28,116,28,50,14,0,7,0,3,
128,3,128,1,192,0,224,0,112,0,112,35,56,19,156,17,
150,8,139,8,56,120,0,30,3,39,7,71,7,71,7,135,
14,135,14,14,14,14,14,28,28,28,28,28,28,28,28,56,
56,56,56,24,56,24,56,28,112,7,240,0,112,0,112,0,
224,224,192,225,192,129,128,71,0,60,0,3,130,15,132,63,
176,65,193,0,128,2,0,8,0,32,0,128,2,0,8,0,
32,0,128,130,1,14,12,51,248,67,225,3,128,28,38,71,
71,142,142,14,28,28,28,56,56,113,113,113,114,114,60,0,
240,8,192,134,4,48,67,130,28,0,224,7,0,112,3,128,
28,0,224,14,0,112,3,128,28,1,192,14,0,112,3,128,
56,1,195,12,28,224,198,3,192,0,0,1,248,0,7,14,
0,12,6,0,28,7,0,28,7,0,56,7,0,56,7,0,
56,14,0,56,12,0,56,8,0,112,24,7,240,16,0,112,
32,0,112,96,0,112,96,0,224,96,0,224,96,0,224,112,
0,224,112,0,224,56,1,192,56,1,192,28,1,192,28,1,
192,56,1,192,56,3,152,48,3,152,112,3,144,192,3,143,
128,3,128,0,7,0,0,7,0,0,6,0,0,198,0,0,
236,0,0,200,0,0,112,0,0,0,3,62,0,3,240,128,
7,224,64,12,224,16,28,240,16,24,112,8,56,56,24,56,
31,240,24,28,0,28,14,0,28,7,0,14,3,128,14,3,
192,23,1,224,11,128,240,8,192,248,8,49,246,24,15,24,
240,0,1,224,124,6,49,194,12,27,2,24,30,1,56,30,
2,48,28,2,112,28,12,112,31,240,224,56,0,224,56,0,
224,56,0,224,56,0,224,120,2,224,120,2,224,216,4,97,
216,8,35,12,48,30,3,192,0,0,64,0,32,0,8,0,
4,0,122,0,113,128,48,112,24,44,14,11,3,4,225,194,
56,112,142,56,71,14,33,195,136,112,228,24,58,14,14,131,
3,193,128,96,192,12,96,5,224,2,0,0,128,0,64,0,
32,0,0,0,1,252,60,0,6,14,103,0,12,14,199,0,
28,13,198,0,28,1,192,0,56,1,192,0,56,3,128,0,
56,3,128,0,56,3,128,0,56,3,128,0,112,3,128,7,
255,255,248,0,112,7,0,0,112,7,0,0,112,7,0,0,
224,7,0,0,224,14,0,0,224,14,0,0,224,14,0,0,
224,14,0,1,192,14,0,1,192,28,0,1,192,28,0,1,
192,28,0,1,192,28,0,3,128,28,0,3,128,56,0,3,
128,56,0,3,128,56,0,3,0,56,0,7,0,48,0,7,
0,112,0,6,0,96,0,198,96,96,0,228,112,192,0,200,
97,128,0,112,62,0,0,0,1,252,0,7,3,0,12,3,
0,28,7,0,28,3,0,24,0,0,56,0,0,56,0,0,
56,0,0,56,0,0,112,0,7,255,252,0,112,28,0,112,
28,0,112,28,0,224,56,0,224,56,0,224,56,0,224,56,
0,224,112,1,192,112,1,192,112,1,192,112,1,192,226,1,
192,226,1,192,226,3,128,228,3,128,100,3,128,56,3,128,
0,3,0,0,7,0,0,6,0,0,198,0,0,228,0,0,
204,0,0,112,0,0,0,1,253,192,1,195,240,0,192,252,
0,112,62,0,28,3,128,14,0,224,3,128,56,0,224,28,
0,56,7,0,14,1,192,7,0,112,31,255,248,0,112,14,
0,28,3,128,7,0,224,3,128,112,0,224,28,0,56,7,
0,14,1,192,3,128,224,1,192,56,0,112,14,0,28,3,
128,7,1,196,1,192,113,0,224,28,64,56,7,32,14,0,
200,3,128,28,0,224,0,0,48,0,0,28,0,0,6,0,
0,49,128,0,14,64,0,3,48,0,0,112,0,0,0,0,
1,254,15,240,0,12,28,224,48,0,96,119,0,192,3,1,
152,7,0,28,0,224,12,0,112,3,0,0,1,192,28,0,
0,14,0,112,0,0,56,1,192,0,0,224,7,0,0,3,
128,56,0,0,255,255,255,252,0,112,3,128,112,1,192,14,
1,192,7,0,56,7,0,28,1,224,56,0,112,7,0,224,
1,192,28,3,128,14,0,112,14,0,56,1,192,112,0,224,
7,1,192,3,128,56,7,0,14,0,224,28,0,112,3,128,
226,1,192,14,3,136,7,0,56,14,32,28,1,192,57,0,
112,7,0,100,3,128,28,0,224,14,0,112,0,0,56,3,
128,0,0,192,14,0,0,3,0,48,0,3,24,192,192,0,
14,99,134,0,0,51,12,48,0,0,120,31,0,0,0,0,
1,252,15,238,0,14,28,96,252,0,48,57,129,248,0,96,
103,3,224,1,192,12,1,192,3,128,56,3,128,14,0,112,
7,0,28,0,224,28,0,56,1,192,56,0,112,7,0,112,
0,224,14,0,224,31,255,255,255,128,7,0,56,7,0,14,
0,112,14,0,28,1,192,28,0,56,3,128,112,0,224,7,
0,224,1,192,14,1,192,3,128,28,3,128,7,0,112,14,
0,14,0,224,28,0,56,1,192,56,0,112,3,128,112,0,
224,7,1,196,1,192,28,3,136,3,128,56,7,16,6,0,
112,14,64,28,0,224,12,128,56,1,192,14,0,112,7,0,
0,0,192,14,0,0,1,128,28,0,0,7,0,48,0,1,
140,96,192,0,3,152,225,128,0,6,97,140,0,0,7,129,
240,0,0,0,0,0,24,0,0,6,0,0,3,128,0,0,
224,0,0,120,0,0,30,0,0,11,128,0,6,224,0,1,
56,0,0,142,0,0,35,192,0,16,112,0,4,28,0,2,
7,0,1,129,192,0,64,112,0,32,28,0,8,7,0,7,
255,192,1,0,112,0,128,28,0,96,7,0,16,1,192,8,
0,112,2,0,28,1,0,3,128,192,0,224,112,0,120,255,
0,255,192,1,255,254,0,15,1,224,3,128,60,0,224,7,
0,56,1,192,14,0,112,7,0,28,1,192,15,0,112,3,
192,28,1,224,14,0,112,3,128,56,0,224,60,0,63,254,
0,28,1,192,7,0,56,1,192,15,0,112,3,192,56,0,
240,14,0,60,3,128,15,0,224,3,192,112,1,224,28,0,
112,7,0,60,1,192,30,0,224,30,3,255,254,0,0,3,
240,32,3,193,140,0,192,11,128,112,1,224,28,0,28,7,
0,3,129,192,0,48,112,0,4,30,0,0,131,128,0,16,
240,0,2,30,0,0,7,128,0,0,240,0,0,30,0,0,
7,128,0,0,240,0,0,30,0,0,3,192,0,0,120,0,
2,15,0,0,65,224,0,8,60,0,2,3,128,0,64,112,
0,16,7,0,4,0,96,1,0,6,0,192,0,112,96,0,
3,240,0,0,1,255,254,0,3,192,120,0,56,1,192,3,
128,28,0,56,0,224,3,128,14,0,112,0,240,7,0,7,
0,112,0,112,7,0,15,0,224,0,240,14,0,15,0,224,
0,240,14,0,15,1,192,1,224,28,0,30,1,192,1,224,
28,0,28,3,128,3,192,56,0,56,3,128,7,128,56,0,
112,7,0,14,0,112,1,192,7,0,56,0,112,7,0,14,
1,192,15,255,240,0,1,255,255,224,7,128,28,0,224,1,
128,28,0,32,3,128,4,0,112,0,128,28,0,16,3,128,
2,0,112,32,64,14,4,0,3,129,0,0,112,32,0,14,
12,0,1,255,128,0,112,32,0,14,4,0,1,192,128,0,
56,16,0,14,4,4,1,192,0,128,56,0,32,7,0,4,
1,192,1,0,56,0,32,7,0,12,0,224,3,0,56,1,
224,127,255,248,0,1,255,255,192,15,0,112,3,128,12,0,
224,2,0,56,0,128,14,0,32,7,0,8,1,192,2,0,
112,32,128,28,8,0,14,4,0,3,129,0,0,224,192,0,
63,240,0,28,8,0,7,2,0,1,192,128,0,112,32,0,
56,16,0,14,0,0,3,128,0,0,224,0,0,112,0,0,
28,0,0,7,0,0,1,192,0,0,240,0,3,255,192,0,
0,3,240,32,3,193,140,0,192,11,128,112,1,224,28,0,
28,7,0,3,129,192,0,48,112,0,4,30,0,0,131,128,
0,16,240,0,2,30,0,0,7,128,0,0,240,0,0,30,
0,0,7,128,0,0,240,0,0,30,0,63,251,192,0,120,
120,0,14,15,0,1,193,224,0,56,60,0,7,3,128,1,
192,112,0,56,7,0,7,0,96,1,224,6,0,72,0,112,
113,0,3,240,0,0,1,255,207,254,0,120,3,192,0,224,
7,0,1,192,14,0,3,128,28,0,7,0,56,0,28,0,
224,0,56,1,192,0,112,3,128,0,224,7,0,3,128,28,
0,7,0,56,0,14,0,112,0,31,255,224,0,112,3,128,
0,224,7,0,1,192,14,0,3,128,28,0,14,0,112,0,
28,0,224,0,56,1,192,0,112,3,128,1,192,14,0,3,
128,28,0,7,0,56,0,14,0,112,0,60,1,224,7,255,
63,248,0,1,255,192,15,0,3,128,0,224,0,56,0,14,
0,7,0,1,192,0,112,0,28,0,14,0,3,128,0,224,
0,56,0,28,0,7,0,1,192,0,112,0,56,0,14,0,
3,128,0,224,0,112,0,28,0,7,0,1,192,0,240,3,
255,128,0,127,240,0,120,0,7,0,0,112,0,7,0,0,
112,0,14,0,0,224,0,14,0,0,224,0,28,0,1,192,
0,28,0,1,192,0,56,0,3,128,0,56,0,3,128,0,
112,0,7,0,0,112,0,7,0,96,224,14,14,0,192,192,
8,28,0,131,128,4,112,0,60,0,0,1,255,192,255,0,
60,0,60,0,56,0,48,0,56,0,64,0,56,0,128,0,
56,1,0,0,112,2,0,0,112,4,0,0,112,16,0,0,
112,32,0,0,224,64,0,0,224,192,0,0,225,192,0,0,
229,192,0,1,200,224,0,1,208,224,0,1,224,224,0,1,
192,112,0,3,128,112,0,3,128,112,0,3,128,56,0,3,
128,56,0,7,0,56,0,7,0,28,0,7,0,28,0,7,
0,28,0,15,0,30,0,255,224,255,128,1,255,224,1,224,
0,14,0,0,112,0,3,128,0,28,0,1,192,0,14,0,
0,112,0,3,128,0,56,0,1,192,0,14,0,0,112,0,
7,0,0,56,0,1,192,0,14,0,0,224,2,7,0,16,
56,0,129,192,8,28,0,64,224,6,7,0,96,56,7,3,
128,241,255,255,128,1,254,0,7,248,1,240,0,60,0,11,
128,3,192,0,92,0,46,0,2,224,1,112,0,23,0,19,
128,1,56,0,184,0,9,192,9,192,0,78,0,78,0,2,
112,4,112,0,35,128,71,0,1,28,2,56,0,8,224,33,
192,0,71,1,14,0,4,28,16,224,0,32,224,135,0,1,
7,8,56,0,8,56,129,192,0,129,196,28,0,4,14,64,
224,0,32,114,7,0,1,3,160,56,0,16,29,3,128,0,
128,240,28,0,4,7,0,224,0,96,56,7,0,7,1,128,
120,1,255,12,63,248,0,1,252,3,254,0,56,0,224,0,
240,1,128,1,112,2,0,2,224,4,0,5,192,8,0,17,
192,32,0,35,128,64,0,71,0,128,0,135,1,0,2,14,
4,0,4,28,8,0,8,24,16,0,16,56,32,0,64,112,
128,0,128,225,0,1,0,226,0,2,1,196,0,8,3,144,
0,16,3,160,0,32,7,64,0,64,14,128,1,0,14,0,
2,0,28,0,4,0,56,0,24,0,48,0,112,0,64,7,
252,0,128,0,0,7,240,0,7,7,0,7,0,224,3,128,
28,1,192,3,128,224,0,224,112,0,56,56,0,15,30,0,
3,199,0,0,243,192,0,60,240,0,15,120,0,3,222,0,
0,247,128,0,63,192,0,30,240,0,7,188,0,1,239,0,
0,243,192,0,60,240,0,14,60,0,7,143,0,3,193,192,
0,224,112,0,112,30,0,56,3,128,28,0,112,14,0,14,
14,0,0,254,0,0,1,255,252,0,15,1,192,3,128,56,
0,224,7,0,56,1,192,14,0,112,7,0,60,1,192,15,
0,112,3,192,28,0,224,14,0,120,3,128,28,0,224,14,
0,56,14,0,31,254,0,7,0,0,1,192,0,0,112,0,
0,56,0,0,14,0,0,3,128,0,0,224,0,0,112,0,
0,28,0,0,7,0,0,1,192,0,0,240,0,3,255,128,
0,0,7,240,0,7,7,0,7,0,224,3,128,28,1,192,
7,128,224,0,224,112,0,56,56,0,15,30,0,3,199,0,
0,243,192,0,60,240,0,15,120,0,3,222,0,0,247,128,
0,63,192,0,30,240,0,7,188,0,1,239,0,0,243,192,
0,60,240,0,14,60,0,7,143,0,1,193,193,224,224,112,
128,112,28,65,56,3,144,92,0,116,30,0,15,14,4,0,
255,129,0,0,96,192,0,24,32,0,14,24,0,3,252,0,
0,127,0,0,31,128,0,3,128,0,1,255,248,0,15,3,
128,3,128,112,0,224,14,0,56,3,128,14,0,224,7,0,
120,1,192,30,0,112,7,128,28,3,192,14,0,224,3,128,
112,0,224,112,0,63,240,0,28,12,0,7,1,128,1,192,
112,0,112,28,0,56,7,0,14,1,192,3,128,112,0,224,
28,0,112,15,0,28,3,193,7,0,240,65,192,60,32,240,
7,139,255,129,228,0,0,30,0,0,15,132,0,193,48,4,
3,192,32,6,1,0,24,12,0,96,48,1,129,128,4,6,
0,16,24,0,0,112,0,1,192,0,3,224,0,15,240,0,
31,240,0,31,224,0,7,192,0,7,0,0,28,0,0,48,
0,0,192,128,3,2,0,12,24,0,96,96,1,129,128,12,
6,0,32,60,1,0,204,24,2,15,128,0,31,255,255,199,
1,192,115,0,224,12,128,56,2,96,14,0,144,3,128,36,
1,192,10,0,112,2,128,28,0,128,7,0,0,3,128,0,
0,224,0,0,56,0,0,14,0,0,7,0,0,1,192,0,
0,112,0,0,28,0,0,14,0,0,3,128,0,0,224,0,
0,56,0,0,28,0,0,7,0,0,1,192,0,0,112,0,
0,60,0,1,255,248,0,127,240,255,135,128,14,3,128,6,
1,192,2,0,224,1,0,112,0,128,112,0,128,56,0,64,
28,0,32,14,0,16,14,0,16,7,0,8,3,128,4,1,
192,2,1,192,2,0,224,1,0,112,0,128,56,0,64,56,
0,64,28,0,32,14,0,16,7,0,16,3,128,8,1,192,
8,0,224,4,0,48,4,0,12,12,0,2,8,0,0,248,
0,0,255,128,63,199,0,3,193,192,0,64,112,0,32,28,
0,8,7,0,4,1,192,1,0,112,0,128,28,0,64,7,
0,16,1,224,8,0,56,2,0,14,1,0,3,128,128,0,
224,32,0,56,16,0,14,4,0,3,130,0,0,225,128,0,
56,64,0,14,32,0,1,200,0,0,116,0,0,31,0,0,
7,128,0,1,192,0,0,112,0,0,24,0,0,6,0,0,
0,255,131,255,15,243,192,7,0,28,56,0,112,1,131,128,
15,0,16,56,0,240,2,3,128,23,0,32,56,1,112,4,
3,128,39,0,64,56,2,112,8,1,192,71,0,128,28,4,
112,16,1,192,135,1,0,28,8,112,32,1,193,7,2,0,
28,16,112,64,1,194,7,4,0,28,32,112,128,1,196,7,
24,0,28,64,113,0,1,200,7,32,0,29,128,114,0,1,
208,7,64,0,30,0,116,0,1,224,7,128,0,28,0,120,
0,1,192,7,0,0,24,0,112,0,1,128,6,0,0,16,
0,96,0,0,1,255,129,254,0,60,1,224,0,112,1,128,
0,240,4,0,0,224,24,0,1,192,32,0,3,192,128,0,
3,130,0,0,7,8,0,0,15,32,0,0,14,128,0,0,
30,0,0,0,28,0,0,0,56,0,0,0,120,0,0,1,
112,0,0,4,224,0,0,17,224,0,0,65,192,0,1,3,
128,0,6,3,128,0,8,7,0,0,32,15,0,0,128,14,
0,2,0,28,0,12,0,60,0,120,0,120,7,252,7,254,
0,255,192,15,225,192,0,240,56,0,24,7,128,2,0,112,
0,128,14,0,48,1,224,4,0,28,1,0,3,128,64,0,
120,16,0,7,6,0,0,224,128,0,30,32,0,1,200,0,
0,59,0,0,7,192,0,0,112,0,0,14,0,0,3,128,
0,0,112,0,0,14,0,0,1,192,0,0,112,0,0,14,
0,0,1,192,0,0,56,0,0,14,0,0,31,252,0,0,
0,255,255,128,120,3,128,112,3,128,48,3,192,16,1,192,
8,1,192,8,1,192,4,1,192,2,1,192,0,0,224,0,
0,224,0,0,224,0,0,224,0,0,224,0,0,112,0,0,
112,0,0,112,0,0,112,4,0,112,4,0,56,2,0,56,
1,0,56,1,0,56,0,128,56,0,192,60,0,192,28,0,
224,28,1,224,31,255,240,0,1,255,255,128,30,1,192,14,
0,96,7,0,32,3,128,16,1,192,8,1,192,4,0,224,
2,0,112,1,0,56,0,0,56,0,0,28,0,0,14,0,
0,7,0,0,7,0,0,3,128,0,1,192,0,0,224,0,
0,224,0,0,112,0,0,56,0,0,28,0,0,28,0,0,
14,0,0,7,0,0,3,128,0,3,192,0,31,254,0,0,
0,0,8,0,0,3,128,0,0,240,0,0,30,0,0,5,
192,0,1,56,0,0,39,128,0,8,112,0,2,14,0,0,
193,192,0,16,60,0,4,7,128,1,0,112,0,32,14,0,
8,1,192,2,0,60,0,192,7,128,16,0,112,4,0,14,
1,0,1,192,32,0,60,8,0,3,130,0,0,112,192,0,
14,16,0,1,196,0,0,61,255,255,255,191,255,255,255,255,
255,254,0,7,240,0,7,7,0,7,0,224,3,128,28,1,
192,7,128,224,0,224,112,0,56,56,0,15,30,0,3,199,
0,0,243,192,0,60,240,0,15,120,128,35,222,63,248,247,
143,254,63,199,255,30,241,255,199,188,64,17,239,0,0,243,
192,0,60,240,0,14,60,0,7,143,0,1,193,192,0,224,
112,0,112,28,0,56,3,128,28,0,112,14,0,14,14,0,
0,254,0,0,0,0,48,0,0,48,0,0,112,0,0,112,
0,0,240,0,0,240,0,1,112,0,1,112,0,2,112,0,
2,112,0,4,112,0,8,112,0,8,112,0,16,112,0,16,
112,0,32,120,0,32,120,0,64,120,0,64,56,0,128,56,
1,128,56,1,0,56,2,0,56,2,0,56,4,0,56,4,
0,56,12,0,56,28,0,120,255,3,255,1,255,255,240,31,
255,255,3,0,0,32,32,0,2,2,0,0,32,32,0,2,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,
4,0,16,0,64,1,255,252,0,31,255,192,2,0,8,0,
32,0,128,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,64,0,4,4,0,0,64,64,
0,4,4,0,0,192,255,255,248,15,255,255,128,1,255,255,
254,0,120,1,192,0,224,7,0,1,192,14,0,3,128,28,
0,7,0,56,0,28,0,224,0,56,1,192,0,112,3,128,
0,224,7,0,3,128,28,0,7,0,56,0,14,0,112,0,
28,0,224,0,112,3,128,0,224,7,0,1,192,14,0,3,
128,28,0,14,0,112,0,28,0,224,0,56,1,192,0,112,
3,128,1,192,14,0,3,128,28,0,7,0,56,0,14,0,
112,0,60,1,224,7,255,63,248,0,1,255,255,248,15,0,
3,192,56,0,14,1,192,0,32,15,0,1,0,56,0,8,
1,192,0,64,15,0,2,0,56,0,16,1,192,0,0,7,
0,0,0,56,0,0,1,224,0,0,7,0,0,0,56,0,
0,1,128,0,0,8,0,0,1,128,0,0,16,0,32,1,
0,1,0,16,0,8,1,0,0,128,16,0,4,1,0,0,
96,16,0,6,1,0,1,240,31,255,255,1,255,255,248,0,
15,0,30,15,224,63,143,240,63,236,60,56,124,14,56,28,
3,56,6,1,152,2,0,216,0,0,108,0,0,60,0,0,
30,0,0,15,0,0,7,0,0,3,128,0,1,192,0,1,
192,0,0,224,0,0,112,0,0,56,0,0,56,0,0,28,
0,0,14,0,0,7,0,0,7,0,0,3,128,0,1,192,
0,0,224,0,0,240,0,15,255,128,0,0,127,252,0,7,
128,0,7,0,0,7,0,0,7,0,0,63,192,0,238,112,
3,142,28,14,14,28,28,14,14,60,28,14,56,28,15,120,
28,15,120,28,15,240,56,30,240,56,30,240,56,28,112,56,
60,112,112,56,56,112,112,56,113,192,14,119,0,3,252,0,
0,224,0,0,224,0,0,224,0,1,224,0,63,254,0,1,
255,240,0,15,0,0,7,0,0,3,128,0,1,192,7,0,
224,61,192,224,120,112,112,56,56,56,56,28,28,28,28,28,
28,14,14,14,7,7,7,3,131,131,129,195,131,128,193,193,
192,96,225,192,48,112,192,28,112,224,14,56,224,3,28,224,
0,239,192,0,31,128,0,7,0,0,3,128,0,1,192,0,
1,224,0,31,255,0,0,0,3,248,0,3,193,192,0,224,
12,0,112,1,192,30,0,28,7,128,3,129,224,0,120,60,
0,15,15,0,1,225,224,0,60,60,0,7,143,0,1,225,
224,0,60,60,0,7,7,128,1,224,112,0,56,14,0,14,
1,192,1,192,24,0,112,3,0,28,0,96,3,0,6,0,
192,16,192,16,66,24,6,16,131,0,130,16,96,32,131,248,
7,240,127,1,254,15,224,63,128,0,60,0,99,0,96,192,
96,96,96,48,48,28,56,14,24,7,28,3,142,1,142,1,
199,0,227,128,113,192,57,192,56,224,28,112,14,56,7,56,
7,28,3,142,1,199,1,195,128,225,192,96,192,96,112,48,
24,48,14,48,1,224,0,0,2,0,12,0,48,0,224,7,
192,115,128,14,0,28,0,56,0,112,1,192,3,128,7,0,
14,0,56,0,112,0,224,1,192,7,0,14,0,28,0,56,
0,224,1,192,3,128,7,0,30,7,255,128,0,60,0,48,
192,16,24,8,6,2,1,193,16,112,68,28,33,7,8,65,
194,16,113,8,56,66,14,17,7,3,129,128,0,192,0,96,
0,48,0,48,0,16,0,24,0,8,0,4,0,65,0,32,
128,8,64,6,31,131,12,127,130,15,224,128,224,0,0,62,
0,48,96,16,24,8,3,4,0,193,16,48,132,28,33,7,
8,65,194,32,224,112,56,0,28,0,14,0,126,0,3,128,
0,48,0,12,0,3,128,0,224,0,56,0,14,28,3,142,
1,194,0,112,128,56,32,12,4,6,0,135,0,31,0,0,
0,1,128,0,112,0,56,0,14,0,3,128,0,192,0,112,
0,28,0,6,0,3,128,0,192,0,112,0,24,0,6,0,
3,0,0,192,0,96,0,16,0,12,96,6,56,1,14,0,
131,128,97,192,48,112,24,28,15,199,4,15,130,0,252,0,
56,0,14,0,7,0,1,192,0,112,0,28,0,14,0,1,
128,0,192,96,31,248,7,254,0,255,0,16,0,2,0,0,
128,0,16,0,2,0,0,64,0,17,224,2,198,0,96,96,
8,12,2,1,128,0,56,0,7,0,0,192,0,56,0,7,
7,0,224,224,24,56,7,4,0,192,128,56,8,6,1,1,
128,16,224,1,240,0,0,15,0,24,64,48,32,48,112,56,
56,24,0,24,0,28,0,12,0,14,0,7,60,7,99,3,
192,193,192,97,224,48,224,28,112,14,56,7,24,7,28,3,
142,1,199,0,227,128,225,192,112,224,112,48,48,24,48,4,
48,1,224,0,9,192,69,248,33,254,8,241,132,48,39,24,
6,132,0,97,0,16,128,12,32,2,0,1,128,0,64,0,
48,0,24,0,6,0,1,128,0,192,0,48,0,28,0,6,
0,3,128,0,224,0,48,0,28,0,7,0,1,192,0,224,
0,56,0,12,0,0,0,30,0,24,64,8,24,6,2,3,
0,192,192,48,96,24,24,6,6,1,129,192,192,120,96,15,
48,3,240,0,124,0,55,128,24,240,8,30,6,3,131,0,
225,128,24,96,6,24,1,140,0,195,0,48,192,24,24,4,
6,2,0,195,0,15,0,0,0,60,0,99,0,96,192,96,
96,96,48,112,24,48,14,56,7,28,3,14,1,142,1,199,
0,227,128,113,192,120,224,56,48,60,24,30,6,54,1,231,
0,3,128,1,128,1,192,0,192,0,192,224,192,112,96,32,
96,16,192,7,192,0,0,0,124,0,0,97,0,0,224,64,
0,224,32,0,96,8,0,112,4,0,56,14,0,56,14,0,
28,2,0,14,0,0,7,0,0,7,0,0,3,128,0,1,
192,0,15,254,0,0,224,0,0,112,0,0,56,0,0,28,
0,0,28,0,0,14,0,0,7,0,0,3,128,0,63,128,
16,127,192,8,97,240,8,32,254,28,17,199,248,7,128,248,
0,0,60,0,0,6,48,3,0,193,0,112,24,8,7,3,
128,128,32,48,24,2,7,3,128,64,112,48,8,14,0,1,
0,224,0,32,14,0,4,0,238,1,160,15,16,34,0,225,
4,16,14,16,129,1,162,8,8,57,193,0,131,0,22,8,
112,0,224,135,0,14,8,224,0,1,14,0,0,16,224,0,
1,14,0,0,32,224,0,2,6,0,0,64,112,0,8,3,
0,1,0,24,0,96,0,224,56,0,1,252,0,0,1,224,
24,96,128,132,3,16,12,128,52,0,208,7,128,27,128,238,
7,48,120,3,192,30,0,240,7,0,56,0,194,3,8,12,
64,30,0,0,0,0,0,0,0,0,0,3,0,30,0,240,
1,128,0,0,24,0,240,3,192,14,0,0,0,0,0,0,
0,0,0,3,192,19,0,140,2,48,1,192,14,0,120,7,
192,60,1,224,207,7,112,13,128,22,0,176,2,192,19,0,
68,2,16,16,33,128,120,0,0,0,255,255,248,0,0,188,
1,192,0,9,192,6,0,0,78,0,32,0,4,112,1,0,
0,35,128,8,0,2,56,0,64,0,33,192,2,0,1,14,
0,16,0,16,112,16,0,0,135,1,0,0,8,56,8,0,
0,129,192,64,0,4,14,6,0,0,127,255,224,0,2,7,
3,0,0,32,56,8,0,2,1,192,64,0,16,28,4,16,
1,0,224,32,128,8,7,0,8,0,128,56,0,64,12,3,
128,4,0,64,28,0,32,4,0,224,3,0,32,7,0,48,
7,0,112,7,129,254,31,255,248,0,0,7,240,0,0,1,
195,255,255,0,112,31,0,112,14,1,224,3,1,192,30,0,
32,56,1,224,2,7,0,30,0,32,240,1,192,2,30,0,
28,0,33,192,3,192,66,60,0,60,4,3,192,3,128,128,
120,0,56,8,7,128,7,129,128,120,0,127,248,15,0,7,
1,0,240,0,112,16,15,0,15,1,0,240,0,240,16,15,
0,14,2,8,240,0,224,0,143,0,30,0,16,240,1,224,
1,15,0,28,0,32,112,3,192,2,7,128,60,0,96,56,
3,192,12,1,192,120,3,192,14,15,255,248,0,63,0,0,
0,0,0,0,16,0,63,130,0,14,14,64,3,128,56,0,
96,1,128,12,0,28,1,128,2,192,48,0,76,7,0,8,
224,224,1,14,12,0,32,225,192,4,14,28,0,64,227,128,
8,14,56,1,0,227,128,32,14,112,4,1,199,0,128,28,
112,16,1,199,2,0,56,112,32,3,135,4,0,48,112,128,
7,7,16,0,96,50,0,12,3,64,1,128,56,0,48,1,
128,6,0,28,0,192,2,112,112,0,65,252,0,8,0,0,
0,0,192,60,7,128,240,28,7,128,240,28,3,128,112,12,
3,128,112,12,1,128,48,4,1,128,48,4,0,128,0,0,
0,0,0,0,0,192,60,15,0,192,0,0,192,60,7,128,
224,0,0,0,0,0,0,0,64,8,1,0,96,12,1,128,
96,12,1,128,112,14,1,192,112,14,1,192,120,15,1,224,
120,15,0,192,0,0,0,48,24,0,0,192,96,0,6,3,
0,0,24,12,0,0,192,96,0,3,1,128,0,24,12,0,
0,96,48,0,3,1,128,0,12,6,0,0,48,24,0,1,
128,192,0,6,3,0,63,255,255,225,255,255,255,192,12,6,
0,0,48,24,0,0,192,96,0,6,3,0,0,24,12,0,
0,192,96,0,3,1,128,0,12,6,0,63,255,255,248,127,
255,255,192,12,6,0,0,96,48,0,1,128,192,0,12,6,
0,0,48,24,0,0,192,96,0,6,3,0,0,24,12,0,
0,192,96,0,3,1,128,0,24,12,0,0,32,16,0,0,
0,224,0,16,25,0,3,3,8,0,96,96,192,28,14,11,
143,128,192,135,24,28,8,3,1,192,128,96,28,8,12,3,
129,1,128,56,16,48,3,129,6,0,56,32,96,3,130,12,
0,24,65,128,1,136,48,0,7,6,7,0,0,192,200,0,
24,24,64,1,131,4,0,48,112,64,6,6,4,0,192,224,
64,24,14,4,3,0,224,64,96,28,8,12,1,192,128,192,
28,8,24,1,193,3,0,28,16,96,0,194,12,0,12,64,
128,0,56,0,24,243,207,4,16,130,16,132,32,0,3,0,
6,0,8,0,24,0,48,0,96,0,192,0,192,1,128,3,
0,3,0,6,0,6,0,12,0,12,0,28,0,24,0,24,
0,56,0,48,0,48,0,112,0,112,0,96,0,96,0,96,
0,96,0,224,0,224,0,224,0,224,0,224,0,96,0,96,
0,96,0,96,0,96,0,32,0,48,0,16,0,8,0,8,
0,0,16,0,16,0,8,0,12,0,4,0,6,0,6,0,
6,0,6,0,6,0,7,0,7,0,7,0,7,0,7,0,
6,0,6,0,6,0,6,0,14,0,14,0,12,0,12,0,
28,0,24,0,24,0,56,0,48,0,48,0,96,0,96,0,
192,0,192,1,128,3,0,3,0,6,0,12,0,24,0,16,
0,96,0,192,0,0,48,0,48,0,24,0,12,1,132,25,
226,56,59,56,15,112,1,224,1,224,3,188,7,55,7,17,
230,8,96,12,0,6,0,3,0,3,0,0,0,1,0,0,
3,0,0,3,0,0,3,0,0,3,0,0,6,0,0,6,
0,0,6,0,0,6,0,0,12,0,0,12,0,0,12,0,
255,255,255,255,255,255,0,24,0,0,24,0,0,24,0,0,
48,0,0,48,0,0,48,0,0,48,0,0,96,0,0,96,
0,0,96,0,0,96,0,0,64,0,25,222,112,132,66,34,
17,0,51,252,192,0,0,2,0,0,12,0,0,24,0,0,
96,0,0,192,0,3,0,0,12,0,0,24,0,0,96,0,
0,192,0,3,0,0,6,0,0,24,0,0,48,0,0,192,
0,3,0,0,6,0,0,24,0,0,48,0,0,192,0,1,
128,0,6,0,0,12,0,0,48,0,0,192,0,1,128,0,
6,0,0,12,0,0,48,0,0,96,0,1,128,0,6,0,
0,12,0,0,48,0,0,96,0,1,128,0,3,0,0,12,
0,0,24,0,0,96,0,0,128,0,0,6,15,15,6,0,
0,0,0,0,0,0,0,0,0,48,120,240,96,3,3,193,
224,96,0,0,0,0,0,0,0,0,0,0,0,96,112,120,
28,2,1,1,0,128,128,128,64,64,0,15,255,255,225,255,
255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,255,255,
255,135,255,255,240,0,7,240,0,6,3,0,6,0,32,2,
0,4,1,0,0,128,128,0,32,64,120,8,32,97,1,16,
112,32,68,56,7,18,12,1,196,135,0,225,67,128,56,80,
224,14,20,56,3,134,28,1,194,135,0,112,161,192,28,40,
48,7,18,12,3,132,131,129,226,32,113,152,132,7,131,193,
0,0,0,32,0,0,8,0,0,1,128,0,224,24,7,192,
1,255,0,0,0,63,128,63,128,24,0,12,0,6,0,6,
0,3,0,1,128,0,192,0,192,0,96,0,48,0,24,0,
24,0,12,0,6,0,3,0,3,0,1,128,0,192,0,96,
0,96,0,48,0,24,0,12,0,12,0,6,0,3,0,1,
128,1,128,0,192,0,96,0,48,0,48,0,24,0,12,0,
6,0,6,0,3,0,1,252,0,254,0,0,0,63,128,63,
128,0,192,0,96,0,48,0,48,0,24,0,12,0,6,0,
6,0,3,0,1,128,0,192,0,192,0,96,0,48,0,24,
0,24,0,12,0,6,0,3,0,3,0,1,128,0,192,0,
96,0,96,0,48,0,24,0,12,0,12,0,6,0,3,0,
1,128,1,128,0,192,0,96,0,48,0,48,0,24,1,252,
0,254,0,0,6,16,65,2,8,16,64,177,243,227,128,231,
56,227,8,97,3,7,14,28,56,96,192,128,192,88,99,112,
112,8,0,128,96,24,10,6,195,31,131,192,255,240,60,66,
129,129,130,132,120,14,3,128,240,14,1,192,224,115,248,0,
224,224,96,112,0,2,3,6,198,38,14,4,111,254,12,49,
199,56,227,12,48,198,24,65,8,32,12,9,240,144,249,3,
0,97,188,254,63,6,24,49,227,207,30,120,240,64,130,4,
32,65,2,16,33,2,16,33,2,0,255,255,255,252,6,12,
32,65,2,8,16,32,65,2,4,8,32,64,177,99,231,207,
159,28,56,255,255,192,255,255,255,255,248,
};
METRIC cmtt10_metric[128] = {
{2856, -1, 24, 19, 25, 344061},
{2916, -2, 24, 17, 25, 344061},
{2970, -2, 24, 17, 25, 344061},
{3024, -1, 24, 19, 25, 344061},
{3084, -1, 24, 19, 25, 344061},
{3144, -1, 24, 19, 25, 344061},
{3204, -2, 24, 17, 25, 344061},
{3258, -2, 24, 17, 25, 344061},
{3312, -2, 24, 17, 25, 344061},
{3366, -2, 24, 17, 25, 344061},
{3420, -1, 24, 19, 25, 344061},
{5502, -2, 24, 17, 25, 344061},
{5448, -2, 24, 17, 25, 344061},
{5887, -8, 25, 5, 12, 344061},
{4736, -9, 15, 4, 25, 344061},
{4193, -3, 15, 15, 25, 344061},
{4240, -3, 17, 16, 18, 344061},
{4276, -2, 17, 13, 27, 344061},
{5362, -5, 24, 8, 5, 344061},
{5367, -8, 24, 8, 5, 344061},
{5372, -4, 22, 13, 5, 344061},
{5381, -3, 24, 15, 4, 344061},
{5389, -3, 22, 15, 3, 344061},
{5395, -7, 24, 7, 5, 344061},
{5400, -6, 2, 11, 11, 344061},
{4320, -1, 24, 19, 25, 344061},
{4380, -1, 17, 19, 18, 344061},
{4423, -1, 17, 19, 18, 344061},
{4466, -2, 23, 17, 31, 344061},
{4532, -1, 24, 19, 25, 344061},
{4592, -1, 24, 19, 25, 344061},
{4652, -2, 28, 17, 33, 344061},
{5895, -2, 9, 17, 16, 344061},
{4723, -9, 24, 4, 25, 344061},
{5869, -4, 24, 13, 11, 344061},
{4749, -1, 24, 19, 25, 344061},
{4018, -2, 28, 17, 32, 344061},
{4809, -1, 28, 19, 32, 344061},
{4086, -1, 24, 19, 25, 344061},
{4885, -8, 24, 6, 13, 344061},
{4895, -6, 28, 11, 33, 344061},
{4941, -5, 28, 10, 33, 344061},
{4983, -2, 21, 17, 18, 344061},
{5022, -2, 21, 17, 19, 344061},
{5063, -7, 4, 7, 11, 344061},
{5441, -2, 13, 17, 3, 344061},
{5073, -8, 4, 5, 5, 344061},
{5077, -2, 28, 17, 32, 344061},
{3480, -2, 24, 17, 25, 344061},
{3534, -4, 24, 14, 25, 344061},
{3578, -2, 24, 17, 25, 344061},
{3632, -2, 24, 17, 25, 344061},
{3686, -1, 24, 19, 25, 344061},
{3746, -2, 24, 17, 25, 344061},
{3800, -2, 24, 17, 25, 344061},
{3854, -2, 25, 17, 26, 344061},
{3910, -2, 24, 17, 25, 344061},
{3964, -2, 24, 17, 25, 344061},
{5145, -8, 17, 5, 18, 344061},
{5157, -7, 17, 6, 24, 344061},
{5556, -2, 22, 17, 21, 344061},
{5175, -2, 17, 17, 11, 344061},
{5601, -2, 22, 17, 21, 344061},
{4146, -3, 24, 15, 25, 344061},
{5199, -2, 24, 18, 25, 344061},
{0, -1, 24, 19, 25, 344061},
{60, -1, 24, 18, 25, 344061},
{117, -2, 24, 17, 25, 344061},
{171, 0, 24, 19, 25, 344061},
{231, -1, 24, 19, 25, 344061},
{291, -1, 24, 19, 25, 344061},
{351, -2, 24, 18, 25, 344061},
{408, -1, 24, 19, 25, 344061},
{468, -3, 24, 15, 25, 344061},
{515, -3, 24, 16, 25, 344061},
{565, 0, 24, 20, 25, 344061},
{628, -1, 24, 19, 25, 344061},
{688, -1, 24, 19, 25, 344061},
{748, -1, 24, 19, 25, 344061},
{808, -2, 24, 17, 25, 344061},
{862, -1, 24, 18, 25, 344061},
{919, -2, 24, 17, 31, 344061},
{985, -1, 24, 20, 25, 344061},
{1048, -2, 24, 17, 25, 344061},
{1102, -1, 24, 19, 25, 344061},
{1162, 0, 24, 21, 25, 344061},
{1228, -1, 24, 19, 25, 344061},
{1288, -1, 24, 19, 25, 344061},
{1348, -1, 24, 19, 25, 344061},
{1408, -1, 24, 19, 25, 344061},
{1468, -2, 24, 17, 25, 344061},
{5256, -8, 28, 12, 32, 344061},
{5653, -2, 28, 17, 32, 344061},
{5304, -1, 28, 12, 32, 344061},
{5416, -4, 24, 13, 6, 344061},
{5646, -2, -2, 17, 3, 344061},
{5352, -8, 27, 6, 13, 344061},
{1522, -2, 17, 19, 18, 344061},
{1565, 0, 24, 20, 25, 344061},
{1628, -3, 17, 16, 18, 344061},
{1664, -1, 24, 20, 25, 344061},
{1727, -3, 17, 16, 18, 344061},
{1763, -1, 24, 17, 25, 344061},
{1817, -1, 17, 19, 28, 344061},
{1884, 0, 24, 21, 25, 344061},
{1950, -3, 25, 16, 26, 344061},
{2002, -2, 25, 13, 35, 344061},
{2059, 0, 24, 20, 25, 344061},
{2122, -2, 24, 17, 25, 344061},
{2176, 0, 17, 21, 18, 344061},
{2224, 0, 17, 21, 18, 344061},
{2272, -2, 17, 17, 18, 344061},
{2311, 0, 17, 20, 27, 344061},
{2379, -2, 17, 20, 27, 344061},
{2447, -1, 17, 19, 18, 344061},
{2490, -3, 17, 15, 18, 344061},
{2524, -1, 22, 17, 23, 344061},
{2573, 0, 17, 21, 18, 344061},
{2621, -1, 17, 19, 18, 344061},
{2664, -1, 17, 19, 18, 344061},
{2707, -1, 17, 19, 18, 344061},
{2750, -1, 17, 19, 27, 344061},
{2815, -1, 17, 18, 18, 344061},
{5733, -2, 28, 17, 32, 344061},
{5721, -9, 28, 3, 32, 344061},
{5801, -2, 28, 17, 32, 344061},
{5426, -4, 24, 13, 5, 344061},
{5435, -5, 25, 11, 4, 344061},
};
char cmtt10_bitmap[5929] = {
0,224,0,62,0,7,192,0,216,0,27,0,7,112,0,238,
0,29,192,3,24,0,227,128,28,112,3,142,0,113,192,14,
56,3,131,128,112,112,15,254,1,255,192,127,252,14,3,129,
192,112,56,14,31,199,247,248,255,127,31,192,127,248,63,255,
135,255,240,112,60,28,7,135,0,225,192,56,112,14,28,7,
135,1,193,255,240,127,248,31,254,7,7,193,192,56,112,15,
28,1,199,0,113,192,28,112,7,28,3,199,1,231,255,251,
255,252,127,252,0,1,241,131,253,195,255,227,225,243,192,121,
192,29,192,14,224,7,240,0,112,0,56,0,28,0,14,0,
7,0,3,128,1,192,0,240,0,56,1,220,0,231,0,115,
192,112,248,120,63,248,15,248,1,240,0,127,248,31,255,193,
255,252,14,7,129,192,120,56,7,135,0,112,224,14,28,1,
227,128,28,112,3,142,0,113,192,14,56,1,199,0,56,224,
7,28,0,227,128,56,112,7,14,1,225,192,120,56,31,31,
255,199,255,240,127,248,0,127,255,223,255,249,255,255,14,0,
225,192,28,56,3,135,0,112,224,0,28,0,3,131,128,112,
112,15,254,1,255,192,63,248,7,7,0,224,224,28,0,3,
128,28,112,3,142,0,113,192,14,56,1,223,255,255,255,255,
127,255,224,255,255,255,255,255,255,255,142,0,113,192,14,56,
1,199,0,56,224,0,28,0,3,131,128,112,112,15,254,1,
255,192,63,248,7,7,0,224,224,28,0,3,128,0,112,0,
14,0,1,192,0,56,0,63,224,7,254,0,255,128,0,3,
227,1,255,192,255,240,120,124,60,15,14,1,199,0,113,192,
28,240,0,56,0,14,0,3,128,0,224,0,56,15,238,7,
255,128,254,240,7,28,1,199,0,112,224,60,60,15,7,135,
192,255,240,31,220,3,231,0,127,31,223,247,253,252,127,14,
3,129,192,112,56,14,7,1,192,224,56,28,7,3,128,224,
127,252,15,255,129,255,240,56,14,7,1,192,224,56,28,7,
3,128,224,112,28,14,3,129,192,112,56,14,31,199,247,253,
255,127,31,192,255,255,255,255,255,248,28,0,56,0,112,0,
224,1,192,3,128,7,0,14,0,28,0,56,0,112,0,224,
1,192,3,128,7,0,14,0,28,0,56,0,112,63,255,255,
255,255,254,7,254,7,255,7,254,0,56,0,56,0,56,0,
56,0,56,0,56,0,56,0,56,0,56,0,56,0,56,0,
56,0,56,0,56,0,56,0,56,96,56,240,56,240,112,127,
240,127,224,31,128,127,15,239,248,255,127,15,225,192,120,28,
15,1,192,224,28,28,1,195,192,28,120,1,207,0,28,224,
1,223,0,31,240,1,255,128,31,56,1,225,192,28,28,1,
192,224,28,14,1,192,112,28,7,1,192,56,127,7,239,248,
255,127,7,224,255,192,31,248,3,255,0,14,0,1,192,0,
56,0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,
1,192,0,56,0,7,0,0,224,0,28,0,3,128,8,112,
3,142,0,113,192,14,56,1,255,255,255,255,255,255,255,224,
252,7,255,193,255,248,63,157,5,195,177,184,118,55,14,198,
225,216,220,59,27,135,119,112,230,206,28,217,195,155,56,115,
103,14,124,225,199,28,56,227,135,0,112,224,14,28,1,195,
128,56,112,7,63,131,255,240,127,254,15,224,126,31,223,231,
253,252,127,14,131,129,216,112,59,14,7,97,192,238,56,29,
199,3,152,224,115,28,14,115,129,206,112,57,206,7,25,192,
227,56,28,119,3,142,224,112,220,14,27,129,195,112,56,46,
31,199,199,252,248,127,15,0,31,252,31,255,31,255,207,1,
239,0,127,0,31,128,15,192,7,224,3,240,1,248,0,252,
0,126,0,63,0,31,128,15,192,7,224,3,240,1,248,0,
254,0,255,0,123,192,121,255,252,127,252,31,252,0,127,248,
63,255,135,255,240,112,62,28,3,135,0,241,192,28,112,7,
28,1,199,0,241,192,56,112,62,31,255,7,255,129,255,128,
112,0,28,0,7,0,1,192,0,112,0,28,0,7,0,7,
240,3,254,0,127,0,0,31,252,31,255,31,255,207,1,239,
0,127,0,31,128,15,192,7,224,3,240,1,248,0,252,0,
126,0,63,0,31,128,15,192,7,224,3,240,1,248,56,252,
60,127,15,123,195,249,255,252,127,252,31,252,0,14,0,7,
128,1,192,0,240,0,56,0,28,127,224,15,255,128,127,252,
1,193,224,28,15,1,192,112,28,7,1,192,112,28,7,1,
192,240,28,30,1,255,192,31,248,1,255,192,28,28,1,192,
224,28,14,1,192,224,28,14,1,192,226,28,14,113,192,231,
127,7,239,248,126,127,3,192,7,227,15,255,143,255,207,3,
239,0,119,0,59,128,29,192,0,240,0,60,0,15,224,3,
254,0,127,192,7,240,0,60,0,14,0,3,128,1,216,0,
252,0,126,0,119,192,123,255,249,255,248,199,240,0,127,255,
255,255,255,255,255,240,112,126,14,15,193,193,248,56,56,7,
0,0,224,0,28,0,3,128,0,112,0,14,0,1,192,0,
56,0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,
1,192,1,255,0,127,240,7,252,0,127,7,247,252,127,223,
193,252,56,3,129,192,28,14,0,224,112,7,3,128,56,28,
1,192,224,14,7,0,112,56,3,129,192,28,14,0,224,112,
7,3,128,56,28,1,192,224,14,7,0,112,28,7,0,224,
56,3,131,128,31,252,0,127,192,0,248,0,254,15,255,227,
255,248,63,156,1,193,192,112,56,14,7,1,192,224,56,14,
14,1,193,192,56,56,7,7,0,96,192,14,56,1,199,0,
56,224,7,28,0,99,0,14,224,1,220,0,59,128,3,96,
0,124,0,15,128,0,224,0,252,7,255,193,255,240,31,184,
0,231,0,28,224,3,140,0,97,192,28,56,3,135,0,112,
227,142,28,249,195,159,56,115,103,6,108,192,205,152,25,179,
3,54,96,102,204,12,217,129,145,48,54,54,3,199,128,120,
240,14,14,0,127,31,143,231,241,252,126,7,15,0,225,192,
14,120,1,206,0,29,192,3,240,0,62,0,7,128,0,112,
0,30,0,3,224,0,252,0,29,192,7,184,0,227,128,28,
112,7,7,0,224,224,56,14,31,199,247,248,255,127,31,192,
254,15,255,227,255,248,63,142,3,129,192,112,28,28,3,131,
128,56,224,7,28,0,227,128,14,224,1,220,0,31,0,3,
224,0,56,0,7,0,0,224,0,28,0,3,128,0,112,0,
14,0,1,192,0,254,0,63,224,3,248,0,127,255,255,255,
255,255,252,0,238,0,247,0,243,128,112,0,120,0,120,0,
56,0,60,0,60,0,28,0,30,0,30,0,14,0,15,0,
15,1,199,0,231,128,119,128,59,128,31,255,255,255,255,255,
255,128,31,224,7,254,1,255,224,60,30,3,0,224,0,28,
0,3,128,31,240,31,254,7,193,193,192,56,112,7,14,0,
225,192,28,30,15,131,255,255,63,231,225,240,124,126,0,15,
224,0,126,0,0,224,0,14,0,0,224,0,14,0,0,227,
224,14,255,0,255,248,15,131,192,240,14,14,0,224,224,7,
14,0,112,224,7,14,0,112,224,7,14,0,112,224,14,15,
1,224,248,60,15,255,128,239,240,6,60,0,3,248,15,252,
31,254,60,30,120,12,112,0,224,0,224,0,224,0,224,0,
224,0,240,0,112,7,120,7,62,14,31,252,15,248,3,240,
0,63,0,7,240,0,63,0,0,112,0,7,0,0,112,0,
7,0,60,112,15,247,1,255,240,60,31,7,128,240,112,7,
14,0,112,224,7,14,0,112,224,7,14,0,112,224,7,7,
0,240,112,15,3,193,240,31,255,224,254,127,7,199,224,3,
224,15,248,31,252,60,30,120,14,112,7,224,7,255,255,255,
255,255,255,224,0,224,0,112,7,120,7,60,15,31,254,15,
252,3,240,0,31,0,63,192,63,224,60,240,28,48,14,0,
7,0,255,254,255,255,127,255,128,112,0,56,0,28,0,14,
0,7,0,3,128,1,192,0,224,0,112,0,56,0,28,0,
14,0,255,248,255,254,63,254,0,3,227,192,254,252,63,255,
142,14,99,128,224,112,28,14,3,129,192,112,56,14,3,131,
128,63,224,15,248,1,190,0,112,0,6,0,0,255,224,31,
255,7,255,241,224,15,112,0,126,0,15,192,1,248,0,59,
128,14,124,7,199,255,240,63,248,1,252,0,126,0,7,240,
0,31,128,0,28,0,0,224,0,7,0,0,56,0,1,199,
128,14,254,0,127,248,3,225,224,30,7,0,224,56,7,1,
192,56,14,1,192,112,14,3,128,112,28,3,128,224,28,7,
0,224,56,7,1,193,255,31,223,252,255,127,199,240,1,128,
3,192,3,192,1,128,0,0,0,0,0,0,0,0,127,192,
127,192,127,192,1,192,1,192,1,192,1,192,1,192,1,192,
1,192,1,192,1,192,1,192,1,192,1,192,127,255,255,255,
127,255,0,48,3,192,30,0,96,0,0,0,0,0,0,31,
248,255,199,254,0,112,3,128,28,0,224,7,0,56,1,192,
14,0,112,3,128,28,0,224,7,0,56,1,192,14,0,112,
3,128,29,129,222,30,255,227,254,15,224,126,0,15,224,0,
126,0,0,224,0,14,0,0,224,0,14,0,0,231,254,14,
127,224,231,254,14,15,0,225,224,14,60,0,231,128,14,240,
0,255,0,15,248,0,251,192,15,30,0,224,224,14,7,0,
224,120,127,135,255,252,255,127,135,240,255,192,127,224,63,240,
0,56,0,28,0,14,0,7,0,3,128,1,192,0,224,0,
112,0,56,0,28,0,14,0,7,0,3,128,1,192,0,224,
0,112,0,56,0,28,0,14,3,255,255,255,255,255,255,128,
249,195,135,255,126,63,255,248,120,241,195,199,142,28,56,112,
225,195,135,14,28,56,112,225,195,135,14,28,56,112,225,195,
135,14,28,56,112,225,195,159,207,159,254,124,255,241,227,192,
126,60,7,247,240,31,255,192,31,15,0,240,56,7,1,192,
56,14,1,192,112,14,3,128,112,28,3,128,224,28,7,0,
224,56,7,1,192,56,14,15,248,254,255,231,251,254,63,128,
3,224,7,252,7,255,7,131,199,128,243,128,59,128,15,192,
7,224,3,240,1,248,0,254,0,247,0,115,192,120,240,120,
63,248,15,248,1,240,0,126,62,15,239,240,127,255,128,248,
60,15,0,224,224,14,14,0,112,224,7,14,0,112,224,7,
14,0,112,224,7,14,0,224,240,30,15,131,192,255,248,14,
255,0,227,192,14,0,0,224,0,14,0,0,224,0,14,0,
0,224,0,127,192,15,254,0,127,192,0,7,199,0,254,112,
31,247,3,193,240,112,15,7,0,240,224,7,14,0,112,224,
7,14,0,112,224,7,14,0,112,112,15,7,0,240,60,63,
3,255,112,31,231,0,124,112,0,7,0,0,112,0,7,0,
0,112,0,7,0,0,112,0,63,224,7,255,0,63,224,255,
15,223,231,255,253,255,131,248,32,124,0,15,0,1,224,0,
56,0,7,0,0,224,0,28,0,3,128,0,112,0,14,0,
1,192,7,255,224,255,252,31,255,128,15,236,127,249,255,247,
129,238,1,220,3,156,0,63,192,31,240,15,240,0,243,0,
126,0,254,1,254,7,255,254,255,249,143,192,3,0,3,128,
1,192,0,224,0,112,3,255,251,255,253,255,254,7,0,3,
128,1,192,0,224,0,112,0,56,0,28,0,14,2,7,3,
131,129,193,192,224,240,224,63,224,15,224,3,224,126,31,135,
241,252,31,135,224,28,7,0,224,56,7,1,192,56,14,1,
192,112,14,3,128,112,28,3,128,224,28,7,0,224,56,7,
1,192,56,62,1,255,254,7,251,248,31,31,128,127,31,223,
227,253,252,127,14,3,129,224,240,28,28,3,131,128,112,112,
7,28,0,227,128,28,112,3,142,0,59,128,7,112,0,238,
0,15,128,1,240,0,28,0,255,31,255,247,255,252,127,156,
1,195,128,56,112,7,14,0,225,199,28,25,243,3,62,96,
102,204,14,219,129,219,112,59,110,7,109,192,120,240,15,30,
1,227,192,127,31,207,231,249,252,127,7,142,0,115,192,7,
112,0,252,0,15,128,0,224,0,60,0,7,192,1,220,0,
115,192,14,56,3,131,131,248,254,255,63,239,227,248,127,31,
223,243,253,252,127,14,3,128,224,112,28,28,3,131,128,56,
112,7,28,0,227,128,14,112,1,206,0,57,128,3,112,0,
110,0,7,128,0,240,0,30,0,3,128,0,112,0,14,0,
3,128,30,112,3,222,0,127,128,7,224,0,240,0,0,63,
255,223,255,247,255,253,192,30,112,15,28,7,128,3,192,1,
224,1,240,0,248,0,120,0,60,0,30,1,207,0,119,128,
31,255,255,255,255,255,255,240,255,255,255,255,255,255,255,142,
0,113,192,14,56,1,199,0,56,224,0,28,0,3,128,0,
112,0,14,0,1,192,0,56,0,7,0,0,224,0,28,0,
3,128,0,112,0,14,0,1,192,0,56,0,63,240,7,254,
0,255,192,0,1,192,1,240,0,248,0,108,0,119,0,59,
128,24,192,28,112,14,56,7,28,7,7,3,131,129,193,192,
224,224,224,56,112,28,56,14,56,3,156,1,206,0,239,0,
127,0,31,255,255,255,255,255,255,128,31,252,31,255,31,255,
207,1,239,0,127,0,31,128,15,192,7,224,3,240,1,248,
248,252,124,126,62,63,31,31,143,143,192,7,224,3,240,1,
248,0,252,0,127,0,123,192,121,255,252,127,252,31,252,0,
0,224,0,62,0,7,192,0,216,0,27,0,7,112,0,238,
0,29,192,3,24,0,227,128,28,112,3,142,0,113,192,14,
56,3,131,128,112,112,14,14,1,193,192,112,28,14,3,129,
192,112,56,14,31,199,247,248,255,127,31,192,63,255,207,255,
249,255,255,56,0,231,0,28,0,0,0,0,0,0,0,0,
0,1,193,192,63,248,7,255,0,255,224,28,28,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,14,0,15,192,1,
255,255,255,255,255,127,255,224,127,255,223,255,253,255,255,14,
3,129,192,112,56,14,7,1,192,224,56,28,7,3,128,224,
112,28,14,3,129,192,112,56,14,7,1,192,224,56,28,7,
3,128,224,112,28,14,3,129,192,112,56,14,31,199,247,253,
255,127,31,192,127,255,255,255,255,255,238,0,115,128,57,192,
28,112,14,60,0,14,0,3,128,1,192,0,112,0,56,0,
28,0,28,0,14,0,14,0,15,0,7,0,231,0,115,128,
59,128,29,255,255,255,255,127,255,128,28,28,63,31,157,221,
220,108,126,54,63,14,28,7,0,3,128,1,192,0,224,0,
112,0,56,0,28,0,14,0,7,0,3,128,1,192,0,224,
0,112,0,56,0,28,0,14,0,63,224,31,240,15,248,0,
15,248,7,252,3,254,0,56,0,28,0,14,0,7,0,15,
224,31,252,63,255,158,115,220,56,126,28,63,14,29,231,60,
255,254,31,252,3,248,0,112,0,56,0,28,0,14,0,63,
224,31,240,15,248,0,15,248,7,252,3,254,0,56,0,28,
3,14,31,135,15,227,143,113,199,56,227,156,113,206,56,231,
28,113,206,112,247,120,63,248,15,248,1,240,0,112,0,56,
0,28,0,14,0,63,224,31,240,15,248,0,1,240,0,255,
128,63,248,15,7,131,192,120,112,7,28,0,115,128,14,112,
1,206,0,57,192,7,56,0,227,128,56,112,7,14,0,224,
224,56,28,7,1,193,192,56,56,7,7,14,113,207,206,57,
255,199,255,248,255,127,15,224,3,224,7,252,7,255,3,199,
131,128,227,192,121,192,28,224,14,224,3,240,1,248,0,252,
0,126,0,63,0,31,128,15,192,7,240,7,184,3,156,1,
207,1,227,193,224,241,224,127,240,31,240,3,224,0,1,128,
14,0,56,1,224,15,129,254,15,248,28,224,3,128,14,0,
56,0,224,3,128,14,0,56,0,224,3,128,14,0,56,0,
224,3,128,14,7,255,159,255,127,248,7,224,15,252,15,255,
15,7,206,0,119,128,63,192,14,192,7,0,3,128,1,192,
1,192,0,224,0,224,0,224,0,224,0,224,0,224,0,224,
0,224,1,224,1,224,57,192,29,255,255,255,255,127,255,128,
7,224,15,252,15,255,15,3,199,128,113,128,56,0,28,0,
14,0,14,0,31,1,255,0,254,0,127,192,0,240,0,28,
0,6,0,3,128,1,216,0,254,0,126,0,115,192,240,255,
240,63,240,7,224,0,0,124,0,31,128,3,112,0,238,0,
57,192,7,56,1,199,0,120,224,14,28,3,195,128,240,112,
28,14,7,129,193,224,56,63,255,255,255,255,255,255,224,3,
128,0,112,0,14,0,1,192,0,56,0,127,240,15,254,1,
255,192,63,254,31,255,15,255,135,0,3,128,1,192,0,224,
0,112,0,56,0,28,0,14,252,7,255,131,255,225,224,120,
192,28,0,7,0,3,176,1,252,0,254,0,126,0,115,192,
240,255,240,63,240,7,224,0,0,248,1,254,1,255,129,224,
225,192,241,192,121,224,24,224,0,112,0,113,252,59,255,31,
255,207,128,247,128,59,192,15,192,7,224,3,184,1,220,0,
238,0,243,128,113,224,240,127,240,31,240,3,224,0,224,0,
127,255,255,255,255,255,254,0,247,0,240,0,112,0,112,0,
120,0,56,0,56,0,28,0,28,0,14,0,7,0,7,0,
3,128,1,192,0,224,0,224,0,112,0,56,0,28,0,14,
0,7,0,3,128,0,7,240,15,254,15,255,143,131,239,0,
127,0,31,128,15,192,7,112,7,62,15,135,255,0,254,1,
255,193,224,241,192,29,224,15,224,3,240,1,248,0,254,0,
247,0,115,224,248,255,248,63,248,7,240,0,3,224,7,252,
7,255,7,131,199,0,227,128,59,128,29,192,15,224,3,240,
1,248,1,238,0,247,128,249,255,252,127,238,31,199,0,7,
0,3,140,1,207,1,199,129,195,131,192,255,192,63,192,15,
128,0,0,192,0,224,0,112,0,56,0,127,0,255,224,255,
248,251,158,113,199,240,225,248,113,252,56,255,28,51,206,0,
255,0,63,224,7,252,0,255,0,115,192,56,230,28,63,142,
31,199,15,195,135,113,199,60,239,143,255,131,255,0,126,0,
14,0,7,0,1,128,3,128,0,252,0,63,128,15,56,1,
199,0,56,224,7,28,0,227,191,28,231,227,188,252,63,28,
7,199,0,240,224,60,28,15,199,3,248,224,115,156,28,127,
3,135,224,112,120,238,15,28,227,243,159,255,241,252,252,30,
7,0,15,224,127,241,255,247,128,255,0,254,1,216,7,128,
30,0,120,1,224,7,0,28,0,56,0,112,0,224,1,192,
3,0,0,0,0,0,0,0,0,0,96,1,224,3,192,3,
0,3,0,15,0,30,0,24,0,0,0,0,0,0,0,0,
3,0,7,0,14,0,56,0,112,0,224,3,128,15,0,60,
0,240,3,192,55,0,254,1,254,3,223,255,31,252,15,224,
127,192,127,192,127,192,1,192,1,192,1,192,1,192,1,192,
1,192,1,192,1,192,1,192,1,192,1,192,1,192,127,255,
255,255,127,255,31,248,255,199,254,0,112,3,128,28,0,224,
7,0,56,1,192,14,0,112,3,128,28,0,224,7,0,56,
1,192,14,0,112,3,128,29,129,222,30,255,227,254,15,224,
0,252,0,127,224,31,254,7,129,224,224,28,28,3,131,129,
227,241,248,254,126,15,207,192,56,60,7,1,192,224,28,28,
3,131,128,120,112,7,14,0,225,192,28,56,3,135,0,112,
226,28,28,231,159,159,231,241,248,126,30,0,31,14,7,243,
241,255,255,60,252,227,15,14,1,193,192,56,56,7,255,15,
255,231,255,253,243,128,112,112,14,15,15,193,225,252,126,123,
255,254,63,159,131,192,224,15,7,3,243,240,255,127,56,124,
231,15,143,192,225,248,28,63,3,255,224,127,252,15,255,129,
192,112,56,15,15,14,225,241,222,127,57,254,254,31,143,193,
224,224,0,1,0,1,192,0,224,0,224,0,112,0,112,15,
184,31,248,31,252,30,31,30,15,206,14,238,7,63,7,31,
135,143,195,135,227,195,241,193,221,193,207,225,227,225,224,255,
224,127,224,63,192,56,0,28,0,28,0,14,0,14,0,7,
0,1,0,0,7,255,225,255,252,31,255,129,220,112,59,142,
15,113,193,206,56,57,192,7,56,0,231,0,56,224,7,31,
224,227,252,28,127,131,142,0,255,192,31,248,3,255,28,112,
227,142,28,115,131,142,112,113,223,31,255,247,255,124,127,224,
31,255,239,255,253,255,255,188,60,127,7,143,192,241,248,30,
63,3,192,224,56,28,7,3,128,224,112,31,238,3,253,192,
127,184,14,7,1,192,224,56,28,15,31,129,227,240,60,127,
7,142,240,241,223,255,251,255,255,31,255,224,0,28,0,14,
0,7,0,7,1,255,193,255,241,255,252,240,254,240,119,240,
57,248,56,252,28,126,14,63,7,31,135,15,195,135,225,195,
240,225,248,224,252,112,126,56,63,28,31,156,15,206,7,247,
7,191,7,159,255,199,255,193,255,192,224,0,112,0,56,0,
8,0,0,111,255,255,255,255,255,255,255,96,0,6,255,96,
111,246,0,0,111,255,255,255,255,255,255,255,96,3,135,0,
112,224,14,28,1,195,128,56,112,7,14,31,255,247,255,255,
255,255,224,225,192,28,56,3,135,0,225,192,28,56,3,135,
0,112,224,255,255,255,255,253,255,255,14,28,1,195,128,56,
112,7,14,0,225,192,28,56,0,56,1,143,128,113,240,14,
119,3,142,224,113,220,28,59,131,135,112,112,238,28,15,131,
129,240,224,28,28,0,7,0,0,224,0,28,0,7,0,0,
224,0,56,0,7,0,0,224,0,56,0,7,7,1,193,240,
56,62,14,14,225,193,220,56,59,142,7,113,192,238,112,15,
142,1,240,192,28,49,231,207,28,113,199,57,239,56,64,0,
224,60,14,3,128,224,56,7,1,192,56,14,1,192,56,14,
1,192,56,7,0,224,28,3,128,112,14,0,224,28,3,128,
56,7,0,112,14,0,224,14,0,224,15,0,224,192,56,7,
0,224,28,3,128,224,28,7,0,224,56,14,1,192,112,28,
7,1,192,112,28,7,1,192,224,56,14,7,1,192,224,56,
28,14,7,3,128,192,0,1,192,0,224,0,112,0,56,7,
28,119,206,125,255,252,63,248,7,240,3,248,7,255,15,255,
239,156,251,142,56,7,0,3,128,1,192,0,224,0,1,192,
0,224,0,112,0,56,0,28,0,14,0,7,0,3,128,255,
255,255,255,255,255,224,56,0,28,0,14,0,7,0,3,128,
1,192,0,224,0,112,0,56,249,251,243,225,199,142,121,225,
128,119,255,247,0,0,1,128,1,192,0,224,0,224,0,112,
0,112,0,56,0,56,0,28,0,28,0,14,0,14,0,7,
0,7,0,3,128,3,128,1,192,1,192,0,224,0,224,0,
112,0,112,0,56,0,56,0,28,0,28,0,14,0,14,0,
7,0,7,0,3,128,1,128,0,119,255,247,0,0,0,0,
0,59,255,251,128,57,247,223,56,0,0,0,0,0,0,231,
159,124,241,199,57,206,16,127,255,127,255,255,255,224,0,0,
0,0,0,0,0,0,0,0,255,255,255,255,223,255,192,0,
248,0,255,0,127,224,60,28,28,63,142,31,231,143,249,199,
15,113,195,248,224,126,56,31,142,7,227,129,248,224,126,56,
31,142,7,113,195,156,112,231,143,240,225,248,28,60,3,192,
240,127,252,15,252,0,252,0,255,255,255,255,254,0,224,14,
0,224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,
224,14,0,224,14,0,224,14,0,224,14,0,224,14,0,224,
14,0,224,15,255,255,255,255,255,255,255,255,240,7,0,112,
7,0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,
0,112,7,0,112,7,0,112,7,0,112,7,0,112,7,0,
112,7,0,127,255,255,255,255,8,115,222,115,142,56,227,207,
158,48,224,248,126,31,7,7,31,126,248,224,224,63,239,255,
254,63,128,32,0,96,15,255,255,255,249,255,192,255,255,255,
253,255,248,125,255,31,247,192,6,1,192,60,7,224,126,1,
192,56,7,127,255,249,252,0,3,0,124,15,249,251,255,7,
160,8,28,49,251,255,255,239,198,28,0,96,220,31,131,176,
96,255,255,255,255,255,255,224,1,192,0,224,0,112,0,56,
0,28,0,14,0,7,0,3,128,1,192,0,224,0,112,0,
56,0,28,0,14,0,7,0,3,128,225,195,252,231,255,119,
227,255,128,255,128,63,128,15,128,3,128,0,192,0,0,192,
0,224,0,248,0,254,0,255,128,255,227,247,127,243,159,225,
195,128,224,0,112,0,56,0,28,0,14,0,7,0,3,128,
1,192,0,224,0,112,0,56,0,28,0,14,0,7,0,3,
128,1,192,0,0,1,128,3,192,7,224,7,192,15,128,15,
128,31,0,31,0,62,0,126,0,60,0,31,128,3,224,0,
124,0,31,0,3,224,0,248,0,31,0,7,224,0,240,0,
24,192,0,120,0,63,0,7,192,0,248,0,62,0,7,192,
1,240,0,62,0,15,192,1,224,3,240,3,224,7,192,7,
192,15,128,15,128,31,0,63,0,30,0,12,0,0,255,255,
255,255,255,255,224,192,0,112,0,56,0,14,0,7,0,1,
192,0,224,0,56,0,28,0,7,0,3,128,0,224,0,112,
0,28,0,14,0,3,128,1,192,0,112,0,56,0,14,0,
7,0,1,192,0,224,0,56,0,28,0,7,0,3,128,0,
224,0,112,0,28,0,14,0,3,255,255,255,255,255,255,255,
255,255,255,255,255,0,31,128,63,192,63,224,60,0,28,0,
14,0,7,0,3,128,1,192,0,224,0,112,0,56,0,28,
0,30,1,254,1,254,0,255,0,63,192,0,240,0,56,0,
28,0,14,0,7,0,3,128,1,192,0,224,0,112,0,56,
0,30,0,7,252,1,254,0,63,124,0,127,128,63,224,0,
120,0,28,0,14,0,7,0,3,128,1,192,0,224,0,112,
0,56,0,28,0,15,0,3,252,0,255,0,127,128,127,128,
120,0,56,0,28,0,14,0,7,0,3,128,1,192,0,224,
0,112,0,56,0,60,7,252,3,252,0,248,0,224,63,131,
252,31,224,255,7,248,63,193,254,15,224,63,1,216,12,35,
190,231,57,206,115,156,224,192,1,224,0,240,0,120,0,60,
0,30,0,15,0,7,128,3,192,1,224,0,240,0,120,0,
60,0,30,0,15,255,255,255,255,
};

METRIC cmbx10_metric_small[128] = {

{1022, -1, 13, 11, 14, 453288},

{1042, -1, 13, 17, 14, 628050},

{1072, -1, 13, 16, 14, 586180},

{1100, -1, 13, 14, 14, 527926},

{1125, -1, 13, 13, 14, 502440},

{1148, -1, 13, 16, 14, 589821},

{1176, -1, 13, 14, 14, 544310},

{1201, -1, 13, 16, 14, 586180},

{1229, -1, 13, 14, 14, 544310},

{1254, -1, 13, 16, 14, 586180},

{1282, -1, 13, 14, 14, 544310},

{2028, 0, 13, 15, 14, 439635},

{2055, 0, 13, 11, 14, 418700},

{2075, 0, 13, 11, 14, 418700},

{2095, 0, 13, 18, 14, 628050},

{2127, 0, 13, 18, 14, 628050},

{1533, -1, 8, 4, 9, 209350},

{1538, 2, 8, 6, 13, 230285},

{1997, -3, 13, 3, 4, 376830},

{1999, -5, 13, 3, 4, 376830},

{2001, -3, 12, 5, 2, 376830},

{2003, -2, 13, 7, 4, 376830},

{2007, -2, 11, 7, 1, 376830},

{2008, -6, 13, 5, 3, 569796},

{2010, -3, -1, 6, 4, 334960},

{1548, -1, 13, 9, 14, 391394},

{1564, -1, 8, 14, 9, 544310},

{1580, -1, 8, 16, 9, 586180},

{1598, -1, 10, 9, 13, 376830},

{1613, -1, 13, 19, 14, 682663},

{1647, -1, 13, 21, 14, 766403},

{1684, -1, 14, 16, 16, 586180},

{2013, 0, 7, 6, 2, 209350},

{1716, -2, 13, 3, 14, 229375},

{2159, -1, 13, 8, 7, 395035},

{1728, -1, 13, 17, 18, 628050},

{1457, -1, 15, 9, 17, 376830},

{1767, -1, 15, 17, 17, 628050},

{1477, -1, 13, 16, 14, 586180},

{1804, -2, 13, 3, 7, 209350},

{1807, -1, 15, 6, 22, 293090},

{1824, -1, 15, 6, 22, 293090},

{1841, -1, 15, 9, 9, 376830},

{1852, -1, 12, 16, 16, 586180},

{1884, -2, 2, 3, 7, 209350},

{2166, 0, 4, 6, 2, 251220},

{1887, -2, 2, 3, 3, 209350},

{1889, -1, 15, 9, 21, 376830},

{1307, -1, 12, 9, 13, 376830},

{1322, -2, 12, 7, 13, 376830},

{1334, -1, 12, 9, 13, 376830},

{1349, -1, 12, 9, 13, 376830},

{1364, -1, 12, 9, 13, 376830},

{1379, -1, 12, 9, 13, 376830},

{1394, -1, 12, 9, 13, 376830},

{1409, -1, 13, 10, 14, 376830},

{1427, -1, 12, 9, 13, 376830},

{1442, -1, 12, 9, 13, 376830},

{1913, -2, 8, 3, 9, 209350},

{1917, -2, 8, 3, 13, 209350},

{1722, -2, 9, 3, 14, 229375},

{1922, -1, 8, 16, 8, 586180},

{1519, -1, 9, 8, 14, 355895},

{1505, -1, 13, 8, 14, 355895},

{1938, -1, 13, 16, 14, 586180},

{0, -1, 13, 15, 14, 569796},

{27, -1, 13, 14, 14, 536118},

{52, -1, 13, 14, 14, 544310},

{77, -1, 13, 15, 14, 577988},

{104, -1, 13, 13, 14, 495158},

{127, -1, 13, 12, 14, 474223},

{148, -1, 13, 16, 14, 592552},

{176, -1, 13, 16, 14, 589821},

{204, -1, 13, 7, 14, 285809},

{217, -1, 13, 9, 14, 389573},

{233, -1, 13, 16, 14, 590731},

{261, -1, 13, 11, 14, 453288},

{281, -1, 13, 20, 14, 715431},

{316, -1, 13, 16, 14, 589821},

{344, -1, 13, 15, 14, 566155},

{371, -1, 13, 13, 14, 515183},

{394, -1, 13, 15, 18, 566155},

{428, -1, 13, 16, 14, 565245},

{456, -1, 13, 10, 14, 418700},

{474, -1, 13, 14, 14, 524285},

{499, -1, 13, 15, 14, 579809},

{526, -1, 13, 15, 14, 569796},

{553, -1, 13, 22, 14, 779146},

{592, -1, 13, 15, 14, 569796},

{619, 0, 13, 17, 14, 569796},

{649, -1, 13, 12, 14, 460570},

{1966, -1, 15, 5, 21, 209350},

{2168, -3, 13, 8, 7, 395035},

{1980, 0, 15, 5, 21, 209350},

{2015, -3, 13, 5, 3, 376830},

{2017, -1, 14, 4, 3, 209350},

{1994, -2, 13, 3, 7, 209350},

{670, -1, 8, 10, 9, 366362},

{682, -1, 13, 10, 14, 418700},

{700, -1, 8, 8, 9, 334960},

{709, -1, 13, 10, 14, 418700},

{727, -1, 8, 8, 9, 345428},

{736, 0, 13, 8, 14, 230285},

{750, -1, 8, 10, 14, 376830},

{768, -1, 13, 10, 14, 418700},

{786, -1, 14, 4, 15, 209350},

{794, 2, 14, 6, 19, 230285},

{809, -1, 13, 11, 14, 397765},

{829, -1, 13, 4, 14, 209350},

{836, -1, 8, 18, 9, 628050},

{857, -1, 8, 10, 9, 418700},

{869, -1, 8, 9, 9, 376830},

{880, -1, 8, 10, 13, 418700},

{897, -1, 8, 11, 13, 397765},

{915, -1, 8, 8, 9, 310385},

{924, -1, 8, 6, 9, 297277},

{931, 0, 12, 7, 13, 293090},

{943, -1, 8, 10, 9, 418700},

{955, -1, 8, 10, 9, 397765},

{967, -1, 8, 14, 9, 544310},

{983, -1, 8, 10, 9, 397765},

{995, 0, 8, 11, 13, 397765},

{1013, -1, 8, 8, 9, 334960},

{2175, 0, 5, 11, 1, 376830},

{2177, 0, 5, 23, 1, 753660},

{2019, -3, 13, 6, 3, 376830},

{2022, -2, 13, 7, 3, 376830},

{2025, -2, 14, 7, 3, 376830},

};

char cmbx10_bitmap_small[2180] = {

1,0,7,0,14,0,28,0,92,0,184,2,120,4,112,8,

224,63,224,65,193,131,194,3,159,31,192,255,224,225,195,131,

142,14,56,56,224,195,254,14,14,56,28,224,115,129,206,7,

56,59,255,192,7,228,112,115,0,220,1,224,7,128,14,0,

56,0,224,3,128,23,0,76,2,28,16,31,128,255,224,112,

112,224,113,192,99,128,231,1,206,3,156,7,56,14,112,28,

224,49,192,227,131,159,252,0,255,241,193,142,4,112,19,136,

156,64,254,7,16,56,137,192,78,6,112,35,131,127,248,255,

227,134,56,35,129,57,19,144,63,3,144,57,3,128,56,3,

128,56,15,240,7,228,28,28,48,12,112,4,224,4,224,0,

224,0,224,255,224,28,224,28,112,28,48,28,28,28,7,228,

254,127,56,28,56,28,56,28,56,28,56,28,63,252,56,28,

56,28,56,28,56,28,56,28,56,28,254,127,254,112,225,195,

135,14,28,56,112,225,195,159,192,127,135,3,129,192,224,112,

56,28,14,7,19,157,206,195,192,254,62,56,8,56,16,56,

32,56,192,57,128,59,192,61,192,56,224,56,112,56,56,56,

56,56,28,254,63,255,7,0,224,28,3,128,112,14,1,192,

56,39,4,225,156,35,141,255,128,252,3,243,192,60,46,5,

194,224,92,46,5,194,112,156,39,9,194,57,28,35,145,194,

30,28,33,225,194,30,28,32,193,207,140,127,248,31,60,4,

46,4,39,4,39,132,35,132,33,196,32,228,32,116,32,60,

32,60,32,28,32,12,248,4,15,224,48,96,192,99,128,238,

0,252,1,248,3,240,7,224,15,192,29,192,115,128,225,131,

1,252,0,255,193,195,142,14,112,115,131,156,28,225,199,248,

56,1,192,14,0,112,3,128,127,0,15,224,48,96,192,99,

128,238,0,252,1,248,3,240,7,224,15,192,29,192,115,156,

225,199,1,252,64,24,128,63,0,60,0,48,255,192,56,112,

56,56,56,56,56,56,56,112,63,192,56,224,56,112,56,112,

56,112,56,113,56,113,254,30,62,144,108,11,2,224,63,135,

240,254,3,192,56,14,3,193,175,192,255,255,14,56,56,96,

225,131,132,14,0,56,0,224,3,128,14,0,56,0,224,3,

128,127,192,254,62,112,16,224,33,192,67,128,135,1,14,2,

28,4,56,8,112,16,224,32,192,128,195,0,248,0,254,62,

112,16,224,32,224,129,193,3,130,3,136,7,16,7,64,14,

128,31,0,28,0,56,0,32,0,252,127,125,224,240,99,131,

193,14,15,4,28,78,32,113,56,129,196,226,3,161,208,14,

135,64,58,31,0,112,56,1,192,224,7,3,128,8,4,0,

254,124,120,64,112,128,114,0,248,0,240,0,224,1,224,7,

192,9,192,35,192,131,129,3,159,159,192,255,15,142,2,7,

131,1,193,0,113,0,60,128,14,128,7,128,1,192,0,224,

0,112,0,56,0,28,0,63,128,127,247,14,96,228,28,67,

128,120,7,0,224,30,17,193,56,23,3,112,127,255,60,24,

128,48,124,115,48,204,51,28,57,192,224,24,6,1,128,96,

27,199,25,131,96,216,54,13,131,113,147,192,62,99,192,192,

192,192,192,97,62,3,128,96,24,6,1,143,102,59,6,193,

176,108,27,6,99,143,112,60,98,195,193,255,192,192,97,62,

14,19,48,48,48,252,48,48,48,48,48,48,48,124,59,209,

28,99,24,68,62,12,3,248,127,48,108,27,6,99,15,128,

224,24,6,1,128,96,27,199,25,134,97,152,102,25,134,97,

188,240,78,64,0,230,102,102,102,240,8,112,128,0,3,195,

12,48,195,12,48,195,15,39,0,224,12,1,128,48,6,0,

199,152,67,48,108,15,193,152,49,134,25,231,192,230,102,102,

102,102,102,111,239,159,28,56,102,12,25,131,6,96,193,152,

48,102,12,25,131,6,241,227,192,239,28,102,25,134,97,152,

102,25,134,243,192,62,49,176,120,60,30,15,6,198,62,0,

239,28,102,13,131,96,216,54,13,198,111,24,6,1,128,240,

0,30,76,57,3,96,108,13,129,176,51,14,30,192,24,3,

0,96,30,254,119,103,96,96,96,96,96,248,126,24,124,120,

56,97,248,16,32,193,143,198,12,24,48,100,201,145,192,227,

152,102,25,134,97,152,102,25,142,61,192,241,216,35,16,196,

51,6,129,160,48,12,0,243,221,142,38,56,141,36,52,208,

211,65,134,6,24,24,96,243,220,67,32,112,12,7,131,48,

134,227,192,120,230,8,98,12,65,152,26,3,64,48,6,0,

128,16,100,7,0,255,198,142,140,24,49,113,99,255,255,199,

24,225,28,19,130,112,14,1,192,56,7,0,224,28,3,129,

254,0,0,128,0,224,0,112,0,92,0,111,0,35,128,33,

224,16,112,16,60,8,14,8,3,143,255,231,255,247,255,252,

7,224,28,56,48,12,112,14,228,39,231,231,231,231,231,231,

228,39,224,7,112,14,48,12,28,56,7,224,3,0,12,0,

48,1,224,7,128,47,0,156,2,112,16,224,67,129,14,8,

28,32,115,231,240,255,255,255,224,3,0,16,0,16,16,255,

135,252,32,32,0,32,3,0,31,255,255,252,255,255,56,28,

56,28,56,28,56,28,56,28,56,28,56,28,56,28,56,28,

56,28,56,28,56,28,254,127,255,249,192,103,128,143,1,28,

4,56,0,240,1,192,2,4,48,17,0,200,6,127,251,255,

224,120,30,254,63,135,113,3,96,3,192,1,192,1,192,1,

192,1,192,1,192,1,192,1,192,1,192,15,248,31,240,14,

0,56,7,248,115,187,142,126,57,248,231,227,157,206,225,254,

0,224,3,128,127,192,15,248,1,192,1,192,225,199,113,206,

113,206,113,206,113,206,57,220,29,216,7,240,1,192,1,192,

15,248,15,192,225,198,1,184,7,224,31,128,126,1,216,6,

48,50,192,217,2,94,30,120,121,225,224,62,49,144,88,60,

30,15,7,131,193,224,208,76,99,224,49,224,193,131,6,12,

24,48,96,193,143,224,62,33,184,252,48,24,28,12,12,12,

8,72,47,239,240,62,51,152,196,224,96,240,12,7,3,241,

248,232,99,224,6,3,3,130,193,97,49,25,140,255,131,1,

128,193,248,97,63,31,10,4,3,241,12,7,3,225,240,232,

99,192,30,25,152,200,12,6,243,141,131,193,224,208,108,97,

224,64,31,247,250,4,129,0,128,64,48,8,2,1,128,96,

24,6,0,28,17,24,204,103,177,240,252,223,195,224,240,108,

99,224,60,49,176,88,60,30,13,142,123,1,128,152,204,195,

192,8,4,15,141,44,142,79,161,248,126,31,130,249,60,156,

77,44,124,8,0,6,0,9,0,25,0,25,0,26,0,28,

31,12,4,28,8,110,16,198,16,195,32,193,192,97,195,62,

62,126,131,195,6,12,24,16,16,0,0,0,16,56,16,16,

56,16,0,0,0,16,16,16,32,192,195,193,126,230,102,102,

102,240,60,48,195,12,48,195,12,48,242,112,30,48,216,108,

54,55,113,140,194,97,176,216,108,182,119,56,60,241,142,32,

48,64,193,63,253,140,12,48,48,225,124,120,30,60,97,226,

64,193,192,193,192,255,192,192,64,192,97,225,30,30,1,0,

143,140,108,94,111,39,179,209,177,143,136,4,0,7,255,192,

46,24,5,193,1,56,16,39,2,8,226,1,28,64,127,248,

8,113,35,14,36,65,193,136,56,34,7,13,243,255,128,7,

255,240,225,225,140,15,4,224,120,30,3,196,240,14,35,128,

127,28,3,136,224,28,79,1,224,92,15,6,96,120,33,195,

195,3,255,248,0,4,7,232,24,24,48,60,112,46,224,71,

224,199,224,135,225,7,227,7,226,7,100,6,60,12,24,24,

23,224,32,0,95,254,146,0,46,128,93,0,18,95,254,128,

1,140,0,198,0,99,0,49,128,49,131,255,251,255,254,12,

96,6,48,3,24,1,140,31,255,247,255,240,99,0,99,0,

49,128,24,192,12,96,0,56,6,51,7,48,255,24,67,12,

35,6,17,131,9,128,201,128,57,128,0,199,0,196,64,198,

16,99,8,97,132,96,194,96,34,16,14,0,93,146,160,4,

33,12,33,132,48,195,12,48,195,12,16,96,131,4,8,16,

129,2,12,16,96,131,12,48,195,12,48,194,24,67,8,66,

0,8,4,58,255,241,199,255,174,16,8,0,1,128,1,128,

1,128,1,128,1,128,1,128,1,128,255,255,255,255,1,128,

1,128,1,128,1,128,1,128,1,128,1,128,93,146,160,93,

0,0,128,192,192,96,48,48,24,12,12,6,6,3,1,129,

128,192,96,96,48,24,24,12,0,93,0,23,64,93,0,23,

100,168,127,254,255,255,0,0,0,0,0,0,0,0,255,255,

127,254,7,224,24,24,32,4,67,194,134,49,140,49,140,49,

140,49,140,49,134,50,67,220,32,0,24,7,7,248,255,241,

140,99,24,198,49,140,99,24,198,49,255,128,255,198,49,140,

99,24,198,49,140,99,24,199,255,128,42,77,208,217,144,111,

64,219,128,131,5,17,192,254,116,92,48,96,254,31,128,34,

162,111,96,207,56,128,50,250,96,109,221,176,15,188,49,140,

195,1,134,3,12,31,255,12,48,24,96,48,192,97,128,195,

1,134,3,12,15,62,0,15,131,24,195,24,3,1,255,140,

49,134,48,198,24,195,24,99,12,243,192,15,195,24,195,24,

99,13,255,140,49,134,48,198,24,195,24,99,12,243,192,15,

223,6,28,99,6,24,193,128,48,96,63,255,227,6,24,193,

134,48,97,140,24,99,6,24,193,134,48,97,158,60,240,15,

223,134,28,99,6,24,193,134,48,97,191,255,227,6,24,193,

134,48,97,140,24,99,6,24,193,134,48,97,158,60,240,66,

231,99,33,33,66,132,255,240,33,66,132,132,198,231,66,255,

224,255,255,254,

};

METRIC cmbx5_metric_small[128] = {

{328, -1, 6, 6, 7, 293996},

{334, -1, 6, 10, 7, 406862},

{343, -1, 6, 9, 7, 380466},

{351, -1, 6, 8, 7, 338596},

{358, -1, 6, 7, 7, 327674},

{365, -1, 6, 9, 7, 377735},

{373, -1, 6, 8, 7, 354070},

{380, -1, 6, 9, 7, 380466},

{388, -1, 6, 8, 7, 354070},

{395, -1, 6, 9, 7, 380466},

{403, -1, 6, 8, 7, 354070},

{660, 1, 6, 10, 7, 299002},

{669, 1, 6, 8, 7, 285804},

{676, 1, 6, 8, 7, 285804},

{683, 1, 6, 13, 7, 428706},

{695, 1, 6, 13, 7, 428706},

{485, 0, 4, 3, 5, 142902},

{487, 1, 4, 4, 7, 156100},

{646, -2, 6, 2, 1, 248486},

{647, -3, 6, 2, 1, 248486},

{648, -2, 6, 3, 1, 248486},

{649, -1, 6, 5, 1, 248486},

{650, -1, 5, 5, 1, 248486},

{651, -4, 6, 3, 1, 364992},

{652, -2, -1, 3, 2, 222090},

{491, -1, 6, 6, 7, 249854},

{497, -1, 4, 9, 5, 354070},

{503, -1, 4, 10, 5, 380466},

{510, -1, 5, 5, 7, 248486},

{515, -1, 6, 12, 7, 438719},

{526, -1, 6, 14, 7, 491511},

{539, -1, 7, 9, 9, 380466},

{653, 0, 3, 4, 1, 142902},

{550, -1, 6, 2, 7, 155645},

{707, -1, 6, 5, 4, 252128},

{554, -1, 6, 10, 9, 406862},

{463, -1, 7, 5, 9, 248486},

{566, -1, 7, 10, 9, 406862},

{469, -1, 6, 9, 7, 380466},

{578, -1, 6, 2, 4, 142902},

{579, 0, 7, 4, 11, 195694},

{585, -1, 7, 4, 11, 195694},

{591, -1, 7, 5, 5, 248486},

{595, -1, 7, 9, 11, 380466},

{608, -1, 1, 2, 4, 142902},

{710, 0, 2, 3, 1, 169298},

{609, -1, 1, 2, 2, 142902},

{610, -1, 7, 5, 11, 248486},

{410, -1, 6, 5, 7, 248486},

{415, -1, 6, 5, 7, 248486},

{420, -1, 6, 5, 7, 248486},

{425, -1, 6, 5, 7, 248486},

{430, 0, 6, 7, 7, 248486},

{437, -1, 6, 5, 7, 248486},

{442, -1, 6, 5, 7, 248486},

{447, -1, 6, 6, 7, 248486},

{453, -1, 6, 5, 7, 248486},

{458, -1, 6, 5, 7, 248486},

{617, -1, 4, 2, 5, 142902},

{619, -1, 4, 2, 7, 142902},

{552, -1, 4, 2, 7, 155645},

{621, -1, 4, 9, 5, 380466},

{481, -1, 4, 4, 7, 235288},

{477, -1, 6, 4, 7, 235288},

{627, -1, 6, 9, 7, 380466},

{0, -1, 6, 9, 7, 364992},

{8, -1, 6, 8, 7, 346333},

{15, -1, 6, 8, 7, 354070},

{22, -1, 6, 9, 7, 372729},

{30, -1, 6, 8, 7, 320392},

{37, -1, 6, 7, 7, 307194},

{44, -1, 6, 9, 7, 383197},

{52, -1, 6, 9, 7, 377735},

{60, -1, 6, 4, 7, 180220},

{64, 0, 6, 6, 7, 253947},

{70, -1, 6, 10, 7, 378190},

{79, -1, 6, 6, 7, 293996},

{85, -1, 6, 11, 7, 456923},

{95, -1, 6, 9, 7, 377735},

{103, -1, 6, 9, 7, 367723},

{111, -1, 6, 8, 7, 333135},

{118, -1, 6, 9, 9, 367723},

{129, -1, 6, 9, 7, 361352},

{137, -1, 6, 6, 7, 274882},

{143, -1, 6, 8, 7, 341327},

{150, -1, 6, 9, 7, 371364},

{158, -1, 6, 9, 7, 364992},

{166, 0, 6, 15, 7, 496972},

{180, -1, 6, 9, 7, 364992},

{188, -1, 6, 9, 7, 364992},

{196, -1, 6, 7, 7, 301278},

{635, -1, 7, 3, 11, 143794},

{711, -1, 6, 5, 4, 252128},

{640, 0, 7, 3, 11, 143794},

{654, -2, 6, 3, 2, 248486},

{655, -1, 7, 2, 2, 142902},

{645, -1, 6, 2, 4, 142902},

{203, -1, 4, 6, 5, 241887},

{207, -1, 6, 6, 7, 274882},

{213, -1, 4, 5, 5, 222090},

{217, -1, 6, 6, 7, 274882},

{223, -1, 4, 5, 5, 226869},

{227, 1, 6, 6, 7, 156100},

{233, 0, 4, 7, 8, 248486},

{240, -1, 6, 6, 7, 274882},

{246, 0, 7, 3, 8, 142902},

{249, 1, 7, 4, 10, 156100},

{254, -1, 6, 5, 7, 261684},

{259, 0, 6, 3, 7, 142902},

{262, -1, 4, 11, 5, 406862},

{269, -1, 4, 6, 5, 274882},

{273, -1, 4, 5, 5, 248486},

{277, -1, 4, 6, 7, 274882},

{283, -1, 4, 5, 7, 261684},

{288, -1, 4, 4, 5, 203887},

{291, -1, 4, 3, 5, 198334},

{293, 0, 6, 4, 7, 195694},

{297, -1, 4, 6, 5, 274882},

{301, 0, 4, 7, 5, 261684},

{306, 0, 4, 10, 5, 354070},

{313, -1, 4, 5, 5, 261684},

{317, 0, 4, 7, 7, 261684},

{324, 0, 4, 5, 5, 222090},

{714, 0, 3, 7, 1, 248486},

{715, 0, 3, 15, 1, 496972},

{656, -2, 6, 4, 1, 248486},

{657, -1, 6, 5, 2, 248486},

{659, -2, 7, 3, 2, 248486},

};

char cmbx5_bitmap_small[717] = {

28,14,15,132,195,226,27,158,254,99,99,126,99,99,254,63,

97,192,192,192,97,62,254,49,152,108,54,27,27,248,254,98,

104,120,105,98,254,254,197,163,198,140,60,0,63,48,176,25,

252,51,24,252,247,177,152,207,230,51,27,222,246,102,102,240,

60,97,134,27,111,0,241,216,198,65,240,102,24,207,60,241,

134,24,101,159,192,241,238,57,107,45,101,172,153,187,120,227,

184,150,73,164,114,27,132,62,49,176,120,60,27,24,248,254,

99,99,126,96,96,240,62,49,176,120,60,27,216,250,15,3,

0,252,51,25,143,134,195,103,220,126,14,30,12,31,128,255,

153,24,24,24,24,126,243,176,152,76,38,19,8,248,225,176,

152,70,67,224,224,112,243,206,101,144,203,33,150,65,199,3,

142,3,24,0,247,155,15,3,129,225,59,158,243,184,142,131,

128,192,96,120,255,24,97,195,44,127,128,224,79,36,124,193,

7,145,69,23,128,116,33,23,0,24,39,162,138,39,192,116,

127,23,0,12,79,132,16,67,128,62,145,35,135,208,96,190,

193,7,18,73,46,192,216,100,151,51,3,17,17,30,194,22,

167,43,32,201,36,184,251,136,137,17,34,46,238,241,36,146,

236,116,99,23,0,249,20,81,121,14,0,116,165,39,8,224,

231,68,224,245,220,38,242,51,48,217,36,146,124,238,136,161,

65,0,238,212,165,40,204,51,0,218,136,169,128,110,72,161,

193,2,56,0,122,136,159,128,253,150,24,97,143,0,12,7,

129,96,140,67,159,239,252,62,49,178,121,60,27,24,248,24,

24,44,44,70,70,239,255,255,251,232,63,255,128,255,177,152,

204,102,51,27,222,255,97,48,24,16,97,255,243,223,67,1,

128,192,96,120,60,24,126,219,126,24,60,30,6,59,237,227,

224,96,120,126,195,195,195,231,231,231,116,99,24,197,192,101,

8,66,19,224,118,194,51,39,224,116,198,97,237,192,24,81,

39,240,129,7,0,255,161,225,205,192,58,33,232,197,192,254,

33,8,32,130,0,116,126,233,197,192,116,98,240,139,128,35,

171,199,222,174,32,48,36,18,238,42,36,225,222,251,32,6,

96,102,0,109,240,201,46,49,17,17,224,49,45,145,69,29,

128,247,4,95,241,23,112,119,162,24,254,33,119,128,11,175,

94,186,0,31,225,98,54,131,248,102,148,98,239,224,63,249,

142,44,25,48,124,193,149,142,35,255,128,2,31,25,217,188,

158,205,204,124,32,0,252,60,240,252,9,2,79,252,72,18,

4,143,252,144,36,0,96,167,201,34,80,105,130,145,36,137,

65,128,246,18,72,136,136,66,16,132,33,17,17,36,128,39,

221,242,0,8,4,2,1,0,135,252,32,16,8,4,2,0,

246,240,8,68,34,16,136,68,32,243,192,243,216,255,128,0,

0,15,248,62,46,168,180,90,42,248,254,242,73,36,147,128,

228,146,73,39,128,111,192,192,224,248,248,224,124,240,244,240,

144,109,128,180,15,196,143,248,72,18,4,131,48,14,16,254,

18,18,18,51,14,18,254,18,18,18,51,15,240,136,63,252,

34,33,17,8,136,196,96,15,240,136,191,252,34,33,17,8,

136,238,224,222,211,32,224,76,183,176,254,255,254,

};

METRIC cmbx7_metric_small[128] = {

{566, 0, 9, 9, 10, 354804},

{578, -1, 9, 13, 10, 492430},

{595, -1, 9, 12, 10, 459844},

{610, -1, 9, 10, 10, 411420},

{623, -1, 9, 9, 10, 394672},

{635, 0, 9, 14, 10, 459662},

{653, -1, 9, 11, 10, 427258},

{667, -1, 9, 12, 10, 459844},

{682, -1, 9, 11, 10, 427258},

{696, -1, 9, 12, 10, 459844},

{711, -1, 9, 11, 10, 427258},

{1107, 0, 9, 12, 10, 349433},

{1122, 0, 9, 9, 10, 333140},

{1134, 0, 9, 9, 10, 333140},

{1146, 0, 9, 14, 10, 499710},

{1164, 0, 9, 14, 10, 499710},

{836, 0, 5, 4, 6, 166570},

{839, 2, 5, 6, 9, 182863},

{1085, -2, 9, 3, 3, 296914},

{1087, -3, 9, 3, 3, 296914},

{1089, -2, 8, 4, 2, 296914},

{1090, -2, 9, 4, 3, 296914},

{1092, -1, 8, 6, 1, 296914},

{1093, -4, 9, 5, 3, 444006},

{1095, -1, -1, 6, 3, 264328},

{846, -1, 9, 7, 10, 306836},

{855, -1, 5, 11, 6, 427258},

{864, -1, 5, 12, 6, 459844},

{873, -1, 7, 6, 9, 296914},

{880, -1, 9, 14, 10, 533390},

{898, -1, 9, 16, 10, 598562},

{918, -1, 10, 12, 12, 459844},

{1098, 0, 4, 5, 1, 166570},

{936, -2, 9, 2, 10, 182226},

{1182, -1, 9, 6, 5, 306380},

{942, -1, 9, 13, 13, 492430},

{796, -1, 10, 6, 12, 296914},

{964, -1, 10, 13, 12, 492430},

{805, -1, 9, 12, 10, 459844},

{984, -1, 9, 2, 5, 166570},

{986, 0, 10, 5, 14, 231742},

{995, -1, 10, 5, 14, 231742},

{1004, -1, 10, 6, 6, 296914},

{1009, -1, 9, 12, 12, 459844},

{1027, -1, 1, 2, 5, 166570},

{1186, 0, 3, 4, 2, 199156},

{1029, -1, 1, 2, 2, 166570},

{1030, -1, 10, 6, 15, 296914},

{725, -1, 8, 6, 9, 296914},

{732, -1, 8, 6, 9, 296914},

{739, -1, 8, 6, 9, 296914},

{746, -1, 8, 6, 9, 296914},

{753, -1, 8, 6, 9, 296914},

{760, -1, 8, 6, 9, 296914},

{767, -1, 8, 6, 9, 296914},

{774, -1, 8, 7, 9, 296914},

{782, -1, 8, 6, 9, 296914},

{789, -1, 8, 6, 9, 296914},

{1042, -1, 5, 2, 6, 166570},

{1044, -1, 5, 2, 9, 166570},

{939, -2, 6, 2, 10, 182226},

{1047, -1, 6, 12, 6, 459844},

{828, -1, 6, 6, 10, 280621},

{820, -1, 9, 6, 10, 280621},

{1056, -1, 9, 12, 10, 459844},

{0, -1, 9, 11, 10, 444006},

{14, 0, 9, 11, 10, 419339},

{28, -1, 9, 11, 10, 427258},

{42, 0, 9, 12, 10, 451925},

{57, 0, 9, 10, 10, 387390},

{70, 0, 9, 10, 10, 371097},

{83, -1, 9, 12, 10, 464031},

{98, 0, 9, 14, 10, 459662},

{116, 0, 9, 6, 10, 219909},

{124, -1, 9, 7, 10, 305288},

{133, 0, 9, 13, 10, 460299},

{150, 0, 9, 9, 10, 354804},

{162, 0, 9, 17, 10, 557420},

{184, 0, 9, 14, 10, 459662},

{202, -1, 9, 11, 10, 444188},

{216, 0, 9, 11, 10, 403046},

{230, -1, 9, 11, 13, 444188},

{248, 0, 9, 13, 10, 440365},

{265, -1, 9, 7, 10, 329500},

{274, -1, 9, 10, 10, 411602},

{287, 0, 9, 13, 10, 451834},

{304, -1, 9, 11, 10, 444006},

{318, 0, 9, 18, 10, 606936},

{341, -1, 9, 11, 10, 444006},

{355, -1, 9, 12, 10, 444006},

{370, -1, 9, 8, 10, 362086},

{1071, -1, 10, 3, 15, 166989},

{1187, -2, 9, 6, 5, 306380},

{1077, 0, 10, 3, 15, 166989},

{1099, -2, 9, 4, 3, 296914},

{1101, -1, 10, 2, 2, 166570},

{1083, -1, 9, 2, 5, 166570},

{380, -1, 5, 7, 6, 288767},

{386, -1, 9, 7, 10, 329500},

{395, -1, 5, 5, 6, 264328},

{399, -1, 9, 7, 10, 329500},

{408, -1, 5, 6, 6, 271747},

{413, 0, 9, 7, 10, 182863},

{422, 0, 5, 8, 10, 296914},

{432, -1, 9, 7, 10, 329500},

{441, 0, 10, 4, 11, 166570},

{447, 2, 10, 6, 14, 182863},

{458, -1, 9, 8, 10, 313207},

{468, 0, 9, 4, 10, 166570},

{473, -1, 5, 14, 6, 492430},

{484, -1, 5, 7, 6, 329500},

{490, -1, 5, 6, 6, 296914},

{495, -1, 5, 7, 9, 329500},

{503, -1, 5, 8, 9, 313207},

{512, -1, 5, 6, 6, 245214},

{517, -1, 5, 4, 6, 235001},

{520, 0, 8, 5, 9, 231742},

{526, -1, 5, 7, 6, 329500},

{532, -1, 5, 7, 6, 313207},

{538, -1, 5, 11, 6, 427258},

{547, -1, 5, 7, 6, 313207},

{553, 0, 5, 8, 9, 313207},

{562, -1, 5, 5, 6, 264328},

{1191, 0, 3, 8, 1, 296914},

{1192, 0, 3, 18, 1, 593828},

{1102, -1, 9, 5, 3, 296914},

{1104, -2, 9, 4, 3, 296914},

{1106, -2, 10, 4, 2, 296914},

};

char cmbx7_bitmap_small[1195] = {

4,1,192,56,15,129,48,38,15,225,12,64,222,124,255,134,

24,195,24,99,248,97,140,25,131,48,255,248,31,166,13,128,

224,28,1,128,48,11,1,48,67,240,255,131,6,48,35,3,

48,51,3,48,51,2,48,111,248,255,204,51,36,201,62,12,

131,36,193,48,191,224,255,204,51,36,201,62,12,131,32,192,

48,63,0,31,163,6,96,44,2,192,12,31,192,102,6,48,

97,250,252,252,192,195,3,12,12,63,240,192,195,3,12,12,

48,51,243,240,252,195,12,48,195,12,51,240,62,24,48,96,

193,131,6,204,240,252,121,131,12,32,98,3,112,29,128,198,

6,24,48,103,231,192,252,24,12,6,3,1,128,194,97,49,

255,192,248,15,156,7,11,5,133,130,194,98,97,49,48,141,

24,70,140,33,134,124,207,192,240,124,224,66,193,9,132,35,

16,134,66,13,8,28,32,51,224,64,31,12,25,1,96,60,

7,128,240,27,6,113,195,224,255,134,12,193,152,51,6,127,

12,1,128,48,31,128,31,14,57,131,96,60,7,128,240,27,

6,127,195,228,4,128,240,12,255,129,131,12,24,96,195,248,

24,96,195,6,24,48,207,227,128,125,10,6,7,199,192,129,

131,248,255,227,24,198,49,12,3,0,192,48,12,15,192,252,

249,129,12,8,96,67,2,24,16,192,134,4,24,64,124,0,

240,236,8,194,24,67,152,50,6,64,112,14,0,128,248,243,

204,56,67,30,16,100,200,25,50,6,206,128,225,192,56,112,

12,12,1,2,0,251,198,32,108,15,0,192,28,6,192,156,

33,158,124,240,247,4,56,129,152,13,0,96,6,0,96,6,

1,240,255,198,134,140,24,24,49,97,99,255,113,177,230,205,

143,192,224,193,131,7,204,217,179,102,248,118,241,140,184,28,

24,48,103,217,179,102,204,252,123,63,240,68,224,28,108,193,

143,134,12,24,48,240,63,108,108,120,124,127,195,195,195,60,

224,193,131,7,141,155,54,109,252,102,0,14,102,102,240,12,

48,0,0,112,195,12,48,195,205,224,224,96,96,96,103,108,

120,120,108,231,230,102,102,102,111,254,241,140,102,49,152,198,

99,27,222,240,248,217,179,102,223,192,123,60,243,205,224,252,

205,155,54,111,152,48,240,58,70,198,198,70,62,6,6,15,

249,182,24,99,192,121,231,158,16,141,243,24,231,24,252,217,

179,102,199,192,230,201,145,195,130,0,231,109,73,173,53,163,

24,99,0,238,208,193,198,221,192,115,50,50,28,28,8,8,

208,240,253,172,214,252,255,152,204,38,19,1,128,192,96,48,

126,0,2,0,56,3,224,19,1,12,8,96,129,136,14,127,

247,255,192,31,131,12,96,108,147,207,60,243,201,54,6,112,

225,248,12,3,1,224,88,22,8,194,48,140,65,184,240,255,

255,224,36,67,225,240,137,1,255,255,192,255,252,192,195,3,

12,12,48,48,192,195,3,12,12,48,51,243,240,255,236,12,

192,140,16,192,24,6,9,1,64,127,252,112,237,155,15,0,

96,6,0,96,6,0,96,6,1,248,31,128,192,254,51,108,

103,140,217,177,252,6,3,240,31,128,96,198,54,102,102,102,

102,54,193,248,6,1,248,63,140,27,1,224,60,6,193,136,

37,141,113,206,56,123,60,243,207,60,243,120,51,195,12,48,

195,12,124,123,60,195,8,66,95,252,123,60,195,56,60,243,

120,24,99,150,219,241,134,124,121,196,30,76,60,226,112,57,

188,254,207,60,211,56,255,250,32,129,6,12,24,48,123,60,

254,123,60,243,120,115,44,243,205,252,246,112,17,233,103,209,

227,245,214,87,132,24,3,64,52,3,128,48,119,132,216,140,

208,199,23,222,123,48,196,16,64,0,48,192,48,192,0,16,

130,48,205,224,230,102,111,28,48,195,12,48,243,120,60,205,

155,62,204,217,179,103,204,125,216,197,255,227,12,98,251,128,

60,228,49,195,252,48,67,19,206,8,39,183,255,190,222,64,

31,252,44,48,176,68,197,19,16,127,194,49,72,197,67,11,

159,224,31,255,112,195,96,197,192,197,192,252,192,196,192,197,

96,193,112,194,31,254,0,65,252,48,198,22,195,60,35,196,

60,195,104,103,14,63,130,0,255,208,240,240,127,240,6,96,

51,1,153,255,255,255,134,96,51,31,255,255,248,204,12,192,

102,3,48,0,112,102,127,50,49,147,12,152,57,128,25,193,

153,12,200,198,76,50,32,224,245,128,8,136,140,99,24,198,

16,65,4,130,8,33,140,99,24,196,68,64,51,255,255,252,

192,6,0,96,6,0,96,6,15,255,255,240,96,6,0,96,

6,0,96,245,128,240,4,48,198,24,99,12,49,134,24,195,

12,0,240,240,240,245,128,255,255,255,0,0,0,255,255,255,

31,130,4,78,41,153,153,153,153,153,148,238,32,1,255,255,

109,182,219,109,248,253,182,219,109,183,248,107,192,217,128,111,

0,246,153,96,252,116,92,57,248,0,248,111,144,240,222,210,

22,128,255,31,227,51,51,3,48,255,195,48,51,3,48,51,

7,56,30,25,140,198,15,241,152,204,102,51,57,192,31,25,

140,198,111,241,152,204,102,51,57,192,15,240,198,99,25,140,

96,255,248,198,99,25,140,102,49,153,198,112,15,248,198,99,

25,140,102,255,248,198,99,25,140,102,49,153,239,240,207,52,

81,136,255,70,40,179,204,255,255,255,192,

};

METRIC cmex10_metric_small[128] = {

{0, -3, 0, 5, 24, 300375},

{281, -1, 0, 5, 24, 300375},

{562, -4, 0, 4, 24, 273068},

{733, 0, 0, 4, 24, 273068},

{904, -4, 0, 5, 24, 309477},

{1038, 0, 0, 5, 24, 309477},

{1172, -4, 0, 5, 24, 309477},

{1306, 0, 0, 5, 24, 309477},

{1440, -3, 0, 6, 24, 382295},

{1717, -2, 0, 7, 24, 382295},

{2034, -2, 0, 5, 25, 309477},

{2232, -2, 0, 5, 25, 309477},

{3214, -3, 0, 1, 14, 218454},

{3216, -3, 0, 5, 14, 364090},

{2430, -1, 0, 9, 24, 378653},

{2822, -1, 0, 9, 24, 378653},

{15, -3, 0, 7, 37, 391397},

{296, -1, 0, 7, 37, 391397},

{48, -4, 0, 9, 49, 482420},

{329, -1, 0, 9, 49, 482420},

{598, -5, 0, 5, 49, 345886},

{769, 0, 0, 5, 49, 345886},

{947, -5, 0, 6, 49, 382295},

{1081, 0, 0, 6, 49, 382295},

{1215, -5, 0, 6, 49, 382295},

{1349, 0, 0, 6, 49, 382295},

{1500, -2, 0, 11, 49, 491521},

{1780, -2, 0, 11, 49, 491521},

{2087, -2, 0, 11, 49, 491521},

{2285, -2, 0, 11, 49, 491521},

{2522, -1, 0, 19, 49, 684489},

{2914, -1, 0, 19, 49, 684489},

{104, -4, 0, 10, 61, 518829},

{385, -1, 0, 10, 61, 518829},

{629, -5, 0, 6, 61, 382295},

{800, 0, 0, 6, 61, 382295},

{984, -5, 0, 7, 61, 418704},

{1118, 0, 0, 7, 61, 418704},

{1252, -5, 0, 7, 61, 418704},

{1386, 0, 0, 7, 61, 418704},

{1568, -3, 0, 10, 61, 527932},

{1848, -3, 0, 10, 61, 527932},

{2155, -3, 0, 10, 61, 527932},

{2353, -3, 0, 10, 61, 527932},

{2639, -1, 0, 24, 61, 837407},

{3031, -1, 0, 24, 61, 837407},

{2457, -1, 0, 14, 37, 531572},

{2849, -1, 0, 14, 37, 531572},

{181, -6, -1, 10, 38, 573442},

{462, -1, -1, 10, 38, 573442},

{675, -7, -1, 6, 37, 436908},

{846, 0, -1, 6, 37, 436908},

{703, -7, 0, 6, 37, 436908},

{874, 0, 0, 6, 37, 436908},

{731, -7, 0, 1, 14, 436908},

{902, -5, 0, 1, 14, 436908},

{1645, -7, -2, 7, 19, 582544},

{1925, -3, -2, 7, 19, 582544},

{1662, -7, 0, 7, 19, 582544},

{1942, -3, 0, 7, 19, 582544},

{1679, -3, 0, 7, 39, 582544},

{1959, -7, 0, 7, 39, 582544},

{1714, -7, 0, 3, 8, 582544},

{3253, -6, 0, 1, 14, 436908},

{229, -6, 1, 10, 38, 573442},

{510, -1, 1, 10, 38, 573442},

{277, -6, 0, 2, 14, 573442},

{558, -9, 0, 2, 14, 573442},

{2050, -2, 0, 8, 37, 400499},

{2248, -2, 0, 8, 37, 400499},

{3812, -1, -1, 14, 21, 546135},

{3849, -1, -1, 20, 29, 728180},

{4935, -1, -1, 11, 23, 309476},

{4967, -1, -1, 18, 46, 364090},

{3922, -1, -1, 20, 21, 728180},

{3975, -1, -1, 28, 29, 990325},

{4077, -1, -1, 20, 21, 728180},

{4130, -1, -1, 28, 29, 990325},

{4232, -1, -1, 20, 21, 728180},

{4285, -1, -1, 28, 29, 990325},

{4387, -1, -1, 19, 21, 691771},

{4535, -1, -1, 17, 21, 618953},

{4799, -1, -1, 11, 23, 309476},

{5071, -1, -1, 14, 21, 546135},

{5181, -1, -1, 14, 21, 546135},

{5291, -1, -1, 14, 21, 546135},

{5401, -1, -1, 14, 21, 546135},

{5511, -1, -1, 14, 21, 546135},

{4437, -1, -1, 27, 29, 946634},

{4580, -1, -1, 24, 29, 837407},

{4831, -1, -1, 18, 46, 364090},

{5108, -1, -1, 20, 29, 728180},

{5218, -1, -1, 20, 29, 728180},

{5328, -1, -1, 20, 29, 728180},

{5438, -1, -1, 20, 29, 728180},

{5548, -1, -1, 20, 29, 728180},

{4667, -1, -1, 17, 21, 618953},

{4712, -1, -1, 24, 29, 837407},

{5621, 0, 14, 11, 4, 364090},

{5627, 0, 15, 20, 4, 655362},

{5637, 0, 15, 29, 4, 946634},

{5652, 0, 14, 11, 3, 364090},

{5657, 0, 15, 20, 3, 655362},

{5665, 0, 15, 29, 3, 946634},

{574, -4, 0, 5, 37, 309477},

{745, 0, 0, 5, 37, 309477},

{919, -4, 0, 6, 37, 345886},

{1053, 0, 0, 6, 37, 345886},

{1187, -4, 0, 6, 37, 345886},

{1321, 0, 0, 6, 37, 345886},

{1458, -2, 0, 9, 37, 436908},

{1738, -2, 0, 9, 37, 436908},

{3296, -1, 0, 20, 25, 655362},

{3359, -1, 0, 20, 38, 655362},

{3454, -1, 0, 20, 50, 655362},

{3579, -1, 0, 20, 62, 655362},

{3734, -2, 0, 13, 38, 691771},

{3796, -14, 0, 1, 14, 691771},

{3798, -14, 0, 8, 14, 691771},

{3289, -6, 0, 4, 14, 509726},

{3225, -2, -1, 9, 12, 436908},

{3239, -2, -1, 9, 12, 436908},

{1994, 1, 2, 11, 7, 294915},

{2004, 1, 2, 11, 7, 294915},

{2014, 1, 6, 11, 7, 294915},

{2024, 1, 6, 11, 7, 294915},

{3255, -2, -2, 12, 11, 509726},

{3272, -2, -1, 12, 11, 509726},

};

char cmex10_bitmap_small[5676] = {

8,136,68,33,16,132,33,8,66,16,130,16,130,16,65,2,

8,32,65,2,8,16,64,129,2,8,16,32,64,129,2,4,

8,16,32,64,128,129,2,4,4,8,8,16,16,32,32,32,

0,128,128,128,192,64,96,32,48,16,24,12,4,6,3,1,

128,192,64,96,48,24,12,6,3,1,128,192,96,48,24,12,

6,3,1,128,64,48,24,12,6,1,0,192,96,16,12,2,

1,128,64,48,8,2,0,128,0,64,32,24,4,2,1,128,

64,48,8,6,1,128,64,48,12,3,0,128,96,24,6,1,

128,96,48,12,3,0,192,48,12,3,0,192,48,12,3,0,

192,48,12,3,0,192,48,12,3,0,96,24,6,1,128,96,

8,3,0,192,48,4,1,128,96,8,3,0,64,24,2,0,

64,24,2,0,64,0,64,32,8,4,3,0,128,96,16,12,

3,0,128,96,24,4,3,0,192,48,8,6,1,128,96,24,

6,1,128,64,48,12,3,0,192,48,12,3,0,192,48,12,

3,0,192,48,0,192,48,12,3,0,192,48,12,3,0,192,

48,12,3,0,192,16,6,1,128,96,24,6,1,128,32,12,

3,0,192,16,6,1,128,32,12,3,0,64,24,2,0,192,

16,2,0,128,16,255,255,255,240,130,8,65,8,65,8,66,

16,132,33,8,132,34,17,16,128,128,129,1,2,2,4,4,

8,16,32,32,64,129,2,4,8,16,32,64,129,2,8,16,

32,65,2,8,16,64,130,8,0,128,32,8,6,1,0,192,

32,24,4,3,1,128,64,48,24,12,6,1,0,192,96,48,

24,12,6,3,1,128,192,96,48,24,12,6,3,1,1,128,

192,96,48,16,24,12,4,6,2,3,1,1,128,128,128,128,

0,128,16,6,0,128,16,6,0,128,48,4,1,128,96,8,

3,0,192,48,4,1,128,96,24,6,1,128,48,12,3,0,

192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,

48,12,3,1,128,96,24,6,1,128,64,48,12,3,0,128,

96,24,4,3,0,128,96,16,8,6,1,0,128,0,128,16,

4,0,128,48,4,1,128,32,12,3,0,64,24,6,0,128,

48,12,3,0,64,24,6,1,128,96,24,6,0,128,48,12,

3,0,192,48,12,3,0,192,48,12,3,0,192,48,0,192,

48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,32,

24,6,1,128,96,24,6,1,0,192,48,12,2,1,128,96,

16,12,3,0,128,96,16,12,2,1,0,64,32,0,255,255,

255,240,248,136,136,136,136,136,136,136,136,136,136,143,252,33,

8,66,16,132,33,8,66,16,132,33,8,66,16,132,33,8,

66,16,132,33,15,128,252,33,8,66,16,132,33,8,66,16,

132,33,8,66,16,132,33,8,66,16,132,33,8,66,16,132,

33,8,66,16,248,254,8,32,130,8,32,130,8,32,130,8,

32,130,8,32,130,8,32,130,8,32,130,8,32,130,8,32,

130,8,32,130,8,32,130,8,32,130,8,32,130,8,32,130,

8,32,252,254,8,32,130,8,32,130,8,32,130,8,32,130,

8,32,130,8,32,130,8,32,130,8,32,130,8,32,128,130,

8,32,130,8,32,130,8,32,130,8,32,130,8,32,130,8,

32,130,8,32,130,8,32,130,8,32,252,255,252,241,17,17,

17,17,17,17,17,17,17,17,31,248,66,16,132,33,8,66,

16,132,33,8,66,16,132,33,8,66,16,132,33,8,66,31,

128,248,66,16,132,33,8,66,16,132,33,8,66,16,132,33,

8,66,16,132,33,8,66,16,132,33,8,66,16,132,33,248,

252,16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,

16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,16,

65,4,16,65,4,16,65,4,16,65,4,16,65,252,252,16,

65,4,16,65,4,16,65,4,16,65,4,16,65,4,16,65,

4,16,65,4,16,65,4,16,65,4,4,16,65,4,16,65,

4,16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,

16,65,4,16,65,252,255,252,132,33,8,66,16,132,33,8,

66,16,132,33,8,66,31,130,8,32,130,8,32,130,8,32,

130,8,32,130,8,32,130,8,32,130,8,32,130,8,32,130,

8,32,252,130,8,32,130,8,32,130,8,32,130,8,32,130,

8,32,130,8,32,130,8,32,130,8,32,130,8,32,130,8,

32,130,8,32,130,8,32,252,129,2,4,8,16,32,64,129,

2,4,8,16,32,64,129,2,4,8,16,32,64,129,2,4,

8,16,32,64,129,2,4,8,16,32,64,129,2,4,8,16,

32,64,129,2,4,8,16,32,64,129,2,4,15,224,8,66,

16,132,33,8,66,16,132,33,8,66,16,132,63,4,16,65,

4,16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,

16,65,4,16,65,4,16,65,252,4,16,65,4,16,65,4,

16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,16,

65,4,16,65,4,16,65,4,16,65,4,16,65,252,2,4,

8,16,32,64,129,2,4,8,16,32,64,129,2,4,8,16,

32,64,129,2,4,8,16,32,64,129,2,4,8,16,32,64,

129,2,4,8,16,32,64,129,2,4,8,16,32,64,129,2,

4,8,31,224,252,33,8,66,16,132,33,8,66,16,132,33,

8,66,16,254,8,32,130,8,32,130,8,32,130,8,32,130,

8,32,130,8,32,130,8,32,130,8,32,130,8,32,128,254,

8,32,130,8,32,130,8,32,130,8,32,130,8,32,130,8,

32,130,8,32,130,8,32,130,8,32,130,8,32,130,8,32,

130,8,32,128,255,2,4,8,16,32,64,129,2,4,8,16,

32,64,129,2,4,8,16,32,64,129,2,4,8,16,32,64,

129,2,4,8,16,32,64,129,2,4,8,16,32,64,129,2,

4,8,16,32,64,129,2,4,8,0,248,66,16,132,33,8,

66,16,132,33,8,66,16,132,33,252,16,65,4,16,65,4,

16,65,4,16,65,4,16,65,4,16,65,4,16,65,4,16,

65,4,16,65,4,252,16,65,4,16,65,4,16,65,4,16,

65,4,16,65,4,16,65,4,16,65,4,16,65,4,16,65,

4,16,65,4,16,65,4,16,65,4,254,4,8,16,32,64,

129,2,4,8,16,32,64,129,2,4,8,16,32,64,129,2,

4,8,16,32,64,129,2,4,8,16,32,64,129,2,4,8,

16,32,64,129,2,4,8,16,32,64,129,2,4,8,16,32,

12,66,8,32,130,8,32,134,32,129,130,8,32,130,8,32,

129,3,1,129,1,1,0,128,64,32,16,8,4,2,1,0,

128,64,32,16,16,16,48,4,1,0,64,32,16,8,4,2,

1,0,128,64,32,16,8,4,1,0,64,24,0,96,24,4,

1,128,96,12,1,128,48,6,0,192,24,3,0,96,12,1,

128,48,6,0,192,24,3,0,96,24,6,1,128,192,6,0,

96,6,0,96,12,1,128,48,6,0,192,24,3,0,96,12,

1,128,48,6,0,192,24,3,0,96,6,0,64,6,0,96,

0,64,96,48,24,6,3,0,192,48,12,3,0,192,48,12,

3,0,192,48,12,3,0,192,48,12,3,0,192,48,12,3,

1,128,96,48,24,8,1,128,48,6,1,128,48,12,3,0,

192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,

48,12,3,0,192,48,6,1,128,48,6,0,64,2,24,97,

135,14,56,112,225,195,135,14,28,56,112,225,195,128,225,195,

135,14,28,56,112,225,195,135,14,14,28,24,24,24,8,14,

28,56,112,225,195,135,14,28,56,112,225,135,12,24,97,132,

6,6,6,12,28,24,56,112,225,195,135,14,28,56,112,225,

195,128,255,255,255,192,64,64,129,2,4,8,16,32,96,48,

99,4,8,16,32,64,129,2,8,96,192,16,4,1,0,128,

64,32,16,8,4,2,1,0,128,64,32,16,4,1,0,96,

64,64,64,32,16,8,4,2,1,0,128,64,32,16,8,4,

4,4,12,0,192,6,0,96,6,0,96,12,1,128,48,6,

0,192,24,3,0,96,12,1,128,48,6,0,192,24,3,0,

96,6,0,64,6,0,96,24,4,1,128,96,12,1,128,48,

6,0,192,24,3,0,96,12,1,128,48,6,0,192,24,3,

0,96,24,6,1,128,192,0,128,24,3,0,96,24,3,0,

192,48,12,3,0,192,48,12,3,0,192,48,12,3,0,192,

48,12,3,0,192,48,12,3,0,96,24,3,0,96,4,6,

3,1,128,96,48,12,3,0,192,48,12,3,0,192,48,12,

3,0,192,48,12,3,0,192,48,12,3,0,192,48,24,6,

3,1,128,128,0,128,192,192,193,195,131,135,14,28,56,112,

225,195,135,14,28,56,14,28,56,112,225,195,135,14,28,56,

112,227,135,12,48,194,0,225,195,135,14,28,56,112,225,195,

135,14,12,28,24,48,48,48,16,195,12,24,112,195,135,14,

28,56,112,225,195,135,14,28,56,0,1,224,252,63,142,3,

0,128,32,0,240,31,131,248,3,128,24,0,128,8,128,8,

0,192,14,0,254,15,192,120,0,32,8,6,3,143,225,248,

60,0,8,68,33,16,132,66,17,8,65,8,65,8,65,8,

65,8,1,1,2,2,4,4,4,8,8,16,16,16,32,32,

32,64,64,128,128,128,64,64,32,32,32,16,16,16,8,8,

4,4,4,2,2,1,1,0,32,4,1,0,32,8,1,0,

32,8,1,0,64,8,2,0,64,8,2,0,64,16,2,0,

128,16,2,0,128,16,4,0,128,16,1,0,32,2,0,64,

8,0,128,16,1,0,32,2,0,64,8,0,128,16,1,0,

32,2,0,64,8,0,128,16,1,0,32,0,64,16,8,2,

0,128,32,16,4,1,0,128,32,8,4,1,0,64,32,8,

2,0,128,64,16,4,2,0,128,32,16,4,1,0,64,32,

8,2,0,64,16,4,1,0,32,8,2,0,64,16,4,0,

128,32,8,2,0,64,16,4,0,128,32,8,1,0,64,16,

2,0,128,32,8,1,0,64,132,16,132,16,132,16,132,16,

132,66,17,8,68,33,16,128,128,128,64,64,32,32,32,16,

16,8,8,8,4,4,4,2,2,1,1,1,2,2,4,4,

4,8,8,8,16,16,32,32,32,64,64,128,128,128,16,1,

0,32,2,0,64,8,0,128,16,1,0,32,2,0,64,8,

0,128,16,1,0,32,2,0,64,8,0,128,16,1,0,32,

4,1,0,32,8,1,0,32,8,1,0,64,8,2,0,64,

8,2,0,64,16,2,0,128,16,2,0,128,16,4,0,128,

0,128,32,4,1,0,64,16,2,0,128,32,4,1,0,64,

8,2,0,128,16,4,1,0,64,8,2,0,128,16,4,1,

0,32,8,2,0,128,16,4,1,0,128,32,8,2,1,0,

64,16,8,2,0,128,64,16,4,1,0,128,32,8,4,1,

0,64,32,8,2,1,0,64,16,4,2,0,128,0,0,128,

64,64,32,16,16,8,4,4,2,1,1,0,128,128,64,32,

32,16,8,8,4,2,2,1,0,0,4,0,16,0,128,2,

0,8,0,64,1,0,8,0,32,0,128,4,0,16,0,64,

2,0,8,0,32,1,0,4,0,32,0,128,2,0,16,0,

64,1,0,8,0,32,0,128,4,0,16,0,64,2,0,8,

0,64,1,0,4,0,32,0,128,0,0,0,32,0,4,0,

1,0,0,32,0,8,0,1,0,0,32,0,8,0,1,0,

0,32,0,8,0,1,0,0,64,0,8,0,1,0,0,64,

0,8,0,1,0,0,64,0,8,0,2,0,0,64,0,8,

0,2,0,0,64,0,8,0,2,0,0,64,0,8,0,2,

0,0,64,0,16,0,2,0,0,64,0,16,0,2,0,0,

64,0,16,0,2,0,0,128,0,16,0,2,0,0,128,0,

16,0,2,0,0,128,0,16,0,4,0,0,128,0,0,0,

0,1,0,0,3,0,0,2,0,0,2,0,0,6,0,0,

4,0,0,4,0,0,12,0,0,8,0,0,24,0,0,16,

0,0,16,0,0,48,0,0,32,0,0,96,0,0,64,0,

0,64,0,0,192,0,0,128,0,0,128,0,1,128,0,1,

0,0,3,0,0,2,0,0,2,0,0,6,0,0,4,0,

0,12,0,0,8,0,0,8,0,0,24,0,0,16,0,0,

16,0,0,48,0,0,32,0,0,96,0,0,64,0,0,64,

0,0,192,0,0,128,0,1,128,0,1,0,0,1,0,0,

3,0,0,2,0,0,2,0,0,6,0,0,4,0,0,12,

0,0,8,0,0,8,0,0,24,0,0,16,0,0,48,0,

0,32,0,0,32,0,0,96,0,0,64,0,0,64,0,0,

192,0,0,128,0,0,128,64,16,8,4,1,0,128,64,16,

8,4,1,0,128,32,16,8,2,1,0,128,32,16,8,2,

1,128,2,0,4,0,16,0,64,0,128,2,0,4,0,16,

0,64,0,128,2,0,8,0,16,0,64,1,0,2,0,8,

0,32,0,64,1,0,2,0,8,0,32,0,64,1,0,4,

0,8,0,32,0,128,1,0,4,0,8,0,32,0,128,1,

0,4,128,0,16,0,1,0,0,32,0,2,0,0,64,0,

8,0,0,128,0,16,0,2,0,0,32,0,4,0,0,64,

0,8,0,1,0,0,16,0,2,0,0,64,0,4,0,0,

128,0,8,0,1,0,0,32,0,2,0,0,64,0,8,0,

0,128,0,16,0,2,0,0,32,0,4,0,0,64,0,8,

0,1,0,0,16,0,2,0,0,64,0,4,0,0,128,0,

8,0,1,0,0,32,0,2,0,0,64,0,8,0,0,128,

0,16,0,1,0,0,32,128,0,0,192,0,0,64,0,0,

64,0,0,96,0,0,32,0,0,32,0,0,48,0,0,16,

0,0,24,0,0,8,0,0,8,0,0,12,0,0,4,0,

0,6,0,0,2,0,0,2,0,0,3,0,0,1,0,0,

1,0,0,1,128,0,0,128,0,0,192,0,0,64,0,0,

64,0,0,96,0,0,32,0,0,48,0,0,16,0,0,16,

0,0,24,0,0,8,0,0,8,0,0,12,0,0,4,0,

0,6,0,0,2,0,0,2,0,0,3,0,0,1,0,0,

1,128,0,0,128,0,0,128,0,0,192,0,0,64,0,0,

64,0,0,96,0,0,32,0,0,48,0,0,16,0,0,16,

0,0,24,0,0,8,0,0,12,0,0,4,0,0,4,0,

0,6,0,0,2,0,0,2,0,0,3,0,0,1,255,252,

140,99,24,198,49,140,99,24,196,8,14,15,153,48,128,64,

32,16,8,4,2,1,0,8,4,2,1,0,128,64,32,16,

201,159,7,1,0,255,252,6,0,96,9,1,152,105,104,145,

9,0,144,9,0,144,9,0,9,0,144,9,0,144,9,8,

145,105,97,152,9,0,96,6,0,153,153,153,153,153,153,153,

0,0,16,0,1,0,0,32,0,2,0,0,64,0,4,0,

0,128,0,8,0,1,0,0,16,0,2,0,0,32,0,4,

6,0,64,160,8,1,0,128,16,16,0,129,0,8,32,0,

66,0,4,64,0,68,0,2,128,0,40,0,1,0,0,0,

0,16,0,1,0,0,32,0,2,0,0,32,0,4,0,0,

64,0,4,0,0,128,0,8,0,0,128,0,16,0,1,0,

0,16,0,2,0,0,32,0,2,0,0,64,0,4,2,0,

64,96,4,14,0,128,16,8,1,0,128,16,16,1,1,0,

8,16,0,130,0,8,32,0,66,0,4,64,0,68,0,4,

64,0,40,0,2,128,0,40,0,1,0,0,16,0,0,0,

16,0,1,0,0,16,0,2,0,0,32,0,2,0,0,32,

0,4,0,0,64,0,4,0,0,64,0,8,0,0,128,0,

8,0,0,128,0,16,0,1,0,0,16,0,1,0,0,32,

0,2,0,0,32,0,2,0,0,64,0,4,2,0,64,96,

4,6,0,128,160,8,2,0,128,16,8,1,1,0,16,16,

1,1,0,8,16,0,130,0,8,32,0,130,0,8,32,0,

68,0,4,64,0,68,0,4,64,0,40,0,2,128,0,40,

0,2,128,0,48,0,1,0,0,16,0,0,0,16,0,1,

0,0,16,0,2,0,0,32,0,2,0,0,32,0,2,0,

0,64,0,4,0,0,64,0,4,0,0,64,0,8,0,0,

128,0,8,0,0,128,0,8,0,1,0,0,16,0,1,0,

0,16,0,1,0,0,32,0,2,0,0,32,0,2,0,0,

32,0,4,0,0,64,0,4,2,0,64,32,4,6,0,64,

96,8,10,0,128,32,8,1,0,128,16,8,1,1,0,16,

16,1,1,0,8,16,0,129,0,8,32,0,130,0,8,32,

0,130,0,4,32,0,68,0,4,64,0,68,0,4,64,0,

68,0,2,128,0,40,0,2,128,0,40,0,2,128,0,16,

0,1,0,0,16,0,0,8,0,64,2,0,16,0,160,5,

128,60,1,160,8,128,68,2,32,16,128,132,4,32,32,129,

4,8,32,64,130,4,16,32,128,132,4,32,33,0,136,4,

64,34,0,144,4,128,36,0,160,5,0,40,0,192,6,0,

48,0,128,4,255,252,255,128,128,128,128,128,128,128,128,128,

128,128,128,128,128,6,0,24,0,96,1,128,6,0,24,0,

96,1,128,6,0,24,0,96,1,128,6,0,24,0,96,1,

128,6,0,24,0,96,1,255,252,128,0,24,0,1,128,0,

24,0,1,128,0,24,0,1,128,0,24,0,1,128,0,24,

0,1,128,0,24,0,1,128,0,24,0,1,128,0,24,0,

1,128,0,24,0,1,128,0,24,0,1,128,0,24,0,1,

128,0,24,0,1,128,0,24,0,1,128,0,24,0,1,255,

255,240,1,248,0,96,96,8,1,1,0,8,32,0,68,0,

2,64,0,40,0,1,128,0,24,6,1,128,96,24,6,1,

128,0,24,0,1,64,0,36,0,2,32,0,65,0,8,8,

1,0,96,96,1,248,0,0,63,192,0,12,3,0,3,0,

12,0,64,0,32,8,0,1,1,0,0,8,32,0,0,66,

0,0,4,64,0,0,36,0,0,2,64,0,0,40,0,0,

1,128,0,0,24,0,96,1,128,15,0,24,0,96,1,128,

0,0,24,0,0,1,64,0,0,36,0,0,2,64,0,0,

34,0,0,4,32,0,0,65,0,0,8,8,0,1,0,64,

0,32,3,0,12,0,12,3,0,0,63,192,0,1,248,0,

98,96,8,33,1,2,8,32,32,68,2,2,64,32,40,2,

1,128,32,24,2,1,255,255,248,2,1,128,32,24,2,1,

64,32,36,2,2,32,32,65,2,8,8,33,0,98,96,1,

248,0,0,63,192,0,12,35,0,3,2,12,0,64,32,32,

8,2,1,1,0,32,8,32,2,0,66,0,32,4,64,2,

0,36,0,32,2,64,2,0,40,0,32,1,128,2,0,24,

0,32,1,255,255,255,248,0,32,1,128,2,0,24,0,32,

1,64,2,0,36,0,32,2,64,2,0,34,0,32,4,32,

2,0,65,0,32,8,8,2,1,0,64,32,32,3,2,12,

0,12,35,0,0,63,192,0,1,248,0,96,96,8,1,1,

0,8,40,1,68,64,34,66,4,40,16,129,128,144,24,6,

1,128,64,24,6,1,128,144,24,16,129,66,4,36,64,34,

40,1,65,0,8,8,1,0,96,96,1,248,0,0,63,192,

0,12,3,0,3,0,12,0,64,0,32,8,0,1,1,64,

0,40,34,0,4,66,16,0,132,64,128,16,36,4,2,2,

64,32,64,40,1,8,1,128,9,0,24,0,96,1,128,4,

0,24,0,96,1,128,9,0,24,1,8,1,64,32,64,36,

4,2,2,64,128,16,34,16,0,132,34,0,4,65,64,0,

40,8,0,1,0,64,0,32,3,0,12,0,12,3,0,0,

63,192,0,255,255,152,0,121,0,3,16,0,33,0,2,32,

0,2,0,0,32,0,6,0,0,64,0,4,0,1,0,0,

32,0,8,0,2,0,0,64,0,16,0,36,0,8,128,3,

32,1,239,255,248,255,255,255,30,0,1,241,224,0,14,28,

0,0,97,192,0,4,60,0,0,67,192,0,0,56,0,0,

7,128,0,0,120,0,0,7,0,0,0,112,0,0,15,0,

0,0,240,0,0,14,0,0,1,128,0,0,48,0,0,12,

0,0,3,0,0,0,64,0,0,16,0,0,6,0,0,1,

128,0,8,96,0,2,8,0,0,194,0,0,48,192,0,62,

63,255,255,207,255,255,240,255,255,152,3,12,1,134,0,195,

0,97,128,48,192,24,96,12,48,6,24,3,12,1,134,0,

195,0,97,128,48,192,24,96,12,48,6,24,3,12,1,134,

0,207,193,248,255,255,255,60,0,60,28,0,56,28,0,56,

28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,28,

0,56,28,0,56,28,0,56,28,0,56,28,0,56,28,0,

56,28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,

28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,28,

0,56,28,0,56,62,0,124,255,129,255,252,31,152,3,12,

1,134,0,195,0,97,128,48,192,24,96,12,48,6,24,3,

12,1,134,0,195,0,97,128,48,192,24,96,12,48,6,24,

3,12,1,134,0,207,255,248,255,129,255,62,0,124,28,0,

56,28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,

28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,28,

0,56,28,0,56,28,0,56,28,0,56,28,0,56,28,0,

56,28,0,56,28,0,56,28,0,56,28,0,56,28,0,56,

28,0,56,28,0,56,28,0,56,60,0,60,255,255,255,1,

192,68,8,1,0,32,4,0,128,32,4,0,128,16,2,0,

64,8,1,0,32,8,1,0,32,4,0,129,16,28,0,0,

1,128,0,208,0,44,0,27,0,6,0,1,128,0,64,0,

48,0,12,0,3,0,0,128,0,96,0,24,0,6,0,1,

128,0,64,0,48,0,12,0,3,0,0,192,0,48,0,24,

0,6,0,1,128,0,96,0,48,0,12,0,3,0,0,192,

0,48,0,8,0,6,0,1,128,0,96,0,24,0,4,0,

3,0,0,192,0,48,0,8,0,6,0,1,128,12,64,3,

48,0,136,0,28,0,0,1,192,68,8,1,0,32,4,0,

128,32,14,2,160,146,18,66,72,42,3,128,32,8,1,0,

32,4,0,129,16,28,0,0,1,128,0,208,0,44,0,27,

0,6,0,1,128,0,64,0,48,0,12,0,3,0,0,128,

0,96,0,24,0,6,0,1,128,0,64,0,48,0,62,0,

19,64,8,200,4,49,1,24,64,70,16,17,132,4,97,0,

176,128,28,64,3,224,0,192,0,48,0,8,0,6,0,1,

128,0,96,0,24,0,4,0,3,0,0,192,0,48,0,8,

0,6,0,1,128,12,64,3,48,0,136,0,28,0,0,128,

6,0,24,0,96,1,128,6,0,24,0,96,1,128,6,0,

24,0,96,1,128,6,0,24,0,96,1,64,9,0,34,1,

6,24,7,128,128,0,24,0,1,128,0,24,0,1,128,0,

24,0,1,128,0,24,0,1,128,0,24,0,1,128,0,24,

0,1,128,0,24,0,1,128,0,24,0,1,128,0,24,0,

1,128,0,24,0,1,128,0,24,0,1,64,0,36,0,2,

32,0,65,0,8,8,1,0,96,96,1,248,0,7,128,97,

130,1,16,2,64,10,0,24,0,96,1,128,6,0,24,0,

96,1,128,6,0,24,0,96,1,128,6,0,24,0,96,1,

128,4,1,248,0,96,96,8,1,1,0,8,32,0,68,0,

2,64,0,40,0,1,128,0,24,0,1,128,0,24,0,1,

128,0,24,0,1,128,0,24,0,1,128,0,24,0,1,128,

0,24,0,1,128,0,24,0,1,128,0,24,0,1,128,0,

24,0,1,128,0,24,0,1,128,0,16,128,6,0,24,0,

96,1,128,6,4,24,16,96,65,129,6,255,216,16,96,65,

129,6,4,24,0,96,1,64,9,0,34,1,6,24,7,128,

128,0,24,0,1,128,0,24,0,1,128,0,24,0,1,128,

32,24,2,1,128,32,24,2,1,128,32,24,2,1,128,32,

27,255,253,128,32,24,2,1,128,32,24,2,1,128,32,24,

2,1,128,32,24,0,1,64,0,36,0,2,32,0,65,0,

8,8,1,0,96,96,1,248,0,1,0,12,0,48,0,192,

4,128,18,0,72,2,16,8,64,33,1,2,4,8,16,32,

128,66,1,8,4,64,9,0,36,0,160,1,128,4,0,32,

0,6,0,0,96,0,9,0,0,144,0,9,0,1,8,0,

16,128,1,8,0,32,64,2,4,0,32,64,4,2,0,64,

32,4,2,0,128,16,8,1,0,128,16,16,0,129,0,8,

16,0,130,0,4,32,0,66,0,4,64,0,36,0,2,64,

0,40,0,1,128,0,16,128,6,0,20,0,144,2,64,8,

128,66,1,8,4,16,32,64,129,2,2,16,8,64,33,0,

72,1,32,4,128,12,0,48,0,192,1,0,128,0,24,0,

1,64,0,36,0,2,64,0,34,0,4,32,0,66,0,4,

16,0,129,0,8,16,0,128,128,16,8,1,0,128,16,4,

2,0,64,32,4,2,0,32,64,2,4,0,32,64,1,8,

0,16,128,1,8,0,9,0,0,144,0,9,0,0,96,0,

6,0,0,32,0,14,7,115,131,192,16,0,240,0,112,224,

56,1,204,0,3,0,15,128,0,7,131,192,7,192,1,241,

192,0,0,112,56,45,218,14,0,31,128,23,31,142,128,31,

128,7,248,0,25,224,254,15,48,0,63,192,

};

METRIC cmmi10_metric_small[128] = {

{992, 0, 13, 13, 14, 403229},

{1015, -1, 13, 14, 14, 546135},

{1040, -1, 13, 13, 14, 499893},

{1063, -1, 13, 12, 14, 455112},

{1084, -1, 13, 14, 14, 486515},

{1109, 0, 13, 19, 14, 544769},

{1143, -2, 13, 13, 14, 511091},

{1166, -1, 13, 13, 14, 382293},

{1189, -1, 13, 12, 14, 436907},

{1210, -1, 13, 13, 14, 401226},

{1233, -2, 13, 13, 14, 506199},

{1256, -1, 8, 11, 9, 419235},

{1269, 0, 13, 11, 18, 370689},

{1294, -1, 8, 10, 13, 339300},

{1311, -1, 13, 7, 14, 291271},

{1324, -1, 8, 6, 9, 266013},

{1331, -1, 13, 8, 18, 286721},

{1349, -1, 8, 8, 13, 325407},

{1362, -1, 13, 8, 14, 307655},

{1376, -1, 8, 5, 9, 231956},

{1382, -1, 8, 9, 9, 377592},

{1393, -1, 13, 9, 14, 382295},

{1409, -1, 8, 10, 13, 394887},

{1426, -1, 8, 9, 9, 323737},

{1437, -1, 13, 7, 18, 286721},

{1453, -1, 8, 10, 9, 373573},

{1465, -1, 8, 9, 13, 338831},

{1480, -1, 8, 10, 9, 374482},

{1492, -1, 8, 8, 9, 286494},

{1501, -1, 8, 8, 9, 354078},

{1510, -1, 13, 10, 18, 390486},

{1533, -1, 8, 11, 13, 410054},

{1551, -1, 13, 11, 18, 426896},

{1576, -1, 8, 11, 9, 407932},

{1589, -1, 8, 7, 9, 305606},

{1597, -1, 13, 8, 14, 387606},

{1611, -1, 8, 15, 9, 542723},

{1628, -2, 8, 8, 13, 338831},

{1641, 0, 8, 7, 11, 237796},

{1651, -1, 8, 10, 13, 428715},

{1848, -1, 8, 18, 4, 655362},

{1857, -1, 5, 18, 4, 655362},

{1866, -1, 8, 18, 4, 655362},

{1875, -1, 5, 18, 4, 655362},

{1884, -1, 10, 3, 6, 182045},

{1887, -1, 10, 3, 6, 182045},

{1890, -1, 9, 7, 9, 327681},

{1898, -1, 9, 7, 9, 327681},

{1738, -1, 8, 7, 9, 327681},

{1746, -1, 8, 7, 9, 327681},

{1754, -1, 8, 7, 9, 327681},

{1762, -1, 8, 7, 13, 327681},

{1774, -1, 8, 7, 13, 327681},

{1786, -1, 8, 7, 13, 327681},

{1798, -1, 12, 7, 13, 327681},

{1810, -1, 9, 8, 14, 327681},

{1824, -1, 12, 7, 13, 327681},

{1836, -1, 8, 7, 13, 327681},

{1906, -2, 1, 2, 2, 182045},

{1907, -2, 1, 2, 6, 182045},

{1909, -2, 11, 11, 13, 509726},

{1927, -1, 15, 7, 21, 327681},

{1946, -2, 11, 11, 13, 509726},

{1964, 0, 6, 10, 6, 327681},

{1668, -1, 13, 10, 14, 347933},

{0, -1, 13, 14, 14, 491521},

{25, 0, 13, 15, 14, 497097},

{52, -1, 13, 13, 14, 468400},

{75, 0, 13, 16, 14, 542584},

{103, 0, 13, 14, 14, 483784},

{128, 0, 13, 14, 14, 421434},

{153, -1, 13, 13, 14, 515276},

{176, 0, 13, 19, 14, 544769},

{210, 0, 13, 10, 14, 288086},

{228, -2, 13, 11, 14, 363406},

{248, 0, 13, 18, 14, 556602},

{280, 0, 13, 12, 14, 446010},

{301, 0, 13, 22, 14, 635791},

{340, 0, 13, 19, 14, 526564},

{374, -1, 13, 13, 14, 499893},

{397, 0, 13, 15, 14, 420750},

{424, -1, 13, 13, 18, 518098},

{454, 0, 13, 14, 14, 497608},

{479, -1, 13, 11, 14, 401864},

{499, 0, 13, 13, 14, 382977},

{522, 0, 13, 16, 14, 447465},

{550, -1, 13, 14, 14, 382294},

{575, -1, 13, 19, 14, 618952},

{609, 0, 13, 17, 14, 542949},

{639, 0, 13, 15, 14, 380474},

{666, -1, 13, 13, 14, 447375},

{1972, -1, 15, 5, 16, 254863},

{1982, -1, 13, 5, 18, 254863},

{1994, -1, 13, 5, 18, 254863},

{2006, -1, 7, 18, 5, 655362},

{2018, -1, 7, 18, 5, 655362},

{1686, 0, 13, 7, 14, 273069},

{689, -1, 8, 8, 9, 346417},

{698, -1, 13, 6, 14, 281259},

{709, -1, 8, 7, 9, 283611},

{717, -1, 13, 8, 14, 341107},

{731, -1, 8, 7, 9, 305153},

{739, -1, 13, 9, 18, 320855},

{760, -1, 8, 8, 13, 312586},

{773, -1, 13, 9, 14, 377592},

{789, -1, 14, 4, 15, 225780},

{797, 1, 14, 9, 19, 269882},

{819, -1, 13, 8, 14, 341183},

{833, -1, 13, 3, 14, 195546},

{839, -1, 8, 15, 9, 575415},

{856, -1, 8, 10, 9, 393370},

{868, -1, 8, 7, 9, 317668},

{876, 1, 8, 9, 13, 329729},

{891, -1, 8, 7, 13, 292562},

{903, -1, 8, 6, 9, 295672},

{910, -2, 8, 6, 9, 307200},

{917, 0, 12, 5, 13, 236659},

{926, -1, 8, 9, 9, 375166},

{937, -1, 8, 7, 9, 317669},

{945, -1, 8, 12, 9, 469184},

{959, 0, 8, 10, 9, 374557},

{971, -1, 8, 8, 13, 321311},

{984, -1, 8, 7, 9, 304775},

{1699, -1, 8, 4, 9, 211325},

{1704, 1, 8, 7, 13, 251678},

{1716, -2, 8, 10, 13, 417110},

{1733, -4, 13, 7, 3, 327681},

{1736, -5, 12, 8, 2, 182045},

};

char cmmi10_bitmap_small[2030] = {

0,96,1,128,14,0,88,1,96,9,128,38,1,24,4,96,

63,129,6,4,28,32,51,227,240,31,248,12,24,24,24,48,

48,192,193,131,131,252,6,12,24,12,48,24,96,48,192,195,

3,31,252,0,3,200,97,196,6,64,54,1,96,3,0,24,

0,128,4,1,48,9,128,134,24,15,0,31,248,6,6,6,

2,6,3,12,3,12,3,12,3,12,3,24,6,24,6,24,

12,24,24,48,48,255,192,31,252,24,48,96,65,129,12,68,

49,0,252,3,48,24,136,96,33,129,6,4,48,115,255,128,

31,252,24,48,96,65,129,12,36,48,128,254,3,24,24,64,

96,1,128,6,0,48,3,248,0,3,200,97,196,6,64,54,

1,96,3,0,24,0,129,252,3,48,25,128,198,12,15,160,

31,207,224,192,96,24,12,3,1,128,192,96,24,12,3,255,

128,96,48,24,12,3,1,128,96,48,12,6,3,1,129,248,

252,0,31,193,128,96,24,12,3,0,192,48,24,6,1,128,

96,48,63,0,7,224,48,6,0,192,48,6,0,192,24,6,

0,192,24,99,8,193,224,0,31,207,193,129,128,96,128,24,

64,12,96,3,48,0,220,0,59,0,28,96,6,24,1,135,

0,96,192,48,48,63,31,0,31,192,96,6,0,96,12,0,

192,12,0,192,24,17,129,24,33,130,48,239,252,31,0,124,

28,3,128,112,22,1,224,88,9,130,192,38,19,0,152,76,

2,98,48,17,137,128,70,70,1,26,24,4,56,96,32,195,

3,226,63,0,31,7,224,224,32,22,4,2,192,128,140,32,

17,132,2,24,128,67,16,16,116,2,6,128,64,240,8,14,

2,1,129,240,16,0,7,192,97,140,4,64,54,1,224,15,

0,120,3,192,52,1,176,25,129,134,24,31,0,31,248,12,

24,24,24,48,48,192,193,129,131,6,7,240,24,0,48,0,

96,0,192,3,0,31,128,0,7,192,97,140,4,64,54,1,

224,15,0,120,3,128,52,1,176,25,153,135,56,31,0,8,

0,100,3,192,12,0,31,240,24,96,96,193,131,12,24,48,

192,254,3,24,24,96,97,129,134,6,24,48,103,241,224,7,

161,140,96,140,19,128,56,7,224,62,0,192,25,3,32,204,

17,124,0,127,251,24,208,194,134,24,97,3,0,24,0,192,

12,0,96,3,0,24,1,128,127,128,254,63,48,8,48,8,

48,8,96,16,96,16,96,16,96,16,192,32,192,32,192,32,

192,64,96,128,62,0,248,124,192,67,2,12,16,48,64,194,

3,8,12,64,50,0,232,1,192,7,0,24,0,64,0,249,

251,236,28,17,131,132,48,176,134,22,32,228,196,12,153,1,

163,32,52,104,7,13,0,225,192,24,56,3,6,0,64,128,

0,15,207,129,130,0,98,0,50,0,31,0,7,0,3,0,

1,192,1,96,1,48,1,12,1,6,1,131,135,227,240,252,

62,48,48,96,64,97,0,196,1,216,1,160,3,128,6,0,

12,0,24,0,48,0,192,7,224,0,31,248,225,196,12,32,

194,12,0,192,12,0,192,14,32,225,6,8,96,198,12,127,

224,26,38,70,132,132,132,141,158,102,96,130,8,65,102,81,

134,24,98,153,128,28,69,4,8,16,32,194,120,3,1,1,

1,2,26,38,70,132,132,132,141,158,102,28,69,15,232,16,

32,162,120,1,129,65,0,128,64,120,16,8,8,4,2,1,

0,128,128,64,32,144,48,0,13,19,35,66,66,66,70,76,

52,4,4,8,240,48,8,4,2,2,1,112,196,66,98,33,

16,136,152,84,24,16,0,0,74,162,68,154,96,0,128,0,

0,0,0,0,24,18,9,1,0,128,64,32,32,16,8,4,

4,60,0,48,16,16,16,32,35,45,40,112,112,72,73,138,

142,100,148,146,146,93,128,207,57,98,138,130,19,12,68,16,

136,33,16,68,193,10,130,12,207,44,74,9,132,65,16,68,

38,10,129,128,28,69,12,24,48,97,68,112,51,22,74,34,

17,8,132,68,102,44,16,8,8,15,0,24,77,28,40,80,

163,76,104,16,32,135,192,218,218,24,65,4,32,128,57,20,

16,48,32,164,224,8,66,47,136,68,33,10,99,0,65,80,

168,68,68,34,17,26,142,59,0,67,70,137,36,72,145,36,

48,65,26,17,161,18,34,66,36,34,70,36,100,57,128,27,

137,18,64,32,8,2,0,138,100,102,0,65,161,161,34,66,

66,70,68,60,4,8,144,96,26,76,144,65,4,80,178,152,

31,248,48,65,130,12,16,192,134,0,48,1,128,24,0,192,

6,0,48,3,0,127,0,0,32,1,128,15,0,108,1,48,

8,192,67,1,14,8,24,64,98,1,136,7,127,255,255,240,

7,192,97,140,4,64,54,1,228,79,62,121,243,145,52,1,

176,25,129,134,24,31,0,0,192,12,1,192,28,2,192,44,

4,192,76,8,193,12,16,194,12,32,207,63,31,252,127,241,

0,68,1,0,0,32,128,254,3,248,16,64,0,4,1,16,

4,255,227,255,128,31,255,224,192,96,24,12,3,1,128,192,

96,24,12,3,1,128,96,48,24,12,3,1,128,96,48,12,

6,3,1,129,248,252,0,63,249,192,198,2,56,16,192,135,

0,24,0,192,4,0,64,132,4,64,68,6,127,224,56,119,

231,225,66,10,0,96,3,0,24,0,192,12,0,96,3,0,

24,1,128,127,128,15,224,48,3,0,252,54,100,99,198,60,

99,204,108,204,63,0,192,24,15,240,15,224,24,0,193,134,

124,103,99,51,25,152,216,204,195,108,15,128,24,1,128,127,

128,3,224,97,198,6,96,54,1,176,13,128,108,6,96,33,

3,40,21,65,78,14,112,224,28,4,69,4,192,168,21,3,

32,66,56,120,192,1,192,196,16,132,17,4,47,4,16,130,

32,68,8,129,16,69,16,156,16,2,0,128,16,0,56,81,

24,72,10,2,128,192,48,8,2,0,128,32,16,4,0,12,

36,64,129,3,9,34,133,10,20,72,142,0,28,132,62,130,

8,16,56,4,4,7,8,16,32,64,64,128,128,128,128,128,

96,24,4,4,24,206,177,161,98,66,66,66,132,132,4,4,

8,8,6,9,17,33,33,65,127,127,130,130,132,132,72,48,

33,8,132,34,50,96,35,18,10,14,7,130,33,19,10,135,

0,56,4,1,0,128,64,32,8,12,6,4,132,68,36,20,

4,16,132,33,8,132,33,8,66,53,142,93,144,4,2,0,

128,0,224,144,72,72,36,34,33,33,96,192,0,8,16,121,

4,8,16,30,65,2,4,8,16,28,4,4,112,127,228,137,

32,80,20,9,2,64,144,66,0,7,4,68,36,18,9,4,

132,196,92,32,16,16,8,0,31,200,132,34,8,130,32,136,

66,32,112,0,127,144,144,32,32,32,32,64,64,65,161,161,

34,66,66,68,68,56,1,0,64,32,8,2,3,195,73,17,

68,97,24,134,34,75,15,1,0,64,16,4,0,32,42,8,

65,4,64,144,12,1,0,80,18,2,32,132,32,168,12,1,

0,32,4,0,128,32,132,104,141,17,36,72,137,17,34,70,

144,60,2,0,64,16,2,0,32,40,5,0,194,40,69,8,

163,36,236,119,0,60,133,3,134,16,32,68,112,6,9,9,

9,9,197,163,161,66,66,68,68,72,48,127,255,32,18,128,

33,8,68,16,136,33,16,196,34,152,56,192,14,17,33,65,

65,65,66,196,184,128,128,128,124,30,65,4,8,16,24,8,

8,16,64,33,144,148,70,17,136,98,24,137,68,62,4,1,

0,128,32,0,15,4,34,8,1,0,71,146,29,3,128,160,

40,18,4,66,15,0,2,12,40,81,66,134,12,48,65,133,

2,35,128,74,162,68,154,96,12,20,40,32,64,129,4,8,

16,32,142,0,48,17,228,134,65,160,112,28,11,70,174,36,

9,2,128,192,0,5,252,48,126,129,56,138,12,24,48,96,

162,56,17,224,64,129,2,4,8,126,121,14,8,48,65,12,

190,252,121,15,24,48,71,3,3,7,143,26,99,128,8,48,

97,66,137,18,68,254,16,32,67,224,68,241,194,4,15,17,

3,7,14,26,99,128,28,69,2,8,23,49,65,131,5,10,

35,128,64,127,65,130,4,4,8,8,16,16,16,16,16,16,

56,10,12,30,79,15,39,131,6,10,35,128,56,138,20,24,

48,81,157,2,8,20,71,0,16,0,8,0,6,0,3,255,

255,255,255,216,0,2,0,0,64,0,0,2,0,0,64,0,

27,255,255,255,255,192,0,96,0,16,0,8,114,72,192,196,

147,128,129,194,68,104,49,164,112,128,2,28,75,24,44,68,

135,2,240,245,96,0,32,24,12,2,1,128,192,32,3,0,

24,0,128,12,0,96,2,2,4,16,32,64,130,4,8,32,

64,130,4,8,32,64,129,4,8,0,128,12,0,96,2,0,

48,1,128,8,6,3,0,128,96,48,8,0,128,79,192,192,

0,18,8,64,132,33,8,66,26,140,99,25,91,152,132,33,

9,246,49,140,99,24,207,161,8,64,82,148,165,249,74,82,

148,165,47,202,82,128,128,0,80,0,34,0,16,112,56,3,

240,0,3,240,7,3,130,0,17,0,2,128,0,64,

};

METRIC cmmi5_metric_small[128] = {

{285, -2, 6, 7, 7, 272842},

{292, -2, 6, 8, 7, 377748},

{299, -2, 6, 7, 7, 343249},

{306, -1, 6, 8, 7, 316762},

{313, -2, 6, 8, 7, 333487},

{320, -2, 6, 9, 7, 363866},

{328, -3, 6, 8, 7, 349075},

{335, -2, 6, 6, 7, 275346},

{341, -2, 6, 7, 7, 309480},

{348, -1, 6, 7, 7, 292777},

{355, -2, 6, 8, 7, 347539},

{362, -1, 3, 6, 4, 299277},

{365, -2, 6, 5, 9, 259075},

{371, -1, 3, 5, 6, 244746},

{375, -1, 6, 4, 7, 214359},

{379, -2, 3, 3, 4, 201047},

{381, -2, 6, 4, 9, 218456},

{386, -2, 3, 5, 6, 243716},

{390, -2, 6, 3, 7, 221868},

{393, -2, 3, 2, 4, 180150},

{394, -2, 3, 5, 4, 271174},

{397, -2, 6, 5, 7, 275346},

{402, -2, 3, 5, 6, 278456},

{406, -2, 3, 4, 4, 233323},

{408, -1, 6, 4, 9, 218456},

{413, -2, 3, 5, 4, 272350},

{416, -2, 3, 4, 6, 237912},

{419, -1, 3, 6, 4, 267115},

{422, -2, 3, 5, 4, 223349},

{425, -2, 3, 6, 4, 259417},

{428, -1, 6, 6, 9, 275800},

{435, -2, 3, 6, 6, 283310},

{440, -2, 6, 7, 9, 304929},

{448, -1, 3, 7, 4, 290441},

{452, -1, 3, 4, 4, 221072},

{454, -2, 6, 6, 7, 280277},

{460, -2, 3, 9, 4, 375131},

{465, -2, 3, 4, 6, 237912},

{468, -2, 3, 3, 5, 189442},

{470, -2, 3, 7, 6, 300832},

{531, -2, 3, 10, 2, 446016},

{534, -2, 2, 10, 2, 446016},

{537, -2, 3, 10, 2, 446016},

{540, -2, 2, 10, 2, 446016},

{543, -2, 5, 1, 4, 150188},

{544, -2, 5, 1, 4, 150188},

{545, -2, 4, 4, 5, 241212},

{548, -2, 4, 4, 5, 241212},

{499, -2, 3, 4, 4, 241212},

{501, -2, 3, 4, 4, 241212},

{503, -2, 3, 4, 4, 241212},

{505, -2, 3, 4, 6, 241212},

{508, -1, 3, 6, 6, 241212},

{513, -2, 3, 4, 6, 241212},

{516, -2, 6, 4, 7, 241212},

{520, -2, 3, 5, 6, 241212},

{524, -2, 6, 4, 7, 241212},

{528, -2, 3, 4, 6, 241212},

{551, -2, 0, 1, 1, 150188},

{552, -2, 0, 1, 3, 150188},

{553, -2, 5, 7, 7, 354992},

{560, -2, 7, 4, 11, 241212},

{566, -2, 5, 7, 7, 354992},

{573, -3, 2, 2, 2, 241212},

{476, -1, 6, 5, 7, 245420},

{0, -1, 6, 9, 7, 339518},

{8, -2, 6, 7, 7, 339802},

{15, -2, 6, 8, 7, 326365},

{22, -2, 6, 8, 7, 363684},

{29, -2, 6, 8, 7, 329960},

{36, -2, 6, 8, 7, 284220},

{43, -2, 6, 8, 7, 348665},

{50, -2, 6, 9, 7, 363866},

{58, -2, 6, 5, 7, 205371},

{63, -2, 6, 6, 7, 253272},

{69, -2, 6, 9, 7, 373197},

{77, -2, 6, 6, 7, 313121},

{83, -2, 6, 11, 7, 420756},

{93, -2, 6, 9, 7, 352488},

{101, -2, 6, 7, 7, 343249},

{108, -2, 6, 7, 7, 286837},

{115, -2, 6, 7, 9, 354627},

{123, -2, 6, 8, 7, 337498},

{130, -2, 6, 6, 7, 280580},

{136, -1, 6, 10, 7, 272729},

{145, -2, 6, 8, 7, 309297},

{152, -2, 6, 8, 7, 271250},

{159, -2, 6, 11, 7, 419164},

{169, -1, 6, 10, 7, 364663},

{178, -2, 6, 9, 7, 270112},

{186, -2, 6, 7, 7, 309025},

{574, -2, 7, 2, 8, 195700},

{576, -1, 6, 4, 9, 195700},

{581, -2, 6, 2, 9, 195700},

{584, -2, 4, 10, 4, 446016},

{589, -2, 4, 10, 4, 446016},

{481, -1, 6, 4, 7, 195699},

{193, -1, 3, 5, 4, 253765},

{196, -1, 6, 4, 7, 207532},

{200, -1, 3, 4, 4, 212804},

{202, -1, 6, 6, 7, 242387},

{208, -2, 3, 3, 4, 221845},

{210, -2, 6, 5, 9, 224144},

{216, -2, 3, 4, 6, 229568},

{219, -2, 6, 5, 7, 271174},

{224, -2, 7, 3, 8, 174841},

{227, -2, 7, 3, 10, 192741},

{231, -2, 6, 5, 7, 248418},

{236, -2, 6, 3, 7, 157394},

{239, -2, 3, 9, 4, 402401},

{244, -2, 3, 5, 4, 288621},

{247, -1, 3, 5, 4, 230288},

{250, -1, 3, 6, 6, 242388},

{255, -1, 3, 4, 6, 215914},

{258, -2, 3, 5, 4, 219973},

{261, -2, 3, 3, 4, 219593},

{263, -1, 5, 3, 6, 184702},

{266, -2, 3, 5, 4, 277243},

{269, -2, 3, 5, 4, 236661},

{272, -2, 3, 8, 4, 330624},

{276, -1, 3, 6, 4, 259416},

{279, -2, 3, 4, 6, 239392},

{282, -1, 3, 5, 4, 225662},

{485, -2, 3, 2, 4, 174841},

{486, -2, 3, 3, 6, 181363},

{489, -2, 3, 6, 6, 291389},

{494, -3, 7, 6, 3, 241212},

{497, -3, 6, 7, 2, 150188},

};

char cmmi5_bitmap_small[594] = {

6,3,2,129,65,225,27,206,126,68,137,226,40,190,0,61,

66,130,128,130,132,120,126,33,33,33,32,66,252,127,34,42,

56,42,66,252,127,34,38,60,36,64,240,61,66,130,143,130,

132,124,115,145,8,135,194,34,35,184,121,8,66,35,128,28,

32,130,10,79,0,123,147,11,7,2,66,35,156,120,130,8,

37,47,128,112,230,48,166,21,66,168,154,58,224,119,153,10,

133,66,162,35,144,60,134,12,24,81,156,0,126,68,137,226,

8,56,0,60,142,12,24,86,156,8,8,126,34,34,60,34,

68,231,61,4,12,8,78,0,127,242,40,136,32,8,4,7,

192,255,68,68,68,68,72,112,231,66,68,72,40,48,32,238,

105,73,74,41,70,48,198,16,128,57,194,64,224,16,14,4,

142,56,231,178,10,6,1,1,1,192,62,201,32,130,73,62,

0,58,102,208,100,70,90,96,52,135,12,32,142,74,102,128,

250,112,25,8,226,33,8,128,117,166,44,98,16,228,206,96,

32,13,182,32,4,210,80,98,16,230,218,64,105,37,24,187,

98,114,113,48,182,103,48,50,100,192,89,148,156,67,128,117,

166,39,246,97,0,104,240,39,150,64,206,102,208,214,100,192,

202,201,146,108,121,90,158,221,166,44,123,157,224,126,72,145,

2,8,60,0,4,10,18,50,33,65,255,60,142,12,152,80,

156,0,4,12,12,18,18,34,231,63,66,36,60,36,132,252,

127,145,8,132,66,34,35,184,127,66,34,16,34,68,252,204,

194,8,33,15,0,60,33,244,151,196,30,0,60,35,90,167,

132,30,0,30,33,65,65,34,166,238,53,40,157,25,82,228,

203,144,128,232,196,33,8,52,66,90,96,254,112,19,72,136,

134,96,182,101,33,8,110,251,160,95,91,167,16,97,4,35,

38,32,74,103,216,64,213,172,19,67,72,135,112,249,24,160,

101,174,136,61,41,24,249,16,128,203,24,156,8,65,14,86,

167,8,32,133,35,12,82,32,8,16,70,173,50,158,16,32,

133,38,214,192,118,135,16,162,167,202,70,0,255,162,99,78,

192,101,174,142,114,36,141,38,147,194,4,0,50,66,117,201,

128,50,102,76,112,255,77,37,0,90,156,182,163,0,11,241,

0,61,132,105,150,226,47,225,63,233,97,150,49,205,63,16,

224,248,233,158,116,142,153,96,252,136,68,32,105,214,153,96,

105,151,30,64,63,240,255,208,0,0,191,240,255,192,32,240,

240,142,158,128,23,151,16,128,224,6,49,132,6,3,1,128,

17,34,36,68,72,128,192,96,48,16,198,48,0,240,170,254,

136,189,153,189,16,255,255,192,128,80,34,16,120,30,8,68,

10,1,

};

METRIC cmmi7_metric_small[128] = {

{525, -1, 9, 9, 10, 318261},

{537, -1, 9, 11, 10, 437819},

{551, -1, 9, 10, 10, 398623},

{564, -1, 9, 10, 10, 365912},

{577, -1, 9, 10, 10, 387415},

{590, -1, 9, 12, 10, 429218},

{605, -1, 9, 11, 10, 406598},

{619, -1, 9, 10, 10, 310841},

{632, 0, 9, 9, 10, 353167},

{644, -1, 9, 10, 10, 328873},

{657, -2, 9, 10, 10, 403720},

{670, -1, 5, 7, 6, 340706},

{676, 0, 9, 9, 13, 297167},

{691, 0, 5, 7, 9, 275286},

{699, -1, 9, 6, 10, 238206},

{707, -1, 5, 5, 6, 218432},

{711, -1, 9, 6, 13, 238479},

{721, -1, 5, 7, 9, 270020},

{729, -1, 9, 6, 10, 249628},

{737, -1, 5, 3, 6, 193955},

{740, -1, 5, 7, 6, 306823},

{746, -1, 9, 7, 10, 310843},

{755, 0, 5, 8, 9, 318657},

{764, -1, 5, 6, 6, 262753},

{769, 0, 9, 7, 13, 238479},

{781, -1, 5, 8, 6, 306467},

{787, 0, 5, 7, 9, 271908},

{795, -1, 5, 8, 6, 303690},

{801, -1, 5, 8, 6, 241688},

{807, -1, 5, 8, 6, 290363},

{813, -1, 9, 7, 13, 315120},

{825, -1, 5, 8, 9, 327454},

{834, -1, 9, 9, 13, 346797},

{849, 0, 5, 9, 6, 330155},

{856, -1, 5, 5, 6, 247604},

{860, -1, 9, 8, 10, 316472},

{870, -1, 5, 12, 6, 435795},

{879, -1, 5, 6, 9, 271908},

{886, -1, 5, 5, 7, 201501},

{891, -1, 5, 9, 9, 344703},

{1008, -1, 5, 14, 3, 522470},

{1014, -1, 3, 14, 3, 522470},

{1020, -1, 5, 14, 3, 522470},

{1026, -1, 3, 14, 3, 522470},

{1032, -1, 7, 2, 5, 155649},

{1034, -1, 7, 2, 5, 155649},

{1036, -1, 6, 6, 7, 268517},

{1042, -1, 6, 6, 7, 268517},

{942, -1, 5, 6, 6, 268517},

{947, -2, 5, 5, 6, 268517},

{951, -1, 5, 6, 6, 268517},

{956, -1, 5, 6, 9, 268517},

{963, 0, 5, 8, 9, 268517},

{972, -1, 5, 6, 9, 268517},

{979, -1, 8, 6, 9, 268517},

{986, -1, 5, 7, 9, 268517},

{994, -1, 8, 6, 9, 268517},

{1001, -1, 5, 6, 9, 268517},

{1048, -1, 1, 2, 2, 155649},

{1049, -1, 1, 2, 5, 155649},

{1051, -1, 8, 10, 11, 409602},

{1065, -1, 10, 6, 15, 268517},

{1077, -1, 7, 10, 10, 409602},

{1090, -2, 3, 4, 2, 268517},

{902, -1, 9, 6, 10, 279871},

{0, -1, 9, 10, 10, 394129},

{13, -1, 9, 11, 10, 395984},

{27, -1, 9, 10, 10, 375896},

{40, -1, 9, 11, 10, 428507},

{54, -1, 9, 10, 10, 384753},

{67, -1, 9, 10, 10, 332370},

{80, -1, 9, 10, 10, 408026},

{93, -1, 9, 12, 10, 429218},

{108, -1, 9, 6, 10, 232268},

{116, -1, 9, 8, 10, 289952},

{126, -1, 9, 12, 10, 440369},

{141, -1, 9, 8, 10, 359540},

{151, -1, 9, 15, 10, 499760},

{170, -1, 9, 12, 10, 415109},

{185, -1, 9, 10, 10, 398623},

{198, -1, 9, 10, 10, 333666},

{211, -1, 9, 10, 13, 412732},

{228, -1, 9, 10, 10, 394806},

{241, -1, 9, 8, 10, 321811},

{251, -1, 9, 9, 10, 309545},

{263, -1, 9, 10, 10, 357009},

{276, -1, 9, 10, 10, 309478},

{289, -1, 9, 15, 10, 492888},

{308, 0, 9, 12, 10, 429787},

{323, 0, 9, 11, 10, 308068},

{337, -1, 9, 9, 10, 357082},

{1091, -1, 10, 4, 11, 212083},

{1097, -1, 9, 4, 13, 212083},

{1104, -1, 9, 4, 13, 212083},

{1111, -1, 5, 14, 4, 522470},

{1118, -1, 5, 14, 4, 522470},

{910, 0, 9, 6, 10, 219366},

{349, -1, 5, 6, 6, 284272},

{354, -1, 9, 5, 10, 230469},

{361, -1, 5, 5, 6, 234209},

{365, -1, 9, 6, 10, 272818},

{373, -1, 5, 5, 6, 248652},

{377, -1, 9, 7, 13, 255548},

{389, 0, 5, 7, 9, 255676},

{397, -1, 9, 7, 10, 306823},

{406, -1, 10, 4, 11, 185421},

{412, 0, 10, 6, 14, 216863},

{423, -1, 9, 7, 10, 278606},

{432, -1, 9, 3, 10, 165737},

{436, -1, 5, 12, 6, 465051},

{445, -1, 5, 7, 6, 323966},

{451, -1, 5, 6, 6, 258686},

{456, 0, 5, 8, 9, 270163},

{465, -1, 5, 5, 9, 240202},

{471, -1, 5, 6, 6, 243335},

{476, 0, 5, 6, 6, 247354},

{481, 0, 8, 4, 9, 197975},

{486, -1, 5, 7, 6, 309858},

{492, -1, 5, 7, 6, 262146},

{498, -1, 5, 10, 6, 379129},

{506, 0, 5, 8, 6, 297189},

{512, -1, 5, 6, 9, 265788},

{519, 0, 5, 7, 6, 250390},

{918, -1, 5, 3, 6, 182881},

{921, 0, 5, 5, 9, 202755},

{927, -1, 5, 8, 9, 334941},

{936, -2, 9, 8, 3, 268517},

{939, -4, 9, 6, 3, 155649},

};

char cmmi7_bitmap_small[1125] = {

3,0,192,80,36,9,4,65,240,132,33,56,240,63,194,12,

64,136,35,248,65,136,17,2,65,159,224,15,68,50,5,1,

128,32,8,10,4,67,15,0,63,194,12,64,136,18,2,64,

72,9,2,64,159,192,63,196,17,20,77,62,8,130,36,130,

65,191,192,63,196,17,20,77,62,8,130,32,128,64,60,0,

15,68,50,5,1,128,32,248,10,2,65,15,192,60,241,4,

16,65,4,63,130,8,32,130,8,65,15,60,60,65,4,32,

130,8,67,192,15,2,2,2,4,4,4,4,136,112,60,241,

12,17,1,32,46,3,160,34,2,16,65,15,28,60,16,16,

16,32,32,33,35,66,254,56,14,48,48,96,160,161,66,69,

4,146,9,36,18,136,70,33,236,240,56,241,132,24,65,68,

36,130,72,34,130,40,67,15,16,15,132,50,5,1,128,96,

24,6,2,67,15,0,63,132,17,4,65,32,143,194,0,128,

64,60,0,15,132,50,5,1,128,96,24,6,2,91,15,0,

64,16,3,0,63,132,17,4,66,63,8,66,16,132,66,60,

112,31,33,32,32,24,4,2,2,132,248,255,196,98,49,17,

0,128,64,32,32,124,0,243,208,68,17,4,130,32,136,34,

8,132,28,0,225,208,68,17,8,68,17,2,128,160,48,8,

0,227,222,134,17,20,34,40,132,145,9,20,20,40,40,96,

96,192,193,0,30,112,68,4,128,48,2,0,80,5,0,152,

16,143,30,240,228,16,68,9,1,192,16,2,0,64,16,7,

128,63,152,200,196,64,64,64,68,66,66,127,0,57,40,162,

149,176,194,16,139,102,49,147,0,58,97,8,184,12,16,65,

57,40,162,149,176,62,125,8,184,6,16,32,65,193,4,8,

16,32,65,14,0,14,36,137,18,67,129,2,248,96,64,129,

5,140,145,34,139,8,16,0,4,164,70,96,4,0,0,0,

98,130,8,65,4,35,128,96,64,129,4,234,92,40,149,24,

100,148,146,172,108,203,50,34,34,34,68,84,66,125,104,145,

36,168,128,57,24,97,137,192,54,89,17,17,50,44,32,32,

240,50,99,25,56,66,120,126,146,8,65,0,28,147,129,133,

224,17,31,34,37,48,101,72,145,34,166,192,103,68,137,18,

71,0,100,233,18,68,145,44,141,192,55,73,8,8,146,124,

102,146,73,40,224,148,96,58,136,33,134,91,0,63,136,68,

34,18,1,0,128,64,64,120,0,1,128,80,26,2,96,132,

32,140,17,1,64,63,252,15,132,50,5,37,158,100,152,6,

2,67,15,0,3,0,192,80,36,9,4,65,16,132,65,56,

240,63,200,16,0,66,63,8,64,0,0,129,63,192,63,241,

4,16,65,4,32,130,8,32,130,8,65,15,60,63,226,12,

64,132,16,192,16,4,17,130,96,159,240,113,226,144,192,16,

8,2,0,128,32,16,31,0,15,2,7,200,152,140,71,36,

124,16,62,0,15,1,12,79,18,137,34,68,160,240,16,31,

0,31,8,36,5,1,64,144,36,17,8,195,49,128,50,150,

44,40,206,192,7,4,68,34,35,225,8,132,66,98,46,16,

8,8,0,115,22,48,96,193,2,4,16,28,132,8,49,40,

162,145,128,62,61,8,184,16,114,16,66,8,32,129,129,130,

48,109,100,137,20,72,129,2,8,24,164,81,251,232,164,145,

128,73,42,128,70,177,226,72,176,192,96,32,64,64,129,133,

18,67,4,34,34,34,34,101,123,64,64,128,197,20,146,179,

0,8,60,129,2,3,136,32,128,128,224,33,128,127,152,40,

40,40,72,12,36,137,22,75,16,32,128,63,68,132,132,136,

112,127,144,16,16,32,32,99,161,33,34,34,60,8,16,32,

67,202,100,201,164,240,129,4,0,65,162,36,24,16,40,40,

68,130,2,1,1,0,134,93,36,162,82,43,14,4,2,1,

0,65,196,98,49,57,147,112,124,29,8,248,14,18,17,17,

110,163,34,36,36,56,127,249,34,34,34,100,38,67,184,24,

148,81,202,200,32,112,62,33,131,8,192,67,66,98,49,45,

49,224,64,32,32,0,57,16,65,61,24,98,137,192,12,82,

74,73,70,16,196,224,85,38,192,25,66,17,8,68,224,96,

142,177,193,210,204,160,160,64,2,255,2,122,24,64,122,24,

97,133,224,101,8,66,124,122,16,66,55,240,122,16,65,56,

24,97,120,12,28,52,100,196,255,4,4,31,254,8,46,196,

24,98,112,61,8,62,134,24,81,56,255,4,16,65,2,8,

16,32,122,24,94,122,24,97,120,49,40,97,133,240,66,240,

32,1,128,15,255,192,255,253,128,2,0,0,0,16,0,111,

255,192,255,252,0,96,1,0,106,64,149,128,131,137,161,155,

136,0,4,118,97,100,112,64,240,245,128,0,64,112,96,96,

96,32,6,0,96,6,0,96,4,4,16,130,8,65,4,32,

132,16,66,8,0,96,6,0,96,6,0,64,96,96,96,96,

32,0,246,136,136,139,153,170,192,136,139,217,153,155,209,16,

25,155,217,153,155,217,128,128,5,0,35,3,3,240,15,192,

192,196,0,160,1,

};

METRIC cmr10_metric_small[128] = {

{937, 0, 13, 11, 14, 409601},

{957, -1, 13, 14, 14, 546135},

{982, -1, 13, 13, 14, 509726},

{1005, -1, 13, 11, 14, 455112},

{1025, -1, 13, 11, 14, 436908},

{1045, 0, 13, 15, 14, 491521},

{1072, -1, 13, 12, 14, 473317},

{1093, -1, 13, 13, 14, 509726},

{1116, -1, 13, 12, 14, 473317},

{1137, -1, 13, 13, 14, 509726},

{1160, -1, 13, 12, 14, 473317},

{1762, 0, 13, 12, 14, 382295},

{1783, 0, 13, 11, 14, 364090},

{1803, 0, 13, 11, 14, 364090},

{1823, 0, 13, 16, 14, 546135},

{1851, 0, 13, 16, 14, 546135},

{1367, 0, 8, 4, 9, 182045},

{1372, 1, 8, 4, 13, 200250},

{1738, -2, 13, 3, 3, 327681},

{1740, -4, 13, 3, 3, 327681},

{1742, -3, 12, 3, 2, 327681},

{1743, -2, 13, 5, 4, 327681},

{1746, -1, 11, 7, 1, 327681},

{1747, -5, 13, 5, 3, 491521},

{1749, -3, -1, 4, 4, 291272},

{1379, 0, 13, 8, 14, 327682},

{1393, -1, 8, 12, 9, 473317},

{1407, -1, 8, 13, 9, 509726},

{1422, -1, 10, 7, 13, 327681},

{1434, -1, 13, 16, 14, 591646},

{1462, -1, 13, 18, 14, 664464},

{1494, -1, 14, 13, 16, 509726},

{1751, 0, 7, 5, 2, 182045},

{1520, -2, 13, 2, 14, 182045},

{1879, -1, 13, 6, 6, 327681},

{1528, -1, 13, 14, 18, 546135},

{1303, -1, 15, 7, 17, 327681},

{1560, -1, 15, 14, 17, 546135},

{1318, -1, 13, 13, 14, 509726},

{1590, -2, 13, 2, 6, 182045},

{1592, -1, 15, 5, 21, 254863},

{1606, -1, 15, 5, 21, 254863},

{1620, -1, 15, 7, 9, 327681},

{1628, -1, 11, 13, 13, 509726},

{1650, -2, 1, 2, 6, 182045},

{1884, 0, 4, 5, 1, 218454},

{1652, -2, 1, 2, 2, 182045},

{1653, -1, 15, 7, 21, 327681},

{1181, -1, 12, 7, 13, 327681},

{1193, -1, 12, 7, 13, 327681},

{1205, -1, 12, 7, 13, 327681},

{1217, -1, 12, 7, 13, 327681},

{1229, -1, 12, 7, 13, 327681},

{1241, -1, 12, 7, 13, 327681},

{1253, -1, 12, 7, 13, 327681},

{1265, -1, 13, 8, 14, 327681},

{1279, -1, 12, 7, 13, 327681},

{1291, -1, 12, 7, 13, 327681},

{1672, -2, 8, 2, 9, 182045},

{1675, -2, 8, 2, 13, 182045},

{1524, -2, 9, 2, 14, 182045},

{1679, -1, 8, 13, 7, 509726},

{1354, -1, 9, 7, 14, 309477},

{1341, -1, 13, 7, 14, 309477},

{1691, -1, 13, 13, 14, 509726},

{0, -1, 13, 13, 14, 491521},

{23, 0, 13, 13, 14, 464215},

{46, -1, 13, 12, 14, 473317},

{67, 0, 13, 14, 14, 500624},

{92, 0, 13, 12, 14, 446010},

{113, 0, 13, 12, 14, 427806},

{134, -1, 13, 13, 14, 514277},

{157, 0, 13, 15, 14, 491521},

{184, 0, 13, 6, 14, 236658},

{195, -1, 13, 9, 14, 336783},

{211, 0, 13, 14, 14, 509726},

{236, 0, 13, 11, 14, 409601},

{256, 0, 13, 18, 14, 600748},

{288, 0, 13, 15, 14, 491521},

{315, -1, 13, 13, 14, 509726},

{338, 0, 13, 12, 14, 446010},

{359, -1, 13, 13, 18, 509726},

{389, 0, 13, 14, 14, 482419},

{414, -1, 13, 9, 14, 364090},

{430, -1, 13, 12, 14, 473317},

{451, 0, 13, 15, 14, 491521},

{478, 0, 13, 15, 14, 491521},

{505, 0, 13, 20, 14, 673566},

{540, 0, 13, 15, 14, 491521},

{567, 0, 13, 15, 14, 491521},

{594, -1, 13, 10, 14, 400499},

{1714, -1, 15, 4, 21, 182045},

{1885, -2, 13, 6, 6, 327681},

{1725, 0, 15, 4, 21, 182045},

{1753, -3, 13, 3, 3, 327681},

{1755, -2, 14, 1, 2, 182045},

{1736, -2, 13, 2, 6, 182045},

{612, -1, 8, 8, 9, 327681},

{621, 0, 13, 10, 14, 364090},

{639, -1, 8, 6, 9, 291272},

{646, -1, 13, 10, 14, 364090},

{664, -1, 8, 6, 9, 291272},

{671, 0, 13, 6, 14, 200250},

{682, -1, 8, 8, 14, 327681},

{696, 0, 13, 11, 14, 364090},

{716, 0, 14, 4, 15, 182045},

{724, 1, 14, 4, 19, 200250},

{734, 0, 13, 10, 14, 345886},

{752, 0, 13, 5, 14, 182045},

{761, 0, 8, 17, 9, 546135},

{781, 0, 8, 11, 9, 364090},

{794, -1, 8, 7, 9, 327681},

{802, 0, 8, 10, 13, 364090},

{819, -1, 8, 10, 13, 345885},

{836, 0, 8, 7, 9, 256684},

{844, -1, 8, 5, 9, 258504},

{850, 0, 12, 6, 13, 254863},

{860, 0, 8, 11, 9, 364090},

{873, 0, 8, 10, 9, 345886},

{885, 0, 8, 14, 9, 473317},

{901, 0, 8, 10, 9, 345886},

{913, 0, 8, 10, 13, 345886},

{930, -1, 8, 6, 9, 291272},

{1890, 0, 5, 9, 1, 327681},

{1892, 0, 5, 20, 1, 655362},

{1756, -3, 13, 4, 3, 327681},

{1758, -2, 13, 5, 2, 327681},

{1760, -2, 14, 5, 2, 327681},

};

char cmr10_bitmap_small[1895] = {

2,0,56,1,192,14,0,248,4,192,38,3,56,16,192,254,

12,56,64,194,6,120,124,255,193,131,12,12,96,99,3,24,

48,255,6,14,48,25,128,204,6,96,51,3,127,240,15,145,

135,32,54,1,192,28,0,192,12,0,192,12,1,96,18,2,

24,64,248,255,192,193,195,1,140,6,48,12,192,51,0,204,

3,48,12,192,51,1,140,6,48,51,255,0,255,227,6,48,

35,1,49,19,16,63,3,16,49,19,1,48,51,2,48,111,

254,255,227,6,48,35,1,48,147,8,63,131,8,48,131,0,

48,3,0,48,15,224,15,144,195,136,12,192,44,1,96,3,

0,24,0,193,254,1,152,12,64,97,135,7,200,252,126,96,

48,192,97,128,195,1,134,3,15,254,24,12,48,24,96,48,

192,97,128,195,1,159,143,192,252,195,12,48,195,12,48,195,

12,51,240,63,131,1,128,192,96,48,24,12,6,3,1,152,

204,227,192,252,124,192,195,6,12,48,49,128,204,3,112,15,

96,57,128,195,3,6,12,24,48,51,241,240,254,6,0,192,

24,3,0,96,12,1,128,48,38,4,193,152,35,13,255,128,

248,7,206,1,195,128,112,176,44,44,11,9,132,194,97,48,

152,76,35,35,8,200,194,28,48,135,12,33,195,62,35,240,

248,62,112,16,176,33,112,66,96,132,97,8,98,16,228,32,

200,64,208,129,225,1,194,1,159,1,0,15,129,199,8,8,

192,108,1,224,15,0,120,3,192,30,0,216,12,64,67,142,

7,192,255,195,6,48,51,3,48,51,3,48,99,252,48,3,

0,48,3,0,48,15,192,15,129,199,8,8,192,108,1,224,

15,0,120,3,192,30,0,216,12,78,67,206,7,192,2,0,

9,0,120,1,128,255,128,195,3,6,12,24,48,96,195,3,

248,12,96,48,192,195,3,12,12,48,48,199,241,224,61,49,

176,88,44,7,129,248,62,3,128,224,112,60,53,240,255,252,

99,134,24,97,134,16,96,6,0,96,6,0,96,6,0,96,

6,3,252,252,62,96,16,192,33,128,67,0,134,1,12,2,

24,4,48,8,96,16,192,32,128,128,131,0,248,0,252,62,

96,48,96,64,192,128,194,1,132,3,8,3,32,6,64,12,

128,14,0,28,0,56,0,32,0,252,249,243,7,4,56,112,

193,135,8,24,240,129,137,136,12,153,0,201,144,13,13,0,

240,240,7,14,0,112,224,6,6,0,32,64,126,252,48,64,

49,0,114,0,104,0,112,0,192,1,192,3,192,9,128,33,

128,67,129,3,31,143,192,252,62,48,48,112,64,97,128,98,

0,200,0,208,1,192,1,128,3,0,6,0,12,0,24,0,

252,0,255,240,104,26,12,135,1,128,192,48,24,78,19,5,

129,96,255,240,120,4,4,60,68,132,133,141,118,224,8,2,

0,128,32,11,195,8,129,32,72,18,4,129,48,139,192,61,

8,32,130,8,17,56,7,0,64,16,4,1,15,68,50,4,

129,32,72,18,4,67,15,112,57,24,127,130,8,17,56,28,

130,8,35,226,8,32,130,8,35,224,55,72,72,72,72,72,

112,64,124,130,130,130,68,56,224,4,0,128,16,2,0,94,

12,33,4,32,132,16,130,16,66,9,247,192,102,0,0,226,

34,34,34,240,51,0,0,113,17,17,17,17,17,224,224,8,

2,0,128,32,11,226,96,176,40,13,2,32,136,33,62,240,

225,8,66,16,132,33,8,66,124,239,60,24,97,8,32,132,

16,66,8,33,4,16,130,8,65,4,251,239,128,239,6,16,

130,16,66,8,65,8,33,4,251,224,56,138,12,24,48,96,

162,56,239,12,34,4,129,32,72,18,4,194,47,8,2,0,

128,248,0,61,16,200,18,4,129,32,72,17,12,61,0,64,

16,4,7,192,252,108,129,2,4,8,16,248,124,99,131,70,

49,240,32,130,24,252,130,8,32,146,73,24,227,132,16,130,

16,66,8,65,8,33,12,30,224,243,200,66,16,72,18,4,

128,192,48,12,0,243,188,140,66,49,9,36,20,160,82,129,

74,2,16,8,64,243,200,129,32,48,12,3,1,32,132,243,

192,243,200,66,16,72,18,4,128,192,48,12,2,0,130,96,

240,0,254,40,164,48,148,81,252,255,198,24,193,24,19,2,

96,12,1,128,48,6,0,192,24,3,1,252,0,3,0,12,

0,88,1,96,9,192,35,1,14,4,24,48,112,128,194,3,

16,6,127,251,255,240,15,129,199,8,8,192,108,1,228,79,

62,121,243,200,158,0,216,12,64,67,142,7,192,4,0,128,

56,7,0,224,46,4,192,152,19,4,48,134,16,198,29,231,

192,255,255,254,0,192,16,0,65,15,225,252,32,128,2,0,

192,31,255,255,192,255,254,96,48,192,97,128,195,1,134,3,

12,6,24,12,48,24,96,48,192,97,128,195,1,159,143,192,

255,238,6,112,35,1,56,17,192,12,0,64,12,1,129,16,

18,2,64,111,254,120,119,231,225,98,14,0,48,1,128,12,

0,96,3,0,24,0,192,6,0,48,15,240,63,192,96,6,

1,248,102,108,99,198,60,99,198,54,102,31,128,96,6,3,

252,31,224,24,0,193,198,118,51,49,153,140,204,102,51,97,

219,3,224,6,0,48,15,240,31,131,12,96,108,3,192,60,

3,192,52,2,96,98,4,160,91,13,112,231,14,56,136,4,

24,48,96,193,131,6,10,35,128,17,224,64,129,2,4,8,

16,32,64,135,224,56,138,30,16,32,193,6,24,32,138,47,

192,121,15,24,48,71,3,3,7,143,26,99,128,8,48,225,

70,137,34,127,8,16,32,67,224,68,241,194,4,15,17,3,

7,14,26,99,128,28,69,2,8,23,49,65,131,5,10,35,

128,64,127,65,130,4,4,8,8,16,16,16,16,16,16,56,

10,12,30,79,15,39,131,6,10,35,128,56,138,20,24,48,

81,157,2,8,20,71,0,16,32,226,169,50,100,120,120,56,

88,153,50,85,28,16,12,0,144,4,128,36,1,64,14,124,

96,135,8,104,70,100,33,33,6,20,48,158,120,125,6,8,

48,130,4,8,0,0,0,3,6,0,48,96,0,0,0,8,

16,32,130,4,24,47,128,226,34,34,34,240,113,17,17,17,

17,17,224,30,33,33,33,34,236,34,34,33,33,33,33,37,

230,120,224,115,2,17,255,98,8,32,130,8,113,120,224,60,

114,20,224,67,3,248,16,64,131,12,8,81,60,112,2,8,

242,104,179,100,217,162,201,226,8,0,15,254,5,134,5,130,

9,129,9,129,9,136,17,136,31,248,17,137,17,137,33,131,

33,130,33,134,243,254,15,255,134,56,99,6,9,129,129,192,

97,112,24,76,7,243,1,132,192,97,112,24,22,6,12,193,

130,24,225,131,255,224,0,32,125,12,24,65,198,11,96,143,

4,120,67,194,30,32,241,6,208,99,130,24,48,190,4,0,

255,255,0,240,240,15,255,240,2,16,8,64,33,1,8,4,

35,255,240,132,2,16,8,64,33,0,132,63,255,16,128,66,

2,16,8,64,33,0,132,0,32,17,64,136,254,34,16,136,

66,34,8,136,20,64,34,0,8,224,66,129,17,8,68,33,

17,4,68,10,32,56,245,96,8,136,68,34,16,132,33,8,

66,8,65,8,32,128,130,8,65,8,33,8,66,16,132,34,

17,8,136,0,16,34,75,225,15,164,136,16,2,0,16,0,

128,4,0,32,1,3,255,224,64,2,0,16,0,128,4,0,

32,0,245,96,240,2,4,16,32,64,130,4,8,32,64,130,

4,8,32,64,129,4,8,0,240,3,192,240,3,213,128,255,

248,0,0,0,0,0,0,0,3,255,224,15,128,130,8,8,

142,40,140,200,38,65,50,9,144,76,70,209,204,64,1,1,

135,240,248,136,136,136,136,136,136,136,136,136,240,241,17,17,

17,17,17,17,17,17,17,240,106,240,136,128,42,0,168,140,

118,224,254,116,92,99,30,63,0,86,128,192,153,144,109,128,

222,192,15,241,16,33,2,16,33,15,254,33,2,16,33,2,

16,33,2,16,33,14,28,15,130,16,128,16,2,1,255,8,

225,4,32,132,16,130,16,66,9,227,192,15,130,16,130,16,

66,9,255,8,33,4,32,132,16,130,16,66,9,227,192,15,

188,16,196,32,128,32,128,32,128,255,252,32,156,32,132,32,

132,32,132,32,132,32,132,32,132,241,207,15,188,16,68,32,

132,32,132,32,132,255,252,32,132,32,132,32,132,32,132,32,

132,32,132,32,132,241,207,207,52,81,70,32,248,70,40,162,

207,48,255,128,255,255,240,

};

METRIC cmr5_metric_small[128] = {

{257, -1, 6, 5, 7, 272161},

{262, -1, 6, 7, 7, 359544},

{269, -1, 6, 7, 7, 336788},

{276, 0, 6, 7, 7, 298558},

{283, -1, 6, 5, 7, 291276},

{288, -1, 6, 6, 7, 321314},

{294, -1, 6, 6, 7, 314032},

{300, -1, 6, 7, 7, 336788},

{307, -1, 6, 6, 7, 314032},

{313, -1, 6, 7, 7, 336788},

{320, -1, 6, 6, 7, 314032},

{521, 0, 6, 5, 9, 223008},

{515, 0, 6, 5, 9, 223008},

{541, -1, 6, 1, 3, 131984},

{427, -1, 4, 1, 7, 131984},

{373, -1, 4, 3, 7, 211630},

{376, 0, 3, 3, 4, 131984},

{378, 1, 3, 3, 6, 143362},

{494, -1, 6, 2, 2, 223008},

{495, -2, 6, 2, 2, 223008},

{496, -2, 5, 1, 1, 223008},

{497, -1, 6, 3, 2, 223008},

{498, -1, 5, 3, 1, 223008},

{499, -2, 6, 4, 2, 321314},

{500, -1, -1, 3, 2, 200252},

{381, 0, 6, 5, 7, 223008},

{386, -1, 3, 7, 4, 314032},

{390, 0, 3, 9, 4, 336788},

{395, 0, 4, 5, 6, 223008},

{399, 0, 6, 10, 7, 385941},

{408, -1, 6, 11, 7, 431453},

{418, -1, 7, 7, 9, 336788},

{501, 1, 3, 4, 1, 131984},

{426, -1, 6, 1, 7, 131984},

{507, 0, 6, 4, 3, 223008},

{428, -1, 6, 7, 9, 359544},

{359, -1, 7, 3, 9, 223008},

{436, -1, 7, 7, 9, 359544},

{363, -1, 6, 7, 7, 336788},

{444, -1, 6, 1, 3, 131984},

{445, 0, 7, 3, 11, 177496},

{450, -1, 7, 3, 11, 177496},

{455, 0, 7, 5, 4, 223008},

{458, -1, 6, 7, 9, 336788},

{466, -1, 0, 1, 3, 131984},

{509, 0, 2, 2, 1, 154740},

{467, -1, 0, 1, 1, 131984},

{468, -1, 7, 3, 11, 223008},

{326, -1, 6, 3, 7, 223008},

{329, -1, 6, 3, 7, 223008},

{332, -1, 6, 3, 7, 223008},

{335, -1, 6, 3, 7, 223008},

{338, 0, 6, 5, 7, 223008},

{343, -1, 6, 3, 7, 223008},

{346, -1, 6, 3, 7, 223008},

{349, -1, 6, 4, 7, 223008},

{353, -1, 6, 3, 7, 223008},

{356, -1, 6, 3, 7, 223008},

{473, -1, 3, 1, 4, 131984},

{474, -1, 3, 1, 6, 131984},

{527, -1, 5, 7, 7, 336788},

{475, -1, 4, 7, 5, 336788},

{534, -1, 5, 7, 7, 336788},

{370, -1, 6, 3, 7, 211630},

{480, -1, 6, 7, 7, 336788},

{0, 0, 6, 8, 7, 321314},

{7, -1, 6, 6, 7, 306295},

{13, -1, 6, 6, 7, 314032},

{19, -1, 6, 6, 7, 329051},

{25, -1, 6, 6, 7, 294917},

{31, -1, 6, 5, 7, 283539},

{36, -1, 6, 8, 7, 338609},

{43, -1, 6, 6, 7, 321314},

{49, 0, 6, 3, 7, 162022},

{52, -1, 6, 4, 7, 226649},

{56, -1, 6, 8, 7, 332692},

{63, -1, 6, 5, 7, 272161},

{68, -1, 6, 8, 7, 389582},

{75, -1, 6, 6, 7, 321314},

{81, -1, 6, 7, 7, 336788},

{88, -1, 6, 5, 7, 294917},

{93, -1, 6, 7, 9, 336788},

{101, -1, 6, 7, 7, 317673},

{108, -1, 6, 4, 7, 245764},

{112, 0, 6, 8, 7, 314032},

{119, -1, 6, 6, 7, 321314},

{125, 0, 6, 8, 7, 321314},

{132, 0, 6, 12, 7, 435094},

{143, 0, 6, 8, 7, 321314},

{150, 0, 6, 8, 7, 321314},

{157, -1, 6, 5, 7, 268520},

{487, 0, 7, 2, 11, 131984},

{510, -1, 6, 4, 3, 223008},

{490, 0, 7, 2, 11, 131984},

{502, -2, 6, 1, 1, 223008},

{503, -1, 7, 1, 1, 131984},

{493, -1, 6, 1, 3, 131984},

{162, -1, 3, 4, 4, 223008},

{164, 0, 6, 6, 7, 245764},

{170, 0, 3, 4, 4, 200252},

{172, 0, 6, 6, 7, 245764},

{178, 0, 3, 4, 4, 200252},

{180, 1, 6, 5, 7, 143362},

{185, 0, 3, 5, 7, 223008},

{190, 0, 6, 6, 7, 245764},

{196, 0, 7, 3, 8, 131984},

{199, 1, 7, 3, 10, 143362},

{203, 0, 6, 5, 7, 234386},

{208, 0, 6, 3, 7, 131984},

{211, 0, 3, 9, 4, 359544},

{216, 0, 3, 6, 4, 245764},

{219, 0, 3, 5, 4, 223008},

{222, 0, 3, 6, 6, 245764},

{227, 0, 3, 5, 6, 234386},

{231, 0, 3, 4, 4, 177496},

{233, 0, 3, 4, 4, 179772},

{235, 0, 5, 3, 6, 177496},

{238, 0, 3, 6, 4, 245764},

{241, 0, 3, 5, 4, 234386},

{244, 0, 3, 8, 4, 314032},

{248, 0, 3, 5, 4, 234386},

{251, 0, 3, 5, 6, 234386},

{255, 0, 3, 3, 4, 200252},

{512, 0, 2, 5, 1, 223008},

{513, 0, 2, 12, 1, 446016},

{504, -1, 6, 3, 2, 223008},

{505, -1, 6, 3, 2, 223008},

{506, -1, 7, 3, 1, 223008},

};

char cmr5_bitmap_small[542] = {

24,24,24,36,60,36,231,249,20,95,69,31,128,61,24,96,

133,19,128,249,36,81,69,63,128,249,38,152,101,47,128,250,

86,229,35,128,62,66,130,143,130,66,62,205,36,158,73,44,

192,233,36,184,114,34,42,224,239,72,80,120,72,68,231,226,

16,132,167,224,231,102,102,114,90,90,255,221,166,154,89,110,

128,56,138,12,24,40,142,0,242,82,228,35,128,56,138,12,

24,46,142,0,6,240,145,35,132,137,59,128,120,134,17,224,

255,137,137,8,8,8,62,253,36,146,73,35,0,231,36,36,

36,24,24,24,231,116,98,41,66,148,41,65,8,16,128,230,

40,24,24,24,36,231,231,36,20,24,8,8,28,252,164,68,

167,224,238,191,193,4,30,69,23,128,104,150,24,32,158,138,

39,192,120,150,25,9,226,17,192,58,152,232,197,192,193,4,

30,73,46,192,64,12,151,32,6,73,60,194,16,246,51,96,

201,36,184,255,36,146,93,176,249,36,187,116,98,224,249,20,

94,67,128,116,164,225,28,244,78,110,30,75,182,192,217,36,

159,218,136,64,219,102,102,36,217,9,176,218,140,66,112,109,

240,250,82,132,35,128,16,80,162,36,80,127,128,56,138,12,

152,40,142,0,16,32,161,66,133,59,128,252,84,229,71,224,

253,36,146,73,44,192,255,22,76,37,31,192,238,0,64,129,

2,14,0,56,71,165,120,67,128,56,35,90,163,130,14,0,

122,24,97,135,60,192,86,219,80,89,36,184,84,148,248,212,

163,80,35,41,242,17,192,242,99,112,114,107,80,249,36,68,

64,86,171,80,86,178,112,75,229,250,64,48,161,67,126,151,

31,128,245,32,16,64,37,120,201,112,100,147,192,34,149,164,

167,64,255,34,75,96,119,196,34,46,224,11,171,87,64,63,

138,34,152,252,41,74,44,248,63,200,138,19,67,200,74,136,

143,240,4,113,52,153,50,89,28,64,242,158,20,43,249,66,

133,63,168,80,197,211,102,141,100,201,163,70,224,41,73,36,

72,128,137,18,73,74,0,39,254,64,16,32,64,143,226,4,

8,16,224,128,36,164,146,146,0,144,156,254,0,0,15,224,

56,251,30,60,111,207,128,234,170,172,213,85,92,224,144,96,

128,188,224,255,124,240,128,128,180,48,160,153,144,192,153,144,

248,255,240,33,8,66,16,159,32,39,200,66,16,132,32,6,

49,132,6,3,1,128,192,96,48,16,198,48,0,224,

};

METRIC cmr7_metric_small[128] = {

{476, -1, 9, 7, 10, 324041},

{485, -1, 9, 11, 10, 430537},

{499, -1, 9, 10, 10, 402320},

{512, 0, 9, 10, 10, 358630},

{525, -1, 9, 8, 10, 345886},

{535, -1, 9, 9, 10, 386847},

{547, -1, 9, 9, 10, 374103},

{559, -1, 9, 10, 10, 402320},

{572, -1, 9, 9, 10, 374103},

{584, -1, 9, 10, 10, 402320},

{597, -1, 9, 9, 10, 374103},

{944, 1, 9, 11, 10, 310843},

{958, 1, 9, 8, 10, 296734},

{968, 1, 9, 8, 10, 296734},

{978, 1, 9, 13, 10, 445101},

{995, 1, 9, 13, 10, 445101},

{707, 0, 5, 3, 6, 148367},

{710, 1, 5, 4, 9, 162476},

{927, -2, 9, 2, 3, 261235},

{928, -3, 9, 2, 3, 261235},

{929, -2, 8, 3, 2, 261235},

{930, -1, 9, 5, 3, 261235},

{932, -1, 7, 5, 1, 261235},

{933, -4, 9, 3, 3, 386847},

{935, -2, -1, 3, 2, 233018},

{715, -1, 9, 6, 10, 261236},

{723, -1, 5, 10, 6, 374103},

{731, -1, 5, 11, 6, 402320},

{740, -1, 7, 5, 9, 261235},

{746, 0, 9, 14, 10, 465126},

{764, -1, 9, 15, 10, 521560},

{783, -1, 10, 10, 12, 402320},

{936, 0, 4, 4, 1, 148367},

{798, -1, 9, 2, 10, 148367},

{1012, -1, 9, 5, 5, 261235},

{804, -1, 9, 11, 13, 430537},

{672, -1, 10, 5, 12, 261235},

{822, -1, 10, 11, 12, 430537},

{680, -1, 9, 10, 10, 402320},

{839, -1, 9, 2, 5, 148367},

{841, 0, 10, 4, 15, 204801},

{849, -1, 10, 4, 15, 204801},

{857, -1, 10, 5, 6, 261235},

{861, -1, 8, 10, 11, 402320},

{875, -1, 1, 2, 5, 148367},

{1016, 0, 3, 4, 1, 176584},

{877, -1, 1, 2, 2, 148367},

{878, -1, 10, 5, 15, 261235},

{609, -1, 8, 5, 9, 261235},

{615, -1, 8, 5, 9, 261235},

{621, -1, 8, 5, 9, 261235},

{627, -1, 8, 5, 9, 261235},

{633, 0, 8, 7, 9, 261235},

{641, -1, 8, 5, 9, 261235},

{647, -1, 8, 5, 9, 261235},

{653, -1, 8, 6, 9, 261235},

{660, -1, 8, 5, 9, 261235},

{666, -1, 8, 5, 9, 261235},

{888, -1, 5, 2, 6, 148367},

{890, -1, 5, 2, 9, 148367},

{801, -1, 6, 2, 10, 148367},

{893, -1, 5, 10, 5, 402320},

{700, -1, 6, 5, 10, 247127},

{693, -1, 9, 5, 10, 247127},

{900, -1, 9, 10, 10, 402320},

{0, 0, 9, 11, 10, 386847},

{14, -1, 9, 9, 10, 366367},

{26, -1, 9, 9, 10, 374103},

{38, -1, 9, 9, 10, 394584},

{50, -1, 9, 9, 10, 352258},

{62, -1, 9, 8, 10, 338150},

{72, -1, 9, 10, 10, 405506},

{85, -1, 9, 9, 10, 386847},

{97, 0, 9, 5, 10, 189328},

{104, -1, 9, 5, 10, 267607},

{111, -1, 9, 10, 10, 400956},

{124, -1, 9, 7, 10, 324041},

{133, -1, 9, 12, 10, 471498},

{148, -1, 9, 9, 10, 386847},

{160, -1, 9, 10, 10, 402320},

{173, -1, 9, 8, 10, 352258},

{183, -1, 9, 10, 13, 402320},

{200, -1, 9, 10, 10, 380475},

{213, -1, 9, 6, 10, 289452},

{221, -1, 9, 9, 10, 374103},

{233, -1, 9, 9, 10, 386847},

{245, 0, 9, 11, 10, 386847},

{259, 0, 9, 16, 10, 527932},

{279, 0, 9, 11, 10, 386847},

{293, 0, 9, 11, 10, 386847},

{307, -1, 9, 7, 10, 317669},

{913, -1, 10, 3, 15, 148367},

{1017, -1, 9, 5, 5, 261235},

{919, 0, 10, 3, 15, 148367},

{937, -2, 9, 3, 2, 261235},

{938, -1, 10, 2, 2, 148367},

{925, -1, 9, 2, 5, 148367},

{316, -1, 5, 6, 6, 261235},

{321, -1, 9, 6, 10, 289452},

{329, -1, 5, 5, 6, 233018},

{333, -1, 9, 6, 10, 289452},

{341, -1, 5, 4, 6, 233018},

{344, 1, 9, 6, 10, 162476},

{352, 0, 5, 7, 10, 261235},

{361, -1, 9, 6, 10, 289452},

{369, 0, 10, 3, 11, 148367},

{374, 1, 10, 4, 14, 162476},

{381, -1, 9, 7, 10, 275344},

{390, 0, 9, 3, 10, 148367},

{394, -1, 5, 11, 6, 430537},

{403, -1, 5, 6, 6, 289452},

{408, -1, 5, 5, 6, 261235},

{412, -1, 5, 6, 9, 289452},

{419, -1, 5, 6, 9, 275343},

{426, -1, 5, 4, 6, 204801},

{429, -1, 5, 4, 6, 207623},

{432, 0, 8, 4, 9, 204801},

{437, -1, 5, 6, 6, 289452},

{442, 0, 5, 8, 6, 275344},

{448, 0, 5, 11, 6, 374103},

{457, 0, 5, 8, 6, 275344},

{463, 0, 5, 8, 9, 275344},

{472, 0, 5, 5, 6, 233018},

{1021, 0, 3, 7, 1, 261235},

{1022, 0, 3, 16, 1, 522470},

{939, -2, 9, 4, 3, 261235},

{941, -1, 9, 5, 2, 261235},

{943, -2, 10, 3, 2, 261235},

};

char cmr7_bitmap_small[1024] = {

4,1,64,40,5,1,16,34,7,193,4,32,158,60,254,32,

208,40,23,242,9,2,129,65,255,0,30,144,208,48,24,4,

2,2,129,33,15,0,254,33,144,72,20,10,5,2,130,67,

127,0,255,33,145,72,167,194,33,18,129,65,127,128,255,67,

69,69,124,68,68,64,64,240,30,136,100,10,2,128,32,248,

9,2,32,135,224,227,160,144,72,39,242,9,4,130,65,113,

192,249,8,66,16,132,39,192,120,132,33,8,66,147,0,227,

208,132,65,32,88,25,4,33,8,65,56,240,240,129,2,4,

8,16,161,71,252,224,118,6,112,165,10,80,164,146,73,36,

162,70,46,103,195,176,152,74,36,146,73,20,138,67,112,128,

30,24,100,10,1,128,96,24,5,2,97,135,128,254,67,65,

65,67,126,64,64,64,224,30,24,100,10,1,128,96,24,5,

2,125,135,128,32,1,1,128,252,16,68,17,4,124,17,132,

33,8,66,56,112,126,24,32,112,32,65,135,224,255,196,98,

49,16,128,64,32,16,8,31,0,227,160,144,72,36,18,9,

4,130,34,14,0,241,228,16,130,8,129,16,34,2,128,80,

10,0,128,241,207,33,132,33,132,34,68,18,72,18,72,20,

40,12,48,12,48,8,16,243,194,32,72,5,0,64,28,2,

128,136,32,158,60,241,228,16,68,13,128,160,8,1,0,32,

4,1,192,255,138,20,65,2,8,161,71,252,224,71,36,149,

224,193,4,16,121,20,81,69,224,116,33,8,184,24,32,130,

122,40,162,137,240,127,136,150,12,65,4,248,65,4,16,224,

62,145,35,132,15,160,193,198,112,193,4,16,113,36,146,75,

176,216,1,146,75,128,51,0,3,17,17,17,30,192,129,2,

5,234,28,40,73,220,201,36,146,92,251,136,137,17,34,36,

69,204,192,241,36,146,75,176,116,99,24,184,249,20,81,69,

228,16,224,122,40,162,137,224,130,28,244,68,78,121,225,158,

34,111,34,51,48,217,36,146,72,240,231,36,36,36,24,24,

238,233,72,170,21,67,24,34,0,231,36,24,24,36,231,231,

36,36,44,24,24,16,16,224,122,156,212,252,254,133,10,20,

8,16,32,65,224,4,1,64,40,8,129,16,65,8,34,2,

192,127,252,30,24,100,10,73,158,100,152,5,2,97,135,128,

12,3,0,192,72,18,4,129,32,132,33,60,240,255,129,0,

66,126,66,0,0,129,255,255,160,144,72,36,18,9,4,130,

65,113,192,255,224,216,36,17,0,128,194,65,65,255,192,115,

162,16,0,16,4,1,0,64,16,4,7,192,62,4,15,137,

40,140,69,36,124,8,31,0,31,1,12,77,18,68,145,34,

80,120,4,7,192,62,32,160,48,24,12,5,4,130,162,177,

128,116,99,24,198,49,112,39,8,66,16,132,248,116,98,16,

136,137,248,116,66,19,6,49,112,24,48,162,76,159,194,4,

62,252,33,236,134,49,112,58,33,232,198,41,112,254,16,132,

16,130,8,32,116,98,167,70,49,112,116,163,24,188,34,224,

35,171,90,48,197,173,92,64,24,9,2,64,160,51,220,77,

34,40,132,94,224,116,68,66,0,0,99,0,99,0,0,33,

16,139,128,201,37,192,49,17,17,17,224,49,36,146,217,36,

81,71,96,247,143,252,130,32,140,92,224,59,192,254,16,66,

12,226,243,128,8,157,90,215,46,128,15,248,40,96,160,130,

138,18,32,127,129,34,72,137,34,11,223,224,31,252,66,25,

4,84,8,168,31,16,34,32,68,160,129,97,4,63,248,0,

135,198,57,10,132,97,24,134,33,80,156,99,225,0,255,192,

240,240,63,240,4,128,144,18,2,79,254,18,4,128,144,255,

228,128,144,18,2,64,96,83,50,90,72,137,32,200,1,48,

73,17,36,36,132,160,96,245,128,18,68,136,136,136,132,66,

16,132,34,17,17,17,18,36,128,37,127,250,144,4,1,0,

64,16,4,63,240,64,16,4,1,0,64,245,128,240,8,68,

33,8,132,34,16,132,66,0,240,240,240,245,128,255,192,0,

0,0,255,192,30,8,69,234,205,161,104,91,53,126,32,7,

240,242,73,36,146,73,56,228,146,73,36,146,120,107,192,148,

104,168,140,92,248,247,128,124,240,84,240,153,144,109,128,180,

15,226,32,68,8,143,252,34,4,64,136,17,6,48,14,16,

16,16,254,22,18,18,18,51,14,18,18,18,254,18,18,18,

18,51,15,240,136,4,64,34,15,255,8,152,68,66,34,17,

17,136,192,15,240,136,132,68,34,47,255,8,136,68,66,34,

17,17,136,192,222,210,153,0,240,76,165,189,128,254,255,255,



};

METRIC cmsl10_metric_small[128] = {

{984, 0, 13, 12, 14, 409601},

{1005, -1, 13, 14, 14, 546135},

{1030, -2, 13, 13, 14, 509726},

{1053, -1, 13, 12, 14, 455112},

{1074, -1, 13, 13, 14, 436908},

{1097, 0, 13, 17, 14, 491521},

{1127, -1, 13, 13, 14, 473317},

{1150, -3, 13, 13, 14, 509726},

{1173, -2, 13, 12, 14, 473317},

{1194, -3, 13, 13, 14, 509726},

{1217, -2, 13, 13, 14, 473317},

{1897, 0, 13, 14, 14, 382295},

{1922, 0, 13, 12, 14, 364090},

{1943, 0, 13, 11, 14, 364090},

{1963, 0, 13, 17, 14, 546135},

{1993, 0, 13, 16, 14, 546135},

{1438, 0, 8, 5, 9, 182045},

{1444, 1, 8, 5, 13, 200250},

{1874, -4, 13, 3, 4, 327681},

{1876, -6, 13, 3, 3, 327681},

{1878, -5, 12, 3, 2, 327681},

{1879, -4, 13, 5, 4, 327681},

{1882, -3, 11, 7, 1, 327681},

{1883, -8, 13, 4, 3, 529954},

{1885, -2, -1, 4, 4, 291272},

{1453, 0, 13, 10, 14, 327682},

{1471, -1, 8, 13, 9, 473317},

{1486, -2, 8, 13, 9, 509726},

{1501, -1, 10, 9, 13, 327681},

{1516, -1, 13, 17, 14, 591646},

{1546, -2, 13, 18, 14, 664464},

{1578, -2, 14, 13, 16, 509726},

{1887, -1, 7, 5, 2, 182045},

{1604, -2, 13, 4, 14, 182045},

{2021, -2, 13, 7, 6, 327681},

{1617, -2, 13, 14, 18, 546135},

{1370, -2, 15, 8, 17, 327681},

{1649, -3, 15, 13, 17, 546135},

{1387, -1, 13, 14, 14, 509726},

{1677, -3, 13, 3, 6, 182045},

{1680, -2, 15, 7, 21, 254863},

{1699, 0, 15, 7, 21, 254863},

{1718, -2, 15, 9, 9, 327681},

{1729, -2, 11, 13, 13, 509726},

{1751, -1, 1, 3, 6, 182045},

{2027, -1, 4, 5, 1, 218454},

{1754, -2, 1, 2, 2, 182045},

{1755, 0, 15, 11, 21, 327681},

{1240, -2, 12, 7, 13, 327681},

{1252, -2, 12, 6, 13, 327681},

{1262, -1, 12, 9, 13, 327681},

{1277, -1, 12, 9, 13, 327681},

{1292, -2, 12, 7, 13, 327681},

{1304, -1, 12, 8, 13, 327681},

{1317, -2, 12, 8, 13, 327681},

{1330, -3, 13, 8, 14, 327681},

{1344, -2, 12, 8, 13, 327681},

{1357, -1, 12, 8, 13, 327681},

{1784, -2, 8, 3, 9, 182045},

{1788, -1, 8, 4, 13, 182045},

{1611, -2, 9, 3, 14, 182045},

{1795, -2, 8, 13, 7, 509726},

{1425, -1, 9, 7, 14, 309477},

{1412, -3, 13, 7, 14, 309477},

{1807, -2, 13, 13, 14, 509726},

{0, -1, 13, 14, 14, 491521},

{25, 0, 13, 14, 14, 464215},

{50, -2, 13, 13, 14, 473317},

{73, 0, 13, 15, 14, 500624},

{100, 0, 13, 13, 14, 446010},

{123, 0, 13, 13, 14, 427806},

{146, -2, 13, 13, 14, 514277},

{169, 0, 13, 17, 14, 491521},

{199, 0, 13, 8, 14, 236658},

{213, -1, 13, 11, 14, 336783},

{233, 0, 13, 17, 14, 509726},

{263, 0, 13, 12, 14, 409601},

{284, 0, 13, 20, 14, 600748},

{319, 0, 13, 17, 14, 491521},

{349, -2, 13, 13, 14, 509726},

{372, 0, 13, 14, 14, 446010},

{397, -2, 13, 13, 18, 509726},

{427, 0, 13, 14, 14, 482419},

{452, -1, 13, 10, 14, 364090},

{470, -3, 13, 12, 14, 473317},

{491, -2, 13, 15, 14, 491521},

{518, -3, 13, 14, 14, 491521},

{543, -3, 13, 19, 14, 673566},

{577, 0, 13, 16, 14, 491521},

{605, -2, 13, 15, 14, 491521},

{632, -1, 13, 12, 14, 400499},

{1830, 0, 15, 8, 21, 182045},

{2028, -4, 13, 6, 6, 327681},

{1851, 1, 15, 8, 21, 182045},

{1889, -5, 13, 3, 3, 327681},

{1891, -4, 14, 2, 2, 182045},

{1872, -4, 13, 2, 6, 182045},

{653, -1, 8, 8, 9, 327681},

{662, -2, 13, 9, 14, 364090},

{678, -2, 8, 6, 9, 291272},

{685, -2, 13, 9, 14, 364090},

{701, -2, 8, 6, 9, 291272},

{708, 0, 13, 8, 14, 200250},

{722, -1, 8, 9, 14, 327681},

{738, 0, 13, 11, 14, 364090},

{758, 0, 14, 5, 15, 182045},

{768, 1, 14, 6, 19, 200250},

{783, 0, 13, 10, 14, 345886},

{801, 0, 13, 5, 14, 182045},

{810, 0, 8, 17, 9, 546135},

{830, 0, 8, 11, 9, 364090},

{843, -2, 8, 7, 9, 327681},

{851, 0, 8, 11, 13, 364090},

{869, -2, 8, 8, 13, 345885},

{882, 0, 8, 8, 9, 256684},

{891, -1, 8, 6, 9, 258504},

{898, -1, 12, 6, 13, 254863},

{908, -2, 8, 9, 9, 364090},

{919, -2, 8, 9, 9, 345886},

{930, -2, 8, 13, 9, 473317},

{945, 0, 8, 11, 9, 345886},

{958, 0, 8, 11, 13, 345886},

{976, -1, 8, 7, 9, 291272},

{2033, -1, 5, 9, 1, 327681},

{2035, -1, 5, 20, 1, 655362},

{1892, -4, 13, 5, 3, 327681},

{1894, -4, 13, 5, 2, 327681},

{1896, -5, 14, 4, 2, 327681},

};

char cmsl10_bitmap_small[2038] = {

0,128,3,0,28,0,112,2,192,11,0,76,1,24,8,96,

63,129,6,4,24,32,99,227,240,63,240,48,96,192,195,3,

12,8,96,97,255,6,14,24,28,96,49,128,204,6,48,51,

255,128,3,200,97,196,6,64,22,0,176,3,0,24,0,192,

6,0,144,8,192,67,12,7,128,63,240,24,24,48,16,96,

48,192,99,0,198,1,140,3,24,6,48,8,96,49,128,195,

3,31,248,0,63,248,96,195,2,24,16,196,140,64,126,3,

16,24,136,192,134,4,96,35,6,127,240,63,248,96,195,2,

24,16,194,140,32,127,3,8,24,64,192,6,0,96,3,0,

127,0,3,200,97,196,6,64,22,0,176,3,0,24,0,193,

254,1,144,12,192,195,6,7,208,63,31,134,3,3,1,129,

128,192,192,96,192,96,127,240,48,24,24,12,12,6,6,3,

6,3,3,1,135,227,240,63,12,12,12,12,24,24,24,24,

24,24,48,48,252,15,224,48,6,0,192,24,6,0,192,24,

3,0,96,12,99,12,96,240,0,63,31,134,6,3,4,1,

132,0,196,0,196,0,110,0,61,128,24,192,12,48,6,24,

6,6,3,3,7,227,224,63,128,192,12,0,192,12,1,128,

24,1,128,24,17,129,24,35,6,48,239,252,62,0,240,224,

28,14,2,192,176,44,11,4,193,48,88,19,9,129,24,152,

17,145,129,25,24,17,161,130,28,48,32,195,15,136,252,62,

15,135,1,2,192,129,96,64,152,32,142,32,67,16,33,200,

16,100,8,26,4,13,4,3,2,1,135,192,64,7,192,97,

140,4,64,54,1,160,15,0,120,3,192,30,1,176,12,192,

195,12,15,128,63,240,48,96,192,195,3,12,12,96,97,131,

7,248,24,0,96,1,128,12,0,48,3,240,0,7,192,97,

140,4,64,54,1,176,15,0,120,3,192,30,1,176,12,204,

195,156,15,128,4,0,50,1,224,6,0,63,224,48,192,193,

131,6,12,48,96,193,252,6,16,24,96,97,129,134,12,48,

48,231,241,224,15,70,51,4,193,48,14,1,240,62,1,192,

52,15,6,193,47,128,255,252,99,134,24,97,134,16,192,12,

0,192,12,0,192,12,1,128,24,15,240,252,62,96,16,192,

33,128,67,0,140,2,24,4,48,8,96,16,192,33,128,131,

1,3,12,3,224,0,248,124,192,195,2,12,8,48,64,195,

3,136,6,64,25,0,104,1,160,7,0,12,0,32,0,249,

251,236,28,17,131,132,24,112,131,22,48,98,196,12,152,129,

147,32,52,52,6,135,0,224,224,28,24,1,131,0,32,64,

0,31,159,6,8,3,16,3,48,1,160,1,192,1,192,1,

192,3,224,2,96,4,112,8,48,24,48,252,126,252,62,48,

48,96,64,97,0,198,0,200,1,160,1,128,3,0,6,0,

12,0,48,0,96,3,240,0,63,243,7,32,98,12,33,128,

48,7,0,96,12,33,130,56,99,4,96,207,252,28,2,2,

30,34,66,133,69,126,96,16,8,4,2,2,241,132,129,64,

160,80,88,44,37,224,61,8,32,130,8,34,112,1,128,64,

32,16,8,233,140,130,129,64,160,80,72,99,220,57,24,127,

130,8,50,112,7,8,8,8,8,124,16,16,16,16,16,32,

32,248,27,138,9,4,130,65,64,192,128,124,65,32,144,72,

67,192,24,1,0,32,4,0,128,47,6,16,130,16,66,8,

65,16,66,9,247,192,8,192,0,0,194,16,132,34,19,224,

4,48,0,0,1,130,8,32,130,16,65,4,16,140,0,24,

2,0,128,32,8,4,241,48,72,20,6,129,32,132,33,62,

240,24,66,16,136,66,16,132,66,124,55,158,12,48,132,16,

66,8,33,4,16,130,8,130,8,65,4,251,239,128,55,131,

8,65,8,33,4,32,136,33,4,251,224,60,138,12,24,48,

97,68,112,55,131,8,64,136,17,2,32,140,17,132,47,4,

0,128,16,15,0,29,99,65,129,129,131,130,134,122,2,2,

2,15,55,27,16,16,16,16,32,32,248,61,20,88,57,44,

178,240,16,67,12,252,130,8,32,148,146,48,195,32,144,72,

36,18,10,9,12,123,128,247,161,16,132,130,65,64,160,96,

48,0,231,58,49,17,136,148,68,148,24,160,198,6,48,33,

0,57,225,16,36,3,0,64,20,4,129,136,243,192,61,226,

16,66,4,128,144,20,2,128,96,12,1,0,32,8,14,0,

126,197,18,65,4,73,34,252,63,240,193,12,16,193,12,17,

128,24,1,128,24,1,128,24,3,0,48,15,224,0,192,3,

0,30,0,88,2,96,17,192,67,2,12,16,48,64,98,1,

152,6,127,255,255,240,7,192,97,140,4,64,54,1,180,79,

62,121,243,200,158,1,176,12,192,195,12,15,128,1,0,24,

3,128,56,5,128,88,9,128,152,9,129,24,17,194,12,32,

207,63,63,249,255,200,2,64,16,0,8,32,127,3,248,16,

64,0,16,5,0,79,254,127,240,63,255,134,3,3,1,129,

128,192,192,96,192,96,96,48,48,24,24,12,12,6,6,3,

6,3,3,1,135,227,240,63,249,192,198,2,24,16,224,131,

0,28,0,64,4,0,96,70,4,96,38,7,127,240,120,119,

231,225,66,10,0,112,3,0,24,0,192,6,0,48,1,128,

24,0,192,63,192,15,224,48,3,0,252,51,102,99,198,60,

99,198,38,108,31,128,192,12,7,248,15,224,24,0,193,198,

126,51,99,51,25,152,204,102,195,190,7,192,24,0,192,63,

192,7,192,195,12,12,192,54,1,224,25,0,204,4,96,99,

2,40,57,65,78,30,112,224,28,69,10,16,48,96,193,131,

10,20,71,0,9,224,130,16,65,4,16,66,8,252,14,8,

136,102,48,16,8,8,8,12,8,8,136,79,192,30,16,136,

96,32,48,224,8,6,3,33,177,136,131,128,4,24,80,162,

137,50,127,8,16,64,135,192,17,30,28,16,32,60,34,3,

3,3,134,68,56,14,17,32,64,64,184,196,130,130,130,132,

136,112,96,127,129,130,4,8,16,16,32,32,32,64,64,64,

28,2,65,65,98,124,60,76,134,130,132,136,112,14,17,33,

33,65,65,35,29,1,2,4,140,112,4,8,28,42,73,73,

72,112,120,28,22,18,146,164,168,112,32,3,0,18,0,72,

1,32,5,0,9,240,65,7,8,52,33,17,4,40,48,194,

67,136,243,192,125,6,8,32,130,8,16,0,0,0,12,24,

0,8,16,0,0,2,4,8,32,130,4,24,79,0,48,132,

33,8,132,248,24,66,16,132,66,16,132,76,0,3,129,16,

132,33,8,141,193,16,66,16,132,33,8,132,37,57,128,30,

112,12,64,66,63,242,16,32,130,12,8,114,60,224,28,115,

20,80,67,3,248,16,65,130,12,24,178,120,224,1,128,135,

132,68,82,73,36,162,114,50,30,8,8,0,3,255,128,176,

64,88,32,76,16,38,8,38,32,19,16,31,248,8,196,136,

98,132,48,68,48,34,24,103,159,240,7,255,195,12,19,3,

4,192,193,96,48,216,24,76,7,243,1,132,192,97,112,24,

46,6,9,131,2,48,195,7,255,192,0,8,62,131,12,96,

226,5,176,77,4,120,99,194,30,32,242,5,144,103,6,24,

97,252,8,0,19,51,34,34,0,0,204,36,0,146,91,109,

0,1,8,4,32,16,128,132,2,17,255,240,132,2,16,8,

64,33,0,132,63,254,33,0,132,4,32,16,128,66,2,16,

0,48,10,64,147,249,16,136,132,68,66,68,18,64,98,0,

35,130,20,17,17,8,144,137,4,72,20,128,224,44,149,0,

6,16,64,130,8,16,32,129,2,4,8,16,32,64,129,1,

2,2,0,8,16,16,16,32,64,129,2,4,8,16,64,129,

4,8,32,130,8,0,4,4,18,103,193,129,243,44,32,16,

0,1,0,8,0,64,4,0,32,1,3,255,224,64,2,0,

32,1,0,8,0,64,0,44,149,0,112,0,32,8,1,0,

64,8,2,0,64,16,2,0,128,16,4,0,128,16,4,0,

128,32,4,1,0,32,8,0,36,0,2,192,17,0,0,2,

98,36,128,127,248,0,0,0,0,0,0,0,3,255,192,7,

192,65,4,4,71,20,196,164,38,65,50,9,144,76,70,145,

216,128,2,3,15,224,15,16,16,16,16,16,16,32,32,32,

32,32,32,64,64,64,64,64,64,128,240,15,2,2,2,2,

2,2,4,4,4,4,4,4,8,8,8,8,8,8,16,240,

90,160,136,144,42,0,168,140,100,224,254,121,224,49,31,31,

0,86,128,240,74,100,109,128,187,3,220,17,128,132,2,16,

8,65,255,193,8,4,32,16,128,66,1,8,8,64,33,3,

135,0,3,224,65,8,0,128,8,7,252,16,193,4,16,65,

4,16,66,8,32,143,30,3,224,132,32,132,16,130,255,132,

16,130,16,66,8,65,16,66,9,227,192,3,239,2,8,66,

8,1,4,0,130,3,255,240,65,24,32,132,16,66,8,33,

4,16,132,16,130,8,71,142,120,3,239,4,17,8,33,8,

33,8,33,127,254,16,66,16,66,16,66,16,66,16,66,32,

132,32,132,241,207,34,204,137,20,81,0,248,69,24,162,138,

32,255,128,255,255,240,

};

METRIC cmsy10_metric_small[128] = {

{680, -2, 5, 11, 1, 509726},

{1086, -2, 6, 2, 2, 182045},

{1175, -3, 9, 9, 9, 509726},

{1186, -1, 9, 7, 9, 327681},

{1194, -1, 10, 14, 11, 509726},

{1087, 0, 9, 9, 9, 327681},

{682, -1, 13, 13, 13, 509726},

{1214, -1, 9, 13, 13, 509726},

{704, -1, 11, 13, 13, 509726},

{1236, -1, 11, 13, 13, 509726},

{726, -1, 11, 13, 13, 509726},

{1258, -1, 11, 13, 13, 509726},

{1280, -1, 11, 13, 13, 509726},

{1302, -1, 13, 18, 18, 655362},

{1343, -1, 8, 7, 7, 327681},

{1350, -1, 8, 7, 7, 327681},

{1357, -1, 10, 13, 11, 509726},

{1046, -1, 10, 13, 11, 509726},

{1375, -2, 12, 11, 18, 509726},

{1400, -2, 12, 11, 18, 509726},

{996, -2, 12, 11, 18, 509726},

{1021, -2, 12, 11, 18, 509726},

{1425, -2, 12, 11, 18, 509726},

{1450, -2, 12, 11, 18, 509726},

{1475, -1, 7, 13, 5, 509726},

{1484, -1, 9, 13, 10, 509726},

{900, -2, 11, 11, 13, 509726},

{918, -2, 11, 11, 13, 509726},

{1501, -1, 12, 18, 15, 655362},

{1535, -1, 12, 18, 15, 655362},

{1569, -2, 10, 11, 13, 509726},

{1587, -2, 10, 11, 13, 509726},

{852, -1, 8, 18, 7, 655362},

{868, -1, 8, 18, 7, 655362},

{836, -1, 13, 7, 18, 327681},

{820, -1, 13, 7, 18, 327681},

{884, -1, 8, 18, 7, 655362},

{1605, -1, 13, 18, 18, 655362},

{1646, -1, 13, 18, 18, 655362},

{1687, -1, 9, 13, 10, 509726},

{1704, -2, 10, 17, 11, 655362},

{1728, -1, 10, 17, 11, 655362},

{1752, 0, 12, 12, 17, 400499},

{1778, 0, 13, 12, 17, 400499},

{1804, -2, 10, 16, 11, 655362},

{1826, -1, 13, 18, 18, 655362},

{1867, -1, 13, 18, 18, 655362},

{1908, -1, 8, 13, 9, 509726},

{1923, -1, 11, 4, 11, 180224},

{1136, -1, 8, 18, 9, 655362},

{936, -2, 11, 9, 13, 436908},

{1929, -2, 11, 9, 13, 436908},

{1944, -1, 13, 15, 14, 582544},

{1971, -1, 9, 15, 14, 582544},

{1998, -3, 13, 9, 17, 0},

{2018, -1, 8, 2, 7, 0},

{951, 0, 13, 11, 14, 364090},

{971, -1, 13, 9, 14, 364090},

{987, -1, 7, 11, 6, 436908},

{2020, -1, 15, 7, 17, 327681},

{2035, -1, 13, 13, 14, 473317},

{2058, -1, 13, 12, 14, 473317},

{2079, -1, 12, 13, 13, 509726},

{2101, -1, 12, 13, 13, 509726},

{2123, -1, 13, 10, 14, 400499},

{0, 0, 14, 15, 16, 523286},

{30, -1, 13, 12, 14, 430447},

{51, 0, 13, 10, 14, 345066},

{69, -1, 13, 14, 14, 505540},

{94, -1, 13, 10, 14, 345887},

{112, 0, 13, 16, 14, 471039},

{140, -1, 13, 10, 16, 389851},

{160, -1, 13, 15, 14, 553463},

{187, 0, 13, 12, 14, 356854},

{208, -1, 13, 15, 16, 444190},

{238, -1, 13, 14, 14, 499352},

{263, -1, 13, 11, 14, 452018},

{283, -1, 13, 21, 15, 787022},

{323, 1, 15, 20, 17, 537717},

{366, -2, 13, 14, 14, 521741},

{391, -1, 13, 13, 15, 455842},

{416, -3, 13, 13, 16, 535213},

{442, -1, 13, 15, 14, 555420},

{469, -1, 13, 11, 14, 396859},

{489, -1, 14, 14, 15, 356938},

{516, 1, 13, 14, 14, 410145},

{541, -1, 13, 12, 15, 401593},

{564, -1, 13, 20, 15, 647354},

{602, -1, 13, 15, 14, 467466},

{629, -1, 13, 14, 16, 438001},

{657, -2, 13, 13, 14, 474956},

{748, -1, 12, 11, 13, 436908},

{766, -1, 12, 11, 13, 436908},

{2141, -1, 12, 11, 13, 436908},

{784, -1, 12, 11, 13, 436908},

{802, -1, 12, 11, 13, 436908},

{2159, -1, 13, 10, 14, 400499},

{2177, -2, 13, 9, 14, 400499},

{2193, -4, 15, 5, 21, 291272},

{2207, 1, 15, 5, 21, 291272},

{2221, -4, 15, 5, 21, 291272},

{2235, 1, 15, 5, 21, 291272},

{1098, -1, 15, 7, 21, 327681},

{1117, -1, 15, 7, 21, 327681},

{2249, -1, 15, 5, 21, 254863},

{2263, -1, 15, 5, 21, 254863},

{1083, -2, 15, 1, 21, 182045},

{2277, -1, 15, 7, 21, 327681},

{2296, -1, 15, 7, 21, 327681},

{2315, 0, 14, 12, 19, 400499},

{1064, -1, 15, 7, 21, 327681},

{2344, -1, 11, 3, 13, 182045},

{2349, -2, 0, 15, 21, 546135},

{2389, 0, 13, 15, 14, 491521},

{2416, -1, 13, 14, 14, 546135},

{1157, -1, 13, 8, 18, 273068},

{2441, -1, 12, 11, 13, 436908},

{2459, -1, 12, 11, 13, 436908},

{2477, -2, 12, 12, 18, 509726},

{2504, -1, 12, 12, 18, 509726},

{2531, -1, 13, 6, 18, 291273},

{2545, -1, 13, 7, 18, 291272},

{2561, -1, 13, 7, 18, 291272},

{2577, -1, 13, 10, 18, 400499},

{2600, -1, 13, 13, 17, 509726},

{2628, -1, 13, 13, 17, 509726},

{2656, -1, 13, 13, 14, 509726},

{2679, -1, 13, 13, 17, 509726},

};

char cmsy10_bitmap_small[2707] = {

0,8,0,48,0,224,1,192,7,128,11,0,38,0,76,1,

24,6,48,15,224,32,224,192,211,1,188,3,176,0,25,227,

167,28,49,194,24,65,152,55,195,14,48,99,6,32,102,4,

88,141,240,3,195,49,12,131,96,152,4,3,0,192,48,12,

3,130,113,15,128,31,224,153,228,97,225,131,6,12,24,48,

64,195,2,12,24,32,65,130,6,16,17,128,248,0,7,194,

49,8,192,48,14,1,224,64,32,16,12,3,4,226,31,0,

3,255,12,195,24,128,17,128,1,128,1,0,3,0,3,240,

2,0,6,0,4,0,12,0,200,0,240,0,7,194,49,12,

130,97,16,12,7,2,193,176,108,59,156,123,0,132,67,224,

30,2,76,13,24,20,48,96,96,192,129,3,254,6,12,12,

16,16,96,96,192,193,129,3,38,7,128,15,243,16,99,0,

48,2,0,96,6,0,96,4,0,192,12,0,136,17,143,240,

0,254,2,16,8,64,33,128,130,0,12,0,24,0,48,0,

64,1,128,3,1,4,4,24,28,32,24,128,62,0,24,28,

225,145,136,6,192,28,0,112,1,64,13,0,50,0,136,6,

16,24,65,64,139,1,192,1,224,76,17,134,0,192,16,6,

0,192,16,6,0,192,16,23,231,143,128,3,0,8,24,0,

192,192,14,7,0,112,88,5,130,192,104,22,6,192,184,102,

8,198,48,70,97,132,62,12,32,224,107,6,3,112,0,31,

0,0,0,0,0,112,0,15,3,0,128,48,16,3,1,0,

120,16,5,130,0,88,32,5,194,0,76,32,8,196,0,142,

64,8,100,1,7,128,144,56,15,3,128,96,0,0,4,240,

36,225,33,201,131,100,13,0,60,0,240,2,192,27,0,78,

3,24,24,112,128,252,0,31,225,51,145,143,12,48,97,131,

8,16,193,132,12,192,88,6,0,48,1,0,24,0,128,0,

3,224,99,132,14,64,54,1,160,15,0,120,2,192,54,1,

56,144,249,0,48,126,4,124,96,124,31,248,76,57,24,52,

48,96,96,128,194,1,8,6,224,13,192,17,128,99,128,195,

17,7,70,7,0,3,224,140,33,140,1,128,60,1,192,28,

1,200,27,3,96,78,48,248,0,0,4,255,228,24,48,64,

195,0,12,0,48,0,128,6,0,24,0,64,3,0,8,0,

64,7,224,0,60,5,48,48,192,130,6,24,24,64,227,3,

8,28,96,241,2,140,22,48,152,236,97,193,192,224,103,7,

48,19,129,24,17,130,24,97,132,24,129,152,27,1,96,60,

3,128,32,0,224,48,102,3,7,112,120,19,5,129,48,152,

19,9,130,49,28,35,48,196,50,12,195,64,200,60,13,3,

128,240,48,14,2,0,192,32,8,0,7,134,27,12,6,16,

12,64,25,0,60,0,96,1,192,13,128,35,0,134,2,12,

12,29,28,28,0,60,25,56,16,96,65,129,7,4,12,32,

48,128,196,3,32,12,128,52,0,224,3,2,8,8,192,28,

0,7,200,71,132,8,0,192,4,0,64,63,0,64,4,0,

64,4,12,64,71,198,71,224,255,224,2,0,16,0,128,4,

0,32,1,3,255,224,64,2,0,16,0,128,4,15,255,128,

7,0,214,8,136,132,36,33,65,7,255,240,65,66,18,16,

136,136,53,128,112,0,7,0,198,8,8,160,164,137,66,134,

8,48,161,72,146,130,136,8,49,128,112,0,128,48,6,0,

192,24,3,0,96,12,1,128,48,5,1,16,65,240,31,4,

17,1,64,24,3,0,96,12,1,128,48,6,0,192,24,2,

4,0,128,40,5,1,16,34,8,33,4,32,136,9,1,64,

24,2,128,48,5,1,32,34,8,65,8,32,136,17,1,64,

40,2,0,64,16,32,64,129,2,4,8,16,32,64,129,2,

36,170,56,32,16,113,84,145,2,4,8,16,32,64,129,2,

4,8,16,32,16,0,8,0,6,0,3,255,255,96,0,8,

0,1,0,0,0,2,0,0,64,0,27,255,255,0,1,128,

0,64,0,32,16,2,8,0,70,0,27,255,255,96,1,136,

0,65,0,32,31,228,1,0,32,8,1,0,32,4,0,128,

8,1,0,16,1,254,255,0,16,1,0,32,2,0,64,8,

1,0,32,8,1,0,79,240,31,144,16,8,8,4,3,255,

0,128,32,16,4,1,248,128,48,5,1,32,35,248,65,8,

32,136,17,2,32,40,5,0,64,8,0,255,128,64,32,16,

8,5,254,1,0,128,64,32,16,15,252,255,224,4,0,128,

16,2,0,64,0,32,24,12,2,1,128,192,32,3,0,24,

0,128,12,0,96,2,0,0,0,0,0,31,252,128,12,0,

96,2,0,48,1,128,8,6,3,0,128,96,48,8,0,0,

0,0,0,0,31,252,255,248,0,0,0,0,0,0,127,252,

0,0,0,0,0,0,63,254,129,1,2,4,8,8,16,32,

32,64,128,129,2,2,4,8,16,16,32,255,255,248,240,8,

10,8,136,40,10,8,136,40,8,0,6,16,64,129,2,4,

8,16,67,1,1,2,4,8,16,32,64,64,96,192,64,64,

129,2,4,8,16,16,24,65,2,4,8,16,32,65,12,0,

60,15,16,132,40,18,6,3,1,128,192,96,48,24,18,5,

8,66,60,15,0,2,5,4,4,4,4,8,8,8,8,8,

8,16,16,16,16,144,96,128,160,136,130,128,128,160,136,130,

128,128,16,34,75,225,15,164,136,16,3,0,12,0,0,0,

0,0,3,255,240,0,0,0,0,0,12,0,48,0,255,248,

16,0,128,4,0,32,1,3,255,224,64,2,0,16,0,128,

4,0,32,0,7,0,198,8,8,128,36,1,64,7,255,240,

1,64,18,0,136,8,49,128,112,0,7,0,198,8,8,128,

164,9,64,134,8,48,129,72,18,128,136,8,49,128,112,0,

7,0,198,8,8,128,36,1,67,134,28,48,225,64,18,0,

136,8,49,128,112,0,3,240,3,3,1,0,32,128,4,64,

0,144,0,40,0,6,0,1,128,0,96,0,24,0,6,0,

1,64,0,144,0,34,0,16,64,8,12,12,0,252,0,56,

138,12,24,40,142,0,56,251,255,255,239,142,0,128,10,0,

140,24,31,0,0,0,0,0,1,240,48,98,0,160,2,31,

228,1,0,32,8,1,0,32,4,0,128,8,1,0,16,1,

254,0,0,0,0,0,31,252,255,0,16,1,0,32,2,0,

64,8,1,0,32,8,1,0,79,240,0,0,0,0,0,31,

252,0,32,4,0,128,32,8,14,62,0,56,0,128,8,0,

128,16,2,0,0,0,0,0,31,252,128,16,2,0,32,2,

0,56,0,248,224,32,8,2,0,64,8,0,0,0,0,0,

0,31,252,56,10,32,96,131,2,40,14,0,56,10,32,96,

131,2,40,14,28,5,16,48,65,129,20,7,0,0,16,64,

24,96,8,32,12,48,12,48,4,16,6,24,2,8,0,97,

128,4,16,0,195,0,12,48,0,130,0,24,96,1,4,130,

0,24,96,1,4,0,48,192,3,12,0,32,128,6,24,0,

65,0,97,128,32,128,48,192,48,192,16,64,24,96,8,32,

0,0,32,4,0,128,32,8,14,62,0,56,0,128,8,0,

128,16,2,128,16,2,0,32,2,0,56,0,248,224,32,8,

2,0,64,8,0,0,7,192,0,48,0,20,0,9,0,4,

64,2,0,1,0,0,128,0,64,0,32,0,16,0,8,0,

4,0,2,0,1,0,0,128,0,64,0,32,0,0,128,0,

16,0,2,0,0,64,0,8,0,1,0,0,32,0,4,0,

0,128,0,16,0,2,0,0,64,0,8,0,1,16,0,36,

0,5,0,0,192,1,240,56,10,32,96,131,2,40,14,0,

0,0,0,0,0,7,255,192,4,0,4,0,7,255,230,0,

12,0,6,0,0,128,0,32,0,15,255,132,0,1,0,0,

0,16,0,4,63,255,0,0,192,0,24,0,12,0,8,0,

8,255,248,0,4,0,4,0,6,0,96,9,1,8,112,233,

9,16,129,8,16,129,8,16,129,8,16,129,8,16,129,8,

16,128,16,129,8,16,129,8,16,129,8,16,129,8,16,129,

8,16,137,9,112,225,8,9,0,96,6,0,8,16,16,8,

31,248,96,6,192,3,128,1,64,2,32,4,31,248,16,8,

8,16,248,0,48,0,10,0,2,64,0,136,0,1,0,0,

32,0,4,0,0,128,0,16,0,2,0,0,64,0,8,0,

1,0,0,32,0,4,0,0,128,0,16,0,0,64,0,32,

0,16,0,8,0,4,0,2,0,1,0,0,128,0,64,0,

32,0,16,0,8,0,4,0,34,0,9,0,2,128,0,192,

0,62,0,0,56,26,35,32,145,3,8,24,64,194,9,8,

196,56,24,51,54,102,68,200,128,252,1,0,64,32,8,7,

254,1,0,128,128,64,79,192,1,0,5,0,10,0,34,0,

68,1,4,2,8,8,8,16,16,64,16,128,34,0,36,0,

95,255,192,255,254,128,9,0,17,0,66,0,130,2,4,4,

4,16,8,32,8,128,17,0,20,0,40,0,32,0,0,128,

128,64,64,32,32,16,16,8,4,4,2,2,1,1,0,128,

128,0,171,168,4,8,242,96,145,98,201,147,38,141,26,36,

17,60,64,113,196,87,35,25,24,100,195,23,12,191,213,240,

78,192,102,3,49,153,136,141,184,48,60,4,48,129,152,14,

128,8,14,97,145,153,1,128,12,224,99,134,12,96,124,255,

248,16,0,128,4,0,32,1,0,8,0,64,2,0,16,0,

128,4,0,32,0,2,0,16,0,128,4,0,32,1,0,8,

0,64,2,0,16,0,128,4,15,255,128,130,48,204,25,135,

49,78,68,209,52,71,16,198,24,131,48,248,16,128,48,6,

0,194,24,67,8,111,236,33,132,48,133,1,16,65,240,128,

32,8,2,0,128,32,15,254,0,128,32,8,2,0,128,32,

0,0,128,64,32,16,8,7,254,1,0,128,64,32,16,8,

4,132,33,8,66,16,132,33,8,66,16,132,33,15,128,8,

66,16,132,33,8,66,16,132,33,8,66,31,128,252,33,8,

66,16,132,33,8,66,16,132,33,8,0,248,66,16,132,33,

8,66,16,132,33,8,66,16,128,8,68,34,16,136,68,33,

4,32,132,32,132,16,128,132,16,130,16,130,16,66,17,8,

132,34,17,8,0,131,6,12,24,48,96,193,131,6,12,24,

48,96,193,131,6,12,24,32,16,113,84,145,2,4,8,16,

32,64,129,2,4,8,17,37,81,193,0,6,0,144,16,135,

14,144,145,8,16,129,8,16,129,8,16,129,8,16,129,8,

144,151,14,16,128,144,6,0,196,146,146,146,70,0,2,0,

4,0,16,0,32,0,128,1,0,4,0,8,0,32,0,64,

1,2,2,10,4,4,16,4,32,8,128,17,0,20,0,40,

0,96,0,64,0,252,126,96,48,192,97,128,195,1,134,3,

12,6,24,12,48,24,96,48,192,97,128,195,1,159,255,192,

255,253,255,230,0,140,4,48,16,224,193,130,7,8,12,64,

57,0,104,1,160,3,0,12,0,128,48,6,0,192,24,3,

0,96,12,1,128,48,6,0,192,31,254,255,240,6,0,192,

24,3,0,96,12,1,128,48,6,0,192,24,2,255,248,0,

128,8,0,128,8,0,128,8,0,128,8,0,128,8,0,255,

240,0,0,0,0,0,15,254,255,240,1,0,16,1,0,16,

1,0,16,1,0,16,1,0,16,1,127,240,0,0,0,0,

0,7,255,49,40,160,129,135,34,134,20,78,24,16,81,72,

192,16,32,64,129,31,196,8,16,32,64,129,2,4,8,16,

32,16,32,64,143,226,4,8,16,0,64,129,15,255,136,16,

32,63,222,79,147,228,249,62,79,147,228,121,14,64,144,36,

9,2,64,144,36,9,2,64,7,0,124,7,240,63,129,252,

7,192,62,15,254,255,255,255,255,255,255,247,175,25,48,8,

0,224,7,0,2,0,40,1,64,17,0,136,8,32,128,136,

2,128,10,0,136,8,32,128,136,4,64,20,0,160,2,0,

56,226,40,160,131,4,24,0,192,5,0,72,2,32,32,130,

2,32,10,0,32,1,0,2,0,56,1,192,31,1,252,31,

241,255,207,254,255,255,255,255,255,255,255,175,185,56,8,0,

224,7,0,

};

METRIC cmsy5_metric_small[128] = {

{205, -2, 2, 7, 1, 354992},

{350, -2, 2, 1, 1, 150188},

{380, -3, 5, 5, 7, 354992},

{385, -2, 4, 4, 4, 241212},

{387, -2, 4, 7, 5, 354992},

{351, -1, 5, 7, 7, 241212},

{206, -2, 7, 7, 9, 354992},

{392, -2, 5, 7, 9, 354992},

{214, -2, 6, 7, 9, 354992},

{400, -2, 6, 7, 9, 354992},

{222, -2, 6, 7, 9, 354992},

{408, -2, 6, 7, 9, 354992},

{416, -2, 6, 7, 9, 354992},

{424, -2, 6, 10, 9, 446016},

{436, -2, 4, 4, 5, 241212},

{439, -2, 4, 4, 5, 241212},

{442, -2, 5, 7, 7, 354992},

{335, -2, 5, 7, 7, 354992},

{449, -2, 6, 7, 11, 354992},

{459, -2, 6, 7, 11, 354992},

{315, -2, 6, 7, 11, 354992},

{325, -2, 6, 7, 11, 354992},

{469, -2, 6, 7, 11, 354992},

{479, -2, 6, 7, 11, 354992},

{489, -2, 3, 7, 3, 354992},

{492, -2, 5, 7, 6, 354992},

{278, -2, 6, 7, 9, 354992},

{286, -2, 6, 7, 9, 354992},

{498, -2, 6, 10, 9, 446016},

{510, -2, 6, 10, 9, 446016},

{522, -2, 5, 7, 8, 354992},

{529, -2, 5, 7, 8, 354992},

{266, -2, 3, 10, 3, 446016},

{270, -2, 3, 10, 3, 446016},

{260, -2, 6, 5, 9, 241212},

{254, -2, 6, 5, 9, 241212},

{274, -2, 3, 10, 3, 446016},

{536, -2, 6, 10, 9, 446016},

{548, -2, 6, 10, 9, 446016},

{560, -2, 4, 7, 6, 354992},

{566, -3, 5, 9, 7, 446016},

{574, -2, 5, 9, 7, 446016},

{582, -1, 5, 8, 8, 286724},

{590, -1, 6, 8, 8, 286724},

{598, -3, 5, 8, 7, 446016},

{605, -2, 6, 10, 9, 446016},

{617, -2, 6, 10, 9, 446016},

{629, -2, 3, 7, 4, 354992},

{633, -1, 5, 3, 6, 144499},

{370, -2, 3, 10, 4, 446016},

{294, -2, 6, 6, 9, 309480},

{636, -2, 6, 6, 9, 309480},

{643, -2, 6, 9, 7, 400504},

{651, -2, 4, 9, 7, 400504},

{659, -3, 6, 5, 9, 0},

{665, -2, 4, 1, 5, 0},

{301, -1, 6, 7, 7, 263968},

{308, -2, 6, 4, 7, 263968},

{312, -2, 3, 6, 3, 309480},

{666, -2, 7, 4, 9, 241212},

{671, -1, 6, 9, 7, 332236},

{679, -2, 6, 8, 7, 332236},

{686, -2, 6, 7, 7, 354992},

{693, -2, 6, 7, 7, 354992},

{700, -2, 6, 5, 7, 286724},

{0, -1, 6, 9, 7, 364867},

{8, -1, 6, 7, 7, 301242},

{15, -1, 6, 5, 7, 256276},

{20, -2, 6, 8, 7, 355175},

{27, -1, 6, 6, 7, 252590},

{33, -1, 6, 10, 7, 323815},

{42, -2, 6, 6, 8, 278669},

{48, -2, 6, 9, 7, 391425},

{56, -1, 6, 7, 7, 268543},

{63, -1, 6, 10, 8, 314031},

{73, -1, 6, 8, 7, 345709},

{80, -1, 6, 7, 7, 316125},

{87, -1, 6, 13, 7, 513757},

{99, 0, 7, 12, 8, 368987},

{111, -2, 6, 8, 7, 356903},

{118, -2, 6, 8, 8, 326912},

{126, -2, 6, 8, 8, 370921},

{134, -2, 6, 9, 7, 391949},

{142, -1, 6, 7, 7, 284448},

{149, -1, 6, 8, 7, 269495},

{156, -1, 6, 8, 7, 286313},

{163, -1, 6, 7, 7, 294553},

{170, -1, 6, 12, 7, 448156},

{181, -1, 6, 9, 7, 326579},

{189, -1, 6, 8, 8, 304565},

{197, -1, 6, 9, 7, 324863},

{230, -2, 6, 6, 7, 309480},

{236, -2, 6, 6, 7, 309480},

{705, -2, 6, 6, 7, 309480},

{242, -2, 6, 6, 7, 309480},

{248, -2, 6, 6, 7, 309480},

{711, -2, 6, 5, 7, 286724},

{716, -2, 6, 5, 7, 286724},

{721, -3, 7, 4, 11, 218456},

{727, 0, 7, 4, 11, 218456},

{733, -3, 7, 4, 11, 218456},

{739, 0, 7, 4, 11, 218456},

{358, -2, 7, 4, 11, 241212},

{364, -2, 7, 4, 11, 241212},

{745, -1, 7, 3, 11, 195700},

{750, -2, 7, 3, 11, 195700},

{348, -2, 7, 1, 11, 150188},

{755, -2, 7, 4, 11, 241212},

{761, -2, 7, 5, 11, 241212},

{768, -1, 6, 8, 9, 286724},

{342, -2, 7, 4, 11, 241212},

{777, -2, 5, 1, 7, 150188},

{778, -2, 0, 10, 11, 359544},

{792, -2, 6, 7, 7, 339518},

{799, -2, 6, 8, 7, 377748},

{375, -2, 6, 4, 9, 207078},

{806, -2, 6, 6, 7, 309480},

{812, -2, 6, 6, 7, 309480},

{818, -2, 6, 8, 11, 354992},

{829, -1, 6, 8, 11, 354992},

{840, -2, 6, 3, 9, 218456},

{844, -2, 6, 3, 9, 218456},

{848, -2, 6, 3, 9, 218456},

{852, -2, 6, 6, 9, 286724},

{859, -1, 6, 9, 8, 354992},

{868, -3, 6, 6, 8, 354992},

{874, -2, 6, 7, 7, 354992},

{881, -2, 6, 7, 8, 354992},

};

char cmsy5_bitmap_small[888] = {

2,3,2,129,65,241,11,6,108,101,178,228,40,127,0,58,

99,8,69,192,126,145,17,17,34,38,120,28,146,12,66,39,

0,31,202,0,128,60,16,4,14,0,61,24,33,141,224,188,

112,200,132,71,194,33,17,14,126,16,64,129,2,63,0,15,

196,65,32,8,2,1,8,65,224,103,56,32,96,80,73,134,

12,36,129,2,8,127,128,8,8,64,198,12,40,161,73,17,

139,128,96,0,49,4,24,65,136,20,130,72,35,14,16,62,

81,129,129,130,70,56,126,145,33,34,60,32,64,64,30,97,

129,129,130,124,240,15,126,72,132,195,130,65,33,14,30,68,

128,228,48,95,0,127,136,8,8,16,16,120,193,66,66,134,

140,148,102,198,68,137,34,133,12,0,194,50,49,37,18,82,

40,163,12,32,128,49,132,195,129,3,134,67,24,98,17,17,

10,10,12,152,96,31,144,129,131,193,3,11,248,254,16,32,

64,143,226,4,8,254,56,170,76,159,242,100,170,56,56,138,

157,89,53,105,162,56,134,24,97,134,23,128,122,24,97,134,

24,64,48,195,18,74,24,64,134,20,146,48,195,0,33,8,

66,16,159,32,39,200,66,16,132,32,64,63,244,0,0,191,

240,8,64,191,244,8,62,130,4,8,16,32,32,62,248,8,

8,16,32,64,130,248,61,8,32,254,8,16,60,131,5,242,

34,133,4,0,241,31,17,240,252,16,64,2,24,198,12,6,

3,1,0,3,248,128,192,96,48,99,24,64,0,3,248,254,

0,7,240,0,63,128,136,68,68,34,33,16,255,224,128,16,

81,20,20,69,4,0,18,34,44,34,34,16,194,34,33,34,

34,192,115,163,24,197,206,17,34,34,44,192,138,148,69,42,

32,47,242,16,3,248,1,0,254,32,64,143,226,4,8,16,

56,138,12,31,240,96,162,56,56,138,28,89,52,104,162,56,

56,138,12,25,48,96,162,56,63,16,40,6,1,128,96,24,

5,2,63,0,105,153,96,111,255,96,130,136,224,3,136,160,

128,62,130,4,8,16,16,31,0,3,248,248,8,8,16,32,

65,124,0,3,248,2,4,16,207,1,128,129,0,3,248,129,

1,1,129,236,32,64,0,3,248,99,38,48,99,38,51,25,

49,128,2,67,97,33,176,144,27,1,32,54,2,64,144,27,

1,32,54,2,67,97,33,176,144,0,2,4,16,207,1,128,

129,129,1,1,129,236,32,64,1,192,48,52,24,12,6,2,

1,0,128,0,128,16,2,0,96,12,1,128,52,3,1,192,

99,38,48,0,31,192,16,31,208,16,4,1,252,64,4,127,

0,64,16,23,240,16,24,102,165,36,36,36,36,36,36,36,

36,36,36,165,102,24,36,126,195,129,66,126,36,224,48,11,

0,96,12,1,128,16,2,0,64,0,64,32,16,24,12,6,

11,3,0,224,0,119,18,35,176,37,41,0,240,32,65,252,

16,66,240,8,10,5,4,68,18,11,254,255,160,144,68,65,

64,160,32,8,132,66,17,8,128,248,23,187,221,222,128,118,

36,138,45,230,166,81,206,124,131,128,99,3,194,60,254,32,

64,129,2,4,0,16,32,64,129,2,63,128,146,183,105,43,

32,134,25,109,134,23,128,132,33,248,66,0,8,67,240,132,

32,136,136,136,136,136,240,17,17,17,17,17,240,248,136,136,

136,136,128,241,17,17,17,17,16,37,41,34,68,128,145,34,

74,82,0,153,153,153,153,153,144,39,200,66,16,132,39,200,

60,231,36,36,36,36,36,231,60,254,0,64,32,8,4,1,

0,140,65,16,40,10,1,0,198,137,18,36,72,191,128,255,

195,66,102,36,24,24,134,24,97,134,31,192,254,24,97,134,

24,64,255,128,128,128,128,128,128,255,0,0,254,255,1,1,

1,1,1,1,127,0,0,127,86,123,205,64,73,116,146,64,

93,32,159,64,127,174,186,104,162,138,40,28,31,15,139,175,

255,253,172,16,33,69,35,137,69,8,109,38,12,20,69,4,

0,16,113,247,255,255,245,136,

};

METRIC cmsy7_metric_small[128] = {

{374, -1, 3, 10, 1, 409602},

{624, -1, 4, 2, 2, 155649},

{677, -2, 7, 8, 9, 409602},

{686, -1, 6, 6, 6, 268517},

{691, -1, 7, 10, 9, 409602},

{625, -1, 6, 7, 7, 268517},

{376, -1, 9, 10, 11, 409602},

{703, -1, 7, 10, 11, 409602},

{390, -1, 8, 10, 11, 409602},

{717, -1, 8, 10, 11, 409602},

{404, -1, 8, 10, 11, 409602},

{731, -1, 8, 10, 11, 409602},

{745, -1, 8, 10, 11, 409602},

{759, -1, 9, 14, 13, 522470},

{782, -1, 6, 6, 7, 268517},

{788, -1, 6, 6, 7, 268517},

{794, -1, 7, 10, 9, 409602},

{598, -1, 7, 10, 9, 409602},

{806, -1, 9, 10, 14, 409602},

{824, -1, 9, 10, 14, 409602},

{562, -1, 9, 10, 14, 409602},

{580, -1, 9, 10, 14, 409602},

{842, -1, 9, 10, 14, 409602},

{860, -1, 9, 10, 14, 409602},

{878, -1, 5, 10, 4, 409602},

{883, -1, 6, 10, 8, 409602},

{499, -1, 8, 10, 11, 409602},

{513, -1, 8, 10, 11, 409602},

{893, -1, 8, 14, 11, 522470},

{913, -1, 8, 14, 11, 522470},

{933, -1, 7, 10, 10, 409602},

{946, -1, 7, 10, 10, 409602},

{472, -1, 5, 14, 5, 522470},

{481, -1, 5, 14, 5, 522470},

{463, -2, 9, 5, 13, 268517},

{454, -2, 9, 5, 13, 268517},

{490, -1, 5, 14, 5, 522470},

{959, -1, 9, 14, 14, 522470},

{984, -1, 10, 14, 14, 522470},

{1009, -1, 6, 10, 8, 409602},

{1019, -2, 7, 13, 9, 522470},

{1034, -1, 7, 13, 9, 522470},

{1049, 0, 8, 10, 12, 324951},

{1064, 0, 9, 10, 12, 324951},

{1079, -2, 7, 12, 9, 522470},

{1093, -1, 9, 14, 14, 522470},

{1118, -1, 10, 14, 14, 522470},

{1143, -1, 5, 10, 6, 409602},

{1151, 0, 7, 4, 8, 151097},

{656, -1, 5, 14, 6, 522470},

{527, -1, 8, 8, 11, 353168},

{1155, -1, 8, 8, 11, 353168},

{1166, -1, 9, 13, 10, 466036},

{1183, -1, 6, 13, 10, 466036},

{1200, -2, 9, 8, 13, 0},

{1213, -1, 5, 1, 5, 0},

{538, 0, 9, 9, 10, 296734},

{550, -1, 9, 6, 10, 296734},

{558, -1, 4, 8, 4, 353168},

{1214, -1, 10, 6, 12, 268517},

{1223, -1, 9, 10, 10, 381385},

{1236, -1, 9, 10, 10, 381385},

{1249, -1, 9, 11, 10, 409602},

{1263, -1, 9, 11, 10, 409602},

{1277, -1, 9, 7, 10, 324951},

{0, 0, 9, 12, 10, 420952},

{15, 0, 9, 9, 10, 345641},

{27, 0, 9, 8, 10, 284511},

{37, 0, 9, 11, 10, 408037},

{51, -1, 9, 7, 10, 282627},

{60, -1, 9, 11, 10, 375422},

{74, 0, 9, 9, 11, 315859},

{87, 0, 9, 12, 10, 448955},

{102, 1, 9, 10, 10, 296584},

{115, 0, 9, 12, 11, 358812},

{132, 0, 9, 10, 10, 399884},

{145, 0, 9, 9, 10, 363200},

{157, 0, 9, 17, 10, 615961},

{179, 0, 10, 15, 11, 429195},

{200, -1, 9, 11, 10, 415555},

{214, 0, 9, 11, 11, 371200},

{230, -1, 9, 10, 11, 429355},

{244, 0, 9, 12, 10, 450052},

{259, -1, 9, 9, 10, 322130},

{271, -1, 9, 11, 10, 297189},

{285, 0, 9, 10, 10, 329292},

{298, -1, 9, 10, 10, 330268},

{311, -1, 9, 16, 10, 520733},

{331, 0, 9, 12, 10, 375652},

{346, -1, 9, 10, 11, 350657},

{360, 0, 9, 11, 10, 377618},

{418, -1, 8, 8, 9, 353168},

{427, -1, 8, 8, 9, 353168},

{1286, -1, 8, 8, 9, 353168},

{436, -1, 8, 8, 9, 353168},

{445, -1, 8, 8, 9, 353168},

{1295, -1, 9, 7, 10, 324951},

{1304, -2, 9, 6, 10, 324951},

{1312, -3, 10, 4, 15, 240300},

{1320, 0, 10, 4, 15, 240300},

{1328, -3, 10, 4, 15, 240300},

{1336, 0, 10, 4, 15, 240300},

{632, -1, 10, 6, 15, 268517},

{644, -1, 10, 6, 15, 268517},

{1344, -1, 10, 4, 15, 212083},

{1352, -1, 10, 4, 15, 212083},

{622, -2, 10, 1, 15, 155649},

{1360, -1, 10, 6, 15, 268517},

{1372, -2, 10, 5, 15, 268517},

{1382, 0, 9, 10, 13, 324951},

{610, -1, 10, 6, 15, 268517},

{1399, -1, 7, 2, 9, 155649},

{1402, -1, 0, 13, 15, 430537},

{1427, -1, 9, 9, 10, 394129},

{1439, -1, 9, 11, 10, 437819},

{667, -1, 9, 6, 13, 226420},

{1453, -1, 8, 8, 9, 353168},

{1462, -1, 8, 8, 9, 353168},

{1471, -1, 9, 11, 14, 409602},

{1491, 0, 9, 11, 14, 409602},

{1511, -1, 9, 5, 13, 240301},

{1520, -1, 9, 5, 13, 240300},

{1529, -1, 9, 5, 13, 240300},

{1538, -1, 9, 8, 13, 324951},

{1551, 0, 9, 13, 12, 409602},

{1571, -2, 9, 10, 12, 409602},

{1586, -1, 9, 11, 10, 409602},

{1600, -1, 9, 11, 12, 409602},

};

char cmsy7_bitmap_small[1617] = {

0,64,20,1,64,36,2,64,68,15,193,4,16,46,3,39,

20,76,36,98,241,5,2,129,65,95,0,15,49,66,64,128,

128,128,128,66,60,63,137,10,32,132,16,130,16,68,16,132,

17,7,192,30,69,2,3,140,32,64,132,240,31,228,133,16,

2,0,128,31,4,0,128,32,28,0,15,24,80,72,8,4,

10,4,140,58,2,62,0,56,20,129,136,32,130,15,225,4,

16,65,4,32,66,3,63,208,128,64,16,4,1,0,128,32,

17,63,192,3,240,68,8,64,8,0,128,8,1,0,16,1,

8,32,124,0,33,200,146,192,192,48,20,4,129,16,132,96,

224,7,4,66,2,1,1,0,128,64,120,195,128,4,0,130,

0,131,0,193,128,160,160,144,144,144,68,136,34,132,32,130,

96,1,128,0,6,16,16,32,64,96,129,65,2,68,4,136,

8,144,33,32,65,131,1,0,11,134,137,32,164,24,3,0,

96,20,4,65,7,192,63,137,10,32,132,16,132,17,5,192,

128,16,4,0,128,0,15,12,36,5,1,128,96,24,9,20,

62,30,0,124,63,196,130,136,32,130,8,192,176,17,1,16,

16,146,6,15,8,72,4,1,192,18,5,2,130,62,0,63,

232,66,16,2,0,64,8,2,0,64,16,15,128,96,72,18,

5,2,65,144,104,50,20,153,24,96,193,136,18,4,129,16,

132,65,32,80,40,12,0,192,130,64,193,33,65,33,65,34,

66,36,36,40,36,40,40,48,48,32,32,28,49,65,2,32,

44,3,0,224,50,4,16,129,12,28,225,132,17,4,65,8,

130,64,144,40,4,38,6,0,15,226,8,2,0,128,248,8,

2,0,129,96,95,240,255,192,4,1,0,64,16,4,63,240,

64,16,4,1,15,252,30,9,68,74,17,132,127,248,70,17,

68,137,65,224,30,8,70,26,73,140,98,24,198,73,97,136,

65,224,129,129,129,129,129,129,129,66,60,60,66,129,129,129,

129,129,129,129,24,24,24,36,36,66,66,129,129,129,129,66,

66,36,36,24,24,24,33,8,66,16,132,33,42,226,0,35,

170,66,16,132,33,8,66,0,32,1,0,15,255,208,0,32,

0,0,16,0,47,255,192,2,0,16,32,17,0,47,255,208,

2,32,16,31,200,4,2,0,128,32,8,2,0,64,8,1,

252,254,0,64,8,1,0,64,16,4,1,0,128,79,224,31,

32,64,128,128,255,128,128,64,32,31,128,192,80,79,226,33,

16,80,40,8,4,0,252,16,65,124,16,65,7,240,255,1,

1,1,0,64,96,96,96,96,48,3,0,48,3,0,48,0,

0,0,63,240,128,24,1,128,24,1,128,48,48,48,48,48,

0,0,0,0,63,240,255,192,0,0,0,255,192,0,0,0,

255,192,130,4,16,64,130,4,16,64,130,8,16,64,255,254,

240,16,81,20,20,69,4,0,12,65,4,16,65,56,16,65,

4,16,64,192,224,65,4,16,65,3,16,65,4,16,78,0,

56,113,18,40,48,96,193,68,136,225,192,12,48,130,16,65,

4,16,130,40,192,129,66,36,24,16,24,36,66,129,18,87,

158,148,64,12,3,0,0,0,255,192,0,0,48,12,0,255,

193,0,64,16,4,63,240,64,16,4,1,0,64,30,8,68,

10,1,128,127,248,6,1,64,136,65,224,30,8,68,26,9,

132,98,24,134,65,96,136,65,224,30,8,68,10,1,140,99,

24,198,1,64,136,65,224,7,128,97,130,1,16,2,64,10,

0,24,0,96,1,64,9,0,34,1,6,24,7,128,49,40,

97,133,35,0,49,239,255,253,227,0,128,88,97,224,0,0,

0,1,225,134,128,64,31,200,4,2,0,128,32,8,1,0,

32,7,240,0,0,0,63,240,254,0,64,8,1,0,64,16,

4,2,1,63,128,0,0,0,63,240,0,64,16,4,2,3,

63,0,112,2,0,64,16,0,0,0,63,240,128,32,8,1,

0,48,3,243,129,0,128,32,0,0,0,0,63,240,112,98,

24,70,14,112,98,24,70,14,112,98,24,70,14,0,132,12,

96,66,6,48,99,2,16,6,48,4,32,12,96,12,96,8,

64,132,1,140,1,8,3,24,3,24,2,16,49,129,8,24,

193,140,8,64,0,0,64,16,4,2,3,63,0,112,2,0,

64,16,128,32,8,1,0,48,3,243,129,0,128,32,0,0,

60,0,48,1,64,9,0,64,2,0,16,0,128,4,0,32,

1,0,8,0,64,2,0,0,128,1,0,2,0,4,0,8,

0,16,0,32,0,64,0,128,1,0,2,64,5,0,12,0,

240,112,98,24,70,14,0,0,0,3,255,8,0,128,15,255,

128,8,0,48,0,255,226,0,8,0,0,128,2,63,248,0,

48,0,128,27,255,128,8,0,128,12,4,130,19,135,33,8,

66,16,132,33,8,66,16,132,33,8,66,16,132,33,8,66,

16,132,225,200,65,32,48,16,130,4,127,236,3,128,20,2,

63,194,4,16,128,240,3,0,10,0,36,0,8,0,16,0,

32,0,64,0,128,1,0,2,0,4,0,8,0,16,0,4,

0,32,1,0,8,0,64,2,0,16,0,128,4,0,32,9,

0,40,0,192,3,192,0,112,226,72,98,24,137,28,48,51,

102,68,136,248,4,2,1,1,255,1,1,2,4,248,2,0,

40,1,64,17,1,4,8,32,128,136,2,64,23,255,192,255,

250,0,144,4,64,65,4,8,32,34,0,160,5,0,16,0,

1,2,2,4,4,8,16,16,32,32,64,64,128,248,9,229,

165,150,90,105,165,167,144,102,38,72,137,33,79,178,128,160,

40,146,56,112,62,16,88,14,0,64,204,80,19,4,49,3,

128,255,224,128,16,2,0,64,8,1,0,32,4,0,128,4,

0,128,16,2,0,64,8,1,0,32,4,31,252,136,137,19,

57,83,34,34,69,132,129,129,137,137,189,137,129,66,60,129,

2,4,15,240,32,64,129,0,4,16,65,252,16,65,4,16,

136,136,136,136,136,136,136,240,17,17,17,17,17,17,17,240,

248,136,136,136,136,136,136,128,241,17,17,17,17,17,17,16,

17,34,68,136,132,66,33,16,136,68,34,17,18,36,72,128,

134,24,97,134,24,97,134,24,97,134,24,64,35,170,66,16,

132,33,8,74,184,128,12,12,206,28,132,33,8,66,16,132,

33,8,78,28,204,12,0,149,106,64,0,8,0,128,4,0,

64,2,0,32,1,0,16,193,2,8,8,128,68,1,64,10,

0,32,0,227,160,144,72,36,18,9,4,130,65,127,192,255,

248,13,1,16,66,8,34,4,64,80,10,0,128,129,129,129,

129,129,129,129,129,255,255,129,129,129,129,129,129,129,129,255,

240,2,0,64,8,1,0,32,4,0,128,31,252,0,0,0,

1,255,128,255,224,4,0,128,16,2,0,64,8,1,0,47,

252,0,0,0,0,255,192,116,97,7,70,49,112,67,23,0,

33,8,79,144,132,33,8,66,0,33,62,66,16,4,35,254,

66,0,63,114,242,242,242,114,50,18,18,18,18,18,18,15,

128,254,7,240,63,128,248,63,251,255,255,255,250,251,147,128,

128,4,0,8,5,1,64,136,65,32,56,9,4,34,5,1,

64,32,123,208,134,16,192,24,2,128,136,32,136,10,0,128,

4,1,192,56,31,199,253,255,255,255,255,245,236,152,16,2,

0,

};

METRIC cmti10_metric_small[128] = {

{1167, -1, 13, 12, 14, 411054},

{1188, -2, 13, 13, 14, 535936},

{1211, -4, 13, 12, 14, 502440},

{1232, -2, 13, 11, 14, 453652},

{1252, -2, 13, 13, 14, 435448},

{1275, -1, 13, 16, 14, 487148},

{1303, -2, 13, 13, 14, 468944},

{1326, -5, 13, 12, 14, 502440},

{1347, -4, 13, 11, 14, 468944},

{1367, -5, 13, 12, 14, 502440},

{1388, -3, 13, 12, 14, 468944},

{375, 0, 13, 15, 18, 401952},

{409, 0, 13, 12, 18, 368456},

{436, 0, 13, 12, 18, 385204},

{463, 0, 13, 18, 18, 577806},

{504, 0, 13, 19, 18, 586180},

{291, -3, 8, 3, 9, 200976},

{295, 1, 8, 7, 13, 217724},

{1984, -6, 13, 3, 4, 334960},

{1986, -8, 13, 3, 3, 334960},

{1988, -6, 12, 5, 2, 334960},

{1990, -6, 13, 5, 4, 334960},

{1993, -5, 11, 7, 1, 334960},

{1994, -9, 13, 5, 3, 544795},

{1996, -3, -1, 4, 4, 301464},

{307, 0, 13, 11, 18, 351708},

{332, -3, 8, 12, 9, 468944},

{346, -3, 8, 12, 9, 468944},

{360, -2, 10, 9, 13, 334960},

{1622, -2, 13, 16, 14, 578534},

{1650, -3, 13, 17, 14, 645526},

{1680, -4, 14, 12, 16, 502440},

{1998, -2, 7, 5, 2, 167480},

{1704, -2, 13, 5, 14, 200976},

{2009, -4, 13, 7, 6, 337145},

{1722, -3, 13, 14, 18, 535936},

{1554, -2, 13, 12, 14, 504033},

{1754, -4, 15, 13, 17, 535936},

{1575, -3, 13, 13, 14, 502440},

{1782, -4, 13, 3, 6, 200976},

{1785, -2, 15, 8, 21, 267968},

{1806, -1, 15, 8, 21, 267968},

{1827, -4, 15, 8, 9, 334960},

{1836, -3, 11, 12, 13, 502440},

{1856, -1, 1, 3, 6, 200976},

{2015, -2, 4, 4, 1, 234472},

{1859, -2, 1, 2, 2, 200976},

{1860, -1, 15, 12, 21, 334960},

{1409, -3, 12, 8, 13, 334960},

{1422, -3, 11, 6, 12, 334960},

{1431, -2, 12, 9, 13, 334960},

{1446, -3, 12, 8, 13, 334960},

{1459, -1, 12, 9, 17, 334960},

{1479, -3, 12, 8, 13, 334960},

{1492, -3, 12, 9, 13, 334960},

{1507, -3, 12, 10, 13, 334960},

{1524, -3, 12, 9, 13, 334960},

{1539, -2, 12, 9, 13, 334960},

{1892, -2, 8, 4, 9, 200976},

{1897, -1, 8, 5, 13, 200976},

{1713, -1, 9, 5, 14, 200976},

{1906, -3, 8, 13, 7, 502440},

{1611, -3, 9, 6, 14, 334960},

{1598, -4, 13, 7, 14, 334960},

{1918, -4, 13, 12, 14, 502440},

{547, -2, 13, 12, 14, 487148},

{568, -1, 13, 14, 14, 461298},

{593, -4, 13, 12, 14, 468944},

{614, -1, 13, 15, 14, 494794},

{641, -1, 13, 13, 14, 444550},

{664, -1, 13, 13, 14, 427802},

{687, -4, 13, 12, 14, 506991},

{708, -1, 13, 16, 14, 487148},

{736, -1, 13, 9, 14, 252676},

{752, -3, 13, 10, 14, 344062},

{770, -1, 13, 17, 14, 503896},

{800, -1, 13, 11, 14, 411054},

{820, -1, 13, 20, 14, 587636},

{855, -1, 13, 16, 14, 487148},

{883, -4, 13, 12, 14, 502440},

{904, -1, 13, 14, 14, 444550},

{929, -4, 13, 12, 18, 502440},

{956, -1, 13, 14, 14, 478046},

{981, -2, 13, 10, 14, 368456},

{999, -4, 13, 12, 14, 468944},

{1020, -4, 13, 13, 14, 487148},

{1043, -5, 13, 12, 14, 487148},

{1064, -5, 13, 18, 14, 654628},

{1096, -1, 13, 15, 14, 487148},

{1123, -4, 13, 13, 14, 487148},

{1146, -2, 13, 12, 14, 401952},

{1939, -1, 15, 8, 21, 200976},

{2016, -5, 13, 7, 6, 337145},

{1960, 0, 15, 8, 21, 200976},

{2000, -6, 13, 5, 3, 334960},

{2002, -6, 14, 1, 2, 200976},

{1981, -4, 13, 3, 6, 200976},

{0, -3, 8, 8, 9, 334960},

{9, -3, 13, 6, 14, 301464},

{20, -3, 8, 7, 9, 301464},

{28, -3, 13, 8, 14, 334960},

{42, -3, 8, 7, 9, 301464},

{50, 0, 13, 8, 18, 200976},

{68, -2, 8, 8, 13, 301464},

{81, -2, 13, 9, 14, 334960},

{97, -3, 14, 4, 15, 200976},

{105, 1, 14, 8, 19, 200976},

{124, -2, 13, 8, 14, 301464},

{138, -3, 13, 3, 14, 167480},

{144, -3, 8, 14, 9, 535936},

{160, -3, 8, 9, 9, 368456},

{171, -3, 8, 7, 9, 334960},

{179, -1, 8, 9, 13, 334960},

{194, -3, 8, 7, 13, 301464},

{206, -3, 8, 7, 9, 276342},

{214, -3, 8, 5, 9, 267968},

{220, -2, 12, 5, 13, 217724},

{229, -3, 8, 8, 9, 351708},

{238, -3, 8, 7, 9, 301464},

{246, -3, 8, 11, 9, 435448},

{259, -1, 8, 10, 9, 304013},

{271, -2, 8, 8, 13, 318212},

{284, -2, 8, 6, 9, 267968},

{2022, -2, 5, 9, 1, 334960},

{2024, -3, 5, 17, 1, 669920},

{2003, -6, 13, 5, 3, 334960},

{2005, -6, 13, 5, 2, 334960},

{2007, -6, 14, 6, 2, 334960},

};

char cmti10_bitmap_small[2027] = {

26,38,70,132,132,132,141,158,102,96,130,8,65,102,81,134,

24,98,153,128,28,69,4,8,16,32,198,112,3,1,1,1,

2,26,38,70,132,132,132,141,158,102,28,69,15,232,16,32,

166,112,3,5,4,4,4,30,8,8,8,8,16,16,16,16,

16,32,160,64,13,19,35,66,66,66,70,76,52,4,4,8,

240,48,8,4,2,2,1,112,196,66,98,33,16,136,152,84,

24,16,0,0,202,196,72,172,192,1,0,0,0,0,0,12,

10,10,4,4,4,4,8,8,8,8,16,224,48,16,16,16,

32,35,37,40,112,120,68,69,134,134,100,148,146,146,109,128,

206,114,198,42,16,152,196,66,17,8,68,34,97,10,132,24,

206,88,168,76,68,34,17,19,10,131,0,28,69,12,24,48,

97,68,112,51,22,74,34,17,8,132,68,102,44,16,8,8,

15,0,24,77,28,40,80,163,76,104,16,32,135,192,205,102,

131,4,8,16,64,128,50,80,131,8,84,192,8,66,47,136,

68,33,10,99,0,66,162,162,68,68,68,77,78,54,67,70,

138,36,72,145,36,48,66,52,70,136,162,36,68,136,145,18,

100,51,0,27,137,18,64,32,8,2,0,138,100,102,0,33,

81,81,34,34,34,38,36,28,4,8,136,112,20,178,132,16,

148,90,176,215,37,46,192,12,20,40,32,64,129,4,8,16,

32,142,0,1,192,68,16,130,16,68,56,130,32,68,8,129,

16,33,8,33,72,38,4,1,0,160,8,0,6,225,177,34,

20,126,132,8,64,140,25,162,107,192,24,226,81,70,24,126,

132,8,64,140,25,162,99,192,0,128,67,194,98,114,41,36,

162,82,50,30,8,8,0,3,230,8,84,16,32,32,64,128,

131,255,130,4,4,8,8,16,16,32,64,128,129,1,2,2,

4,8,8,16,32,168,192,142,0,3,240,65,4,0,64,8,

1,254,8,32,130,8,64,132,16,65,5,16,97,6,32,2,

0,160,4,0,3,240,67,4,16,129,8,33,254,8,32,130,

16,65,4,16,65,5,16,98,6,32,2,0,160,4,0,3,

231,193,10,16,129,0,32,64,8,16,15,255,225,1,8,64,

130,16,33,4,8,65,2,16,128,133,32,65,136,16,98,4,

1,1,0,208,128,51,192,0,3,199,224,137,12,32,32,132,

4,16,129,4,63,255,130,4,16,128,130,16,16,130,4,16,

64,130,8,16,82,2,12,64,65,136,16,1,2,0,168,128,

8,224,0,0,192,12,1,192,28,2,192,44,4,192,76,8,

193,252,16,194,12,32,207,63,31,248,24,96,96,193,131,12,

24,48,224,254,3,14,24,24,96,97,129,134,12,48,99,255,

0,7,144,199,48,50,3,96,44,0,192,12,0,128,8,4,

192,76,8,97,3,224,31,248,12,24,24,24,48,48,192,97,

128,195,1,134,3,24,12,48,24,96,96,192,131,6,31,248,

0,31,248,48,65,130,12,16,196,134,32,63,1,152,24,144,

192,134,8,48,67,6,127,224,31,248,48,65,130,12,16,196,

134,32,63,1,152,24,128,192,6,0,48,3,0,127,0,7,

144,199,48,50,3,96,44,0,192,12,0,131,248,12,192,204,

12,97,131,232,31,159,6,12,6,12,6,12,12,24,12,24,

15,248,12,24,24,48,24,48,24,48,24,48,48,96,253,248,

31,131,1,128,192,192,96,48,24,24,12,6,3,3,7,224,

15,192,192,48,12,6,1,128,96,24,12,3,0,195,48,152,

60,0,31,143,131,6,1,134,0,198,0,196,0,100,0,55,

0,29,128,28,192,12,112,6,24,3,12,3,7,7,231,192,

31,192,192,24,3,0,192,24,3,0,96,24,35,4,97,12,

35,13,255,0,31,0,240,112,28,7,2,192,112,44,11,5,

128,152,88,9,137,128,153,24,17,147,1,26,48,17,163,1,

28,48,33,134,15,153,248,31,31,7,4,7,4,5,132,9,

136,9,136,8,200,8,200,16,208,16,112,16,112,16,112,32,

32,248,32,7,193,134,48,38,3,96,60,3,192,60,3,128,

104,6,192,204,24,99,3,192,31,240,24,96,96,193,131,12,

24,48,96,195,3,240,24,0,96,1,128,6,0,48,3,240,

0,7,193,134,48,38,3,96,60,3,192,60,3,128,104,6,

128,205,216,99,3,224,2,0,36,3,128,48,31,240,24,96,

96,193,131,12,24,48,192,252,3,24,24,96,97,129,134,6,

24,48,107,241,224,7,67,49,132,97,56,7,1,240,30,1,

128,100,25,4,194,47,0,127,246,49,67,20,49,134,32,96,

6,0,96,12,0,192,12,0,192,24,15,240,252,249,129,12,

8,96,70,4,48,33,129,12,8,192,134,4,48,33,130,4,

32,30,0,248,246,4,96,134,8,113,3,16,50,3,32,52,

3,192,56,3,0,48,2,0,251,243,216,48,70,12,49,135,

8,97,196,24,177,6,44,129,147,32,100,208,26,52,3,142,

0,195,128,32,192,8,48,0,15,158,6,48,12,64,29,0,

28,0,56,0,96,0,224,2,192,13,128,19,0,71,1,6,

31,63,0,252,121,193,6,16,49,129,136,6,128,60,1,192,

12,0,96,3,0,24,1,128,63,0,31,241,131,16,97,12,

33,128,56,3,0,96,12,65,132,56,67,12,97,143,248,31,

240,97,6,16,97,12,16,192,12,0,192,24,1,128,24,1,

128,48,15,224,0,96,7,0,56,2,192,38,1,56,16,193,

6,8,48,129,136,14,64,55,255,255,252,7,193,134,48,38,

3,96,60,147,207,60,243,146,104,6,128,204,24,99,3,192,

0,128,48,6,1,192,56,11,1,96,76,9,130,48,70,16,

194,25,239,192,31,248,255,196,2,32,16,0,4,32,63,1,

248,16,128,0,16,8,128,79,252,127,224,31,255,6,12,6,

12,6,12,12,24,12,24,12,24,12,24,24,48,24,48,24,

48,24,48,48,96,253,248,31,248,224,195,2,28,16,96,131,

0,12,0,96,6,0,96,134,4,96,70,6,127,224,120,239,

159,139,16,160,14,0,192,12,0,192,24,1,128,24,1,128,

48,15,224,15,224,96,12,15,195,108,140,241,158,51,204,217,

176,248,6,1,128,254,0,31,192,96,6,12,103,204,236,204,

204,204,216,217,143,176,124,1,128,48,15,224,7,192,195,16,

51,3,96,54,3,96,54,6,96,66,12,160,170,20,225,206,

56,14,17,33,33,65,65,65,130,130,130,132,136,112,4,82,

130,8,33,4,16,66,63,7,4,70,37,18,137,132,4,12,

24,16,80,62,40,224,14,17,49,81,49,6,28,2,2,2,

132,140,112,0,128,64,32,32,16,16,8,8,9,4,132,79,

72,120,16,8,8,4,0,17,30,44,32,32,60,102,6,6,

134,140,136,112,7,4,68,4,4,2,225,137,132,130,65,33,

9,7,0,60,88,228,11,4,2,0,128,64,16,8,2,0,

128,64,16,0,14,0,136,44,39,33,224,240,140,130,65,32,

16,135,0,7,4,68,36,18,9,4,134,62,1,1,0,145,

135,0,0,224,17,2,16,33,4,0,64,15,128,64,8,0,

128,8,7,130,156,70,56,14,8,136,72,68,64,36,2,44,

48,146,75,18,128,228,1,32,9,0,134,8,15,128,28,69,

10,24,67,8,18,56,0,0,2,12,0,4,48,0,0,66,

132,33,24,98,145,128,3,255,1,97,1,97,2,97,2,193,

4,196,4,196,15,252,9,138,17,138,17,132,33,132,35,12,

247,248,3,255,134,24,70,12,38,6,19,6,43,3,17,129,

248,192,204,96,197,112,96,152,48,140,24,67,24,96,255,224,

0,16,126,24,99,6,96,182,19,194,60,35,196,56,134,144,

105,12,224,134,48,188,8,0,8,198,33,24,196,32,0,4,

96,8,192,0,0,132,35,24,204,96,0,136,2,32,8,128,

68,1,17,255,240,68,1,16,8,128,34,0,136,63,252,34,

0,136,4,64,17,0,68,2,32,0,24,9,32,137,248,136,

68,68,34,65,36,10,32,98,0,35,2,36,18,33,17,16,

137,4,136,36,129,192,44,165,0,1,2,4,8,16,32,32,

64,64,64,128,128,128,128,128,128,128,128,64,64,32,4,2,

2,1,1,1,1,1,1,1,2,2,2,4,4,8,8,16,

32,64,128,4,4,105,62,24,62,210,16,16,0,128,8,1,

0,16,1,0,16,255,240,32,2,0,32,4,0,64,4,0,

44,165,0,112,0,16,2,0,32,4,0,64,8,0,128,16,

2,0,32,4,0,64,8,0,128,16,1,0,32,2,0,64,

8,0,128,0,51,0,0,4,192,24,192,0,0,4,97,16,

136,0,127,248,0,0,0,0,0,0,0,3,255,128,7,129,

134,32,36,113,72,153,9,144,153,9,145,41,50,140,200,0,

64,195,240,7,4,8,8,8,8,16,16,16,16,32,32,32,

32,64,64,64,64,128,128,224,7,1,2,2,2,2,4,4,

4,4,8,8,8,8,16,16,16,16,32,32,224,37,44,128,

200,144,42,0,203,128,140,100,192,254,124,92,49,30,62,0,

34,162,192,74,100,109,128,79,32,34,204,138,36,81,0,240,

34,69,18,44,200,128,255,128,255,255,128,

};

METRIC cmtt10_metric_small[128] = {

{698, 0, 12, 9, 13, 344061},

{713, -1, 12, 8, 13, 344061},

{726, -1, 12, 8, 13, 344061},

{739, -1, 12, 8, 13, 344061},

{752, -1, 12, 8, 13, 344061},

{765, -1, 12, 8, 13, 344061},

{778, -1, 12, 8, 13, 344061},

{791, -1, 12, 8, 13, 344061},

{804, -1, 12, 8, 13, 344061},

{817, -1, 12, 8, 13, 344061},

{830, -1, 12, 8, 13, 344061},

{1334, -1, 12, 8, 13, 344061},

{1321, -1, 12, 8, 13, 344061},

{1426, -4, 12, 2, 6, 344061},

{1144, -4, 7, 2, 13, 344061},

{1012, -1, 7, 8, 13, 344061},

{1025, -2, 8, 7, 9, 344061},

{1033, -1, 8, 6, 14, 344061},

{1297, -2, 12, 4, 3, 344061},

{1299, -4, 12, 4, 3, 344061},

{1301, -2, 11, 6, 2, 344061},

{1303, -2, 12, 6, 3, 344061},

{1306, -2, 11, 6, 1, 344061},

{1307, -3, 12, 4, 3, 344061},

{1309, -3, 0, 5, 5, 344061},

{1044, -1, 12, 8, 13, 344061},

{1057, -1, 8, 9, 9, 344061},

{1068, 0, 8, 10, 9, 344061},

{1080, -1, 10, 8, 14, 344061},

{1094, 0, 12, 9, 13, 344061},

{1109, -1, 12, 9, 13, 344061},

{1124, -1, 13, 8, 16, 344061},

{1428, -1, 4, 8, 7, 344061},

{1140, -4, 12, 2, 13, 344061},

{1421, -2, 12, 6, 6, 344061},

{1148, -1, 12, 8, 13, 344061},

{970, -1, 13, 8, 16, 344061},

{1161, -1, 13, 8, 16, 344061},

{986, -1, 12, 8, 13, 344061},

{1177, -3, 12, 4, 7, 344061},

{1181, -3, 13, 5, 16, 344061},

{1191, -2, 13, 5, 16, 344061},

{1201, -1, 10, 8, 9, 344061},

{1210, -1, 10, 8, 10, 344061},

{1220, -4, 2, 3, 6, 344061},

{1319, -1, 6, 8, 2, 344061},

{1223, -4, 2, 3, 3, 344061},

{1225, -1, 13, 8, 16, 344061},

{843, -1, 12, 8, 13, 344061},

{856, -2, 12, 6, 13, 344061},

{866, -1, 12, 8, 13, 344061},

{879, -1, 12, 8, 13, 344061},

{892, -1, 12, 8, 13, 344061},

{905, -1, 12, 8, 13, 344061},

{918, -1, 12, 8, 13, 344061},

{931, -1, 12, 8, 13, 344061},

{944, -1, 12, 8, 13, 344061},

{957, -1, 12, 8, 13, 344061},

{1241, -4, 8, 3, 9, 344061},

{1245, -4, 8, 3, 12, 344061},

{1347, -1, 10, 8, 10, 344061},

{1250, -1, 8, 8, 6, 344061},

{1357, -1, 10, 8, 10, 344061},

{999, -1, 12, 8, 13, 344061},

{1256, -1, 12, 8, 13, 344061},

{0, -1, 12, 8, 13, 344061},

{13, -1, 12, 8, 13, 344061},

{26, -1, 12, 8, 13, 344061},

{39, -1, 12, 8, 13, 344061},

{52, -1, 12, 9, 13, 344061},

{67, 0, 12, 9, 13, 344061},

{82, -1, 12, 8, 13, 344061},

{95, -1, 12, 8, 13, 344061},

{108, -2, 12, 7, 13, 344061},

{120, -2, 12, 7, 13, 344061},

{132, -1, 12, 9, 13, 344061},

{147, 0, 12, 9, 13, 344061},

{162, -1, 12, 8, 13, 344061},

{175, -1, 12, 8, 13, 344061},

{188, -1, 12, 8, 13, 344061},

{201, -1, 12, 8, 13, 344061},

{214, -1, 12, 8, 16, 344061},

{230, -1, 12, 9, 13, 344061},

{245, -1, 12, 8, 13, 344061},

{258, -1, 12, 8, 13, 344061},

{271, -1, 12, 8, 13, 344061},

{284, 0, 12, 10, 13, 344061},

{301, 0, 12, 10, 13, 344061},

{318, 0, 12, 9, 13, 344061},

{333, 0, 12, 10, 13, 344061},

{350, -1, 12, 8, 13, 344061},

{1269, -4, 13, 6, 16, 344061},

{1369, -1, 13, 8, 16, 344061},

{1281, 0, 13, 6, 16, 344061},

{1313, -2, 12, 6, 3, 344061},

{1367, -1, -1, 8, 2, 344061},

{1293, -3, 14, 4, 7, 344061},

{363, -1, 8, 9, 9, 344061},

{374, 0, 12, 9, 13, 344061},

{389, -2, 8, 6, 9, 344061},

{396, -1, 12, 9, 13, 344061},

{411, -1, 8, 7, 9, 344061},

{419, 0, 12, 9, 13, 344061},

{434, -1, 8, 9, 15, 344061},

{451, 0, 12, 10, 13, 344061},

{468, -2, 13, 7, 14, 344061},

{481, -1, 13, 6, 19, 344061},

{496, 0, 12, 10, 13, 344061},

{513, -1, 12, 9, 13, 344061},

{528, 0, 8, 11, 9, 344061},

{541, 0, 8, 10, 9, 344061},

{553, -2, 8, 6, 9, 344061},

{560, 0, 8, 9, 14, 344061},

{576, -1, 8, 10, 14, 344061},

{594, 0, 8, 10, 9, 344061},

{606, -2, 8, 6, 9, 344061},

{613, -1, 10, 7, 11, 344061},

{623, 0, 8, 10, 9, 344061},

{635, 0, 8, 10, 9, 344061},

{647, 0, 8, 10, 9, 344061},

{659, 0, 8, 10, 9, 344061},

{671, 0, 8, 10, 14, 344061},

{689, -1, 8, 8, 9, 344061},

{1389, -1, 13, 8, 16, 344061},

{1385, -4, 13, 2, 16, 344061},

{1405, -1, 13, 8, 16, 344061},

{1316, -2, 12, 6, 2, 344061},

{1318, -3, 13, 4, 2, 344061},

};

char cmtt10_bitmap_small[1435] = {

24,24,24,24,36,36,36,36,126,66,66,66,231,252,66,65,

65,65,70,126,67,65,65,65,66,254,29,35,65,65,128,128,

128,128,128,65,65,34,28,252,70,66,67,65,65,65,65,65,

65,66,70,252,255,32,144,72,4,2,33,240,136,64,32,16,

40,31,248,255,144,72,36,2,1,16,248,68,32,16,8,4,

15,128,58,70,66,130,128,128,128,143,130,130,66,70,58,231,

66,66,66,66,66,126,66,66,66,66,66,231,254,32,64,129,

2,4,8,16,32,64,143,224,62,8,16,32,64,129,2,4,

10,20,39,128,239,34,17,9,4,130,193,160,216,68,35,16,

136,110,56,248,16,8,4,2,1,0,128,64,32,16,8,36,

31,248,231,102,102,102,90,90,90,90,90,66,66,66,231,231,

98,98,82,82,82,90,74,74,74,70,70,230,126,195,129,129,

129,129,129,129,129,129,129,195,126,252,66,65,65,65,66,124,

64,64,64,64,64,224,126,195,129,129,129,129,129,129,129,137,

137,199,126,2,2,1,248,34,16,136,68,34,33,224,136,68,

34,17,40,158,48,62,66,130,128,128,96,28,2,1,1,129,

130,252,255,137,137,8,8,8,8,8,8,8,8,8,62,231,

66,66,66,66,66,66,66,66,66,66,36,24,243,200,66,16,

132,33,4,129,32,72,18,4,128,192,48,12,0,225,208,36,

9,2,64,147,34,208,204,51,12,195,48,204,51,0,115,145,

4,130,192,192,96,48,24,18,9,8,196,47,56,243,200,66,

16,68,18,2,128,192,16,4,1,0,64,16,14,0,255,131,

130,4,4,8,24,16,32,32,65,193,255,120,2,1,1,135,

68,34,17,8,123,128,224,16,8,4,2,225,136,130,65,32,

144,72,38,34,224,61,8,32,130,8,17,56,14,1,0,128,

67,162,50,9,4,130,65,32,136,195,184,56,138,12,31,240,

32,33,60,7,132,2,1,15,248,64,32,16,8,4,2,1,

7,240,59,150,17,8,132,65,65,224,128,60,33,32,80,40,

18,16,240,224,8,2,0,128,46,12,66,16,132,33,8,66,

16,132,251,192,48,96,0,0,30,4,8,16,32,64,129,31,

192,12,48,0,1,240,65,4,16,65,4,16,65,6,47,0,

224,8,2,0,128,47,136,130,64,160,52,8,130,48,132,251,

192,248,4,2,1,0,128,64,32,16,8,4,2,1,15,248,

217,140,201,17,34,36,68,136,145,18,34,230,96,238,12,66,

16,132,33,8,66,16,132,251,192,49,40,97,134,24,82,48,

238,24,136,36,18,9,4,130,98,46,16,8,4,2,7,192,

61,16,200,18,4,129,32,72,17,12,61,0,64,16,4,1,

1,240,243,135,49,128,64,16,4,1,0,64,254,0,126,24,

80,58,24,113,184,32,67,249,2,4,8,16,34,68,112,231,

8,66,16,132,33,8,66,16,140,29,192,243,200,66,16,72,

18,4,129,224,48,12,0,243,208,34,16,132,45,11,66,208,

204,51,0,115,140,129,32,48,12,3,1,32,132,243,192,243,

200,65,16,68,18,2,128,160,24,4,1,0,64,48,72,28,

0,255,130,4,8,24,16,33,65,255,255,144,72,36,2,1,

0,128,64,32,16,8,4,15,128,24,24,24,36,36,36,66,

66,66,66,129,129,255,126,195,129,129,129,153,153,153,129,129,

129,195,126,24,24,24,24,36,36,36,36,66,66,66,66,231,

255,129,0,0,0,66,126,66,0,0,0,129,255,255,66,66,

66,66,66,66,66,66,66,66,66,231,255,65,65,32,32,16,

16,16,32,32,65,65,255,102,145,129,8,8,8,8,8,8,

8,8,8,62,62,8,8,60,74,137,137,137,74,60,8,8,

62,62,8,8,137,74,74,74,74,44,28,8,8,62,60,66,

129,129,129,129,129,66,66,66,36,165,231,60,102,66,129,129,

129,129,129,129,195,66,102,60,16,71,4,16,65,4,16,65,

4,252,60,66,129,129,1,1,2,4,8,16,32,65,255,60,

66,66,2,4,28,2,1,1,129,129,66,60,12,28,20,36,

36,68,132,255,4,4,4,4,31,126,64,64,64,64,124,66,

1,1,129,129,66,60,30,33,64,64,128,188,194,129,129,129,

65,66,60,255,130,2,4,4,8,8,8,16,16,16,16,16,

60,66,129,129,195,126,126,195,129,129,129,66,60,60,66,130,

129,129,129,67,61,1,2,2,132,120,8,60,74,137,137,136,

104,28,10,9,137,137,74,60,8,8,48,72,72,72,87,84,

100,100,164,152,153,153,102,126,129,129,2,12,8,24,24,0,

0,0,24,24,24,24,0,0,0,24,24,16,48,64,129,129,

126,240,32,64,129,2,4,8,254,124,16,65,4,16,65,4,

16,65,139,192,30,33,33,34,236,34,34,33,33,33,33,37,

230,243,6,66,33,19,254,66,33,25,115,0,49,146,152,70,

17,135,225,8,65,41,49,128,1,2,62,70,199,137,153,145,

227,98,124,64,128,128,63,138,69,34,129,65,32,158,72,60,

18,9,40,158,248,127,68,162,81,8,132,66,61,16,136,68,

34,57,23,248,2,126,199,133,141,137,137,153,145,145,177,161,

227,126,64,64,255,255,3,192,240,63,255,192,54,54,54,126,

255,54,108,108,108,255,126,108,108,66,166,166,172,76,12,24,

24,24,24,50,53,53,101,101,34,103,17,18,192,8,136,132,

66,16,132,32,132,16,65,130,8,33,4,33,8,66,33,17,

16,8,8,203,110,24,110,203,8,8,24,24,24,24,255,255,

24,24,24,24,93,149,0,93,0,1,3,6,6,6,12,12,

24,24,48,48,96,96,96,192,192,93,0,23,64,93,0,23,

101,64,255,255,0,0,255,255,28,34,69,75,145,145,145,145,

145,74,68,33,30,254,8,32,130,8,32,130,8,32,130,8,

63,252,16,65,4,16,65,4,16,65,4,16,127,52,136,142,

96,134,16,22,128,205,224,134,23,128,252,105,96,33,130,31,

0,49,40,64,102,96,153,255,255,24,24,24,24,24,24,24,

24,24,219,60,24,8,8,24,60,219,24,24,24,24,24,24,

24,24,24,3,14,28,48,224,224,48,28,14,3,192,112,56,

12,7,7,12,56,112,192,255,255,192,192,96,96,96,48,48,

24,24,12,12,6,6,6,3,1,255,255,255,255,7,8,8,

8,8,8,8,240,240,8,8,8,8,8,8,7,240,8,8,

8,8,8,8,7,7,8,8,8,8,8,8,240,134,24,97,

134,16,255,240,129,129,129,129,129,129,255,

};

typedef struct {
	char *name;
	int id;
	METRIC *metric;
	char *bitmap;
} FONT;

#define NFONT 16

FONT *font;

FONT font_large[NFONT] = {
	{"cmr10",	0,	cmr10_metric,		cmr10_bitmap},
	{"cmr7",	0,	cmr7_metric,		cmr7_bitmap},
	{"cmr5",	0,	cmr5_metric,		cmr5_bitmap},
	{"cmbx10",	0,	cmbx10_metric,		cmbx10_bitmap},
	{"cmbx7",	0,	cmbx7_metric,		cmbx7_bitmap},
	{"cmbx5",	0,	cmbx5_metric,		cmbx5_bitmap},
	{"cmsl10",	0,	cmsl10_metric,		cmsl10_bitmap},
	{"cmti10",	0,	cmti10_metric,		cmti10_bitmap},
	{"cmtt10",	0,	cmtt10_metric,		cmtt10_bitmap},
	{"cmmi10",	0,	cmmi10_metric,		cmmi10_bitmap},
	{"cmmi7",	0,	cmmi7_metric,		cmmi7_bitmap},
	{"cmmi5",	0,	cmmi5_metric,		cmmi5_bitmap},
	{"cmsy10",	0,	cmsy10_metric,		cmsy10_bitmap},
	{"cmsy7",	0,	cmsy7_metric,		cmsy7_bitmap},
	{"cmsy5",	0,	cmsy5_metric,		cmsy5_bitmap},
	{"cmex10",	0,	cmex10_metric,		cmex10_bitmap},
};

FONT font_small[NFONT] = {
	{"cmr10",	0,	cmr10_metric_small,	cmr10_bitmap_small},
	{"cmr7",	0,	cmr7_metric_small,	cmr7_bitmap_small},
	{"cmr5",	0,	cmr5_metric_small,	cmr5_bitmap_small},
	{"cmbx10",	0,	cmbx10_metric_small,	cmbx10_bitmap_small},
	{"cmbx7",	0,	cmbx7_metric_small,	cmbx7_bitmap_small},
	{"cmbx5",	0,	cmbx5_metric_small,	cmbx5_bitmap_small},
	{"cmsl10",	0,	cmsl10_metric_small,	cmsl10_bitmap_small},
	{"cmti10",	0,	cmti10_metric_small,	cmti10_bitmap_small},
	{"cmtt10",	0,	cmtt10_metric_small,	cmtt10_bitmap_small},
	{"cmmi10",	0,	cmmi10_metric_small,	cmmi10_bitmap_small},
	{"cmmi7",	0,	cmmi7_metric_small,	cmmi7_bitmap_small},
	{"cmmi5",	0,	cmmi5_metric_small,	cmmi5_bitmap_small},
	{"cmsy10",	0,	cmsy10_metric_small,	cmsy10_bitmap_small},
	{"cmsy7",	0,	cmsy7_metric_small,	cmsy7_bitmap_small},
	{"cmsy5",	0,	cmsy5_metric_small,	cmsy5_bitmap_small},
	{"cmex10",	0,	cmex10_metric_small,	cmex10_bitmap_small},
};

char n[100];
METRIC *metric;
char *bitmap;
int f;
int c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, p;
int h, v, w, x, y, z;
#define MAXSP 20
struct {
	int h, v, w, x, y, z;
} stack[MAXSP];
int sp;
int eop_flag;
FILE *yf;

int M;

read_dvi_file(int small)
{
	if (small) {
		M = M_SMALL;
		font = font_small;
	} else {
		M = M_LARGE;
		font = font_large;
	}

	yf = fopen(dvi_file_name, "rb");

	if (yf == NULL) {
		printf("cannot open %s\n", dvi_file_name);
		exit(1);
	}

	while (eop_flag == 0)
		dvi_cmd();

	fclose(yf);
}

ffgetc()
{
	int c;
	c = fgetc(yf);
	if (c == EOF) {
		printf("unexpected end of file\n");
		exit(1);
	}
	return c;
}

fget1()
{
	return ffgetc();
}

fget2()
{
	int c1, c2;
	c1 = ffgetc();
	c2 = ffgetc();
	if (c1 & 0x80)
		return 0xffff0000 | c1 << 8 | c2;
	else
		return c1 << 8 | c2;
}

fget3()
{
	int c1, c2, c3;
	c1 = ffgetc();
	c2 = ffgetc();
	c3 = ffgetc();
	if (c1 & 0x80)
		return 0xff000000 | c1 << 16 | c2 << 8 | c3;
	else
		return c1 << 16 | c2 << 8 | c3;
}

fget4()
{
	int c1, c2, c3, c4;
	c1 = ffgetc();
	c2 = ffgetc();
	c3 = ffgetc();
	c4 = ffgetc();
	return c1 << 24 | c2 << 16 | c3 << 8 | c4;
}

dvi_cmd()
{
	int c;
	c = fget1();
	if (c < 128)
		set_char(c);
	else if (c >= 171 && c <= 234)
		fnt_num(c);
	else
		switch (c) {
		case 132:
			set_rule();
			break;
		case 137:
			put_rule();
			break;
		case 139:
			bop();
			break;
		case 140:
			eop();
			break;
		case 141:
			push();
			break;
		case 142:
			pop();
			break;
		case 143:
			right1();
			break;
		case 144:
			right2();
			break;
		case 145:
			right3();
			break;
		case 146:
			right4();
			break;
		case 147:
			w0();
			break;
		case 148:
			w1();
			break;
		case 149:
			w2();
			break;
		case 150:
			w3();
			break;
		case 151:
			w4();
			break;
		case 152:
			x0();
			break;
		case 153:
			x1();
			break;
		case 154:
			x2();
			break;
		case 155:
			x3();
			break;
		case 156:
			x4();
			break;
		case 157:
			down1();
			break;
		case 158:
			down2();
			break;
		case 159:
			down3();
			break;
		case 160:
			down4();
			break;
		case 161:
			y0();
			break;
		case 162:
			y1();
			break;
		case 163:
			y2();
			break;
		case 164:
			y3();
			break;
		case 165:
			y4();
			break;
		case 166:
			z0();
			break;
		case 167:
			z1();
			break;
		case 168:
			z2();
			break;
		case 169:
			z3();
			break;
		case 170:
			z4();
			break;
		case 239:
			xxx1();
			break;
		case 240:
			xxx2();
			break;
		case 241:
			xxx3();
			break;
		case 242:
			xxx4();
			break;
		case 243:
			fnt_def1();
			break;
		case 247:
			pre();
			break;
		case 248:
			post();
			break;
		default:
			printf("dvi_cmd: c = %d?\n", c);
			exit(1);
		}
}

/* 0 through 127 */

set_char(c)
int c;
{
	int a;
	a = draw_char(h, v, c);
	h += a;
}

/* 132 */

set_rule()
{
	int a, b;
	a = fget4();
	b = fget4();
	if (a > 0 && b > 0)
		draw_rule(h, v, a, b);
	h += b;
}

/* 137 */

put_rule()
{
	int a, b;
	a = fget4();
	b = fget4();
	if (a > 0 && b > 0)
		draw_rule(h, v, a, b);
}

/* 139 */

bop()
{
	c0 = fget4();
	c1 = fget4();
	c2 = fget4();
	c3 = fget4();
	c4 = fget4();
	c5 = fget4();
	c6 = fget4();
	c7 = fget4();
	c8 = fget4();
	c9 = fget4();
	p = fget4();
	sp = 0;
	h = 0;
	v = 0;
	w = 0;
	x = 0;
	y = 0;
	z = 0;
}

/* 140 */

eop()
{
	eop_flag = 1;
}

/* 141 */

push()
{
	if (sp == MAXSP) {
		printf("dvi stack overflow\n");
		exit(1);
	}
	stack[sp].h = h;
	stack[sp].v = v;
	stack[sp].w = w;
	stack[sp].x = x;
	stack[sp].y = y;
	stack[sp].z = z;
	sp++;
}

/* 142 */

pop()
{
	if (sp == 0) {
		printf("dvi stack underflow\n");
		exit(1);
	}
	sp--;
	h = stack[sp].h;
	v = stack[sp].v;
	w = stack[sp].w;
	x = stack[sp].x;
	y = stack[sp].y;
	z = stack[sp].z;
}

/* 143 */

right1()
{
	int b;
	b = fget1();
	h += b;
}

/* 144 */

right2()
{
	int b;
	b = fget2();
	h += b;
}

/* 145 */

right3()
{
	int b;
	b = fget3();
	h += b;
}

/* 146 */

right4()
{
	int b;
	b = fget4();
	h += b;
}

/* 147 */

w0()
{
	h += w;
}

/* 148 */

w1()
{
	w = fget1();
	h += w;
}

/* 149 */

w2()
{
	w = fget2();
	h += w;
}

/* 150 */

w3()
{
	w = fget3();
	h += w;
}

/* 151 */

w4()
{
	w = fget4();
	h += w;
}

/* 152 */

x0()
{
	h += x;
}

/* 153 */

x1()
{
	x = fget1();
	h += x;
}

/* 154 */

x2()
{
	x = fget2();
	h += x;
}

/* 155 */

x3()
{
	x = fget3();
	h += x;
}

/* 156 */

x4()
{
	x = fget4();
	h += x;
}

/* 157 */

down1()
{
	int a;
	a = fget1();
	v += a;
}

/* 158 */

down2()
{
	int a;
	a = fget2();
	v += a;
}

/* 159 */

down3()
{
	int a;
	a = fget3();
	v += a;
}

/* 160 */

down4()
{
	int a;
	a = fget4();
	v += a;
}

/* 161 */

y0()
{
	v += y;
}

/* 162 */

y1()
{
	y = fget1();
	v += y;
}

/* 163 */

y2()
{
	y = fget2();
	v += y;
}

/* 164 */

y3()
{
	y = fget3();
	v += y;
}

/* 165 */

y4()
{
	y = fget4();
	v += y;
}

/* 166 */

z0()
{
	v += z;
}

/* 167 */

z1()
{
	z = fget1();
	v += z;
}

/* 168 */

z2()
{
	z = fget2();
	v += z;
}

/* 169 */

z3()
{
	z = fget3();
	v += z;
}

/* 170 */

z4()
{
	z = fget4();
	v += z;
}

/* 171 through 234 */

fnt_num(c)
int c;
{
	f = c - 171;
	select_font(f);
}

/* 239 */

xxx1()
{
	int k;
	k = fget1();
	xxx(k);
}

/* 240 */

xxx2()
{
	int k;
	k = fget2();
	xxx(k);
}

/* 241 */

xxx3()
{
	int k;
	k = fget3();
	xxx(k);
}

/* 242 */

xxx4()
{
	int k;
	k = fget4();
	xxx(k);
}

xxx(k)
int k;
{
	int i;
	for (i = 0; i < k; i++)
		fget1();
}

/* 243 */

fnt_def1()
{
	int a, i, k, l;
	k = fget1();
	/* c = */ fget4();
	/* s = */ fget4();
	/* d = */ fget4();
	a = fget1();
	l = fget1();
	for (i = 0; i < a; i++)
		fget1();
	for (i = 0; i < l; i++)
		n[i] = fget1();
	n[i] = 0;
	load_font(k, n);
}

pre()
{
	int j, k;
	/* i = */	fget1();
	/* num = */	fget4();
	/* den = */	fget4();
	/* mag = */	fget4();
	k = fget1();
	for (j = 0; j < k; j++)
		fget1();
}

/* 248 */

post()
{
}

load_font(k, n)
int k;
char *n;
{
	int i;
	for (i = 0; i < NFONT; i++)
		if (strcmp(font[i].name, n) == 0) {
			font[i].id = k;
			return;
		}
	printf("load_font %s?\n", n);
	exit(1);
}

select_font(font_id)
int font_id;
{
	int i;
	for (i = 0; i < NFONT; i++)
		if (font[i].id == font_id) {
			metric = font[i].metric;
			bitmap = font[i].bitmap;
			return;
		}
	printf("select_font %d?\n", font_id);
	exit(1);
}

draw_char(h, v, n)
int h, v, n;
{
	int b, i, j, m, ncol, nrow, w, x, y;
	char *p;
	if (metric == NULL || bitmap == NULL)
		return 0;
	x = (h + M/2) / M;
	y = (v + M/2) / M;
	x -= metric[n].x_offset;
	y -= metric[n].y_offset;
	p = bitmap + metric[n].index;
	ncol = metric[n].ncol;
	nrow = metric[n].nrow;
	w = metric[n].tfm_width;
	b = *p++;
	m = 0x80;
	for (i = 0; i < nrow; i++)
		for (j = 0; j < ncol; j++) {
			if (b & m)
				set_pixel(x + j, y + i);
			m >>= 1;
			if (m == 0) {
				m = 0x80;
				b = *p++;
			}
		}
	return w;
}

draw_rule(h, v, a, b)
int h, v, a, b;
{
	int i, j;
	int dx, dy, x, y;
	x = (h + M/2) / M;
	y = (v + M/2) / M;
	dx = (b + M/2) / M;
	dy = (a + M/2) / M;
	if (dx < 0) {
		dx = -dx;
		x -= dx;
	}
	if (dy < 0) {
		dy = -dy;
		y -= dy;
	}
	for (i = 0; i < dy; i++)
		for (j = 0; j < dx; j++)
			set_pixel(x + j, y + i);
}

set_pixel(x, y)
int x, y;
{
	if (x < 0 || x >= MAXX || y < 0 || y >= MAXY)
		return;
	buf[y * MAXX + x] = 1;
	if (x < min_x)
		min_x = x;
	if (x > max_x)
		max_x = x;
	if (y < min_y)
		min_y = y;
	if (y > max_y)
		max_y = y;
}
